// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// GENERATOR-BEGIN: LegacyHandlerFlags
// ⚠️This was generated by GENERATOR!🦹‍♂️
pub(crate) struct LegacyHandlerFlags;
#[allow(dead_code)]
impl LegacyHandlerFlags {
	pub(crate) const HANDLER_REG: u32 = 0x0000_0001;
	pub(crate) const HANDLER_MEM: u32 = 0x0000_0002;
	pub(crate) const HANDLER_66_REG: u32 = 0x0000_0004;
	pub(crate) const HANDLER_66_MEM: u32 = 0x0000_0008;
	pub(crate) const HANDLER_F3_REG: u32 = 0x0000_0010;
	pub(crate) const HANDLER_F3_MEM: u32 = 0x0000_0020;
	pub(crate) const HANDLER_F2_REG: u32 = 0x0000_0040;
	pub(crate) const HANDLER_F2_MEM: u32 = 0x0000_0080;
}
// GENERATOR-END: LegacyHandlerFlags
