// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::MemorySize;

#[rustfmt::skip]
pub(crate) static MVEX_MEMSZ_LUT: [MemorySize; 112] = [
	// MvexTupleTypeLutKind::Int32
	MemorySize::Packed512_Int32,// 0
	MemorySize::Int32,// 1
	MemorySize::Packed128_Int32,// 2
	MemorySize::Packed256_Float16,// 3
	MemorySize::Packed128_UInt8,// 4
	MemorySize::Packed128_Int8,// 5
	MemorySize::Packed256_UInt16,// 6
	MemorySize::Packed256_Int16,// 7
	// MvexTupleTypeLutKind::Int32_Half
	MemorySize::Packed256_Int32,// 0
	MemorySize::Int32,// 1
	MemorySize::Packed128_Int32,// 2
	MemorySize::Packed128_Float16,// 3
	MemorySize::Packed64_UInt8,// 4
	MemorySize::Packed64_Int8,// 5
	MemorySize::Packed128_UInt16,// 6
	MemorySize::Packed128_Int16,// 7
	// MvexTupleTypeLutKind::Int32_4to16
	MemorySize::Packed128_Int32,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Packed64_Float16,// 3
	MemorySize::Packed32_UInt8,// 4
	MemorySize::Packed32_Int8,// 5
	MemorySize::Packed64_UInt16,// 6
	MemorySize::Packed64_Int16,// 7
	// MvexTupleTypeLutKind::Int32_1to16_or_elem
	MemorySize::Int32,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Float16,// 3
	MemorySize::UInt8,// 4
	MemorySize::Int8,// 5
	MemorySize::UInt16,// 6
	MemorySize::Int16,// 7
	// MvexTupleTypeLutKind::Int64
	MemorySize::Packed512_Int64,// 0
	MemorySize::Int64,// 1
	MemorySize::Packed256_Int64,// 2
	MemorySize::Packed128_Float16,// 3
	MemorySize::Packed64_UInt8,// 4
	MemorySize::Packed64_Int8,// 5
	MemorySize::Packed128_UInt16,// 6
	MemorySize::Packed128_Int16,// 7
	// MvexTupleTypeLutKind::Int64_4to8
	MemorySize::Packed256_Int64,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Packed64_Float16,// 3
	MemorySize::Packed32_UInt8,// 4
	MemorySize::Packed32_Int8,// 5
	MemorySize::Packed64_UInt16,// 6
	MemorySize::Packed64_Int16,// 7
	// MvexTupleTypeLutKind::Int64_1to8_or_elem
	MemorySize::Int64,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Float16,// 3
	MemorySize::UInt8,// 4
	MemorySize::Int8,// 5
	MemorySize::UInt16,// 6
	MemorySize::Int16,// 7
	// MvexTupleTypeLutKind::Float32
	MemorySize::Packed512_Float32,// 0
	MemorySize::Float32,// 1
	MemorySize::Packed128_Float32,// 2
	MemorySize::Packed256_Float16,// 3
	MemorySize::Packed128_UInt8,// 4
	MemorySize::Packed128_Int8,// 5
	MemorySize::Packed256_UInt16,// 6
	MemorySize::Packed256_Int16,// 7
	// MvexTupleTypeLutKind::Float32_Half
	MemorySize::Packed256_Float32,// 0
	MemorySize::Float32,// 1
	MemorySize::Packed128_Float32,// 2
	MemorySize::Packed128_Float16,// 3
	MemorySize::Packed64_UInt8,// 4
	MemorySize::Packed64_Int8,// 5
	MemorySize::Packed128_UInt16,// 6
	MemorySize::Packed128_Int16,// 7
	// MvexTupleTypeLutKind::Float32_4to16
	MemorySize::Packed128_Float32,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Packed64_Float16,// 3
	MemorySize::Packed32_UInt8,// 4
	MemorySize::Packed32_Int8,// 5
	MemorySize::Packed64_UInt16,// 6
	MemorySize::Packed64_Int16,// 7
	// MvexTupleTypeLutKind::Float32_1to16_or_elem
	MemorySize::Float32,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Float16,// 3
	MemorySize::UInt8,// 4
	MemorySize::Int8,// 5
	MemorySize::UInt16,// 6
	MemorySize::Int16,// 7
	// MvexTupleTypeLutKind::Float64
	MemorySize::Packed512_Float64,// 0
	MemorySize::Float64,// 1
	MemorySize::Packed256_Float64,// 2
	MemorySize::Packed128_Float16,// 3
	MemorySize::Packed64_UInt8,// 4
	MemorySize::Packed64_Int8,// 5
	MemorySize::Packed128_UInt16,// 6
	MemorySize::Packed128_Int16,// 7
	// MvexTupleTypeLutKind::Float64_4to8
	MemorySize::Packed256_Float64,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Packed64_Float16,// 3
	MemorySize::Packed32_UInt8,// 4
	MemorySize::Packed32_Int8,// 5
	MemorySize::Packed64_UInt16,// 6
	MemorySize::Packed64_Int16,// 7
	// MvexTupleTypeLutKind::Float64_1to8_or_elem
	MemorySize::Float64,// 0
	MemorySize::Unknown,// 1
	MemorySize::Unknown,// 2
	MemorySize::Float16,// 3
	MemorySize::UInt8,// 4
	MemorySize::Int8,// 5
	MemorySize::UInt16,// 6
	MemorySize::Int16,// 7
];
