// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::TupleType;

#[rustfmt::skip]
pub(crate) static MVEX_TUPLE_TYPE_LUT: [TupleType; 112] = [
	// MvexTupleTypeLutKind::Int32
	TupleType::N64,// 0
	TupleType::N4,// 1
	TupleType::N16,// 2
	TupleType::N32,// 3
	TupleType::N16,// 4
	TupleType::N16,// 5
	TupleType::N32,// 6
	TupleType::N32,// 7
	// MvexTupleTypeLutKind::Int32_Half
	TupleType::N32,// 0
	TupleType::N4,// 1
	TupleType::N16,// 2
	TupleType::N16,// 3
	TupleType::N8,// 4
	TupleType::N8,// 5
	TupleType::N16,// 6
	TupleType::N16,// 7
	// MvexTupleTypeLutKind::Int32_4to16
	TupleType::N16,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N8,// 3
	TupleType::N4,// 4
	TupleType::N4,// 5
	TupleType::N8,// 6
	TupleType::N8,// 7
	// MvexTupleTypeLutKind::Int32_1to16_or_elem
	TupleType::N4,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N2,// 3
	TupleType::N1,// 4
	TupleType::N1,// 5
	TupleType::N2,// 6
	TupleType::N2,// 7
	// MvexTupleTypeLutKind::Int64
	TupleType::N64,// 0
	TupleType::N8,// 1
	TupleType::N32,// 2
	TupleType::N16,// 3
	TupleType::N8,// 4
	TupleType::N8,// 5
	TupleType::N16,// 6
	TupleType::N16,// 7
	// MvexTupleTypeLutKind::Int64_4to8
	TupleType::N32,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N8,// 3
	TupleType::N4,// 4
	TupleType::N4,// 5
	TupleType::N8,// 6
	TupleType::N8,// 7
	// MvexTupleTypeLutKind::Int64_1to8_or_elem
	TupleType::N8,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N2,// 3
	TupleType::N1,// 4
	TupleType::N1,// 5
	TupleType::N2,// 6
	TupleType::N2,// 7
	// MvexTupleTypeLutKind::Float32
	TupleType::N64,// 0
	TupleType::N4,// 1
	TupleType::N16,// 2
	TupleType::N32,// 3
	TupleType::N16,// 4
	TupleType::N16,// 5
	TupleType::N32,// 6
	TupleType::N32,// 7
	// MvexTupleTypeLutKind::Float32_Half
	TupleType::N32,// 0
	TupleType::N4,// 1
	TupleType::N16,// 2
	TupleType::N16,// 3
	TupleType::N8,// 4
	TupleType::N8,// 5
	TupleType::N16,// 6
	TupleType::N16,// 7
	// MvexTupleTypeLutKind::Float32_4to16
	TupleType::N16,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N8,// 3
	TupleType::N4,// 4
	TupleType::N4,// 5
	TupleType::N8,// 6
	TupleType::N8,// 7
	// MvexTupleTypeLutKind::Float32_1to16_or_elem
	TupleType::N4,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N2,// 3
	TupleType::N1,// 4
	TupleType::N1,// 5
	TupleType::N2,// 6
	TupleType::N2,// 7
	// MvexTupleTypeLutKind::Float64
	TupleType::N64,// 0
	TupleType::N8,// 1
	TupleType::N32,// 2
	TupleType::N16,// 3
	TupleType::N8,// 4
	TupleType::N8,// 5
	TupleType::N16,// 6
	TupleType::N16,// 7
	// MvexTupleTypeLutKind::Float64_4to8
	TupleType::N32,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N8,// 3
	TupleType::N4,// 4
	TupleType::N4,// 5
	TupleType::N8,// 6
	TupleType::N8,// 7
	// MvexTupleTypeLutKind::Float64_1to8_or_elem
	TupleType::N8,// 0
	TupleType::N1,// 1
	TupleType::N1,// 2
	TupleType::N2,// 3
	TupleType::N1,// 4
	TupleType::N1,// 5
	TupleType::N2,// 6
	TupleType::N2,// 7
];
