// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::mvex::mvex_info::MvexInfo;
use crate::{MvexConvFn, MvexEHBit, MvexTupleTypeLutKind};

#[rustfmt::skip]
pub(super) static MVEX_INFO: [MvexInfo; 207] = [
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetchnta_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetch0_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetch1_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetch2_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetchenta_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetche0_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetche1_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Vprefetche2_m
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x06, 0x00, 0x04, 0x03),// Code::MVEX_Vmovaps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xFE, 0x00, 0x04, 0x01),// Code::MVEX_Vmovapd_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Df32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vmovaps_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Df64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vmovapd_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::EH0, MvexConvFn::Df64, 0xFE, 0xFF, 0x00, 0x00),// Code::MVEX_Vmovnrapd_m_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::EH1, MvexConvFn::Df64, 0xFE, 0xFF, 0x00, 0x00),// Code::MVEX_Vmovnrngoapd_m_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::EH0, MvexConvFn::Df32, 0x06, 0xFF, 0x00, 0x02),// Code::MVEX_Vmovnraps_m_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::EH1, MvexConvFn::Df32, 0x06, 0xFF, 0x00, 0x02),// Code::MVEX_Vmovnrngoaps_m_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vaddps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vmulps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_Half, MvexEHBit::None, MvexConvFn::Sf32, 0xF8, 0x00, 0x24, 0x02),// Code::MVEX_Vcvtps2pd_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x34, 0x00),// Code::MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vsubps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x0E, 0x00, 0x04, 0x03),// Code::MVEX_Vmovdqa32_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xFE, 0x00, 0x04, 0x01),// Code::MVEX_Vmovdqa64_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x04, 0x03),// Code::MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x05, 0x03),// Code::MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x05, 0x03),// Code::MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x05, 0x03),// Code::MVEX_Vpslld_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_Half, MvexEHBit::None, MvexConvFn::Si32, 0xF8, 0x00, 0x04, 0x02),// Code::MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Di32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vmovdqa32_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Di64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vmovdqa64_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Clevict1_m
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFF, 0x40, 0x06),// Code::MVEX_Clevict0_m
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x26, 0x02),// Code::MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x26, 0x00),// Code::MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpandd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vpandq_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_Half, MvexEHBit::None, MvexConvFn::Si32, 0xF8, 0x00, 0x04, 0x02),// Code::MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpord_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vporq_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpxord_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vbroadcastss_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vbroadcastsd_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_4to16, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vbroadcastf32x4_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float64_4to8, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vbroadcastf64x4_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vptestmd_kr_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x06, 0x03),// Code::MVEX_Vpermd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpminud_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x24, 0x02),// Code::MVEX_Vgetexpps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x24, 0x00),// Code::MVEX_Vgetexppd_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x26, 0x02),// Code::MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x26, 0x02),// Code::MVEX_Vgminps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x26, 0x00),// Code::MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x26, 0x02),// Code::MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x26, 0x00),// Code::MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x26, 0x02),// Code::MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x26, 0x00),// Code::MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Ui32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vpbroadcastd_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Ui64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vpbroadcastq_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_4to16, MvexEHBit::None, MvexConvFn::Ui32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vbroadcasti32x4_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int64_4to8, MvexEHBit::None, MvexConvFn::Ui64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vbroadcasti64x4_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpadcd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int64, MvexEHBit::None, MvexConvFn::Si64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x06, 0x02),// Code::MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x06, 0x00),// Code::MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x36, 0x02),// Code::MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Ui32, 0x0E, 0xFF, 0x84, 0x02),// Code::MVEX_Vpgatherdd_zmm_k1_mvt
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Ui64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vpgatherdq_zmm_k1_mvt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vgatherdps_zmm_k1_mvt
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vgatherdpd_zmm_k1_mvt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Di32, 0x0E, 0xFF, 0x84, 0x02),// Code::MVEX_Vpscatterdd_mvt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Di64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vpscatterdq_mvt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Df32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vscatterdps_mvt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Df64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vscatterdpd_mvt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0xFA, 0xFE, 0x36, 0x02),// Code::MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::None, 0x00, 0xFF, 0xB4, 0x02),// Code::MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::None, 0x00, 0xFF, 0xB4, 0x02),// Code::MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0xFA, 0xFE, 0x06, 0x02),// Code::MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x02),// Code::MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x36, 0x00),// Code::MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::None, 0x00, 0xFF, 0xB4, 0x02),// Code::MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vgatherpf0hintdps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vgatherpf0hintdpd_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vgatherpf0dps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vgatherpf1dps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vscatterpf0hintdps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x84, 0x00),// Code::MVEX_Vscatterpf0hintdpd_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vscatterpf0dps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x84, 0x02),// Code::MVEX_Vscatterpf1dps_mvt_k1
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x24, 0x02),// Code::MVEX_Vexp223ps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x24, 0x02),// Code::MVEX_Vlog2ps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x24, 0x02),// Code::MVEX_Vrcp23ps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x24, 0x02),// Code::MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0xB6, 0x02),// Code::MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x86, 0x02),// Code::MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x36, 0x02),// Code::MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Ui32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vloadunpackld_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Ui64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vloadunpacklq_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Di32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vpackstoreld_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Di64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vpackstorelq_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vloadunpacklps_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vloadunpacklpd_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Df32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vpackstorelps_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Df64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vpackstorelpd_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Ui32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vloadunpackhd_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Ui64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vloadunpackhq_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Int32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Di32, 0x0E, 0xFF, 0x04, 0x02),// Code::MVEX_Vpackstorehd_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Int64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Di64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vpackstorehq_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Uf32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vloadunpackhps_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Uf64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vloadunpackhpd_zmm_k1_mt
	MvexInfo::new(MvexTupleTypeLutKind::Float32_1to16_or_elem, MvexEHBit::None, MvexConvFn::Df32, 0x06, 0xFF, 0x04, 0x02),// Code::MVEX_Vpackstorehps_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float64_1to8_or_elem, MvexEHBit::None, MvexConvFn::Df64, 0xFE, 0xFF, 0x04, 0x00),// Code::MVEX_Vpackstorehpd_mt_k1_zmm
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x06, 0x03),// Code::MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0xFE, 0xFE, 0x04, 0x03),// Code::MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x06, 0x03),// Code::MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x24, 0x02),// Code::MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x24, 0x00),// Code::MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x2C, 0x02),// Code::MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x2C, 0x00),// Code::MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x2C, 0x02),// Code::MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x2C, 0x02),// Code::MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x2C, 0x00),// Code::MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Int32, MvexEHBit::None, MvexConvFn::Si32, 0x08, 0x00, 0x2C, 0x02),// Code::MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::Sf32, 0x00, 0x00, 0x2C, 0x02),// Code::MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	MvexInfo::new(MvexTupleTypeLutKind::Float32, MvexEHBit::None, MvexConvFn::None, 0x00, 0x00, 0x34, 0x02),// Code::MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	MvexInfo::new(MvexTupleTypeLutKind::Float64, MvexEHBit::None, MvexConvFn::Sf64, 0xF8, 0x00, 0x2C, 0x00),// Code::MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
];
