// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// GENERATOR-BEGIN: FastFmtFlags
// ⚠️This was generated by GENERATOR!🦹‍♂️
pub(crate) struct FastFmtFlags;
#[allow(dead_code)]
impl FastFmtFlags {
	pub(crate) const NONE: u32 = 0x0000_0000;
	pub(crate) const HAS_VPREFIX: u32 = 0x0000_0001;
	pub(crate) const SAME_AS_PREV: u32 = 0x0000_0002;
	pub(crate) const FORCE_MEM_SIZE: u32 = 0x0000_0004;
	pub(crate) const PSEUDO_OPS_KIND_SHIFT: u32 = 0x0000_0003;
	pub(crate) const CMPPS: u32 = 0x0000_0008;
	pub(crate) const VCMPPS: u32 = 0x0000_0010;
	pub(crate) const CMPPD: u32 = 0x0000_0018;
	pub(crate) const VCMPPD: u32 = 0x0000_0020;
	pub(crate) const CMPSS: u32 = 0x0000_0028;
	pub(crate) const VCMPSS: u32 = 0x0000_0030;
	pub(crate) const CMPSD: u32 = 0x0000_0038;
	pub(crate) const VCMPSD: u32 = 0x0000_0040;
	pub(crate) const PCLMULQDQ: u32 = 0x0000_0048;
	pub(crate) const VPCLMULQDQ: u32 = 0x0000_0050;
	pub(crate) const VPCOMB: u32 = 0x0000_0058;
	pub(crate) const VPCOMW: u32 = 0x0000_0060;
	pub(crate) const VPCOMD: u32 = 0x0000_0068;
	pub(crate) const VPCOMQ: u32 = 0x0000_0070;
	pub(crate) const VPCOMUB: u32 = 0x0000_0078;
	pub(crate) const VPCOMUW: u32 = 0x0000_0080;
	pub(crate) const VPCOMUD: u32 = 0x0000_0088;
	pub(crate) const VPCOMUQ: u32 = 0x0000_0090;
	pub(crate) const VPCMPB: u32 = 0x0000_0098;
	pub(crate) const VPCMPW: u32 = 0x0000_00A0;
	pub(crate) const VPCMPD: u32 = 0x0000_00A8;
	pub(crate) const VPCMPQ: u32 = 0x0000_00B0;
	pub(crate) const VPCMPUB: u32 = 0x0000_00B8;
	pub(crate) const VPCMPUW: u32 = 0x0000_00C0;
	pub(crate) const VPCMPUD: u32 = 0x0000_00C8;
	pub(crate) const VPCMPUQ: u32 = 0x0000_00D0;
	pub(crate) const VCMPPH: u32 = 0x0000_00D8;
	pub(crate) const VCMPSH: u32 = 0x0000_00E0;
	pub(crate) const VCMPPS8: u32 = 0x0000_00E8;
	pub(crate) const VCMPPD8: u32 = 0x0000_00F0;
	pub(crate) const VPCMPD6: u32 = 0x0000_00F8;
}
// GENERATOR-END: FastFmtFlags
