// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

#[rustfmt::skip]
pub(super) static TBL_DATA: &[u8] = &[
	// handlers_Grp_0F18
	0x01,// ArrayReference
	0x08,// 0x8
	// 0 = 0x00
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x83, 0x24,// MVEX_Vprefetchnta_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 1 = 0x01
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x84, 0x24,// MVEX_Vprefetch0_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 2 = 0x02
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x85, 0x24,// MVEX_Vprefetch1_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 3 = 0x03
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x86, 0x24,// MVEX_Vprefetch2_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 4 = 0x04
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x87, 0x24,// MVEX_Vprefetchenta_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 5 = 0x05
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x88, 0x24,// MVEX_Vprefetche0_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 6 = 0x06
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x89, 0x24,// MVEX_Vprefetche1_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 7 = 0x07
	0x08,// MandatoryPrefix2
		0x0A,// M
			0x8A, 0x24,// MVEX_Vprefetche2_m
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// handlers_Grp_0F72
	0x01,// ArrayReference
	0x08,// 0x8
	// 0 = 0x00
	0x02,// Dup
		0x02,// 2
		0x00,// Invalid

	// 2 = 0x02
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0D,// HWIb
				0x9F, 0x24,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 3 = 0x03
	0x00,// Invalid

	// 4 = 0x04
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0D,// HWIb
				0xA0, 0x24,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 5 = 0x05
	0x00,// Invalid

	// 6 = 0x06
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0D,// HWIb
				0xA1, 0x24,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 7 = 0x07
	0x00,// Invalid

	// handlers_Grp_0FAE
	0x01,// ArrayReference
	0x08,// 0x8
	// 0 = 0x00
	0x02,// Dup
		0x07,// 7
		0x00,// Invalid

	// 7 = 0x07
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x00,// Invalid
		0x0A,// M
			0xA6, 0x24,// MVEX_Clevict1_m
		0x0A,// M
			0xA7, 0x24,// MVEX_Clevict0_m

	// handlers_Grp_0F38C6
	0x01,// ArrayReference
	0x08,// 0x8
	// 0 = 0x00
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0x9B, 0x25,// MVEX_Vgatherpf0hintdps_mvt_k1
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0x9C, 0x25,// MVEX_Vgatherpf0hintdpd_mvt_k1
		0x00,// Invalid
		0x00,// Invalid

	// 1 = 0x01
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0x9D, 0x25,// MVEX_Vgatherpf0dps_mvt_k1
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 2 = 0x02
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0x9E, 0x25,// MVEX_Vgatherpf1dps_mvt_k1
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 3 = 0x03
	0x00,// Invalid

	// 4 = 0x04
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0x9F, 0x25,// MVEX_Vscatterpf0hintdps_mvt_k1
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0xA0, 0x25,// MVEX_Vscatterpf0hintdpd_mvt_k1
		0x00,// Invalid
		0x00,// Invalid

	// 5 = 0x05
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0xA1, 0x25,// MVEX_Vscatterpf0dps_mvt_k1
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 6 = 0x06
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x14,// VSIB
					0xA2, 0x25,// MVEX_Vscatterpf1dps_mvt_k1
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 7 = 0x07
	0x00,// Invalid

	// Handlers_0F38
	0x01,// ArrayReference
	0x80, 0x02,// 0x100
	// 0 = 0x00
	0x02,// Dup
		0x18,// 24
		0x00,// Invalid

	// 24 = 0x18
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB5, 0x24,// MVEX_Vbroadcastss_zmm_k1_mt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 25 = 0x19
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB6, 0x24,// MVEX_Vbroadcastsd_zmm_k1_mt
		0x00,// Invalid
		0x00,// Invalid

	// 26 = 0x1A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB7, 0x24,// MVEX_Vbroadcastf32x4_zmm_k1_mt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 27 = 0x1B
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB8, 0x24,// MVEX_Vbroadcastf64x4_zmm_k1_mt
		0x00,// Invalid
		0x00,// Invalid

	// 28 = 0x1C
	0x02,// Dup
		0x0B,// 11
		0x00,// Invalid

	// 39 = 0x27
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x12,// KHW
				0xB9, 0x24,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 40 = 0x28
	0x02,// Dup
		0x0E,// 14
		0x00,// Invalid

	// 54 = 0x36
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBA, 0x24,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 55 = 0x37
	0x02,// Dup
		0x02,// 2
		0x00,// Invalid

	// 57 = 0x39
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBB, 0x24,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 58 = 0x3A
	0x00,// Invalid

	// 59 = 0x3B
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBC, 0x24,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 60 = 0x3C
	0x00,// Invalid

	// 61 = 0x3D
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBD, 0x24,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 62 = 0x3E
	0x00,// Invalid

	// 63 = 0x3F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBE, 0x24,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 64 = 0x40
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xBF, 0x24,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 65 = 0x41
	0x00,// Invalid

	// 66 = 0x42
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC0, 0x24,// MVEX_Vgetexpps_zmm_k1_zmmmt
			0x0C,// VW
				0xC1, 0x24,// MVEX_Vgetexppd_zmm_k1_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 67 = 0x43
	0x02,// Dup
		0x02,// 2
		0x00,// Invalid

	// 69 = 0x45
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xC2, 0x24,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 70 = 0x46
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xC3, 0x24,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 71 = 0x47
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xC4, 0x24,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 72 = 0x48
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC5, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 73 = 0x49
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC6, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 74 = 0x4A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC7, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 75 = 0x4B
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC8, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 76 = 0x4C
	0x02,// Dup
		0x04,// 4
		0x00,// Invalid

	// 80 = 0x50
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xC9, 0x24,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xCA, 0x24,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 81 = 0x51
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xCB, 0x24,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 82 = 0x52
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xCC, 0x24,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xCD, 0x24,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 83 = 0x53
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xCE, 0x24,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xCF, 0x24,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 84 = 0x54
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xD0, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 85 = 0x55
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xD1, 0x24,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xD2, 0x24,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 86 = 0x56
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xD3, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 87 = 0x57
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xD4, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 88 = 0x58
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xD5, 0x24,// MVEX_Vpbroadcastd_zmm_k1_mt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 89 = 0x59
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xD6, 0x24,// MVEX_Vpbroadcastq_zmm_k1_mt
		0x00,// Invalid
		0x00,// Invalid

	// 90 = 0x5A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xD7, 0x24,// MVEX_Vbroadcasti32x4_zmm_k1_mt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 91 = 0x5B
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xD8, 0x24,// MVEX_Vbroadcasti64x4_zmm_k1_mt
		0x00,// Invalid
		0x00,// Invalid

	// 92 = 0x5C
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xD9, 0x24,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 93 = 0x5D
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xDA, 0x24,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 94 = 0x5E
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xDB, 0x24,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 95 = 0x5F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xDC, 0x24,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 96 = 0x60
	0x02,// Dup
		0x04,// 4
		0x00,// Invalid

	// 100 = 0x64
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xDD, 0x24,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xDE, 0x24,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 101 = 0x65
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xDF, 0x24,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xE0, 0x24,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 102 = 0x66
	0x00,// Invalid

	// 103 = 0x67
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xE1, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 104 = 0x68
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xE2, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 105 = 0x69
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xE3, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 106 = 0x6A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xE4, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 107 = 0x6B
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xE5, 0x24,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 108 = 0x6C
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xE6, 0x24,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 109 = 0x6D
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xE7, 0x24,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xE8, 0x24,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 110 = 0x6E
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xE9, 0x24,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 111 = 0x6F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x11,// VKW
				0xEA, 0x24,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 112 = 0x70
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xEB, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 113 = 0x71
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xEC, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 114 = 0x72
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xED, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 115 = 0x73
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xEE, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 116 = 0x74
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x12,// KHW
				0xEF, 0x24,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 117 = 0x75
	0x02,// Dup
		0x0F,// 15
		0x00,// Invalid

	// 132 = 0x84
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xF0, 0x24,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 133 = 0x85
	0x00,// Invalid

	// 134 = 0x86
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xF1, 0x24,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 135 = 0x87
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xF2, 0x24,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 136 = 0x88
	0x02,// Dup
		0x08,// 8
		0x00,// Invalid

	// 144 = 0x90
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x16,// V_VSIB
				0xF3, 0x24,// MVEX_Vpgatherdd_zmm_k1_mvt
			0x16,// V_VSIB
				0xF4, 0x24,// MVEX_Vpgatherdq_zmm_k1_mvt
		0x00,// Invalid
		0x00,// Invalid

	// 145 = 0x91
	0x00,// Invalid

	// 146 = 0x92
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x16,// V_VSIB
				0xF5, 0x24,// MVEX_Vgatherdps_zmm_k1_mvt
			0x16,// V_VSIB
				0xF6, 0x24,// MVEX_Vgatherdpd_zmm_k1_mvt
		0x00,// Invalid
		0x00,// Invalid

	// 147 = 0x93
	0x00,// Invalid

	// 148 = 0x94
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xF7, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
			0x0F,// VHW
				0xF8, 0x24,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
		0x00,// Invalid
		0x00,// Invalid

	// 149 = 0x95
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 152 = 0x98
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xF9, 0x24,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xFA, 0x24,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 153 = 0x99
	0x00,// Invalid

	// 154 = 0x9A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xFB, 0x24,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xFC, 0x24,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 155 = 0x9B
	0x00,// Invalid

	// 156 = 0x9C
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xFD, 0x24,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xFE, 0x24,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 157 = 0x9D
	0x00,// Invalid

	// 158 = 0x9E
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xFF, 0x24,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x80, 0x25,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 159 = 0x9F
	0x00,// Invalid

	// 160 = 0xA0
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x15,// VSIB_V
				0x81, 0x25,// MVEX_Vpscatterdd_mvt_k1_zmm
			0x15,// VSIB_V
				0x82, 0x25,// MVEX_Vpscatterdq_mvt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 161 = 0xA1
	0x00,// Invalid

	// 162 = 0xA2
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x15,// VSIB_V
				0x83, 0x25,// MVEX_Vscatterdps_mvt_k1_zmm
			0x15,// VSIB_V
				0x84, 0x25,// MVEX_Vscatterdpd_mvt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 163 = 0xA3
	0x00,// Invalid

	// 164 = 0xA4
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x85, 0x25,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 165 = 0xA5
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 168 = 0xA8
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x86, 0x25,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x87, 0x25,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 169 = 0xA9
	0x00,// Invalid

	// 170 = 0xAA
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x88, 0x25,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x89, 0x25,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 171 = 0xAB
	0x00,// Invalid

	// 172 = 0xAC
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x8A, 0x25,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x8B, 0x25,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 173 = 0xAD
	0x00,// Invalid

	// 174 = 0xAE
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x8C, 0x25,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x8D, 0x25,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 175 = 0xAF
	0x00,// Invalid

	// 176 = 0xB0
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x16,// V_VSIB
				0x8E, 0x25,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 177 = 0xB1
	0x00,// Invalid

	// 178 = 0xB2
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x16,// V_VSIB
				0x8F, 0x25,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 179 = 0xB3
	0x00,// Invalid

	// 180 = 0xB4
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x90, 0x25,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 181 = 0xB5
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x91, 0x25,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 182 = 0xB6
	0x02,// Dup
		0x02,// 2
		0x00,// Invalid

	// 184 = 0xB8
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x92, 0x25,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x93, 0x25,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 185 = 0xB9
	0x00,// Invalid

	// 186 = 0xBA
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x94, 0x25,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x95, 0x25,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 187 = 0xBB
	0x00,// Invalid

	// 188 = 0xBC
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x96, 0x25,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x97, 0x25,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 189 = 0xBD
	0x00,// Invalid

	// 190 = 0xBE
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0x98, 0x25,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0x99, 0x25,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 191 = 0xBF
	0x00,// Invalid

	// 192 = 0xC0
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x16,// V_VSIB
				0x9A, 0x25,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 193 = 0xC1
	0x02,// Dup
		0x05,// 5
		0x00,// Invalid

	// 198 = 0xC6
	0x06,// Group
		0x04,// ArrayReference
			0x03,// 0x3 = handlers_Grp_0F38C6

	// 199 = 0xC7
	0x00,// Invalid

	// 200 = 0xC8
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xA3, 0x25,// MVEX_Vexp223ps_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 201 = 0xC9
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xA4, 0x25,// MVEX_Vlog2ps_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 202 = 0xCA
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xA5, 0x25,// MVEX_Vrcp23ps_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 203 = 0xCB
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xA6, 0x25,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 204 = 0xCC
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xA7, 0x25,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 205 = 0xCD
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xA8, 0x25,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 206 = 0xCE
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xA9, 0x25,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
			0x0F,// VHW
				0xAA, 0x25,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
		0x00,// Invalid
		0x00,// Invalid

	// 207 = 0xCF
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xAB, 0x25,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 208 = 0xD0
	0x08,// MandatoryPrefix2
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xAC, 0x25,// MVEX_Vloadunpackld_zmm_k1_mt
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xAD, 0x25,// MVEX_Vloadunpacklq_zmm_k1_mt
		0x07,// W
			0x0B,// MV
				0xAE, 0x25,// MVEX_Vpackstoreld_mt_k1_zmm
			0x0B,// MV
				0xAF, 0x25,// MVEX_Vpackstorelq_mt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 209 = 0xD1
	0x08,// MandatoryPrefix2
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB0, 0x25,// MVEX_Vloadunpacklps_zmm_k1_mt
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB1, 0x25,// MVEX_Vloadunpacklpd_zmm_k1_mt
		0x07,// W
			0x0B,// MV
				0xB2, 0x25,// MVEX_Vpackstorelps_mt_k1_zmm
			0x0B,// MV
				0xB3, 0x25,// MVEX_Vpackstorelpd_mt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 210 = 0xD2
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0xB4, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
			0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xB5, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 211 = 0xD3
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0xB6, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 212 = 0xD4
	0x08,// MandatoryPrefix2
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB7, 0x25,// MVEX_Vloadunpackhd_zmm_k1_mt
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xB8, 0x25,// MVEX_Vloadunpackhq_zmm_k1_mt
		0x07,// W
			0x0B,// MV
				0xB9, 0x25,// MVEX_Vpackstorehd_mt_k1_zmm
			0x0B,// MV
				0xBA, 0x25,// MVEX_Vpackstorehq_mt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 213 = 0xD5
	0x08,// MandatoryPrefix2
		0x07,// W
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xBB, 0x25,// MVEX_Vloadunpackhps_zmm_k1_mt
			0x05,// RM
				0x00,// Invalid
				0x0C,// VW
					0xBC, 0x25,// MVEX_Vloadunpackhpd_zmm_k1_mt
		0x07,// W
			0x0B,// MV
				0xBD, 0x25,// MVEX_Vpackstorehps_mt_k1_zmm
			0x0B,// MV
				0xBE, 0x25,// MVEX_Vpackstorehpd_mt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 214 = 0xD6
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0xBF, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
			0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xC0, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 215 = 0xD7
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0xC1, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 216 = 0xD8
	0x02,// Dup
		0x28,// 40
		0x00,// Invalid

	// Handlers_0F3A
	0x01,// ArrayReference
	0x80, 0x02,// 0x100
	// 0 = 0x00
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 3 = 0x03
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x10,// VHWIb
				0xC2, 0x25,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 4 = 0x04
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 7 = 0x07
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xC3, 0x25,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 8 = 0x08
	0x02,// Dup
		0x16,// 22
		0x00,// Invalid

	// 30 = 0x1E
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x13,// KHWIb
				0xC4, 0x25,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 31 = 0x1F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x13,// KHWIb
				0xC5, 0x25,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 32 = 0x20
	0x02,// Dup
		0x06,// 6
		0x00,// Invalid

	// 38 = 0x26
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xC6, 0x25,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
			0x0E,// VWIb
				0xC7, 0x25,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
		0x00,// Invalid
		0x00,// Invalid

	// 39 = 0x27
	0x02,// Dup
		0x2B,// 43
		0x00,// Invalid

	// 82 = 0x52
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xC8, 0x25,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
			0x0E,// VWIb
				0xC9, 0x25,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
		0x00,// Invalid
		0x00,// Invalid

	// 83 = 0x53
	0x02,// Dup
		0x77,// 119
		0x00,// Invalid

	// 202 = 0xCA
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0E,// VWIb
				0xCA, 0x25,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xCB, 0x25,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0E,// VWIb
				0xCC, 0x25,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8

	// 203 = 0xCB
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0E,// VWIb
				0xCD, 0x25,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xCE, 0x25,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 204 = 0xCC
	0x02,// Dup
		0x04,// 4
		0x00,// Invalid

	// 208 = 0xD0
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xCF, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 209 = 0xD1
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0xD0, 0x25,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 210 = 0xD2
	0x02,// Dup
		0x14,// 20
		0x00,// Invalid

	// 230 = 0xE6
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0E,// VWIb
				0xD1, 0x25,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8

	// 231 = 0xE7
	0x02,// Dup
		0x19,// 25
		0x00,// Invalid

	// Handlers_0F
	0x01,// ArrayReference
	0x80, 0x02,// 0x100
	// 0 = 0x00
	0x02,// Dup
		0x18,// 24
		0x00,// Invalid

	// 24 = 0x18
	0x06,// Group
		0x04,// ArrayReference
			0x00,// 0x0 = handlers_Grp_0F18

	// 25 = 0x19
	0x02,// Dup
		0x0F,// 15
		0x00,// Invalid

	// 40 = 0x28
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0x8B, 0x24,// MVEX_Vmovaps_zmm_k1_zmmmt
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0C,// VW
				0x8C, 0x24,// MVEX_Vmovapd_zmm_k1_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 41 = 0x29
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0B,// MV
				0x8D, 0x24,// MVEX_Vmovaps_mt_k1_zmm
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0B,// MV
				0x8E, 0x24,// MVEX_Vmovapd_mt_k1_zmm
		0x07,// W
			0x00,// Invalid
			0x09,// EH
				0x0B,// MV
					0x8F, 0x24,// MVEX_Vmovnrapd_m_k1_zmm
				0x0B,// MV
					0x90, 0x24,// MVEX_Vmovnrngoapd_m_k1_zmm
		0x07,// W
			0x09,// EH
				0x0B,// MV
					0x91, 0x24,// MVEX_Vmovnraps_m_k1_zmm
				0x0B,// MV
					0x92, 0x24,// MVEX_Vmovnrngoaps_m_k1_zmm
			0x00,// Invalid

	// 42 = 0x2A
	0x02,// Dup
		0x2E,// 46
		0x00,// Invalid

	// 88 = 0x58
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0F,// VHW
				0x93, 0x24,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0F,// VHW
				0x94, 0x24,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 89 = 0x59
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0F,// VHW
				0x95, 0x24,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0F,// VHW
				0x96, 0x24,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 90 = 0x5A
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0C,// VW
				0x97, 0x24,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0C,// VW
				0x98, 0x24,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 91 = 0x5B
	0x00,// Invalid

	// 92 = 0x5C
	0x08,// MandatoryPrefix2
		0x07,// W
			0x0F,// VHW
				0x99, 0x24,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x0F,// VHW
				0x9A, 0x24,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 93 = 0x5D
	0x02,// Dup
		0x09,// 9
		0x00,// Invalid

	// 102 = 0x66
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x12,// KHW
				0x9B, 0x24,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 103 = 0x67
	0x02,// Dup
		0x08,// 8
		0x00,// Invalid

	// 111 = 0x6F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0x9C, 0x24,// MVEX_Vmovdqa32_zmm_k1_zmmmt
			0x0C,// VW
				0x9D, 0x24,// MVEX_Vmovdqa64_zmm_k1_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 112 = 0x70
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0E,// VWIb
				0x9E, 0x24,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 113 = 0x71
	0x00,// Invalid

	// 114 = 0x72
	0x06,// Group
		0x04,// ArrayReference
			0x01,// 0x1 = handlers_Grp_0F72

	// 115 = 0x73
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 118 = 0x76
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x12,// KHW
				0xA2, 0x24,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 119 = 0x77
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 122 = 0x7A
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xA3, 0x24,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid

	// 123 = 0x7B
	0x02,// Dup
		0x04,// 4
		0x00,// Invalid

	// 127 = 0x7F
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0B,// MV
				0xA4, 0x24,// MVEX_Vmovdqa32_mt_k1_zmm
			0x0B,// MV
				0xA5, 0x24,// MVEX_Vmovdqa64_mt_k1_zmm
		0x00,// Invalid
		0x00,// Invalid

	// 128 = 0x80
	0x02,// Dup
		0x2E,// 46
		0x00,// Invalid

	// 174 = 0xAE
	0x06,// Group
		0x04,// ArrayReference
			0x02,// 0x2 = handlers_Grp_0FAE

	// 175 = 0xAF
	0x02,// Dup
		0x13,// 19
		0x00,// Invalid

	// 194 = 0xC2
	0x08,// MandatoryPrefix2
		0x07,// W
			0x13,// KHWIb
				0xA8, 0x24,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
			0x00,// Invalid
		0x07,// W
			0x00,// Invalid
			0x13,// KHWIb
				0xA9, 0x24,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
		0x00,// Invalid
		0x00,// Invalid

	// 195 = 0xC3
	0x02,// Dup
		0x18,// 24
		0x00,// Invalid

	// 219 = 0xDB
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xAA, 0x24,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xAB, 0x24,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 220 = 0xDC
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 223 = 0xDF
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xAC, 0x24,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xAD, 0x24,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 224 = 0xE0
	0x02,// Dup
		0x06,// 6
		0x00,// Invalid

	// 230 = 0xE6
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x00,// Invalid
		0x07,// W
			0x0C,// VW
				0xAE, 0x24,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
			0x00,// Invalid
		0x00,// Invalid

	// 231 = 0xE7
	0x02,// Dup
		0x04,// 4
		0x00,// Invalid

	// 235 = 0xEB
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xAF, 0x24,// MVEX_Vpord_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xB0, 0x24,// MVEX_Vporq_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 236 = 0xEC
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 239 = 0xEF
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xB1, 0x24,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
			0x0F,// VHW
				0xB2, 0x24,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
		0x00,// Invalid
		0x00,// Invalid

	// 240 = 0xF0
	0x02,// Dup
		0x0A,// 10
		0x00,// Invalid

	// 250 = 0xFA
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xB3, 0x24,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 251 = 0xFB
	0x02,// Dup
		0x03,// 3
		0x00,// Invalid

	// 254 = 0xFE
	0x08,// MandatoryPrefix2
		0x00,// Invalid
		0x07,// W
			0x0F,// VHW
				0xB4, 0x24,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
			0x00,// Invalid
		0x00,// Invalid
		0x00,// Invalid

	// 255 = 0xFF
	0x00,// Invalid
];
pub(super) const MAX_ID_NAMES: usize = 7;
#[allow(dead_code)]
pub(super) const HANDLERS_0F_INDEX: usize = 6;
#[allow(dead_code)]
pub(super) const HANDLERS_0F38_INDEX: usize = 4;
#[allow(dead_code)]
pub(super) const HANDLERS_0F3A_INDEX: usize = 5;
