// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

#[rustfmt::skip]
pub(super) static FORMATTER_TBL_DATA: &[u8] = &[
	// INVALID
	0x00,// No flags set
	0x80, 0x01,// 128 = "(bad)"

	// DeclareByte
	0x00,// No flags set
	0x82, 0x01,// 130 = "db"

	// DeclareWord
	0x00,// No flags set
	0x84, 0x01,// 132 = "dw"

	// DeclareDword
	0x00,// No flags set
	0x86, 0x01,// 134 = "dd"

	// DeclareQword
	0x00,// No flags set
	0x88, 0x01,// 136 = "dq"

	// Add_rm8_r8
	0x00,// No flags set
	0x04,// 4 = "add"

	// Add_rm16_r16
	0x02,// SameAsPrev

	// Add_rm32_r32
	0x02,// SameAsPrev

	// Add_rm64_r64
	0x02,// SameAsPrev

	// Add_r8_rm8
	0x02,// SameAsPrev

	// Add_r16_rm16
	0x02,// SameAsPrev

	// Add_r32_rm32
	0x02,// SameAsPrev

	// Add_r64_rm64
	0x02,// SameAsPrev

	// Add_AL_imm8
	0x02,// SameAsPrev

	// Add_AX_imm16
	0x02,// SameAsPrev

	// Add_EAX_imm32
	0x02,// SameAsPrev

	// Add_RAX_imm32
	0x02,// SameAsPrev

	// Pushw_ES
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_ES
	0x02,// SameAsPrev

	// Popw_ES
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Popd_ES
	0x02,// SameAsPrev

	// Or_rm8_r8
	0x00,// No flags set
	0x05,// 5 = "or"

	// Or_rm16_r16
	0x02,// SameAsPrev

	// Or_rm32_r32
	0x02,// SameAsPrev

	// Or_rm64_r64
	0x02,// SameAsPrev

	// Or_r8_rm8
	0x02,// SameAsPrev

	// Or_r16_rm16
	0x02,// SameAsPrev

	// Or_r32_rm32
	0x02,// SameAsPrev

	// Or_r64_rm64
	0x02,// SameAsPrev

	// Or_AL_imm8
	0x02,// SameAsPrev

	// Or_AX_imm16
	0x02,// SameAsPrev

	// Or_EAX_imm32
	0x02,// SameAsPrev

	// Or_RAX_imm32
	0x02,// SameAsPrev

	// Pushw_CS
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_CS
	0x02,// SameAsPrev

	// Popw_CS
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Adc_rm8_r8
	0x00,// No flags set
	0x06,// 6 = "adc"

	// Adc_rm16_r16
	0x02,// SameAsPrev

	// Adc_rm32_r32
	0x02,// SameAsPrev

	// Adc_rm64_r64
	0x02,// SameAsPrev

	// Adc_r8_rm8
	0x02,// SameAsPrev

	// Adc_r16_rm16
	0x02,// SameAsPrev

	// Adc_r32_rm32
	0x02,// SameAsPrev

	// Adc_r64_rm64
	0x02,// SameAsPrev

	// Adc_AL_imm8
	0x02,// SameAsPrev

	// Adc_AX_imm16
	0x02,// SameAsPrev

	// Adc_EAX_imm32
	0x02,// SameAsPrev

	// Adc_RAX_imm32
	0x02,// SameAsPrev

	// Pushw_SS
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_SS
	0x02,// SameAsPrev

	// Popw_SS
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Popd_SS
	0x02,// SameAsPrev

	// Sbb_rm8_r8
	0x00,// No flags set
	0x07,// 7 = "sbb"

	// Sbb_rm16_r16
	0x02,// SameAsPrev

	// Sbb_rm32_r32
	0x02,// SameAsPrev

	// Sbb_rm64_r64
	0x02,// SameAsPrev

	// Sbb_r8_rm8
	0x02,// SameAsPrev

	// Sbb_r16_rm16
	0x02,// SameAsPrev

	// Sbb_r32_rm32
	0x02,// SameAsPrev

	// Sbb_r64_rm64
	0x02,// SameAsPrev

	// Sbb_AL_imm8
	0x02,// SameAsPrev

	// Sbb_AX_imm16
	0x02,// SameAsPrev

	// Sbb_EAX_imm32
	0x02,// SameAsPrev

	// Sbb_RAX_imm32
	0x02,// SameAsPrev

	// Pushw_DS
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_DS
	0x02,// SameAsPrev

	// Popw_DS
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Popd_DS
	0x02,// SameAsPrev

	// And_rm8_r8
	0x00,// No flags set
	0x08,// 8 = "and"

	// And_rm16_r16
	0x02,// SameAsPrev

	// And_rm32_r32
	0x02,// SameAsPrev

	// And_rm64_r64
	0x02,// SameAsPrev

	// And_r8_rm8
	0x02,// SameAsPrev

	// And_r16_rm16
	0x02,// SameAsPrev

	// And_r32_rm32
	0x02,// SameAsPrev

	// And_r64_rm64
	0x02,// SameAsPrev

	// And_AL_imm8
	0x02,// SameAsPrev

	// And_AX_imm16
	0x02,// SameAsPrev

	// And_EAX_imm32
	0x02,// SameAsPrev

	// And_RAX_imm32
	0x02,// SameAsPrev

	// Daa
	0x00,// No flags set
	0x89, 0x01,// 137 = "daa"

	// Sub_rm8_r8
	0x00,// No flags set
	0x09,// 9 = "sub"

	// Sub_rm16_r16
	0x02,// SameAsPrev

	// Sub_rm32_r32
	0x02,// SameAsPrev

	// Sub_rm64_r64
	0x02,// SameAsPrev

	// Sub_r8_rm8
	0x02,// SameAsPrev

	// Sub_r16_rm16
	0x02,// SameAsPrev

	// Sub_r32_rm32
	0x02,// SameAsPrev

	// Sub_r64_rm64
	0x02,// SameAsPrev

	// Sub_AL_imm8
	0x02,// SameAsPrev

	// Sub_AX_imm16
	0x02,// SameAsPrev

	// Sub_EAX_imm32
	0x02,// SameAsPrev

	// Sub_RAX_imm32
	0x02,// SameAsPrev

	// Das
	0x00,// No flags set
	0x8A, 0x01,// 138 = "das"

	// Xor_rm8_r8
	0x00,// No flags set
	0x0A,// 10 = "xor"

	// Xor_rm16_r16
	0x02,// SameAsPrev

	// Xor_rm32_r32
	0x02,// SameAsPrev

	// Xor_rm64_r64
	0x02,// SameAsPrev

	// Xor_r8_rm8
	0x02,// SameAsPrev

	// Xor_r16_rm16
	0x02,// SameAsPrev

	// Xor_r32_rm32
	0x02,// SameAsPrev

	// Xor_r64_rm64
	0x02,// SameAsPrev

	// Xor_AL_imm8
	0x02,// SameAsPrev

	// Xor_AX_imm16
	0x02,// SameAsPrev

	// Xor_EAX_imm32
	0x02,// SameAsPrev

	// Xor_RAX_imm32
	0x02,// SameAsPrev

	// Aaa
	0x00,// No flags set
	0x8B, 0x01,// 139 = "aaa"

	// Cmp_rm8_r8
	0x00,// No flags set
	0x0B,// 11 = "cmp"

	// Cmp_rm16_r16
	0x02,// SameAsPrev

	// Cmp_rm32_r32
	0x02,// SameAsPrev

	// Cmp_rm64_r64
	0x02,// SameAsPrev

	// Cmp_r8_rm8
	0x02,// SameAsPrev

	// Cmp_r16_rm16
	0x02,// SameAsPrev

	// Cmp_r32_rm32
	0x02,// SameAsPrev

	// Cmp_r64_rm64
	0x02,// SameAsPrev

	// Cmp_AL_imm8
	0x02,// SameAsPrev

	// Cmp_AX_imm16
	0x02,// SameAsPrev

	// Cmp_EAX_imm32
	0x02,// SameAsPrev

	// Cmp_RAX_imm32
	0x02,// SameAsPrev

	// Aas
	0x00,// No flags set
	0x8C, 0x01,// 140 = "aas"

	// Inc_r16
	0x00,// No flags set
	0x8D, 0x01,// 141 = "inc"

	// Inc_r32
	0x02,// SameAsPrev

	// Dec_r16
	0x00,// No flags set
	0x8E, 0x01,// 142 = "dec"

	// Dec_r32
	0x02,// SameAsPrev

	// Push_r16
	0x00,// No flags set
	0x03,// 3 = "push"

	// Push_r32
	0x02,// SameAsPrev

	// Push_r64
	0x02,// SameAsPrev

	// Pop_r16
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Pop_r32
	0x02,// SameAsPrev

	// Pop_r64
	0x02,// SameAsPrev

	// Pushaw
	0x00,// No flags set
	0x8F, 0x01,// 143 = "pusha"

	// Pushad
	0x00,// No flags set
	0x90, 0x01,// 144 = "pushad"

	// Popaw
	0x00,// No flags set
	0x91, 0x01,// 145 = "popa"

	// Popad
	0x00,// No flags set
	0x92, 0x01,// 146 = "popad"

	// Bound_r16_m1616
	0x00,// No flags set
	0x93, 0x01,// 147 = "bound"

	// Bound_r32_m3232
	0x02,// SameAsPrev

	// Arpl_rm16_r16
	0x00,// No flags set
	0x94, 0x01,// 148 = "arpl"

	// Arpl_r32m16_r32
	0x02,// SameAsPrev

	// Movsxd_r16_rm16
	0x00,// No flags set
	0x95, 0x01,// 149 = "movsxd"

	// Movsxd_r32_rm32
	0x02,// SameAsPrev

	// Movsxd_r64_rm32
	0x02,// SameAsPrev

	// Push_imm16
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_imm32
	0x02,// SameAsPrev

	// Pushq_imm32
	0x02,// SameAsPrev

	// Imul_r16_rm16_imm16
	0x00,// No flags set
	0x1B,// 27 = "imul"

	// Imul_r32_rm32_imm32
	0x02,// SameAsPrev

	// Imul_r64_rm64_imm32
	0x02,// SameAsPrev

	// Pushw_imm8
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_imm8
	0x02,// SameAsPrev

	// Pushq_imm8
	0x02,// SameAsPrev

	// Imul_r16_rm16_imm8
	0x00,// No flags set
	0x1B,// 27 = "imul"

	// Imul_r32_rm32_imm8
	0x02,// SameAsPrev

	// Imul_r64_rm64_imm8
	0x02,// SameAsPrev

	// Insb_m8_DX
	0x00,// No flags set
	0x98, 0x01,// 152 = "insb"

	// Insw_m16_DX
	0x00,// No flags set
	0x99, 0x01,// 153 = "insw"

	// Insd_m32_DX
	0x00,// No flags set
	0x9A, 0x01,// 154 = "insd"

	// Outsb_DX_m8
	0x00,// No flags set
	0x9C, 0x01,// 156 = "outsb"

	// Outsw_DX_m16
	0x00,// No flags set
	0x9D, 0x01,// 157 = "outsw"

	// Outsd_DX_m32
	0x00,// No flags set
	0x9E, 0x01,// 158 = "outsd"

	// Jo_rel8_16
	0x00,// No flags set
	0x9F, 0x01,// 159 = "jo"

	// Jo_rel8_32
	0x02,// SameAsPrev

	// Jo_rel8_64
	0x02,// SameAsPrev

	// Jno_rel8_16
	0x00,// No flags set
	0xA0, 0x01,// 160 = "jno"

	// Jno_rel8_32
	0x02,// SameAsPrev

	// Jno_rel8_64
	0x02,// SameAsPrev

	// Jb_rel8_16
	0x00,// No flags set
	0xA1, 0x01,// 161 = "jb"

	// Jb_rel8_32
	0x02,// SameAsPrev

	// Jb_rel8_64
	0x02,// SameAsPrev

	// Jae_rel8_16
	0x00,// No flags set
	0xA4, 0x01,// 164 = "jae"

	// Jae_rel8_32
	0x02,// SameAsPrev

	// Jae_rel8_64
	0x02,// SameAsPrev

	// Je_rel8_16
	0x00,// No flags set
	0xA7, 0x01,// 167 = "je"

	// Je_rel8_32
	0x02,// SameAsPrev

	// Je_rel8_64
	0x02,// SameAsPrev

	// Jne_rel8_16
	0x00,// No flags set
	0xA9, 0x01,// 169 = "jne"

	// Jne_rel8_32
	0x02,// SameAsPrev

	// Jne_rel8_64
	0x02,// SameAsPrev

	// Jbe_rel8_16
	0x00,// No flags set
	0xAB, 0x01,// 171 = "jbe"

	// Jbe_rel8_32
	0x02,// SameAsPrev

	// Jbe_rel8_64
	0x02,// SameAsPrev

	// Ja_rel8_16
	0x00,// No flags set
	0xAD, 0x01,// 173 = "ja"

	// Ja_rel8_32
	0x02,// SameAsPrev

	// Ja_rel8_64
	0x02,// SameAsPrev

	// Js_rel8_16
	0x00,// No flags set
	0xAF, 0x01,// 175 = "js"

	// Js_rel8_32
	0x02,// SameAsPrev

	// Js_rel8_64
	0x02,// SameAsPrev

	// Jns_rel8_16
	0x00,// No flags set
	0xB0, 0x01,// 176 = "jns"

	// Jns_rel8_32
	0x02,// SameAsPrev

	// Jns_rel8_64
	0x02,// SameAsPrev

	// Jp_rel8_16
	0x00,// No flags set
	0xB1, 0x01,// 177 = "jp"

	// Jp_rel8_32
	0x02,// SameAsPrev

	// Jp_rel8_64
	0x02,// SameAsPrev

	// Jnp_rel8_16
	0x00,// No flags set
	0xB3, 0x01,// 179 = "jnp"

	// Jnp_rel8_32
	0x02,// SameAsPrev

	// Jnp_rel8_64
	0x02,// SameAsPrev

	// Jl_rel8_16
	0x00,// No flags set
	0xB5, 0x01,// 181 = "jl"

	// Jl_rel8_32
	0x02,// SameAsPrev

	// Jl_rel8_64
	0x02,// SameAsPrev

	// Jge_rel8_16
	0x00,// No flags set
	0xB7, 0x01,// 183 = "jge"

	// Jge_rel8_32
	0x02,// SameAsPrev

	// Jge_rel8_64
	0x02,// SameAsPrev

	// Jle_rel8_16
	0x00,// No flags set
	0xB9, 0x01,// 185 = "jle"

	// Jle_rel8_32
	0x02,// SameAsPrev

	// Jle_rel8_64
	0x02,// SameAsPrev

	// Jg_rel8_16
	0x00,// No flags set
	0xBB, 0x01,// 187 = "jg"

	// Jg_rel8_32
	0x02,// SameAsPrev

	// Jg_rel8_64
	0x02,// SameAsPrev

	// Add_rm8_imm8
	0x04,// ForceMemSize
	0x04,// 4 = "add"

	// Or_rm8_imm8
	0x04,// ForceMemSize
	0x05,// 5 = "or"

	// Adc_rm8_imm8
	0x04,// ForceMemSize
	0x06,// 6 = "adc"

	// Sbb_rm8_imm8
	0x04,// ForceMemSize
	0x07,// 7 = "sbb"

	// And_rm8_imm8
	0x04,// ForceMemSize
	0x08,// 8 = "and"

	// Sub_rm8_imm8
	0x04,// ForceMemSize
	0x09,// 9 = "sub"

	// Xor_rm8_imm8
	0x04,// ForceMemSize
	0x0A,// 10 = "xor"

	// Cmp_rm8_imm8
	0x04,// ForceMemSize
	0x0B,// 11 = "cmp"

	// Add_rm16_imm16
	0x04,// ForceMemSize
	0x04,// 4 = "add"

	// Add_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Add_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Or_rm16_imm16
	0x04,// ForceMemSize
	0x05,// 5 = "or"

	// Or_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Or_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Adc_rm16_imm16
	0x04,// ForceMemSize
	0x06,// 6 = "adc"

	// Adc_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Adc_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Sbb_rm16_imm16
	0x04,// ForceMemSize
	0x07,// 7 = "sbb"

	// Sbb_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Sbb_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// And_rm16_imm16
	0x04,// ForceMemSize
	0x08,// 8 = "and"

	// And_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// And_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Sub_rm16_imm16
	0x04,// ForceMemSize
	0x09,// 9 = "sub"

	// Sub_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Sub_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Xor_rm16_imm16
	0x04,// ForceMemSize
	0x0A,// 10 = "xor"

	// Xor_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Xor_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Cmp_rm16_imm16
	0x04,// ForceMemSize
	0x0B,// 11 = "cmp"

	// Cmp_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Cmp_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Add_rm8_imm8_82
	0x04,// ForceMemSize
	0x04,// 4 = "add"

	// Or_rm8_imm8_82
	0x04,// ForceMemSize
	0x05,// 5 = "or"

	// Adc_rm8_imm8_82
	0x04,// ForceMemSize
	0x06,// 6 = "adc"

	// Sbb_rm8_imm8_82
	0x04,// ForceMemSize
	0x07,// 7 = "sbb"

	// And_rm8_imm8_82
	0x04,// ForceMemSize
	0x08,// 8 = "and"

	// Sub_rm8_imm8_82
	0x04,// ForceMemSize
	0x09,// 9 = "sub"

	// Xor_rm8_imm8_82
	0x04,// ForceMemSize
	0x0A,// 10 = "xor"

	// Cmp_rm8_imm8_82
	0x04,// ForceMemSize
	0x0B,// 11 = "cmp"

	// Add_rm16_imm8
	0x04,// ForceMemSize
	0x04,// 4 = "add"

	// Add_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Add_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Or_rm16_imm8
	0x04,// ForceMemSize
	0x05,// 5 = "or"

	// Or_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Or_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Adc_rm16_imm8
	0x04,// ForceMemSize
	0x06,// 6 = "adc"

	// Adc_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Adc_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sbb_rm16_imm8
	0x04,// ForceMemSize
	0x07,// 7 = "sbb"

	// Sbb_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sbb_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// And_rm16_imm8
	0x04,// ForceMemSize
	0x08,// 8 = "and"

	// And_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// And_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sub_rm16_imm8
	0x04,// ForceMemSize
	0x09,// 9 = "sub"

	// Sub_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sub_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Xor_rm16_imm8
	0x04,// ForceMemSize
	0x0A,// 10 = "xor"

	// Xor_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Xor_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Cmp_rm16_imm8
	0x04,// ForceMemSize
	0x0B,// 11 = "cmp"

	// Cmp_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Cmp_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Test_rm8_r8
	0x00,// No flags set
	0x0D,// 13 = "test"

	// Test_rm16_r16
	0x02,// SameAsPrev

	// Test_rm32_r32
	0x02,// SameAsPrev

	// Test_rm64_r64
	0x02,// SameAsPrev

	// Xchg_rm8_r8
	0x00,// No flags set
	0x3D,// 61 = "xchg"

	// Xchg_rm16_r16
	0x02,// SameAsPrev

	// Xchg_rm32_r32
	0x02,// SameAsPrev

	// Xchg_rm64_r64
	0x02,// SameAsPrev

	// Mov_rm8_r8
	0x00,// No flags set
	0x01,// 1 = "mov"

	// Mov_rm16_r16
	0x02,// SameAsPrev

	// Mov_rm32_r32
	0x02,// SameAsPrev

	// Mov_rm64_r64
	0x02,// SameAsPrev

	// Mov_r8_rm8
	0x02,// SameAsPrev

	// Mov_r16_rm16
	0x02,// SameAsPrev

	// Mov_r32_rm32
	0x02,// SameAsPrev

	// Mov_r64_rm64
	0x02,// SameAsPrev

	// Mov_rm16_Sreg
	0x02,// SameAsPrev

	// Mov_r32m16_Sreg
	0x02,// SameAsPrev

	// Mov_r64m16_Sreg
	0x02,// SameAsPrev

	// Lea_r16_m
	0x00,// No flags set
	0xBD, 0x01,// 189 = "lea"

	// Lea_r32_m
	0x02,// SameAsPrev

	// Lea_r64_m
	0x02,// SameAsPrev

	// Mov_Sreg_rm16
	0x00,// No flags set
	0x01,// 1 = "mov"

	// Mov_Sreg_r32m16
	0x02,// SameAsPrev

	// Mov_Sreg_r64m16
	0x02,// SameAsPrev

	// Pop_rm16
	0x04,// ForceMemSize
	0x0C,// 12 = "pop"

	// Pop_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Pop_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Nopw
	0x00,// No flags set
	0x02,// 2 = "nop"

	// Nopd
	0x02,// SameAsPrev

	// Nopq
	0x02,// SameAsPrev

	// Xchg_r16_AX
	0x00,// No flags set
	0x3D,// 61 = "xchg"

	// Xchg_r32_EAX
	0x02,// SameAsPrev

	// Xchg_r64_RAX
	0x02,// SameAsPrev

	// Pause
	0x00,// No flags set
	0xBE, 0x01,// 190 = "pause"

	// Cbw
	0x00,// No flags set
	0xC0, 0x01,// 192 = "cbw"

	// Cwde
	0x00,// No flags set
	0xC1, 0x01,// 193 = "cwde"

	// Cdqe
	0x00,// No flags set
	0xC3, 0x01,// 195 = "cdqe"

	// Cwd
	0x00,// No flags set
	0xC5, 0x01,// 197 = "cwd"

	// Cdq
	0x00,// No flags set
	0xC7, 0x01,// 199 = "cdq"

	// Cqo
	0x00,// No flags set
	0xC9, 0x01,// 201 = "cqo"

	// Call_ptr1616
	0x00,// No flags set
	0x2B,// 43 = "call"

	// Call_ptr1632
	0x02,// SameAsPrev

	// Wait
	0x00,// No flags set
	0xCD, 0x01,// 205 = "wait"

	// Pushfw
	0x00,// No flags set
	0xCE, 0x01,// 206 = "pushf"

	// Pushfd
	0x00,// No flags set
	0xCF, 0x01,// 207 = "pushfd"

	// Pushfq
	0x00,// No flags set
	0xD0, 0x01,// 208 = "pushfq"

	// Popfw
	0x00,// No flags set
	0xD1, 0x01,// 209 = "popf"

	// Popfd
	0x00,// No flags set
	0xD2, 0x01,// 210 = "popfd"

	// Popfq
	0x00,// No flags set
	0xD3, 0x01,// 211 = "popfq"

	// Sahf
	0x00,// No flags set
	0xD4, 0x01,// 212 = "sahf"

	// Lahf
	0x00,// No flags set
	0xD5, 0x01,// 213 = "lahf"

	// Mov_AL_moffs8
	0x00,// No flags set
	0x01,// 1 = "mov"

	// Mov_AX_moffs16
	0x02,// SameAsPrev

	// Mov_EAX_moffs32
	0x02,// SameAsPrev

	// Mov_RAX_moffs64
	0x02,// SameAsPrev

	// Mov_moffs8_AL
	0x02,// SameAsPrev

	// Mov_moffs16_AX
	0x02,// SameAsPrev

	// Mov_moffs32_EAX
	0x02,// SameAsPrev

	// Mov_moffs64_RAX
	0x02,// SameAsPrev

	// Movsb_m8_m8
	0x00,// No flags set
	0xD8, 0x01,// 216 = "movsb"

	// Movsw_m16_m16
	0x00,// No flags set
	0xD9, 0x01,// 217 = "movsw"

	// Movsd_m32_m32
	0x00,// No flags set
	0x25,// 37 = "movsd"

	// Movsq_m64_m64
	0x00,// No flags set
	0xDA, 0x01,// 218 = "movsq"

	// Cmpsb_m8_m8
	0x00,// No flags set
	0xDC, 0x01,// 220 = "cmpsb"

	// Cmpsw_m16_m16
	0x00,// No flags set
	0xDD, 0x01,// 221 = "cmpsw"

	// Cmpsd_m32_m32
	0x00,// No flags set
	0x9D, 0x06,// 797 = "cmpsd"

	// Cmpsq_m64_m64
	0x00,// No flags set
	0xDE, 0x01,// 222 = "cmpsq"

	// Test_AL_imm8
	0x00,// No flags set
	0x0D,// 13 = "test"

	// Test_AX_imm16
	0x02,// SameAsPrev

	// Test_EAX_imm32
	0x02,// SameAsPrev

	// Test_RAX_imm32
	0x02,// SameAsPrev

	// Stosb_m8_AL
	0x00,// No flags set
	0xE0, 0x01,// 224 = "stosb"

	// Stosw_m16_AX
	0x00,// No flags set
	0xE1, 0x01,// 225 = "stosw"

	// Stosd_m32_EAX
	0x00,// No flags set
	0xE2, 0x01,// 226 = "stosd"

	// Stosq_m64_RAX
	0x00,// No flags set
	0xE3, 0x01,// 227 = "stosq"

	// Lodsb_AL_m8
	0x00,// No flags set
	0xE5, 0x01,// 229 = "lodsb"

	// Lodsw_AX_m16
	0x00,// No flags set
	0xE6, 0x01,// 230 = "lodsw"

	// Lodsd_EAX_m32
	0x00,// No flags set
	0xE7, 0x01,// 231 = "lodsd"

	// Lodsq_RAX_m64
	0x00,// No flags set
	0xE8, 0x01,// 232 = "lodsq"

	// Scasb_AL_m8
	0x00,// No flags set
	0xEA, 0x01,// 234 = "scasb"

	// Scasw_AX_m16
	0x00,// No flags set
	0xEB, 0x01,// 235 = "scasw"

	// Scasd_EAX_m32
	0x00,// No flags set
	0xEC, 0x01,// 236 = "scasd"

	// Scasq_RAX_m64
	0x00,// No flags set
	0xED, 0x01,// 237 = "scasq"

	// Mov_r8_imm8
	0x00,// No flags set
	0x01,// 1 = "mov"

	// Mov_r16_imm16
	0x02,// SameAsPrev

	// Mov_r32_imm32
	0x02,// SameAsPrev

	// Mov_r64_imm64
	0x02,// SameAsPrev

	// Rol_rm8_imm8
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Ror_rm8_imm8
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Rcl_rm8_imm8
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcr_rm8_imm8
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Shl_rm8_imm8
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shr_rm8_imm8
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Sal_rm8_imm8
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sar_rm8_imm8
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Rol_rm16_imm8
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Rol_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Rol_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm16_imm8
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Ror_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm16_imm8
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcl_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm16_imm8
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Rcr_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm16_imm8
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shl_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm16_imm8
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Shr_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm16_imm8
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sal_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm16_imm8
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Sar_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Retnw_imm16
	0x00,// No flags set
	0x2A,// 42 = "ret"

	// Retnd_imm16
	0x02,// SameAsPrev

	// Retnq_imm16
	0x02,// SameAsPrev

	// Retnw
	0x02,// SameAsPrev

	// Retnd
	0x02,// SameAsPrev

	// Retnq
	0x02,// SameAsPrev

	// Les_r16_m1616
	0x00,// No flags set
	0xF2, 0x01,// 242 = "les"

	// Les_r32_m1632
	0x02,// SameAsPrev

	// Lds_r16_m1616
	0x00,// No flags set
	0xF3, 0x01,// 243 = "lds"

	// Lds_r32_m1632
	0x02,// SameAsPrev

	// Mov_rm8_imm8
	0x04,// ForceMemSize
	0x01,// 1 = "mov"

	// Xabort_imm8
	0x00,// No flags set
	0xF4, 0x01,// 244 = "xabort"

	// Mov_rm16_imm16
	0x04,// ForceMemSize
	0x01,// 1 = "mov"

	// Mov_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Mov_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Xbegin_rel16
	0x00,// No flags set
	0xF5, 0x01,// 245 = "xbegin"

	// Xbegin_rel32
	0x02,// SameAsPrev

	// Enterw_imm16_imm8
	0x00,// No flags set
	0xF6, 0x01,// 246 = "enter"

	// Enterd_imm16_imm8
	0x02,// SameAsPrev

	// Enterq_imm16_imm8
	0x02,// SameAsPrev

	// Leavew
	0x00,// No flags set
	0xF7, 0x01,// 247 = "leave"

	// Leaved
	0x02,// SameAsPrev

	// Leaveq
	0x02,// SameAsPrev

	// Retfw_imm16
	0x00,// No flags set
	0xF9, 0x01,// 249 = "retf"

	// Retfd_imm16
	0x02,// SameAsPrev

	// Retfq_imm16
	0x00,// No flags set
	0xFC, 0x01,// 252 = "retfq"

	// Retfw
	0x00,// No flags set
	0xF9, 0x01,// 249 = "retf"

	// Retfd
	0x02,// SameAsPrev

	// Retfq
	0x00,// No flags set
	0xFC, 0x01,// 252 = "retfq"

	// Int3
	0x00,// No flags set
	0xFD, 0x01,// 253 = "int3"

	// Int_imm8
	0x00,// No flags set
	0xFE, 0x01,// 254 = "int"

	// Into
	0x00,// No flags set
	0xFF, 0x01,// 255 = "into"

	// Iretw
	0x00,// No flags set
	0x80, 0x02,// 256 = "iret"

	// Iretd
	0x00,// No flags set
	0x81, 0x02,// 257 = "iretd"

	// Iretq
	0x00,// No flags set
	0x82, 0x02,// 258 = "iretq"

	// Rol_rm8_1
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Ror_rm8_1
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Rcl_rm8_1
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcr_rm8_1
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Shl_rm8_1
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shr_rm8_1
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Sal_rm8_1
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sar_rm8_1
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Rol_rm16_1
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Rol_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Rol_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm16_1
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Ror_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm16_1
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcl_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm16_1
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Rcr_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm16_1
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shl_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm16_1
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Shr_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm16_1
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sal_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm16_1
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Sar_rm32_1
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm64_1
	0x06,// SameAsPrev, ForceMemSize

	// Rol_rm8_CL
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Ror_rm8_CL
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Rcl_rm8_CL
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcr_rm8_CL
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Shl_rm8_CL
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shr_rm8_CL
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Sal_rm8_CL
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sar_rm8_CL
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Rol_rm16_CL
	0x04,// ForceMemSize
	0x1D,// 29 = "rol"

	// Rol_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Rol_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm16_CL
	0x04,// ForceMemSize
	0x1E,// 30 = "ror"

	// Ror_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Ror_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm16_CL
	0x04,// ForceMemSize
	0x1F,// 31 = "rcl"

	// Rcl_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Rcl_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm16_CL
	0x04,// ForceMemSize
	0x20,// 32 = "rcr"

	// Rcr_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Rcr_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm16_CL
	0x04,// ForceMemSize
	0x12,// 18 = "shl"

	// Shl_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Shl_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm16_CL
	0x04,// ForceMemSize
	0x21,// 33 = "shr"

	// Shr_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Shr_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm16_CL
	0x04,// ForceMemSize
	0x29,// 41 = "sal"

	// Sal_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Sal_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm16_CL
	0x04,// ForceMemSize
	0x22,// 34 = "sar"

	// Sar_rm32_CL
	0x06,// SameAsPrev, ForceMemSize

	// Sar_rm64_CL
	0x06,// SameAsPrev, ForceMemSize

	// Aam_imm8
	0x00,// No flags set
	0x83, 0x02,// 259 = "aam"

	// Aad_imm8
	0x00,// No flags set
	0x84, 0x02,// 260 = "aad"

	// Salc
	0x00,// No flags set
	0x85, 0x02,// 261 = "salc"

	// Xlat_m8
	0x00,// No flags set
	0x86, 0x02,// 262 = "xlat"

	// Fadd_m32fp
	0x04,// ForceMemSize
	0x88, 0x02,// 264 = "fadd"

	// Fmul_m32fp
	0x04,// ForceMemSize
	0x89, 0x02,// 265 = "fmul"

	// Fcom_m32fp
	0x04,// ForceMemSize
	0x8A, 0x02,// 266 = "fcom"

	// Fcomp_m32fp
	0x04,// ForceMemSize
	0x8B, 0x02,// 267 = "fcomp"

	// Fsub_m32fp
	0x04,// ForceMemSize
	0x8C, 0x02,// 268 = "fsub"

	// Fsubr_m32fp
	0x04,// ForceMemSize
	0x8D, 0x02,// 269 = "fsubr"

	// Fdiv_m32fp
	0x04,// ForceMemSize
	0x8E, 0x02,// 270 = "fdiv"

	// Fdivr_m32fp
	0x04,// ForceMemSize
	0x8F, 0x02,// 271 = "fdivr"

	// Fadd_st0_sti
	0x00,// No flags set
	0x88, 0x02,// 264 = "fadd"

	// Fmul_st0_sti
	0x00,// No flags set
	0x89, 0x02,// 265 = "fmul"

	// Fcom_st0_sti
	0x00,// No flags set
	0x8A, 0x02,// 266 = "fcom"

	// Fcomp_st0_sti
	0x00,// No flags set
	0x8B, 0x02,// 267 = "fcomp"

	// Fsub_st0_sti
	0x00,// No flags set
	0x8C, 0x02,// 268 = "fsub"

	// Fsubr_st0_sti
	0x00,// No flags set
	0x8D, 0x02,// 269 = "fsubr"

	// Fdiv_st0_sti
	0x00,// No flags set
	0x8E, 0x02,// 270 = "fdiv"

	// Fdivr_st0_sti
	0x00,// No flags set
	0x8F, 0x02,// 271 = "fdivr"

	// Fld_m32fp
	0x04,// ForceMemSize
	0x90, 0x02,// 272 = "fld"

	// Fst_m32fp
	0x04,// ForceMemSize
	0x91, 0x02,// 273 = "fst"

	// Fstp_m32fp
	0x04,// ForceMemSize
	0x7D,// 125 = "fstp"

	// Fldenv_m14byte
	0x04,// ForceMemSize
	0x92, 0x02,// 274 = "fldenv"

	// Fldenv_m28byte
	0x06,// SameAsPrev, ForceMemSize

	// Fldcw_m2byte
	0x00,// No flags set
	0x93, 0x02,// 275 = "fldcw"

	// Fnstenv_m14byte
	0x04,// ForceMemSize
	0x94, 0x02,// 276 = "fnstenv"

	// Fstenv_m14byte
	0x04,// ForceMemSize
	0x95, 0x02,// 277 = "fstenv"

	// Fnstenv_m28byte
	0x04,// ForceMemSize
	0x94, 0x02,// 276 = "fnstenv"

	// Fstenv_m28byte
	0x04,// ForceMemSize
	0x95, 0x02,// 277 = "fstenv"

	// Fnstcw_m2byte
	0x00,// No flags set
	0x96, 0x02,// 278 = "fnstcw"

	// Fstcw_m2byte
	0x00,// No flags set
	0x97, 0x02,// 279 = "fstcw"

	// Fld_sti
	0x00,// No flags set
	0x90, 0x02,// 272 = "fld"

	// Fxch_st0_sti
	0x00,// No flags set
	0x98, 0x02,// 280 = "fxch"

	// Fnop
	0x00,// No flags set
	0x99, 0x02,// 281 = "fnop"

	// Fstpnce_sti
	0x00,// No flags set
	0x7D,// 125 = "fstp"

	// Fchs
	0x00,// No flags set
	0x9B, 0x02,// 283 = "fchs"

	// Fabs
	0x00,// No flags set
	0x9C, 0x02,// 284 = "fabs"

	// Ftst
	0x00,// No flags set
	0x9D, 0x02,// 285 = "ftst"

	// Fxam
	0x00,// No flags set
	0x9E, 0x02,// 286 = "fxam"

	// Fld1
	0x00,// No flags set
	0x9F, 0x02,// 287 = "fld1"

	// Fldl2t
	0x00,// No flags set
	0xA0, 0x02,// 288 = "fldl2t"

	// Fldl2e
	0x00,// No flags set
	0xA1, 0x02,// 289 = "fldl2e"

	// Fldpi
	0x00,// No flags set
	0xA2, 0x02,// 290 = "fldpi"

	// Fldlg2
	0x00,// No flags set
	0xA3, 0x02,// 291 = "fldlg2"

	// Fldln2
	0x00,// No flags set
	0xA4, 0x02,// 292 = "fldln2"

	// Fldz
	0x00,// No flags set
	0xA5, 0x02,// 293 = "fldz"

	// F2xm1
	0x00,// No flags set
	0xA6, 0x02,// 294 = "f2xm1"

	// Fyl2x
	0x00,// No flags set
	0xA7, 0x02,// 295 = "fyl2x"

	// Fptan
	0x00,// No flags set
	0xA8, 0x02,// 296 = "fptan"

	// Fpatan
	0x00,// No flags set
	0xA9, 0x02,// 297 = "fpatan"

	// Fxtract
	0x00,// No flags set
	0xAA, 0x02,// 298 = "fxtract"

	// Fprem1
	0x00,// No flags set
	0xAB, 0x02,// 299 = "fprem1"

	// Fdecstp
	0x00,// No flags set
	0xAC, 0x02,// 300 = "fdecstp"

	// Fincstp
	0x00,// No flags set
	0xAD, 0x02,// 301 = "fincstp"

	// Fprem
	0x00,// No flags set
	0xAE, 0x02,// 302 = "fprem"

	// Fyl2xp1
	0x00,// No flags set
	0xAF, 0x02,// 303 = "fyl2xp1"

	// Fsqrt
	0x00,// No flags set
	0xB0, 0x02,// 304 = "fsqrt"

	// Fsincos
	0x00,// No flags set
	0xB1, 0x02,// 305 = "fsincos"

	// Frndint
	0x00,// No flags set
	0xB2, 0x02,// 306 = "frndint"

	// Fscale
	0x00,// No flags set
	0xB3, 0x02,// 307 = "fscale"

	// Fsin
	0x00,// No flags set
	0xB4, 0x02,// 308 = "fsin"

	// Fcos
	0x00,// No flags set
	0xB5, 0x02,// 309 = "fcos"

	// Fiadd_m32int
	0x04,// ForceMemSize
	0xB6, 0x02,// 310 = "fiadd"

	// Fimul_m32int
	0x04,// ForceMemSize
	0xB7, 0x02,// 311 = "fimul"

	// Ficom_m32int
	0x04,// ForceMemSize
	0xB8, 0x02,// 312 = "ficom"

	// Ficomp_m32int
	0x04,// ForceMemSize
	0xB9, 0x02,// 313 = "ficomp"

	// Fisub_m32int
	0x04,// ForceMemSize
	0xBA, 0x02,// 314 = "fisub"

	// Fisubr_m32int
	0x04,// ForceMemSize
	0xBB, 0x02,// 315 = "fisubr"

	// Fidiv_m32int
	0x04,// ForceMemSize
	0xBC, 0x02,// 316 = "fidiv"

	// Fidivr_m32int
	0x04,// ForceMemSize
	0xBD, 0x02,// 317 = "fidivr"

	// Fcmovb_st0_sti
	0x00,// No flags set
	0xBE, 0x02,// 318 = "fcmovb"

	// Fcmove_st0_sti
	0x00,// No flags set
	0xBF, 0x02,// 319 = "fcmove"

	// Fcmovbe_st0_sti
	0x00,// No flags set
	0xC0, 0x02,// 320 = "fcmovbe"

	// Fcmovu_st0_sti
	0x00,// No flags set
	0xC1, 0x02,// 321 = "fcmovu"

	// Fucompp
	0x00,// No flags set
	0xC2, 0x02,// 322 = "fucompp"

	// Fild_m32int
	0x04,// ForceMemSize
	0xC3, 0x02,// 323 = "fild"

	// Fisttp_m32int
	0x04,// ForceMemSize
	0xC4, 0x02,// 324 = "fisttp"

	// Fist_m32int
	0x04,// ForceMemSize
	0xC5, 0x02,// 325 = "fist"

	// Fistp_m32int
	0x04,// ForceMemSize
	0xC6, 0x02,// 326 = "fistp"

	// Fld_m80fp
	0x04,// ForceMemSize
	0x90, 0x02,// 272 = "fld"

	// Fstp_m80fp
	0x04,// ForceMemSize
	0x7D,// 125 = "fstp"

	// Fcmovnb_st0_sti
	0x00,// No flags set
	0xC7, 0x02,// 327 = "fcmovnb"

	// Fcmovne_st0_sti
	0x00,// No flags set
	0xC8, 0x02,// 328 = "fcmovne"

	// Fcmovnbe_st0_sti
	0x00,// No flags set
	0xC9, 0x02,// 329 = "fcmovnbe"

	// Fcmovnu_st0_sti
	0x00,// No flags set
	0xCA, 0x02,// 330 = "fcmovnu"

	// Fneni
	0x00,// No flags set
	0xCB, 0x02,// 331 = "fneni"

	// Feni
	0x00,// No flags set
	0xCC, 0x02,// 332 = "feni"

	// Fndisi
	0x00,// No flags set
	0xCD, 0x02,// 333 = "fndisi"

	// Fdisi
	0x00,// No flags set
	0xCE, 0x02,// 334 = "fdisi"

	// Fnclex
	0x00,// No flags set
	0xCF, 0x02,// 335 = "fnclex"

	// Fclex
	0x00,// No flags set
	0xD0, 0x02,// 336 = "fclex"

	// Fninit
	0x00,// No flags set
	0xD1, 0x02,// 337 = "fninit"

	// Finit
	0x00,// No flags set
	0xD2, 0x02,// 338 = "finit"

	// Fnsetpm
	0x00,// No flags set
	0xD3, 0x02,// 339 = "fnsetpm"

	// Fsetpm
	0x00,// No flags set
	0xD4, 0x02,// 340 = "fsetpm"

	// Frstpm
	0x00,// No flags set
	0xD5, 0x02,// 341 = "frstpm"

	// Fucomi_st0_sti
	0x00,// No flags set
	0xD6, 0x02,// 342 = "fucomi"

	// Fcomi_st0_sti
	0x00,// No flags set
	0xD7, 0x02,// 343 = "fcomi"

	// Fadd_m64fp
	0x04,// ForceMemSize
	0x88, 0x02,// 264 = "fadd"

	// Fmul_m64fp
	0x04,// ForceMemSize
	0x89, 0x02,// 265 = "fmul"

	// Fcom_m64fp
	0x04,// ForceMemSize
	0x8A, 0x02,// 266 = "fcom"

	// Fcomp_m64fp
	0x04,// ForceMemSize
	0x8B, 0x02,// 267 = "fcomp"

	// Fsub_m64fp
	0x04,// ForceMemSize
	0x8C, 0x02,// 268 = "fsub"

	// Fsubr_m64fp
	0x04,// ForceMemSize
	0x8D, 0x02,// 269 = "fsubr"

	// Fdiv_m64fp
	0x04,// ForceMemSize
	0x8E, 0x02,// 270 = "fdiv"

	// Fdivr_m64fp
	0x04,// ForceMemSize
	0x8F, 0x02,// 271 = "fdivr"

	// Fadd_sti_st0
	0x00,// No flags set
	0x88, 0x02,// 264 = "fadd"

	// Fmul_sti_st0
	0x00,// No flags set
	0x89, 0x02,// 265 = "fmul"

	// Fcom_st0_sti_DCD0
	0x00,// No flags set
	0x8A, 0x02,// 266 = "fcom"

	// Fcomp_st0_sti_DCD8
	0x00,// No flags set
	0x8B, 0x02,// 267 = "fcomp"

	// Fsubr_sti_st0
	0x00,// No flags set
	0x8D, 0x02,// 269 = "fsubr"

	// Fsub_sti_st0
	0x00,// No flags set
	0x8C, 0x02,// 268 = "fsub"

	// Fdivr_sti_st0
	0x00,// No flags set
	0x8F, 0x02,// 271 = "fdivr"

	// Fdiv_sti_st0
	0x00,// No flags set
	0x8E, 0x02,// 270 = "fdiv"

	// Fld_m64fp
	0x04,// ForceMemSize
	0x90, 0x02,// 272 = "fld"

	// Fisttp_m64int
	0x04,// ForceMemSize
	0xC4, 0x02,// 324 = "fisttp"

	// Fst_m64fp
	0x04,// ForceMemSize
	0x91, 0x02,// 273 = "fst"

	// Fstp_m64fp
	0x04,// ForceMemSize
	0x7D,// 125 = "fstp"

	// Frstor_m94byte
	0x04,// ForceMemSize
	0xD9, 0x02,// 345 = "frstor"

	// Frstor_m108byte
	0x06,// SameAsPrev, ForceMemSize

	// Fnsave_m94byte
	0x04,// ForceMemSize
	0xDA, 0x02,// 346 = "fnsave"

	// Fsave_m94byte
	0x04,// ForceMemSize
	0xDB, 0x02,// 347 = "fsave"

	// Fnsave_m108byte
	0x04,// ForceMemSize
	0xDA, 0x02,// 346 = "fnsave"

	// Fsave_m108byte
	0x04,// ForceMemSize
	0xDB, 0x02,// 347 = "fsave"

	// Fnstsw_m2byte
	0x00,// No flags set
	0xDC, 0x02,// 348 = "fnstsw"

	// Fstsw_m2byte
	0x00,// No flags set
	0xDD, 0x02,// 349 = "fstsw"

	// Ffree_sti
	0x00,// No flags set
	0xDE, 0x02,// 350 = "ffree"

	// Fxch_st0_sti_DDC8
	0x00,// No flags set
	0x98, 0x02,// 280 = "fxch"

	// Fst_sti
	0x00,// No flags set
	0x91, 0x02,// 273 = "fst"

	// Fstp_sti
	0x00,// No flags set
	0x7D,// 125 = "fstp"

	// Fucom_st0_sti
	0x00,// No flags set
	0xDF, 0x02,// 351 = "fucom"

	// Fucomp_st0_sti
	0x00,// No flags set
	0xE0, 0x02,// 352 = "fucomp"

	// Fiadd_m16int
	0x04,// ForceMemSize
	0xB6, 0x02,// 310 = "fiadd"

	// Fimul_m16int
	0x04,// ForceMemSize
	0xB7, 0x02,// 311 = "fimul"

	// Ficom_m16int
	0x04,// ForceMemSize
	0xB8, 0x02,// 312 = "ficom"

	// Ficomp_m16int
	0x04,// ForceMemSize
	0xB9, 0x02,// 313 = "ficomp"

	// Fisub_m16int
	0x04,// ForceMemSize
	0xBA, 0x02,// 314 = "fisub"

	// Fisubr_m16int
	0x04,// ForceMemSize
	0xBB, 0x02,// 315 = "fisubr"

	// Fidiv_m16int
	0x04,// ForceMemSize
	0xBC, 0x02,// 316 = "fidiv"

	// Fidivr_m16int
	0x04,// ForceMemSize
	0xBD, 0x02,// 317 = "fidivr"

	// Faddp_sti_st0
	0x00,// No flags set
	0xE1, 0x02,// 353 = "faddp"

	// Fmulp_sti_st0
	0x00,// No flags set
	0xE2, 0x02,// 354 = "fmulp"

	// Fcomp_st0_sti_DED0
	0x00,// No flags set
	0x8B, 0x02,// 267 = "fcomp"

	// Fcompp
	0x00,// No flags set
	0xE3, 0x02,// 355 = "fcompp"

	// Fsubrp_sti_st0
	0x00,// No flags set
	0xE5, 0x02,// 357 = "fsubrp"

	// Fsubp_sti_st0
	0x00,// No flags set
	0xE4, 0x02,// 356 = "fsubp"

	// Fdivrp_sti_st0
	0x00,// No flags set
	0xE7, 0x02,// 359 = "fdivrp"

	// Fdivp_sti_st0
	0x00,// No flags set
	0xE6, 0x02,// 358 = "fdivp"

	// Fild_m16int
	0x04,// ForceMemSize
	0xC3, 0x02,// 323 = "fild"

	// Fisttp_m16int
	0x04,// ForceMemSize
	0xC4, 0x02,// 324 = "fisttp"

	// Fist_m16int
	0x04,// ForceMemSize
	0xC5, 0x02,// 325 = "fist"

	// Fistp_m16int
	0x04,// ForceMemSize
	0xC6, 0x02,// 326 = "fistp"

	// Fbld_m80bcd
	0x04,// ForceMemSize
	0xE8, 0x02,// 360 = "fbld"

	// Fild_m64int
	0x04,// ForceMemSize
	0xC3, 0x02,// 323 = "fild"

	// Fbstp_m80bcd
	0x04,// ForceMemSize
	0xEA, 0x02,// 362 = "fbstp"

	// Fistp_m64int
	0x04,// ForceMemSize
	0xC6, 0x02,// 326 = "fistp"

	// Ffreep_sti
	0x00,// No flags set
	0xEC, 0x02,// 364 = "ffreep"

	// Fxch_st0_sti_DFC8
	0x00,// No flags set
	0x98, 0x02,// 280 = "fxch"

	// Fstp_sti_DFD0
	0x00,// No flags set
	0x7D,// 125 = "fstp"

	// Fstp_sti_DFD8
	0x02,// SameAsPrev

	// Fnstsw_AX
	0x00,// No flags set
	0xDC, 0x02,// 348 = "fnstsw"

	// Fstsw_AX
	0x00,// No flags set
	0xDD, 0x02,// 349 = "fstsw"

	// Fstdw_AX
	0x00,// No flags set
	0xED, 0x02,// 365 = "fstdw"

	// Fstsg_AX
	0x00,// No flags set
	0xEE, 0x02,// 366 = "fstsg"

	// Fucomip_st0_sti
	0x00,// No flags set
	0xEF, 0x02,// 367 = "fucomip"

	// Fcomip_st0_sti
	0x00,// No flags set
	0xF0, 0x02,// 368 = "fcomip"

	// Loopne_rel8_16_CX
	0x00,// No flags set
	0x7E,// 126 = "loopne"

	// Loopne_rel8_32_CX
	0x02,// SameAsPrev

	// Loopne_rel8_16_ECX
	0x02,// SameAsPrev

	// Loopne_rel8_32_ECX
	0x02,// SameAsPrev

	// Loopne_rel8_64_ECX
	0x00,// No flags set
	0xF2, 0x02,// 370 = "loopned"

	// Loopne_rel8_16_RCX
	0x00,// No flags set
	0x7E,// 126 = "loopne"

	// Loopne_rel8_64_RCX
	0x02,// SameAsPrev

	// Loope_rel8_16_CX
	0x00,// No flags set
	0x7F,// 127 = "loope"

	// Loope_rel8_32_CX
	0x02,// SameAsPrev

	// Loope_rel8_16_ECX
	0x02,// SameAsPrev

	// Loope_rel8_32_ECX
	0x02,// SameAsPrev

	// Loope_rel8_64_ECX
	0x00,// No flags set
	0xF5, 0x02,// 373 = "looped"

	// Loope_rel8_16_RCX
	0x00,// No flags set
	0x7F,// 127 = "loope"

	// Loope_rel8_64_RCX
	0x02,// SameAsPrev

	// Loop_rel8_16_CX
	0x00,// No flags set
	0xF7, 0x02,// 375 = "loop"

	// Loop_rel8_32_CX
	0x02,// SameAsPrev

	// Loop_rel8_16_ECX
	0x02,// SameAsPrev

	// Loop_rel8_32_ECX
	0x02,// SameAsPrev

	// Loop_rel8_64_ECX
	0x00,// No flags set
	0xF8, 0x02,// 376 = "loopd"

	// Loop_rel8_16_RCX
	0x00,// No flags set
	0xF7, 0x02,// 375 = "loop"

	// Loop_rel8_64_RCX
	0x02,// SameAsPrev

	// Jcxz_rel8_16
	0x00,// No flags set
	0xF9, 0x02,// 377 = "jcxz"

	// Jcxz_rel8_32
	0x02,// SameAsPrev

	// Jecxz_rel8_16
	0x00,// No flags set
	0xFA, 0x02,// 378 = "jecxz"

	// Jecxz_rel8_32
	0x02,// SameAsPrev

	// Jecxz_rel8_64
	0x02,// SameAsPrev

	// Jrcxz_rel8_16
	0x00,// No flags set
	0xFB, 0x02,// 379 = "jrcxz"

	// Jrcxz_rel8_64
	0x02,// SameAsPrev

	// In_AL_imm8
	0x00,// No flags set
	0xFC, 0x02,// 380 = "in"

	// In_AX_imm8
	0x02,// SameAsPrev

	// In_EAX_imm8
	0x02,// SameAsPrev

	// Out_imm8_AL
	0x00,// No flags set
	0xFD, 0x02,// 381 = "out"

	// Out_imm8_AX
	0x02,// SameAsPrev

	// Out_imm8_EAX
	0x02,// SameAsPrev

	// Call_rel16
	0x00,// No flags set
	0x2B,// 43 = "call"

	// Call_rel32_32
	0x02,// SameAsPrev

	// Call_rel32_64
	0x02,// SameAsPrev

	// Jmp_rel16
	0x00,// No flags set
	0x1C,// 28 = "jmp"

	// Jmp_rel32_32
	0x02,// SameAsPrev

	// Jmp_rel32_64
	0x02,// SameAsPrev

	// Jmp_ptr1616
	0x02,// SameAsPrev

	// Jmp_ptr1632
	0x02,// SameAsPrev

	// Jmp_rel8_16
	0x02,// SameAsPrev

	// Jmp_rel8_32
	0x02,// SameAsPrev

	// Jmp_rel8_64
	0x02,// SameAsPrev

	// In_AL_DX
	0x00,// No flags set
	0xFC, 0x02,// 380 = "in"

	// In_AX_DX
	0x02,// SameAsPrev

	// In_EAX_DX
	0x02,// SameAsPrev

	// Out_DX_AL
	0x00,// No flags set
	0xFD, 0x02,// 381 = "out"

	// Out_DX_AX
	0x02,// SameAsPrev

	// Out_DX_EAX
	0x02,// SameAsPrev

	// Int1
	0x00,// No flags set
	0xFF, 0x02,// 383 = "int1"

	// Hlt
	0x00,// No flags set
	0x80, 0x03,// 384 = "hlt"

	// Cmc
	0x00,// No flags set
	0x81, 0x03,// 385 = "cmc"

	// Test_rm8_imm8
	0x04,// ForceMemSize
	0x0D,// 13 = "test"

	// Test_rm8_imm8_F6r1
	0x06,// SameAsPrev, ForceMemSize

	// Not_rm8
	0x04,// ForceMemSize
	0x82, 0x03,// 386 = "not"

	// Neg_rm8
	0x04,// ForceMemSize
	0x83, 0x03,// 387 = "neg"

	// Mul_rm8
	0x04,// ForceMemSize
	0x84, 0x03,// 388 = "mul"

	// Imul_rm8
	0x04,// ForceMemSize
	0x1B,// 27 = "imul"

	// Div_rm8
	0x04,// ForceMemSize
	0x85, 0x03,// 389 = "div"

	// Idiv_rm8
	0x04,// ForceMemSize
	0x86, 0x03,// 390 = "idiv"

	// Test_rm16_imm16
	0x04,// ForceMemSize
	0x0D,// 13 = "test"

	// Test_rm32_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Test_rm64_imm32
	0x06,// SameAsPrev, ForceMemSize

	// Test_rm16_imm16_F7r1
	0x06,// SameAsPrev, ForceMemSize

	// Test_rm32_imm32_F7r1
	0x06,// SameAsPrev, ForceMemSize

	// Test_rm64_imm32_F7r1
	0x06,// SameAsPrev, ForceMemSize

	// Not_rm16
	0x04,// ForceMemSize
	0x82, 0x03,// 386 = "not"

	// Not_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Not_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Neg_rm16
	0x04,// ForceMemSize
	0x83, 0x03,// 387 = "neg"

	// Neg_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Neg_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Mul_rm16
	0x04,// ForceMemSize
	0x84, 0x03,// 388 = "mul"

	// Mul_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Mul_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Imul_rm16
	0x04,// ForceMemSize
	0x1B,// 27 = "imul"

	// Imul_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Imul_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Div_rm16
	0x04,// ForceMemSize
	0x85, 0x03,// 389 = "div"

	// Div_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Div_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Idiv_rm16
	0x04,// ForceMemSize
	0x86, 0x03,// 390 = "idiv"

	// Idiv_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Idiv_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Clc
	0x00,// No flags set
	0x87, 0x03,// 391 = "clc"

	// Stc
	0x00,// No flags set
	0x88, 0x03,// 392 = "stc"

	// Cli
	0x00,// No flags set
	0x89, 0x03,// 393 = "cli"

	// Sti
	0x00,// No flags set
	0x8A, 0x03,// 394 = "sti"

	// Cld
	0x00,// No flags set
	0x8B, 0x03,// 395 = "cld"

	// Std
	0x00,// No flags set
	0x8C, 0x03,// 396 = "std"

	// Inc_rm8
	0x04,// ForceMemSize
	0x8D, 0x01,// 141 = "inc"

	// Dec_rm8
	0x04,// ForceMemSize
	0x8E, 0x01,// 142 = "dec"

	// Inc_rm16
	0x04,// ForceMemSize
	0x8D, 0x01,// 141 = "inc"

	// Inc_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Inc_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Dec_rm16
	0x04,// ForceMemSize
	0x8E, 0x01,// 142 = "dec"

	// Dec_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Dec_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Call_rm16
	0x04,// ForceMemSize
	0x2B,// 43 = "call"

	// Call_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Call_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Call_m1616
	0x04,// ForceMemSize
	0x8D, 0x03,// 397 = "callf"

	// Call_m1632
	0x06,// SameAsPrev, ForceMemSize

	// Call_m1664
	0x06,// SameAsPrev, ForceMemSize

	// Jmp_rm16
	0x04,// ForceMemSize
	0x1C,// 28 = "jmp"

	// Jmp_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Jmp_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Jmp_m1616
	0x04,// ForceMemSize
	0x8E, 0x03,// 398 = "jmpf"

	// Jmp_m1632
	0x06,// SameAsPrev, ForceMemSize

	// Jmp_m1664
	0x06,// SameAsPrev, ForceMemSize

	// Push_rm16
	0x04,// ForceMemSize
	0x03,// 3 = "push"

	// Push_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Push_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Sldt_rm16
	0x00,// No flags set
	0x8F, 0x03,// 399 = "sldt"

	// Sldt_r32m16
	0x02,// SameAsPrev

	// Sldt_r64m16
	0x02,// SameAsPrev

	// Str_rm16
	0x00,// No flags set
	0x90, 0x03,// 400 = "str"

	// Str_r32m16
	0x02,// SameAsPrev

	// Str_r64m16
	0x02,// SameAsPrev

	// Lldt_rm16
	0x00,// No flags set
	0x91, 0x03,// 401 = "lldt"

	// Lldt_r32m16
	0x02,// SameAsPrev

	// Lldt_r64m16
	0x02,// SameAsPrev

	// Ltr_rm16
	0x00,// No flags set
	0x92, 0x03,// 402 = "ltr"

	// Ltr_r32m16
	0x02,// SameAsPrev

	// Ltr_r64m16
	0x02,// SameAsPrev

	// Verr_rm16
	0x00,// No flags set
	0x93, 0x03,// 403 = "verr"

	// Verr_r32m16
	0x02,// SameAsPrev

	// Verr_r64m16
	0x02,// SameAsPrev

	// Verw_rm16
	0x00,// No flags set
	0x94, 0x03,// 404 = "verw"

	// Verw_r32m16
	0x02,// SameAsPrev

	// Verw_r64m16
	0x02,// SameAsPrev

	// Jmpe_rm16
	0x04,// ForceMemSize
	0x95, 0x03,// 405 = "jmpe"

	// Jmpe_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Sgdt_m1632_16
	0x00,// No flags set
	0x96, 0x03,// 406 = "sgdt"

	// Sgdt_m1632
	0x02,// SameAsPrev

	// Sgdt_m1664
	0x02,// SameAsPrev

	// Sidt_m1632_16
	0x00,// No flags set
	0x97, 0x03,// 407 = "sidt"

	// Sidt_m1632
	0x02,// SameAsPrev

	// Sidt_m1664
	0x02,// SameAsPrev

	// Lgdt_m1632_16
	0x00,// No flags set
	0x98, 0x03,// 408 = "lgdt"

	// Lgdt_m1632
	0x02,// SameAsPrev

	// Lgdt_m1664
	0x02,// SameAsPrev

	// Lidt_m1632_16
	0x00,// No flags set
	0x99, 0x03,// 409 = "lidt"

	// Lidt_m1632
	0x02,// SameAsPrev

	// Lidt_m1664
	0x02,// SameAsPrev

	// Smsw_rm16
	0x00,// No flags set
	0x9A, 0x03,// 410 = "smsw"

	// Smsw_r32m16
	0x02,// SameAsPrev

	// Smsw_r64m16
	0x02,// SameAsPrev

	// Rstorssp_m64
	0x00,// No flags set
	0x9B, 0x03,// 411 = "rstorssp"

	// Lmsw_rm16
	0x00,// No flags set
	0x9C, 0x03,// 412 = "lmsw"

	// Lmsw_r32m16
	0x02,// SameAsPrev

	// Lmsw_r64m16
	0x02,// SameAsPrev

	// Invlpg_m
	0x00,// No flags set
	0x9D, 0x03,// 413 = "invlpg"

	// Enclv
	0x00,// No flags set
	0x9E, 0x03,// 414 = "enclv"

	// Vmcall
	0x00,// No flags set
	0x9F, 0x03,// 415 = "vmcall"

	// Vmlaunch
	0x00,// No flags set
	0xA0, 0x03,// 416 = "vmlaunch"

	// Vmresume
	0x00,// No flags set
	0xA1, 0x03,// 417 = "vmresume"

	// Vmxoff
	0x00,// No flags set
	0xA2, 0x03,// 418 = "vmxoff"

	// Pconfig
	0x00,// No flags set
	0xA3, 0x03,// 419 = "pconfig"

	// Monitorw
	0x00,// No flags set
	0xA4, 0x03,// 420 = "monitor"

	// Monitord
	0x02,// SameAsPrev

	// Monitorq
	0x02,// SameAsPrev

	// Mwait
	0x00,// No flags set
	0xA5, 0x03,// 421 = "mwait"

	// Clac
	0x00,// No flags set
	0xA6, 0x03,// 422 = "clac"

	// Stac
	0x00,// No flags set
	0xA7, 0x03,// 423 = "stac"

	// Encls
	0x00,// No flags set
	0xA8, 0x03,// 424 = "encls"

	// Xgetbv
	0x00,// No flags set
	0xA9, 0x03,// 425 = "xgetbv"

	// Xsetbv
	0x00,// No flags set
	0xAA, 0x03,// 426 = "xsetbv"

	// Vmfunc
	0x00,// No flags set
	0xAB, 0x03,// 427 = "vmfunc"

	// Xend
	0x00,// No flags set
	0xAC, 0x03,// 428 = "xend"

	// Xtest
	0x00,// No flags set
	0xAD, 0x03,// 429 = "xtest"

	// Enclu
	0x00,// No flags set
	0xAE, 0x03,// 430 = "enclu"

	// Vmrunw
	0x00,// No flags set
	0xAF, 0x03,// 431 = "vmrun"

	// Vmrund
	0x02,// SameAsPrev

	// Vmrunq
	0x02,// SameAsPrev

	// Vmmcall
	0x00,// No flags set
	0xB0, 0x03,// 432 = "vmmcall"

	// Vmloadw
	0x00,// No flags set
	0xB1, 0x03,// 433 = "vmload"

	// Vmloadd
	0x02,// SameAsPrev

	// Vmloadq
	0x02,// SameAsPrev

	// Vmsavew
	0x00,// No flags set
	0xB2, 0x03,// 434 = "vmsave"

	// Vmsaved
	0x02,// SameAsPrev

	// Vmsaveq
	0x02,// SameAsPrev

	// Stgi
	0x00,// No flags set
	0xB3, 0x03,// 435 = "stgi"

	// Clgi
	0x00,// No flags set
	0xB4, 0x03,// 436 = "clgi"

	// Skinit
	0x00,// No flags set
	0xB5, 0x03,// 437 = "skinit"

	// Invlpgaw
	0x00,// No flags set
	0xB6, 0x03,// 438 = "invlpga"

	// Invlpgad
	0x02,// SameAsPrev

	// Invlpgaq
	0x02,// SameAsPrev

	// Setssbsy
	0x00,// No flags set
	0xB7, 0x03,// 439 = "setssbsy"

	// Saveprevssp
	0x00,// No flags set
	0xB8, 0x03,// 440 = "saveprevssp"

	// Rdpkru
	0x00,// No flags set
	0xB9, 0x03,// 441 = "rdpkru"

	// Wrpkru
	0x00,// No flags set
	0xBA, 0x03,// 442 = "wrpkru"

	// Swapgs
	0x00,// No flags set
	0xBB, 0x03,// 443 = "swapgs"

	// Rdtscp
	0x00,// No flags set
	0xBC, 0x03,// 444 = "rdtscp"

	// Monitorxw
	0x00,// No flags set
	0xBD, 0x03,// 445 = "monitorx"

	// Monitorxd
	0x02,// SameAsPrev

	// Monitorxq
	0x02,// SameAsPrev

	// Mcommit
	0x00,// No flags set
	0xBE, 0x03,// 446 = "mcommit"

	// Mwaitx
	0x00,// No flags set
	0xBF, 0x03,// 447 = "mwaitx"

	// Clzerow
	0x00,// No flags set
	0xC0, 0x03,// 448 = "clzero"

	// Clzerod
	0x02,// SameAsPrev

	// Clzeroq
	0x02,// SameAsPrev

	// Rdpru
	0x00,// No flags set
	0xC1, 0x03,// 449 = "rdpru"

	// Lar_r16_rm16
	0x00,// No flags set
	0xC2, 0x03,// 450 = "lar"

	// Lar_r32_r32m16
	0x02,// SameAsPrev

	// Lar_r64_r64m16
	0x02,// SameAsPrev

	// Lsl_r16_rm16
	0x00,// No flags set
	0xC3, 0x03,// 451 = "lsl"

	// Lsl_r32_r32m16
	0x02,// SameAsPrev

	// Lsl_r64_r64m16
	0x02,// SameAsPrev

	// Storeall
	0x00,// No flags set
	0xC4, 0x03,// 452 = "storeall"

	// Loadall286
	0x00,// No flags set
	0xC5, 0x03,// 453 = "loadall286"

	// Syscall
	0x00,// No flags set
	0xC6, 0x03,// 454 = "syscall"

	// Clts
	0x00,// No flags set
	0xC7, 0x03,// 455 = "clts"

	// Loadall386
	0x00,// No flags set
	0xC9, 0x03,// 457 = "loadall386"

	// Sysretd
	0x00,// No flags set
	0xCA, 0x03,// 458 = "sysret"

	// Sysretq
	0x00,// No flags set
	0xCB, 0x03,// 459 = "sysretq"

	// Invd
	0x00,// No flags set
	0xCC, 0x03,// 460 = "invd"

	// Wbinvd
	0x00,// No flags set
	0xCD, 0x03,// 461 = "wbinvd"

	// Wbnoinvd
	0x00,// No flags set
	0xCE, 0x03,// 462 = "wbnoinvd"

	// Cl1invmb
	0x00,// No flags set
	0xCF, 0x03,// 463 = "cl1invmb"

	// Ud2
	0x00,// No flags set
	0xD0, 0x03,// 464 = "ud2"

	// Reservednop_rm16_r16_0F0D
	0x00,// No flags set
	0x02,// 2 = "nop"

	// Reservednop_rm32_r32_0F0D
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F0D
	0x02,// SameAsPrev

	// Prefetch_m8
	0x00,// No flags set
	0xD1, 0x03,// 465 = "prefetch"

	// Prefetchw_m8
	0x00,// No flags set
	0xD3, 0x03,// 467 = "prefetchw"

	// Prefetchwt1_m8
	0x00,// No flags set
	0xD4, 0x03,// 468 = "prefetchwt1"

	// Femms
	0x00,// No flags set
	0xD5, 0x03,// 469 = "femms"

	// Umov_rm8_r8
	0x00,// No flags set
	0xD6, 0x03,// 470 = "umov"

	// Umov_rm16_r16
	0x02,// SameAsPrev

	// Umov_rm32_r32
	0x02,// SameAsPrev

	// Umov_r8_rm8
	0x02,// SameAsPrev

	// Umov_r16_rm16
	0x02,// SameAsPrev

	// Umov_r32_rm32
	0x02,// SameAsPrev

	// Movups_xmm_xmmm128
	0x00,// No flags set
	0x23,// 35 = "movups"

	// VEX_Vmovups_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovups_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movupd_xmm_xmmm128
	0x00,// No flags set
	0x24,// 36 = "movupd"

	// VEX_Vmovupd_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovupd_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movss_xmm_xmmm32
	0x00,// No flags set
	0x26,// 38 = "movss"

	// VEX_Vmovss_xmm_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovss_xmm_m32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovss_xmm_k1z_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovss_xmm_k1z_m32
	0x03,// HasVPrefix, SameAsPrev

	// Movsd_xmm_xmmm64
	0x00,// No flags set
	0x25,// 37 = "movsd"

	// VEX_Vmovsd_xmm_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovsd_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsd_xmm_k1z_m64
	0x03,// HasVPrefix, SameAsPrev

	// Movups_xmmm128_xmm
	0x00,// No flags set
	0x23,// 35 = "movups"

	// VEX_Vmovups_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovups_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_xmmm128_k1z_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_ymmm256_k1z_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovups_zmmm512_k1z_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Movupd_xmmm128_xmm
	0x00,// No flags set
	0x24,// 36 = "movupd"

	// VEX_Vmovupd_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovupd_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_xmmm128_k1z_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_ymmm256_k1z_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovupd_zmmm512_k1z_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Movss_xmmm32_xmm
	0x00,// No flags set
	0x26,// 38 = "movss"

	// VEX_Vmovss_xmm_xmm_xmm_0F11
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovss_m32_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovss_m32_k1_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movsd_xmmm64_xmm
	0x00,// No flags set
	0x25,// 37 = "movsd"

	// VEX_Vmovsd_xmm_xmm_xmm_0F11
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovsd_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsd_m64_k1_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movhlps_xmm_xmm
	0x00,// No flags set
	0xD7, 0x03,// 471 = "movhlps"

	// Movlps_xmm_m64
	0x00,// No flags set
	0xD8, 0x03,// 472 = "movlps"

	// VEX_Vmovhlps_xmm_xmm_xmm
	0x01,// HasVPrefix
	0xD7, 0x03,// 471 = "vmovhlps"

	// VEX_Vmovlps_xmm_xmm_m64
	0x01,// HasVPrefix
	0xD8, 0x03,// 472 = "vmovlps"

	// EVEX_Vmovhlps_xmm_xmm_xmm
	0x01,// HasVPrefix
	0xD7, 0x03,// 471 = "vmovhlps"

	// EVEX_Vmovlps_xmm_xmm_m64
	0x01,// HasVPrefix
	0xD8, 0x03,// 472 = "vmovlps"

	// Movlpd_xmm_m64
	0x00,// No flags set
	0xD9, 0x03,// 473 = "movlpd"

	// VEX_Vmovlpd_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovlpd_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// Movsldup_xmm_xmmm128
	0x00,// No flags set
	0xDA, 0x03,// 474 = "movsldup"

	// VEX_Vmovsldup_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovsldup_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsldup_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsldup_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovsldup_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movddup_xmm_xmmm64
	0x00,// No flags set
	0xDB, 0x03,// 475 = "movddup"

	// VEX_Vmovddup_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovddup_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovddup_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovddup_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovddup_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movlps_m64_xmm
	0x00,// No flags set
	0xD8, 0x03,// 472 = "movlps"

	// VEX_Vmovlps_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovlps_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movlpd_m64_xmm
	0x00,// No flags set
	0xD9, 0x03,// 473 = "movlpd"

	// VEX_Vmovlpd_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovlpd_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Unpcklps_xmm_xmmm128
	0x00,// No flags set
	0xDC, 0x03,// 476 = "unpcklps"

	// VEX_Vunpcklps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vunpcklps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Unpcklpd_xmm_xmmm128
	0x00,// No flags set
	0xDD, 0x03,// 477 = "unpcklpd"

	// VEX_Vunpcklpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vunpcklpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Unpckhps_xmm_xmmm128
	0x00,// No flags set
	0xDE, 0x03,// 478 = "unpckhps"

	// VEX_Vunpckhps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vunpckhps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Unpckhpd_xmm_xmmm128
	0x00,// No flags set
	0xDF, 0x03,// 479 = "unpckhpd"

	// VEX_Vunpckhpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vunpckhpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Movlhps_xmm_xmm
	0x00,// No flags set
	0xE0, 0x03,// 480 = "movlhps"

	// VEX_Vmovlhps_xmm_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovlhps_xmm_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movhps_xmm_m64
	0x00,// No flags set
	0xE1, 0x03,// 481 = "movhps"

	// VEX_Vmovhps_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovhps_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// Movhpd_xmm_m64
	0x00,// No flags set
	0xE2, 0x03,// 482 = "movhpd"

	// VEX_Vmovhpd_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovhpd_xmm_xmm_m64
	0x03,// HasVPrefix, SameAsPrev

	// Movshdup_xmm_xmmm128
	0x00,// No flags set
	0xE3, 0x03,// 483 = "movshdup"

	// VEX_Vmovshdup_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovshdup_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovshdup_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovshdup_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovshdup_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movhps_m64_xmm
	0x00,// No flags set
	0xE1, 0x03,// 481 = "movhps"

	// VEX_Vmovhps_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovhps_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movhpd_m64_xmm
	0x00,// No flags set
	0xE2, 0x03,// 482 = "movhpd"

	// VEX_Vmovhpd_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovhpd_m64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Reservednop_rm16_r16_0F18
	0x00,// No flags set
	0x02,// 2 = "nop"

	// Reservednop_rm32_r32_0F18
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F18
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F19
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F19
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F19
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1A
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1A
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1A
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1B
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1B
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1B
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1C
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1C
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1C
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1D
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1D
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1D
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1E
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1E
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1E
	0x02,// SameAsPrev

	// Reservednop_rm16_r16_0F1F
	0x02,// SameAsPrev

	// Reservednop_rm32_r32_0F1F
	0x02,// SameAsPrev

	// Reservednop_rm64_r64_0F1F
	0x02,// SameAsPrev

	// Prefetchnta_m8
	0x00,// No flags set
	0xE4, 0x03,// 484 = "prefetchnta"

	// Prefetcht0_m8
	0x00,// No flags set
	0xE5, 0x03,// 485 = "prefetcht0"

	// Prefetcht1_m8
	0x00,// No flags set
	0xE6, 0x03,// 486 = "prefetcht1"

	// Prefetcht2_m8
	0x00,// No flags set
	0xE7, 0x03,// 487 = "prefetcht2"

	// Bndldx_bnd_mib
	0x00,// No flags set
	0xE8, 0x03,// 488 = "bndldx"

	// Bndmov_bnd_bndm64
	0x00,// No flags set
	0xE9, 0x03,// 489 = "bndmov"

	// Bndmov_bnd_bndm128
	0x02,// SameAsPrev

	// Bndcl_bnd_rm32
	0x00,// No flags set
	0xEA, 0x03,// 490 = "bndcl"

	// Bndcl_bnd_rm64
	0x02,// SameAsPrev

	// Bndcu_bnd_rm32
	0x00,// No flags set
	0xEB, 0x03,// 491 = "bndcu"

	// Bndcu_bnd_rm64
	0x02,// SameAsPrev

	// Bndstx_mib_bnd
	0x00,// No flags set
	0xEC, 0x03,// 492 = "bndstx"

	// Bndmov_bndm64_bnd
	0x00,// No flags set
	0xE9, 0x03,// 489 = "bndmov"

	// Bndmov_bndm128_bnd
	0x02,// SameAsPrev

	// Bndmk_bnd_m32
	0x00,// No flags set
	0xED, 0x03,// 493 = "bndmk"

	// Bndmk_bnd_m64
	0x02,// SameAsPrev

	// Bndcn_bnd_rm32
	0x00,// No flags set
	0xEE, 0x03,// 494 = "bndcn"

	// Bndcn_bnd_rm64
	0x02,// SameAsPrev

	// Cldemote_m8
	0x00,// No flags set
	0xEF, 0x03,// 495 = "cldemote"

	// Rdsspd_r32
	0x00,// No flags set
	0xF0, 0x03,// 496 = "rdsspd"

	// Rdsspq_r64
	0x00,// No flags set
	0xF1, 0x03,// 497 = "rdsspq"

	// Endbr64
	0x00,// No flags set
	0xF2, 0x03,// 498 = "endbr64"

	// Endbr32
	0x00,// No flags set
	0xF3, 0x03,// 499 = "endbr32"

	// Nop_rm16
	0x04,// ForceMemSize
	0x02,// 2 = "nop"

	// Nop_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Nop_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Mov_r32_cr
	0x00,// No flags set
	0x01,// 1 = "mov"

	// Mov_r64_cr
	0x02,// SameAsPrev

	// Mov_r32_dr
	0x02,// SameAsPrev

	// Mov_r64_dr
	0x02,// SameAsPrev

	// Mov_cr_r32
	0x02,// SameAsPrev

	// Mov_cr_r64
	0x02,// SameAsPrev

	// Mov_dr_r32
	0x02,// SameAsPrev

	// Mov_dr_r64
	0x02,// SameAsPrev

	// Mov_r32_tr
	0x02,// SameAsPrev

	// Mov_tr_r32
	0x02,// SameAsPrev

	// Movaps_xmm_xmmm128
	0x00,// No flags set
	0x13,// 19 = "movaps"

	// VEX_Vmovaps_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovaps_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movapd_xmm_xmmm128
	0x00,// No flags set
	0x14,// 20 = "movapd"

	// VEX_Vmovapd_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovapd_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movaps_xmmm128_xmm
	0x00,// No flags set
	0x13,// 19 = "movaps"

	// VEX_Vmovaps_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovaps_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_xmmm128_k1z_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_ymmm256_k1z_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovaps_zmmm512_k1z_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Movapd_xmmm128_xmm
	0x00,// No flags set
	0x14,// 20 = "movapd"

	// VEX_Vmovapd_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovapd_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_xmmm128_k1z_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_ymmm256_k1z_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovapd_zmmm512_k1z_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Cvtpi2ps_xmm_mmm64
	0x00,// No flags set
	0xF4, 0x03,// 500 = "cvtpi2ps"

	// Cvtpi2pd_xmm_mmm64
	0x00,// No flags set
	0xF5, 0x03,// 501 = "cvtpi2pd"

	// Cvtsi2ss_xmm_rm32
	0x04,// ForceMemSize
	0xF6, 0x03,// 502 = "cvtsi2ss"

	// Cvtsi2ss_xmm_rm64
	0x06,// SameAsPrev, ForceMemSize

	// VEX_Vcvtsi2ss_xmm_xmm_rm32
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// VEX_Vcvtsi2ss_xmm_xmm_rm64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// Cvtsi2sd_xmm_rm32
	0x04,// ForceMemSize
	0xF7, 0x03,// 503 = "cvtsi2sd"

	// Cvtsi2sd_xmm_rm64
	0x06,// SameAsPrev, ForceMemSize

	// VEX_Vcvtsi2sd_xmm_xmm_rm32
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// VEX_Vcvtsi2sd_xmm_xmm_rm64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// Movntps_m128_xmm
	0x00,// No flags set
	0xF8, 0x03,// 504 = "movntps"

	// VEX_Vmovntps_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovntps_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntps_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntps_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntps_m512_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Movntpd_m128_xmm
	0x00,// No flags set
	0xF9, 0x03,// 505 = "movntpd"

	// VEX_Vmovntpd_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovntpd_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntpd_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntpd_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntpd_m512_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Movntss_m32_xmm
	0x00,// No flags set
	0xFA, 0x03,// 506 = "movntss"

	// Movntsd_m64_xmm
	0x00,// No flags set
	0xFB, 0x03,// 507 = "movntsd"

	// Cvttps2pi_mm_xmmm64
	0x00,// No flags set
	0xFC, 0x03,// 508 = "cvttps2pi"

	// Cvttpd2pi_mm_xmmm128
	0x00,// No flags set
	0xFD, 0x03,// 509 = "cvttpd2pi"

	// Cvttss2si_r32_xmmm32
	0x00,// No flags set
	0xFE, 0x03,// 510 = "cvttss2si"

	// Cvttss2si_r64_xmmm32
	0x02,// SameAsPrev

	// VEX_Vcvttss2si_r32_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvttss2si_r64_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttss2si_r32_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttss2si_r64_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Cvttsd2si_r32_xmmm64
	0x00,// No flags set
	0xFF, 0x03,// 511 = "cvttsd2si"

	// Cvttsd2si_r64_xmmm64
	0x02,// SameAsPrev

	// VEX_Vcvttsd2si_r32_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvttsd2si_r64_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttsd2si_r32_xmmm64_sae
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttsd2si_r64_xmmm64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Cvtps2pi_mm_xmmm64
	0x00,// No flags set
	0x80, 0x04,// 512 = "cvtps2pi"

	// Cvtpd2pi_mm_xmmm128
	0x00,// No flags set
	0x81, 0x04,// 513 = "cvtpd2pi"

	// Cvtss2si_r32_xmmm32
	0x00,// No flags set
	0x82, 0x04,// 514 = "cvtss2si"

	// Cvtss2si_r64_xmmm32
	0x02,// SameAsPrev

	// VEX_Vcvtss2si_r32_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtss2si_r64_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtss2si_r32_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtss2si_r64_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Cvtsd2si_r32_xmmm64
	0x00,// No flags set
	0x83, 0x04,// 515 = "cvtsd2si"

	// Cvtsd2si_r64_xmmm64
	0x02,// SameAsPrev

	// VEX_Vcvtsd2si_r32_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtsd2si_r64_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtsd2si_r32_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtsd2si_r64_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Ucomiss_xmm_xmmm32
	0x00,// No flags set
	0x84, 0x04,// 516 = "ucomiss"

	// VEX_Vucomiss_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vucomiss_xmm_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Ucomisd_xmm_xmmm64
	0x00,// No flags set
	0x85, 0x04,// 517 = "ucomisd"

	// VEX_Vucomisd_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vucomisd_xmm_xmmm64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Comiss_xmm_xmmm32
	0x00,// No flags set
	0x86, 0x04,// 518 = "comiss"

	// Comisd_xmm_xmmm64
	0x00,// No flags set
	0x87, 0x04,// 519 = "comisd"

	// VEX_Vcomiss_xmm_xmmm32
	0x01,// HasVPrefix
	0x86, 0x04,// 518 = "vcomiss"

	// VEX_Vcomisd_xmm_xmmm64
	0x01,// HasVPrefix
	0x87, 0x04,// 519 = "vcomisd"

	// EVEX_Vcomiss_xmm_xmmm32_sae
	0x01,// HasVPrefix
	0x86, 0x04,// 518 = "vcomiss"

	// EVEX_Vcomisd_xmm_xmmm64_sae
	0x01,// HasVPrefix
	0x87, 0x04,// 519 = "vcomisd"

	// Wrmsr
	0x00,// No flags set
	0x88, 0x04,// 520 = "wrmsr"

	// Rdtsc
	0x00,// No flags set
	0x89, 0x04,// 521 = "rdtsc"

	// Rdmsr
	0x00,// No flags set
	0x8A, 0x04,// 522 = "rdmsr"

	// Rdpmc
	0x00,// No flags set
	0x8B, 0x04,// 523 = "rdpmc"

	// Sysenter
	0x00,// No flags set
	0x8C, 0x04,// 524 = "sysenter"

	// Sysexitd
	0x00,// No flags set
	0x8D, 0x04,// 525 = "sysexit"

	// Sysexitq
	0x00,// No flags set
	0x8E, 0x04,// 526 = "sysexitq"

	// Getsecd
	0x00,// No flags set
	0x8F, 0x04,// 527 = "getsec"

	// Cmovo_r16_rm16
	0x00,// No flags set
	0x90, 0x04,// 528 = "cmovo"

	// Cmovo_r32_rm32
	0x02,// SameAsPrev

	// Cmovo_r64_rm64
	0x02,// SameAsPrev

	// Cmovno_r16_rm16
	0x00,// No flags set
	0x91, 0x04,// 529 = "cmovno"

	// Cmovno_r32_rm32
	0x02,// SameAsPrev

	// Cmovno_r64_rm64
	0x02,// SameAsPrev

	// Cmovb_r16_rm16
	0x00,// No flags set
	0x92, 0x04,// 530 = "cmovb"

	// Cmovb_r32_rm32
	0x02,// SameAsPrev

	// Cmovb_r64_rm64
	0x02,// SameAsPrev

	// Cmovae_r16_rm16
	0x00,// No flags set
	0x95, 0x04,// 533 = "cmovae"

	// Cmovae_r32_rm32
	0x02,// SameAsPrev

	// Cmovae_r64_rm64
	0x02,// SameAsPrev

	// Cmove_r16_rm16
	0x00,// No flags set
	0x98, 0x04,// 536 = "cmove"

	// Cmove_r32_rm32
	0x02,// SameAsPrev

	// Cmove_r64_rm64
	0x02,// SameAsPrev

	// Cmovne_r16_rm16
	0x00,// No flags set
	0x9A, 0x04,// 538 = "cmovne"

	// Cmovne_r32_rm32
	0x02,// SameAsPrev

	// Cmovne_r64_rm64
	0x02,// SameAsPrev

	// Cmovbe_r16_rm16
	0x00,// No flags set
	0x9C, 0x04,// 540 = "cmovbe"

	// Cmovbe_r32_rm32
	0x02,// SameAsPrev

	// Cmovbe_r64_rm64
	0x02,// SameAsPrev

	// Cmova_r16_rm16
	0x00,// No flags set
	0x9E, 0x04,// 542 = "cmova"

	// Cmova_r32_rm32
	0x02,// SameAsPrev

	// Cmova_r64_rm64
	0x02,// SameAsPrev

	// Cmovs_r16_rm16
	0x00,// No flags set
	0xA0, 0x04,// 544 = "cmovs"

	// Cmovs_r32_rm32
	0x02,// SameAsPrev

	// Cmovs_r64_rm64
	0x02,// SameAsPrev

	// Cmovns_r16_rm16
	0x00,// No flags set
	0xA1, 0x04,// 545 = "cmovns"

	// Cmovns_r32_rm32
	0x02,// SameAsPrev

	// Cmovns_r64_rm64
	0x02,// SameAsPrev

	// Cmovp_r16_rm16
	0x00,// No flags set
	0xA2, 0x04,// 546 = "cmovp"

	// Cmovp_r32_rm32
	0x02,// SameAsPrev

	// Cmovp_r64_rm64
	0x02,// SameAsPrev

	// Cmovnp_r16_rm16
	0x00,// No flags set
	0xA4, 0x04,// 548 = "cmovnp"

	// Cmovnp_r32_rm32
	0x02,// SameAsPrev

	// Cmovnp_r64_rm64
	0x02,// SameAsPrev

	// Cmovl_r16_rm16
	0x00,// No flags set
	0xA6, 0x04,// 550 = "cmovl"

	// Cmovl_r32_rm32
	0x02,// SameAsPrev

	// Cmovl_r64_rm64
	0x02,// SameAsPrev

	// Cmovge_r16_rm16
	0x00,// No flags set
	0xA8, 0x04,// 552 = "cmovge"

	// Cmovge_r32_rm32
	0x02,// SameAsPrev

	// Cmovge_r64_rm64
	0x02,// SameAsPrev

	// Cmovle_r16_rm16
	0x00,// No flags set
	0xAA, 0x04,// 554 = "cmovle"

	// Cmovle_r32_rm32
	0x02,// SameAsPrev

	// Cmovle_r64_rm64
	0x02,// SameAsPrev

	// Cmovg_r16_rm16
	0x00,// No flags set
	0xAC, 0x04,// 556 = "cmovg"

	// Cmovg_r32_rm32
	0x02,// SameAsPrev

	// Cmovg_r64_rm64
	0x02,// SameAsPrev

	// VEX_Kandw_kr_kr_kr
	0x00,// No flags set
	0xAE, 0x04,// 558 = "kandw"

	// VEX_Kandq_kr_kr_kr
	0x00,// No flags set
	0xAF, 0x04,// 559 = "kandq"

	// VEX_Kandb_kr_kr_kr
	0x00,// No flags set
	0xB0, 0x04,// 560 = "kandb"

	// VEX_Kandd_kr_kr_kr
	0x00,// No flags set
	0xB1, 0x04,// 561 = "kandd"

	// VEX_Kandnw_kr_kr_kr
	0x00,// No flags set
	0xB2, 0x04,// 562 = "kandnw"

	// VEX_Kandnq_kr_kr_kr
	0x00,// No flags set
	0xB3, 0x04,// 563 = "kandnq"

	// VEX_Kandnb_kr_kr_kr
	0x00,// No flags set
	0xB4, 0x04,// 564 = "kandnb"

	// VEX_Kandnd_kr_kr_kr
	0x00,// No flags set
	0xB5, 0x04,// 565 = "kandnd"

	// VEX_Knotw_kr_kr
	0x00,// No flags set
	0xB6, 0x04,// 566 = "knotw"

	// VEX_Knotq_kr_kr
	0x00,// No flags set
	0xB7, 0x04,// 567 = "knotq"

	// VEX_Knotb_kr_kr
	0x00,// No flags set
	0xB8, 0x04,// 568 = "knotb"

	// VEX_Knotd_kr_kr
	0x00,// No flags set
	0xB9, 0x04,// 569 = "knotd"

	// VEX_Korw_kr_kr_kr
	0x00,// No flags set
	0xBA, 0x04,// 570 = "korw"

	// VEX_Korq_kr_kr_kr
	0x00,// No flags set
	0xBB, 0x04,// 571 = "korq"

	// VEX_Korb_kr_kr_kr
	0x00,// No flags set
	0xBC, 0x04,// 572 = "korb"

	// VEX_Kord_kr_kr_kr
	0x00,// No flags set
	0xBD, 0x04,// 573 = "kord"

	// VEX_Kxnorw_kr_kr_kr
	0x00,// No flags set
	0xBE, 0x04,// 574 = "kxnorw"

	// VEX_Kxnorq_kr_kr_kr
	0x00,// No flags set
	0xBF, 0x04,// 575 = "kxnorq"

	// VEX_Kxnorb_kr_kr_kr
	0x00,// No flags set
	0xC0, 0x04,// 576 = "kxnorb"

	// VEX_Kxnord_kr_kr_kr
	0x00,// No flags set
	0xC1, 0x04,// 577 = "kxnord"

	// VEX_Kxorw_kr_kr_kr
	0x00,// No flags set
	0xC2, 0x04,// 578 = "kxorw"

	// VEX_Kxorq_kr_kr_kr
	0x00,// No flags set
	0xC3, 0x04,// 579 = "kxorq"

	// VEX_Kxorb_kr_kr_kr
	0x00,// No flags set
	0xC4, 0x04,// 580 = "kxorb"

	// VEX_Kxord_kr_kr_kr
	0x00,// No flags set
	0xC5, 0x04,// 581 = "kxord"

	// VEX_Kaddw_kr_kr_kr
	0x00,// No flags set
	0xC6, 0x04,// 582 = "kaddw"

	// VEX_Kaddq_kr_kr_kr
	0x00,// No flags set
	0xC7, 0x04,// 583 = "kaddq"

	// VEX_Kaddb_kr_kr_kr
	0x00,// No flags set
	0xC8, 0x04,// 584 = "kaddb"

	// VEX_Kaddd_kr_kr_kr
	0x00,// No flags set
	0xC9, 0x04,// 585 = "kaddd"

	// VEX_Kunpckwd_kr_kr_kr
	0x00,// No flags set
	0xCA, 0x04,// 586 = "kunpckwd"

	// VEX_Kunpckdq_kr_kr_kr
	0x00,// No flags set
	0xCB, 0x04,// 587 = "kunpckdq"

	// VEX_Kunpckbw_kr_kr_kr
	0x00,// No flags set
	0xCC, 0x04,// 588 = "kunpckbw"

	// Movmskps_r32_xmm
	0x00,// No flags set
	0xCD, 0x04,// 589 = "movmskps"

	// Movmskps_r64_xmm
	0x02,// SameAsPrev

	// VEX_Vmovmskps_r32_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskps_r64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskps_r32_ymm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskps_r64_ymm
	0x03,// HasVPrefix, SameAsPrev

	// Movmskpd_r32_xmm
	0x00,// No flags set
	0xCE, 0x04,// 590 = "movmskpd"

	// Movmskpd_r64_xmm
	0x02,// SameAsPrev

	// VEX_Vmovmskpd_r32_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskpd_r64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskpd_r32_ymm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovmskpd_r64_ymm
	0x03,// HasVPrefix, SameAsPrev

	// Sqrtps_xmm_xmmm128
	0x00,// No flags set
	0xCF, 0x04,// 591 = "sqrtps"

	// VEX_Vsqrtps_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vsqrtps_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Sqrtpd_xmm_xmmm128
	0x00,// No flags set
	0xD0, 0x04,// 592 = "sqrtpd"

	// VEX_Vsqrtpd_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vsqrtpd_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Sqrtss_xmm_xmmm32
	0x00,// No flags set
	0xD1, 0x04,// 593 = "sqrtss"

	// VEX_Vsqrtss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Sqrtsd_xmm_xmmm64
	0x00,// No flags set
	0xD2, 0x04,// 594 = "sqrtsd"

	// VEX_Vsqrtsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Rsqrtps_xmm_xmmm128
	0x00,// No flags set
	0xD3, 0x04,// 595 = "rsqrtps"

	// VEX_Vrsqrtps_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vrsqrtps_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Rsqrtss_xmm_xmmm32
	0x00,// No flags set
	0xD4, 0x04,// 596 = "rsqrtss"

	// VEX_Vrsqrtss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// Rcpps_xmm_xmmm128
	0x00,// No flags set
	0xD5, 0x04,// 597 = "rcpps"

	// VEX_Vrcpps_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vrcpps_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Rcpss_xmm_xmmm32
	0x00,// No flags set
	0xD6, 0x04,// 598 = "rcpss"

	// VEX_Vrcpss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// Andps_xmm_xmmm128
	0x00,// No flags set
	0xD7, 0x04,// 599 = "andps"

	// VEX_Vandps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vandps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Andpd_xmm_xmmm128
	0x00,// No flags set
	0xD8, 0x04,// 600 = "andpd"

	// VEX_Vandpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vandpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Andnps_xmm_xmmm128
	0x00,// No flags set
	0xD9, 0x04,// 601 = "andnps"

	// VEX_Vandnps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vandnps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Andnpd_xmm_xmmm128
	0x00,// No flags set
	0xDA, 0x04,// 602 = "andnpd"

	// VEX_Vandnpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vandnpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Orps_xmm_xmmm128
	0x00,// No flags set
	0xDB, 0x04,// 603 = "orps"

	// VEX_Vorps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vorps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Orpd_xmm_xmmm128
	0x00,// No flags set
	0xDC, 0x04,// 604 = "orpd"

	// VEX_Vorpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vorpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Xorps_xmm_xmmm128
	0x00,// No flags set
	0xDD, 0x04,// 605 = "xorps"

	// VEX_Vxorps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vxorps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Xorpd_xmm_xmmm128
	0x00,// No flags set
	0xDE, 0x04,// 606 = "xorpd"

	// VEX_Vxorpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vxorpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Addps_xmm_xmmm128
	0x00,// No flags set
	0x3E,// 62 = "addps"

	// VEX_Vaddps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaddps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Addpd_xmm_xmmm128
	0x00,// No flags set
	0x3F,// 63 = "addpd"

	// VEX_Vaddpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaddpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Addss_xmm_xmmm32
	0x00,// No flags set
	0xDF, 0x04,// 607 = "addss"

	// VEX_Vaddss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Addsd_xmm_xmmm64
	0x00,// No flags set
	0xE0, 0x04,// 608 = "addsd"

	// VEX_Vaddsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Mulps_xmm_xmmm128
	0x00,// No flags set
	0x40,// 64 = "mulps"

	// VEX_Vmulps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmulps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Mulpd_xmm_xmmm128
	0x00,// No flags set
	0x41,// 65 = "mulpd"

	// VEX_Vmulpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmulpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Mulss_xmm_xmmm32
	0x00,// No flags set
	0xE1, 0x04,// 609 = "mulss"

	// VEX_Vmulss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Mulsd_xmm_xmmm64
	0x00,// No flags set
	0xE2, 0x04,// 610 = "mulsd"

	// VEX_Vmulsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Cvtps2pd_xmm_xmmm64
	0x00,// No flags set
	0x42,// 66 = "cvtps2pd"

	// VEX_Vcvtps2pd_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtps2pd_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Cvtpd2ps_xmm_xmmm128
	0x00,// No flags set
	0x43,// 67 = "cvtpd2ps"

	// VEX_Vcvtpd2ps_xmm_xmmm128
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// VEX_Vcvtpd2ps_xmm_ymmm256
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Cvtss2sd_xmm_xmmm32
	0x00,// No flags set
	0xE3, 0x04,// 611 = "cvtss2sd"

	// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Cvtsd2ss_xmm_xmmm64
	0x00,// No flags set
	0xE4, 0x04,// 612 = "cvtsd2ss"

	// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Cvtdq2ps_xmm_xmmm128
	0x00,// No flags set
	0xE5, 0x04,// 613 = "cvtdq2ps"

	// VEX_Vcvtdq2ps_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtdq2ps_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0xE6, 0x04,// 614 = "vcvtqq2ps"

	// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// Cvtps2dq_xmm_xmmm128
	0x00,// No flags set
	0xE7, 0x04,// 615 = "cvtps2dq"

	// VEX_Vcvtps2dq_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtps2dq_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Cvttps2dq_xmm_xmmm128
	0x00,// No flags set
	0xE8, 0x04,// 616 = "cvttps2dq"

	// VEX_Vcvttps2dq_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvttps2dq_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Subps_xmm_xmmm128
	0x00,// No flags set
	0x44,// 68 = "subps"

	// VEX_Vsubps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vsubps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Subpd_xmm_xmmm128
	0x00,// No flags set
	0x45,// 69 = "subpd"

	// VEX_Vsubpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vsubpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Subss_xmm_xmmm32
	0x00,// No flags set
	0xE9, 0x04,// 617 = "subss"

	// VEX_Vsubss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Subsd_xmm_xmmm64
	0x00,// No flags set
	0xEA, 0x04,// 618 = "subsd"

	// VEX_Vsubsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Minps_xmm_xmmm128
	0x00,// No flags set
	0xEB, 0x04,// 619 = "minps"

	// VEX_Vminps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vminps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Minpd_xmm_xmmm128
	0x00,// No flags set
	0xEC, 0x04,// 620 = "minpd"

	// VEX_Vminpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vminpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Minss_xmm_xmmm32
	0x00,// No flags set
	0xED, 0x04,// 621 = "minss"

	// VEX_Vminss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Minsd_xmm_xmmm64
	0x00,// No flags set
	0xEE, 0x04,// 622 = "minsd"

	// VEX_Vminsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Divps_xmm_xmmm128
	0x00,// No flags set
	0xEF, 0x04,// 623 = "divps"

	// VEX_Vdivps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vdivps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	0x03,// HasVPrefix, SameAsPrev

	// Divpd_xmm_xmmm128
	0x00,// No flags set
	0xF0, 0x04,// 624 = "divpd"

	// VEX_Vdivpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vdivpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Divss_xmm_xmmm32
	0x00,// No flags set
	0xF1, 0x04,// 625 = "divss"

	// VEX_Vdivss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	0x03,// HasVPrefix, SameAsPrev

	// Divsd_xmm_xmmm64
	0x00,// No flags set
	0xF2, 0x04,// 626 = "divsd"

	// VEX_Vdivsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	0x03,// HasVPrefix, SameAsPrev

	// Maxps_xmm_xmmm128
	0x00,// No flags set
	0xF3, 0x04,// 627 = "maxps"

	// VEX_Vmaxps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmaxps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Maxpd_xmm_xmmm128
	0x00,// No flags set
	0xF4, 0x04,// 628 = "maxpd"

	// VEX_Vmaxpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmaxpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Maxss_xmm_xmmm32
	0x00,// No flags set
	0xF5, 0x04,// 629 = "maxss"

	// VEX_Vmaxss_xmm_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	0x03,// HasVPrefix, SameAsPrev

	// Maxsd_xmm_xmmm64
	0x00,// No flags set
	0xF6, 0x04,// 630 = "maxsd"

	// VEX_Vmaxsd_xmm_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Punpcklbw_mm_mmm32
	0x00,// No flags set
	0x46,// 70 = "punpcklbw"

	// Punpcklbw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Punpcklwd_mm_mmm32
	0x00,// No flags set
	0x47,// 71 = "punpcklwd"

	// Punpcklwd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Punpckldq_mm_mmm32
	0x00,// No flags set
	0x48,// 72 = "punpckldq"

	// Punpckldq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpckldq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpckldq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Packsswb_mm_mmm64
	0x00,// No flags set
	0x49,// 73 = "packsswb"

	// Packsswb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpacksswb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpacksswb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpgtb_mm_mmm64
	0x00,// No flags set
	0x4A,// 74 = "pcmpgtb"

	// Pcmpgtb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpgtw_mm_mmm64
	0x00,// No flags set
	0x4B,// 75 = "pcmpgtw"

	// Pcmpgtw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpgtd_mm_mmm64
	0x00,// No flags set
	0x2E,// 46 = "pcmpgtd"

	// Pcmpgtd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Packuswb_mm_mmm64
	0x00,// No flags set
	0x4C,// 76 = "packuswb"

	// Packuswb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpackuswb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpackuswb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Punpckhbw_mm_mmm64
	0x00,// No flags set
	0x4D,// 77 = "punpckhbw"

	// Punpckhbw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Punpckhwd_mm_mmm64
	0x00,// No flags set
	0x4E,// 78 = "punpckhwd"

	// Punpckhwd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Punpckhdq_mm_mmm64
	0x00,// No flags set
	0x4F,// 79 = "punpckhdq"

	// Punpckhdq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Packssdw_mm_mmm64
	0x00,// No flags set
	0x50,// 80 = "packssdw"

	// Packssdw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpackssdw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpackssdw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Punpcklqdq_xmm_xmmm128
	0x00,// No flags set
	0xF7, 0x04,// 631 = "punpcklqdq"

	// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Punpckhqdq_xmm_xmmm128
	0x00,// No flags set
	0xF8, 0x04,// 632 = "punpckhqdq"

	// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Movd_mm_rm32
	0x00,// No flags set
	0x2F,// 47 = "movd"

	// Movq_mm_rm64
	0x00,// No flags set
	0x0E,// 14 = "movq"

	// Movd_xmm_rm32
	0x00,// No flags set
	0x2F,// 47 = "movd"

	// Movq_xmm_rm64
	0x00,// No flags set
	0x0E,// 14 = "movq"

	// VEX_Vmovd_xmm_rm32
	0x01,// HasVPrefix
	0x2F,// 47 = "vmovd"

	// VEX_Vmovq_xmm_rm64
	0x01,// HasVPrefix
	0x0E,// 14 = "vmovq"

	// EVEX_Vmovd_xmm_rm32
	0x01,// HasVPrefix
	0x2F,// 47 = "vmovd"

	// EVEX_Vmovq_xmm_rm64
	0x01,// HasVPrefix
	0x0E,// 14 = "vmovq"

	// Movq_mm_mmm64
	0x02,// SameAsPrev

	// Movdqa_xmm_xmmm128
	0x00,// No flags set
	0xF9, 0x04,// 633 = "movdqa"

	// VEX_Vmovdqa_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovdqa_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	0x00,// No flags set
	0x30,// 48 = "vmovdqa32"

	// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	0x00,// No flags set
	0x31,// 49 = "vmovdqa64"

	// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// Movdqu_xmm_xmmm128
	0x00,// No flags set
	0xFA, 0x04,// 634 = "movdqu"

	// VEX_Vmovdqu_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovdqu_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFB, 0x04,// 635 = "vmovdqu32"

	// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFC, 0x04,// 636 = "vmovdqu64"

	// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFD, 0x04,// 637 = "vmovdqu8"

	// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFE, 0x04,// 638 = "vmovdqu16"

	// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// Pshufw_mm_mmm64_imm8
	0x00,// No flags set
	0xFF, 0x04,// 639 = "pshufw"

	// Pshufd_xmm_xmmm128_imm8
	0x00,// No flags set
	0x51,// 81 = "pshufd"

	// VEX_Vpshufd_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpshufd_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pshufhw_xmm_xmmm128_imm8
	0x00,// No flags set
	0x80, 0x05,// 640 = "pshufhw"

	// VEX_Vpshufhw_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpshufhw_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pshuflw_xmm_xmmm128_imm8
	0x00,// No flags set
	0x81, 0x05,// 641 = "pshuflw"

	// VEX_Vpshuflw_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpshuflw_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psrlw_mm_imm8
	0x00,// No flags set
	0x15,// 21 = "psrlw"

	// Psrlw_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsrlw_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrlw_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psraw_mm_imm8
	0x00,// No flags set
	0x16,// 22 = "psraw"

	// Psraw_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsraw_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsraw_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psllw_mm_imm8
	0x00,// No flags set
	0x17,// 23 = "psllw"

	// Psllw_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsllw_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsllw_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	0x00,// No flags set
	0x82, 0x05,// 642 = "vprord"

	// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0x83, 0x05,// 643 = "vprorq"

	// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	0x00,// No flags set
	0x84, 0x05,// 644 = "vprold"

	// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0x85, 0x05,// 645 = "vprolq"

	// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// Psrld_mm_imm8
	0x00,// No flags set
	0x0F,// 15 = "psrld"

	// Psrld_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsrld_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrld_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psrad_mm_imm8
	0x00,// No flags set
	0x10,// 16 = "psrad"

	// Psrad_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsrad_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrad_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0x86, 0x05,// 646 = "vpsraq"

	// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// Pslld_mm_imm8
	0x00,// No flags set
	0x11,// 17 = "pslld"

	// Pslld_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpslld_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpslld_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psrlq_mm_imm8
	0x00,// No flags set
	0x18,// 24 = "psrlq"

	// Psrlq_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsrlq_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrlq_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psrldq_xmm_imm8
	0x00,// No flags set
	0x87, 0x05,// 647 = "psrldq"

	// VEX_Vpsrldq_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrldq_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrldq_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrldq_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrldq_zmm_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Psllq_mm_imm8
	0x00,// No flags set
	0x19,// 25 = "psllq"

	// Psllq_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpsllq_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsllq_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pslldq_xmm_imm8
	0x00,// No flags set
	0x88, 0x05,// 648 = "pslldq"

	// VEX_Vpslldq_xmm_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpslldq_ymm_ymm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslldq_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslldq_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslldq_zmm_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpeqb_mm_mmm64
	0x00,// No flags set
	0x52,// 82 = "pcmpeqb"

	// Pcmpeqb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpeqw_mm_mmm64
	0x00,// No flags set
	0x53,// 83 = "pcmpeqw"

	// Pcmpeqw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpeqd_mm_mmm64
	0x00,// No flags set
	0x32,// 50 = "pcmpeqd"

	// Pcmpeqd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Emms
	0x00,// No flags set
	0x89, 0x05,// 649 = "emms"

	// VEX_Vzeroupper
	0x00,// No flags set
	0x8A, 0x05,// 650 = "vzeroupper"

	// VEX_Vzeroall
	0x00,// No flags set
	0x8B, 0x05,// 651 = "vzeroall"

	// Vmread_rm32_r32
	0x00,// No flags set
	0x8C, 0x05,// 652 = "vmread"

	// Vmread_rm64_r64
	0x02,// SameAsPrev

	// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0x8D, 0x05,// 653 = "vcvttps2udq"

	// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0x8E, 0x05,// 654 = "vcvttpd2udq"

	// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	0x02,// SameAsPrev

	// Extrq_xmm_imm8_imm8
	0x00,// No flags set
	0x8F, 0x05,// 655 = "extrq"

	// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	0x00,// No flags set
	0x90, 0x05,// 656 = "vcvttps2uqq"

	// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0x91, 0x05,// 657 = "vcvttpd2uqq"

	// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttss2usi_r32_xmmm32_sae
	0x00,// No flags set
	0x92, 0x05,// 658 = "vcvttss2usi"

	// EVEX_Vcvttss2usi_r64_xmmm32_sae
	0x02,// SameAsPrev

	// Insertq_xmm_xmm_imm8_imm8
	0x00,// No flags set
	0x93, 0x05,// 659 = "insertq"

	// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	0x00,// No flags set
	0x94, 0x05,// 660 = "vcvttsd2usi"

	// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	0x02,// SameAsPrev

	// Vmwrite_r32_rm32
	0x00,// No flags set
	0x95, 0x05,// 661 = "vmwrite"

	// Vmwrite_r64_rm64
	0x02,// SameAsPrev

	// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0x96, 0x05,// 662 = "vcvtps2udq"

	// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0x97, 0x05,// 663 = "vcvtpd2udq"

	// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// Extrq_xmm_xmm
	0x00,// No flags set
	0x8F, 0x05,// 655 = "extrq"

	// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	0x00,// No flags set
	0x98, 0x05,// 664 = "vcvtps2uqq"

	// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0x99, 0x05,// 665 = "vcvtpd2uqq"

	// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_Vcvtss2usi_r32_xmmm32_er
	0x00,// No flags set
	0x9A, 0x05,// 666 = "vcvtss2usi"

	// EVEX_Vcvtss2usi_r64_xmmm32_er
	0x02,// SameAsPrev

	// Insertq_xmm_xmm
	0x00,// No flags set
	0x93, 0x05,// 659 = "insertq"

	// EVEX_Vcvtsd2usi_r32_xmmm64_er
	0x00,// No flags set
	0x9B, 0x05,// 667 = "vcvtsd2usi"

	// EVEX_Vcvtsd2usi_r64_xmmm64_er
	0x02,// SameAsPrev

	// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	0x00,// No flags set
	0x9C, 0x05,// 668 = "vcvttps2qq"

	// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0x9D, 0x05,// 669 = "vcvttpd2qq"

	// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	0x02,// SameAsPrev

	// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	0x00,// No flags set
	0x9E, 0x05,// 670 = "vcvtudq2pd"

	// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0x9F, 0x05,// 671 = "vcvtuqq2pd"

	// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xA0, 0x05,// 672 = "vcvtudq2ps"

	// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0xA1, 0x05,// 673 = "vcvtuqq2ps"

	// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	0x00,// No flags set
	0xA2, 0x05,// 674 = "vcvtps2qq"

	// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xA3, 0x05,// 675 = "vcvtpd2qq"

	// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	0x04,// ForceMemSize
	0xA4, 0x05,// 676 = "vcvtusi2ss"

	// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	0x04,// ForceMemSize
	0xA5, 0x05,// 677 = "vcvtusi2sd"

	// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	0x06,// SameAsPrev, ForceMemSize

	// Haddpd_xmm_xmmm128
	0x00,// No flags set
	0xA6, 0x05,// 678 = "haddpd"

	// VEX_Vhaddpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vhaddpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Haddps_xmm_xmmm128
	0x00,// No flags set
	0xA7, 0x05,// 679 = "haddps"

	// VEX_Vhaddps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vhaddps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Hsubpd_xmm_xmmm128
	0x00,// No flags set
	0xA8, 0x05,// 680 = "hsubpd"

	// VEX_Vhsubpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vhsubpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Hsubps_xmm_xmmm128
	0x00,// No flags set
	0xA9, 0x05,// 681 = "hsubps"

	// VEX_Vhsubps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vhsubps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Movd_rm32_mm
	0x00,// No flags set
	0x2F,// 47 = "movd"

	// Movq_rm64_mm
	0x00,// No flags set
	0x0E,// 14 = "movq"

	// Movd_rm32_xmm
	0x00,// No flags set
	0x2F,// 47 = "movd"

	// Movq_rm64_xmm
	0x00,// No flags set
	0x0E,// 14 = "movq"

	// VEX_Vmovd_rm32_xmm
	0x01,// HasVPrefix
	0x2F,// 47 = "vmovd"

	// VEX_Vmovq_rm64_xmm
	0x01,// HasVPrefix
	0x0E,// 14 = "vmovq"

	// EVEX_Vmovd_rm32_xmm
	0x01,// HasVPrefix
	0x2F,// 47 = "vmovd"

	// EVEX_Vmovq_rm64_xmm
	0x01,// HasVPrefix
	0x0E,// 14 = "vmovq"

	// Movq_xmm_xmmm64
	0x02,// SameAsPrev

	// VEX_Vmovq_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovq_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// Movq_mmm64_mm
	0x02,// SameAsPrev

	// Movdqa_xmmm128_xmm
	0x00,// No flags set
	0xF9, 0x04,// 633 = "movdqa"

	// VEX_Vmovdqa_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovdqa_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	0x00,// No flags set
	0x30,// 48 = "vmovdqa32"

	// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	0x00,// No flags set
	0x31,// 49 = "vmovdqa64"

	// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// Movdqu_xmmm128_xmm
	0x00,// No flags set
	0xFA, 0x04,// 634 = "movdqu"

	// VEX_Vmovdqu_xmmm128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovdqu_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	0x00,// No flags set
	0xFB, 0x04,// 635 = "vmovdqu32"

	// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	0x00,// No flags set
	0xFC, 0x04,// 636 = "vmovdqu64"

	// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	0x00,// No flags set
	0xFD, 0x04,// 637 = "vmovdqu8"

	// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	0x00,// No flags set
	0xFE, 0x04,// 638 = "vmovdqu16"

	// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// Jo_rel16
	0x00,// No flags set
	0x9F, 0x01,// 159 = "jo"

	// Jo_rel32_32
	0x02,// SameAsPrev

	// Jo_rel32_64
	0x02,// SameAsPrev

	// Jno_rel16
	0x00,// No flags set
	0xA0, 0x01,// 160 = "jno"

	// Jno_rel32_32
	0x02,// SameAsPrev

	// Jno_rel32_64
	0x02,// SameAsPrev

	// Jb_rel16
	0x00,// No flags set
	0xA1, 0x01,// 161 = "jb"

	// Jb_rel32_32
	0x02,// SameAsPrev

	// Jb_rel32_64
	0x02,// SameAsPrev

	// Jae_rel16
	0x00,// No flags set
	0xA4, 0x01,// 164 = "jae"

	// Jae_rel32_32
	0x02,// SameAsPrev

	// Jae_rel32_64
	0x02,// SameAsPrev

	// Je_rel16
	0x00,// No flags set
	0xA7, 0x01,// 167 = "je"

	// Je_rel32_32
	0x02,// SameAsPrev

	// Je_rel32_64
	0x02,// SameAsPrev

	// Jne_rel16
	0x00,// No flags set
	0xA9, 0x01,// 169 = "jne"

	// Jne_rel32_32
	0x02,// SameAsPrev

	// Jne_rel32_64
	0x02,// SameAsPrev

	// Jbe_rel16
	0x00,// No flags set
	0xAB, 0x01,// 171 = "jbe"

	// Jbe_rel32_32
	0x02,// SameAsPrev

	// Jbe_rel32_64
	0x02,// SameAsPrev

	// Ja_rel16
	0x00,// No flags set
	0xAD, 0x01,// 173 = "ja"

	// Ja_rel32_32
	0x02,// SameAsPrev

	// Ja_rel32_64
	0x02,// SameAsPrev

	// Js_rel16
	0x00,// No flags set
	0xAF, 0x01,// 175 = "js"

	// Js_rel32_32
	0x02,// SameAsPrev

	// Js_rel32_64
	0x02,// SameAsPrev

	// Jns_rel16
	0x00,// No flags set
	0xB0, 0x01,// 176 = "jns"

	// Jns_rel32_32
	0x02,// SameAsPrev

	// Jns_rel32_64
	0x02,// SameAsPrev

	// Jp_rel16
	0x00,// No flags set
	0xB1, 0x01,// 177 = "jp"

	// Jp_rel32_32
	0x02,// SameAsPrev

	// Jp_rel32_64
	0x02,// SameAsPrev

	// Jnp_rel16
	0x00,// No flags set
	0xB3, 0x01,// 179 = "jnp"

	// Jnp_rel32_32
	0x02,// SameAsPrev

	// Jnp_rel32_64
	0x02,// SameAsPrev

	// Jl_rel16
	0x00,// No flags set
	0xB5, 0x01,// 181 = "jl"

	// Jl_rel32_32
	0x02,// SameAsPrev

	// Jl_rel32_64
	0x02,// SameAsPrev

	// Jge_rel16
	0x00,// No flags set
	0xB7, 0x01,// 183 = "jge"

	// Jge_rel32_32
	0x02,// SameAsPrev

	// Jge_rel32_64
	0x02,// SameAsPrev

	// Jle_rel16
	0x00,// No flags set
	0xB9, 0x01,// 185 = "jle"

	// Jle_rel32_32
	0x02,// SameAsPrev

	// Jle_rel32_64
	0x02,// SameAsPrev

	// Jg_rel16
	0x00,// No flags set
	0xBB, 0x01,// 187 = "jg"

	// Jg_rel32_32
	0x02,// SameAsPrev

	// Jg_rel32_64
	0x02,// SameAsPrev

	// Seto_rm8
	0x00,// No flags set
	0xAA, 0x05,// 682 = "seto"

	// Setno_rm8
	0x00,// No flags set
	0xAB, 0x05,// 683 = "setno"

	// Setb_rm8
	0x00,// No flags set
	0xAC, 0x05,// 684 = "setb"

	// Setae_rm8
	0x00,// No flags set
	0xAF, 0x05,// 687 = "setae"

	// Sete_rm8
	0x00,// No flags set
	0xB2, 0x05,// 690 = "sete"

	// Setne_rm8
	0x00,// No flags set
	0xB4, 0x05,// 692 = "setne"

	// Setbe_rm8
	0x00,// No flags set
	0xB6, 0x05,// 694 = "setbe"

	// Seta_rm8
	0x00,// No flags set
	0xB8, 0x05,// 696 = "seta"

	// Sets_rm8
	0x00,// No flags set
	0xBA, 0x05,// 698 = "sets"

	// Setns_rm8
	0x00,// No flags set
	0xBB, 0x05,// 699 = "setns"

	// Setp_rm8
	0x00,// No flags set
	0xBC, 0x05,// 700 = "setp"

	// Setnp_rm8
	0x00,// No flags set
	0xBE, 0x05,// 702 = "setnp"

	// Setl_rm8
	0x00,// No flags set
	0xC0, 0x05,// 704 = "setl"

	// Setge_rm8
	0x00,// No flags set
	0xC2, 0x05,// 706 = "setge"

	// Setle_rm8
	0x00,// No flags set
	0xC4, 0x05,// 708 = "setle"

	// Setg_rm8
	0x00,// No flags set
	0xC6, 0x05,// 710 = "setg"

	// VEX_Kmovw_kr_km16
	0x00,// No flags set
	0xC8, 0x05,// 712 = "kmovw"

	// VEX_Kmovq_kr_km64
	0x00,// No flags set
	0xC9, 0x05,// 713 = "kmovq"

	// VEX_Kmovb_kr_km8
	0x00,// No flags set
	0xCA, 0x05,// 714 = "kmovb"

	// VEX_Kmovd_kr_km32
	0x00,// No flags set
	0xCB, 0x05,// 715 = "kmovd"

	// VEX_Kmovw_m16_kr
	0x00,// No flags set
	0xC8, 0x05,// 712 = "kmovw"

	// VEX_Kmovq_m64_kr
	0x00,// No flags set
	0xC9, 0x05,// 713 = "kmovq"

	// VEX_Kmovb_m8_kr
	0x00,// No flags set
	0xCA, 0x05,// 714 = "kmovb"

	// VEX_Kmovd_m32_kr
	0x00,// No flags set
	0xCB, 0x05,// 715 = "kmovd"

	// VEX_Kmovw_kr_r32
	0x00,// No flags set
	0xC8, 0x05,// 712 = "kmovw"

	// VEX_Kmovb_kr_r32
	0x00,// No flags set
	0xCA, 0x05,// 714 = "kmovb"

	// VEX_Kmovd_kr_r32
	0x00,// No flags set
	0xCB, 0x05,// 715 = "kmovd"

	// VEX_Kmovq_kr_r64
	0x00,// No flags set
	0xC9, 0x05,// 713 = "kmovq"

	// VEX_Kmovw_r32_kr
	0x00,// No flags set
	0xC8, 0x05,// 712 = "kmovw"

	// VEX_Kmovb_r32_kr
	0x00,// No flags set
	0xCA, 0x05,// 714 = "kmovb"

	// VEX_Kmovd_r32_kr
	0x00,// No flags set
	0xCB, 0x05,// 715 = "kmovd"

	// VEX_Kmovq_r64_kr
	0x00,// No flags set
	0xC9, 0x05,// 713 = "kmovq"

	// VEX_Kortestw_kr_kr
	0x00,// No flags set
	0xCC, 0x05,// 716 = "kortestw"

	// VEX_Kortestq_kr_kr
	0x00,// No flags set
	0xCD, 0x05,// 717 = "kortestq"

	// VEX_Kortestb_kr_kr
	0x00,// No flags set
	0xCE, 0x05,// 718 = "kortestb"

	// VEX_Kortestd_kr_kr
	0x00,// No flags set
	0xCF, 0x05,// 719 = "kortestd"

	// VEX_Ktestw_kr_kr
	0x00,// No flags set
	0xD0, 0x05,// 720 = "ktestw"

	// VEX_Ktestq_kr_kr
	0x00,// No flags set
	0xD1, 0x05,// 721 = "ktestq"

	// VEX_Ktestb_kr_kr
	0x00,// No flags set
	0xD2, 0x05,// 722 = "ktestb"

	// VEX_Ktestd_kr_kr
	0x00,// No flags set
	0xD3, 0x05,// 723 = "ktestd"

	// Pushw_FS
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_FS
	0x02,// SameAsPrev

	// Pushq_FS
	0x02,// SameAsPrev

	// Popw_FS
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Popd_FS
	0x02,// SameAsPrev

	// Popq_FS
	0x02,// SameAsPrev

	// Cpuid
	0x00,// No flags set
	0xD4, 0x05,// 724 = "cpuid"

	// Bt_rm16_r16
	0x00,// No flags set
	0xD5, 0x05,// 725 = "bt"

	// Bt_rm32_r32
	0x02,// SameAsPrev

	// Bt_rm64_r64
	0x02,// SameAsPrev

	// Shld_rm16_r16_imm8
	0x00,// No flags set
	0xD6, 0x05,// 726 = "shld"

	// Shld_rm32_r32_imm8
	0x02,// SameAsPrev

	// Shld_rm64_r64_imm8
	0x02,// SameAsPrev

	// Shld_rm16_r16_CL
	0x02,// SameAsPrev

	// Shld_rm32_r32_CL
	0x02,// SameAsPrev

	// Shld_rm64_r64_CL
	0x02,// SameAsPrev

	// Montmul_16
	0x00,// No flags set
	0xD7, 0x05,// 727 = "montmul"

	// Montmul_32
	0x02,// SameAsPrev

	// Montmul_64
	0x02,// SameAsPrev

	// Xsha1_16
	0x00,// No flags set
	0xD8, 0x05,// 728 = "xsha1"

	// Xsha1_32
	0x02,// SameAsPrev

	// Xsha1_64
	0x02,// SameAsPrev

	// Xsha256_16
	0x00,// No flags set
	0xD9, 0x05,// 729 = "xsha256"

	// Xsha256_32
	0x02,// SameAsPrev

	// Xsha256_64
	0x02,// SameAsPrev

	// Xbts_r16_rm16
	0x00,// No flags set
	0xDA, 0x05,// 730 = "xbts"

	// Xbts_r32_rm32
	0x02,// SameAsPrev

	// Xstore_16
	0x00,// No flags set
	0xDB, 0x05,// 731 = "xstore"

	// Xstore_32
	0x02,// SameAsPrev

	// Xstore_64
	0x02,// SameAsPrev

	// Xcryptecb_16
	0x00,// No flags set
	0xDC, 0x05,// 732 = "xcryptecb"

	// Xcryptecb_32
	0x02,// SameAsPrev

	// Xcryptecb_64
	0x02,// SameAsPrev

	// Xcryptcbc_16
	0x00,// No flags set
	0xDD, 0x05,// 733 = "xcryptcbc"

	// Xcryptcbc_32
	0x02,// SameAsPrev

	// Xcryptcbc_64
	0x02,// SameAsPrev

	// Xcryptctr_16
	0x00,// No flags set
	0xDE, 0x05,// 734 = "xcryptctr"

	// Xcryptctr_32
	0x02,// SameAsPrev

	// Xcryptctr_64
	0x02,// SameAsPrev

	// Xcryptcfb_16
	0x00,// No flags set
	0xDF, 0x05,// 735 = "xcryptcfb"

	// Xcryptcfb_32
	0x02,// SameAsPrev

	// Xcryptcfb_64
	0x02,// SameAsPrev

	// Xcryptofb_16
	0x00,// No flags set
	0xE0, 0x05,// 736 = "xcryptofb"

	// Xcryptofb_32
	0x02,// SameAsPrev

	// Xcryptofb_64
	0x02,// SameAsPrev

	// Ibts_rm16_r16
	0x00,// No flags set
	0xE1, 0x05,// 737 = "ibts"

	// Ibts_rm32_r32
	0x02,// SameAsPrev

	// Cmpxchg486_rm8_r8
	0x00,// No flags set
	0xE2, 0x05,// 738 = "cmpxchg486"

	// Cmpxchg486_rm16_r16
	0x02,// SameAsPrev

	// Cmpxchg486_rm32_r32
	0x02,// SameAsPrev

	// Pushw_GS
	0x00,// No flags set
	0x03,// 3 = "push"

	// Pushd_GS
	0x02,// SameAsPrev

	// Pushq_GS
	0x02,// SameAsPrev

	// Popw_GS
	0x00,// No flags set
	0x0C,// 12 = "pop"

	// Popd_GS
	0x02,// SameAsPrev

	// Popq_GS
	0x02,// SameAsPrev

	// Rsm
	0x00,// No flags set
	0xE3, 0x05,// 739 = "rsm"

	// Bts_rm16_r16
	0x00,// No flags set
	0xE4, 0x05,// 740 = "bts"

	// Bts_rm32_r32
	0x02,// SameAsPrev

	// Bts_rm64_r64
	0x02,// SameAsPrev

	// Shrd_rm16_r16_imm8
	0x00,// No flags set
	0xE5, 0x05,// 741 = "shrd"

	// Shrd_rm32_r32_imm8
	0x02,// SameAsPrev

	// Shrd_rm64_r64_imm8
	0x02,// SameAsPrev

	// Shrd_rm16_r16_CL
	0x02,// SameAsPrev

	// Shrd_rm32_r32_CL
	0x02,// SameAsPrev

	// Shrd_rm64_r64_CL
	0x02,// SameAsPrev

	// Fxsave_m512byte
	0x00,// No flags set
	0xE6, 0x05,// 742 = "fxsave"

	// Fxsave64_m512byte
	0x00,// No flags set
	0xE7, 0x05,// 743 = "fxsave64"

	// Rdfsbase_r32
	0x00,// No flags set
	0xE8, 0x05,// 744 = "rdfsbase"

	// Rdfsbase_r64
	0x02,// SameAsPrev

	// Fxrstor_m512byte
	0x00,// No flags set
	0xE9, 0x05,// 745 = "fxrstor"

	// Fxrstor64_m512byte
	0x00,// No flags set
	0xEA, 0x05,// 746 = "fxrstor64"

	// Rdgsbase_r32
	0x00,// No flags set
	0xEB, 0x05,// 747 = "rdgsbase"

	// Rdgsbase_r64
	0x02,// SameAsPrev

	// Ldmxcsr_m32
	0x00,// No flags set
	0xEC, 0x05,// 748 = "ldmxcsr"

	// Wrfsbase_r32
	0x00,// No flags set
	0xED, 0x05,// 749 = "wrfsbase"

	// Wrfsbase_r64
	0x02,// SameAsPrev

	// VEX_Vldmxcsr_m32
	0x01,// HasVPrefix
	0xEC, 0x05,// 748 = "vldmxcsr"

	// Stmxcsr_m32
	0x00,// No flags set
	0xEE, 0x05,// 750 = "stmxcsr"

	// Wrgsbase_r32
	0x00,// No flags set
	0xEF, 0x05,// 751 = "wrgsbase"

	// Wrgsbase_r64
	0x02,// SameAsPrev

	// VEX_Vstmxcsr_m32
	0x01,// HasVPrefix
	0xEE, 0x05,// 750 = "vstmxcsr"

	// Xsave_mem
	0x00,// No flags set
	0xF0, 0x05,// 752 = "xsave"

	// Xsave64_mem
	0x00,// No flags set
	0xF1, 0x05,// 753 = "xsave64"

	// Ptwrite_rm32
	0x04,// ForceMemSize
	0xF2, 0x05,// 754 = "ptwrite"

	// Ptwrite_rm64
	0x06,// SameAsPrev, ForceMemSize

	// Xrstor_mem
	0x00,// No flags set
	0xF3, 0x05,// 755 = "xrstor"

	// Xrstor64_mem
	0x00,// No flags set
	0xF4, 0x05,// 756 = "xrstor64"

	// Incsspd_r32
	0x00,// No flags set
	0xF5, 0x05,// 757 = "incsspd"

	// Incsspq_r64
	0x00,// No flags set
	0xF6, 0x05,// 758 = "incsspq"

	// Xsaveopt_mem
	0x00,// No flags set
	0xF7, 0x05,// 759 = "xsaveopt"

	// Xsaveopt64_mem
	0x00,// No flags set
	0xF8, 0x05,// 760 = "xsaveopt64"

	// Clwb_m8
	0x00,// No flags set
	0xF9, 0x05,// 761 = "clwb"

	// Tpause_r32
	0x00,// No flags set
	0xFA, 0x05,// 762 = "tpause"

	// Tpause_r64
	0x02,// SameAsPrev

	// Clrssbsy_m64
	0x00,// No flags set
	0xFB, 0x05,// 763 = "clrssbsy"

	// Umonitor_r16
	0x00,// No flags set
	0xFC, 0x05,// 764 = "umonitor"

	// Umonitor_r32
	0x02,// SameAsPrev

	// Umonitor_r64
	0x02,// SameAsPrev

	// Umwait_r32
	0x00,// No flags set
	0xFD, 0x05,// 765 = "umwait"

	// Umwait_r64
	0x02,// SameAsPrev

	// Clflush_m8
	0x00,// No flags set
	0xFE, 0x05,// 766 = "clflush"

	// Clflushopt_m8
	0x00,// No flags set
	0xFF, 0x05,// 767 = "clflushopt"

	// Lfence
	0x00,// No flags set
	0x33,// 51 = "lfence"

	// Lfence_E9
	0x02,// SameAsPrev

	// Lfence_EA
	0x02,// SameAsPrev

	// Lfence_EB
	0x02,// SameAsPrev

	// Lfence_EC
	0x02,// SameAsPrev

	// Lfence_ED
	0x02,// SameAsPrev

	// Lfence_EE
	0x02,// SameAsPrev

	// Lfence_EF
	0x02,// SameAsPrev

	// Mfence
	0x00,// No flags set
	0x34,// 52 = "mfence"

	// Mfence_F1
	0x02,// SameAsPrev

	// Mfence_F2
	0x02,// SameAsPrev

	// Mfence_F3
	0x02,// SameAsPrev

	// Mfence_F4
	0x02,// SameAsPrev

	// Mfence_F5
	0x02,// SameAsPrev

	// Mfence_F6
	0x02,// SameAsPrev

	// Mfence_F7
	0x02,// SameAsPrev

	// Sfence
	0x00,// No flags set
	0x35,// 53 = "sfence"

	// Sfence_F9
	0x02,// SameAsPrev

	// Sfence_FA
	0x02,// SameAsPrev

	// Sfence_FB
	0x02,// SameAsPrev

	// Sfence_FC
	0x02,// SameAsPrev

	// Sfence_FD
	0x02,// SameAsPrev

	// Sfence_FE
	0x02,// SameAsPrev

	// Sfence_FF
	0x02,// SameAsPrev

	// Pcommit
	0x00,// No flags set
	0x80, 0x06,// 768 = "pcommit"

	// Imul_r16_rm16
	0x00,// No flags set
	0x1B,// 27 = "imul"

	// Imul_r32_rm32
	0x02,// SameAsPrev

	// Imul_r64_rm64
	0x02,// SameAsPrev

	// Cmpxchg_rm8_r8
	0x00,// No flags set
	0x81, 0x06,// 769 = "cmpxchg"

	// Cmpxchg_rm16_r16
	0x02,// SameAsPrev

	// Cmpxchg_rm32_r32
	0x02,// SameAsPrev

	// Cmpxchg_rm64_r64
	0x02,// SameAsPrev

	// Lss_r16_m1616
	0x00,// No flags set
	0x82, 0x06,// 770 = "lss"

	// Lss_r32_m1632
	0x02,// SameAsPrev

	// Lss_r64_m1664
	0x02,// SameAsPrev

	// Btr_rm16_r16
	0x00,// No flags set
	0x83, 0x06,// 771 = "btr"

	// Btr_rm32_r32
	0x02,// SameAsPrev

	// Btr_rm64_r64
	0x02,// SameAsPrev

	// Lfs_r16_m1616
	0x00,// No flags set
	0x84, 0x06,// 772 = "lfs"

	// Lfs_r32_m1632
	0x02,// SameAsPrev

	// Lfs_r64_m1664
	0x02,// SameAsPrev

	// Lgs_r16_m1616
	0x00,// No flags set
	0x85, 0x06,// 773 = "lgs"

	// Lgs_r32_m1632
	0x02,// SameAsPrev

	// Lgs_r64_m1664
	0x02,// SameAsPrev

	// Movzx_r16_rm8
	0x04,// ForceMemSize
	0x87, 0x06,// 775 = "movzx"

	// Movzx_r32_rm8
	0x06,// SameAsPrev, ForceMemSize

	// Movzx_r64_rm8
	0x06,// SameAsPrev, ForceMemSize

	// Movzx_r16_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Movzx_r32_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Movzx_r64_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Jmpe_disp16
	0x00,// No flags set
	0x95, 0x03,// 405 = "jmpe"

	// Jmpe_disp32
	0x02,// SameAsPrev

	// Popcnt_r16_rm16
	0x00,// No flags set
	0x8D, 0x06,// 781 = "popcnt"

	// Popcnt_r32_rm32
	0x02,// SameAsPrev

	// Popcnt_r64_rm64
	0x02,// SameAsPrev

	// Ud1_r16_rm16
	0x00,// No flags set
	0x8E, 0x06,// 782 = "ud1"

	// Ud1_r32_rm32
	0x02,// SameAsPrev

	// Ud1_r64_rm64
	0x02,// SameAsPrev

	// Bt_rm16_imm8
	0x04,// ForceMemSize
	0xD5, 0x05,// 725 = "bt"

	// Bt_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Bt_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Bts_rm16_imm8
	0x04,// ForceMemSize
	0xE4, 0x05,// 740 = "bts"

	// Bts_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Bts_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Btr_rm16_imm8
	0x04,// ForceMemSize
	0x83, 0x06,// 771 = "btr"

	// Btr_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Btr_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Btc_rm16_imm8
	0x04,// ForceMemSize
	0x8F, 0x06,// 783 = "btc"

	// Btc_rm32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Btc_rm64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// Btc_rm16_r16
	0x02,// SameAsPrev

	// Btc_rm32_r32
	0x02,// SameAsPrev

	// Btc_rm64_r64
	0x02,// SameAsPrev

	// Bsf_r16_rm16
	0x00,// No flags set
	0x90, 0x06,// 784 = "bsf"

	// Bsf_r32_rm32
	0x02,// SameAsPrev

	// Bsf_r64_rm64
	0x02,// SameAsPrev

	// Tzcnt_r16_rm16
	0x00,// No flags set
	0x91, 0x06,// 785 = "tzcnt"

	// Tzcnt_r32_rm32
	0x02,// SameAsPrev

	// Tzcnt_r64_rm64
	0x02,// SameAsPrev

	// Bsr_r16_rm16
	0x00,// No flags set
	0x92, 0x06,// 786 = "bsr"

	// Bsr_r32_rm32
	0x02,// SameAsPrev

	// Bsr_r64_rm64
	0x02,// SameAsPrev

	// Lzcnt_r16_rm16
	0x00,// No flags set
	0x93, 0x06,// 787 = "lzcnt"

	// Lzcnt_r32_rm32
	0x02,// SameAsPrev

	// Lzcnt_r64_rm64
	0x02,// SameAsPrev

	// Movsx_r16_rm8
	0x04,// ForceMemSize
	0x95, 0x06,// 789 = "movsx"

	// Movsx_r32_rm8
	0x06,// SameAsPrev, ForceMemSize

	// Movsx_r64_rm8
	0x06,// SameAsPrev, ForceMemSize

	// Movsx_r16_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Movsx_r32_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Movsx_r64_rm16
	0x06,// SameAsPrev, ForceMemSize

	// Xadd_rm8_r8
	0x00,// No flags set
	0x9B, 0x06,// 795 = "xadd"

	// Xadd_rm16_r16
	0x02,// SameAsPrev

	// Xadd_rm32_r32
	0x02,// SameAsPrev

	// Xadd_rm64_r64
	0x02,// SameAsPrev

	// Cmpps_xmm_xmmm128_imm8
	0x08,// cmpps
	0x54,// 84 = "cmpps"

	// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	0x13,// HasVPrefix, SameAsPrev, vcmpps

	// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	0x13,// HasVPrefix, SameAsPrev, vcmpps

	// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	0x13,// HasVPrefix, SameAsPrev, vcmpps

	// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	0x13,// HasVPrefix, SameAsPrev, vcmpps

	// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	0x13,// HasVPrefix, SameAsPrev, vcmpps

	// Cmppd_xmm_xmmm128_imm8
	0x18,// cmppd
	0x55,// 85 = "cmppd"

	// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	0x23,// HasVPrefix, SameAsPrev, vcmppd

	// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	0x23,// HasVPrefix, SameAsPrev, vcmppd

	// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	0x23,// HasVPrefix, SameAsPrev, vcmppd

	// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	0x23,// HasVPrefix, SameAsPrev, vcmppd

	// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	0x23,// HasVPrefix, SameAsPrev, vcmppd

	// Cmpss_xmm_xmmm32_imm8
	0x28,// cmpss
	0x9C, 0x06,// 796 = "cmpss"

	// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	0x33,// HasVPrefix, SameAsPrev, vcmpss

	// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	0x33,// HasVPrefix, SameAsPrev, vcmpss

	// Cmpsd_xmm_xmmm64_imm8
	0x38,// cmpsd
	0x9D, 0x06,// 797 = "cmpsd"

	// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	0x43,// HasVPrefix, SameAsPrev, vcmpsd

	// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	0x43,// HasVPrefix, SameAsPrev, vcmpsd

	// Movnti_m32_r32
	0x00,// No flags set
	0x9E, 0x06,// 798 = "movnti"

	// Movnti_m64_r64
	0x02,// SameAsPrev

	// Pinsrw_mm_r32m16_imm8
	0x00,// No flags set
	0x36,// 54 = "pinsrw"

	// Pinsrw_mm_r64m16_imm8
	0x02,// SameAsPrev

	// Pinsrw_xmm_r32m16_imm8
	0x02,// SameAsPrev

	// Pinsrw_xmm_r64m16_imm8
	0x02,// SameAsPrev

	// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pextrw_r32_mm_imm8
	0x00,// No flags set
	0x1A,// 26 = "pextrw"

	// Pextrw_r64_mm_imm8
	0x02,// SameAsPrev

	// Pextrw_r32_xmm_imm8
	0x02,// SameAsPrev

	// Pextrw_r64_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpextrw_r32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpextrw_r64_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrw_r32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrw_r64_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Shufps_xmm_xmmm128_imm8
	0x00,// No flags set
	0x9F, 0x06,// 799 = "shufps"

	// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Shufpd_xmm_xmmm128_imm8
	0x00,// No flags set
	0xA0, 0x06,// 800 = "shufpd"

	// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Cmpxchg8b_m64
	0x00,// No flags set
	0xA1, 0x06,// 801 = "cmpxchg8b"

	// Cmpxchg16b_m128
	0x00,// No flags set
	0xA2, 0x06,// 802 = "cmpxchg16b"

	// Xrstors_mem
	0x00,// No flags set
	0xA3, 0x06,// 803 = "xrstors"

	// Xrstors64_mem
	0x00,// No flags set
	0xA4, 0x06,// 804 = "xrstors64"

	// Xsavec_mem
	0x00,// No flags set
	0xA5, 0x06,// 805 = "xsavec"

	// Xsavec64_mem
	0x00,// No flags set
	0xA6, 0x06,// 806 = "xsavec64"

	// Xsaves_mem
	0x00,// No flags set
	0xA7, 0x06,// 807 = "xsaves"

	// Xsaves64_mem
	0x00,// No flags set
	0xA8, 0x06,// 808 = "xsaves64"

	// Vmptrld_m64
	0x00,// No flags set
	0xA9, 0x06,// 809 = "vmptrld"

	// Vmclear_m64
	0x00,// No flags set
	0xAA, 0x06,// 810 = "vmclear"

	// Vmxon_m64
	0x00,// No flags set
	0xAB, 0x06,// 811 = "vmxon"

	// Rdrand_r16
	0x00,// No flags set
	0xAC, 0x06,// 812 = "rdrand"

	// Rdrand_r32
	0x02,// SameAsPrev

	// Rdrand_r64
	0x02,// SameAsPrev

	// Vmptrst_m64
	0x00,// No flags set
	0xAD, 0x06,// 813 = "vmptrst"

	// Rdseed_r16
	0x00,// No flags set
	0xAE, 0x06,// 814 = "rdseed"

	// Rdseed_r32
	0x02,// SameAsPrev

	// Rdseed_r64
	0x02,// SameAsPrev

	// Rdpid_r32
	0x00,// No flags set
	0xAF, 0x06,// 815 = "rdpid"

	// Rdpid_r64
	0x02,// SameAsPrev

	// Bswap_r16
	0x00,// No flags set
	0xB0, 0x06,// 816 = "bswap"

	// Bswap_r32
	0x02,// SameAsPrev

	// Bswap_r64
	0x02,// SameAsPrev

	// Addsubpd_xmm_xmmm128
	0x00,// No flags set
	0xB1, 0x06,// 817 = "addsubpd"

	// VEX_Vaddsubpd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaddsubpd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Addsubps_xmm_xmmm128
	0x00,// No flags set
	0xB2, 0x06,// 818 = "addsubps"

	// VEX_Vaddsubps_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaddsubps_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Psrlw_mm_mmm64
	0x00,// No flags set
	0x15,// 21 = "psrlw"

	// Psrlw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsrlw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrlw_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Psrld_mm_mmm64
	0x00,// No flags set
	0x0F,// 15 = "psrld"

	// Psrld_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsrld_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrld_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Psrlq_mm_mmm64
	0x00,// No flags set
	0x18,// 24 = "psrlq"

	// Psrlq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsrlq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrlq_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Paddq_mm_mmm64
	0x00,// No flags set
	0x56,// 86 = "paddq"

	// Paddq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Pmullw_mm_mmm64
	0x00,// No flags set
	0x57,// 87 = "pmullw"

	// Pmullw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmullw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmullw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movq_xmmm64_xmm
	0x00,// No flags set
	0x0E,// 14 = "movq"

	// VEX_Vmovq_xmmm64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovq_xmmm64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Movq2dq_xmm_mm
	0x00,// No flags set
	0xB3, 0x06,// 819 = "movq2dq"

	// Movdq2q_mm_xmm
	0x00,// No flags set
	0xB4, 0x06,// 820 = "movdq2q"

	// Pmovmskb_r32_mm
	0x00,// No flags set
	0x37,// 55 = "pmovmskb"

	// Pmovmskb_r64_mm
	0x02,// SameAsPrev

	// Pmovmskb_r32_xmm
	0x02,// SameAsPrev

	// Pmovmskb_r64_xmm
	0x02,// SameAsPrev

	// VEX_Vpmovmskb_r32_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovmskb_r64_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovmskb_r32_ymm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovmskb_r64_ymm
	0x03,// HasVPrefix, SameAsPrev

	// Psubusb_mm_mmm64
	0x00,// No flags set
	0x58,// 88 = "psubusb"

	// Psubusb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubusb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubusb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psubusw_mm_mmm64
	0x00,// No flags set
	0x59,// 89 = "psubusw"

	// Psubusw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubusw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubusw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pminub_mm_mmm64
	0x00,// No flags set
	0x5A,// 90 = "pminub"

	// Pminub_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpminub_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminub_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pand_mm_mmm64
	0x00,// No flags set
	0xB5, 0x06,// 821 = "pand"

	// Pand_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpand_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpand_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB6, 0x06,// 822 = "vpandd"

	// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB7, 0x06,// 823 = "vpandq"

	// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Paddusb_mm_mmm64
	0x00,// No flags set
	0x5B,// 91 = "paddusb"

	// Paddusb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddusb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddusb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Paddusw_mm_mmm64
	0x00,// No flags set
	0x5C,// 92 = "paddusw"

	// Paddusw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddusw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddusw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmaxub_mm_mmm64
	0x00,// No flags set
	0x5D,// 93 = "pmaxub"

	// Pmaxub_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmaxub_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxub_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pandn_mm_mmm64
	0x00,// No flags set
	0xB8, 0x06,// 824 = "pandn"

	// Pandn_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpandn_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpandn_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB9, 0x06,// 825 = "vpandnd"

	// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xBA, 0x06,// 826 = "vpandnq"

	// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pavgb_mm_mmm64
	0x00,// No flags set
	0x5E,// 94 = "pavgb"

	// Pavgb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpavgb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpavgb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psraw_mm_mmm64
	0x00,// No flags set
	0x16,// 22 = "psraw"

	// Psraw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsraw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsraw_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Psrad_mm_mmm64
	0x00,// No flags set
	0x10,// 16 = "psrad"

	// Psrad_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsrad_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsrad_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0x86, 0x05,// 646 = "vpsraq"

	// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	0x02,// SameAsPrev

	// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	0x02,// SameAsPrev

	// Pavgw_mm_mmm64
	0x00,// No flags set
	0x5F,// 95 = "pavgw"

	// Pavgw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpavgw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpavgw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmulhuw_mm_mmm64
	0x00,// No flags set
	0x60,// 96 = "pmulhuw"

	// Pmulhuw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmulhuw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmulhuw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmulhw_mm_mmm64
	0x00,// No flags set
	0x61,// 97 = "pmulhw"

	// Pmulhw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmulhw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmulhw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Cvttpd2dq_xmm_xmmm128
	0x00,// No flags set
	0xBB, 0x06,// 827 = "cvttpd2dq"

	// VEX_Vcvttpd2dq_xmm_xmmm128
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// VEX_Vcvttpd2dq_xmm_ymmm256
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	0x03,// HasVPrefix, SameAsPrev

	// Cvtdq2pd_xmm_xmmm64
	0x00,// No flags set
	0x62,// 98 = "cvtdq2pd"

	// VEX_Vcvtdq2pd_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vcvtdq2pd_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xBC, 0x06,// 828 = "vcvtqq2pd"

	// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	0x02,// SameAsPrev

	// Cvtpd2dq_xmm_xmmm128
	0x00,// No flags set
	0xBD, 0x06,// 829 = "cvtpd2dq"

	// VEX_Vcvtpd2dq_xmm_xmmm128
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// VEX_Vcvtpd2dq_xmm_ymmm256
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	0x07,// HasVPrefix, SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	0x03,// HasVPrefix, SameAsPrev

	// Movntq_m64_mm
	0x00,// No flags set
	0xBE, 0x06,// 830 = "movntq"

	// Movntdq_m128_xmm
	0x00,// No flags set
	0xBF, 0x06,// 831 = "movntdq"

	// VEX_Vmovntdq_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovntdq_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdq_m128_xmm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdq_m256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdq_m512_zmm
	0x03,// HasVPrefix, SameAsPrev

	// Psubsb_mm_mmm64
	0x00,// No flags set
	0x63,// 99 = "psubsb"

	// Psubsb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubsb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubsb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psubsw_mm_mmm64
	0x00,// No flags set
	0x64,// 100 = "psubsw"

	// Psubsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pminsw_mm_mmm64
	0x00,// No flags set
	0x65,// 101 = "pminsw"

	// Pminsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpminsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Por_mm_mmm64
	0x00,// No flags set
	0xC0, 0x06,// 832 = "por"

	// Por_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpor_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpor_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC1, 0x06,// 833 = "vpord"

	// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC2, 0x06,// 834 = "vporq"

	// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Paddsb_mm_mmm64
	0x00,// No flags set
	0x66,// 102 = "paddsb"

	// Paddsb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddsb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddsb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Paddsw_mm_mmm64
	0x00,// No flags set
	0x67,// 103 = "paddsw"

	// Paddsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmaxsw_mm_mmm64
	0x00,// No flags set
	0x68,// 104 = "pmaxsw"

	// Pmaxsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmaxsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pxor_mm_mmm64
	0x00,// No flags set
	0xC3, 0x06,// 835 = "pxor"

	// Pxor_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpxor_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpxor_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC4, 0x06,// 836 = "vpxord"

	// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC5, 0x06,// 837 = "vpxorq"

	// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Lddqu_xmm_m128
	0x00,// No flags set
	0xC6, 0x06,// 838 = "lddqu"

	// VEX_Vlddqu_xmm_m128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vlddqu_ymm_m256
	0x03,// HasVPrefix, SameAsPrev

	// Psllw_mm_mmm64
	0x00,// No flags set
	0x17,// 23 = "psllw"

	// Psllw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsllw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsllw_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Pslld_mm_mmm64
	0x00,// No flags set
	0x11,// 17 = "pslld"

	// Pslld_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpslld_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpslld_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Psllq_mm_mmm64
	0x00,// No flags set
	0x19,// 25 = "psllq"

	// Psllq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsllq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsllq_ymm_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Pmuludq_mm_mmm64
	0x00,// No flags set
	0x69,// 105 = "pmuludq"

	// Pmuludq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmuludq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmuludq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Pmaddwd_mm_mmm64
	0x00,// No flags set
	0x6A,// 106 = "pmaddwd"

	// Pmaddwd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmaddwd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaddwd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psadbw_mm_mmm64
	0x00,// No flags set
	0x6B,// 107 = "psadbw"

	// Psadbw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsadbw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsadbw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsadbw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsadbw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsadbw_zmm_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Maskmovq_rDI_mm_mm
	0x00,// No flags set
	0xC7, 0x06,// 839 = "maskmovq"

	// Maskmovdqu_rDI_xmm_xmm
	0x00,// No flags set
	0xC8, 0x06,// 840 = "maskmovdqu"

	// VEX_Vmaskmovdqu_rDI_xmm_xmm
	0x03,// HasVPrefix, SameAsPrev

	// Psubb_mm_mmm64
	0x00,// No flags set
	0x6C,// 108 = "psubb"

	// Psubb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psubw_mm_mmm64
	0x00,// No flags set
	0x6D,// 109 = "psubw"

	// Psubw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Psubd_mm_mmm64
	0x00,// No flags set
	0x38,// 56 = "psubd"

	// Psubd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Psubq_mm_mmm64
	0x00,// No flags set
	0x6E,// 110 = "psubq"

	// Psubq_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsubq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsubq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Paddb_mm_mmm64
	0x00,// No flags set
	0x6F,// 111 = "paddb"

	// Paddb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Paddw_mm_mmm64
	0x00,// No flags set
	0x70,// 112 = "paddw"

	// Paddw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Paddd_mm_mmm64
	0x00,// No flags set
	0x39,// 57 = "paddd"

	// Paddd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpaddd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpaddd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// Ud0_r16_rm16
	0x00,// No flags set
	0xC9, 0x06,// 841 = "ud0"

	// Ud0_r32_rm32
	0x02,// SameAsPrev

	// Ud0_r64_rm64
	0x02,// SameAsPrev

	// Pshufb_mm_mmm64
	0x00,// No flags set
	0x71,// 113 = "pshufb"

	// Pshufb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpshufb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpshufb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Phaddw_mm_mmm64
	0x00,// No flags set
	0xCA, 0x06,// 842 = "phaddw"

	// Phaddw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphaddw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphaddw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Phaddd_mm_mmm64
	0x00,// No flags set
	0xCB, 0x06,// 843 = "phaddd"

	// Phaddd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphaddd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphaddd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Phaddsw_mm_mmm64
	0x00,// No flags set
	0xCC, 0x06,// 844 = "phaddsw"

	// Phaddsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphaddsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphaddsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Pmaddubsw_mm_mmm64
	0x00,// No flags set
	0x72,// 114 = "pmaddubsw"

	// Pmaddubsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Phsubw_mm_mmm64
	0x00,// No flags set
	0xCD, 0x06,// 845 = "phsubw"

	// Phsubw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphsubw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphsubw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Phsubd_mm_mmm64
	0x00,// No flags set
	0xCE, 0x06,// 846 = "phsubd"

	// Phsubd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphsubd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphsubd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Phsubsw_mm_mmm64
	0x00,// No flags set
	0xCF, 0x06,// 847 = "phsubsw"

	// Phsubsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vphsubsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vphsubsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Psignb_mm_mmm64
	0x00,// No flags set
	0xD0, 0x06,// 848 = "psignb"

	// Psignb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsignb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsignb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Psignw_mm_mmm64
	0x00,// No flags set
	0xD1, 0x06,// 849 = "psignw"

	// Psignw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsignw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsignw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Psignd_mm_mmm64
	0x00,// No flags set
	0xD2, 0x06,// 850 = "psignd"

	// Psignd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpsignd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpsignd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// Pmulhrsw_mm_mmm64
	0x00,// No flags set
	0x73,// 115 = "pmulhrsw"

	// Pmulhrsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpermilps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x27,// 39 = "vpermilps"

	// VEX_Vpermilps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// VEX_Vpermilpd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x28,// 40 = "vpermilpd"

	// VEX_Vpermilpd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vtestps_xmm_xmmm128
	0x00,// No flags set
	0xD3, 0x06,// 851 = "vtestps"

	// VEX_Vtestps_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vtestpd_xmm_xmmm128
	0x00,// No flags set
	0xD4, 0x06,// 852 = "vtestpd"

	// VEX_Vtestpd_ymm_ymmm256
	0x02,// SameAsPrev

	// Pblendvb_xmm_xmmm128
	0x00,// No flags set
	0xD5, 0x06,// 853 = "pblendvb"

	// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xD6, 0x06,// 854 = "vpsrlvw"

	// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	0x00,// No flags set
	0xD7, 0x06,// 855 = "vpmovuswb"

	// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xD8, 0x06,// 856 = "vpsravw"

	// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	0x00,// No flags set
	0xD9, 0x06,// 857 = "vpmovusdb"

	// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xDA, 0x06,// 858 = "vpsllvw"

	// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	0x00,// No flags set
	0xDB, 0x06,// 859 = "vpmovusqb"

	// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	0x02,// SameAsPrev

	// VEX_Vcvtph2ps_xmm_xmmm64
	0x00,// No flags set
	0xDC, 0x06,// 860 = "vcvtph2ps"

	// VEX_Vcvtph2ps_ymm_xmmm128
	0x02,// SameAsPrev

	// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	0x02,// SameAsPrev

	// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	0x02,// SameAsPrev

	// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	0x00,// No flags set
	0xDD, 0x06,// 861 = "vpmovusdw"

	// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// Blendvps_xmm_xmmm128
	0x00,// No flags set
	0xDE, 0x06,// 862 = "blendvps"

	// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xDF, 0x06,// 863 = "vprorvd"

	// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xE0, 0x06,// 864 = "vprorvq"

	// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	0x00,// No flags set
	0xE1, 0x06,// 865 = "vpmovusqw"

	// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// Blendvpd_xmm_xmmm128
	0x00,// No flags set
	0xE2, 0x06,// 866 = "blendvpd"

	// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xE3, 0x06,// 867 = "vprolvd"

	// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xE4, 0x06,// 868 = "vprolvq"

	// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	0x00,// No flags set
	0xE5, 0x06,// 869 = "vpmovusqd"

	// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// VEX_Vpermps_ymm_ymm_ymmm256
	0x00,// No flags set
	0xE6, 0x06,// 870 = "vpermps"

	// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	0x00,// No flags set
	0xE7, 0x06,// 871 = "vpermpd"

	// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Ptest_xmm_xmmm128
	0x00,// No flags set
	0xE8, 0x06,// 872 = "ptest"

	// VEX_Vptest_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vptest_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vbroadcastss_xmm_m32
	0x00,// No flags set
	0x3A,// 58 = "vbroadcastss"

	// VEX_Vbroadcastss_ymm_m32
	0x02,// SameAsPrev

	// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	0x02,// SameAsPrev

	// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	0x02,// SameAsPrev

	// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	0x02,// SameAsPrev

	// VEX_Vbroadcastsd_ymm_m64
	0x00,// No flags set
	0xE9, 0x06,// 873 = "vbroadcastsd"

	// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	0x00,// No flags set
	0xEA, 0x06,// 874 = "vbroadcastf32x2"

	// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	0x00,// No flags set
	0xE9, 0x06,// 873 = "vbroadcastsd"

	// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	0x02,// SameAsPrev

	// VEX_Vbroadcastf128_ymm_m128
	0x00,// No flags set
	0xEB, 0x06,// 875 = "vbroadcastf128"

	// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	0x00,// No flags set
	0xEC, 0x06,// 876 = "vbroadcastf32x4"

	// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	0x02,// SameAsPrev

	// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	0x00,// No flags set
	0xED, 0x06,// 877 = "vbroadcastf64x2"

	// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	0x02,// SameAsPrev

	// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	0x00,// No flags set
	0xEE, 0x06,// 878 = "vbroadcastf32x8"

	// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	0x00,// No flags set
	0xEF, 0x06,// 879 = "vbroadcastf64x4"

	// Pabsb_mm_mmm64
	0x00,// No flags set
	0x74,// 116 = "pabsb"

	// Pabsb_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpabsb_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpabsb_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsb_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsb_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsb_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pabsw_mm_mmm64
	0x00,// No flags set
	0x75,// 117 = "pabsw"

	// Pabsw_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpabsw_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpabsw_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsw_xmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsw_ymm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsw_zmm_k1z_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pabsd_mm_mmm64
	0x00,// No flags set
	0x76,// 118 = "pabsd"

	// Pabsd_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vpabsd_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpabsd_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xF0, 0x06,// 880 = "vpabsq"

	// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// Pmovsxbw_xmm_xmmm64
	0x00,// No flags set
	0xF1, 0x06,// 881 = "pmovsxbw"

	// VEX_Vpmovsxbw_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxbw_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovswb_xmmm64_k1z_xmm
	0x00,// No flags set
	0xF2, 0x06,// 882 = "vpmovswb"

	// EVEX_Vpmovswb_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovswb_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// Pmovsxbd_xmm_xmmm32
	0x00,// No flags set
	0xF3, 0x06,// 883 = "pmovsxbd"

	// VEX_Vpmovsxbd_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxbd_ymm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	0x00,// No flags set
	0xF4, 0x06,// 884 = "vpmovsdb"

	// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// Pmovsxbq_xmm_xmmm16
	0x00,// No flags set
	0xF5, 0x06,// 885 = "pmovsxbq"

	// VEX_Vpmovsxbq_xmm_xmmm16
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxbq_ymm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	0x00,// No flags set
	0xF6, 0x06,// 886 = "vpmovsqb"

	// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	0x02,// SameAsPrev

	// Pmovsxwd_xmm_xmmm64
	0x00,// No flags set
	0xF7, 0x06,// 887 = "pmovsxwd"

	// VEX_Vpmovsxwd_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxwd_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	0x00,// No flags set
	0xF8, 0x06,// 888 = "vpmovsdw"

	// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// Pmovsxwq_xmm_xmmm32
	0x00,// No flags set
	0xF9, 0x06,// 889 = "pmovsxwq"

	// VEX_Vpmovsxwq_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxwq_ymm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	0x00,// No flags set
	0xFA, 0x06,// 890 = "vpmovsqw"

	// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// Pmovsxdq_xmm_xmmm64
	0x00,// No flags set
	0xFB, 0x06,// 891 = "pmovsxdq"

	// VEX_Vpmovsxdq_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovsxdq_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	0x00,// No flags set
	0xFC, 0x06,// 892 = "vpmovsqd"

	// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	0x00,// No flags set
	0xFD, 0x06,// 893 = "vptestmb"

	// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	0x00,// No flags set
	0xFE, 0x06,// 894 = "vptestmw"

	// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	0x00,// No flags set
	0xFF, 0x06,// 895 = "vptestnmb"

	// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	0x00,// No flags set
	0x80, 0x07,// 896 = "vptestnmw"

	// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	0x00,// No flags set
	0x81, 0x07,// 897 = "vptestmd"

	// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	0x00,// No flags set
	0x82, 0x07,// 898 = "vptestmq"

	// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	0x00,// No flags set
	0x83, 0x07,// 899 = "vptestnmd"

	// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	0x00,// No flags set
	0x84, 0x07,// 900 = "vptestnmq"

	// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pmuldq_xmm_xmmm128
	0x00,// No flags set
	0x85, 0x07,// 901 = "pmuldq"

	// VEX_Vpmuldq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmuldq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovm2b_xmm_kr
	0x00,// No flags set
	0x86, 0x07,// 902 = "vpmovm2b"

	// EVEX_Vpmovm2b_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2b_zmm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2w_xmm_kr
	0x00,// No flags set
	0x87, 0x07,// 903 = "vpmovm2w"

	// EVEX_Vpmovm2w_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2w_zmm_kr
	0x02,// SameAsPrev

	// Pcmpeqq_xmm_xmmm128
	0x00,// No flags set
	0x88, 0x07,// 904 = "pcmpeqq"

	// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovb2m_kr_xmm
	0x00,// No flags set
	0x89, 0x07,// 905 = "vpmovb2m"

	// EVEX_Vpmovb2m_kr_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovb2m_kr_zmm
	0x02,// SameAsPrev

	// EVEX_Vpmovw2m_kr_xmm
	0x00,// No flags set
	0x8A, 0x07,// 906 = "vpmovw2m"

	// EVEX_Vpmovw2m_kr_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovw2m_kr_zmm
	0x02,// SameAsPrev

	// Movntdqa_xmm_m128
	0x00,// No flags set
	0x8B, 0x07,// 907 = "movntdqa"

	// VEX_Vmovntdqa_xmm_m128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmovntdqa_ymm_m256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdqa_xmm_m128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdqa_ymm_m256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vmovntdqa_zmm_m512
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpbroadcastmb2q_xmm_kr
	0x00,// No flags set
	0x8C, 0x07,// 908 = "vpbroadcastmb2q"

	// EVEX_Vpbroadcastmb2q_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastmb2q_zmm_kr
	0x02,// SameAsPrev

	// Packusdw_xmm_xmmm128
	0x00,// No flags set
	0x8D, 0x07,// 909 = "packusdw"

	// VEX_Vpackusdw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpackusdw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmaskmovps_xmm_xmm_m128
	0x00,// No flags set
	0x8E, 0x07,// 910 = "vmaskmovps"

	// VEX_Vmaskmovps_ymm_ymm_m256
	0x02,// SameAsPrev

	// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x8F, 0x07,// 911 = "vscalefps"

	// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x90, 0x07,// 912 = "vscalefpd"

	// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vmaskmovpd_xmm_xmm_m128
	0x00,// No flags set
	0x91, 0x07,// 913 = "vmaskmovpd"

	// VEX_Vmaskmovpd_ymm_ymm_m256
	0x02,// SameAsPrev

	// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0x92, 0x07,// 914 = "vscalefss"

	// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0x93, 0x07,// 915 = "vscalefsd"

	// VEX_Vmaskmovps_m128_xmm_xmm
	0x00,// No flags set
	0x8E, 0x07,// 910 = "vmaskmovps"

	// VEX_Vmaskmovps_m256_ymm_ymm
	0x02,// SameAsPrev

	// VEX_Vmaskmovpd_m128_xmm_xmm
	0x00,// No flags set
	0x91, 0x07,// 913 = "vmaskmovpd"

	// VEX_Vmaskmovpd_m256_ymm_ymm
	0x02,// SameAsPrev

	// Pmovzxbw_xmm_xmmm64
	0x00,// No flags set
	0x94, 0x07,// 916 = "pmovzxbw"

	// VEX_Vpmovzxbw_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxbw_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovwb_xmmm64_k1z_xmm
	0x00,// No flags set
	0x95, 0x07,// 917 = "vpmovwb"

	// EVEX_Vpmovwb_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovwb_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// Pmovzxbd_xmm_xmmm32
	0x00,// No flags set
	0x96, 0x07,// 918 = "pmovzxbd"

	// VEX_Vpmovzxbd_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxbd_ymm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovdb_xmmm32_k1z_xmm
	0x00,// No flags set
	0x97, 0x07,// 919 = "vpmovdb"

	// EVEX_Vpmovdb_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovdb_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// Pmovzxbq_xmm_xmmm16
	0x00,// No flags set
	0x98, 0x07,// 920 = "pmovzxbq"

	// VEX_Vpmovzxbq_xmm_xmmm16
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxbq_ymm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovqb_xmmm16_k1z_xmm
	0x00,// No flags set
	0x99, 0x07,// 921 = "vpmovqb"

	// EVEX_Vpmovqb_xmmm32_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovqb_xmmm64_k1z_zmm
	0x02,// SameAsPrev

	// Pmovzxwd_xmm_xmmm64
	0x00,// No flags set
	0x9A, 0x07,// 922 = "pmovzxwd"

	// VEX_Vpmovzxwd_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxwd_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovdw_xmmm64_k1z_xmm
	0x00,// No flags set
	0x9B, 0x07,// 923 = "vpmovdw"

	// EVEX_Vpmovdw_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovdw_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// Pmovzxwq_xmm_xmmm32
	0x00,// No flags set
	0x9C, 0x07,// 924 = "pmovzxwq"

	// VEX_Vpmovzxwq_xmm_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxwq_ymm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovqw_xmmm32_k1z_xmm
	0x00,// No flags set
	0x9D, 0x07,// 925 = "vpmovqw"

	// EVEX_Vpmovqw_xmmm64_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovqw_xmmm128_k1z_zmm
	0x02,// SameAsPrev

	// Pmovzxdq_xmm_xmmm64
	0x00,// No flags set
	0x9E, 0x07,// 926 = "pmovzxdq"

	// VEX_Vpmovzxdq_xmm_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmovzxdq_ymm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovqd_xmmm64_k1z_xmm
	0x00,// No flags set
	0x9F, 0x07,// 927 = "vpmovqd"

	// EVEX_Vpmovqd_xmmm128_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovqd_ymmm256_k1z_zmm
	0x02,// SameAsPrev

	// VEX_Vpermd_ymm_ymm_ymmm256
	0x00,// No flags set
	0xA0, 0x07,// 928 = "vpermd"

	// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	0x00,// No flags set
	0xA1, 0x07,// 929 = "vpermq"

	// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pcmpgtq_xmm_xmmm128
	0x00,// No flags set
	0xA2, 0x07,// 930 = "pcmpgtq"

	// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	0x03,// HasVPrefix, SameAsPrev

	// Pminsb_xmm_xmmm128
	0x00,// No flags set
	0xA3, 0x07,// 931 = "pminsb"

	// VEX_Vpminsb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminsb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmovm2d_xmm_kr
	0x00,// No flags set
	0xA4, 0x07,// 932 = "vpmovm2d"

	// EVEX_Vpmovm2d_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2d_zmm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2q_xmm_kr
	0x00,// No flags set
	0xA5, 0x07,// 933 = "vpmovm2q"

	// EVEX_Vpmovm2q_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpmovm2q_zmm_kr
	0x02,// SameAsPrev

	// Pminsd_xmm_xmmm128
	0x00,// No flags set
	0x77,// 119 = "pminsd"

	// VEX_Vpminsd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminsd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xA6, 0x07,// 934 = "vpminsq"

	// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpmovd2m_kr_xmm
	0x00,// No flags set
	0xA7, 0x07,// 935 = "vpmovd2m"

	// EVEX_Vpmovd2m_kr_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovd2m_kr_zmm
	0x02,// SameAsPrev

	// EVEX_Vpmovq2m_kr_xmm
	0x00,// No flags set
	0xA8, 0x07,// 936 = "vpmovq2m"

	// EVEX_Vpmovq2m_kr_ymm
	0x02,// SameAsPrev

	// EVEX_Vpmovq2m_kr_zmm
	0x02,// SameAsPrev

	// Pminuw_xmm_xmmm128
	0x00,// No flags set
	0xA9, 0x07,// 937 = "pminuw"

	// VEX_Vpminuw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminuw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpbroadcastmw2d_xmm_kr
	0x00,// No flags set
	0xAA, 0x07,// 938 = "vpbroadcastmw2d"

	// EVEX_Vpbroadcastmw2d_ymm_kr
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastmw2d_zmm_kr
	0x02,// SameAsPrev

	// Pminud_xmm_xmmm128
	0x00,// No flags set
	0x78,// 120 = "pminud"

	// VEX_Vpminud_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpminud_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xAB, 0x07,// 939 = "vpminuq"

	// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pmaxsb_xmm_xmmm128
	0x00,// No flags set
	0xAC, 0x07,// 940 = "pmaxsb"

	// VEX_Vpmaxsb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxsb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmaxsd_xmm_xmmm128
	0x00,// No flags set
	0x79,// 121 = "pmaxsd"

	// VEX_Vpmaxsd_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxsd_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xAD, 0x07,// 941 = "vpmaxsq"

	// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pmaxuw_xmm_xmmm128
	0x00,// No flags set
	0xAE, 0x07,// 942 = "pmaxuw"

	// VEX_Vpmaxuw_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxuw_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Pmaxud_xmm_xmmm128
	0x00,// No flags set
	0x7A,// 122 = "pmaxud"

	// VEX_Vpmaxud_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmaxud_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xAF, 0x07,// 943 = "vpmaxuq"

	// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Pmulld_xmm_xmmm128
	0x00,// No flags set
	0x7B,// 123 = "pmulld"

	// VEX_Vpmulld_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpmulld_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB0, 0x07,// 944 = "vpmullq"

	// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Phminposuw_xmm_xmmm128
	0x00,// No flags set
	0xB1, 0x07,// 945 = "phminposuw"

	// VEX_Vphminposuw_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xB2, 0x07,// 946 = "vgetexpps"

	// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	0x02,// SameAsPrev

	// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xB3, 0x07,// 947 = "vgetexppd"

	// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	0x02,// SameAsPrev

	// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	0x00,// No flags set
	0xB4, 0x07,// 948 = "vgetexpss"

	// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	0x00,// No flags set
	0xB5, 0x07,// 949 = "vgetexpsd"

	// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xB6, 0x07,// 950 = "vplzcntd"

	// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xB7, 0x07,// 951 = "vplzcntq"

	// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vpsrlvd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB8, 0x07,// 952 = "vpsrlvd"

	// VEX_Vpsrlvd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpsrlvq_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB9, 0x07,// 953 = "vpsrlvq"

	// VEX_Vpsrlvq_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB8, 0x07,// 952 = "vpsrlvd"

	// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB9, 0x07,// 953 = "vpsrlvq"

	// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vpsravd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xBA, 0x07,// 954 = "vpsravd"

	// VEX_Vpsravd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	0x02,// SameAsPrev

	// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xBB, 0x07,// 955 = "vpsravq"

	// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vpsllvd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xBC, 0x07,// 956 = "vpsllvd"

	// VEX_Vpsllvd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpsllvq_xmm_xmm_xmmm128
	0x00,// No flags set
	0xBD, 0x07,// 957 = "vpsllvq"

	// VEX_Vpsllvq_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xBC, 0x07,// 956 = "vpsllvd"

	// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xBD, 0x07,// 957 = "vpsllvq"

	// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xBE, 0x07,// 958 = "vrcp14ps"

	// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xBF, 0x07,// 959 = "vrcp14pd"

	// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	0x00,// No flags set
	0xC0, 0x07,// 960 = "vrcp14ss"

	// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	0x00,// No flags set
	0xC1, 0x07,// 961 = "vrcp14sd"

	// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xC2, 0x07,// 962 = "vrsqrt14ps"

	// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xC3, 0x07,// 963 = "vrsqrt14pd"

	// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	0x00,// No flags set
	0xC4, 0x07,// 964 = "vrsqrt14ss"

	// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	0x00,// No flags set
	0xC5, 0x07,// 965 = "vrsqrt14sd"

	// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC6, 0x07,// 966 = "vpdpbusd"

	// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC7, 0x07,// 967 = "vpdpbusds"

	// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC8, 0x07,// 968 = "vpdpwssd"

	// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC9, 0x07,// 969 = "vdpbf16ps"

	// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	0x00,// No flags set
	0xCA, 0x07,// 970 = "vp4dpwssd"

	// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xCB, 0x07,// 971 = "vpdpwssds"

	// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	0x00,// No flags set
	0xCC, 0x07,// 972 = "vp4dpwssds"

	// EVEX_Vpopcntb_xmm_k1z_xmmm128
	0x00,// No flags set
	0xCD, 0x07,// 973 = "vpopcntb"

	// EVEX_Vpopcntb_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpopcntb_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpopcntw_xmm_k1z_xmmm128
	0x00,// No flags set
	0xCE, 0x07,// 974 = "vpopcntw"

	// EVEX_Vpopcntw_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpopcntw_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xCF, 0x07,// 975 = "vpopcntd"

	// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xD0, 0x07,// 976 = "vpopcntq"

	// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vpbroadcastd_xmm_xmmm32
	0x00,// No flags set
	0x2C,// 44 = "vpbroadcastd"

	// VEX_Vpbroadcastd_ymm_xmmm32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	0x02,// SameAsPrev

	// VEX_Vpbroadcastq_xmm_xmmm64
	0x00,// No flags set
	0x2D,// 45 = "vpbroadcastq"

	// VEX_Vpbroadcastq_ymm_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	0x00,// No flags set
	0xD1, 0x07,// 977 = "vbroadcasti32x2"

	// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	0x00,// No flags set
	0x2D,// 45 = "vpbroadcastq"

	// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	0x02,// SameAsPrev

	// VEX_Vbroadcasti128_ymm_m128
	0x00,// No flags set
	0xD2, 0x07,// 978 = "vbroadcasti128"

	// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	0x00,// No flags set
	0xD3, 0x07,// 979 = "vbroadcasti32x4"

	// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	0x02,// SameAsPrev

	// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	0x00,// No flags set
	0xD4, 0x07,// 980 = "vbroadcasti64x2"

	// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	0x02,// SameAsPrev

	// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	0x00,// No flags set
	0xD5, 0x07,// 981 = "vbroadcasti32x8"

	// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	0x00,// No flags set
	0xD6, 0x07,// 982 = "vbroadcasti64x4"

	// EVEX_Vpexpandb_xmm_k1z_xmmm128
	0x00,// No flags set
	0xD7, 0x07,// 983 = "vpexpandb"

	// EVEX_Vpexpandb_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpexpandb_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpexpandw_xmm_k1z_xmmm128
	0x00,// No flags set
	0xD8, 0x07,// 984 = "vpexpandw"

	// EVEX_Vpexpandw_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpexpandw_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpcompressb_xmmm128_k1z_xmm
	0x00,// No flags set
	0xD9, 0x07,// 985 = "vpcompressb"

	// EVEX_Vpcompressb_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpcompressb_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpcompressw_xmmm128_k1z_xmm
	0x00,// No flags set
	0xDA, 0x07,// 986 = "vpcompressw"

	// EVEX_Vpcompressw_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpcompressw_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xDB, 0x07,// 987 = "vpblendmd"

	// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xDC, 0x07,// 988 = "vpblendmq"

	// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xDD, 0x07,// 989 = "vblendmps"

	// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xDE, 0x07,// 990 = "vblendmpd"

	// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xDF, 0x07,// 991 = "vpblendmb"

	// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xE0, 0x07,// 992 = "vpblendmw"

	// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	0x00,// No flags set
	0xE1, 0x07,// 993 = "vp2intersectd"

	// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	0x00,// No flags set
	0xE2, 0x07,// 994 = "vp2intersectq"

	// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xE3, 0x07,// 995 = "vpshldvw"

	// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xE4, 0x07,// 996 = "vpshldvd"

	// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xE5, 0x07,// 997 = "vpshldvq"

	// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xE6, 0x07,// 998 = "vpshrdvw"

	// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	0x04,// ForceMemSize
	0xE7, 0x07,// 999 = "vcvtneps2bf16"

	// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xE8, 0x07,// 1000 = "vcvtne2ps2bf16"

	// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xE9, 0x07,// 1001 = "vpshrdvd"

	// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xEA, 0x07,// 1002 = "vpshrdvq"

	// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xEB, 0x07,// 1003 = "vpermi2b"

	// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xEC, 0x07,// 1004 = "vpermi2w"

	// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xED, 0x07,// 1005 = "vpermi2d"

	// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xEE, 0x07,// 1006 = "vpermi2q"

	// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xEF, 0x07,// 1007 = "vpermi2ps"

	// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xF0, 0x07,// 1008 = "vpermi2pd"

	// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vpbroadcastb_xmm_xmmm8
	0x00,// No flags set
	0x3B,// 59 = "vpbroadcastb"

	// VEX_Vpbroadcastb_ymm_xmmm8
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	0x02,// SameAsPrev

	// VEX_Vpbroadcastw_xmm_xmmm16
	0x00,// No flags set
	0x3C,// 60 = "vpbroadcastw"

	// VEX_Vpbroadcastw_ymm_xmmm16
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastb_xmm_k1z_r32
	0x00,// No flags set
	0x3B,// 59 = "vpbroadcastb"

	// EVEX_Vpbroadcastb_ymm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastb_zmm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastw_xmm_k1z_r32
	0x00,// No flags set
	0x3C,// 60 = "vpbroadcastw"

	// EVEX_Vpbroadcastw_ymm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastw_zmm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastd_xmm_k1z_r32
	0x00,// No flags set
	0x2C,// 44 = "vpbroadcastd"

	// EVEX_Vpbroadcastd_ymm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastd_zmm_k1z_r32
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastq_xmm_k1z_r64
	0x00,// No flags set
	0x2D,// 45 = "vpbroadcastq"

	// EVEX_Vpbroadcastq_ymm_k1z_r64
	0x02,// SameAsPrev

	// EVEX_Vpbroadcastq_zmm_k1z_r64
	0x02,// SameAsPrev

	// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xF1, 0x07,// 1009 = "vpermt2b"

	// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0xF2, 0x07,// 1010 = "vpermt2w"

	// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xF3, 0x07,// 1011 = "vpermt2d"

	// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xF4, 0x07,// 1012 = "vpermt2q"

	// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xF5, 0x07,// 1013 = "vpermt2ps"

	// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xF6, 0x07,// 1014 = "vpermt2pd"

	// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// Invept_r32_m128
	0x00,// No flags set
	0xF7, 0x07,// 1015 = "invept"

	// Invept_r64_m128
	0x02,// SameAsPrev

	// Invvpid_r32_m128
	0x00,// No flags set
	0xF8, 0x07,// 1016 = "invvpid"

	// Invvpid_r64_m128
	0x02,// SameAsPrev

	// Invpcid_r32_m128
	0x00,// No flags set
	0xF9, 0x07,// 1017 = "invpcid"

	// Invpcid_r64_m128
	0x02,// SameAsPrev

	// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xFA, 0x07,// 1018 = "vpmultishiftqb"

	// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vexpandps_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFB, 0x07,// 1019 = "vexpandps"

	// EVEX_Vexpandps_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vexpandps_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vexpandpd_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFC, 0x07,// 1020 = "vexpandpd"

	// EVEX_Vexpandpd_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vexpandpd_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpexpandd_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFD, 0x07,// 1021 = "vpexpandd"

	// EVEX_Vpexpandd_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpexpandd_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpexpandq_xmm_k1z_xmmm128
	0x00,// No flags set
	0xFE, 0x07,// 1022 = "vpexpandq"

	// EVEX_Vpexpandq_ymm_k1z_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpexpandq_zmm_k1z_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vcompressps_xmmm128_k1z_xmm
	0x00,// No flags set
	0xFF, 0x07,// 1023 = "vcompressps"

	// EVEX_Vcompressps_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vcompressps_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vcompresspd_xmmm128_k1z_xmm
	0x00,// No flags set
	0x80, 0x08,// 1024 = "vcompresspd"

	// EVEX_Vcompresspd_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vcompresspd_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpcompressd_xmmm128_k1z_xmm
	0x00,// No flags set
	0x81, 0x08,// 1025 = "vpcompressd"

	// EVEX_Vpcompressd_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpcompressd_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// EVEX_Vpcompressq_xmmm128_k1z_xmm
	0x00,// No flags set
	0x82, 0x08,// 1026 = "vpcompressq"

	// EVEX_Vpcompressq_ymmm256_k1z_ymm
	0x02,// SameAsPrev

	// EVEX_Vpcompressq_zmmm512_k1z_zmm
	0x02,// SameAsPrev

	// VEX_Vpmaskmovd_xmm_xmm_m128
	0x00,// No flags set
	0x83, 0x08,// 1027 = "vpmaskmovd"

	// VEX_Vpmaskmovd_ymm_ymm_m256
	0x02,// SameAsPrev

	// VEX_Vpmaskmovq_xmm_xmm_m128
	0x00,// No flags set
	0x84, 0x08,// 1028 = "vpmaskmovq"

	// VEX_Vpmaskmovq_ymm_ymm_m256
	0x02,// SameAsPrev

	// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0x85, 0x08,// 1029 = "vpermb"

	// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	0x00,// No flags set
	0x86, 0x08,// 1030 = "vpermw"

	// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	0x02,// SameAsPrev

	// VEX_Vpmaskmovd_m128_xmm_xmm
	0x00,// No flags set
	0x83, 0x08,// 1027 = "vpmaskmovd"

	// VEX_Vpmaskmovd_m256_ymm_ymm
	0x02,// SameAsPrev

	// VEX_Vpmaskmovq_m128_xmm_xmm
	0x00,// No flags set
	0x84, 0x08,// 1028 = "vpmaskmovq"

	// VEX_Vpmaskmovq_m256_ymm_ymm
	0x02,// SameAsPrev

	// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	0x00,// No flags set
	0x87, 0x08,// 1031 = "vpshufbitqmb"

	// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	0x02,// SameAsPrev

	// VEX_Vpgatherdd_xmm_vm32x_xmm
	0x00,// No flags set
	0x88, 0x08,// 1032 = "vpgatherdd"

	// VEX_Vpgatherdd_ymm_vm32y_ymm
	0x02,// SameAsPrev

	// VEX_Vpgatherdq_xmm_vm32x_xmm
	0x00,// No flags set
	0x89, 0x08,// 1033 = "vpgatherdq"

	// VEX_Vpgatherdq_ymm_vm32x_ymm
	0x02,// SameAsPrev

	// EVEX_Vpgatherdd_xmm_k1_vm32x
	0x00,// No flags set
	0x88, 0x08,// 1032 = "vpgatherdd"

	// EVEX_Vpgatherdd_ymm_k1_vm32y
	0x02,// SameAsPrev

	// EVEX_Vpgatherdd_zmm_k1_vm32z
	0x02,// SameAsPrev

	// EVEX_Vpgatherdq_xmm_k1_vm32x
	0x00,// No flags set
	0x89, 0x08,// 1033 = "vpgatherdq"

	// EVEX_Vpgatherdq_ymm_k1_vm32x
	0x02,// SameAsPrev

	// EVEX_Vpgatherdq_zmm_k1_vm32y
	0x02,// SameAsPrev

	// VEX_Vpgatherqd_xmm_vm64x_xmm
	0x00,// No flags set
	0x8A, 0x08,// 1034 = "vpgatherqd"

	// VEX_Vpgatherqd_xmm_vm64y_xmm
	0x02,// SameAsPrev

	// VEX_Vpgatherqq_xmm_vm64x_xmm
	0x00,// No flags set
	0x8B, 0x08,// 1035 = "vpgatherqq"

	// VEX_Vpgatherqq_ymm_vm64y_ymm
	0x02,// SameAsPrev

	// EVEX_Vpgatherqd_xmm_k1_vm64x
	0x00,// No flags set
	0x8A, 0x08,// 1034 = "vpgatherqd"

	// EVEX_Vpgatherqd_xmm_k1_vm64y
	0x02,// SameAsPrev

	// EVEX_Vpgatherqd_ymm_k1_vm64z
	0x02,// SameAsPrev

	// EVEX_Vpgatherqq_xmm_k1_vm64x
	0x00,// No flags set
	0x8B, 0x08,// 1035 = "vpgatherqq"

	// EVEX_Vpgatherqq_ymm_k1_vm64y
	0x02,// SameAsPrev

	// EVEX_Vpgatherqq_zmm_k1_vm64z
	0x02,// SameAsPrev

	// VEX_Vgatherdps_xmm_vm32x_xmm
	0x00,// No flags set
	0x8C, 0x08,// 1036 = "vgatherdps"

	// VEX_Vgatherdps_ymm_vm32y_ymm
	0x02,// SameAsPrev

	// VEX_Vgatherdpd_xmm_vm32x_xmm
	0x00,// No flags set
	0x8D, 0x08,// 1037 = "vgatherdpd"

	// VEX_Vgatherdpd_ymm_vm32x_ymm
	0x02,// SameAsPrev

	// EVEX_Vgatherdps_xmm_k1_vm32x
	0x00,// No flags set
	0x8C, 0x08,// 1036 = "vgatherdps"

	// EVEX_Vgatherdps_ymm_k1_vm32y
	0x02,// SameAsPrev

	// EVEX_Vgatherdps_zmm_k1_vm32z
	0x02,// SameAsPrev

	// EVEX_Vgatherdpd_xmm_k1_vm32x
	0x00,// No flags set
	0x8D, 0x08,// 1037 = "vgatherdpd"

	// EVEX_Vgatherdpd_ymm_k1_vm32x
	0x02,// SameAsPrev

	// EVEX_Vgatherdpd_zmm_k1_vm32y
	0x02,// SameAsPrev

	// VEX_Vgatherqps_xmm_vm64x_xmm
	0x00,// No flags set
	0x8E, 0x08,// 1038 = "vgatherqps"

	// VEX_Vgatherqps_xmm_vm64y_xmm
	0x02,// SameAsPrev

	// VEX_Vgatherqpd_xmm_vm64x_xmm
	0x00,// No flags set
	0x8F, 0x08,// 1039 = "vgatherqpd"

	// VEX_Vgatherqpd_ymm_vm64y_ymm
	0x02,// SameAsPrev

	// EVEX_Vgatherqps_xmm_k1_vm64x
	0x00,// No flags set
	0x8E, 0x08,// 1038 = "vgatherqps"

	// EVEX_Vgatherqps_xmm_k1_vm64y
	0x02,// SameAsPrev

	// EVEX_Vgatherqps_ymm_k1_vm64z
	0x02,// SameAsPrev

	// EVEX_Vgatherqpd_xmm_k1_vm64x
	0x00,// No flags set
	0x8F, 0x08,// 1039 = "vgatherqpd"

	// EVEX_Vgatherqpd_ymm_k1_vm64y
	0x02,// SameAsPrev

	// EVEX_Vgatherqpd_zmm_k1_vm64z
	0x02,// SameAsPrev

	// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x90, 0x08,// 1040 = "vfmaddsub132ps"

	// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x91, 0x08,// 1041 = "vfmaddsub132pd"

	// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x90, 0x08,// 1040 = "vfmaddsub132ps"

	// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x91, 0x08,// 1041 = "vfmaddsub132pd"

	// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x92, 0x08,// 1042 = "vfmsubadd132ps"

	// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x93, 0x08,// 1043 = "vfmsubadd132pd"

	// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x92, 0x08,// 1042 = "vfmsubadd132ps"

	// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x93, 0x08,// 1043 = "vfmsubadd132pd"

	// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x94, 0x08,// 1044 = "vfmadd132ps"

	// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x95, 0x08,// 1045 = "vfmadd132pd"

	// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x94, 0x08,// 1044 = "vfmadd132ps"

	// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x95, 0x08,// 1045 = "vfmadd132pd"

	// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0x96, 0x08,// 1046 = "vfmadd132ss"

	// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0x97, 0x08,// 1047 = "vfmadd132sd"

	// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0x96, 0x08,// 1046 = "vfmadd132ss"

	// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0x97, 0x08,// 1047 = "vfmadd132sd"

	// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x98, 0x08,// 1048 = "vfmsub132ps"

	// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x99, 0x08,// 1049 = "vfmsub132pd"

	// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x98, 0x08,// 1048 = "vfmsub132ps"

	// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x99, 0x08,// 1049 = "vfmsub132pd"

	// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	0x00,// No flags set
	0x9A, 0x08,// 1050 = "v4fmaddps"

	// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0x9B, 0x08,// 1051 = "vfmsub132ss"

	// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0x9C, 0x08,// 1052 = "vfmsub132sd"

	// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0x9B, 0x08,// 1051 = "vfmsub132ss"

	// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0x9C, 0x08,// 1052 = "vfmsub132sd"

	// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	0x00,// No flags set
	0x9D, 0x08,// 1053 = "v4fmaddss"

	// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0x9E, 0x08,// 1054 = "vfnmadd132ps"

	// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0x9F, 0x08,// 1055 = "vfnmadd132pd"

	// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0x9E, 0x08,// 1054 = "vfnmadd132ps"

	// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0x9F, 0x08,// 1055 = "vfnmadd132pd"

	// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xA0, 0x08,// 1056 = "vfnmadd132ss"

	// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xA1, 0x08,// 1057 = "vfnmadd132sd"

	// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xA0, 0x08,// 1056 = "vfnmadd132ss"

	// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xA1, 0x08,// 1057 = "vfnmadd132sd"

	// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA2, 0x08,// 1058 = "vfnmsub132ps"

	// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA3, 0x08,// 1059 = "vfnmsub132pd"

	// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xA2, 0x08,// 1058 = "vfnmsub132ps"

	// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xA3, 0x08,// 1059 = "vfnmsub132pd"

	// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xA4, 0x08,// 1060 = "vfnmsub132ss"

	// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xA5, 0x08,// 1061 = "vfnmsub132sd"

	// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xA4, 0x08,// 1060 = "vfnmsub132ss"

	// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xA5, 0x08,// 1061 = "vfnmsub132sd"

	// EVEX_Vpscatterdd_vm32x_k1_xmm
	0x00,// No flags set
	0xA6, 0x08,// 1062 = "vpscatterdd"

	// EVEX_Vpscatterdd_vm32y_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vpscatterdd_vm32z_k1_zmm
	0x02,// SameAsPrev

	// EVEX_Vpscatterdq_vm32x_k1_xmm
	0x00,// No flags set
	0xA7, 0x08,// 1063 = "vpscatterdq"

	// EVEX_Vpscatterdq_vm32x_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vpscatterdq_vm32y_k1_zmm
	0x02,// SameAsPrev

	// EVEX_Vpscatterqd_vm64x_k1_xmm
	0x00,// No flags set
	0xA8, 0x08,// 1064 = "vpscatterqd"

	// EVEX_Vpscatterqd_vm64y_k1_xmm
	0x02,// SameAsPrev

	// EVEX_Vpscatterqd_vm64z_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vpscatterqq_vm64x_k1_xmm
	0x00,// No flags set
	0xA9, 0x08,// 1065 = "vpscatterqq"

	// EVEX_Vpscatterqq_vm64y_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vpscatterqq_vm64z_k1_zmm
	0x02,// SameAsPrev

	// EVEX_Vscatterdps_vm32x_k1_xmm
	0x00,// No flags set
	0xAA, 0x08,// 1066 = "vscatterdps"

	// EVEX_Vscatterdps_vm32y_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vscatterdps_vm32z_k1_zmm
	0x02,// SameAsPrev

	// EVEX_Vscatterdpd_vm32x_k1_xmm
	0x00,// No flags set
	0xAB, 0x08,// 1067 = "vscatterdpd"

	// EVEX_Vscatterdpd_vm32x_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vscatterdpd_vm32y_k1_zmm
	0x02,// SameAsPrev

	// EVEX_Vscatterqps_vm64x_k1_xmm
	0x00,// No flags set
	0xAC, 0x08,// 1068 = "vscatterqps"

	// EVEX_Vscatterqps_vm64y_k1_xmm
	0x02,// SameAsPrev

	// EVEX_Vscatterqps_vm64z_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vscatterqpd_vm64x_k1_xmm
	0x00,// No flags set
	0xAD, 0x08,// 1069 = "vscatterqpd"

	// EVEX_Vscatterqpd_vm64y_k1_ymm
	0x02,// SameAsPrev

	// EVEX_Vscatterqpd_vm64z_k1_zmm
	0x02,// SameAsPrev

	// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xAE, 0x08,// 1070 = "vfmaddsub213ps"

	// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xAF, 0x08,// 1071 = "vfmaddsub213pd"

	// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xAE, 0x08,// 1070 = "vfmaddsub213ps"

	// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xAF, 0x08,// 1071 = "vfmaddsub213pd"

	// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB0, 0x08,// 1072 = "vfmsubadd213ps"

	// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB1, 0x08,// 1073 = "vfmsubadd213pd"

	// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB0, 0x08,// 1072 = "vfmsubadd213ps"

	// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB1, 0x08,// 1073 = "vfmsubadd213pd"

	// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB2, 0x08,// 1074 = "vfmadd213ps"

	// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB3, 0x08,// 1075 = "vfmadd213pd"

	// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB2, 0x08,// 1074 = "vfmadd213ps"

	// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB3, 0x08,// 1075 = "vfmadd213pd"

	// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xB4, 0x08,// 1076 = "vfmadd213ss"

	// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xB5, 0x08,// 1077 = "vfmadd213sd"

	// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xB4, 0x08,// 1076 = "vfmadd213ss"

	// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xB5, 0x08,// 1077 = "vfmadd213sd"

	// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB6, 0x08,// 1078 = "vfmsub213ps"

	// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xB7, 0x08,// 1079 = "vfmsub213pd"

	// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xB6, 0x08,// 1078 = "vfmsub213ps"

	// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xB7, 0x08,// 1079 = "vfmsub213pd"

	// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	0x00,// No flags set
	0xB8, 0x08,// 1080 = "v4fnmaddps"

	// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xB9, 0x08,// 1081 = "vfmsub213ss"

	// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xBA, 0x08,// 1082 = "vfmsub213sd"

	// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xB9, 0x08,// 1081 = "vfmsub213ss"

	// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xBA, 0x08,// 1082 = "vfmsub213sd"

	// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	0x00,// No flags set
	0xBB, 0x08,// 1083 = "v4fnmaddss"

	// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xBC, 0x08,// 1084 = "vfnmadd213ps"

	// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xBD, 0x08,// 1085 = "vfnmadd213pd"

	// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xBC, 0x08,// 1084 = "vfnmadd213ps"

	// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xBD, 0x08,// 1085 = "vfnmadd213pd"

	// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xBE, 0x08,// 1086 = "vfnmadd213ss"

	// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xBF, 0x08,// 1087 = "vfnmadd213sd"

	// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xBE, 0x08,// 1086 = "vfnmadd213ss"

	// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xBF, 0x08,// 1087 = "vfnmadd213sd"

	// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC0, 0x08,// 1088 = "vfnmsub213ps"

	// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC1, 0x08,// 1089 = "vfnmsub213pd"

	// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC0, 0x08,// 1088 = "vfnmsub213ps"

	// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC1, 0x08,// 1089 = "vfnmsub213pd"

	// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xC2, 0x08,// 1090 = "vfnmsub213ss"

	// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xC3, 0x08,// 1091 = "vfnmsub213sd"

	// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xC2, 0x08,// 1090 = "vfnmsub213ss"

	// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xC3, 0x08,// 1091 = "vfnmsub213sd"

	// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC4, 0x08,// 1092 = "vpmadd52luq"

	// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC5, 0x08,// 1093 = "vpmadd52huq"

	// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	0x02,// SameAsPrev

	// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC6, 0x08,// 1094 = "vfmaddsub231ps"

	// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC7, 0x08,// 1095 = "vfmaddsub231pd"

	// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC6, 0x08,// 1094 = "vfmaddsub231ps"

	// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC7, 0x08,// 1095 = "vfmaddsub231pd"

	// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC8, 0x08,// 1096 = "vfmsubadd231ps"

	// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC9, 0x08,// 1097 = "vfmsubadd231pd"

	// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xC8, 0x08,// 1096 = "vfmsubadd231ps"

	// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xC9, 0x08,// 1097 = "vfmsubadd231pd"

	// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xCA, 0x08,// 1098 = "vfmadd231ps"

	// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xCB, 0x08,// 1099 = "vfmadd231pd"

	// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xCA, 0x08,// 1098 = "vfmadd231ps"

	// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xCB, 0x08,// 1099 = "vfmadd231pd"

	// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xCC, 0x08,// 1100 = "vfmadd231ss"

	// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xCD, 0x08,// 1101 = "vfmadd231sd"

	// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xCC, 0x08,// 1100 = "vfmadd231ss"

	// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xCD, 0x08,// 1101 = "vfmadd231sd"

	// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xCE, 0x08,// 1102 = "vfmsub231ps"

	// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xCF, 0x08,// 1103 = "vfmsub231pd"

	// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xCE, 0x08,// 1102 = "vfmsub231ps"

	// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xCF, 0x08,// 1103 = "vfmsub231pd"

	// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xD0, 0x08,// 1104 = "vfmsub231ss"

	// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xD1, 0x08,// 1105 = "vfmsub231sd"

	// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xD0, 0x08,// 1104 = "vfmsub231ss"

	// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xD1, 0x08,// 1105 = "vfmsub231sd"

	// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD2, 0x08,// 1106 = "vfnmadd231ps"

	// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD3, 0x08,// 1107 = "vfnmadd231pd"

	// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xD2, 0x08,// 1106 = "vfnmadd231ps"

	// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xD3, 0x08,// 1107 = "vfnmadd231pd"

	// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xD4, 0x08,// 1108 = "vfnmadd231ss"

	// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xD5, 0x08,// 1109 = "vfnmadd231sd"

	// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xD4, 0x08,// 1108 = "vfnmadd231ss"

	// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xD5, 0x08,// 1109 = "vfnmadd231sd"

	// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD6, 0x08,// 1110 = "vfnmsub231ps"

	// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD7, 0x08,// 1111 = "vfnmsub231pd"

	// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xD6, 0x08,// 1110 = "vfnmsub231ps"

	// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x00,// No flags set
	0xD7, 0x08,// 1111 = "vfnmsub231pd"

	// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x02,// SameAsPrev

	// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	0x00,// No flags set
	0xD8, 0x08,// 1112 = "vfnmsub231ss"

	// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	0x00,// No flags set
	0xD9, 0x08,// 1113 = "vfnmsub231sd"

	// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xD8, 0x08,// 1112 = "vfnmsub231ss"

	// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xD9, 0x08,// 1113 = "vfnmsub231sd"

	// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	0x00,// No flags set
	0xDA, 0x08,// 1114 = "vpconflictd"

	// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	0x02,// SameAsPrev

	// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	0x00,// No flags set
	0xDB, 0x08,// 1115 = "vpconflictq"

	// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	0x02,// SameAsPrev

	// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	0x02,// SameAsPrev

	// EVEX_Vgatherpf0dps_vm32z_k1
	0x00,// No flags set
	0xDC, 0x08,// 1116 = "vgatherpf0dps"

	// EVEX_Vgatherpf0dpd_vm32y_k1
	0x00,// No flags set
	0xDD, 0x08,// 1117 = "vgatherpf0dpd"

	// EVEX_Vgatherpf1dps_vm32z_k1
	0x00,// No flags set
	0xDE, 0x08,// 1118 = "vgatherpf1dps"

	// EVEX_Vgatherpf1dpd_vm32y_k1
	0x00,// No flags set
	0xDF, 0x08,// 1119 = "vgatherpf1dpd"

	// EVEX_Vscatterpf0dps_vm32z_k1
	0x00,// No flags set
	0xE0, 0x08,// 1120 = "vscatterpf0dps"

	// EVEX_Vscatterpf0dpd_vm32y_k1
	0x00,// No flags set
	0xE1, 0x08,// 1121 = "vscatterpf0dpd"

	// EVEX_Vscatterpf1dps_vm32z_k1
	0x00,// No flags set
	0xE2, 0x08,// 1122 = "vscatterpf1dps"

	// EVEX_Vscatterpf1dpd_vm32y_k1
	0x00,// No flags set
	0xE3, 0x08,// 1123 = "vscatterpf1dpd"

	// EVEX_Vgatherpf0qps_vm64z_k1
	0x00,// No flags set
	0xE4, 0x08,// 1124 = "vgatherpf0qps"

	// EVEX_Vgatherpf0qpd_vm64z_k1
	0x00,// No flags set
	0xE5, 0x08,// 1125 = "vgatherpf0qpd"

	// EVEX_Vgatherpf1qps_vm64z_k1
	0x00,// No flags set
	0xE6, 0x08,// 1126 = "vgatherpf1qps"

	// EVEX_Vgatherpf1qpd_vm64z_k1
	0x00,// No flags set
	0xE7, 0x08,// 1127 = "vgatherpf1qpd"

	// EVEX_Vscatterpf0qps_vm64z_k1
	0x00,// No flags set
	0xE8, 0x08,// 1128 = "vscatterpf0qps"

	// EVEX_Vscatterpf0qpd_vm64z_k1
	0x00,// No flags set
	0xE9, 0x08,// 1129 = "vscatterpf0qpd"

	// EVEX_Vscatterpf1qps_vm64z_k1
	0x00,// No flags set
	0xEA, 0x08,// 1130 = "vscatterpf1qps"

	// EVEX_Vscatterpf1qpd_vm64z_k1
	0x00,// No flags set
	0xEB, 0x08,// 1131 = "vscatterpf1qpd"

	// Sha1nexte_xmm_xmmm128
	0x00,// No flags set
	0xEC, 0x08,// 1132 = "sha1nexte"

	// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	0x00,// No flags set
	0xED, 0x08,// 1133 = "vexp2ps"

	// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	0x00,// No flags set
	0xEE, 0x08,// 1134 = "vexp2pd"

	// Sha1msg1_xmm_xmmm128
	0x00,// No flags set
	0xEF, 0x08,// 1135 = "sha1msg1"

	// Sha1msg2_xmm_xmmm128
	0x00,// No flags set
	0xF0, 0x08,// 1136 = "sha1msg2"

	// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	0x00,// No flags set
	0xF1, 0x08,// 1137 = "vrcp28ps"

	// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	0x00,// No flags set
	0xF2, 0x08,// 1138 = "vrcp28pd"

	// Sha256rnds2_xmm_xmmm128
	0x00,// No flags set
	0xF3, 0x08,// 1139 = "sha256rnds2"

	// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	0x00,// No flags set
	0xF4, 0x08,// 1140 = "vrcp28ss"

	// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	0x00,// No flags set
	0xF5, 0x08,// 1141 = "vrcp28sd"

	// Sha256msg1_xmm_xmmm128
	0x00,// No flags set
	0xF6, 0x08,// 1142 = "sha256msg1"

	// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	0x00,// No flags set
	0xF7, 0x08,// 1143 = "vrsqrt28ps"

	// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	0x00,// No flags set
	0xF8, 0x08,// 1144 = "vrsqrt28pd"

	// Sha256msg2_xmm_xmmm128
	0x00,// No flags set
	0xF9, 0x08,// 1145 = "sha256msg2"

	// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	0x00,// No flags set
	0xFA, 0x08,// 1146 = "vrsqrt28ss"

	// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	0x00,// No flags set
	0xFB, 0x08,// 1147 = "vrsqrt28sd"

	// Gf2p8mulb_xmm_xmmm128
	0x00,// No flags set
	0xFC, 0x08,// 1148 = "gf2p8mulb"

	// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Aesimc_xmm_xmmm128
	0x00,// No flags set
	0xFD, 0x08,// 1149 = "aesimc"

	// VEX_Vaesimc_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// Aesenc_xmm_xmmm128
	0x00,// No flags set
	0xFE, 0x08,// 1150 = "aesenc"

	// VEX_Vaesenc_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaesenc_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenc_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenc_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenc_zmm_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Aesenclast_xmm_xmmm128
	0x00,// No flags set
	0xFF, 0x08,// 1151 = "aesenclast"

	// VEX_Vaesenclast_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaesenclast_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenclast_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenclast_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesenclast_zmm_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Aesdec_xmm_xmmm128
	0x00,// No flags set
	0x80, 0x09,// 1152 = "aesdec"

	// VEX_Vaesdec_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaesdec_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdec_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdec_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdec_zmm_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Aesdeclast_xmm_xmmm128
	0x00,// No flags set
	0x81, 0x09,// 1153 = "aesdeclast"

	// VEX_Vaesdeclast_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vaesdeclast_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	0x03,// HasVPrefix, SameAsPrev

	// Movbe_r16_m16
	0x00,// No flags set
	0x82, 0x09,// 1154 = "movbe"

	// Movbe_r32_m32
	0x02,// SameAsPrev

	// Movbe_r64_m64
	0x02,// SameAsPrev

	// Crc32_r32_rm8
	0x04,// ForceMemSize
	0x83, 0x09,// 1155 = "crc32"

	// Crc32_r64_rm8
	0x06,// SameAsPrev, ForceMemSize

	// Movbe_m16_r16
	0x00,// No flags set
	0x82, 0x09,// 1154 = "movbe"

	// Movbe_m32_r32
	0x02,// SameAsPrev

	// Movbe_m64_r64
	0x02,// SameAsPrev

	// Crc32_r32_rm16
	0x04,// ForceMemSize
	0x83, 0x09,// 1155 = "crc32"

	// Crc32_r32_rm32
	0x06,// SameAsPrev, ForceMemSize

	// Crc32_r64_rm64
	0x06,// SameAsPrev, ForceMemSize

	// VEX_Andn_r32_r32_rm32
	0x00,// No flags set
	0x84, 0x09,// 1156 = "andn"

	// VEX_Andn_r64_r64_rm64
	0x02,// SameAsPrev

	// VEX_Blsr_r32_rm32
	0x00,// No flags set
	0x85, 0x09,// 1157 = "blsr"

	// VEX_Blsr_r64_rm64
	0x02,// SameAsPrev

	// VEX_Blsmsk_r32_rm32
	0x00,// No flags set
	0x86, 0x09,// 1158 = "blsmsk"

	// VEX_Blsmsk_r64_rm64
	0x02,// SameAsPrev

	// VEX_Blsi_r32_rm32
	0x00,// No flags set
	0x87, 0x09,// 1159 = "blsi"

	// VEX_Blsi_r64_rm64
	0x02,// SameAsPrev

	// VEX_Bzhi_r32_rm32_r32
	0x00,// No flags set
	0x88, 0x09,// 1160 = "bzhi"

	// VEX_Bzhi_r64_rm64_r64
	0x02,// SameAsPrev

	// Wrussd_m32_r32
	0x00,// No flags set
	0x89, 0x09,// 1161 = "wrussd"

	// Wrussq_m64_r64
	0x00,// No flags set
	0x8A, 0x09,// 1162 = "wrussq"

	// VEX_Pext_r32_r32_rm32
	0x00,// No flags set
	0x8B, 0x09,// 1163 = "pext"

	// VEX_Pext_r64_r64_rm64
	0x02,// SameAsPrev

	// VEX_Pdep_r32_r32_rm32
	0x00,// No flags set
	0x8C, 0x09,// 1164 = "pdep"

	// VEX_Pdep_r64_r64_rm64
	0x02,// SameAsPrev

	// Wrssd_m32_r32
	0x00,// No flags set
	0x8D, 0x09,// 1165 = "wrssd"

	// Wrssq_m64_r64
	0x00,// No flags set
	0x8E, 0x09,// 1166 = "wrssq"

	// Adcx_r32_rm32
	0x00,// No flags set
	0x8F, 0x09,// 1167 = "adcx"

	// Adcx_r64_rm64
	0x02,// SameAsPrev

	// Adox_r32_rm32
	0x00,// No flags set
	0x90, 0x09,// 1168 = "adox"

	// Adox_r64_rm64
	0x02,// SameAsPrev

	// VEX_Mulx_r32_r32_rm32
	0x00,// No flags set
	0x91, 0x09,// 1169 = "mulx"

	// VEX_Mulx_r64_r64_rm64
	0x02,// SameAsPrev

	// VEX_Bextr_r32_rm32_r32
	0x00,// No flags set
	0x92, 0x09,// 1170 = "bextr"

	// VEX_Bextr_r64_rm64_r64
	0x02,// SameAsPrev

	// VEX_Shlx_r32_rm32_r32
	0x00,// No flags set
	0x93, 0x09,// 1171 = "shlx"

	// VEX_Shlx_r64_rm64_r64
	0x02,// SameAsPrev

	// VEX_Sarx_r32_rm32_r32
	0x00,// No flags set
	0x94, 0x09,// 1172 = "sarx"

	// VEX_Sarx_r64_rm64_r64
	0x02,// SameAsPrev

	// VEX_Shrx_r32_rm32_r32
	0x00,// No flags set
	0x95, 0x09,// 1173 = "shrx"

	// VEX_Shrx_r64_rm64_r64
	0x02,// SameAsPrev

	// Movdir64b_r16_m512
	0x00,// No flags set
	0x96, 0x09,// 1174 = "movdir64b"

	// Movdir64b_r32_m512
	0x02,// SameAsPrev

	// Movdir64b_r64_m512
	0x02,// SameAsPrev

	// Enqcmds_r16_m512
	0x00,// No flags set
	0x97, 0x09,// 1175 = "enqcmds"

	// Enqcmds_r32_m512
	0x02,// SameAsPrev

	// Enqcmds_r64_m512
	0x02,// SameAsPrev

	// Enqcmd_r16_m512
	0x00,// No flags set
	0x98, 0x09,// 1176 = "enqcmd"

	// Enqcmd_r32_m512
	0x02,// SameAsPrev

	// Enqcmd_r64_m512
	0x02,// SameAsPrev

	// Movdiri_m32_r32
	0x00,// No flags set
	0x99, 0x09,// 1177 = "movdiri"

	// Movdiri_m64_r64
	0x02,// SameAsPrev

	// VEX_Vpermq_ymm_ymmm256_imm8
	0x00,// No flags set
	0xA1, 0x07,// 929 = "vpermq"

	// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// VEX_Vpermpd_ymm_ymmm256_imm8
	0x00,// No flags set
	0xE7, 0x06,// 871 = "vpermpd"

	// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	0x00,// No flags set
	0x9A, 0x09,// 1178 = "vpblendd"

	// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0x9B, 0x09,// 1179 = "valignd"

	// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0x9C, 0x09,// 1180 = "valignq"

	// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// VEX_Vpermilps_xmm_xmmm128_imm8
	0x00,// No flags set
	0x27,// 39 = "vpermilps"

	// VEX_Vpermilps_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	0x02,// SameAsPrev

	// VEX_Vpermilpd_xmm_xmmm128_imm8
	0x00,// No flags set
	0x28,// 40 = "vpermilpd"

	// VEX_Vpermilpd_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	0x02,// SameAsPrev

	// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	0x00,// No flags set
	0x9D, 0x09,// 1181 = "vperm2f128"

	// Roundps_xmm_xmmm128_imm8
	0x00,// No flags set
	0x9E, 0x09,// 1182 = "roundps"

	// VEX_Vroundps_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vroundps_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	0x00,// No flags set
	0x9F, 0x09,// 1183 = "vrndscaleps"

	// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	0x02,// SameAsPrev

	// Roundpd_xmm_xmmm128_imm8
	0x00,// No flags set
	0xA0, 0x09,// 1184 = "roundpd"

	// VEX_Vroundpd_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vroundpd_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0xA1, 0x09,// 1185 = "vrndscalepd"

	// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	0x02,// SameAsPrev

	// Roundss_xmm_xmmm32_imm8
	0x00,// No flags set
	0xA2, 0x09,// 1186 = "roundss"

	// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	0x00,// No flags set
	0xA3, 0x09,// 1187 = "vrndscaless"

	// Roundsd_xmm_xmmm64_imm8
	0x00,// No flags set
	0xA4, 0x09,// 1188 = "roundsd"

	// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x00,// No flags set
	0xA5, 0x09,// 1189 = "vrndscalesd"

	// Blendps_xmm_xmmm128_imm8
	0x00,// No flags set
	0xA6, 0x09,// 1190 = "blendps"

	// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Blendpd_xmm_xmmm128_imm8
	0x00,// No flags set
	0xA7, 0x09,// 1191 = "blendpd"

	// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pblendw_xmm_xmmm128_imm8
	0x00,// No flags set
	0xA8, 0x09,// 1192 = "pblendw"

	// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Palignr_mm_mmm64_imm8
	0x00,// No flags set
	0x7C,// 124 = "palignr"

	// Palignr_xmm_xmmm128_imm8
	0x02,// SameAsPrev

	// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pextrb_r32m8_xmm_imm8
	0x00,// No flags set
	0xA9, 0x09,// 1193 = "pextrb"

	// Pextrb_r64m8_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpextrb_r32m8_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpextrb_r64m8_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrb_r32m8_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrb_r64m8_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pextrw_r32m16_xmm_imm8
	0x00,// No flags set
	0x1A,// 26 = "pextrw"

	// Pextrw_r64m16_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vpextrw_r32m16_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpextrw_r64m16_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrw_r32m16_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpextrw_r64m16_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pextrd_rm32_xmm_imm8
	0x00,// No flags set
	0xAA, 0x09,// 1194 = "pextrd"

	// Pextrq_rm64_xmm_imm8
	0x00,// No flags set
	0xAB, 0x09,// 1195 = "pextrq"

	// VEX_Vpextrd_rm32_xmm_imm8
	0x01,// HasVPrefix
	0xAA, 0x09,// 1194 = "vpextrd"

	// VEX_Vpextrq_rm64_xmm_imm8
	0x01,// HasVPrefix
	0xAB, 0x09,// 1195 = "vpextrq"

	// EVEX_Vpextrd_rm32_xmm_imm8
	0x01,// HasVPrefix
	0xAA, 0x09,// 1194 = "vpextrd"

	// EVEX_Vpextrq_rm64_xmm_imm8
	0x01,// HasVPrefix
	0xAB, 0x09,// 1195 = "vpextrq"

	// Extractps_rm32_xmm_imm8
	0x00,// No flags set
	0xAC, 0x09,// 1196 = "extractps"

	// Extractps_r64m32_xmm_imm8
	0x02,// SameAsPrev

	// VEX_Vextractps_rm32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vextractps_r64m32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vextractps_rm32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vextractps_r64m32_xmm_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	0x00,// No flags set
	0xAD, 0x09,// 1197 = "vinsertf128"

	// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	0x00,// No flags set
	0xAE, 0x09,// 1198 = "vinsertf32x4"

	// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	0x02,// SameAsPrev

	// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	0x00,// No flags set
	0xAF, 0x09,// 1199 = "vinsertf64x2"

	// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	0x02,// SameAsPrev

	// VEX_Vextractf128_xmmm128_ymm_imm8
	0x00,// No flags set
	0xB0, 0x09,// 1200 = "vextractf128"

	// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	0x00,// No flags set
	0xB1, 0x09,// 1201 = "vextractf32x4"

	// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	0x02,// SameAsPrev

	// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	0x00,// No flags set
	0xB2, 0x09,// 1202 = "vextractf64x2"

	// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	0x02,// SameAsPrev

	// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	0x00,// No flags set
	0xB3, 0x09,// 1203 = "vinsertf32x8"

	// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	0x00,// No flags set
	0xB4, 0x09,// 1204 = "vinsertf64x4"

	// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	0x00,// No flags set
	0xB5, 0x09,// 1205 = "vextractf32x8"

	// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	0x00,// No flags set
	0xB6, 0x09,// 1206 = "vextractf64x4"

	// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	0x00,// No flags set
	0xB7, 0x09,// 1207 = "vcvtps2ph"

	// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	0x02,// SameAsPrev

	// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	0x02,// SameAsPrev

	// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	0x02,// SameAsPrev

	// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	0xC8,// vpcmpud
	0xB8, 0x09,// 1208 = "vpcmpud"

	// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	0xCA,// SameAsPrev, vpcmpud

	// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	0xCA,// SameAsPrev, vpcmpud

	// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	0xD0,// vpcmpuq
	0xB9, 0x09,// 1209 = "vpcmpuq"

	// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	0xD2,// SameAsPrev, vpcmpuq

	// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	0xD2,// SameAsPrev, vpcmpuq

	// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	0xA8,// vpcmpd
	0xBA, 0x09,// 1210 = "vpcmpd"

	// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	0xAA,// SameAsPrev, vpcmpd

	// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	0xAA,// SameAsPrev, vpcmpd

	// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	0xB0,// vpcmpq
	0xBB, 0x09,// 1211 = "vpcmpq"

	// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	0xB2,// SameAsPrev, vpcmpq

	// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	0xB2,// SameAsPrev, vpcmpq

	// Pinsrb_xmm_r32m8_imm8
	0x00,// No flags set
	0xBC, 0x09,// 1212 = "pinsrb"

	// Pinsrb_xmm_r64m8_imm8
	0x02,// SameAsPrev

	// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Insertps_xmm_xmmm32_imm8
	0x00,// No flags set
	0xBD, 0x09,// 1213 = "insertps"

	// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pinsrd_xmm_rm32_imm8
	0x00,// No flags set
	0xBE, 0x09,// 1214 = "pinsrd"

	// Pinsrq_xmm_rm64_imm8
	0x00,// No flags set
	0xBF, 0x09,// 1215 = "pinsrq"

	// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x01,// HasVPrefix
	0xBE, 0x09,// 1214 = "vpinsrd"

	// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x01,// HasVPrefix
	0xBF, 0x09,// 1215 = "vpinsrq"

	// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x01,// HasVPrefix
	0xBE, 0x09,// 1214 = "vpinsrd"

	// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x01,// HasVPrefix
	0xBF, 0x09,// 1215 = "vpinsrq"

	// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x00,// No flags set
	0xC0, 0x09,// 1216 = "vshuff32x4"

	// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x00,// No flags set
	0xC1, 0x09,// 1217 = "vshuff64x2"

	// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0xC2, 0x09,// 1218 = "vpternlogd"

	// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0xC3, 0x09,// 1219 = "vpternlogq"

	// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	0x00,// No flags set
	0xC4, 0x09,// 1220 = "vgetmantps"

	// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0xC5, 0x09,// 1221 = "vgetmantpd"

	// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x00,// No flags set
	0xC6, 0x09,// 1222 = "vgetmantss"

	// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x00,// No flags set
	0xC7, 0x09,// 1223 = "vgetmantsd"

	// VEX_Kshiftrb_kr_kr_imm8
	0x00,// No flags set
	0xC8, 0x09,// 1224 = "kshiftrb"

	// VEX_Kshiftrw_kr_kr_imm8
	0x00,// No flags set
	0xC9, 0x09,// 1225 = "kshiftrw"

	// VEX_Kshiftrd_kr_kr_imm8
	0x00,// No flags set
	0xCA, 0x09,// 1226 = "kshiftrd"

	// VEX_Kshiftrq_kr_kr_imm8
	0x00,// No flags set
	0xCB, 0x09,// 1227 = "kshiftrq"

	// VEX_Kshiftlb_kr_kr_imm8
	0x00,// No flags set
	0xCC, 0x09,// 1228 = "kshiftlb"

	// VEX_Kshiftlw_kr_kr_imm8
	0x00,// No flags set
	0xCD, 0x09,// 1229 = "kshiftlw"

	// VEX_Kshiftld_kr_kr_imm8
	0x00,// No flags set
	0xCE, 0x09,// 1230 = "kshiftld"

	// VEX_Kshiftlq_kr_kr_imm8
	0x00,// No flags set
	0xCF, 0x09,// 1231 = "kshiftlq"

	// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	0x00,// No flags set
	0xD0, 0x09,// 1232 = "vinserti128"

	// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	0x00,// No flags set
	0xD1, 0x09,// 1233 = "vinserti32x4"

	// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	0x02,// SameAsPrev

	// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	0x00,// No flags set
	0xD2, 0x09,// 1234 = "vinserti64x2"

	// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	0x02,// SameAsPrev

	// VEX_Vextracti128_xmmm128_ymm_imm8
	0x00,// No flags set
	0xD3, 0x09,// 1235 = "vextracti128"

	// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	0x00,// No flags set
	0xD4, 0x09,// 1236 = "vextracti32x4"

	// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	0x02,// SameAsPrev

	// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	0x00,// No flags set
	0xD5, 0x09,// 1237 = "vextracti64x2"

	// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	0x02,// SameAsPrev

	// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	0x00,// No flags set
	0xD6, 0x09,// 1238 = "vinserti32x8"

	// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	0x00,// No flags set
	0xD7, 0x09,// 1239 = "vinserti64x4"

	// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	0x00,// No flags set
	0xD8, 0x09,// 1240 = "vextracti32x8"

	// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	0x00,// No flags set
	0xD9, 0x09,// 1241 = "vextracti64x4"

	// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	0xB8,// vpcmpub
	0xDA, 0x09,// 1242 = "vpcmpub"

	// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	0xBA,// SameAsPrev, vpcmpub

	// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	0xBA,// SameAsPrev, vpcmpub

	// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	0xC0,// vpcmpuw
	0xDB, 0x09,// 1243 = "vpcmpuw"

	// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	0xC2,// SameAsPrev, vpcmpuw

	// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	0xC2,// SameAsPrev, vpcmpuw

	// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	0x98,// vpcmpb
	0xDC, 0x09,// 1244 = "vpcmpb"

	// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	0x9A,// SameAsPrev, vpcmpb

	// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	0x9A,// SameAsPrev, vpcmpb

	// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	0xA0,// vpcmpw
	0xDD, 0x09,// 1245 = "vpcmpw"

	// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	0xA2,// SameAsPrev, vpcmpw

	// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	0xA2,// SameAsPrev, vpcmpw

	// Dpps_xmm_xmmm128_imm8
	0x00,// No flags set
	0xDE, 0x09,// 1246 = "dpps"

	// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Dppd_xmm_xmmm128_imm8
	0x00,// No flags set
	0xDF, 0x09,// 1247 = "dppd"

	// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Mpsadbw_xmm_xmmm128_imm8
	0x00,// No flags set
	0xE0, 0x09,// 1248 = "mpsadbw"

	// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	0x00,// No flags set
	0xE1, 0x09,// 1249 = "vdbpsadbw"

	// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	0x02,// SameAsPrev

	// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x00,// No flags set
	0xE2, 0x09,// 1250 = "vshufi32x4"

	// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x00,// No flags set
	0xE3, 0x09,// 1251 = "vshufi64x2"

	// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// Pclmulqdq_xmm_xmmm128_imm8
	0x48,// pclmulqdq
	0xE4, 0x09,// 1252 = "pclmulqdq"

	// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x53,// HasVPrefix, SameAsPrev, vpclmulqdq

	// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x53,// HasVPrefix, SameAsPrev, vpclmulqdq

	// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x53,// HasVPrefix, SameAsPrev, vpclmulqdq

	// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x53,// HasVPrefix, SameAsPrev, vpclmulqdq

	// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	0x53,// HasVPrefix, SameAsPrev, vpclmulqdq

	// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	0x00,// No flags set
	0xE5, 0x09,// 1253 = "vperm2i128"

	// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	0x00,// No flags set
	0xE6, 0x09,// 1254 = "vpermil2ps"

	// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	0x02,// SameAsPrev

	// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	0x02,// SameAsPrev

	// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	0x02,// SameAsPrev

	// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	0x00,// No flags set
	0xE7, 0x09,// 1255 = "vpermil2pd"

	// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	0x02,// SameAsPrev

	// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	0x02,// SameAsPrev

	// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	0x02,// SameAsPrev

	// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	0x01,// HasVPrefix
	0xDE, 0x06,// 862 = "vblendvps"

	// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	0x01,// HasVPrefix
	0xE2, 0x06,// 866 = "vblendvpd"

	// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	0x01,// HasVPrefix
	0xD5, 0x06,// 853 = "vpblendvb"

	// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0xE8, 0x09,// 1256 = "vrangeps"

	// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0xE9, 0x09,// 1257 = "vrangepd"

	// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x00,// No flags set
	0xEA, 0x09,// 1258 = "vrangess"

	// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x00,// No flags set
	0xEB, 0x09,// 1259 = "vrangesd"

	// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0xEC, 0x09,// 1260 = "vfixupimmps"

	// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0xED, 0x09,// 1261 = "vfixupimmpd"

	// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x00,// No flags set
	0xEE, 0x09,// 1262 = "vfixupimmss"

	// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x00,// No flags set
	0xEF, 0x09,// 1263 = "vfixupimmsd"

	// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	0x00,// No flags set
	0xF0, 0x09,// 1264 = "vreduceps"

	// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	0x00,// No flags set
	0xF1, 0x09,// 1265 = "vreducepd"

	// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x00,// No flags set
	0xF2, 0x09,// 1266 = "vreducess"

	// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x00,// No flags set
	0xF3, 0x09,// 1267 = "vreducesd"

	// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xF4, 0x09,// 1268 = "vfmaddsubps"

	// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xF5, 0x09,// 1269 = "vfmaddsubpd"

	// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xF6, 0x09,// 1270 = "vfmsubaddps"

	// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xF7, 0x09,// 1271 = "vfmsubaddpd"

	// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// Pcmpestrm_xmm_xmmm128_imm8
	0x00,// No flags set
	0xF8, 0x09,// 1272 = "pcmpestrm"

	// Pcmpestrm64_xmm_xmmm128_imm8
	0x00,// No flags set
	0xF9, 0x09,// 1273 = "pcmpestrm64"

	// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	0x01,// HasVPrefix
	0xF8, 0x09,// 1272 = "vpcmpestrm"

	// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	0x01,// HasVPrefix
	0xF9, 0x09,// 1273 = "vpcmpestrm64"

	// Pcmpestri_xmm_xmmm128_imm8
	0x00,// No flags set
	0xFA, 0x09,// 1274 = "pcmpestri"

	// Pcmpestri64_xmm_xmmm128_imm8
	0x00,// No flags set
	0xFB, 0x09,// 1275 = "pcmpestri64"

	// VEX_Vpcmpestri_xmm_xmmm128_imm8
	0x01,// HasVPrefix
	0xFA, 0x09,// 1274 = "vpcmpestri"

	// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	0x01,// HasVPrefix
	0xFB, 0x09,// 1275 = "vpcmpestri64"

	// Pcmpistrm_xmm_xmmm128_imm8
	0x00,// No flags set
	0xFC, 0x09,// 1276 = "pcmpistrm"

	// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Pcmpistri_xmm_xmmm128_imm8
	0x00,// No flags set
	0xFD, 0x09,// 1277 = "pcmpistri"

	// VEX_Vpcmpistri_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	0x04,// ForceMemSize
	0xFE, 0x09,// 1278 = "vfpclassps"

	// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	0x04,// ForceMemSize
	0xFF, 0x09,// 1279 = "vfpclasspd"

	// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	0x00,// No flags set
	0x80, 0x0A,// 1280 = "vfpclassss"

	// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	0x00,// No flags set
	0x81, 0x0A,// 1281 = "vfpclasssd"

	// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x82, 0x0A,// 1282 = "vfmaddps"

	// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x83, 0x0A,// 1283 = "vfmaddpd"

	// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	0x00,// No flags set
	0x84, 0x0A,// 1284 = "vfmaddss"

	// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	0x02,// SameAsPrev

	// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	0x00,// No flags set
	0x85, 0x0A,// 1285 = "vfmaddsd"

	// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	0x02,// SameAsPrev

	// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x86, 0x0A,// 1286 = "vfmsubps"

	// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x87, 0x0A,// 1287 = "vfmsubpd"

	// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	0x00,// No flags set
	0x88, 0x0A,// 1288 = "vfmsubss"

	// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	0x02,// SameAsPrev

	// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	0x00,// No flags set
	0x89, 0x0A,// 1289 = "vfmsubsd"

	// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	0x02,// SameAsPrev

	// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	0x00,// No flags set
	0x8A, 0x0A,// 1290 = "vpshldw"

	// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0x8B, 0x0A,// 1291 = "vpshldd"

	// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0x8C, 0x0A,// 1292 = "vpshldq"

	// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	0x00,// No flags set
	0x8D, 0x0A,// 1293 = "vpshrdw"

	// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	0x00,// No flags set
	0x8E, 0x0A,// 1294 = "vpshrdd"

	// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	0x00,// No flags set
	0x8F, 0x0A,// 1295 = "vpshrdq"

	// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	0x02,// SameAsPrev

	// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	0x02,// SameAsPrev

	// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x90, 0x0A,// 1296 = "vfnmaddps"

	// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x91, 0x0A,// 1297 = "vfnmaddpd"

	// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	0x00,// No flags set
	0x92, 0x0A,// 1298 = "vfnmaddss"

	// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	0x02,// SameAsPrev

	// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	0x00,// No flags set
	0x93, 0x0A,// 1299 = "vfnmaddsd"

	// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	0x02,// SameAsPrev

	// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x94, 0x0A,// 1300 = "vfnmsubps"

	// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x95, 0x0A,// 1301 = "vfnmsubpd"

	// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	0x00,// No flags set
	0x96, 0x0A,// 1302 = "vfnmsubss"

	// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	0x02,// SameAsPrev

	// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	0x00,// No flags set
	0x97, 0x0A,// 1303 = "vfnmsubsd"

	// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	0x02,// SameAsPrev

	// Sha1rnds4_xmm_xmmm128_imm8
	0x00,// No flags set
	0x98, 0x0A,// 1304 = "sha1rnds4"

	// Gf2p8affineqb_xmm_xmmm128_imm8
	0x00,// No flags set
	0x99, 0x0A,// 1305 = "gf2p8affineqb"

	// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Gf2p8affineinvqb_xmm_xmmm128_imm8
	0x00,// No flags set
	0x9A, 0x0A,// 1306 = "gf2p8affineinvqb"

	// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x03,// HasVPrefix, SameAsPrev

	// Aeskeygenassist_xmm_xmmm128_imm8
	0x00,// No flags set
	0x9B, 0x0A,// 1307 = "aeskeygenassist"

	// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	0x03,// HasVPrefix, SameAsPrev

	// VEX_Rorx_r32_rm32_imm8
	0x00,// No flags set
	0x9C, 0x0A,// 1308 = "rorx"

	// VEX_Rorx_r64_rm64_imm8
	0x02,// SameAsPrev

	// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x9D, 0x0A,// 1309 = "vpmacssww"

	// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x9E, 0x0A,// 1310 = "vpmacsswd"

	// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0x9F, 0x0A,// 1311 = "vpmacssdql"

	// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA0, 0x0A,// 1312 = "vpmacssdd"

	// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA1, 0x0A,// 1313 = "vpmacssdqh"

	// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA2, 0x0A,// 1314 = "vpmacsww"

	// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA3, 0x0A,// 1315 = "vpmacswd"

	// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA4, 0x0A,// 1316 = "vpmacsdql"

	// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA5, 0x0A,// 1317 = "vpmacsdd"

	// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA6, 0x0A,// 1318 = "vpmacsdqh"

	// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA7, 0x0A,// 1319 = "vpcmov"

	// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	0x02,// SameAsPrev

	// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA8, 0x0A,// 1320 = "vpperm"

	// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xA9, 0x0A,// 1321 = "vpmadcsswd"

	// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	0x00,// No flags set
	0xAA, 0x0A,// 1322 = "vpmadcswd"

	// XOP_Vprotb_xmm_xmmm128_imm8
	0x00,// No flags set
	0xAB, 0x0A,// 1323 = "vprotb"

	// XOP_Vprotw_xmm_xmmm128_imm8
	0x00,// No flags set
	0xAC, 0x0A,// 1324 = "vprotw"

	// XOP_Vprotd_xmm_xmmm128_imm8
	0x00,// No flags set
	0xAD, 0x0A,// 1325 = "vprotd"

	// XOP_Vprotq_xmm_xmmm128_imm8
	0x00,// No flags set
	0xAE, 0x0A,// 1326 = "vprotq"

	// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	0x58,// vpcomb
	0xAF, 0x0A,// 1327 = "vpcomb"

	// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	0x60,// vpcomw
	0xB0, 0x0A,// 1328 = "vpcomw"

	// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	0x68,// vpcomd
	0xB1, 0x0A,// 1329 = "vpcomd"

	// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	0x70,// vpcomq
	0xB2, 0x0A,// 1330 = "vpcomq"

	// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	0x78,// vpcomub
	0xB3, 0x0A,// 1331 = "vpcomub"

	// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	0x80,// vpcomuw
	0xB4, 0x0A,// 1332 = "vpcomuw"

	// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	0x88,// vpcomud
	0xB5, 0x0A,// 1333 = "vpcomud"

	// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	0x90,// vpcomuq
	0xB6, 0x0A,// 1334 = "vpcomuq"

	// XOP_Blcfill_r32_rm32
	0x00,// No flags set
	0xB7, 0x0A,// 1335 = "blcfill"

	// XOP_Blcfill_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blsfill_r32_rm32
	0x00,// No flags set
	0xB8, 0x0A,// 1336 = "blsfill"

	// XOP_Blsfill_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blcs_r32_rm32
	0x00,// No flags set
	0xB9, 0x0A,// 1337 = "blcs"

	// XOP_Blcs_r64_rm64
	0x02,// SameAsPrev

	// XOP_Tzmsk_r32_rm32
	0x00,// No flags set
	0xBA, 0x0A,// 1338 = "tzmsk"

	// XOP_Tzmsk_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blcic_r32_rm32
	0x00,// No flags set
	0xBB, 0x0A,// 1339 = "blcic"

	// XOP_Blcic_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blsic_r32_rm32
	0x00,// No flags set
	0xBC, 0x0A,// 1340 = "blsic"

	// XOP_Blsic_r64_rm64
	0x02,// SameAsPrev

	// XOP_T1mskc_r32_rm32
	0x00,// No flags set
	0xBD, 0x0A,// 1341 = "t1mskc"

	// XOP_T1mskc_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blcmsk_r32_rm32
	0x00,// No flags set
	0xBE, 0x0A,// 1342 = "blcmsk"

	// XOP_Blcmsk_r64_rm64
	0x02,// SameAsPrev

	// XOP_Blci_r32_rm32
	0x00,// No flags set
	0xBF, 0x0A,// 1343 = "blci"

	// XOP_Blci_r64_rm64
	0x02,// SameAsPrev

	// XOP_Llwpcb_r32
	0x00,// No flags set
	0xC0, 0x0A,// 1344 = "llwpcb"

	// XOP_Llwpcb_r64
	0x02,// SameAsPrev

	// XOP_Slwpcb_r32
	0x00,// No flags set
	0xC1, 0x0A,// 1345 = "slwpcb"

	// XOP_Slwpcb_r64
	0x02,// SameAsPrev

	// XOP_Vfrczps_xmm_xmmm128
	0x00,// No flags set
	0xC2, 0x0A,// 1346 = "vfrczps"

	// XOP_Vfrczps_ymm_ymmm256
	0x02,// SameAsPrev

	// XOP_Vfrczpd_xmm_xmmm128
	0x00,// No flags set
	0xC3, 0x0A,// 1347 = "vfrczpd"

	// XOP_Vfrczpd_ymm_ymmm256
	0x02,// SameAsPrev

	// XOP_Vfrczss_xmm_xmmm32
	0x00,// No flags set
	0xC4, 0x0A,// 1348 = "vfrczss"

	// XOP_Vfrczsd_xmm_xmmm64
	0x00,// No flags set
	0xC5, 0x0A,// 1349 = "vfrczsd"

	// XOP_Vprotb_xmm_xmmm128_xmm
	0x00,// No flags set
	0xAB, 0x0A,// 1323 = "vprotb"

	// XOP_Vprotb_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vprotw_xmm_xmmm128_xmm
	0x00,// No flags set
	0xAC, 0x0A,// 1324 = "vprotw"

	// XOP_Vprotw_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vprotd_xmm_xmmm128_xmm
	0x00,// No flags set
	0xAD, 0x0A,// 1325 = "vprotd"

	// XOP_Vprotd_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vprotq_xmm_xmmm128_xmm
	0x00,// No flags set
	0xAE, 0x0A,// 1326 = "vprotq"

	// XOP_Vprotq_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshlb_xmm_xmmm128_xmm
	0x00,// No flags set
	0xC6, 0x0A,// 1350 = "vpshlb"

	// XOP_Vpshlb_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshlw_xmm_xmmm128_xmm
	0x00,// No flags set
	0xC7, 0x0A,// 1351 = "vpshlw"

	// XOP_Vpshlw_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshld_xmm_xmmm128_xmm
	0x00,// No flags set
	0xC8, 0x0A,// 1352 = "vpshld"

	// XOP_Vpshld_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshlq_xmm_xmmm128_xmm
	0x00,// No flags set
	0xC9, 0x0A,// 1353 = "vpshlq"

	// XOP_Vpshlq_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshab_xmm_xmmm128_xmm
	0x00,// No flags set
	0xCA, 0x0A,// 1354 = "vpshab"

	// XOP_Vpshab_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshaw_xmm_xmmm128_xmm
	0x00,// No flags set
	0xCB, 0x0A,// 1355 = "vpshaw"

	// XOP_Vpshaw_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshad_xmm_xmmm128_xmm
	0x00,// No flags set
	0xCC, 0x0A,// 1356 = "vpshad"

	// XOP_Vpshad_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vpshaq_xmm_xmmm128_xmm
	0x00,// No flags set
	0xCD, 0x0A,// 1357 = "vpshaq"

	// XOP_Vpshaq_xmm_xmm_xmmm128
	0x02,// SameAsPrev

	// XOP_Vphaddbw_xmm_xmmm128
	0x00,// No flags set
	0xCE, 0x0A,// 1358 = "vphaddbw"

	// XOP_Vphaddbd_xmm_xmmm128
	0x00,// No flags set
	0xCF, 0x0A,// 1359 = "vphaddbd"

	// XOP_Vphaddbq_xmm_xmmm128
	0x00,// No flags set
	0xD0, 0x0A,// 1360 = "vphaddbq"

	// XOP_Vphaddwd_xmm_xmmm128
	0x00,// No flags set
	0xD1, 0x0A,// 1361 = "vphaddwd"

	// XOP_Vphaddwq_xmm_xmmm128
	0x00,// No flags set
	0xD2, 0x0A,// 1362 = "vphaddwq"

	// XOP_Vphadddq_xmm_xmmm128
	0x00,// No flags set
	0xD3, 0x0A,// 1363 = "vphadddq"

	// XOP_Vphaddubw_xmm_xmmm128
	0x00,// No flags set
	0xD4, 0x0A,// 1364 = "vphaddubw"

	// XOP_Vphaddubd_xmm_xmmm128
	0x00,// No flags set
	0xD5, 0x0A,// 1365 = "vphaddubd"

	// XOP_Vphaddubq_xmm_xmmm128
	0x00,// No flags set
	0xD6, 0x0A,// 1366 = "vphaddubq"

	// XOP_Vphadduwd_xmm_xmmm128
	0x00,// No flags set
	0xD7, 0x0A,// 1367 = "vphadduwd"

	// XOP_Vphadduwq_xmm_xmmm128
	0x00,// No flags set
	0xD8, 0x0A,// 1368 = "vphadduwq"

	// XOP_Vphaddudq_xmm_xmmm128
	0x00,// No flags set
	0xD9, 0x0A,// 1369 = "vphaddudq"

	// XOP_Vphsubbw_xmm_xmmm128
	0x00,// No flags set
	0xDA, 0x0A,// 1370 = "vphsubbw"

	// XOP_Vphsubwd_xmm_xmmm128
	0x00,// No flags set
	0xDB, 0x0A,// 1371 = "vphsubwd"

	// XOP_Vphsubdq_xmm_xmmm128
	0x00,// No flags set
	0xDC, 0x0A,// 1372 = "vphsubdq"

	// XOP_Bextr_r32_rm32_imm32
	0x00,// No flags set
	0x92, 0x09,// 1170 = "bextr"

	// XOP_Bextr_r64_rm64_imm32
	0x02,// SameAsPrev

	// XOP_Lwpins_r32_rm32_imm32
	0x00,// No flags set
	0xDD, 0x0A,// 1373 = "lwpins"

	// XOP_Lwpins_r64_rm32_imm32
	0x02,// SameAsPrev

	// XOP_Lwpval_r32_rm32_imm32
	0x00,// No flags set
	0xDE, 0x0A,// 1374 = "lwpval"

	// XOP_Lwpval_r64_rm32_imm32
	0x02,// SameAsPrev

	// D3NOW_Pi2fw_mm_mmm64
	0x00,// No flags set
	0xDF, 0x0A,// 1375 = "pi2fw"

	// D3NOW_Pi2fd_mm_mmm64
	0x00,// No flags set
	0xE0, 0x0A,// 1376 = "pi2fd"

	// D3NOW_Pf2iw_mm_mmm64
	0x00,// No flags set
	0xE1, 0x0A,// 1377 = "pf2iw"

	// D3NOW_Pf2id_mm_mmm64
	0x00,// No flags set
	0xE2, 0x0A,// 1378 = "pf2id"

	// D3NOW_Pfrcpv_mm_mmm64
	0x00,// No flags set
	0xE3, 0x0A,// 1379 = "pfrcpv"

	// D3NOW_Pfrsqrtv_mm_mmm64
	0x00,// No flags set
	0xE4, 0x0A,// 1380 = "pfrsqrtv"

	// D3NOW_Pfnacc_mm_mmm64
	0x00,// No flags set
	0xE5, 0x0A,// 1381 = "pfnacc"

	// D3NOW_Pfpnacc_mm_mmm64
	0x00,// No flags set
	0xE6, 0x0A,// 1382 = "pfpnacc"

	// D3NOW_Pfcmpge_mm_mmm64
	0x00,// No flags set
	0xE7, 0x0A,// 1383 = "pfcmpge"

	// D3NOW_Pfmin_mm_mmm64
	0x00,// No flags set
	0xE8, 0x0A,// 1384 = "pfmin"

	// D3NOW_Pfrcp_mm_mmm64
	0x00,// No flags set
	0xE9, 0x0A,// 1385 = "pfrcp"

	// D3NOW_Pfrsqrt_mm_mmm64
	0x00,// No flags set
	0xEA, 0x0A,// 1386 = "pfrsqrt"

	// D3NOW_Pfsub_mm_mmm64
	0x00,// No flags set
	0xEB, 0x0A,// 1387 = "pfsub"

	// D3NOW_Pfadd_mm_mmm64
	0x00,// No flags set
	0xEC, 0x0A,// 1388 = "pfadd"

	// D3NOW_Pfcmpgt_mm_mmm64
	0x00,// No flags set
	0xED, 0x0A,// 1389 = "pfcmpgt"

	// D3NOW_Pfmax_mm_mmm64
	0x00,// No flags set
	0xEE, 0x0A,// 1390 = "pfmax"

	// D3NOW_Pfrcpit1_mm_mmm64
	0x00,// No flags set
	0xEF, 0x0A,// 1391 = "pfrcpit1"

	// D3NOW_Pfrsqit1_mm_mmm64
	0x00,// No flags set
	0xF0, 0x0A,// 1392 = "pfrsqit1"

	// D3NOW_Pfsubr_mm_mmm64
	0x00,// No flags set
	0xF1, 0x0A,// 1393 = "pfsubr"

	// D3NOW_Pfacc_mm_mmm64
	0x00,// No flags set
	0xF2, 0x0A,// 1394 = "pfacc"

	// D3NOW_Pfcmpeq_mm_mmm64
	0x00,// No flags set
	0xF3, 0x0A,// 1395 = "pfcmpeq"

	// D3NOW_Pfmul_mm_mmm64
	0x00,// No flags set
	0xF4, 0x0A,// 1396 = "pfmul"

	// D3NOW_Pfrcpit2_mm_mmm64
	0x00,// No flags set
	0xF5, 0x0A,// 1397 = "pfrcpit2"

	// D3NOW_Pmulhrw_mm_mmm64
	0x00,// No flags set
	0xF6, 0x0A,// 1398 = "pmulhrw"

	// D3NOW_Pswapd_mm_mmm64
	0x00,// No flags set
	0xF8, 0x0A,// 1400 = "pswapd"

	// D3NOW_Pavgusb_mm_mmm64
	0x00,// No flags set
	0xF9, 0x0A,// 1401 = "pavgusb"

	// Rmpadjust
	0x00,// No flags set
	0xFA, 0x0A,// 1402 = "rmpadjust"

	// Rmpupdate
	0x00,// No flags set
	0xFB, 0x0A,// 1403 = "rmpupdate"

	// Psmash
	0x00,// No flags set
	0xFC, 0x0A,// 1404 = "psmash"

	// Pvalidatew
	0x00,// No flags set
	0xFD, 0x0A,// 1405 = "pvalidate"

	// Pvalidated
	0x02,// SameAsPrev

	// Pvalidateq
	0x02,// SameAsPrev

	// Serialize
	0x00,// No flags set
	0xFE, 0x0A,// 1406 = "serialize"

	// Xsusldtrk
	0x00,// No flags set
	0xFF, 0x0A,// 1407 = "xsusldtrk"

	// Xresldtrk
	0x00,// No flags set
	0x80, 0x0B,// 1408 = "xresldtrk"

	// Invlpgbw
	0x00,// No flags set
	0x81, 0x0B,// 1409 = "invlpgb"

	// Invlpgbd
	0x02,// SameAsPrev

	// Invlpgbq
	0x02,// SameAsPrev

	// Tlbsync
	0x00,// No flags set
	0x82, 0x0B,// 1410 = "tlbsync"

	// Prefetchreserved3_m8
	0x00,// No flags set
	0xD3, 0x03,// 467 = "prefetchw"

	// Prefetchreserved4_m8
	0x00,// No flags set
	0xD1, 0x03,// 465 = "prefetch"

	// Prefetchreserved5_m8
	0x02,// SameAsPrev

	// Prefetchreserved6_m8
	0x02,// SameAsPrev

	// Prefetchreserved7_m8
	0x02,// SameAsPrev

	// Ud0
	0x00,// No flags set
	0xC9, 0x06,// 841 = "ud0"

	// Vmgexit
	0x00,// No flags set
	0x84, 0x0B,// 1412 = "vmgexit"

	// Getsecq
	0x00,// No flags set
	0x85, 0x0B,// 1413 = "getsecq"

	// VEX_Ldtilecfg_m512
	0x00,// No flags set
	0x86, 0x0B,// 1414 = "ldtilecfg"

	// VEX_Tilerelease
	0x00,// No flags set
	0x87, 0x0B,// 1415 = "tilerelease"

	// VEX_Sttilecfg_m512
	0x00,// No flags set
	0x88, 0x0B,// 1416 = "sttilecfg"

	// VEX_Tilezero_tmm
	0x00,// No flags set
	0x89, 0x0B,// 1417 = "tilezero"

	// VEX_Tileloaddt1_tmm_sibmem
	0x00,// No flags set
	0x8A, 0x0B,// 1418 = "tileloaddt1"

	// VEX_Tilestored_sibmem_tmm
	0x00,// No flags set
	0x8B, 0x0B,// 1419 = "tilestored"

	// VEX_Tileloadd_tmm_sibmem
	0x00,// No flags set
	0x8C, 0x0B,// 1420 = "tileloadd"

	// VEX_Tdpbf16ps_tmm_tmm_tmm
	0x00,// No flags set
	0x8D, 0x0B,// 1421 = "tdpbf16ps"

	// VEX_Tdpbuud_tmm_tmm_tmm
	0x00,// No flags set
	0x8E, 0x0B,// 1422 = "tdpbuud"

	// VEX_Tdpbusd_tmm_tmm_tmm
	0x00,// No flags set
	0x8F, 0x0B,// 1423 = "tdpbusd"

	// VEX_Tdpbsud_tmm_tmm_tmm
	0x00,// No flags set
	0x90, 0x0B,// 1424 = "tdpbsud"

	// VEX_Tdpbssd_tmm_tmm_tmm
	0x00,// No flags set
	0x91, 0x0B,// 1425 = "tdpbssd"

	// Fnstdw_AX
	0x00,// No flags set
	0x92, 0x0B,// 1426 = "fnstdw"

	// Fnstsg_AX
	0x00,// No flags set
	0x93, 0x0B,// 1427 = "fnstsg"

	// Rdshr_rm32
	0x00,// No flags set
	0x94, 0x0B,// 1428 = "rdshr"

	// Wrshr_rm32
	0x00,// No flags set
	0x95, 0x0B,// 1429 = "wrshr"

	// Smint
	0x00,// No flags set
	0x96, 0x0B,// 1430 = "smint"

	// Dmint
	0x00,// No flags set
	0x97, 0x0B,// 1431 = "dmint"

	// Rdm
	0x00,// No flags set
	0x98, 0x0B,// 1432 = "rdm"

	// Svdc_m80_Sreg
	0x00,// No flags set
	0x99, 0x0B,// 1433 = "svdc"

	// Rsdc_Sreg_m80
	0x00,// No flags set
	0x9A, 0x0B,// 1434 = "rsdc"

	// Svldt_m80
	0x00,// No flags set
	0x9B, 0x0B,// 1435 = "svldt"

	// Rsldt_m80
	0x00,// No flags set
	0x9C, 0x0B,// 1436 = "rsldt"

	// Svts_m80
	0x00,// No flags set
	0x9D, 0x0B,// 1437 = "svts"

	// Rsts_m80
	0x00,// No flags set
	0x9E, 0x0B,// 1438 = "rsts"

	// Smint_0F7E
	0x00,// No flags set
	0x96, 0x0B,// 1430 = "smint"

	// Bb0_reset
	0x00,// No flags set
	0xA0, 0x0B,// 1440 = "bb0_reset"

	// Bb1_reset
	0x00,// No flags set
	0xA1, 0x0B,// 1441 = "bb1_reset"

	// Cpu_write
	0x00,// No flags set
	0xA2, 0x0B,// 1442 = "cpu_write"

	// Cpu_read
	0x00,// No flags set
	0xA3, 0x0B,// 1443 = "cpu_read"

	// Altinst
	0x00,// No flags set
	0xA4, 0x0B,// 1444 = "altinst"

	// Paveb_mm_mmm64
	0x00,// No flags set
	0xA5, 0x0B,// 1445 = "paveb"

	// Paddsiw_mm_mmm64
	0x00,// No flags set
	0xA6, 0x0B,// 1446 = "paddsiw"

	// Pmagw_mm_mmm64
	0x00,// No flags set
	0xA7, 0x0B,// 1447 = "pmagw"

	// Pdistib_mm_m64
	0x00,// No flags set
	0xA8, 0x0B,// 1448 = "pdistib"

	// Psubsiw_mm_mmm64
	0x00,// No flags set
	0xA9, 0x0B,// 1449 = "psubsiw"

	// Pmvzb_mm_m64
	0x00,// No flags set
	0xAA, 0x0B,// 1450 = "pmvzb"

	// Pmulhrw_mm_mmm64
	0x00,// No flags set
	0xF6, 0x0A,// 1398 = "pmulhrw"

	// Pmvnzb_mm_m64
	0x00,// No flags set
	0xAC, 0x0B,// 1452 = "pmvnzb"

	// Pmvlzb_mm_m64
	0x00,// No flags set
	0xAD, 0x0B,// 1453 = "pmvlzb"

	// Pmvgezb_mm_m64
	0x00,// No flags set
	0xAE, 0x0B,// 1454 = "pmvgezb"

	// Pmulhriw_mm_mmm64
	0x00,// No flags set
	0xAF, 0x0B,// 1455 = "pmulhriw"

	// Pmachriw_mm_m64
	0x00,// No flags set
	0xB0, 0x0B,// 1456 = "pmachriw"

	// Cyrix_D9D7
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// Cyrix_D9E2
	0x02,// SameAsPrev

	// Ftstp
	0x00,// No flags set
	0xB1, 0x0B,// 1457 = "ftstp"

	// Cyrix_D9E7
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// Frint2
	0x00,// No flags set
	0xB2, 0x0B,// 1458 = "frint2"

	// Frichop
	0x00,// No flags set
	0xB3, 0x0B,// 1459 = "frichop"

	// Cyrix_DED8
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// Cyrix_DEDA
	0x02,// SameAsPrev

	// Cyrix_DEDC
	0x02,// SameAsPrev

	// Cyrix_DEDD
	0x02,// SameAsPrev

	// Cyrix_DEDE
	0x02,// SameAsPrev

	// Frinear
	0x00,// No flags set
	0xB4, 0x0B,// 1460 = "frinear"

	// Tdcall
	0x00,// No flags set
	0xB5, 0x0B,// 1461 = "tdcall"

	// Seamret
	0x00,// No flags set
	0xB6, 0x0B,// 1462 = "seamret"

	// Seamops
	0x00,// No flags set
	0xB7, 0x0B,// 1463 = "seamops"

	// Seamcall
	0x00,// No flags set
	0xB8, 0x0B,// 1464 = "seamcall"

	// Aesencwide128kl_m384
	0x00,// No flags set
	0xB9, 0x0B,// 1465 = "aesencwide128kl"

	// Aesdecwide128kl_m384
	0x00,// No flags set
	0xBA, 0x0B,// 1466 = "aesdecwide128kl"

	// Aesencwide256kl_m512
	0x00,// No flags set
	0xBB, 0x0B,// 1467 = "aesencwide256kl"

	// Aesdecwide256kl_m512
	0x00,// No flags set
	0xBC, 0x0B,// 1468 = "aesdecwide256kl"

	// Loadiwkey_xmm_xmm
	0x00,// No flags set
	0xBD, 0x0B,// 1469 = "loadiwkey"

	// Aesenc128kl_xmm_m384
	0x00,// No flags set
	0xBE, 0x0B,// 1470 = "aesenc128kl"

	// Aesdec128kl_xmm_m384
	0x00,// No flags set
	0xBF, 0x0B,// 1471 = "aesdec128kl"

	// Aesenc256kl_xmm_m512
	0x00,// No flags set
	0xC0, 0x0B,// 1472 = "aesenc256kl"

	// Aesdec256kl_xmm_m512
	0x00,// No flags set
	0xC1, 0x0B,// 1473 = "aesdec256kl"

	// Encodekey128_r32_r32
	0x00,// No flags set
	0xC2, 0x0B,// 1474 = "encodekey128"

	// Encodekey256_r32_r32
	0x00,// No flags set
	0xC3, 0x0B,// 1475 = "encodekey256"

	// VEX_Vbroadcastss_xmm_xmm
	0x00,// No flags set
	0x3A,// 58 = "vbroadcastss"

	// VEX_Vbroadcastss_ymm_xmm
	0x02,// SameAsPrev

	// VEX_Vbroadcastsd_ymm_xmm
	0x00,// No flags set
	0xE9, 0x06,// 873 = "vbroadcastsd"

	// Vmgexit_F2
	0x00,// No flags set
	0x84, 0x0B,// 1412 = "vmgexit"

	// Uiret
	0x00,// No flags set
	0xC4, 0x0B,// 1476 = "uiret"

	// Testui
	0x00,// No flags set
	0xC5, 0x0B,// 1477 = "testui"

	// Clui
	0x00,// No flags set
	0xC6, 0x0B,// 1478 = "clui"

	// Stui
	0x00,// No flags set
	0xC7, 0x0B,// 1479 = "stui"

	// Senduipi_r64
	0x00,// No flags set
	0xC8, 0x0B,// 1480 = "senduipi"

	// Hreset_imm8
	0x00,// No flags set
	0xC9, 0x0B,// 1481 = "hreset"

	// VEX_Vpdpbusd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC6, 0x07,// 966 = "vpdpbusd"

	// VEX_Vpdpbusd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbusds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC7, 0x07,// 967 = "vpdpbusds"

	// VEX_Vpdpbusds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwssd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC8, 0x07,// 968 = "vpdpwssd"

	// VEX_Vpdpwssd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwssds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xCB, 0x07,// 971 = "vpdpwssds"

	// VEX_Vpdpwssds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// Ccs_hash_16
	0x00,// No flags set
	0xCA, 0x0B,// 1482 = "ccs_hash"

	// Ccs_hash_32
	0x02,// SameAsPrev

	// Ccs_hash_64
	0x02,// SameAsPrev

	// Ccs_encrypt_16
	0x00,// No flags set
	0xCB, 0x0B,// 1483 = "ccs_encrypt"

	// Ccs_encrypt_32
	0x02,// SameAsPrev

	// Ccs_encrypt_64
	0x02,// SameAsPrev

	// Lkgs_rm16
	0x00,// No flags set
	0xCC, 0x0B,// 1484 = "lkgs"

	// Lkgs_r32m16
	0x02,// SameAsPrev

	// Lkgs_r64m16
	0x02,// SameAsPrev

	// Eretu
	0x00,// No flags set
	0xCD, 0x0B,// 1485 = "eretu"

	// Erets
	0x00,// No flags set
	0xCE, 0x0B,// 1486 = "erets"

	// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xCF, 0x0B,// 1487 = "vaddph"

	// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xD0, 0x0B,// 1488 = "vaddsh"

	// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	0xD8,// vcmpph
	0xD1, 0x0B,// 1489 = "vcmpph"

	// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	0xDA,// SameAsPrev, vcmpph

	// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	0xDA,// SameAsPrev, vcmpph

	// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	0xE0,// vcmpsh
	0xD2, 0x0B,// 1490 = "vcmpsh"

	// EVEX_Vcomish_xmm_xmmm16_sae
	0x00,// No flags set
	0xD3, 0x0B,// 1491 = "vcomish"

	// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	0x04,// ForceMemSize
	0xD4, 0x0B,// 1492 = "vcvtdq2ph"

	// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0xD5, 0x0B,// 1493 = "vcvtpd2ph"

	// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	0x00,// No flags set
	0xD6, 0x0B,// 1494 = "vcvtph2dq"

	// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	0x00,// No flags set
	0xD7, 0x0B,// 1495 = "vcvtph2pd"

	// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	0x00,// No flags set
	0xD8, 0x0B,// 1496 = "vcvtph2psx"

	// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	0x00,// No flags set
	0xD9, 0x0B,// 1497 = "vcvtph2qq"

	// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	0x00,// No flags set
	0xDA, 0x0B,// 1498 = "vcvtph2udq"

	// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	0x00,// No flags set
	0xDB, 0x0B,// 1499 = "vcvtph2uqq"

	// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xDC, 0x0B,// 1500 = "vcvtph2uw"

	// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xDD, 0x0B,// 1501 = "vcvtph2w"

	// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	0x04,// ForceMemSize
	0xDE, 0x0B,// 1502 = "vcvtps2phx"

	// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0xDF, 0x0B,// 1503 = "vcvtqq2ph"

	// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	0x00,// No flags set
	0xE0, 0x0B,// 1504 = "vcvtsd2sh"

	// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	0x00,// No flags set
	0xE1, 0x0B,// 1505 = "vcvtsh2sd"

	// EVEX_Vcvtsh2si_r32_xmmm16_er
	0x00,// No flags set
	0xE2, 0x0B,// 1506 = "vcvtsh2si"

	// EVEX_Vcvtsh2si_r64_xmmm16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	0x00,// No flags set
	0xE3, 0x0B,// 1507 = "vcvtsh2ss"

	// EVEX_Vcvtsh2usi_r32_xmmm16_er
	0x00,// No flags set
	0xE4, 0x0B,// 1508 = "vcvtsh2usi"

	// EVEX_Vcvtsh2usi_r64_xmmm16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	0x04,// ForceMemSize
	0xE5, 0x0B,// 1509 = "vcvtsi2sh"

	// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xE6, 0x0B,// 1510 = "vcvtss2sh"

	// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	0x00,// No flags set
	0xE7, 0x0B,// 1511 = "vcvttph2dq"

	// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	0x00,// No flags set
	0xE8, 0x0B,// 1512 = "vcvttph2qq"

	// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	0x00,// No flags set
	0xE9, 0x0B,// 1513 = "vcvttph2udq"

	// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	0x00,// No flags set
	0xEA, 0x0B,// 1514 = "vcvttph2uqq"

	// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xEB, 0x0B,// 1515 = "vcvttph2uw"

	// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xEC, 0x0B,// 1516 = "vcvttph2w"

	// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttsh2si_r32_xmmm16_sae
	0x00,// No flags set
	0xED, 0x0B,// 1517 = "vcvttsh2si"

	// EVEX_Vcvttsh2si_r64_xmmm16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	0x00,// No flags set
	0xEE, 0x0B,// 1518 = "vcvttsh2usi"

	// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	0x02,// SameAsPrev

	// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	0x04,// ForceMemSize
	0xEF, 0x0B,// 1519 = "vcvtudq2ph"

	// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	0x04,// ForceMemSize
	0xF0, 0x0B,// 1520 = "vcvtuqq2ph"

	// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	0x04,// ForceMemSize
	0xF1, 0x0B,// 1521 = "vcvtusi2sh"

	// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xF2, 0x0B,// 1522 = "vcvtuw2ph"

	// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xF3, 0x0B,// 1523 = "vcvtw2ph"

	// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xF4, 0x0B,// 1524 = "vdivph"

	// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xF5, 0x0B,// 1525 = "vdivsh"

	// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xF6, 0x0B,// 1526 = "vfcmaddcph"

	// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xF7, 0x0B,// 1527 = "vfmaddcph"

	// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xF8, 0x0B,// 1528 = "vfcmaddcsh"

	// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xF9, 0x0B,// 1529 = "vfmaddcsh"

	// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xFA, 0x0B,// 1530 = "vfcmulcph"

	// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	0x00,// No flags set
	0xFB, 0x0B,// 1531 = "vfmulcph"

	// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	0x02,// SameAsPrev

	// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x02,// SameAsPrev

	// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xFC, 0x0B,// 1532 = "vfcmulcsh"

	// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	0x00,// No flags set
	0xFD, 0x0B,// 1533 = "vfmulcsh"

	// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xFE, 0x0B,// 1534 = "vfmaddsub132ph"

	// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xFF, 0x0B,// 1535 = "vfmaddsub213ph"

	// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x80, 0x0C,// 1536 = "vfmaddsub231ph"

	// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x81, 0x0C,// 1537 = "vfmsubadd132ph"

	// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x82, 0x0C,// 1538 = "vfmsubadd213ph"

	// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x83, 0x0C,// 1539 = "vfmsubadd231ph"

	// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x84, 0x0C,// 1540 = "vfmadd132ph"

	// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x85, 0x0C,// 1541 = "vfmadd213ph"

	// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x86, 0x0C,// 1542 = "vfmadd231ph"

	// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x87, 0x0C,// 1543 = "vfnmadd132ph"

	// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x88, 0x0C,// 1544 = "vfnmadd213ph"

	// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x89, 0x0C,// 1545 = "vfnmadd231ph"

	// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8A, 0x0C,// 1546 = "vfmadd132sh"

	// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8B, 0x0C,// 1547 = "vfmadd213sh"

	// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8C, 0x0C,// 1548 = "vfmadd231sh"

	// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8D, 0x0C,// 1549 = "vfnmadd132sh"

	// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8E, 0x0C,// 1550 = "vfnmadd213sh"

	// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x8F, 0x0C,// 1551 = "vfnmadd231sh"

	// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x90, 0x0C,// 1552 = "vfmsub132ph"

	// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x91, 0x0C,// 1553 = "vfmsub213ph"

	// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x92, 0x0C,// 1554 = "vfmsub231ph"

	// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x93, 0x0C,// 1555 = "vfnmsub132ph"

	// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x94, 0x0C,// 1556 = "vfnmsub213ph"

	// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0x95, 0x0C,// 1557 = "vfnmsub231ph"

	// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x96, 0x0C,// 1558 = "vfmsub132sh"

	// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x97, 0x0C,// 1559 = "vfmsub213sh"

	// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x98, 0x0C,// 1560 = "vfmsub231sh"

	// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x99, 0x0C,// 1561 = "vfnmsub132sh"

	// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x9A, 0x0C,// 1562 = "vfnmsub213sh"

	// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0x9B, 0x0C,// 1563 = "vfnmsub231sh"

	// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	0x04,// ForceMemSize
	0x9C, 0x0C,// 1564 = "vfpclassph"

	// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	0x06,// SameAsPrev, ForceMemSize

	// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	0x00,// No flags set
	0x9D, 0x0C,// 1565 = "vfpclasssh"

	// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0x9E, 0x0C,// 1566 = "vgetexpph"

	// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	0x02,// SameAsPrev

	// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	0x00,// No flags set
	0x9F, 0x0C,// 1567 = "vgetexpsh"

	// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	0x00,// No flags set
	0xA0, 0x0C,// 1568 = "vgetmantph"

	// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	0x02,// SameAsPrev

	// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x00,// No flags set
	0xA1, 0x0C,// 1569 = "vgetmantsh"

	// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xA2, 0x0C,// 1570 = "vmaxph"

	// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	0x02,// SameAsPrev

	// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	0x00,// No flags set
	0xA3, 0x0C,// 1571 = "vmaxsh"

	// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xA4, 0x0C,// 1572 = "vminph"

	// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	0x02,// SameAsPrev

	// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	0x00,// No flags set
	0xA5, 0x0C,// 1573 = "vminsh"

	// EVEX_Vmovsh_xmm_k1z_m16
	0x00,// No flags set
	0xA6, 0x0C,// 1574 = "vmovsh"

	// EVEX_Vmovsh_m16_k1_xmm
	0x02,// SameAsPrev

	// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	0x02,// SameAsPrev

	// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	0x02,// SameAsPrev

	// EVEX_Vmovw_xmm_r32m16
	0x00,// No flags set
	0xA7, 0x0C,// 1575 = "vmovw"

	// EVEX_Vmovw_xmm_r64m16
	0x02,// SameAsPrev

	// EVEX_Vmovw_r32m16_xmm
	0x02,// SameAsPrev

	// EVEX_Vmovw_r64m16_xmm
	0x02,// SameAsPrev

	// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xA8, 0x0C,// 1576 = "vmulph"

	// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xA9, 0x0C,// 1577 = "vmulsh"

	// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xAA, 0x0C,// 1578 = "vrcpph"

	// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	0x02,// SameAsPrev

	// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	0x00,// No flags set
	0xAB, 0x0C,// 1579 = "vrcpsh"

	// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	0x00,// No flags set
	0xAC, 0x0C,// 1580 = "vreduceph"

	// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	0x02,// SameAsPrev

	// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x00,// No flags set
	0xAD, 0x0C,// 1581 = "vreducesh"

	// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	0x00,// No flags set
	0xAE, 0x0C,// 1582 = "vrndscaleph"

	// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	0x02,// SameAsPrev

	// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	0x02,// SameAsPrev

	// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x00,// No flags set
	0xAF, 0x0C,// 1583 = "vrndscalesh"

	// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xB0, 0x0C,// 1584 = "vrsqrtph"

	// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	0x02,// SameAsPrev

	// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	0x00,// No flags set
	0xB1, 0x0C,// 1585 = "vrsqrtsh"

	// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xB2, 0x0C,// 1586 = "vscalefph"

	// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xB3, 0x0C,// 1587 = "vscalefsh"

	// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	0x00,// No flags set
	0xB4, 0x0C,// 1588 = "vsqrtph"

	// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xB5, 0x0C,// 1589 = "vsqrtsh"

	// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	0x00,// No flags set
	0xB6, 0x0C,// 1590 = "vsubph"

	// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	0x02,// SameAsPrev

	// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	0x02,// SameAsPrev

	// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	0x00,// No flags set
	0xB7, 0x0C,// 1591 = "vsubsh"

	// EVEX_Vucomish_xmm_xmmm16_sae
	0x00,// No flags set
	0xB8, 0x0C,// 1592 = "vucomish"

	// Rdudbg
	0x00,// No flags set
	0xB9, 0x0C,// 1593 = "rdudbg"

	// Wrudbg
	0x00,// No flags set
	0xBA, 0x0C,// 1594 = "wrudbg"

	// VEX_KNC_Jkzd_kr_rel8_64
	0x00,// No flags set
	0xBB, 0x0C,// 1595 = "jkzd"

	// VEX_KNC_Jknzd_kr_rel8_64
	0x00,// No flags set
	0xBC, 0x0C,// 1596 = "jknzd"

	// VEX_KNC_Vprefetchnta_m8
	0x01,// HasVPrefix
	0xE4, 0x03,// 484 = "vprefetchnta"

	// VEX_KNC_Vprefetch0_m8
	0x00,// No flags set
	0xBD, 0x0C,// 1597 = "vprefetch0"

	// VEX_KNC_Vprefetch1_m8
	0x00,// No flags set
	0xBE, 0x0C,// 1598 = "vprefetch1"

	// VEX_KNC_Vprefetch2_m8
	0x00,// No flags set
	0xBF, 0x0C,// 1599 = "vprefetch2"

	// VEX_KNC_Vprefetchenta_m8
	0x00,// No flags set
	0xC0, 0x0C,// 1600 = "vprefetchenta"

	// VEX_KNC_Vprefetche0_m8
	0x00,// No flags set
	0xC1, 0x0C,// 1601 = "vprefetche0"

	// VEX_KNC_Vprefetche1_m8
	0x00,// No flags set
	0xC2, 0x0C,// 1602 = "vprefetche1"

	// VEX_KNC_Vprefetche2_m8
	0x00,// No flags set
	0xC3, 0x0C,// 1603 = "vprefetche2"

	// VEX_KNC_Kand_kr_kr
	0x00,// No flags set
	0xC4, 0x0C,// 1604 = "kand"

	// VEX_KNC_Kandn_kr_kr
	0x00,// No flags set
	0xC5, 0x0C,// 1605 = "kandn"

	// VEX_KNC_Kandnr_kr_kr
	0x00,// No flags set
	0xC6, 0x0C,// 1606 = "kandnr"

	// VEX_KNC_Knot_kr_kr
	0x00,// No flags set
	0xC7, 0x0C,// 1607 = "knot"

	// VEX_KNC_Kor_kr_kr
	0x00,// No flags set
	0xC8, 0x0C,// 1608 = "kor"

	// VEX_KNC_Kxnor_kr_kr
	0x00,// No flags set
	0xC9, 0x0C,// 1609 = "kxnor"

	// VEX_KNC_Kxor_kr_kr
	0x00,// No flags set
	0xCA, 0x0C,// 1610 = "kxor"

	// VEX_KNC_Kmerge2l1h_kr_kr
	0x00,// No flags set
	0xCB, 0x0C,// 1611 = "kmerge2l1h"

	// VEX_KNC_Kmerge2l1l_kr_kr
	0x00,// No flags set
	0xCC, 0x0C,// 1612 = "kmerge2l1l"

	// VEX_KNC_Jkzd_kr_rel32_64
	0x00,// No flags set
	0xBB, 0x0C,// 1595 = "jkzd"

	// VEX_KNC_Jknzd_kr_rel32_64
	0x00,// No flags set
	0xBC, 0x0C,// 1596 = "jknzd"

	// VEX_KNC_Kmov_kr_kr
	0x00,// No flags set
	0xCD, 0x0C,// 1613 = "kmov"

	// VEX_KNC_Kmov_kr_r32
	0x02,// SameAsPrev

	// VEX_KNC_Kmov_r32_kr
	0x02,// SameAsPrev

	// VEX_KNC_Kconcath_r64_kr_kr
	0x00,// No flags set
	0xCE, 0x0C,// 1614 = "kconcath"

	// VEX_KNC_Kconcatl_r64_kr_kr
	0x00,// No flags set
	0xCF, 0x0C,// 1615 = "kconcatl"

	// VEX_KNC_Kortest_kr_kr
	0x00,// No flags set
	0xD0, 0x0C,// 1616 = "kortest"

	// VEX_KNC_Delay_r32
	0x00,// No flags set
	0xD1, 0x0C,// 1617 = "delay"

	// VEX_KNC_Delay_r64
	0x02,// SameAsPrev

	// VEX_KNC_Spflt_r32
	0x00,// No flags set
	0xD2, 0x0C,// 1618 = "spflt"

	// VEX_KNC_Spflt_r64
	0x02,// SameAsPrev

	// VEX_KNC_Clevict1_m8
	0x00,// No flags set
	0xD3, 0x0C,// 1619 = "clevict1"

	// VEX_KNC_Clevict0_m8
	0x00,// No flags set
	0xD4, 0x0C,// 1620 = "clevict0"

	// VEX_KNC_Popcnt_r32_r32
	0x00,// No flags set
	0x8D, 0x06,// 781 = "popcnt"

	// VEX_KNC_Popcnt_r64_r64
	0x02,// SameAsPrev

	// VEX_KNC_Tzcnt_r32_r32
	0x00,// No flags set
	0x91, 0x06,// 785 = "tzcnt"

	// VEX_KNC_Tzcnt_r64_r64
	0x02,// SameAsPrev

	// VEX_KNC_Tzcnti_r32_r32
	0x00,// No flags set
	0xD5, 0x0C,// 1621 = "tzcnti"

	// VEX_KNC_Tzcnti_r64_r64
	0x02,// SameAsPrev

	// VEX_KNC_Lzcnt_r32_r32
	0x00,// No flags set
	0x93, 0x06,// 787 = "lzcnt"

	// VEX_KNC_Lzcnt_r64_r64
	0x02,// SameAsPrev

	// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	0x02,// SameAsPrev

	// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	0x02,// SameAsPrev

	// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	0x02,// SameAsPrev

	// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	0x02,// SameAsPrev

	// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	0x02,// SameAsPrev

	// VEX_KNC_Kextract_kr_r64_imm8
	0x00,// No flags set
	0xD6, 0x0C,// 1622 = "kextract"

	// MVEX_Vprefetchnta_m
	0x01,// HasVPrefix
	0xE4, 0x03,// 484 = "vprefetchnta"

	// MVEX_Vprefetch0_m
	0x00,// No flags set
	0xBD, 0x0C,// 1597 = "vprefetch0"

	// MVEX_Vprefetch1_m
	0x00,// No flags set
	0xBE, 0x0C,// 1598 = "vprefetch1"

	// MVEX_Vprefetch2_m
	0x00,// No flags set
	0xBF, 0x0C,// 1599 = "vprefetch2"

	// MVEX_Vprefetchenta_m
	0x00,// No flags set
	0xC0, 0x0C,// 1600 = "vprefetchenta"

	// MVEX_Vprefetche0_m
	0x00,// No flags set
	0xC1, 0x0C,// 1601 = "vprefetche0"

	// MVEX_Vprefetche1_m
	0x00,// No flags set
	0xC2, 0x0C,// 1602 = "vprefetche1"

	// MVEX_Vprefetche2_m
	0x00,// No flags set
	0xC3, 0x0C,// 1603 = "vprefetche2"

	// MVEX_Vmovaps_zmm_k1_zmmmt
	0x01,// HasVPrefix
	0x13,// 19 = "vmovaps"

	// MVEX_Vmovapd_zmm_k1_zmmmt
	0x01,// HasVPrefix
	0x14,// 20 = "vmovapd"

	// MVEX_Vmovaps_mt_k1_zmm
	0x01,// HasVPrefix
	0x13,// 19 = "vmovaps"

	// MVEX_Vmovapd_mt_k1_zmm
	0x01,// HasVPrefix
	0x14,// 20 = "vmovapd"

	// MVEX_Vmovnrapd_m_k1_zmm
	0x00,// No flags set
	0xD7, 0x0C,// 1623 = "vmovnrapd"

	// MVEX_Vmovnrngoapd_m_k1_zmm
	0x00,// No flags set
	0xD8, 0x0C,// 1624 = "vmovnrngoapd"

	// MVEX_Vmovnraps_m_k1_zmm
	0x00,// No flags set
	0xD9, 0x0C,// 1625 = "vmovnraps"

	// MVEX_Vmovnrngoaps_m_k1_zmm
	0x00,// No flags set
	0xDA, 0x0C,// 1626 = "vmovnrngoaps"

	// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x3E,// 62 = "vaddps"

	// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x3F,// 63 = "vaddpd"

	// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x40,// 64 = "vmulps"

	// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x41,// 65 = "vmulpd"

	// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	0x01,// HasVPrefix
	0x42,// 66 = "vcvtps2pd"

	// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	0x01,// HasVPrefix
	0x43,// 67 = "vcvtpd2ps"

	// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x44,// 68 = "vsubps"

	// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x45,// 69 = "vsubpd"

	// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x2E,// 46 = "vpcmpgtd"

	// MVEX_Vmovdqa32_zmm_k1_zmmmt
	0x00,// No flags set
	0x30,// 48 = "vmovdqa32"

	// MVEX_Vmovdqa64_zmm_k1_zmmmt
	0x00,// No flags set
	0x31,// 49 = "vmovdqa64"

	// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	0x01,// HasVPrefix
	0x51,// 81 = "vpshufd"

	// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	0x01,// HasVPrefix
	0x0F,// 15 = "vpsrld"

	// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	0x01,// HasVPrefix
	0x10,// 16 = "vpsrad"

	// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	0x01,// HasVPrefix
	0x11,// 17 = "vpslld"

	// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x32,// 50 = "vpcmpeqd"

	// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	0x00,// No flags set
	0x9E, 0x05,// 670 = "vcvtudq2pd"

	// MVEX_Vmovdqa32_mt_k1_zmm
	0x00,// No flags set
	0x30,// 48 = "vmovdqa32"

	// MVEX_Vmovdqa64_mt_k1_zmm
	0x00,// No flags set
	0x31,// 49 = "vmovdqa64"

	// MVEX_Clevict1_m
	0x00,// No flags set
	0xD3, 0x0C,// 1619 = "clevict1"

	// MVEX_Clevict0_m
	0x00,// No flags set
	0xD4, 0x0C,// 1620 = "clevict0"

	// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	0xE9,// HasVPrefix, vcmpps8
	0x54,// 84 = "vcmpps"

	// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	0xF1,// HasVPrefix, vcmppd8
	0x55,// 85 = "vcmppd"

	// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB6, 0x06,// 822 = "vpandd"

	// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB7, 0x06,// 823 = "vpandq"

	// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB9, 0x06,// 825 = "vpandnd"

	// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xBA, 0x06,// 826 = "vpandnq"

	// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	0x01,// HasVPrefix
	0x62,// 98 = "vcvtdq2pd"

	// MVEX_Vpord_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC1, 0x06,// 833 = "vpord"

	// MVEX_Vporq_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC2, 0x06,// 834 = "vporq"

	// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC4, 0x06,// 836 = "vpxord"

	// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC5, 0x06,// 837 = "vpxorq"

	// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x38,// 56 = "vpsubd"

	// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x39,// 57 = "vpaddd"

	// MVEX_Vbroadcastss_zmm_k1_mt
	0x00,// No flags set
	0x3A,// 58 = "vbroadcastss"

	// MVEX_Vbroadcastsd_zmm_k1_mt
	0x00,// No flags set
	0xE9, 0x06,// 873 = "vbroadcastsd"

	// MVEX_Vbroadcastf32x4_zmm_k1_mt
	0x00,// No flags set
	0xEC, 0x06,// 876 = "vbroadcastf32x4"

	// MVEX_Vbroadcastf64x4_zmm_k1_mt
	0x00,// No flags set
	0xEF, 0x06,// 879 = "vbroadcastf64x4"

	// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	0x00,// No flags set
	0x81, 0x07,// 897 = "vptestmd"

	// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xA0, 0x07,// 928 = "vpermd"

	// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x77,// 119 = "vpminsd"

	// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x78,// 120 = "vpminud"

	// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x79,// 121 = "vpmaxsd"

	// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x7A,// 122 = "vpmaxud"

	// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	0x01,// HasVPrefix
	0x7B,// 123 = "vpmulld"

	// MVEX_Vgetexpps_zmm_k1_zmmmt
	0x00,// No flags set
	0xB2, 0x07,// 946 = "vgetexpps"

	// MVEX_Vgetexppd_zmm_k1_zmmmt
	0x00,// No flags set
	0xB3, 0x07,// 947 = "vgetexppd"

	// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB8, 0x07,// 952 = "vpsrlvd"

	// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xBA, 0x07,// 954 = "vpsravd"

	// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xBC, 0x07,// 956 = "vpsllvd"

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	0x02,// SameAsPrev

	// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDB, 0x0C,// 1627 = "vaddnps"

	// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDC, 0x0C,// 1628 = "vaddnpd"

	// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDD, 0x0C,// 1629 = "vgmaxabsps"

	// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDE, 0x0C,// 1630 = "vgminps"

	// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDF, 0x0C,// 1631 = "vgminpd"

	// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE0, 0x0C,// 1632 = "vgmaxps"

	// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE1, 0x0C,// 1633 = "vgmaxpd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE2, 0x0C,// 1634 = "vfixupnanps"

	// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE3, 0x0C,// 1635 = "vfixupnanpd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	0x02,// SameAsPrev

	// MVEX_Vpbroadcastd_zmm_k1_mt
	0x00,// No flags set
	0x2C,// 44 = "vpbroadcastd"

	// MVEX_Vpbroadcastq_zmm_k1_mt
	0x00,// No flags set
	0x2D,// 45 = "vpbroadcastq"

	// MVEX_Vbroadcasti32x4_zmm_k1_mt
	0x00,// No flags set
	0xD3, 0x07,// 979 = "vbroadcasti32x4"

	// MVEX_Vbroadcasti64x4_zmm_k1_mt
	0x00,// No flags set
	0xD6, 0x07,// 982 = "vbroadcasti64x4"

	// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xE4, 0x0C,// 1636 = "vpadcd"

	// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xE5, 0x0C,// 1637 = "vpaddsetcd"

	// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xE6, 0x0C,// 1638 = "vpsbbd"

	// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xE7, 0x0C,// 1639 = "vpsubsetbd"

	// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDB, 0x07,// 987 = "vpblendmd"

	// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDC, 0x07,// 988 = "vpblendmq"

	// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDD, 0x07,// 989 = "vblendmps"

	// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xDE, 0x07,// 990 = "vblendmpd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	0x02,// SameAsPrev

	// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE8, 0x0C,// 1640 = "vpsubrd"

	// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xE9, 0x0C,// 1641 = "vsubrps"

	// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xEA, 0x0C,// 1642 = "vsubrpd"

	// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xEB, 0x0C,// 1643 = "vpsbbrd"

	// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	0x00,// No flags set
	0xEC, 0x0C,// 1644 = "vpsubrsetbd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	0x02,// SameAsPrev

	// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	0x00,// No flags set
	0xED, 0x0C,// 1645 = "vpcmpltd"

	// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xEE, 0x0C,// 1646 = "vscaleps"

	// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xEF, 0x0C,// 1647 = "vpmulhud"

	// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xF0, 0x0C,// 1648 = "vpmulhd"

	// MVEX_Vpgatherdd_zmm_k1_mvt
	0x00,// No flags set
	0x88, 0x08,// 1032 = "vpgatherdd"

	// MVEX_Vpgatherdq_zmm_k1_mvt
	0x00,// No flags set
	0x89, 0x08,// 1033 = "vpgatherdq"

	// MVEX_Vgatherdps_zmm_k1_mvt
	0x00,// No flags set
	0x8C, 0x08,// 1036 = "vgatherdps"

	// MVEX_Vgatherdpd_zmm_k1_mvt
	0x00,// No flags set
	0x8D, 0x08,// 1037 = "vgatherdpd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	0x02,// SameAsPrev

	// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x94, 0x08,// 1044 = "vfmadd132ps"

	// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x95, 0x08,// 1045 = "vfmadd132pd"

	// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x98, 0x08,// 1048 = "vfmsub132ps"

	// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x99, 0x08,// 1049 = "vfmsub132pd"

	// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x9E, 0x08,// 1054 = "vfnmadd132ps"

	// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0x9F, 0x08,// 1055 = "vfnmadd132pd"

	// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xA2, 0x08,// 1058 = "vfnmsub132ps"

	// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xA3, 0x08,// 1059 = "vfnmsub132pd"

	// MVEX_Vpscatterdd_mvt_k1_zmm
	0x00,// No flags set
	0xA6, 0x08,// 1062 = "vpscatterdd"

	// MVEX_Vpscatterdq_mvt_k1_zmm
	0x00,// No flags set
	0xA7, 0x08,// 1063 = "vpscatterdq"

	// MVEX_Vscatterdps_mvt_k1_zmm
	0x00,// No flags set
	0xAA, 0x08,// 1066 = "vscatterdps"

	// MVEX_Vscatterdpd_mvt_k1_zmm
	0x00,// No flags set
	0xAB, 0x08,// 1067 = "vscatterdpd"

	// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xF1, 0x0C,// 1649 = "vfmadd233ps"

	// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB2, 0x08,// 1074 = "vfmadd213ps"

	// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB3, 0x08,// 1075 = "vfmadd213pd"

	// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB6, 0x08,// 1078 = "vfmsub213ps"

	// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xB7, 0x08,// 1079 = "vfmsub213pd"

	// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xBC, 0x08,// 1084 = "vfnmadd213ps"

	// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xBD, 0x08,// 1085 = "vfnmadd213pd"

	// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC0, 0x08,// 1088 = "vfnmsub213ps"

	// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xC1, 0x08,// 1089 = "vfnmsub213pd"

	// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	0x02,// SameAsPrev

	// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xF2, 0x0C,// 1650 = "vpmadd233d"

	// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xF3, 0x0C,// 1651 = "vpmadd231d"

	// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xCA, 0x08,// 1098 = "vfmadd231ps"

	// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xCB, 0x08,// 1099 = "vfmadd231pd"

	// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xCE, 0x08,// 1102 = "vfmsub231ps"

	// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xCF, 0x08,// 1103 = "vfmsub231pd"

	// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xD2, 0x08,// 1106 = "vfnmadd231ps"

	// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xD3, 0x08,// 1107 = "vfnmadd231pd"

	// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xD6, 0x08,// 1110 = "vfnmsub231ps"

	// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xD7, 0x08,// 1111 = "vfnmsub231pd"

	// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Vgatherpf0hintdps_mvt_k1
	0x00,// No flags set
	0xF4, 0x0C,// 1652 = "vgatherpf0hintdps"

	// MVEX_Vgatherpf0hintdpd_mvt_k1
	0x00,// No flags set
	0xF5, 0x0C,// 1653 = "vgatherpf0hintdpd"

	// MVEX_Vgatherpf0dps_mvt_k1
	0x00,// No flags set
	0xDC, 0x08,// 1116 = "vgatherpf0dps"

	// MVEX_Vgatherpf1dps_mvt_k1
	0x00,// No flags set
	0xDE, 0x08,// 1118 = "vgatherpf1dps"

	// MVEX_Vscatterpf0hintdps_mvt_k1
	0x00,// No flags set
	0xF6, 0x0C,// 1654 = "vscatterpf0hintdps"

	// MVEX_Vscatterpf0hintdpd_mvt_k1
	0x00,// No flags set
	0xF7, 0x0C,// 1655 = "vscatterpf0hintdpd"

	// MVEX_Vscatterpf0dps_mvt_k1
	0x00,// No flags set
	0xE0, 0x08,// 1120 = "vscatterpf0dps"

	// MVEX_Vscatterpf1dps_mvt_k1
	0x00,// No flags set
	0xE2, 0x08,// 1122 = "vscatterpf1dps"

	// MVEX_Vexp223ps_zmm_k1_zmmmt
	0x00,// No flags set
	0xF8, 0x0C,// 1656 = "vexp223ps"

	// MVEX_Vlog2ps_zmm_k1_zmmmt
	0x00,// No flags set
	0xF9, 0x0C,// 1657 = "vlog2ps"

	// MVEX_Vrcp23ps_zmm_k1_zmmmt
	0x00,// No flags set
	0xFA, 0x0C,// 1658 = "vrcp23ps"

	// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	0x00,// No flags set
	0xFB, 0x0C,// 1659 = "vrsqrt23ps"

	// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xFC, 0x0C,// 1660 = "vaddsetsps"

	// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	0x00,// No flags set
	0xFD, 0x0C,// 1661 = "vpaddsetsd"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	0x02,// SameAsPrev

	// MVEX_Vloadunpackld_zmm_k1_mt
	0x00,// No flags set
	0xFE, 0x0C,// 1662 = "vloadunpackld"

	// MVEX_Vloadunpacklq_zmm_k1_mt
	0x00,// No flags set
	0xFF, 0x0C,// 1663 = "vloadunpacklq"

	// MVEX_Vpackstoreld_mt_k1_zmm
	0x00,// No flags set
	0x80, 0x0D,// 1664 = "vpackstoreld"

	// MVEX_Vpackstorelq_mt_k1_zmm
	0x00,// No flags set
	0x81, 0x0D,// 1665 = "vpackstorelq"

	// MVEX_Vloadunpacklps_zmm_k1_mt
	0x00,// No flags set
	0x82, 0x0D,// 1666 = "vloadunpacklps"

	// MVEX_Vloadunpacklpd_zmm_k1_mt
	0x00,// No flags set
	0x83, 0x0D,// 1667 = "vloadunpacklpd"

	// MVEX_Vpackstorelps_mt_k1_zmm
	0x00,// No flags set
	0x84, 0x0D,// 1668 = "vpackstorelps"

	// MVEX_Vpackstorelpd_mt_k1_zmm
	0x00,// No flags set
	0x85, 0x0D,// 1669 = "vpackstorelpd"

	// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	0x02,// SameAsPrev

	// MVEX_Vloadunpackhd_zmm_k1_mt
	0x00,// No flags set
	0x86, 0x0D,// 1670 = "vloadunpackhd"

	// MVEX_Vloadunpackhq_zmm_k1_mt
	0x00,// No flags set
	0x87, 0x0D,// 1671 = "vloadunpackhq"

	// MVEX_Vpackstorehd_mt_k1_zmm
	0x00,// No flags set
	0x88, 0x0D,// 1672 = "vpackstorehd"

	// MVEX_Vpackstorehq_mt_k1_zmm
	0x00,// No flags set
	0x89, 0x0D,// 1673 = "vpackstorehq"

	// MVEX_Vloadunpackhps_zmm_k1_mt
	0x00,// No flags set
	0x8A, 0x0D,// 1674 = "vloadunpackhps"

	// MVEX_Vloadunpackhpd_zmm_k1_mt
	0x00,// No flags set
	0x8B, 0x0D,// 1675 = "vloadunpackhpd"

	// MVEX_Vpackstorehps_mt_k1_zmm
	0x00,// No flags set
	0x8C, 0x0D,// 1676 = "vpackstorehps"

	// MVEX_Vpackstorehpd_mt_k1_zmm
	0x00,// No flags set
	0x8D, 0x0D,// 1677 = "vpackstorehpd"

	// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	0x02,// SameAsPrev

	// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	0x02,// SameAsPrev

	// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	0x00,// No flags set
	0x9B, 0x09,// 1179 = "valignd"

	// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x8E, 0x0D,// 1678 = "vpermf32x4"

	// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	0xF8,// vpcmpd6
	0xB8, 0x09,// 1208 = "vpcmpud"

	// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	0xF8,// vpcmpd6
	0xBA, 0x09,// 1210 = "vpcmpd"

	// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0xC4, 0x09,// 1220 = "vgetmantps"

	// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0xC5, 0x09,// 1221 = "vgetmantpd"

	// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x8F, 0x0D,// 1679 = "vrndfxpntps"

	// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x90, 0x0D,// 1680 = "vrndfxpntpd"

	// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x91, 0x0D,// 1681 = "vcvtfxpntudq2ps"

	// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x92, 0x0D,// 1682 = "vcvtfxpntps2udq"

	// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x93, 0x0D,// 1683 = "vcvtfxpntpd2udq"

	// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x94, 0x0D,// 1684 = "vcvtfxpntdq2ps"

	// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x95, 0x0D,// 1685 = "vcvtfxpntps2dq"

	// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	0x02,// SameAsPrev

	// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	0x00,// No flags set
	0x96, 0x0D,// 1686 = "vcvtfxpntpd2dq"

	// Via_undoc_F30FA6F0_16
	0x00,// No flags set
	0x00,// 0 = "undoc"

	// Via_undoc_F30FA6F0_32
	0x02,// SameAsPrev

	// Via_undoc_F30FA6F0_64
	0x02,// SameAsPrev

	// Via_undoc_F30FA6F8_16
	0x02,// SameAsPrev

	// Via_undoc_F30FA6F8_32
	0x02,// SameAsPrev

	// Via_undoc_F30FA6F8_64
	0x02,// SameAsPrev

	// Xsha512_16
	0x00,// No flags set
	0x97, 0x0D,// 1687 = "xsha512"

	// Xsha512_32
	0x02,// SameAsPrev

	// Xsha512_64
	0x02,// SameAsPrev

	// Xstore_alt_16
	0x00,// No flags set
	0x98, 0x0D,// 1688 = "xstore_alt"

	// Xstore_alt_32
	0x02,// SameAsPrev

	// Xstore_alt_64
	0x02,// SameAsPrev

	// Xsha512_alt_16
	0x00,// No flags set
	0x99, 0x0D,// 1689 = "xsha512_alt"

	// Xsha512_alt_32
	0x02,// SameAsPrev

	// Xsha512_alt_64
	0x02,// SameAsPrev

	// Zero_bytes
	0x00,// No flags set
	0x9A, 0x0D,// 1690 = "zero_bytes"

	// Wrmsrns
	0x00,// No flags set
	0x9B, 0x0D,// 1691 = "wrmsrns"

	// Wrmsrlist
	0x00,// No flags set
	0x9C, 0x0D,// 1692 = "wrmsrlist"

	// Rdmsrlist
	0x00,// No flags set
	0x9D, 0x0D,// 1693 = "rdmsrlist"

	// Rmpquery
	0x00,// No flags set
	0x9E, 0x0D,// 1694 = "rmpquery"

	// Prefetchit1_m8
	0x00,// No flags set
	0x9F, 0x0D,// 1695 = "prefetchit1"

	// Prefetchit0_m8
	0x00,// No flags set
	0xA0, 0x0D,// 1696 = "prefetchit0"

	// Aadd_m32_r32
	0x00,// No flags set
	0xA1, 0x0D,// 1697 = "aadd"

	// Aadd_m64_r64
	0x02,// SameAsPrev

	// Aand_m32_r32
	0x00,// No flags set
	0xA2, 0x0D,// 1698 = "aand"

	// Aand_m64_r64
	0x02,// SameAsPrev

	// Axor_m32_r32
	0x00,// No flags set
	0xA3, 0x0D,// 1699 = "axor"

	// Axor_m64_r64
	0x02,// SameAsPrev

	// Aor_m32_r32
	0x00,// No flags set
	0xA4, 0x0D,// 1700 = "aor"

	// Aor_m64_r64
	0x02,// SameAsPrev

	// VEX_Vpdpbuud_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA5, 0x0D,// 1701 = "vpdpbuud"

	// VEX_Vpdpbuud_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbsud_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA6, 0x0D,// 1702 = "vpdpbsud"

	// VEX_Vpdpbsud_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbssd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA7, 0x0D,// 1703 = "vpdpbssd"

	// VEX_Vpdpbssd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA8, 0x0D,// 1704 = "vpdpbuuds"

	// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xA9, 0x0D,// 1705 = "vpdpbsuds"

	// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpbssds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xAA, 0x0D,// 1706 = "vpdpbssds"

	// VEX_Vpdpbssds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Tdpfp16ps_tmm_tmm_tmm
	0x00,// No flags set
	0xAB, 0x0D,// 1707 = "tdpfp16ps"

	// VEX_Vcvtneps2bf16_xmm_xmmm128
	0x04,// ForceMemSize
	0xE7, 0x07,// 999 = "vcvtneps2bf16"

	// VEX_Vcvtneps2bf16_xmm_ymmm256
	0x06,// SameAsPrev, ForceMemSize

	// VEX_Vcvtneoph2ps_xmm_m128
	0x00,// No flags set
	0xAC, 0x0D,// 1708 = "vcvtneoph2ps"

	// VEX_Vcvtneoph2ps_ymm_m256
	0x02,// SameAsPrev

	// VEX_Vcvtneeph2ps_xmm_m128
	0x00,// No flags set
	0xAD, 0x0D,// 1709 = "vcvtneeph2ps"

	// VEX_Vcvtneeph2ps_ymm_m256
	0x02,// SameAsPrev

	// VEX_Vcvtneebf162ps_xmm_m128
	0x00,// No flags set
	0xAE, 0x0D,// 1710 = "vcvtneebf162ps"

	// VEX_Vcvtneebf162ps_ymm_m256
	0x02,// SameAsPrev

	// VEX_Vcvtneobf162ps_xmm_m128
	0x00,// No flags set
	0xAF, 0x0D,// 1711 = "vcvtneobf162ps"

	// VEX_Vcvtneobf162ps_ymm_m256
	0x02,// SameAsPrev

	// VEX_Vbcstnesh2ps_xmm_m16
	0x00,// No flags set
	0xB0, 0x0D,// 1712 = "vbcstnesh2ps"

	// VEX_Vbcstnesh2ps_ymm_m16
	0x02,// SameAsPrev

	// VEX_Vbcstnebf162ps_xmm_m16
	0x00,// No flags set
	0xB1, 0x0D,// 1713 = "vbcstnebf162ps"

	// VEX_Vbcstnebf162ps_ymm_m16
	0x02,// SameAsPrev

	// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC4, 0x08,// 1092 = "vpmadd52luq"

	// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	0x00,// No flags set
	0xC5, 0x08,// 1093 = "vpmadd52huq"

	// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Cmpoxadd_m32_r32_r32
	0x00,// No flags set
	0xB2, 0x0D,// 1714 = "cmpoxadd"

	// VEX_Cmpoxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnoxadd_m32_r32_r32
	0x00,// No flags set
	0xB3, 0x0D,// 1715 = "cmpnoxadd"

	// VEX_Cmpnoxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpbxadd_m32_r32_r32
	0x00,// No flags set
	0xB4, 0x0D,// 1716 = "cmpbxadd"

	// VEX_Cmpbxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnbxadd_m32_r32_r32
	0x00,// No flags set
	0xB7, 0x0D,// 1719 = "cmpaexadd"

	// VEX_Cmpnbxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpzxadd_m32_r32_r32
	0x00,// No flags set
	0xBA, 0x0D,// 1722 = "cmpexadd"

	// VEX_Cmpzxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnzxadd_m32_r32_r32
	0x00,// No flags set
	0xBC, 0x0D,// 1724 = "cmpnexadd"

	// VEX_Cmpnzxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpbexadd_m32_r32_r32
	0x00,// No flags set
	0xBE, 0x0D,// 1726 = "cmpbexadd"

	// VEX_Cmpbexadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnbexadd_m32_r32_r32
	0x00,// No flags set
	0xC0, 0x0D,// 1728 = "cmpaxadd"

	// VEX_Cmpnbexadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpsxadd_m32_r32_r32
	0x00,// No flags set
	0xC2, 0x0D,// 1730 = "cmpsxadd"

	// VEX_Cmpsxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnsxadd_m32_r32_r32
	0x00,// No flags set
	0xC3, 0x0D,// 1731 = "cmpnsxadd"

	// VEX_Cmpnsxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmppxadd_m32_r32_r32
	0x00,// No flags set
	0xC5, 0x0D,// 1733 = "cmppxadd"

	// VEX_Cmppxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnpxadd_m32_r32_r32
	0x00,// No flags set
	0xC6, 0x0D,// 1734 = "cmpnpxadd"

	// VEX_Cmpnpxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmplxadd_m32_r32_r32
	0x00,// No flags set
	0xC8, 0x0D,// 1736 = "cmplxadd"

	// VEX_Cmplxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnlxadd_m32_r32_r32
	0x00,// No flags set
	0xCA, 0x0D,// 1738 = "cmpgexadd"

	// VEX_Cmpnlxadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmplexadd_m32_r32_r32
	0x00,// No flags set
	0xCC, 0x0D,// 1740 = "cmplexadd"

	// VEX_Cmplexadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Cmpnlexadd_m32_r32_r32
	0x00,// No flags set
	0xCE, 0x0D,// 1742 = "cmpgxadd"

	// VEX_Cmpnlexadd_m64_r64_r64
	0x02,// SameAsPrev

	// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	0x00,// No flags set
	0xD0, 0x0D,// 1744 = "tcmmrlfp16ps"

	// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	0x00,// No flags set
	0xD1, 0x0D,// 1745 = "tcmmimfp16ps"

	// Pbndkb
	0x00,// No flags set
	0xD2, 0x0D,// 1746 = "pbndkb"

	// VEX_Vsha512rnds2_ymm_ymm_xmm
	0x00,// No flags set
	0xD3, 0x0D,// 1747 = "vsha512rnds2"

	// VEX_Vsha512msg1_ymm_xmm
	0x00,// No flags set
	0xD4, 0x0D,// 1748 = "vsha512msg1"

	// VEX_Vsha512msg2_ymm_ymm
	0x00,// No flags set
	0xD5, 0x0D,// 1749 = "vsha512msg2"

	// VEX_Vpdpwuud_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD6, 0x0D,// 1750 = "vpdpwuud"

	// VEX_Vpdpwuud_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwusd_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD7, 0x0D,// 1751 = "vpdpwusd"

	// VEX_Vpdpwusd_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwsud_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD8, 0x0D,// 1752 = "vpdpwsud"

	// VEX_Vpdpwsud_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xD9, 0x0D,// 1753 = "vpdpwuuds"

	// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwusds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDA, 0x0D,// 1754 = "vpdpwusds"

	// VEX_Vpdpwusds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDB, 0x0D,// 1755 = "vpdpwsuds"

	// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vsm3msg1_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDC, 0x0D,// 1756 = "vsm3msg1"

	// VEX_Vsm3msg2_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDD, 0x0D,// 1757 = "vsm3msg2"

	// VEX_Vsm4key4_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDE, 0x0D,// 1758 = "vsm4key4"

	// VEX_Vsm4key4_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	0x00,// No flags set
	0xDF, 0x0D,// 1759 = "vsm4rnds4"

	// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	0x02,// SameAsPrev

	// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
	0x00,// No flags set
	0xE0, 0x0D,// 1760 = "vsm3rnds2"
];
