// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::iced_constants::IcedConstants;
use crate::Mnemonic;

#[rustfmt::skip]
pub(super) static TO_MNEMONIC: [Mnemonic; IcedConstants::CODE_ENUM_COUNT] = [
	Mnemonic::INVALID,// INVALID
	Mnemonic::Db,// DeclareByte
	Mnemonic::Dw,// DeclareWord
	Mnemonic::Dd,// DeclareDword
	Mnemonic::Dq,// DeclareQword
	Mnemonic::Add,// Add_rm8_r8
	Mnemonic::Add,// Add_rm16_r16
	Mnemonic::Add,// Add_rm32_r32
	Mnemonic::Add,// Add_rm64_r64
	Mnemonic::Add,// Add_r8_rm8
	Mnemonic::Add,// Add_r16_rm16
	Mnemonic::Add,// Add_r32_rm32
	Mnemonic::Add,// Add_r64_rm64
	Mnemonic::Add,// Add_AL_imm8
	Mnemonic::Add,// Add_AX_imm16
	Mnemonic::Add,// Add_EAX_imm32
	Mnemonic::Add,// Add_RAX_imm32
	Mnemonic::Push,// Pushw_ES
	Mnemonic::Push,// Pushd_ES
	Mnemonic::Pop,// Popw_ES
	Mnemonic::Pop,// Popd_ES
	Mnemonic::Or,// Or_rm8_r8
	Mnemonic::Or,// Or_rm16_r16
	Mnemonic::Or,// Or_rm32_r32
	Mnemonic::Or,// Or_rm64_r64
	Mnemonic::Or,// Or_r8_rm8
	Mnemonic::Or,// Or_r16_rm16
	Mnemonic::Or,// Or_r32_rm32
	Mnemonic::Or,// Or_r64_rm64
	Mnemonic::Or,// Or_AL_imm8
	Mnemonic::Or,// Or_AX_imm16
	Mnemonic::Or,// Or_EAX_imm32
	Mnemonic::Or,// Or_RAX_imm32
	Mnemonic::Push,// Pushw_CS
	Mnemonic::Push,// Pushd_CS
	Mnemonic::Pop,// Popw_CS
	Mnemonic::Adc,// Adc_rm8_r8
	Mnemonic::Adc,// Adc_rm16_r16
	Mnemonic::Adc,// Adc_rm32_r32
	Mnemonic::Adc,// Adc_rm64_r64
	Mnemonic::Adc,// Adc_r8_rm8
	Mnemonic::Adc,// Adc_r16_rm16
	Mnemonic::Adc,// Adc_r32_rm32
	Mnemonic::Adc,// Adc_r64_rm64
	Mnemonic::Adc,// Adc_AL_imm8
	Mnemonic::Adc,// Adc_AX_imm16
	Mnemonic::Adc,// Adc_EAX_imm32
	Mnemonic::Adc,// Adc_RAX_imm32
	Mnemonic::Push,// Pushw_SS
	Mnemonic::Push,// Pushd_SS
	Mnemonic::Pop,// Popw_SS
	Mnemonic::Pop,// Popd_SS
	Mnemonic::Sbb,// Sbb_rm8_r8
	Mnemonic::Sbb,// Sbb_rm16_r16
	Mnemonic::Sbb,// Sbb_rm32_r32
	Mnemonic::Sbb,// Sbb_rm64_r64
	Mnemonic::Sbb,// Sbb_r8_rm8
	Mnemonic::Sbb,// Sbb_r16_rm16
	Mnemonic::Sbb,// Sbb_r32_rm32
	Mnemonic::Sbb,// Sbb_r64_rm64
	Mnemonic::Sbb,// Sbb_AL_imm8
	Mnemonic::Sbb,// Sbb_AX_imm16
	Mnemonic::Sbb,// Sbb_EAX_imm32
	Mnemonic::Sbb,// Sbb_RAX_imm32
	Mnemonic::Push,// Pushw_DS
	Mnemonic::Push,// Pushd_DS
	Mnemonic::Pop,// Popw_DS
	Mnemonic::Pop,// Popd_DS
	Mnemonic::And,// And_rm8_r8
	Mnemonic::And,// And_rm16_r16
	Mnemonic::And,// And_rm32_r32
	Mnemonic::And,// And_rm64_r64
	Mnemonic::And,// And_r8_rm8
	Mnemonic::And,// And_r16_rm16
	Mnemonic::And,// And_r32_rm32
	Mnemonic::And,// And_r64_rm64
	Mnemonic::And,// And_AL_imm8
	Mnemonic::And,// And_AX_imm16
	Mnemonic::And,// And_EAX_imm32
	Mnemonic::And,// And_RAX_imm32
	Mnemonic::Daa,// Daa
	Mnemonic::Sub,// Sub_rm8_r8
	Mnemonic::Sub,// Sub_rm16_r16
	Mnemonic::Sub,// Sub_rm32_r32
	Mnemonic::Sub,// Sub_rm64_r64
	Mnemonic::Sub,// Sub_r8_rm8
	Mnemonic::Sub,// Sub_r16_rm16
	Mnemonic::Sub,// Sub_r32_rm32
	Mnemonic::Sub,// Sub_r64_rm64
	Mnemonic::Sub,// Sub_AL_imm8
	Mnemonic::Sub,// Sub_AX_imm16
	Mnemonic::Sub,// Sub_EAX_imm32
	Mnemonic::Sub,// Sub_RAX_imm32
	Mnemonic::Das,// Das
	Mnemonic::Xor,// Xor_rm8_r8
	Mnemonic::Xor,// Xor_rm16_r16
	Mnemonic::Xor,// Xor_rm32_r32
	Mnemonic::Xor,// Xor_rm64_r64
	Mnemonic::Xor,// Xor_r8_rm8
	Mnemonic::Xor,// Xor_r16_rm16
	Mnemonic::Xor,// Xor_r32_rm32
	Mnemonic::Xor,// Xor_r64_rm64
	Mnemonic::Xor,// Xor_AL_imm8
	Mnemonic::Xor,// Xor_AX_imm16
	Mnemonic::Xor,// Xor_EAX_imm32
	Mnemonic::Xor,// Xor_RAX_imm32
	Mnemonic::Aaa,// Aaa
	Mnemonic::Cmp,// Cmp_rm8_r8
	Mnemonic::Cmp,// Cmp_rm16_r16
	Mnemonic::Cmp,// Cmp_rm32_r32
	Mnemonic::Cmp,// Cmp_rm64_r64
	Mnemonic::Cmp,// Cmp_r8_rm8
	Mnemonic::Cmp,// Cmp_r16_rm16
	Mnemonic::Cmp,// Cmp_r32_rm32
	Mnemonic::Cmp,// Cmp_r64_rm64
	Mnemonic::Cmp,// Cmp_AL_imm8
	Mnemonic::Cmp,// Cmp_AX_imm16
	Mnemonic::Cmp,// Cmp_EAX_imm32
	Mnemonic::Cmp,// Cmp_RAX_imm32
	Mnemonic::Aas,// Aas
	Mnemonic::Inc,// Inc_r16
	Mnemonic::Inc,// Inc_r32
	Mnemonic::Dec,// Dec_r16
	Mnemonic::Dec,// Dec_r32
	Mnemonic::Push,// Push_r16
	Mnemonic::Push,// Push_r32
	Mnemonic::Push,// Push_r64
	Mnemonic::Pop,// Pop_r16
	Mnemonic::Pop,// Pop_r32
	Mnemonic::Pop,// Pop_r64
	Mnemonic::Pusha,// Pushaw
	Mnemonic::Pushad,// Pushad
	Mnemonic::Popa,// Popaw
	Mnemonic::Popad,// Popad
	Mnemonic::Bound,// Bound_r16_m1616
	Mnemonic::Bound,// Bound_r32_m3232
	Mnemonic::Arpl,// Arpl_rm16_r16
	Mnemonic::Arpl,// Arpl_r32m16_r32
	Mnemonic::Movsxd,// Movsxd_r16_rm16
	Mnemonic::Movsxd,// Movsxd_r32_rm32
	Mnemonic::Movsxd,// Movsxd_r64_rm32
	Mnemonic::Push,// Push_imm16
	Mnemonic::Push,// Pushd_imm32
	Mnemonic::Push,// Pushq_imm32
	Mnemonic::Imul,// Imul_r16_rm16_imm16
	Mnemonic::Imul,// Imul_r32_rm32_imm32
	Mnemonic::Imul,// Imul_r64_rm64_imm32
	Mnemonic::Push,// Pushw_imm8
	Mnemonic::Push,// Pushd_imm8
	Mnemonic::Push,// Pushq_imm8
	Mnemonic::Imul,// Imul_r16_rm16_imm8
	Mnemonic::Imul,// Imul_r32_rm32_imm8
	Mnemonic::Imul,// Imul_r64_rm64_imm8
	Mnemonic::Insb,// Insb_m8_DX
	Mnemonic::Insw,// Insw_m16_DX
	Mnemonic::Insd,// Insd_m32_DX
	Mnemonic::Outsb,// Outsb_DX_m8
	Mnemonic::Outsw,// Outsw_DX_m16
	Mnemonic::Outsd,// Outsd_DX_m32
	Mnemonic::Jo,// Jo_rel8_16
	Mnemonic::Jo,// Jo_rel8_32
	Mnemonic::Jo,// Jo_rel8_64
	Mnemonic::Jno,// Jno_rel8_16
	Mnemonic::Jno,// Jno_rel8_32
	Mnemonic::Jno,// Jno_rel8_64
	Mnemonic::Jb,// Jb_rel8_16
	Mnemonic::Jb,// Jb_rel8_32
	Mnemonic::Jb,// Jb_rel8_64
	Mnemonic::Jae,// Jae_rel8_16
	Mnemonic::Jae,// Jae_rel8_32
	Mnemonic::Jae,// Jae_rel8_64
	Mnemonic::Je,// Je_rel8_16
	Mnemonic::Je,// Je_rel8_32
	Mnemonic::Je,// Je_rel8_64
	Mnemonic::Jne,// Jne_rel8_16
	Mnemonic::Jne,// Jne_rel8_32
	Mnemonic::Jne,// Jne_rel8_64
	Mnemonic::Jbe,// Jbe_rel8_16
	Mnemonic::Jbe,// Jbe_rel8_32
	Mnemonic::Jbe,// Jbe_rel8_64
	Mnemonic::Ja,// Ja_rel8_16
	Mnemonic::Ja,// Ja_rel8_32
	Mnemonic::Ja,// Ja_rel8_64
	Mnemonic::Js,// Js_rel8_16
	Mnemonic::Js,// Js_rel8_32
	Mnemonic::Js,// Js_rel8_64
	Mnemonic::Jns,// Jns_rel8_16
	Mnemonic::Jns,// Jns_rel8_32
	Mnemonic::Jns,// Jns_rel8_64
	Mnemonic::Jp,// Jp_rel8_16
	Mnemonic::Jp,// Jp_rel8_32
	Mnemonic::Jp,// Jp_rel8_64
	Mnemonic::Jnp,// Jnp_rel8_16
	Mnemonic::Jnp,// Jnp_rel8_32
	Mnemonic::Jnp,// Jnp_rel8_64
	Mnemonic::Jl,// Jl_rel8_16
	Mnemonic::Jl,// Jl_rel8_32
	Mnemonic::Jl,// Jl_rel8_64
	Mnemonic::Jge,// Jge_rel8_16
	Mnemonic::Jge,// Jge_rel8_32
	Mnemonic::Jge,// Jge_rel8_64
	Mnemonic::Jle,// Jle_rel8_16
	Mnemonic::Jle,// Jle_rel8_32
	Mnemonic::Jle,// Jle_rel8_64
	Mnemonic::Jg,// Jg_rel8_16
	Mnemonic::Jg,// Jg_rel8_32
	Mnemonic::Jg,// Jg_rel8_64
	Mnemonic::Add,// Add_rm8_imm8
	Mnemonic::Or,// Or_rm8_imm8
	Mnemonic::Adc,// Adc_rm8_imm8
	Mnemonic::Sbb,// Sbb_rm8_imm8
	Mnemonic::And,// And_rm8_imm8
	Mnemonic::Sub,// Sub_rm8_imm8
	Mnemonic::Xor,// Xor_rm8_imm8
	Mnemonic::Cmp,// Cmp_rm8_imm8
	Mnemonic::Add,// Add_rm16_imm16
	Mnemonic::Add,// Add_rm32_imm32
	Mnemonic::Add,// Add_rm64_imm32
	Mnemonic::Or,// Or_rm16_imm16
	Mnemonic::Or,// Or_rm32_imm32
	Mnemonic::Or,// Or_rm64_imm32
	Mnemonic::Adc,// Adc_rm16_imm16
	Mnemonic::Adc,// Adc_rm32_imm32
	Mnemonic::Adc,// Adc_rm64_imm32
	Mnemonic::Sbb,// Sbb_rm16_imm16
	Mnemonic::Sbb,// Sbb_rm32_imm32
	Mnemonic::Sbb,// Sbb_rm64_imm32
	Mnemonic::And,// And_rm16_imm16
	Mnemonic::And,// And_rm32_imm32
	Mnemonic::And,// And_rm64_imm32
	Mnemonic::Sub,// Sub_rm16_imm16
	Mnemonic::Sub,// Sub_rm32_imm32
	Mnemonic::Sub,// Sub_rm64_imm32
	Mnemonic::Xor,// Xor_rm16_imm16
	Mnemonic::Xor,// Xor_rm32_imm32
	Mnemonic::Xor,// Xor_rm64_imm32
	Mnemonic::Cmp,// Cmp_rm16_imm16
	Mnemonic::Cmp,// Cmp_rm32_imm32
	Mnemonic::Cmp,// Cmp_rm64_imm32
	Mnemonic::Add,// Add_rm8_imm8_82
	Mnemonic::Or,// Or_rm8_imm8_82
	Mnemonic::Adc,// Adc_rm8_imm8_82
	Mnemonic::Sbb,// Sbb_rm8_imm8_82
	Mnemonic::And,// And_rm8_imm8_82
	Mnemonic::Sub,// Sub_rm8_imm8_82
	Mnemonic::Xor,// Xor_rm8_imm8_82
	Mnemonic::Cmp,// Cmp_rm8_imm8_82
	Mnemonic::Add,// Add_rm16_imm8
	Mnemonic::Add,// Add_rm32_imm8
	Mnemonic::Add,// Add_rm64_imm8
	Mnemonic::Or,// Or_rm16_imm8
	Mnemonic::Or,// Or_rm32_imm8
	Mnemonic::Or,// Or_rm64_imm8
	Mnemonic::Adc,// Adc_rm16_imm8
	Mnemonic::Adc,// Adc_rm32_imm8
	Mnemonic::Adc,// Adc_rm64_imm8
	Mnemonic::Sbb,// Sbb_rm16_imm8
	Mnemonic::Sbb,// Sbb_rm32_imm8
	Mnemonic::Sbb,// Sbb_rm64_imm8
	Mnemonic::And,// And_rm16_imm8
	Mnemonic::And,// And_rm32_imm8
	Mnemonic::And,// And_rm64_imm8
	Mnemonic::Sub,// Sub_rm16_imm8
	Mnemonic::Sub,// Sub_rm32_imm8
	Mnemonic::Sub,// Sub_rm64_imm8
	Mnemonic::Xor,// Xor_rm16_imm8
	Mnemonic::Xor,// Xor_rm32_imm8
	Mnemonic::Xor,// Xor_rm64_imm8
	Mnemonic::Cmp,// Cmp_rm16_imm8
	Mnemonic::Cmp,// Cmp_rm32_imm8
	Mnemonic::Cmp,// Cmp_rm64_imm8
	Mnemonic::Test,// Test_rm8_r8
	Mnemonic::Test,// Test_rm16_r16
	Mnemonic::Test,// Test_rm32_r32
	Mnemonic::Test,// Test_rm64_r64
	Mnemonic::Xchg,// Xchg_rm8_r8
	Mnemonic::Xchg,// Xchg_rm16_r16
	Mnemonic::Xchg,// Xchg_rm32_r32
	Mnemonic::Xchg,// Xchg_rm64_r64
	Mnemonic::Mov,// Mov_rm8_r8
	Mnemonic::Mov,// Mov_rm16_r16
	Mnemonic::Mov,// Mov_rm32_r32
	Mnemonic::Mov,// Mov_rm64_r64
	Mnemonic::Mov,// Mov_r8_rm8
	Mnemonic::Mov,// Mov_r16_rm16
	Mnemonic::Mov,// Mov_r32_rm32
	Mnemonic::Mov,// Mov_r64_rm64
	Mnemonic::Mov,// Mov_rm16_Sreg
	Mnemonic::Mov,// Mov_r32m16_Sreg
	Mnemonic::Mov,// Mov_r64m16_Sreg
	Mnemonic::Lea,// Lea_r16_m
	Mnemonic::Lea,// Lea_r32_m
	Mnemonic::Lea,// Lea_r64_m
	Mnemonic::Mov,// Mov_Sreg_rm16
	Mnemonic::Mov,// Mov_Sreg_r32m16
	Mnemonic::Mov,// Mov_Sreg_r64m16
	Mnemonic::Pop,// Pop_rm16
	Mnemonic::Pop,// Pop_rm32
	Mnemonic::Pop,// Pop_rm64
	Mnemonic::Nop,// Nopw
	Mnemonic::Nop,// Nopd
	Mnemonic::Nop,// Nopq
	Mnemonic::Xchg,// Xchg_r16_AX
	Mnemonic::Xchg,// Xchg_r32_EAX
	Mnemonic::Xchg,// Xchg_r64_RAX
	Mnemonic::Pause,// Pause
	Mnemonic::Cbw,// Cbw
	Mnemonic::Cwde,// Cwde
	Mnemonic::Cdqe,// Cdqe
	Mnemonic::Cwd,// Cwd
	Mnemonic::Cdq,// Cdq
	Mnemonic::Cqo,// Cqo
	Mnemonic::Call,// Call_ptr1616
	Mnemonic::Call,// Call_ptr1632
	Mnemonic::Wait,// Wait
	Mnemonic::Pushf,// Pushfw
	Mnemonic::Pushfd,// Pushfd
	Mnemonic::Pushfq,// Pushfq
	Mnemonic::Popf,// Popfw
	Mnemonic::Popfd,// Popfd
	Mnemonic::Popfq,// Popfq
	Mnemonic::Sahf,// Sahf
	Mnemonic::Lahf,// Lahf
	Mnemonic::Mov,// Mov_AL_moffs8
	Mnemonic::Mov,// Mov_AX_moffs16
	Mnemonic::Mov,// Mov_EAX_moffs32
	Mnemonic::Mov,// Mov_RAX_moffs64
	Mnemonic::Mov,// Mov_moffs8_AL
	Mnemonic::Mov,// Mov_moffs16_AX
	Mnemonic::Mov,// Mov_moffs32_EAX
	Mnemonic::Mov,// Mov_moffs64_RAX
	Mnemonic::Movsb,// Movsb_m8_m8
	Mnemonic::Movsw,// Movsw_m16_m16
	Mnemonic::Movsd,// Movsd_m32_m32
	Mnemonic::Movsq,// Movsq_m64_m64
	Mnemonic::Cmpsb,// Cmpsb_m8_m8
	Mnemonic::Cmpsw,// Cmpsw_m16_m16
	Mnemonic::Cmpsd,// Cmpsd_m32_m32
	Mnemonic::Cmpsq,// Cmpsq_m64_m64
	Mnemonic::Test,// Test_AL_imm8
	Mnemonic::Test,// Test_AX_imm16
	Mnemonic::Test,// Test_EAX_imm32
	Mnemonic::Test,// Test_RAX_imm32
	Mnemonic::Stosb,// Stosb_m8_AL
	Mnemonic::Stosw,// Stosw_m16_AX
	Mnemonic::Stosd,// Stosd_m32_EAX
	Mnemonic::Stosq,// Stosq_m64_RAX
	Mnemonic::Lodsb,// Lodsb_AL_m8
	Mnemonic::Lodsw,// Lodsw_AX_m16
	Mnemonic::Lodsd,// Lodsd_EAX_m32
	Mnemonic::Lodsq,// Lodsq_RAX_m64
	Mnemonic::Scasb,// Scasb_AL_m8
	Mnemonic::Scasw,// Scasw_AX_m16
	Mnemonic::Scasd,// Scasd_EAX_m32
	Mnemonic::Scasq,// Scasq_RAX_m64
	Mnemonic::Mov,// Mov_r8_imm8
	Mnemonic::Mov,// Mov_r16_imm16
	Mnemonic::Mov,// Mov_r32_imm32
	Mnemonic::Mov,// Mov_r64_imm64
	Mnemonic::Rol,// Rol_rm8_imm8
	Mnemonic::Ror,// Ror_rm8_imm8
	Mnemonic::Rcl,// Rcl_rm8_imm8
	Mnemonic::Rcr,// Rcr_rm8_imm8
	Mnemonic::Shl,// Shl_rm8_imm8
	Mnemonic::Shr,// Shr_rm8_imm8
	Mnemonic::Sal,// Sal_rm8_imm8
	Mnemonic::Sar,// Sar_rm8_imm8
	Mnemonic::Rol,// Rol_rm16_imm8
	Mnemonic::Rol,// Rol_rm32_imm8
	Mnemonic::Rol,// Rol_rm64_imm8
	Mnemonic::Ror,// Ror_rm16_imm8
	Mnemonic::Ror,// Ror_rm32_imm8
	Mnemonic::Ror,// Ror_rm64_imm8
	Mnemonic::Rcl,// Rcl_rm16_imm8
	Mnemonic::Rcl,// Rcl_rm32_imm8
	Mnemonic::Rcl,// Rcl_rm64_imm8
	Mnemonic::Rcr,// Rcr_rm16_imm8
	Mnemonic::Rcr,// Rcr_rm32_imm8
	Mnemonic::Rcr,// Rcr_rm64_imm8
	Mnemonic::Shl,// Shl_rm16_imm8
	Mnemonic::Shl,// Shl_rm32_imm8
	Mnemonic::Shl,// Shl_rm64_imm8
	Mnemonic::Shr,// Shr_rm16_imm8
	Mnemonic::Shr,// Shr_rm32_imm8
	Mnemonic::Shr,// Shr_rm64_imm8
	Mnemonic::Sal,// Sal_rm16_imm8
	Mnemonic::Sal,// Sal_rm32_imm8
	Mnemonic::Sal,// Sal_rm64_imm8
	Mnemonic::Sar,// Sar_rm16_imm8
	Mnemonic::Sar,// Sar_rm32_imm8
	Mnemonic::Sar,// Sar_rm64_imm8
	Mnemonic::Ret,// Retnw_imm16
	Mnemonic::Ret,// Retnd_imm16
	Mnemonic::Ret,// Retnq_imm16
	Mnemonic::Ret,// Retnw
	Mnemonic::Ret,// Retnd
	Mnemonic::Ret,// Retnq
	Mnemonic::Les,// Les_r16_m1616
	Mnemonic::Les,// Les_r32_m1632
	Mnemonic::Lds,// Lds_r16_m1616
	Mnemonic::Lds,// Lds_r32_m1632
	Mnemonic::Mov,// Mov_rm8_imm8
	Mnemonic::Xabort,// Xabort_imm8
	Mnemonic::Mov,// Mov_rm16_imm16
	Mnemonic::Mov,// Mov_rm32_imm32
	Mnemonic::Mov,// Mov_rm64_imm32
	Mnemonic::Xbegin,// Xbegin_rel16
	Mnemonic::Xbegin,// Xbegin_rel32
	Mnemonic::Enter,// Enterw_imm16_imm8
	Mnemonic::Enter,// Enterd_imm16_imm8
	Mnemonic::Enter,// Enterq_imm16_imm8
	Mnemonic::Leave,// Leavew
	Mnemonic::Leave,// Leaved
	Mnemonic::Leave,// Leaveq
	Mnemonic::Retf,// Retfw_imm16
	Mnemonic::Retf,// Retfd_imm16
	Mnemonic::Retf,// Retfq_imm16
	Mnemonic::Retf,// Retfw
	Mnemonic::Retf,// Retfd
	Mnemonic::Retf,// Retfq
	Mnemonic::Int3,// Int3
	Mnemonic::Int,// Int_imm8
	Mnemonic::Into,// Into
	Mnemonic::Iret,// Iretw
	Mnemonic::Iretd,// Iretd
	Mnemonic::Iretq,// Iretq
	Mnemonic::Rol,// Rol_rm8_1
	Mnemonic::Ror,// Ror_rm8_1
	Mnemonic::Rcl,// Rcl_rm8_1
	Mnemonic::Rcr,// Rcr_rm8_1
	Mnemonic::Shl,// Shl_rm8_1
	Mnemonic::Shr,// Shr_rm8_1
	Mnemonic::Sal,// Sal_rm8_1
	Mnemonic::Sar,// Sar_rm8_1
	Mnemonic::Rol,// Rol_rm16_1
	Mnemonic::Rol,// Rol_rm32_1
	Mnemonic::Rol,// Rol_rm64_1
	Mnemonic::Ror,// Ror_rm16_1
	Mnemonic::Ror,// Ror_rm32_1
	Mnemonic::Ror,// Ror_rm64_1
	Mnemonic::Rcl,// Rcl_rm16_1
	Mnemonic::Rcl,// Rcl_rm32_1
	Mnemonic::Rcl,// Rcl_rm64_1
	Mnemonic::Rcr,// Rcr_rm16_1
	Mnemonic::Rcr,// Rcr_rm32_1
	Mnemonic::Rcr,// Rcr_rm64_1
	Mnemonic::Shl,// Shl_rm16_1
	Mnemonic::Shl,// Shl_rm32_1
	Mnemonic::Shl,// Shl_rm64_1
	Mnemonic::Shr,// Shr_rm16_1
	Mnemonic::Shr,// Shr_rm32_1
	Mnemonic::Shr,// Shr_rm64_1
	Mnemonic::Sal,// Sal_rm16_1
	Mnemonic::Sal,// Sal_rm32_1
	Mnemonic::Sal,// Sal_rm64_1
	Mnemonic::Sar,// Sar_rm16_1
	Mnemonic::Sar,// Sar_rm32_1
	Mnemonic::Sar,// Sar_rm64_1
	Mnemonic::Rol,// Rol_rm8_CL
	Mnemonic::Ror,// Ror_rm8_CL
	Mnemonic::Rcl,// Rcl_rm8_CL
	Mnemonic::Rcr,// Rcr_rm8_CL
	Mnemonic::Shl,// Shl_rm8_CL
	Mnemonic::Shr,// Shr_rm8_CL
	Mnemonic::Sal,// Sal_rm8_CL
	Mnemonic::Sar,// Sar_rm8_CL
	Mnemonic::Rol,// Rol_rm16_CL
	Mnemonic::Rol,// Rol_rm32_CL
	Mnemonic::Rol,// Rol_rm64_CL
	Mnemonic::Ror,// Ror_rm16_CL
	Mnemonic::Ror,// Ror_rm32_CL
	Mnemonic::Ror,// Ror_rm64_CL
	Mnemonic::Rcl,// Rcl_rm16_CL
	Mnemonic::Rcl,// Rcl_rm32_CL
	Mnemonic::Rcl,// Rcl_rm64_CL
	Mnemonic::Rcr,// Rcr_rm16_CL
	Mnemonic::Rcr,// Rcr_rm32_CL
	Mnemonic::Rcr,// Rcr_rm64_CL
	Mnemonic::Shl,// Shl_rm16_CL
	Mnemonic::Shl,// Shl_rm32_CL
	Mnemonic::Shl,// Shl_rm64_CL
	Mnemonic::Shr,// Shr_rm16_CL
	Mnemonic::Shr,// Shr_rm32_CL
	Mnemonic::Shr,// Shr_rm64_CL
	Mnemonic::Sal,// Sal_rm16_CL
	Mnemonic::Sal,// Sal_rm32_CL
	Mnemonic::Sal,// Sal_rm64_CL
	Mnemonic::Sar,// Sar_rm16_CL
	Mnemonic::Sar,// Sar_rm32_CL
	Mnemonic::Sar,// Sar_rm64_CL
	Mnemonic::Aam,// Aam_imm8
	Mnemonic::Aad,// Aad_imm8
	Mnemonic::Salc,// Salc
	Mnemonic::Xlatb,// Xlat_m8
	Mnemonic::Fadd,// Fadd_m32fp
	Mnemonic::Fmul,// Fmul_m32fp
	Mnemonic::Fcom,// Fcom_m32fp
	Mnemonic::Fcomp,// Fcomp_m32fp
	Mnemonic::Fsub,// Fsub_m32fp
	Mnemonic::Fsubr,// Fsubr_m32fp
	Mnemonic::Fdiv,// Fdiv_m32fp
	Mnemonic::Fdivr,// Fdivr_m32fp
	Mnemonic::Fadd,// Fadd_st0_sti
	Mnemonic::Fmul,// Fmul_st0_sti
	Mnemonic::Fcom,// Fcom_st0_sti
	Mnemonic::Fcomp,// Fcomp_st0_sti
	Mnemonic::Fsub,// Fsub_st0_sti
	Mnemonic::Fsubr,// Fsubr_st0_sti
	Mnemonic::Fdiv,// Fdiv_st0_sti
	Mnemonic::Fdivr,// Fdivr_st0_sti
	Mnemonic::Fld,// Fld_m32fp
	Mnemonic::Fst,// Fst_m32fp
	Mnemonic::Fstp,// Fstp_m32fp
	Mnemonic::Fldenv,// Fldenv_m14byte
	Mnemonic::Fldenv,// Fldenv_m28byte
	Mnemonic::Fldcw,// Fldcw_m2byte
	Mnemonic::Fnstenv,// Fnstenv_m14byte
	Mnemonic::Fstenv,// Fstenv_m14byte
	Mnemonic::Fnstenv,// Fnstenv_m28byte
	Mnemonic::Fstenv,// Fstenv_m28byte
	Mnemonic::Fnstcw,// Fnstcw_m2byte
	Mnemonic::Fstcw,// Fstcw_m2byte
	Mnemonic::Fld,// Fld_sti
	Mnemonic::Fxch,// Fxch_st0_sti
	Mnemonic::Fnop,// Fnop
	Mnemonic::Fstpnce,// Fstpnce_sti
	Mnemonic::Fchs,// Fchs
	Mnemonic::Fabs,// Fabs
	Mnemonic::Ftst,// Ftst
	Mnemonic::Fxam,// Fxam
	Mnemonic::Fld1,// Fld1
	Mnemonic::Fldl2t,// Fldl2t
	Mnemonic::Fldl2e,// Fldl2e
	Mnemonic::Fldpi,// Fldpi
	Mnemonic::Fldlg2,// Fldlg2
	Mnemonic::Fldln2,// Fldln2
	Mnemonic::Fldz,// Fldz
	Mnemonic::F2xm1,// F2xm1
	Mnemonic::Fyl2x,// Fyl2x
	Mnemonic::Fptan,// Fptan
	Mnemonic::Fpatan,// Fpatan
	Mnemonic::Fxtract,// Fxtract
	Mnemonic::Fprem1,// Fprem1
	Mnemonic::Fdecstp,// Fdecstp
	Mnemonic::Fincstp,// Fincstp
	Mnemonic::Fprem,// Fprem
	Mnemonic::Fyl2xp1,// Fyl2xp1
	Mnemonic::Fsqrt,// Fsqrt
	Mnemonic::Fsincos,// Fsincos
	Mnemonic::Frndint,// Frndint
	Mnemonic::Fscale,// Fscale
	Mnemonic::Fsin,// Fsin
	Mnemonic::Fcos,// Fcos
	Mnemonic::Fiadd,// Fiadd_m32int
	Mnemonic::Fimul,// Fimul_m32int
	Mnemonic::Ficom,// Ficom_m32int
	Mnemonic::Ficomp,// Ficomp_m32int
	Mnemonic::Fisub,// Fisub_m32int
	Mnemonic::Fisubr,// Fisubr_m32int
	Mnemonic::Fidiv,// Fidiv_m32int
	Mnemonic::Fidivr,// Fidivr_m32int
	Mnemonic::Fcmovb,// Fcmovb_st0_sti
	Mnemonic::Fcmove,// Fcmove_st0_sti
	Mnemonic::Fcmovbe,// Fcmovbe_st0_sti
	Mnemonic::Fcmovu,// Fcmovu_st0_sti
	Mnemonic::Fucompp,// Fucompp
	Mnemonic::Fild,// Fild_m32int
	Mnemonic::Fisttp,// Fisttp_m32int
	Mnemonic::Fist,// Fist_m32int
	Mnemonic::Fistp,// Fistp_m32int
	Mnemonic::Fld,// Fld_m80fp
	Mnemonic::Fstp,// Fstp_m80fp
	Mnemonic::Fcmovnb,// Fcmovnb_st0_sti
	Mnemonic::Fcmovne,// Fcmovne_st0_sti
	Mnemonic::Fcmovnbe,// Fcmovnbe_st0_sti
	Mnemonic::Fcmovnu,// Fcmovnu_st0_sti
	Mnemonic::Fneni,// Fneni
	Mnemonic::Feni,// Feni
	Mnemonic::Fndisi,// Fndisi
	Mnemonic::Fdisi,// Fdisi
	Mnemonic::Fnclex,// Fnclex
	Mnemonic::Fclex,// Fclex
	Mnemonic::Fninit,// Fninit
	Mnemonic::Finit,// Finit
	Mnemonic::Fnsetpm,// Fnsetpm
	Mnemonic::Fsetpm,// Fsetpm
	Mnemonic::Frstpm,// Frstpm
	Mnemonic::Fucomi,// Fucomi_st0_sti
	Mnemonic::Fcomi,// Fcomi_st0_sti
	Mnemonic::Fadd,// Fadd_m64fp
	Mnemonic::Fmul,// Fmul_m64fp
	Mnemonic::Fcom,// Fcom_m64fp
	Mnemonic::Fcomp,// Fcomp_m64fp
	Mnemonic::Fsub,// Fsub_m64fp
	Mnemonic::Fsubr,// Fsubr_m64fp
	Mnemonic::Fdiv,// Fdiv_m64fp
	Mnemonic::Fdivr,// Fdivr_m64fp
	Mnemonic::Fadd,// Fadd_sti_st0
	Mnemonic::Fmul,// Fmul_sti_st0
	Mnemonic::Fcom,// Fcom_st0_sti_DCD0
	Mnemonic::Fcomp,// Fcomp_st0_sti_DCD8
	Mnemonic::Fsubr,// Fsubr_sti_st0
	Mnemonic::Fsub,// Fsub_sti_st0
	Mnemonic::Fdivr,// Fdivr_sti_st0
	Mnemonic::Fdiv,// Fdiv_sti_st0
	Mnemonic::Fld,// Fld_m64fp
	Mnemonic::Fisttp,// Fisttp_m64int
	Mnemonic::Fst,// Fst_m64fp
	Mnemonic::Fstp,// Fstp_m64fp
	Mnemonic::Frstor,// Frstor_m94byte
	Mnemonic::Frstor,// Frstor_m108byte
	Mnemonic::Fnsave,// Fnsave_m94byte
	Mnemonic::Fsave,// Fsave_m94byte
	Mnemonic::Fnsave,// Fnsave_m108byte
	Mnemonic::Fsave,// Fsave_m108byte
	Mnemonic::Fnstsw,// Fnstsw_m2byte
	Mnemonic::Fstsw,// Fstsw_m2byte
	Mnemonic::Ffree,// Ffree_sti
	Mnemonic::Fxch,// Fxch_st0_sti_DDC8
	Mnemonic::Fst,// Fst_sti
	Mnemonic::Fstp,// Fstp_sti
	Mnemonic::Fucom,// Fucom_st0_sti
	Mnemonic::Fucomp,// Fucomp_st0_sti
	Mnemonic::Fiadd,// Fiadd_m16int
	Mnemonic::Fimul,// Fimul_m16int
	Mnemonic::Ficom,// Ficom_m16int
	Mnemonic::Ficomp,// Ficomp_m16int
	Mnemonic::Fisub,// Fisub_m16int
	Mnemonic::Fisubr,// Fisubr_m16int
	Mnemonic::Fidiv,// Fidiv_m16int
	Mnemonic::Fidivr,// Fidivr_m16int
	Mnemonic::Faddp,// Faddp_sti_st0
	Mnemonic::Fmulp,// Fmulp_sti_st0
	Mnemonic::Fcomp,// Fcomp_st0_sti_DED0
	Mnemonic::Fcompp,// Fcompp
	Mnemonic::Fsubrp,// Fsubrp_sti_st0
	Mnemonic::Fsubp,// Fsubp_sti_st0
	Mnemonic::Fdivrp,// Fdivrp_sti_st0
	Mnemonic::Fdivp,// Fdivp_sti_st0
	Mnemonic::Fild,// Fild_m16int
	Mnemonic::Fisttp,// Fisttp_m16int
	Mnemonic::Fist,// Fist_m16int
	Mnemonic::Fistp,// Fistp_m16int
	Mnemonic::Fbld,// Fbld_m80bcd
	Mnemonic::Fild,// Fild_m64int
	Mnemonic::Fbstp,// Fbstp_m80bcd
	Mnemonic::Fistp,// Fistp_m64int
	Mnemonic::Ffreep,// Ffreep_sti
	Mnemonic::Fxch,// Fxch_st0_sti_DFC8
	Mnemonic::Fstp,// Fstp_sti_DFD0
	Mnemonic::Fstp,// Fstp_sti_DFD8
	Mnemonic::Fnstsw,// Fnstsw_AX
	Mnemonic::Fstsw,// Fstsw_AX
	Mnemonic::Fstdw,// Fstdw_AX
	Mnemonic::Fstsg,// Fstsg_AX
	Mnemonic::Fucomip,// Fucomip_st0_sti
	Mnemonic::Fcomip,// Fcomip_st0_sti
	Mnemonic::Loopne,// Loopne_rel8_16_CX
	Mnemonic::Loopne,// Loopne_rel8_32_CX
	Mnemonic::Loopne,// Loopne_rel8_16_ECX
	Mnemonic::Loopne,// Loopne_rel8_32_ECX
	Mnemonic::Loopne,// Loopne_rel8_64_ECX
	Mnemonic::Loopne,// Loopne_rel8_16_RCX
	Mnemonic::Loopne,// Loopne_rel8_64_RCX
	Mnemonic::Loope,// Loope_rel8_16_CX
	Mnemonic::Loope,// Loope_rel8_32_CX
	Mnemonic::Loope,// Loope_rel8_16_ECX
	Mnemonic::Loope,// Loope_rel8_32_ECX
	Mnemonic::Loope,// Loope_rel8_64_ECX
	Mnemonic::Loope,// Loope_rel8_16_RCX
	Mnemonic::Loope,// Loope_rel8_64_RCX
	Mnemonic::Loop,// Loop_rel8_16_CX
	Mnemonic::Loop,// Loop_rel8_32_CX
	Mnemonic::Loop,// Loop_rel8_16_ECX
	Mnemonic::Loop,// Loop_rel8_32_ECX
	Mnemonic::Loop,// Loop_rel8_64_ECX
	Mnemonic::Loop,// Loop_rel8_16_RCX
	Mnemonic::Loop,// Loop_rel8_64_RCX
	Mnemonic::Jcxz,// Jcxz_rel8_16
	Mnemonic::Jcxz,// Jcxz_rel8_32
	Mnemonic::Jecxz,// Jecxz_rel8_16
	Mnemonic::Jecxz,// Jecxz_rel8_32
	Mnemonic::Jecxz,// Jecxz_rel8_64
	Mnemonic::Jrcxz,// Jrcxz_rel8_16
	Mnemonic::Jrcxz,// Jrcxz_rel8_64
	Mnemonic::In,// In_AL_imm8
	Mnemonic::In,// In_AX_imm8
	Mnemonic::In,// In_EAX_imm8
	Mnemonic::Out,// Out_imm8_AL
	Mnemonic::Out,// Out_imm8_AX
	Mnemonic::Out,// Out_imm8_EAX
	Mnemonic::Call,// Call_rel16
	Mnemonic::Call,// Call_rel32_32
	Mnemonic::Call,// Call_rel32_64
	Mnemonic::Jmp,// Jmp_rel16
	Mnemonic::Jmp,// Jmp_rel32_32
	Mnemonic::Jmp,// Jmp_rel32_64
	Mnemonic::Jmp,// Jmp_ptr1616
	Mnemonic::Jmp,// Jmp_ptr1632
	Mnemonic::Jmp,// Jmp_rel8_16
	Mnemonic::Jmp,// Jmp_rel8_32
	Mnemonic::Jmp,// Jmp_rel8_64
	Mnemonic::In,// In_AL_DX
	Mnemonic::In,// In_AX_DX
	Mnemonic::In,// In_EAX_DX
	Mnemonic::Out,// Out_DX_AL
	Mnemonic::Out,// Out_DX_AX
	Mnemonic::Out,// Out_DX_EAX
	Mnemonic::Int1,// Int1
	Mnemonic::Hlt,// Hlt
	Mnemonic::Cmc,// Cmc
	Mnemonic::Test,// Test_rm8_imm8
	Mnemonic::Test,// Test_rm8_imm8_F6r1
	Mnemonic::Not,// Not_rm8
	Mnemonic::Neg,// Neg_rm8
	Mnemonic::Mul,// Mul_rm8
	Mnemonic::Imul,// Imul_rm8
	Mnemonic::Div,// Div_rm8
	Mnemonic::Idiv,// Idiv_rm8
	Mnemonic::Test,// Test_rm16_imm16
	Mnemonic::Test,// Test_rm32_imm32
	Mnemonic::Test,// Test_rm64_imm32
	Mnemonic::Test,// Test_rm16_imm16_F7r1
	Mnemonic::Test,// Test_rm32_imm32_F7r1
	Mnemonic::Test,// Test_rm64_imm32_F7r1
	Mnemonic::Not,// Not_rm16
	Mnemonic::Not,// Not_rm32
	Mnemonic::Not,// Not_rm64
	Mnemonic::Neg,// Neg_rm16
	Mnemonic::Neg,// Neg_rm32
	Mnemonic::Neg,// Neg_rm64
	Mnemonic::Mul,// Mul_rm16
	Mnemonic::Mul,// Mul_rm32
	Mnemonic::Mul,// Mul_rm64
	Mnemonic::Imul,// Imul_rm16
	Mnemonic::Imul,// Imul_rm32
	Mnemonic::Imul,// Imul_rm64
	Mnemonic::Div,// Div_rm16
	Mnemonic::Div,// Div_rm32
	Mnemonic::Div,// Div_rm64
	Mnemonic::Idiv,// Idiv_rm16
	Mnemonic::Idiv,// Idiv_rm32
	Mnemonic::Idiv,// Idiv_rm64
	Mnemonic::Clc,// Clc
	Mnemonic::Stc,// Stc
	Mnemonic::Cli,// Cli
	Mnemonic::Sti,// Sti
	Mnemonic::Cld,// Cld
	Mnemonic::Std,// Std
	Mnemonic::Inc,// Inc_rm8
	Mnemonic::Dec,// Dec_rm8
	Mnemonic::Inc,// Inc_rm16
	Mnemonic::Inc,// Inc_rm32
	Mnemonic::Inc,// Inc_rm64
	Mnemonic::Dec,// Dec_rm16
	Mnemonic::Dec,// Dec_rm32
	Mnemonic::Dec,// Dec_rm64
	Mnemonic::Call,// Call_rm16
	Mnemonic::Call,// Call_rm32
	Mnemonic::Call,// Call_rm64
	Mnemonic::Call,// Call_m1616
	Mnemonic::Call,// Call_m1632
	Mnemonic::Call,// Call_m1664
	Mnemonic::Jmp,// Jmp_rm16
	Mnemonic::Jmp,// Jmp_rm32
	Mnemonic::Jmp,// Jmp_rm64
	Mnemonic::Jmp,// Jmp_m1616
	Mnemonic::Jmp,// Jmp_m1632
	Mnemonic::Jmp,// Jmp_m1664
	Mnemonic::Push,// Push_rm16
	Mnemonic::Push,// Push_rm32
	Mnemonic::Push,// Push_rm64
	Mnemonic::Sldt,// Sldt_rm16
	Mnemonic::Sldt,// Sldt_r32m16
	Mnemonic::Sldt,// Sldt_r64m16
	Mnemonic::Str,// Str_rm16
	Mnemonic::Str,// Str_r32m16
	Mnemonic::Str,// Str_r64m16
	Mnemonic::Lldt,// Lldt_rm16
	Mnemonic::Lldt,// Lldt_r32m16
	Mnemonic::Lldt,// Lldt_r64m16
	Mnemonic::Ltr,// Ltr_rm16
	Mnemonic::Ltr,// Ltr_r32m16
	Mnemonic::Ltr,// Ltr_r64m16
	Mnemonic::Verr,// Verr_rm16
	Mnemonic::Verr,// Verr_r32m16
	Mnemonic::Verr,// Verr_r64m16
	Mnemonic::Verw,// Verw_rm16
	Mnemonic::Verw,// Verw_r32m16
	Mnemonic::Verw,// Verw_r64m16
	Mnemonic::Jmpe,// Jmpe_rm16
	Mnemonic::Jmpe,// Jmpe_rm32
	Mnemonic::Sgdt,// Sgdt_m1632_16
	Mnemonic::Sgdt,// Sgdt_m1632
	Mnemonic::Sgdt,// Sgdt_m1664
	Mnemonic::Sidt,// Sidt_m1632_16
	Mnemonic::Sidt,// Sidt_m1632
	Mnemonic::Sidt,// Sidt_m1664
	Mnemonic::Lgdt,// Lgdt_m1632_16
	Mnemonic::Lgdt,// Lgdt_m1632
	Mnemonic::Lgdt,// Lgdt_m1664
	Mnemonic::Lidt,// Lidt_m1632_16
	Mnemonic::Lidt,// Lidt_m1632
	Mnemonic::Lidt,// Lidt_m1664
	Mnemonic::Smsw,// Smsw_rm16
	Mnemonic::Smsw,// Smsw_r32m16
	Mnemonic::Smsw,// Smsw_r64m16
	Mnemonic::Rstorssp,// Rstorssp_m64
	Mnemonic::Lmsw,// Lmsw_rm16
	Mnemonic::Lmsw,// Lmsw_r32m16
	Mnemonic::Lmsw,// Lmsw_r64m16
	Mnemonic::Invlpg,// Invlpg_m
	Mnemonic::Enclv,// Enclv
	Mnemonic::Vmcall,// Vmcall
	Mnemonic::Vmlaunch,// Vmlaunch
	Mnemonic::Vmresume,// Vmresume
	Mnemonic::Vmxoff,// Vmxoff
	Mnemonic::Pconfig,// Pconfig
	Mnemonic::Monitor,// Monitorw
	Mnemonic::Monitor,// Monitord
	Mnemonic::Monitor,// Monitorq
	Mnemonic::Mwait,// Mwait
	Mnemonic::Clac,// Clac
	Mnemonic::Stac,// Stac
	Mnemonic::Encls,// Encls
	Mnemonic::Xgetbv,// Xgetbv
	Mnemonic::Xsetbv,// Xsetbv
	Mnemonic::Vmfunc,// Vmfunc
	Mnemonic::Xend,// Xend
	Mnemonic::Xtest,// Xtest
	Mnemonic::Enclu,// Enclu
	Mnemonic::Vmrun,// Vmrunw
	Mnemonic::Vmrun,// Vmrund
	Mnemonic::Vmrun,// Vmrunq
	Mnemonic::Vmmcall,// Vmmcall
	Mnemonic::Vmload,// Vmloadw
	Mnemonic::Vmload,// Vmloadd
	Mnemonic::Vmload,// Vmloadq
	Mnemonic::Vmsave,// Vmsavew
	Mnemonic::Vmsave,// Vmsaved
	Mnemonic::Vmsave,// Vmsaveq
	Mnemonic::Stgi,// Stgi
	Mnemonic::Clgi,// Clgi
	Mnemonic::Skinit,// Skinit
	Mnemonic::Invlpga,// Invlpgaw
	Mnemonic::Invlpga,// Invlpgad
	Mnemonic::Invlpga,// Invlpgaq
	Mnemonic::Setssbsy,// Setssbsy
	Mnemonic::Saveprevssp,// Saveprevssp
	Mnemonic::Rdpkru,// Rdpkru
	Mnemonic::Wrpkru,// Wrpkru
	Mnemonic::Swapgs,// Swapgs
	Mnemonic::Rdtscp,// Rdtscp
	Mnemonic::Monitorx,// Monitorxw
	Mnemonic::Monitorx,// Monitorxd
	Mnemonic::Monitorx,// Monitorxq
	Mnemonic::Mcommit,// Mcommit
	Mnemonic::Mwaitx,// Mwaitx
	Mnemonic::Clzero,// Clzerow
	Mnemonic::Clzero,// Clzerod
	Mnemonic::Clzero,// Clzeroq
	Mnemonic::Rdpru,// Rdpru
	Mnemonic::Lar,// Lar_r16_rm16
	Mnemonic::Lar,// Lar_r32_r32m16
	Mnemonic::Lar,// Lar_r64_r64m16
	Mnemonic::Lsl,// Lsl_r16_rm16
	Mnemonic::Lsl,// Lsl_r32_r32m16
	Mnemonic::Lsl,// Lsl_r64_r64m16
	Mnemonic::Storeall,// Storeall
	Mnemonic::Loadall,// Loadall286
	Mnemonic::Syscall,// Syscall
	Mnemonic::Clts,// Clts
	Mnemonic::Loadall,// Loadall386
	Mnemonic::Sysret,// Sysretd
	Mnemonic::Sysretq,// Sysretq
	Mnemonic::Invd,// Invd
	Mnemonic::Wbinvd,// Wbinvd
	Mnemonic::Wbnoinvd,// Wbnoinvd
	Mnemonic::Cl1invmb,// Cl1invmb
	Mnemonic::Ud2,// Ud2
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F0D
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F0D
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F0D
	Mnemonic::Prefetch,// Prefetch_m8
	Mnemonic::Prefetchw,// Prefetchw_m8
	Mnemonic::Prefetchwt1,// Prefetchwt1_m8
	Mnemonic::Femms,// Femms
	Mnemonic::Umov,// Umov_rm8_r8
	Mnemonic::Umov,// Umov_rm16_r16
	Mnemonic::Umov,// Umov_rm32_r32
	Mnemonic::Umov,// Umov_r8_rm8
	Mnemonic::Umov,// Umov_r16_rm16
	Mnemonic::Umov,// Umov_r32_rm32
	Mnemonic::Movups,// Movups_xmm_xmmm128
	Mnemonic::Vmovups,// VEX_Vmovups_xmm_xmmm128
	Mnemonic::Vmovups,// VEX_Vmovups_ymm_ymmm256
	Mnemonic::Vmovups,// EVEX_Vmovups_xmm_k1z_xmmm128
	Mnemonic::Vmovups,// EVEX_Vmovups_ymm_k1z_ymmm256
	Mnemonic::Vmovups,// EVEX_Vmovups_zmm_k1z_zmmm512
	Mnemonic::Movupd,// Movupd_xmm_xmmm128
	Mnemonic::Vmovupd,// VEX_Vmovupd_xmm_xmmm128
	Mnemonic::Vmovupd,// VEX_Vmovupd_ymm_ymmm256
	Mnemonic::Vmovupd,// EVEX_Vmovupd_xmm_k1z_xmmm128
	Mnemonic::Vmovupd,// EVEX_Vmovupd_ymm_k1z_ymmm256
	Mnemonic::Vmovupd,// EVEX_Vmovupd_zmm_k1z_zmmm512
	Mnemonic::Movss,// Movss_xmm_xmmm32
	Mnemonic::Vmovss,// VEX_Vmovss_xmm_xmm_xmm
	Mnemonic::Vmovss,// VEX_Vmovss_xmm_m32
	Mnemonic::Vmovss,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	Mnemonic::Vmovss,// EVEX_Vmovss_xmm_k1z_m32
	Mnemonic::Movsd,// Movsd_xmm_xmmm64
	Mnemonic::Vmovsd,// VEX_Vmovsd_xmm_xmm_xmm
	Mnemonic::Vmovsd,// VEX_Vmovsd_xmm_m64
	Mnemonic::Vmovsd,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	Mnemonic::Vmovsd,// EVEX_Vmovsd_xmm_k1z_m64
	Mnemonic::Movups,// Movups_xmmm128_xmm
	Mnemonic::Vmovups,// VEX_Vmovups_xmmm128_xmm
	Mnemonic::Vmovups,// VEX_Vmovups_ymmm256_ymm
	Mnemonic::Vmovups,// EVEX_Vmovups_xmmm128_k1z_xmm
	Mnemonic::Vmovups,// EVEX_Vmovups_ymmm256_k1z_ymm
	Mnemonic::Vmovups,// EVEX_Vmovups_zmmm512_k1z_zmm
	Mnemonic::Movupd,// Movupd_xmmm128_xmm
	Mnemonic::Vmovupd,// VEX_Vmovupd_xmmm128_xmm
	Mnemonic::Vmovupd,// VEX_Vmovupd_ymmm256_ymm
	Mnemonic::Vmovupd,// EVEX_Vmovupd_xmmm128_k1z_xmm
	Mnemonic::Vmovupd,// EVEX_Vmovupd_ymmm256_k1z_ymm
	Mnemonic::Vmovupd,// EVEX_Vmovupd_zmmm512_k1z_zmm
	Mnemonic::Movss,// Movss_xmmm32_xmm
	Mnemonic::Vmovss,// VEX_Vmovss_xmm_xmm_xmm_0F11
	Mnemonic::Vmovss,// VEX_Vmovss_m32_xmm
	Mnemonic::Vmovss,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	Mnemonic::Vmovss,// EVEX_Vmovss_m32_k1_xmm
	Mnemonic::Movsd,// Movsd_xmmm64_xmm
	Mnemonic::Vmovsd,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	Mnemonic::Vmovsd,// VEX_Vmovsd_m64_xmm
	Mnemonic::Vmovsd,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	Mnemonic::Vmovsd,// EVEX_Vmovsd_m64_k1_xmm
	Mnemonic::Movhlps,// Movhlps_xmm_xmm
	Mnemonic::Movlps,// Movlps_xmm_m64
	Mnemonic::Vmovhlps,// VEX_Vmovhlps_xmm_xmm_xmm
	Mnemonic::Vmovlps,// VEX_Vmovlps_xmm_xmm_m64
	Mnemonic::Vmovhlps,// EVEX_Vmovhlps_xmm_xmm_xmm
	Mnemonic::Vmovlps,// EVEX_Vmovlps_xmm_xmm_m64
	Mnemonic::Movlpd,// Movlpd_xmm_m64
	Mnemonic::Vmovlpd,// VEX_Vmovlpd_xmm_xmm_m64
	Mnemonic::Vmovlpd,// EVEX_Vmovlpd_xmm_xmm_m64
	Mnemonic::Movsldup,// Movsldup_xmm_xmmm128
	Mnemonic::Vmovsldup,// VEX_Vmovsldup_xmm_xmmm128
	Mnemonic::Vmovsldup,// VEX_Vmovsldup_ymm_ymmm256
	Mnemonic::Vmovsldup,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	Mnemonic::Vmovsldup,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	Mnemonic::Vmovsldup,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	Mnemonic::Movddup,// Movddup_xmm_xmmm64
	Mnemonic::Vmovddup,// VEX_Vmovddup_xmm_xmmm64
	Mnemonic::Vmovddup,// VEX_Vmovddup_ymm_ymmm256
	Mnemonic::Vmovddup,// EVEX_Vmovddup_xmm_k1z_xmmm64
	Mnemonic::Vmovddup,// EVEX_Vmovddup_ymm_k1z_ymmm256
	Mnemonic::Vmovddup,// EVEX_Vmovddup_zmm_k1z_zmmm512
	Mnemonic::Movlps,// Movlps_m64_xmm
	Mnemonic::Vmovlps,// VEX_Vmovlps_m64_xmm
	Mnemonic::Vmovlps,// EVEX_Vmovlps_m64_xmm
	Mnemonic::Movlpd,// Movlpd_m64_xmm
	Mnemonic::Vmovlpd,// VEX_Vmovlpd_m64_xmm
	Mnemonic::Vmovlpd,// EVEX_Vmovlpd_m64_xmm
	Mnemonic::Unpcklps,// Unpcklps_xmm_xmmm128
	Mnemonic::Vunpcklps,// VEX_Vunpcklps_xmm_xmm_xmmm128
	Mnemonic::Vunpcklps,// VEX_Vunpcklps_ymm_ymm_ymmm256
	Mnemonic::Vunpcklps,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vunpcklps,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vunpcklps,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Unpcklpd,// Unpcklpd_xmm_xmmm128
	Mnemonic::Vunpcklpd,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	Mnemonic::Vunpcklpd,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	Mnemonic::Vunpcklpd,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vunpcklpd,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vunpcklpd,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Unpckhps,// Unpckhps_xmm_xmmm128
	Mnemonic::Vunpckhps,// VEX_Vunpckhps_xmm_xmm_xmmm128
	Mnemonic::Vunpckhps,// VEX_Vunpckhps_ymm_ymm_ymmm256
	Mnemonic::Vunpckhps,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vunpckhps,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vunpckhps,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Unpckhpd,// Unpckhpd_xmm_xmmm128
	Mnemonic::Vunpckhpd,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	Mnemonic::Vunpckhpd,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	Mnemonic::Vunpckhpd,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vunpckhpd,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vunpckhpd,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Movlhps,// Movlhps_xmm_xmm
	Mnemonic::Vmovlhps,// VEX_Vmovlhps_xmm_xmm_xmm
	Mnemonic::Vmovlhps,// EVEX_Vmovlhps_xmm_xmm_xmm
	Mnemonic::Movhps,// Movhps_xmm_m64
	Mnemonic::Vmovhps,// VEX_Vmovhps_xmm_xmm_m64
	Mnemonic::Vmovhps,// EVEX_Vmovhps_xmm_xmm_m64
	Mnemonic::Movhpd,// Movhpd_xmm_m64
	Mnemonic::Vmovhpd,// VEX_Vmovhpd_xmm_xmm_m64
	Mnemonic::Vmovhpd,// EVEX_Vmovhpd_xmm_xmm_m64
	Mnemonic::Movshdup,// Movshdup_xmm_xmmm128
	Mnemonic::Vmovshdup,// VEX_Vmovshdup_xmm_xmmm128
	Mnemonic::Vmovshdup,// VEX_Vmovshdup_ymm_ymmm256
	Mnemonic::Vmovshdup,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	Mnemonic::Vmovshdup,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	Mnemonic::Vmovshdup,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	Mnemonic::Movhps,// Movhps_m64_xmm
	Mnemonic::Vmovhps,// VEX_Vmovhps_m64_xmm
	Mnemonic::Vmovhps,// EVEX_Vmovhps_m64_xmm
	Mnemonic::Movhpd,// Movhpd_m64_xmm
	Mnemonic::Vmovhpd,// VEX_Vmovhpd_m64_xmm
	Mnemonic::Vmovhpd,// EVEX_Vmovhpd_m64_xmm
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F18
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F18
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F18
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F19
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F19
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F19
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1A
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1A
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1A
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1B
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1B
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1B
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1C
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1C
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1C
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1D
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1D
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1D
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1E
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1E
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1E
	Mnemonic::Reservednop,// Reservednop_rm16_r16_0F1F
	Mnemonic::Reservednop,// Reservednop_rm32_r32_0F1F
	Mnemonic::Reservednop,// Reservednop_rm64_r64_0F1F
	Mnemonic::Prefetchnta,// Prefetchnta_m8
	Mnemonic::Prefetcht0,// Prefetcht0_m8
	Mnemonic::Prefetcht1,// Prefetcht1_m8
	Mnemonic::Prefetcht2,// Prefetcht2_m8
	Mnemonic::Bndldx,// Bndldx_bnd_mib
	Mnemonic::Bndmov,// Bndmov_bnd_bndm64
	Mnemonic::Bndmov,// Bndmov_bnd_bndm128
	Mnemonic::Bndcl,// Bndcl_bnd_rm32
	Mnemonic::Bndcl,// Bndcl_bnd_rm64
	Mnemonic::Bndcu,// Bndcu_bnd_rm32
	Mnemonic::Bndcu,// Bndcu_bnd_rm64
	Mnemonic::Bndstx,// Bndstx_mib_bnd
	Mnemonic::Bndmov,// Bndmov_bndm64_bnd
	Mnemonic::Bndmov,// Bndmov_bndm128_bnd
	Mnemonic::Bndmk,// Bndmk_bnd_m32
	Mnemonic::Bndmk,// Bndmk_bnd_m64
	Mnemonic::Bndcn,// Bndcn_bnd_rm32
	Mnemonic::Bndcn,// Bndcn_bnd_rm64
	Mnemonic::Cldemote,// Cldemote_m8
	Mnemonic::Rdsspd,// Rdsspd_r32
	Mnemonic::Rdsspq,// Rdsspq_r64
	Mnemonic::Endbr64,// Endbr64
	Mnemonic::Endbr32,// Endbr32
	Mnemonic::Nop,// Nop_rm16
	Mnemonic::Nop,// Nop_rm32
	Mnemonic::Nop,// Nop_rm64
	Mnemonic::Mov,// Mov_r32_cr
	Mnemonic::Mov,// Mov_r64_cr
	Mnemonic::Mov,// Mov_r32_dr
	Mnemonic::Mov,// Mov_r64_dr
	Mnemonic::Mov,// Mov_cr_r32
	Mnemonic::Mov,// Mov_cr_r64
	Mnemonic::Mov,// Mov_dr_r32
	Mnemonic::Mov,// Mov_dr_r64
	Mnemonic::Mov,// Mov_r32_tr
	Mnemonic::Mov,// Mov_tr_r32
	Mnemonic::Movaps,// Movaps_xmm_xmmm128
	Mnemonic::Vmovaps,// VEX_Vmovaps_xmm_xmmm128
	Mnemonic::Vmovaps,// VEX_Vmovaps_ymm_ymmm256
	Mnemonic::Vmovaps,// EVEX_Vmovaps_xmm_k1z_xmmm128
	Mnemonic::Vmovaps,// EVEX_Vmovaps_ymm_k1z_ymmm256
	Mnemonic::Vmovaps,// EVEX_Vmovaps_zmm_k1z_zmmm512
	Mnemonic::Movapd,// Movapd_xmm_xmmm128
	Mnemonic::Vmovapd,// VEX_Vmovapd_xmm_xmmm128
	Mnemonic::Vmovapd,// VEX_Vmovapd_ymm_ymmm256
	Mnemonic::Vmovapd,// EVEX_Vmovapd_xmm_k1z_xmmm128
	Mnemonic::Vmovapd,// EVEX_Vmovapd_ymm_k1z_ymmm256
	Mnemonic::Vmovapd,// EVEX_Vmovapd_zmm_k1z_zmmm512
	Mnemonic::Movaps,// Movaps_xmmm128_xmm
	Mnemonic::Vmovaps,// VEX_Vmovaps_xmmm128_xmm
	Mnemonic::Vmovaps,// VEX_Vmovaps_ymmm256_ymm
	Mnemonic::Vmovaps,// EVEX_Vmovaps_xmmm128_k1z_xmm
	Mnemonic::Vmovaps,// EVEX_Vmovaps_ymmm256_k1z_ymm
	Mnemonic::Vmovaps,// EVEX_Vmovaps_zmmm512_k1z_zmm
	Mnemonic::Movapd,// Movapd_xmmm128_xmm
	Mnemonic::Vmovapd,// VEX_Vmovapd_xmmm128_xmm
	Mnemonic::Vmovapd,// VEX_Vmovapd_ymmm256_ymm
	Mnemonic::Vmovapd,// EVEX_Vmovapd_xmmm128_k1z_xmm
	Mnemonic::Vmovapd,// EVEX_Vmovapd_ymmm256_k1z_ymm
	Mnemonic::Vmovapd,// EVEX_Vmovapd_zmmm512_k1z_zmm
	Mnemonic::Cvtpi2ps,// Cvtpi2ps_xmm_mmm64
	Mnemonic::Cvtpi2pd,// Cvtpi2pd_xmm_mmm64
	Mnemonic::Cvtsi2ss,// Cvtsi2ss_xmm_rm32
	Mnemonic::Cvtsi2ss,// Cvtsi2ss_xmm_rm64
	Mnemonic::Vcvtsi2ss,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	Mnemonic::Vcvtsi2ss,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	Mnemonic::Vcvtsi2ss,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	Mnemonic::Vcvtsi2ss,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	Mnemonic::Cvtsi2sd,// Cvtsi2sd_xmm_rm32
	Mnemonic::Cvtsi2sd,// Cvtsi2sd_xmm_rm64
	Mnemonic::Vcvtsi2sd,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	Mnemonic::Vcvtsi2sd,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	Mnemonic::Vcvtsi2sd,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	Mnemonic::Vcvtsi2sd,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	Mnemonic::Movntps,// Movntps_m128_xmm
	Mnemonic::Vmovntps,// VEX_Vmovntps_m128_xmm
	Mnemonic::Vmovntps,// VEX_Vmovntps_m256_ymm
	Mnemonic::Vmovntps,// EVEX_Vmovntps_m128_xmm
	Mnemonic::Vmovntps,// EVEX_Vmovntps_m256_ymm
	Mnemonic::Vmovntps,// EVEX_Vmovntps_m512_zmm
	Mnemonic::Movntpd,// Movntpd_m128_xmm
	Mnemonic::Vmovntpd,// VEX_Vmovntpd_m128_xmm
	Mnemonic::Vmovntpd,// VEX_Vmovntpd_m256_ymm
	Mnemonic::Vmovntpd,// EVEX_Vmovntpd_m128_xmm
	Mnemonic::Vmovntpd,// EVEX_Vmovntpd_m256_ymm
	Mnemonic::Vmovntpd,// EVEX_Vmovntpd_m512_zmm
	Mnemonic::Movntss,// Movntss_m32_xmm
	Mnemonic::Movntsd,// Movntsd_m64_xmm
	Mnemonic::Cvttps2pi,// Cvttps2pi_mm_xmmm64
	Mnemonic::Cvttpd2pi,// Cvttpd2pi_mm_xmmm128
	Mnemonic::Cvttss2si,// Cvttss2si_r32_xmmm32
	Mnemonic::Cvttss2si,// Cvttss2si_r64_xmmm32
	Mnemonic::Vcvttss2si,// VEX_Vcvttss2si_r32_xmmm32
	Mnemonic::Vcvttss2si,// VEX_Vcvttss2si_r64_xmmm32
	Mnemonic::Vcvttss2si,// EVEX_Vcvttss2si_r32_xmmm32_sae
	Mnemonic::Vcvttss2si,// EVEX_Vcvttss2si_r64_xmmm32_sae
	Mnemonic::Cvttsd2si,// Cvttsd2si_r32_xmmm64
	Mnemonic::Cvttsd2si,// Cvttsd2si_r64_xmmm64
	Mnemonic::Vcvttsd2si,// VEX_Vcvttsd2si_r32_xmmm64
	Mnemonic::Vcvttsd2si,// VEX_Vcvttsd2si_r64_xmmm64
	Mnemonic::Vcvttsd2si,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	Mnemonic::Vcvttsd2si,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	Mnemonic::Cvtps2pi,// Cvtps2pi_mm_xmmm64
	Mnemonic::Cvtpd2pi,// Cvtpd2pi_mm_xmmm128
	Mnemonic::Cvtss2si,// Cvtss2si_r32_xmmm32
	Mnemonic::Cvtss2si,// Cvtss2si_r64_xmmm32
	Mnemonic::Vcvtss2si,// VEX_Vcvtss2si_r32_xmmm32
	Mnemonic::Vcvtss2si,// VEX_Vcvtss2si_r64_xmmm32
	Mnemonic::Vcvtss2si,// EVEX_Vcvtss2si_r32_xmmm32_er
	Mnemonic::Vcvtss2si,// EVEX_Vcvtss2si_r64_xmmm32_er
	Mnemonic::Cvtsd2si,// Cvtsd2si_r32_xmmm64
	Mnemonic::Cvtsd2si,// Cvtsd2si_r64_xmmm64
	Mnemonic::Vcvtsd2si,// VEX_Vcvtsd2si_r32_xmmm64
	Mnemonic::Vcvtsd2si,// VEX_Vcvtsd2si_r64_xmmm64
	Mnemonic::Vcvtsd2si,// EVEX_Vcvtsd2si_r32_xmmm64_er
	Mnemonic::Vcvtsd2si,// EVEX_Vcvtsd2si_r64_xmmm64_er
	Mnemonic::Ucomiss,// Ucomiss_xmm_xmmm32
	Mnemonic::Vucomiss,// VEX_Vucomiss_xmm_xmmm32
	Mnemonic::Vucomiss,// EVEX_Vucomiss_xmm_xmmm32_sae
	Mnemonic::Ucomisd,// Ucomisd_xmm_xmmm64
	Mnemonic::Vucomisd,// VEX_Vucomisd_xmm_xmmm64
	Mnemonic::Vucomisd,// EVEX_Vucomisd_xmm_xmmm64_sae
	Mnemonic::Comiss,// Comiss_xmm_xmmm32
	Mnemonic::Comisd,// Comisd_xmm_xmmm64
	Mnemonic::Vcomiss,// VEX_Vcomiss_xmm_xmmm32
	Mnemonic::Vcomisd,// VEX_Vcomisd_xmm_xmmm64
	Mnemonic::Vcomiss,// EVEX_Vcomiss_xmm_xmmm32_sae
	Mnemonic::Vcomisd,// EVEX_Vcomisd_xmm_xmmm64_sae
	Mnemonic::Wrmsr,// Wrmsr
	Mnemonic::Rdtsc,// Rdtsc
	Mnemonic::Rdmsr,// Rdmsr
	Mnemonic::Rdpmc,// Rdpmc
	Mnemonic::Sysenter,// Sysenter
	Mnemonic::Sysexit,// Sysexitd
	Mnemonic::Sysexitq,// Sysexitq
	Mnemonic::Getsec,// Getsecd
	Mnemonic::Cmovo,// Cmovo_r16_rm16
	Mnemonic::Cmovo,// Cmovo_r32_rm32
	Mnemonic::Cmovo,// Cmovo_r64_rm64
	Mnemonic::Cmovno,// Cmovno_r16_rm16
	Mnemonic::Cmovno,// Cmovno_r32_rm32
	Mnemonic::Cmovno,// Cmovno_r64_rm64
	Mnemonic::Cmovb,// Cmovb_r16_rm16
	Mnemonic::Cmovb,// Cmovb_r32_rm32
	Mnemonic::Cmovb,// Cmovb_r64_rm64
	Mnemonic::Cmovae,// Cmovae_r16_rm16
	Mnemonic::Cmovae,// Cmovae_r32_rm32
	Mnemonic::Cmovae,// Cmovae_r64_rm64
	Mnemonic::Cmove,// Cmove_r16_rm16
	Mnemonic::Cmove,// Cmove_r32_rm32
	Mnemonic::Cmove,// Cmove_r64_rm64
	Mnemonic::Cmovne,// Cmovne_r16_rm16
	Mnemonic::Cmovne,// Cmovne_r32_rm32
	Mnemonic::Cmovne,// Cmovne_r64_rm64
	Mnemonic::Cmovbe,// Cmovbe_r16_rm16
	Mnemonic::Cmovbe,// Cmovbe_r32_rm32
	Mnemonic::Cmovbe,// Cmovbe_r64_rm64
	Mnemonic::Cmova,// Cmova_r16_rm16
	Mnemonic::Cmova,// Cmova_r32_rm32
	Mnemonic::Cmova,// Cmova_r64_rm64
	Mnemonic::Cmovs,// Cmovs_r16_rm16
	Mnemonic::Cmovs,// Cmovs_r32_rm32
	Mnemonic::Cmovs,// Cmovs_r64_rm64
	Mnemonic::Cmovns,// Cmovns_r16_rm16
	Mnemonic::Cmovns,// Cmovns_r32_rm32
	Mnemonic::Cmovns,// Cmovns_r64_rm64
	Mnemonic::Cmovp,// Cmovp_r16_rm16
	Mnemonic::Cmovp,// Cmovp_r32_rm32
	Mnemonic::Cmovp,// Cmovp_r64_rm64
	Mnemonic::Cmovnp,// Cmovnp_r16_rm16
	Mnemonic::Cmovnp,// Cmovnp_r32_rm32
	Mnemonic::Cmovnp,// Cmovnp_r64_rm64
	Mnemonic::Cmovl,// Cmovl_r16_rm16
	Mnemonic::Cmovl,// Cmovl_r32_rm32
	Mnemonic::Cmovl,// Cmovl_r64_rm64
	Mnemonic::Cmovge,// Cmovge_r16_rm16
	Mnemonic::Cmovge,// Cmovge_r32_rm32
	Mnemonic::Cmovge,// Cmovge_r64_rm64
	Mnemonic::Cmovle,// Cmovle_r16_rm16
	Mnemonic::Cmovle,// Cmovle_r32_rm32
	Mnemonic::Cmovle,// Cmovle_r64_rm64
	Mnemonic::Cmovg,// Cmovg_r16_rm16
	Mnemonic::Cmovg,// Cmovg_r32_rm32
	Mnemonic::Cmovg,// Cmovg_r64_rm64
	Mnemonic::Kandw,// VEX_Kandw_kr_kr_kr
	Mnemonic::Kandq,// VEX_Kandq_kr_kr_kr
	Mnemonic::Kandb,// VEX_Kandb_kr_kr_kr
	Mnemonic::Kandd,// VEX_Kandd_kr_kr_kr
	Mnemonic::Kandnw,// VEX_Kandnw_kr_kr_kr
	Mnemonic::Kandnq,// VEX_Kandnq_kr_kr_kr
	Mnemonic::Kandnb,// VEX_Kandnb_kr_kr_kr
	Mnemonic::Kandnd,// VEX_Kandnd_kr_kr_kr
	Mnemonic::Knotw,// VEX_Knotw_kr_kr
	Mnemonic::Knotq,// VEX_Knotq_kr_kr
	Mnemonic::Knotb,// VEX_Knotb_kr_kr
	Mnemonic::Knotd,// VEX_Knotd_kr_kr
	Mnemonic::Korw,// VEX_Korw_kr_kr_kr
	Mnemonic::Korq,// VEX_Korq_kr_kr_kr
	Mnemonic::Korb,// VEX_Korb_kr_kr_kr
	Mnemonic::Kord,// VEX_Kord_kr_kr_kr
	Mnemonic::Kxnorw,// VEX_Kxnorw_kr_kr_kr
	Mnemonic::Kxnorq,// VEX_Kxnorq_kr_kr_kr
	Mnemonic::Kxnorb,// VEX_Kxnorb_kr_kr_kr
	Mnemonic::Kxnord,// VEX_Kxnord_kr_kr_kr
	Mnemonic::Kxorw,// VEX_Kxorw_kr_kr_kr
	Mnemonic::Kxorq,// VEX_Kxorq_kr_kr_kr
	Mnemonic::Kxorb,// VEX_Kxorb_kr_kr_kr
	Mnemonic::Kxord,// VEX_Kxord_kr_kr_kr
	Mnemonic::Kaddw,// VEX_Kaddw_kr_kr_kr
	Mnemonic::Kaddq,// VEX_Kaddq_kr_kr_kr
	Mnemonic::Kaddb,// VEX_Kaddb_kr_kr_kr
	Mnemonic::Kaddd,// VEX_Kaddd_kr_kr_kr
	Mnemonic::Kunpckwd,// VEX_Kunpckwd_kr_kr_kr
	Mnemonic::Kunpckdq,// VEX_Kunpckdq_kr_kr_kr
	Mnemonic::Kunpckbw,// VEX_Kunpckbw_kr_kr_kr
	Mnemonic::Movmskps,// Movmskps_r32_xmm
	Mnemonic::Movmskps,// Movmskps_r64_xmm
	Mnemonic::Vmovmskps,// VEX_Vmovmskps_r32_xmm
	Mnemonic::Vmovmskps,// VEX_Vmovmskps_r64_xmm
	Mnemonic::Vmovmskps,// VEX_Vmovmskps_r32_ymm
	Mnemonic::Vmovmskps,// VEX_Vmovmskps_r64_ymm
	Mnemonic::Movmskpd,// Movmskpd_r32_xmm
	Mnemonic::Movmskpd,// Movmskpd_r64_xmm
	Mnemonic::Vmovmskpd,// VEX_Vmovmskpd_r32_xmm
	Mnemonic::Vmovmskpd,// VEX_Vmovmskpd_r64_xmm
	Mnemonic::Vmovmskpd,// VEX_Vmovmskpd_r32_ymm
	Mnemonic::Vmovmskpd,// VEX_Vmovmskpd_r64_ymm
	Mnemonic::Sqrtps,// Sqrtps_xmm_xmmm128
	Mnemonic::Vsqrtps,// VEX_Vsqrtps_xmm_xmmm128
	Mnemonic::Vsqrtps,// VEX_Vsqrtps_ymm_ymmm256
	Mnemonic::Vsqrtps,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	Mnemonic::Vsqrtps,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	Mnemonic::Vsqrtps,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	Mnemonic::Sqrtpd,// Sqrtpd_xmm_xmmm128
	Mnemonic::Vsqrtpd,// VEX_Vsqrtpd_xmm_xmmm128
	Mnemonic::Vsqrtpd,// VEX_Vsqrtpd_ymm_ymmm256
	Mnemonic::Vsqrtpd,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	Mnemonic::Vsqrtpd,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	Mnemonic::Vsqrtpd,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	Mnemonic::Sqrtss,// Sqrtss_xmm_xmmm32
	Mnemonic::Vsqrtss,// VEX_Vsqrtss_xmm_xmm_xmmm32
	Mnemonic::Vsqrtss,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Sqrtsd,// Sqrtsd_xmm_xmmm64
	Mnemonic::Vsqrtsd,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	Mnemonic::Vsqrtsd,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Rsqrtps,// Rsqrtps_xmm_xmmm128
	Mnemonic::Vrsqrtps,// VEX_Vrsqrtps_xmm_xmmm128
	Mnemonic::Vrsqrtps,// VEX_Vrsqrtps_ymm_ymmm256
	Mnemonic::Rsqrtss,// Rsqrtss_xmm_xmmm32
	Mnemonic::Vrsqrtss,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	Mnemonic::Rcpps,// Rcpps_xmm_xmmm128
	Mnemonic::Vrcpps,// VEX_Vrcpps_xmm_xmmm128
	Mnemonic::Vrcpps,// VEX_Vrcpps_ymm_ymmm256
	Mnemonic::Rcpss,// Rcpss_xmm_xmmm32
	Mnemonic::Vrcpss,// VEX_Vrcpss_xmm_xmm_xmmm32
	Mnemonic::Andps,// Andps_xmm_xmmm128
	Mnemonic::Vandps,// VEX_Vandps_xmm_xmm_xmmm128
	Mnemonic::Vandps,// VEX_Vandps_ymm_ymm_ymmm256
	Mnemonic::Vandps,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vandps,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vandps,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Andpd,// Andpd_xmm_xmmm128
	Mnemonic::Vandpd,// VEX_Vandpd_xmm_xmm_xmmm128
	Mnemonic::Vandpd,// VEX_Vandpd_ymm_ymm_ymmm256
	Mnemonic::Vandpd,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vandpd,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vandpd,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Andnps,// Andnps_xmm_xmmm128
	Mnemonic::Vandnps,// VEX_Vandnps_xmm_xmm_xmmm128
	Mnemonic::Vandnps,// VEX_Vandnps_ymm_ymm_ymmm256
	Mnemonic::Vandnps,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vandnps,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vandnps,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Andnpd,// Andnpd_xmm_xmmm128
	Mnemonic::Vandnpd,// VEX_Vandnpd_xmm_xmm_xmmm128
	Mnemonic::Vandnpd,// VEX_Vandnpd_ymm_ymm_ymmm256
	Mnemonic::Vandnpd,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vandnpd,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vandnpd,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Orps,// Orps_xmm_xmmm128
	Mnemonic::Vorps,// VEX_Vorps_xmm_xmm_xmmm128
	Mnemonic::Vorps,// VEX_Vorps_ymm_ymm_ymmm256
	Mnemonic::Vorps,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vorps,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vorps,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Orpd,// Orpd_xmm_xmmm128
	Mnemonic::Vorpd,// VEX_Vorpd_xmm_xmm_xmmm128
	Mnemonic::Vorpd,// VEX_Vorpd_ymm_ymm_ymmm256
	Mnemonic::Vorpd,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vorpd,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vorpd,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Xorps,// Xorps_xmm_xmmm128
	Mnemonic::Vxorps,// VEX_Vxorps_xmm_xmm_xmmm128
	Mnemonic::Vxorps,// VEX_Vxorps_ymm_ymm_ymmm256
	Mnemonic::Vxorps,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vxorps,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vxorps,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Xorpd,// Xorpd_xmm_xmmm128
	Mnemonic::Vxorpd,// VEX_Vxorpd_xmm_xmm_xmmm128
	Mnemonic::Vxorpd,// VEX_Vxorpd_ymm_ymm_ymmm256
	Mnemonic::Vxorpd,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vxorpd,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vxorpd,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Addps,// Addps_xmm_xmmm128
	Mnemonic::Vaddps,// VEX_Vaddps_xmm_xmm_xmmm128
	Mnemonic::Vaddps,// VEX_Vaddps_ymm_ymm_ymmm256
	Mnemonic::Vaddps,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vaddps,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vaddps,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Addpd,// Addpd_xmm_xmmm128
	Mnemonic::Vaddpd,// VEX_Vaddpd_xmm_xmm_xmmm128
	Mnemonic::Vaddpd,// VEX_Vaddpd_ymm_ymm_ymmm256
	Mnemonic::Vaddpd,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vaddpd,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vaddpd,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Addss,// Addss_xmm_xmmm32
	Mnemonic::Vaddss,// VEX_Vaddss_xmm_xmm_xmmm32
	Mnemonic::Vaddss,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Addsd,// Addsd_xmm_xmmm64
	Mnemonic::Vaddsd,// VEX_Vaddsd_xmm_xmm_xmmm64
	Mnemonic::Vaddsd,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Mulps,// Mulps_xmm_xmmm128
	Mnemonic::Vmulps,// VEX_Vmulps_xmm_xmm_xmmm128
	Mnemonic::Vmulps,// VEX_Vmulps_ymm_ymm_ymmm256
	Mnemonic::Vmulps,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vmulps,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vmulps,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Mulpd,// Mulpd_xmm_xmmm128
	Mnemonic::Vmulpd,// VEX_Vmulpd_xmm_xmm_xmmm128
	Mnemonic::Vmulpd,// VEX_Vmulpd_ymm_ymm_ymmm256
	Mnemonic::Vmulpd,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vmulpd,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vmulpd,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Mulss,// Mulss_xmm_xmmm32
	Mnemonic::Vmulss,// VEX_Vmulss_xmm_xmm_xmmm32
	Mnemonic::Vmulss,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Mulsd,// Mulsd_xmm_xmmm64
	Mnemonic::Vmulsd,// VEX_Vmulsd_xmm_xmm_xmmm64
	Mnemonic::Vmulsd,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Cvtps2pd,// Cvtps2pd_xmm_xmmm64
	Mnemonic::Vcvtps2pd,// VEX_Vcvtps2pd_xmm_xmmm64
	Mnemonic::Vcvtps2pd,// VEX_Vcvtps2pd_ymm_xmmm128
	Mnemonic::Vcvtps2pd,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	Mnemonic::Vcvtps2pd,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	Mnemonic::Vcvtps2pd,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	Mnemonic::Cvtpd2ps,// Cvtpd2ps_xmm_xmmm128
	Mnemonic::Vcvtpd2ps,// VEX_Vcvtpd2ps_xmm_xmmm128
	Mnemonic::Vcvtpd2ps,// VEX_Vcvtpd2ps_xmm_ymmm256
	Mnemonic::Vcvtpd2ps,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2ps,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2ps,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	Mnemonic::Cvtss2sd,// Cvtss2sd_xmm_xmmm32
	Mnemonic::Vcvtss2sd,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	Mnemonic::Vcvtss2sd,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Cvtsd2ss,// Cvtsd2ss_xmm_xmmm64
	Mnemonic::Vcvtsd2ss,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	Mnemonic::Vcvtsd2ss,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Cvtdq2ps,// Cvtdq2ps_xmm_xmmm128
	Mnemonic::Vcvtdq2ps,// VEX_Vcvtdq2ps_xmm_xmmm128
	Mnemonic::Vcvtdq2ps,// VEX_Vcvtdq2ps_ymm_ymmm256
	Mnemonic::Vcvtdq2ps,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtdq2ps,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	Mnemonic::Vcvtdq2ps,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	Mnemonic::Vcvtqq2ps,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtqq2ps,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtqq2ps,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	Mnemonic::Cvtps2dq,// Cvtps2dq_xmm_xmmm128
	Mnemonic::Vcvtps2dq,// VEX_Vcvtps2dq_xmm_xmmm128
	Mnemonic::Vcvtps2dq,// VEX_Vcvtps2dq_ymm_ymmm256
	Mnemonic::Vcvtps2dq,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtps2dq,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	Mnemonic::Vcvtps2dq,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	Mnemonic::Cvttps2dq,// Cvttps2dq_xmm_xmmm128
	Mnemonic::Vcvttps2dq,// VEX_Vcvttps2dq_xmm_xmmm128
	Mnemonic::Vcvttps2dq,// VEX_Vcvttps2dq_ymm_ymmm256
	Mnemonic::Vcvttps2dq,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	Mnemonic::Vcvttps2dq,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	Mnemonic::Vcvttps2dq,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	Mnemonic::Subps,// Subps_xmm_xmmm128
	Mnemonic::Vsubps,// VEX_Vsubps_xmm_xmm_xmmm128
	Mnemonic::Vsubps,// VEX_Vsubps_ymm_ymm_ymmm256
	Mnemonic::Vsubps,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vsubps,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vsubps,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Subpd,// Subpd_xmm_xmmm128
	Mnemonic::Vsubpd,// VEX_Vsubpd_xmm_xmm_xmmm128
	Mnemonic::Vsubpd,// VEX_Vsubpd_ymm_ymm_ymmm256
	Mnemonic::Vsubpd,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vsubpd,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vsubpd,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Subss,// Subss_xmm_xmmm32
	Mnemonic::Vsubss,// VEX_Vsubss_xmm_xmm_xmmm32
	Mnemonic::Vsubss,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Subsd,// Subsd_xmm_xmmm64
	Mnemonic::Vsubsd,// VEX_Vsubsd_xmm_xmm_xmmm64
	Mnemonic::Vsubsd,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Minps,// Minps_xmm_xmmm128
	Mnemonic::Vminps,// VEX_Vminps_xmm_xmm_xmmm128
	Mnemonic::Vminps,// VEX_Vminps_ymm_ymm_ymmm256
	Mnemonic::Vminps,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vminps,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vminps,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	Mnemonic::Minpd,// Minpd_xmm_xmmm128
	Mnemonic::Vminpd,// VEX_Vminpd_xmm_xmm_xmmm128
	Mnemonic::Vminpd,// VEX_Vminpd_ymm_ymm_ymmm256
	Mnemonic::Vminpd,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vminpd,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vminpd,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	Mnemonic::Minss,// Minss_xmm_xmmm32
	Mnemonic::Vminss,// VEX_Vminss_xmm_xmm_xmmm32
	Mnemonic::Vminss,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Minsd,// Minsd_xmm_xmmm64
	Mnemonic::Vminsd,// VEX_Vminsd_xmm_xmm_xmmm64
	Mnemonic::Vminsd,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	Mnemonic::Divps,// Divps_xmm_xmmm128
	Mnemonic::Vdivps,// VEX_Vdivps_xmm_xmm_xmmm128
	Mnemonic::Vdivps,// VEX_Vdivps_ymm_ymm_ymmm256
	Mnemonic::Vdivps,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vdivps,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vdivps,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Divpd,// Divpd_xmm_xmmm128
	Mnemonic::Vdivpd,// VEX_Vdivpd_xmm_xmm_xmmm128
	Mnemonic::Vdivpd,// VEX_Vdivpd_ymm_ymm_ymmm256
	Mnemonic::Vdivpd,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vdivpd,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vdivpd,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Divss,// Divss_xmm_xmmm32
	Mnemonic::Vdivss,// VEX_Vdivss_xmm_xmm_xmmm32
	Mnemonic::Vdivss,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Divsd,// Divsd_xmm_xmmm64
	Mnemonic::Vdivsd,// VEX_Vdivsd_xmm_xmm_xmmm64
	Mnemonic::Vdivsd,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Maxps,// Maxps_xmm_xmmm128
	Mnemonic::Vmaxps,// VEX_Vmaxps_xmm_xmm_xmmm128
	Mnemonic::Vmaxps,// VEX_Vmaxps_ymm_ymm_ymmm256
	Mnemonic::Vmaxps,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vmaxps,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vmaxps,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	Mnemonic::Maxpd,// Maxpd_xmm_xmmm128
	Mnemonic::Vmaxpd,// VEX_Vmaxpd_xmm_xmm_xmmm128
	Mnemonic::Vmaxpd,// VEX_Vmaxpd_ymm_ymm_ymmm256
	Mnemonic::Vmaxpd,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vmaxpd,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vmaxpd,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	Mnemonic::Maxss,// Maxss_xmm_xmmm32
	Mnemonic::Vmaxss,// VEX_Vmaxss_xmm_xmm_xmmm32
	Mnemonic::Vmaxss,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Maxsd,// Maxsd_xmm_xmmm64
	Mnemonic::Vmaxsd,// VEX_Vmaxsd_xmm_xmm_xmmm64
	Mnemonic::Vmaxsd,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	Mnemonic::Punpcklbw,// Punpcklbw_mm_mmm32
	Mnemonic::Punpcklbw,// Punpcklbw_xmm_xmmm128
	Mnemonic::Vpunpcklbw,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	Mnemonic::Vpunpcklbw,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	Mnemonic::Vpunpcklbw,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpunpcklbw,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpunpcklbw,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	Mnemonic::Punpcklwd,// Punpcklwd_mm_mmm32
	Mnemonic::Punpcklwd,// Punpcklwd_xmm_xmmm128
	Mnemonic::Vpunpcklwd,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	Mnemonic::Vpunpcklwd,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	Mnemonic::Vpunpcklwd,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpunpcklwd,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpunpcklwd,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	Mnemonic::Punpckldq,// Punpckldq_mm_mmm32
	Mnemonic::Punpckldq,// Punpckldq_xmm_xmmm128
	Mnemonic::Vpunpckldq,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	Mnemonic::Vpunpckldq,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	Mnemonic::Vpunpckldq,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpunpckldq,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpunpckldq,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Packsswb,// Packsswb_mm_mmm64
	Mnemonic::Packsswb,// Packsswb_xmm_xmmm128
	Mnemonic::Vpacksswb,// VEX_Vpacksswb_xmm_xmm_xmmm128
	Mnemonic::Vpacksswb,// VEX_Vpacksswb_ymm_ymm_ymmm256
	Mnemonic::Vpacksswb,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpacksswb,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpacksswb,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	Mnemonic::Pcmpgtb,// Pcmpgtb_mm_mmm64
	Mnemonic::Pcmpgtb,// Pcmpgtb_xmm_xmmm128
	Mnemonic::Vpcmpgtb,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	Mnemonic::Vpcmpgtb,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	Mnemonic::Vpcmpgtb,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	Mnemonic::Vpcmpgtb,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	Mnemonic::Vpcmpgtb,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	Mnemonic::Pcmpgtw,// Pcmpgtw_mm_mmm64
	Mnemonic::Pcmpgtw,// Pcmpgtw_xmm_xmmm128
	Mnemonic::Vpcmpgtw,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	Mnemonic::Vpcmpgtw,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	Mnemonic::Vpcmpgtw,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	Mnemonic::Vpcmpgtw,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	Mnemonic::Vpcmpgtw,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	Mnemonic::Pcmpgtd,// Pcmpgtd_mm_mmm64
	Mnemonic::Pcmpgtd,// Pcmpgtd_xmm_xmmm128
	Mnemonic::Vpcmpgtd,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	Mnemonic::Vpcmpgtd,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	Mnemonic::Vpcmpgtd,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	Mnemonic::Vpcmpgtd,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	Mnemonic::Vpcmpgtd,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	Mnemonic::Packuswb,// Packuswb_mm_mmm64
	Mnemonic::Packuswb,// Packuswb_xmm_xmmm128
	Mnemonic::Vpackuswb,// VEX_Vpackuswb_xmm_xmm_xmmm128
	Mnemonic::Vpackuswb,// VEX_Vpackuswb_ymm_ymm_ymmm256
	Mnemonic::Vpackuswb,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpackuswb,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpackuswb,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	Mnemonic::Punpckhbw,// Punpckhbw_mm_mmm64
	Mnemonic::Punpckhbw,// Punpckhbw_xmm_xmmm128
	Mnemonic::Vpunpckhbw,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	Mnemonic::Vpunpckhbw,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	Mnemonic::Vpunpckhbw,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpunpckhbw,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpunpckhbw,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	Mnemonic::Punpckhwd,// Punpckhwd_mm_mmm64
	Mnemonic::Punpckhwd,// Punpckhwd_xmm_xmmm128
	Mnemonic::Vpunpckhwd,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	Mnemonic::Vpunpckhwd,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	Mnemonic::Vpunpckhwd,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpunpckhwd,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpunpckhwd,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	Mnemonic::Punpckhdq,// Punpckhdq_mm_mmm64
	Mnemonic::Punpckhdq,// Punpckhdq_xmm_xmmm128
	Mnemonic::Vpunpckhdq,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	Mnemonic::Vpunpckhdq,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	Mnemonic::Vpunpckhdq,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpunpckhdq,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpunpckhdq,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Packssdw,// Packssdw_mm_mmm64
	Mnemonic::Packssdw,// Packssdw_xmm_xmmm128
	Mnemonic::Vpackssdw,// VEX_Vpackssdw_xmm_xmm_xmmm128
	Mnemonic::Vpackssdw,// VEX_Vpackssdw_ymm_ymm_ymmm256
	Mnemonic::Vpackssdw,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpackssdw,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpackssdw,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Punpcklqdq,// Punpcklqdq_xmm_xmmm128
	Mnemonic::Vpunpcklqdq,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	Mnemonic::Vpunpcklqdq,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	Mnemonic::Vpunpcklqdq,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpunpcklqdq,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpunpcklqdq,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Punpckhqdq,// Punpckhqdq_xmm_xmmm128
	Mnemonic::Vpunpckhqdq,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	Mnemonic::Vpunpckhqdq,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	Mnemonic::Vpunpckhqdq,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpunpckhqdq,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpunpckhqdq,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Movd,// Movd_mm_rm32
	Mnemonic::Movq,// Movq_mm_rm64
	Mnemonic::Movd,// Movd_xmm_rm32
	Mnemonic::Movq,// Movq_xmm_rm64
	Mnemonic::Vmovd,// VEX_Vmovd_xmm_rm32
	Mnemonic::Vmovq,// VEX_Vmovq_xmm_rm64
	Mnemonic::Vmovd,// EVEX_Vmovd_xmm_rm32
	Mnemonic::Vmovq,// EVEX_Vmovq_xmm_rm64
	Mnemonic::Movq,// Movq_mm_mmm64
	Mnemonic::Movdqa,// Movdqa_xmm_xmmm128
	Mnemonic::Vmovdqa,// VEX_Vmovdqa_xmm_xmmm128
	Mnemonic::Vmovdqa,// VEX_Vmovdqa_ymm_ymmm256
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	Mnemonic::Movdqu,// Movdqu_xmm_xmmm128
	Mnemonic::Vmovdqu,// VEX_Vmovdqu_xmm_xmmm128
	Mnemonic::Vmovdqu,// VEX_Vmovdqu_ymm_ymmm256
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	Mnemonic::Pshufw,// Pshufw_mm_mmm64_imm8
	Mnemonic::Pshufd,// Pshufd_xmm_xmmm128_imm8
	Mnemonic::Vpshufd,// VEX_Vpshufd_xmm_xmmm128_imm8
	Mnemonic::Vpshufd,// VEX_Vpshufd_ymm_ymmm256_imm8
	Mnemonic::Vpshufd,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vpshufd,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vpshufd,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Pshufhw,// Pshufhw_xmm_xmmm128_imm8
	Mnemonic::Vpshufhw,// VEX_Vpshufhw_xmm_xmmm128_imm8
	Mnemonic::Vpshufhw,// VEX_Vpshufhw_ymm_ymmm256_imm8
	Mnemonic::Vpshufhw,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	Mnemonic::Vpshufhw,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	Mnemonic::Vpshufhw,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	Mnemonic::Pshuflw,// Pshuflw_xmm_xmmm128_imm8
	Mnemonic::Vpshuflw,// VEX_Vpshuflw_xmm_xmmm128_imm8
	Mnemonic::Vpshuflw,// VEX_Vpshuflw_ymm_ymmm256_imm8
	Mnemonic::Vpshuflw,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	Mnemonic::Vpshuflw,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	Mnemonic::Vpshuflw,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	Mnemonic::Psrlw,// Psrlw_mm_imm8
	Mnemonic::Psrlw,// Psrlw_xmm_imm8
	Mnemonic::Vpsrlw,// VEX_Vpsrlw_xmm_xmm_imm8
	Mnemonic::Vpsrlw,// VEX_Vpsrlw_ymm_ymm_imm8
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	Mnemonic::Psraw,// Psraw_mm_imm8
	Mnemonic::Psraw,// Psraw_xmm_imm8
	Mnemonic::Vpsraw,// VEX_Vpsraw_xmm_xmm_imm8
	Mnemonic::Vpsraw,// VEX_Vpsraw_ymm_ymm_imm8
	Mnemonic::Vpsraw,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	Mnemonic::Vpsraw,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	Mnemonic::Vpsraw,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	Mnemonic::Psllw,// Psllw_mm_imm8
	Mnemonic::Psllw,// Psllw_xmm_imm8
	Mnemonic::Vpsllw,// VEX_Vpsllw_xmm_xmm_imm8
	Mnemonic::Vpsllw,// VEX_Vpsllw_ymm_ymm_imm8
	Mnemonic::Vpsllw,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	Mnemonic::Vpsllw,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	Mnemonic::Vpsllw,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	Mnemonic::Vprord,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vprord,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vprord,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Vprorq,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vprorq,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vprorq,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Vprold,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vprold,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vprold,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Vprolq,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vprolq,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vprolq,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Psrld,// Psrld_mm_imm8
	Mnemonic::Psrld,// Psrld_xmm_imm8
	Mnemonic::Vpsrld,// VEX_Vpsrld_xmm_xmm_imm8
	Mnemonic::Vpsrld,// VEX_Vpsrld_ymm_ymm_imm8
	Mnemonic::Vpsrld,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vpsrld,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vpsrld,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Psrad,// Psrad_mm_imm8
	Mnemonic::Psrad,// Psrad_xmm_imm8
	Mnemonic::Vpsrad,// VEX_Vpsrad_xmm_xmm_imm8
	Mnemonic::Vpsrad,// VEX_Vpsrad_ymm_ymm_imm8
	Mnemonic::Vpsrad,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vpsrad,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vpsrad,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Vpsraq,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vpsraq,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpsraq,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Pslld,// Pslld_mm_imm8
	Mnemonic::Pslld,// Pslld_xmm_imm8
	Mnemonic::Vpslld,// VEX_Vpslld_xmm_xmm_imm8
	Mnemonic::Vpslld,// VEX_Vpslld_ymm_ymm_imm8
	Mnemonic::Vpslld,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vpslld,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vpslld,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Psrlq,// Psrlq_mm_imm8
	Mnemonic::Psrlq,// Psrlq_xmm_imm8
	Mnemonic::Vpsrlq,// VEX_Vpsrlq_xmm_xmm_imm8
	Mnemonic::Vpsrlq,// VEX_Vpsrlq_ymm_ymm_imm8
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Psrldq,// Psrldq_xmm_imm8
	Mnemonic::Vpsrldq,// VEX_Vpsrldq_xmm_xmm_imm8
	Mnemonic::Vpsrldq,// VEX_Vpsrldq_ymm_ymm_imm8
	Mnemonic::Vpsrldq,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	Mnemonic::Vpsrldq,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	Mnemonic::Vpsrldq,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	Mnemonic::Psllq,// Psllq_mm_imm8
	Mnemonic::Psllq,// Psllq_xmm_imm8
	Mnemonic::Vpsllq,// VEX_Vpsllq_xmm_xmm_imm8
	Mnemonic::Vpsllq,// VEX_Vpsllq_ymm_ymm_imm8
	Mnemonic::Vpsllq,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vpsllq,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpsllq,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Pslldq,// Pslldq_xmm_imm8
	Mnemonic::Vpslldq,// VEX_Vpslldq_xmm_xmm_imm8
	Mnemonic::Vpslldq,// VEX_Vpslldq_ymm_ymm_imm8
	Mnemonic::Vpslldq,// EVEX_Vpslldq_xmm_xmmm128_imm8
	Mnemonic::Vpslldq,// EVEX_Vpslldq_ymm_ymmm256_imm8
	Mnemonic::Vpslldq,// EVEX_Vpslldq_zmm_zmmm512_imm8
	Mnemonic::Pcmpeqb,// Pcmpeqb_mm_mmm64
	Mnemonic::Pcmpeqb,// Pcmpeqb_xmm_xmmm128
	Mnemonic::Vpcmpeqb,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	Mnemonic::Vpcmpeqb,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	Mnemonic::Vpcmpeqb,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	Mnemonic::Vpcmpeqb,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	Mnemonic::Vpcmpeqb,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	Mnemonic::Pcmpeqw,// Pcmpeqw_mm_mmm64
	Mnemonic::Pcmpeqw,// Pcmpeqw_xmm_xmmm128
	Mnemonic::Vpcmpeqw,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	Mnemonic::Vpcmpeqw,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	Mnemonic::Vpcmpeqw,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	Mnemonic::Vpcmpeqw,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	Mnemonic::Vpcmpeqw,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	Mnemonic::Pcmpeqd,// Pcmpeqd_mm_mmm64
	Mnemonic::Pcmpeqd,// Pcmpeqd_xmm_xmmm128
	Mnemonic::Vpcmpeqd,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	Mnemonic::Vpcmpeqd,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	Mnemonic::Vpcmpeqd,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	Mnemonic::Vpcmpeqd,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	Mnemonic::Vpcmpeqd,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	Mnemonic::Emms,// Emms
	Mnemonic::Vzeroupper,// VEX_Vzeroupper
	Mnemonic::Vzeroall,// VEX_Vzeroall
	Mnemonic::Vmread,// Vmread_rm32_r32
	Mnemonic::Vmread,// Vmread_rm64_r64
	Mnemonic::Vcvttps2udq,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	Mnemonic::Vcvttps2udq,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	Mnemonic::Vcvttps2udq,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	Mnemonic::Vcvttpd2udq,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvttpd2udq,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	Mnemonic::Vcvttpd2udq,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	Mnemonic::Extrq,// Extrq_xmm_imm8_imm8
	Mnemonic::Vcvttps2uqq,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	Mnemonic::Vcvttps2uqq,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	Mnemonic::Vcvttps2uqq,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	Mnemonic::Vcvttpd2uqq,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvttpd2uqq,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	Mnemonic::Vcvttpd2uqq,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	Mnemonic::Vcvttss2usi,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	Mnemonic::Vcvttss2usi,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	Mnemonic::Insertq,// Insertq_xmm_xmm_imm8_imm8
	Mnemonic::Vcvttsd2usi,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	Mnemonic::Vcvttsd2usi,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	Mnemonic::Vmwrite,// Vmwrite_r32_rm32
	Mnemonic::Vmwrite,// Vmwrite_r64_rm64
	Mnemonic::Vcvtps2udq,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtps2udq,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	Mnemonic::Vcvtps2udq,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	Mnemonic::Vcvtpd2udq,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2udq,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2udq,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	Mnemonic::Extrq,// Extrq_xmm_xmm
	Mnemonic::Vcvtps2uqq,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	Mnemonic::Vcvtps2uqq,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	Mnemonic::Vcvtps2uqq,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	Mnemonic::Vcvtpd2uqq,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2uqq,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2uqq,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtss2usi,// EVEX_Vcvtss2usi_r32_xmmm32_er
	Mnemonic::Vcvtss2usi,// EVEX_Vcvtss2usi_r64_xmmm32_er
	Mnemonic::Insertq,// Insertq_xmm_xmm
	Mnemonic::Vcvtsd2usi,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	Mnemonic::Vcvtsd2usi,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	Mnemonic::Vcvttps2qq,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	Mnemonic::Vcvttps2qq,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	Mnemonic::Vcvttps2qq,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	Mnemonic::Vcvttpd2qq,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvttpd2qq,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	Mnemonic::Vcvttpd2qq,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	Mnemonic::Vcvtudq2pd,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	Mnemonic::Vcvtudq2pd,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	Mnemonic::Vcvtudq2pd,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	Mnemonic::Vcvtuqq2pd,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtuqq2pd,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	Mnemonic::Vcvtuqq2pd,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtudq2ps,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtudq2ps,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	Mnemonic::Vcvtudq2ps,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	Mnemonic::Vcvtuqq2ps,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtuqq2ps,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtuqq2ps,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	Mnemonic::Vcvtps2qq,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	Mnemonic::Vcvtps2qq,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	Mnemonic::Vcvtps2qq,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	Mnemonic::Vcvtpd2qq,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2qq,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2qq,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtusi2ss,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	Mnemonic::Vcvtusi2ss,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	Mnemonic::Vcvtusi2sd,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	Mnemonic::Vcvtusi2sd,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	Mnemonic::Haddpd,// Haddpd_xmm_xmmm128
	Mnemonic::Vhaddpd,// VEX_Vhaddpd_xmm_xmm_xmmm128
	Mnemonic::Vhaddpd,// VEX_Vhaddpd_ymm_ymm_ymmm256
	Mnemonic::Haddps,// Haddps_xmm_xmmm128
	Mnemonic::Vhaddps,// VEX_Vhaddps_xmm_xmm_xmmm128
	Mnemonic::Vhaddps,// VEX_Vhaddps_ymm_ymm_ymmm256
	Mnemonic::Hsubpd,// Hsubpd_xmm_xmmm128
	Mnemonic::Vhsubpd,// VEX_Vhsubpd_xmm_xmm_xmmm128
	Mnemonic::Vhsubpd,// VEX_Vhsubpd_ymm_ymm_ymmm256
	Mnemonic::Hsubps,// Hsubps_xmm_xmmm128
	Mnemonic::Vhsubps,// VEX_Vhsubps_xmm_xmm_xmmm128
	Mnemonic::Vhsubps,// VEX_Vhsubps_ymm_ymm_ymmm256
	Mnemonic::Movd,// Movd_rm32_mm
	Mnemonic::Movq,// Movq_rm64_mm
	Mnemonic::Movd,// Movd_rm32_xmm
	Mnemonic::Movq,// Movq_rm64_xmm
	Mnemonic::Vmovd,// VEX_Vmovd_rm32_xmm
	Mnemonic::Vmovq,// VEX_Vmovq_rm64_xmm
	Mnemonic::Vmovd,// EVEX_Vmovd_rm32_xmm
	Mnemonic::Vmovq,// EVEX_Vmovq_rm64_xmm
	Mnemonic::Movq,// Movq_xmm_xmmm64
	Mnemonic::Vmovq,// VEX_Vmovq_xmm_xmmm64
	Mnemonic::Vmovq,// EVEX_Vmovq_xmm_xmmm64
	Mnemonic::Movq,// Movq_mmm64_mm
	Mnemonic::Movdqa,// Movdqa_xmmm128_xmm
	Mnemonic::Vmovdqa,// VEX_Vmovdqa_xmmm128_xmm
	Mnemonic::Vmovdqa,// VEX_Vmovdqa_ymmm256_ymm
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	Mnemonic::Vmovdqa32,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	Mnemonic::Vmovdqa64,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	Mnemonic::Movdqu,// Movdqu_xmmm128_xmm
	Mnemonic::Vmovdqu,// VEX_Vmovdqu_xmmm128_xmm
	Mnemonic::Vmovdqu,// VEX_Vmovdqu_ymmm256_ymm
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	Mnemonic::Vmovdqu32,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	Mnemonic::Vmovdqu64,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	Mnemonic::Vmovdqu8,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	Mnemonic::Vmovdqu16,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	Mnemonic::Jo,// Jo_rel16
	Mnemonic::Jo,// Jo_rel32_32
	Mnemonic::Jo,// Jo_rel32_64
	Mnemonic::Jno,// Jno_rel16
	Mnemonic::Jno,// Jno_rel32_32
	Mnemonic::Jno,// Jno_rel32_64
	Mnemonic::Jb,// Jb_rel16
	Mnemonic::Jb,// Jb_rel32_32
	Mnemonic::Jb,// Jb_rel32_64
	Mnemonic::Jae,// Jae_rel16
	Mnemonic::Jae,// Jae_rel32_32
	Mnemonic::Jae,// Jae_rel32_64
	Mnemonic::Je,// Je_rel16
	Mnemonic::Je,// Je_rel32_32
	Mnemonic::Je,// Je_rel32_64
	Mnemonic::Jne,// Jne_rel16
	Mnemonic::Jne,// Jne_rel32_32
	Mnemonic::Jne,// Jne_rel32_64
	Mnemonic::Jbe,// Jbe_rel16
	Mnemonic::Jbe,// Jbe_rel32_32
	Mnemonic::Jbe,// Jbe_rel32_64
	Mnemonic::Ja,// Ja_rel16
	Mnemonic::Ja,// Ja_rel32_32
	Mnemonic::Ja,// Ja_rel32_64
	Mnemonic::Js,// Js_rel16
	Mnemonic::Js,// Js_rel32_32
	Mnemonic::Js,// Js_rel32_64
	Mnemonic::Jns,// Jns_rel16
	Mnemonic::Jns,// Jns_rel32_32
	Mnemonic::Jns,// Jns_rel32_64
	Mnemonic::Jp,// Jp_rel16
	Mnemonic::Jp,// Jp_rel32_32
	Mnemonic::Jp,// Jp_rel32_64
	Mnemonic::Jnp,// Jnp_rel16
	Mnemonic::Jnp,// Jnp_rel32_32
	Mnemonic::Jnp,// Jnp_rel32_64
	Mnemonic::Jl,// Jl_rel16
	Mnemonic::Jl,// Jl_rel32_32
	Mnemonic::Jl,// Jl_rel32_64
	Mnemonic::Jge,// Jge_rel16
	Mnemonic::Jge,// Jge_rel32_32
	Mnemonic::Jge,// Jge_rel32_64
	Mnemonic::Jle,// Jle_rel16
	Mnemonic::Jle,// Jle_rel32_32
	Mnemonic::Jle,// Jle_rel32_64
	Mnemonic::Jg,// Jg_rel16
	Mnemonic::Jg,// Jg_rel32_32
	Mnemonic::Jg,// Jg_rel32_64
	Mnemonic::Seto,// Seto_rm8
	Mnemonic::Setno,// Setno_rm8
	Mnemonic::Setb,// Setb_rm8
	Mnemonic::Setae,// Setae_rm8
	Mnemonic::Sete,// Sete_rm8
	Mnemonic::Setne,// Setne_rm8
	Mnemonic::Setbe,// Setbe_rm8
	Mnemonic::Seta,// Seta_rm8
	Mnemonic::Sets,// Sets_rm8
	Mnemonic::Setns,// Setns_rm8
	Mnemonic::Setp,// Setp_rm8
	Mnemonic::Setnp,// Setnp_rm8
	Mnemonic::Setl,// Setl_rm8
	Mnemonic::Setge,// Setge_rm8
	Mnemonic::Setle,// Setle_rm8
	Mnemonic::Setg,// Setg_rm8
	Mnemonic::Kmovw,// VEX_Kmovw_kr_km16
	Mnemonic::Kmovq,// VEX_Kmovq_kr_km64
	Mnemonic::Kmovb,// VEX_Kmovb_kr_km8
	Mnemonic::Kmovd,// VEX_Kmovd_kr_km32
	Mnemonic::Kmovw,// VEX_Kmovw_m16_kr
	Mnemonic::Kmovq,// VEX_Kmovq_m64_kr
	Mnemonic::Kmovb,// VEX_Kmovb_m8_kr
	Mnemonic::Kmovd,// VEX_Kmovd_m32_kr
	Mnemonic::Kmovw,// VEX_Kmovw_kr_r32
	Mnemonic::Kmovb,// VEX_Kmovb_kr_r32
	Mnemonic::Kmovd,// VEX_Kmovd_kr_r32
	Mnemonic::Kmovq,// VEX_Kmovq_kr_r64
	Mnemonic::Kmovw,// VEX_Kmovw_r32_kr
	Mnemonic::Kmovb,// VEX_Kmovb_r32_kr
	Mnemonic::Kmovd,// VEX_Kmovd_r32_kr
	Mnemonic::Kmovq,// VEX_Kmovq_r64_kr
	Mnemonic::Kortestw,// VEX_Kortestw_kr_kr
	Mnemonic::Kortestq,// VEX_Kortestq_kr_kr
	Mnemonic::Kortestb,// VEX_Kortestb_kr_kr
	Mnemonic::Kortestd,// VEX_Kortestd_kr_kr
	Mnemonic::Ktestw,// VEX_Ktestw_kr_kr
	Mnemonic::Ktestq,// VEX_Ktestq_kr_kr
	Mnemonic::Ktestb,// VEX_Ktestb_kr_kr
	Mnemonic::Ktestd,// VEX_Ktestd_kr_kr
	Mnemonic::Push,// Pushw_FS
	Mnemonic::Push,// Pushd_FS
	Mnemonic::Push,// Pushq_FS
	Mnemonic::Pop,// Popw_FS
	Mnemonic::Pop,// Popd_FS
	Mnemonic::Pop,// Popq_FS
	Mnemonic::Cpuid,// Cpuid
	Mnemonic::Bt,// Bt_rm16_r16
	Mnemonic::Bt,// Bt_rm32_r32
	Mnemonic::Bt,// Bt_rm64_r64
	Mnemonic::Shld,// Shld_rm16_r16_imm8
	Mnemonic::Shld,// Shld_rm32_r32_imm8
	Mnemonic::Shld,// Shld_rm64_r64_imm8
	Mnemonic::Shld,// Shld_rm16_r16_CL
	Mnemonic::Shld,// Shld_rm32_r32_CL
	Mnemonic::Shld,// Shld_rm64_r64_CL
	Mnemonic::Montmul,// Montmul_16
	Mnemonic::Montmul,// Montmul_32
	Mnemonic::Montmul,// Montmul_64
	Mnemonic::Xsha1,// Xsha1_16
	Mnemonic::Xsha1,// Xsha1_32
	Mnemonic::Xsha1,// Xsha1_64
	Mnemonic::Xsha256,// Xsha256_16
	Mnemonic::Xsha256,// Xsha256_32
	Mnemonic::Xsha256,// Xsha256_64
	Mnemonic::Xbts,// Xbts_r16_rm16
	Mnemonic::Xbts,// Xbts_r32_rm32
	Mnemonic::Xstore,// Xstore_16
	Mnemonic::Xstore,// Xstore_32
	Mnemonic::Xstore,// Xstore_64
	Mnemonic::Xcryptecb,// Xcryptecb_16
	Mnemonic::Xcryptecb,// Xcryptecb_32
	Mnemonic::Xcryptecb,// Xcryptecb_64
	Mnemonic::Xcryptcbc,// Xcryptcbc_16
	Mnemonic::Xcryptcbc,// Xcryptcbc_32
	Mnemonic::Xcryptcbc,// Xcryptcbc_64
	Mnemonic::Xcryptctr,// Xcryptctr_16
	Mnemonic::Xcryptctr,// Xcryptctr_32
	Mnemonic::Xcryptctr,// Xcryptctr_64
	Mnemonic::Xcryptcfb,// Xcryptcfb_16
	Mnemonic::Xcryptcfb,// Xcryptcfb_32
	Mnemonic::Xcryptcfb,// Xcryptcfb_64
	Mnemonic::Xcryptofb,// Xcryptofb_16
	Mnemonic::Xcryptofb,// Xcryptofb_32
	Mnemonic::Xcryptofb,// Xcryptofb_64
	Mnemonic::Ibts,// Ibts_rm16_r16
	Mnemonic::Ibts,// Ibts_rm32_r32
	Mnemonic::Cmpxchg,// Cmpxchg486_rm8_r8
	Mnemonic::Cmpxchg,// Cmpxchg486_rm16_r16
	Mnemonic::Cmpxchg,// Cmpxchg486_rm32_r32
	Mnemonic::Push,// Pushw_GS
	Mnemonic::Push,// Pushd_GS
	Mnemonic::Push,// Pushq_GS
	Mnemonic::Pop,// Popw_GS
	Mnemonic::Pop,// Popd_GS
	Mnemonic::Pop,// Popq_GS
	Mnemonic::Rsm,// Rsm
	Mnemonic::Bts,// Bts_rm16_r16
	Mnemonic::Bts,// Bts_rm32_r32
	Mnemonic::Bts,// Bts_rm64_r64
	Mnemonic::Shrd,// Shrd_rm16_r16_imm8
	Mnemonic::Shrd,// Shrd_rm32_r32_imm8
	Mnemonic::Shrd,// Shrd_rm64_r64_imm8
	Mnemonic::Shrd,// Shrd_rm16_r16_CL
	Mnemonic::Shrd,// Shrd_rm32_r32_CL
	Mnemonic::Shrd,// Shrd_rm64_r64_CL
	Mnemonic::Fxsave,// Fxsave_m512byte
	Mnemonic::Fxsave64,// Fxsave64_m512byte
	Mnemonic::Rdfsbase,// Rdfsbase_r32
	Mnemonic::Rdfsbase,// Rdfsbase_r64
	Mnemonic::Fxrstor,// Fxrstor_m512byte
	Mnemonic::Fxrstor64,// Fxrstor64_m512byte
	Mnemonic::Rdgsbase,// Rdgsbase_r32
	Mnemonic::Rdgsbase,// Rdgsbase_r64
	Mnemonic::Ldmxcsr,// Ldmxcsr_m32
	Mnemonic::Wrfsbase,// Wrfsbase_r32
	Mnemonic::Wrfsbase,// Wrfsbase_r64
	Mnemonic::Vldmxcsr,// VEX_Vldmxcsr_m32
	Mnemonic::Stmxcsr,// Stmxcsr_m32
	Mnemonic::Wrgsbase,// Wrgsbase_r32
	Mnemonic::Wrgsbase,// Wrgsbase_r64
	Mnemonic::Vstmxcsr,// VEX_Vstmxcsr_m32
	Mnemonic::Xsave,// Xsave_mem
	Mnemonic::Xsave64,// Xsave64_mem
	Mnemonic::Ptwrite,// Ptwrite_rm32
	Mnemonic::Ptwrite,// Ptwrite_rm64
	Mnemonic::Xrstor,// Xrstor_mem
	Mnemonic::Xrstor64,// Xrstor64_mem
	Mnemonic::Incsspd,// Incsspd_r32
	Mnemonic::Incsspq,// Incsspq_r64
	Mnemonic::Xsaveopt,// Xsaveopt_mem
	Mnemonic::Xsaveopt64,// Xsaveopt64_mem
	Mnemonic::Clwb,// Clwb_m8
	Mnemonic::Tpause,// Tpause_r32
	Mnemonic::Tpause,// Tpause_r64
	Mnemonic::Clrssbsy,// Clrssbsy_m64
	Mnemonic::Umonitor,// Umonitor_r16
	Mnemonic::Umonitor,// Umonitor_r32
	Mnemonic::Umonitor,// Umonitor_r64
	Mnemonic::Umwait,// Umwait_r32
	Mnemonic::Umwait,// Umwait_r64
	Mnemonic::Clflush,// Clflush_m8
	Mnemonic::Clflushopt,// Clflushopt_m8
	Mnemonic::Lfence,// Lfence
	Mnemonic::Lfence,// Lfence_E9
	Mnemonic::Lfence,// Lfence_EA
	Mnemonic::Lfence,// Lfence_EB
	Mnemonic::Lfence,// Lfence_EC
	Mnemonic::Lfence,// Lfence_ED
	Mnemonic::Lfence,// Lfence_EE
	Mnemonic::Lfence,// Lfence_EF
	Mnemonic::Mfence,// Mfence
	Mnemonic::Mfence,// Mfence_F1
	Mnemonic::Mfence,// Mfence_F2
	Mnemonic::Mfence,// Mfence_F3
	Mnemonic::Mfence,// Mfence_F4
	Mnemonic::Mfence,// Mfence_F5
	Mnemonic::Mfence,// Mfence_F6
	Mnemonic::Mfence,// Mfence_F7
	Mnemonic::Sfence,// Sfence
	Mnemonic::Sfence,// Sfence_F9
	Mnemonic::Sfence,// Sfence_FA
	Mnemonic::Sfence,// Sfence_FB
	Mnemonic::Sfence,// Sfence_FC
	Mnemonic::Sfence,// Sfence_FD
	Mnemonic::Sfence,// Sfence_FE
	Mnemonic::Sfence,// Sfence_FF
	Mnemonic::Pcommit,// Pcommit
	Mnemonic::Imul,// Imul_r16_rm16
	Mnemonic::Imul,// Imul_r32_rm32
	Mnemonic::Imul,// Imul_r64_rm64
	Mnemonic::Cmpxchg,// Cmpxchg_rm8_r8
	Mnemonic::Cmpxchg,// Cmpxchg_rm16_r16
	Mnemonic::Cmpxchg,// Cmpxchg_rm32_r32
	Mnemonic::Cmpxchg,// Cmpxchg_rm64_r64
	Mnemonic::Lss,// Lss_r16_m1616
	Mnemonic::Lss,// Lss_r32_m1632
	Mnemonic::Lss,// Lss_r64_m1664
	Mnemonic::Btr,// Btr_rm16_r16
	Mnemonic::Btr,// Btr_rm32_r32
	Mnemonic::Btr,// Btr_rm64_r64
	Mnemonic::Lfs,// Lfs_r16_m1616
	Mnemonic::Lfs,// Lfs_r32_m1632
	Mnemonic::Lfs,// Lfs_r64_m1664
	Mnemonic::Lgs,// Lgs_r16_m1616
	Mnemonic::Lgs,// Lgs_r32_m1632
	Mnemonic::Lgs,// Lgs_r64_m1664
	Mnemonic::Movzx,// Movzx_r16_rm8
	Mnemonic::Movzx,// Movzx_r32_rm8
	Mnemonic::Movzx,// Movzx_r64_rm8
	Mnemonic::Movzx,// Movzx_r16_rm16
	Mnemonic::Movzx,// Movzx_r32_rm16
	Mnemonic::Movzx,// Movzx_r64_rm16
	Mnemonic::Jmpe,// Jmpe_disp16
	Mnemonic::Jmpe,// Jmpe_disp32
	Mnemonic::Popcnt,// Popcnt_r16_rm16
	Mnemonic::Popcnt,// Popcnt_r32_rm32
	Mnemonic::Popcnt,// Popcnt_r64_rm64
	Mnemonic::Ud1,// Ud1_r16_rm16
	Mnemonic::Ud1,// Ud1_r32_rm32
	Mnemonic::Ud1,// Ud1_r64_rm64
	Mnemonic::Bt,// Bt_rm16_imm8
	Mnemonic::Bt,// Bt_rm32_imm8
	Mnemonic::Bt,// Bt_rm64_imm8
	Mnemonic::Bts,// Bts_rm16_imm8
	Mnemonic::Bts,// Bts_rm32_imm8
	Mnemonic::Bts,// Bts_rm64_imm8
	Mnemonic::Btr,// Btr_rm16_imm8
	Mnemonic::Btr,// Btr_rm32_imm8
	Mnemonic::Btr,// Btr_rm64_imm8
	Mnemonic::Btc,// Btc_rm16_imm8
	Mnemonic::Btc,// Btc_rm32_imm8
	Mnemonic::Btc,// Btc_rm64_imm8
	Mnemonic::Btc,// Btc_rm16_r16
	Mnemonic::Btc,// Btc_rm32_r32
	Mnemonic::Btc,// Btc_rm64_r64
	Mnemonic::Bsf,// Bsf_r16_rm16
	Mnemonic::Bsf,// Bsf_r32_rm32
	Mnemonic::Bsf,// Bsf_r64_rm64
	Mnemonic::Tzcnt,// Tzcnt_r16_rm16
	Mnemonic::Tzcnt,// Tzcnt_r32_rm32
	Mnemonic::Tzcnt,// Tzcnt_r64_rm64
	Mnemonic::Bsr,// Bsr_r16_rm16
	Mnemonic::Bsr,// Bsr_r32_rm32
	Mnemonic::Bsr,// Bsr_r64_rm64
	Mnemonic::Lzcnt,// Lzcnt_r16_rm16
	Mnemonic::Lzcnt,// Lzcnt_r32_rm32
	Mnemonic::Lzcnt,// Lzcnt_r64_rm64
	Mnemonic::Movsx,// Movsx_r16_rm8
	Mnemonic::Movsx,// Movsx_r32_rm8
	Mnemonic::Movsx,// Movsx_r64_rm8
	Mnemonic::Movsx,// Movsx_r16_rm16
	Mnemonic::Movsx,// Movsx_r32_rm16
	Mnemonic::Movsx,// Movsx_r64_rm16
	Mnemonic::Xadd,// Xadd_rm8_r8
	Mnemonic::Xadd,// Xadd_rm16_r16
	Mnemonic::Xadd,// Xadd_rm32_r32
	Mnemonic::Xadd,// Xadd_rm64_r64
	Mnemonic::Cmpps,// Cmpps_xmm_xmmm128_imm8
	Mnemonic::Vcmpps,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	Mnemonic::Vcmpps,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	Mnemonic::Vcmpps,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	Mnemonic::Vcmpps,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	Mnemonic::Vcmpps,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	Mnemonic::Cmppd,// Cmppd_xmm_xmmm128_imm8
	Mnemonic::Vcmppd,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	Mnemonic::Vcmppd,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	Mnemonic::Vcmppd,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	Mnemonic::Vcmppd,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	Mnemonic::Vcmppd,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	Mnemonic::Cmpss,// Cmpss_xmm_xmmm32_imm8
	Mnemonic::Vcmpss,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	Mnemonic::Vcmpss,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	Mnemonic::Cmpsd,// Cmpsd_xmm_xmmm64_imm8
	Mnemonic::Vcmpsd,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	Mnemonic::Vcmpsd,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	Mnemonic::Movnti,// Movnti_m32_r32
	Mnemonic::Movnti,// Movnti_m64_r64
	Mnemonic::Pinsrw,// Pinsrw_mm_r32m16_imm8
	Mnemonic::Pinsrw,// Pinsrw_mm_r64m16_imm8
	Mnemonic::Pinsrw,// Pinsrw_xmm_r32m16_imm8
	Mnemonic::Pinsrw,// Pinsrw_xmm_r64m16_imm8
	Mnemonic::Vpinsrw,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	Mnemonic::Vpinsrw,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	Mnemonic::Vpinsrw,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	Mnemonic::Vpinsrw,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	Mnemonic::Pextrw,// Pextrw_r32_mm_imm8
	Mnemonic::Pextrw,// Pextrw_r64_mm_imm8
	Mnemonic::Pextrw,// Pextrw_r32_xmm_imm8
	Mnemonic::Pextrw,// Pextrw_r64_xmm_imm8
	Mnemonic::Vpextrw,// VEX_Vpextrw_r32_xmm_imm8
	Mnemonic::Vpextrw,// VEX_Vpextrw_r64_xmm_imm8
	Mnemonic::Vpextrw,// EVEX_Vpextrw_r32_xmm_imm8
	Mnemonic::Vpextrw,// EVEX_Vpextrw_r64_xmm_imm8
	Mnemonic::Shufps,// Shufps_xmm_xmmm128_imm8
	Mnemonic::Vshufps,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	Mnemonic::Vshufps,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	Mnemonic::Vshufps,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vshufps,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vshufps,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Shufpd,// Shufpd_xmm_xmmm128_imm8
	Mnemonic::Vshufpd,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	Mnemonic::Vshufpd,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	Mnemonic::Vshufpd,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vshufpd,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vshufpd,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Cmpxchg8b,// Cmpxchg8b_m64
	Mnemonic::Cmpxchg16b,// Cmpxchg16b_m128
	Mnemonic::Xrstors,// Xrstors_mem
	Mnemonic::Xrstors64,// Xrstors64_mem
	Mnemonic::Xsavec,// Xsavec_mem
	Mnemonic::Xsavec64,// Xsavec64_mem
	Mnemonic::Xsaves,// Xsaves_mem
	Mnemonic::Xsaves64,// Xsaves64_mem
	Mnemonic::Vmptrld,// Vmptrld_m64
	Mnemonic::Vmclear,// Vmclear_m64
	Mnemonic::Vmxon,// Vmxon_m64
	Mnemonic::Rdrand,// Rdrand_r16
	Mnemonic::Rdrand,// Rdrand_r32
	Mnemonic::Rdrand,// Rdrand_r64
	Mnemonic::Vmptrst,// Vmptrst_m64
	Mnemonic::Rdseed,// Rdseed_r16
	Mnemonic::Rdseed,// Rdseed_r32
	Mnemonic::Rdseed,// Rdseed_r64
	Mnemonic::Rdpid,// Rdpid_r32
	Mnemonic::Rdpid,// Rdpid_r64
	Mnemonic::Bswap,// Bswap_r16
	Mnemonic::Bswap,// Bswap_r32
	Mnemonic::Bswap,// Bswap_r64
	Mnemonic::Addsubpd,// Addsubpd_xmm_xmmm128
	Mnemonic::Vaddsubpd,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	Mnemonic::Vaddsubpd,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	Mnemonic::Addsubps,// Addsubps_xmm_xmmm128
	Mnemonic::Vaddsubps,// VEX_Vaddsubps_xmm_xmm_xmmm128
	Mnemonic::Vaddsubps,// VEX_Vaddsubps_ymm_ymm_ymmm256
	Mnemonic::Psrlw,// Psrlw_mm_mmm64
	Mnemonic::Psrlw,// Psrlw_xmm_xmmm128
	Mnemonic::Vpsrlw,// VEX_Vpsrlw_xmm_xmm_xmmm128
	Mnemonic::Vpsrlw,// VEX_Vpsrlw_ymm_ymm_xmmm128
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsrlw,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	Mnemonic::Psrld,// Psrld_mm_mmm64
	Mnemonic::Psrld,// Psrld_xmm_xmmm128
	Mnemonic::Vpsrld,// VEX_Vpsrld_xmm_xmm_xmmm128
	Mnemonic::Vpsrld,// VEX_Vpsrld_ymm_ymm_xmmm128
	Mnemonic::Vpsrld,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsrld,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsrld,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	Mnemonic::Psrlq,// Psrlq_mm_mmm64
	Mnemonic::Psrlq,// Psrlq_xmm_xmmm128
	Mnemonic::Vpsrlq,// VEX_Vpsrlq_xmm_xmm_xmmm128
	Mnemonic::Vpsrlq,// VEX_Vpsrlq_ymm_ymm_xmmm128
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsrlq,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	Mnemonic::Paddq,// Paddq_mm_mmm64
	Mnemonic::Paddq,// Paddq_xmm_xmmm128
	Mnemonic::Vpaddq,// VEX_Vpaddq_xmm_xmm_xmmm128
	Mnemonic::Vpaddq,// VEX_Vpaddq_ymm_ymm_ymmm256
	Mnemonic::Vpaddq,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpaddq,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpaddq,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pmullw,// Pmullw_mm_mmm64
	Mnemonic::Pmullw,// Pmullw_xmm_xmmm128
	Mnemonic::Vpmullw,// VEX_Vpmullw_xmm_xmm_xmmm128
	Mnemonic::Vpmullw,// VEX_Vpmullw_ymm_ymm_ymmm256
	Mnemonic::Vpmullw,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmullw,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmullw,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	Mnemonic::Movq,// Movq_xmmm64_xmm
	Mnemonic::Vmovq,// VEX_Vmovq_xmmm64_xmm
	Mnemonic::Vmovq,// EVEX_Vmovq_xmmm64_xmm
	Mnemonic::Movq2dq,// Movq2dq_xmm_mm
	Mnemonic::Movdq2q,// Movdq2q_mm_xmm
	Mnemonic::Pmovmskb,// Pmovmskb_r32_mm
	Mnemonic::Pmovmskb,// Pmovmskb_r64_mm
	Mnemonic::Pmovmskb,// Pmovmskb_r32_xmm
	Mnemonic::Pmovmskb,// Pmovmskb_r64_xmm
	Mnemonic::Vpmovmskb,// VEX_Vpmovmskb_r32_xmm
	Mnemonic::Vpmovmskb,// VEX_Vpmovmskb_r64_xmm
	Mnemonic::Vpmovmskb,// VEX_Vpmovmskb_r32_ymm
	Mnemonic::Vpmovmskb,// VEX_Vpmovmskb_r64_ymm
	Mnemonic::Psubusb,// Psubusb_mm_mmm64
	Mnemonic::Psubusb,// Psubusb_xmm_xmmm128
	Mnemonic::Vpsubusb,// VEX_Vpsubusb_xmm_xmm_xmmm128
	Mnemonic::Vpsubusb,// VEX_Vpsubusb_ymm_ymm_ymmm256
	Mnemonic::Vpsubusb,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubusb,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubusb,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	Mnemonic::Psubusw,// Psubusw_mm_mmm64
	Mnemonic::Psubusw,// Psubusw_xmm_xmmm128
	Mnemonic::Vpsubusw,// VEX_Vpsubusw_xmm_xmm_xmmm128
	Mnemonic::Vpsubusw,// VEX_Vpsubusw_ymm_ymm_ymmm256
	Mnemonic::Vpsubusw,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubusw,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubusw,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pminub,// Pminub_mm_mmm64
	Mnemonic::Pminub,// Pminub_xmm_xmmm128
	Mnemonic::Vpminub,// VEX_Vpminub_xmm_xmm_xmmm128
	Mnemonic::Vpminub,// VEX_Vpminub_ymm_ymm_ymmm256
	Mnemonic::Vpminub,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpminub,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpminub,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	Mnemonic::Pand,// Pand_mm_mmm64
	Mnemonic::Pand,// Pand_xmm_xmmm128
	Mnemonic::Vpand,// VEX_Vpand_xmm_xmm_xmmm128
	Mnemonic::Vpand,// VEX_Vpand_ymm_ymm_ymmm256
	Mnemonic::Vpandd,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpandd,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpandd,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpandq,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpandq,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpandq,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Paddusb,// Paddusb_mm_mmm64
	Mnemonic::Paddusb,// Paddusb_xmm_xmmm128
	Mnemonic::Vpaddusb,// VEX_Vpaddusb_xmm_xmm_xmmm128
	Mnemonic::Vpaddusb,// VEX_Vpaddusb_ymm_ymm_ymmm256
	Mnemonic::Vpaddusb,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddusb,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddusb,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	Mnemonic::Paddusw,// Paddusw_mm_mmm64
	Mnemonic::Paddusw,// Paddusw_xmm_xmmm128
	Mnemonic::Vpaddusw,// VEX_Vpaddusw_xmm_xmm_xmmm128
	Mnemonic::Vpaddusw,// VEX_Vpaddusw_ymm_ymm_ymmm256
	Mnemonic::Vpaddusw,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddusw,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddusw,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmaxub,// Pmaxub_mm_mmm64
	Mnemonic::Pmaxub,// Pmaxub_xmm_xmmm128
	Mnemonic::Vpmaxub,// VEX_Vpmaxub_xmm_xmm_xmmm128
	Mnemonic::Vpmaxub,// VEX_Vpmaxub_ymm_ymm_ymmm256
	Mnemonic::Vpmaxub,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaxub,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaxub,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	Mnemonic::Pandn,// Pandn_mm_mmm64
	Mnemonic::Pandn,// Pandn_xmm_xmmm128
	Mnemonic::Vpandn,// VEX_Vpandn_xmm_xmm_xmmm128
	Mnemonic::Vpandn,// VEX_Vpandn_ymm_ymm_ymmm256
	Mnemonic::Vpandnd,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpandnd,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpandnd,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpandnq,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpandnq,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpandnq,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pavgb,// Pavgb_mm_mmm64
	Mnemonic::Pavgb,// Pavgb_xmm_xmmm128
	Mnemonic::Vpavgb,// VEX_Vpavgb_xmm_xmm_xmmm128
	Mnemonic::Vpavgb,// VEX_Vpavgb_ymm_ymm_ymmm256
	Mnemonic::Vpavgb,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpavgb,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpavgb,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	Mnemonic::Psraw,// Psraw_mm_mmm64
	Mnemonic::Psraw,// Psraw_xmm_xmmm128
	Mnemonic::Vpsraw,// VEX_Vpsraw_xmm_xmm_xmmm128
	Mnemonic::Vpsraw,// VEX_Vpsraw_ymm_ymm_xmmm128
	Mnemonic::Vpsraw,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsraw,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsraw,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	Mnemonic::Psrad,// Psrad_mm_mmm64
	Mnemonic::Psrad,// Psrad_xmm_xmmm128
	Mnemonic::Vpsrad,// VEX_Vpsrad_xmm_xmm_xmmm128
	Mnemonic::Vpsrad,// VEX_Vpsrad_ymm_ymm_xmmm128
	Mnemonic::Vpsrad,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsrad,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsrad,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	Mnemonic::Vpsraq,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsraq,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsraq,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	Mnemonic::Pavgw,// Pavgw_mm_mmm64
	Mnemonic::Pavgw,// Pavgw_xmm_xmmm128
	Mnemonic::Vpavgw,// VEX_Vpavgw_xmm_xmm_xmmm128
	Mnemonic::Vpavgw,// VEX_Vpavgw_ymm_ymm_ymmm256
	Mnemonic::Vpavgw,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpavgw,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpavgw,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmulhuw,// Pmulhuw_mm_mmm64
	Mnemonic::Pmulhuw,// Pmulhuw_xmm_xmmm128
	Mnemonic::Vpmulhuw,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	Mnemonic::Vpmulhuw,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	Mnemonic::Vpmulhuw,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmulhuw,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmulhuw,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmulhw,// Pmulhw_mm_mmm64
	Mnemonic::Pmulhw,// Pmulhw_xmm_xmmm128
	Mnemonic::Vpmulhw,// VEX_Vpmulhw_xmm_xmm_xmmm128
	Mnemonic::Vpmulhw,// VEX_Vpmulhw_ymm_ymm_ymmm256
	Mnemonic::Vpmulhw,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmulhw,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmulhw,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	Mnemonic::Cvttpd2dq,// Cvttpd2dq_xmm_xmmm128
	Mnemonic::Vcvttpd2dq,// VEX_Vcvttpd2dq_xmm_xmmm128
	Mnemonic::Vcvttpd2dq,// VEX_Vcvttpd2dq_xmm_ymmm256
	Mnemonic::Vcvttpd2dq,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvttpd2dq,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	Mnemonic::Vcvttpd2dq,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	Mnemonic::Cvtdq2pd,// Cvtdq2pd_xmm_xmmm64
	Mnemonic::Vcvtdq2pd,// VEX_Vcvtdq2pd_xmm_xmmm64
	Mnemonic::Vcvtdq2pd,// VEX_Vcvtdq2pd_ymm_xmmm128
	Mnemonic::Vcvtdq2pd,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	Mnemonic::Vcvtdq2pd,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	Mnemonic::Vcvtdq2pd,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	Mnemonic::Vcvtqq2pd,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtqq2pd,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	Mnemonic::Vcvtqq2pd,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	Mnemonic::Cvtpd2dq,// Cvtpd2dq_xmm_xmmm128
	Mnemonic::Vcvtpd2dq,// VEX_Vcvtpd2dq_xmm_xmmm128
	Mnemonic::Vcvtpd2dq,// VEX_Vcvtpd2dq_xmm_ymmm256
	Mnemonic::Vcvtpd2dq,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2dq,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2dq,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	Mnemonic::Movntq,// Movntq_m64_mm
	Mnemonic::Movntdq,// Movntdq_m128_xmm
	Mnemonic::Vmovntdq,// VEX_Vmovntdq_m128_xmm
	Mnemonic::Vmovntdq,// VEX_Vmovntdq_m256_ymm
	Mnemonic::Vmovntdq,// EVEX_Vmovntdq_m128_xmm
	Mnemonic::Vmovntdq,// EVEX_Vmovntdq_m256_ymm
	Mnemonic::Vmovntdq,// EVEX_Vmovntdq_m512_zmm
	Mnemonic::Psubsb,// Psubsb_mm_mmm64
	Mnemonic::Psubsb,// Psubsb_xmm_xmmm128
	Mnemonic::Vpsubsb,// VEX_Vpsubsb_xmm_xmm_xmmm128
	Mnemonic::Vpsubsb,// VEX_Vpsubsb_ymm_ymm_ymmm256
	Mnemonic::Vpsubsb,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubsb,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubsb,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	Mnemonic::Psubsw,// Psubsw_mm_mmm64
	Mnemonic::Psubsw,// Psubsw_xmm_xmmm128
	Mnemonic::Vpsubsw,// VEX_Vpsubsw_xmm_xmm_xmmm128
	Mnemonic::Vpsubsw,// VEX_Vpsubsw_ymm_ymm_ymmm256
	Mnemonic::Vpsubsw,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubsw,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubsw,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pminsw,// Pminsw_mm_mmm64
	Mnemonic::Pminsw,// Pminsw_xmm_xmmm128
	Mnemonic::Vpminsw,// VEX_Vpminsw_xmm_xmm_xmmm128
	Mnemonic::Vpminsw,// VEX_Vpminsw_ymm_ymm_ymmm256
	Mnemonic::Vpminsw,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpminsw,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpminsw,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Por,// Por_mm_mmm64
	Mnemonic::Por,// Por_xmm_xmmm128
	Mnemonic::Vpor,// VEX_Vpor_xmm_xmm_xmmm128
	Mnemonic::Vpor,// VEX_Vpor_ymm_ymm_ymmm256
	Mnemonic::Vpord,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpord,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpord,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vporq,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vporq,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vporq,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Paddsb,// Paddsb_mm_mmm64
	Mnemonic::Paddsb,// Paddsb_xmm_xmmm128
	Mnemonic::Vpaddsb,// VEX_Vpaddsb_xmm_xmm_xmmm128
	Mnemonic::Vpaddsb,// VEX_Vpaddsb_ymm_ymm_ymmm256
	Mnemonic::Vpaddsb,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddsb,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddsb,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	Mnemonic::Paddsw,// Paddsw_mm_mmm64
	Mnemonic::Paddsw,// Paddsw_xmm_xmmm128
	Mnemonic::Vpaddsw,// VEX_Vpaddsw_xmm_xmm_xmmm128
	Mnemonic::Vpaddsw,// VEX_Vpaddsw_ymm_ymm_ymmm256
	Mnemonic::Vpaddsw,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddsw,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddsw,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmaxsw,// Pmaxsw_mm_mmm64
	Mnemonic::Pmaxsw,// Pmaxsw_xmm_xmmm128
	Mnemonic::Vpmaxsw,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	Mnemonic::Vpmaxsw,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	Mnemonic::Vpmaxsw,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaxsw,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaxsw,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pxor,// Pxor_mm_mmm64
	Mnemonic::Pxor,// Pxor_xmm_xmmm128
	Mnemonic::Vpxor,// VEX_Vpxor_xmm_xmm_xmmm128
	Mnemonic::Vpxor,// VEX_Vpxor_ymm_ymm_ymmm256
	Mnemonic::Vpxord,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpxord,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpxord,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpxorq,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpxorq,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpxorq,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Lddqu,// Lddqu_xmm_m128
	Mnemonic::Vlddqu,// VEX_Vlddqu_xmm_m128
	Mnemonic::Vlddqu,// VEX_Vlddqu_ymm_m256
	Mnemonic::Psllw,// Psllw_mm_mmm64
	Mnemonic::Psllw,// Psllw_xmm_xmmm128
	Mnemonic::Vpsllw,// VEX_Vpsllw_xmm_xmm_xmmm128
	Mnemonic::Vpsllw,// VEX_Vpsllw_ymm_ymm_xmmm128
	Mnemonic::Vpsllw,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsllw,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsllw,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	Mnemonic::Pslld,// Pslld_mm_mmm64
	Mnemonic::Pslld,// Pslld_xmm_xmmm128
	Mnemonic::Vpslld,// VEX_Vpslld_xmm_xmm_xmmm128
	Mnemonic::Vpslld,// VEX_Vpslld_ymm_ymm_xmmm128
	Mnemonic::Vpslld,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpslld,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpslld,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	Mnemonic::Psllq,// Psllq_mm_mmm64
	Mnemonic::Psllq,// Psllq_xmm_xmmm128
	Mnemonic::Vpsllq,// VEX_Vpsllq_xmm_xmm_xmmm128
	Mnemonic::Vpsllq,// VEX_Vpsllq_ymm_ymm_xmmm128
	Mnemonic::Vpsllq,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsllq,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	Mnemonic::Vpsllq,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	Mnemonic::Pmuludq,// Pmuludq_mm_mmm64
	Mnemonic::Pmuludq,// Pmuludq_xmm_xmmm128
	Mnemonic::Vpmuludq,// VEX_Vpmuludq_xmm_xmm_xmmm128
	Mnemonic::Vpmuludq,// VEX_Vpmuludq_ymm_ymm_ymmm256
	Mnemonic::Vpmuludq,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmuludq,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmuludq,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pmaddwd,// Pmaddwd_mm_mmm64
	Mnemonic::Pmaddwd,// Pmaddwd_xmm_xmmm128
	Mnemonic::Vpmaddwd,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	Mnemonic::Vpmaddwd,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	Mnemonic::Vpmaddwd,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaddwd,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaddwd,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	Mnemonic::Psadbw,// Psadbw_mm_mmm64
	Mnemonic::Psadbw,// Psadbw_xmm_xmmm128
	Mnemonic::Vpsadbw,// VEX_Vpsadbw_xmm_xmm_xmmm128
	Mnemonic::Vpsadbw,// VEX_Vpsadbw_ymm_ymm_ymmm256
	Mnemonic::Vpsadbw,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	Mnemonic::Vpsadbw,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	Mnemonic::Vpsadbw,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	Mnemonic::Maskmovq,// Maskmovq_rDI_mm_mm
	Mnemonic::Maskmovdqu,// Maskmovdqu_rDI_xmm_xmm
	Mnemonic::Vmaskmovdqu,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	Mnemonic::Psubb,// Psubb_mm_mmm64
	Mnemonic::Psubb,// Psubb_xmm_xmmm128
	Mnemonic::Vpsubb,// VEX_Vpsubb_xmm_xmm_xmmm128
	Mnemonic::Vpsubb,// VEX_Vpsubb_ymm_ymm_ymmm256
	Mnemonic::Vpsubb,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubb,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubb,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	Mnemonic::Psubw,// Psubw_mm_mmm64
	Mnemonic::Psubw,// Psubw_xmm_xmmm128
	Mnemonic::Vpsubw,// VEX_Vpsubw_xmm_xmm_xmmm128
	Mnemonic::Vpsubw,// VEX_Vpsubw_ymm_ymm_ymmm256
	Mnemonic::Vpsubw,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsubw,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsubw,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	Mnemonic::Psubd,// Psubd_mm_mmm64
	Mnemonic::Psubd,// Psubd_xmm_xmmm128
	Mnemonic::Vpsubd,// VEX_Vpsubd_xmm_xmm_xmmm128
	Mnemonic::Vpsubd,// VEX_Vpsubd_ymm_ymm_ymmm256
	Mnemonic::Vpsubd,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpsubd,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpsubd,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Psubq,// Psubq_mm_mmm64
	Mnemonic::Psubq,// Psubq_xmm_xmmm128
	Mnemonic::Vpsubq,// VEX_Vpsubq_xmm_xmm_xmmm128
	Mnemonic::Vpsubq,// VEX_Vpsubq_ymm_ymm_ymmm256
	Mnemonic::Vpsubq,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpsubq,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpsubq,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Paddb,// Paddb_mm_mmm64
	Mnemonic::Paddb,// Paddb_xmm_xmmm128
	Mnemonic::Vpaddb,// VEX_Vpaddb_xmm_xmm_xmmm128
	Mnemonic::Vpaddb,// VEX_Vpaddb_ymm_ymm_ymmm256
	Mnemonic::Vpaddb,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddb,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddb,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	Mnemonic::Paddw,// Paddw_mm_mmm64
	Mnemonic::Paddw,// Paddw_xmm_xmmm128
	Mnemonic::Vpaddw,// VEX_Vpaddw_xmm_xmm_xmmm128
	Mnemonic::Vpaddw,// VEX_Vpaddw_ymm_ymm_ymmm256
	Mnemonic::Vpaddw,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpaddw,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpaddw,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	Mnemonic::Paddd,// Paddd_mm_mmm64
	Mnemonic::Paddd,// Paddd_xmm_xmmm128
	Mnemonic::Vpaddd,// VEX_Vpaddd_xmm_xmm_xmmm128
	Mnemonic::Vpaddd,// VEX_Vpaddd_ymm_ymm_ymmm256
	Mnemonic::Vpaddd,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpaddd,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpaddd,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Ud0,// Ud0_r16_rm16
	Mnemonic::Ud0,// Ud0_r32_rm32
	Mnemonic::Ud0,// Ud0_r64_rm64
	Mnemonic::Pshufb,// Pshufb_mm_mmm64
	Mnemonic::Pshufb,// Pshufb_xmm_xmmm128
	Mnemonic::Vpshufb,// VEX_Vpshufb_xmm_xmm_xmmm128
	Mnemonic::Vpshufb,// VEX_Vpshufb_ymm_ymm_ymmm256
	Mnemonic::Vpshufb,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpshufb,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpshufb,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	Mnemonic::Phaddw,// Phaddw_mm_mmm64
	Mnemonic::Phaddw,// Phaddw_xmm_xmmm128
	Mnemonic::Vphaddw,// VEX_Vphaddw_xmm_xmm_xmmm128
	Mnemonic::Vphaddw,// VEX_Vphaddw_ymm_ymm_ymmm256
	Mnemonic::Phaddd,// Phaddd_mm_mmm64
	Mnemonic::Phaddd,// Phaddd_xmm_xmmm128
	Mnemonic::Vphaddd,// VEX_Vphaddd_xmm_xmm_xmmm128
	Mnemonic::Vphaddd,// VEX_Vphaddd_ymm_ymm_ymmm256
	Mnemonic::Phaddsw,// Phaddsw_mm_mmm64
	Mnemonic::Phaddsw,// Phaddsw_xmm_xmmm128
	Mnemonic::Vphaddsw,// VEX_Vphaddsw_xmm_xmm_xmmm128
	Mnemonic::Vphaddsw,// VEX_Vphaddsw_ymm_ymm_ymmm256
	Mnemonic::Pmaddubsw,// Pmaddubsw_mm_mmm64
	Mnemonic::Pmaddubsw,// Pmaddubsw_xmm_xmmm128
	Mnemonic::Vpmaddubsw,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	Mnemonic::Vpmaddubsw,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	Mnemonic::Vpmaddubsw,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaddubsw,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaddubsw,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Phsubw,// Phsubw_mm_mmm64
	Mnemonic::Phsubw,// Phsubw_xmm_xmmm128
	Mnemonic::Vphsubw,// VEX_Vphsubw_xmm_xmm_xmmm128
	Mnemonic::Vphsubw,// VEX_Vphsubw_ymm_ymm_ymmm256
	Mnemonic::Phsubd,// Phsubd_mm_mmm64
	Mnemonic::Phsubd,// Phsubd_xmm_xmmm128
	Mnemonic::Vphsubd,// VEX_Vphsubd_xmm_xmm_xmmm128
	Mnemonic::Vphsubd,// VEX_Vphsubd_ymm_ymm_ymmm256
	Mnemonic::Phsubsw,// Phsubsw_mm_mmm64
	Mnemonic::Phsubsw,// Phsubsw_xmm_xmmm128
	Mnemonic::Vphsubsw,// VEX_Vphsubsw_xmm_xmm_xmmm128
	Mnemonic::Vphsubsw,// VEX_Vphsubsw_ymm_ymm_ymmm256
	Mnemonic::Psignb,// Psignb_mm_mmm64
	Mnemonic::Psignb,// Psignb_xmm_xmmm128
	Mnemonic::Vpsignb,// VEX_Vpsignb_xmm_xmm_xmmm128
	Mnemonic::Vpsignb,// VEX_Vpsignb_ymm_ymm_ymmm256
	Mnemonic::Psignw,// Psignw_mm_mmm64
	Mnemonic::Psignw,// Psignw_xmm_xmmm128
	Mnemonic::Vpsignw,// VEX_Vpsignw_xmm_xmm_xmmm128
	Mnemonic::Vpsignw,// VEX_Vpsignw_ymm_ymm_ymmm256
	Mnemonic::Psignd,// Psignd_mm_mmm64
	Mnemonic::Psignd,// Psignd_xmm_xmmm128
	Mnemonic::Vpsignd,// VEX_Vpsignd_xmm_xmm_xmmm128
	Mnemonic::Vpsignd,// VEX_Vpsignd_ymm_ymm_ymmm256
	Mnemonic::Pmulhrsw,// Pmulhrsw_mm_mmm64
	Mnemonic::Pmulhrsw,// Pmulhrsw_xmm_xmmm128
	Mnemonic::Vpmulhrsw,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	Mnemonic::Vpmulhrsw,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	Mnemonic::Vpmulhrsw,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmulhrsw,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmulhrsw,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermilps,// VEX_Vpermilps_xmm_xmm_xmmm128
	Mnemonic::Vpermilps,// VEX_Vpermilps_ymm_ymm_ymmm256
	Mnemonic::Vpermilps,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpermilps,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermilps,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermilpd,// VEX_Vpermilpd_xmm_xmm_xmmm128
	Mnemonic::Vpermilpd,// VEX_Vpermilpd_ymm_ymm_ymmm256
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vtestps,// VEX_Vtestps_xmm_xmmm128
	Mnemonic::Vtestps,// VEX_Vtestps_ymm_ymmm256
	Mnemonic::Vtestpd,// VEX_Vtestpd_xmm_xmmm128
	Mnemonic::Vtestpd,// VEX_Vtestpd_ymm_ymmm256
	Mnemonic::Pblendvb,// Pblendvb_xmm_xmmm128
	Mnemonic::Vpsrlvw,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsrlvw,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsrlvw,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpmovuswb,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	Mnemonic::Vpmovuswb,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	Mnemonic::Vpmovuswb,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	Mnemonic::Vpsravw,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsravw,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsravw,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpmovusdb,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	Mnemonic::Vpmovusdb,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	Mnemonic::Vpmovusdb,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	Mnemonic::Vpsllvw,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpsllvw,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpsllvw,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpmovusqb,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	Mnemonic::Vpmovusqb,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	Mnemonic::Vpmovusqb,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	Mnemonic::Vcvtph2ps,// VEX_Vcvtph2ps_xmm_xmmm64
	Mnemonic::Vcvtph2ps,// VEX_Vcvtph2ps_ymm_xmmm128
	Mnemonic::Vcvtph2ps,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	Mnemonic::Vcvtph2ps,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	Mnemonic::Vcvtph2ps,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	Mnemonic::Vpmovusdw,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	Mnemonic::Vpmovusdw,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	Mnemonic::Vpmovusdw,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	Mnemonic::Blendvps,// Blendvps_xmm_xmmm128
	Mnemonic::Vprorvd,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vprorvd,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vprorvd,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vprorvq,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vprorvq,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vprorvq,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpmovusqw,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	Mnemonic::Vpmovusqw,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	Mnemonic::Vpmovusqw,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	Mnemonic::Blendvpd,// Blendvpd_xmm_xmmm128
	Mnemonic::Vprolvd,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vprolvd,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vprolvd,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vprolvq,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vprolvq,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vprolvq,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpmovusqd,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	Mnemonic::Vpmovusqd,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	Mnemonic::Vpmovusqd,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	Mnemonic::Vpermps,// VEX_Vpermps_ymm_ymm_ymmm256
	Mnemonic::Vpermps,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermps,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermpd,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermpd,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Ptest,// Ptest_xmm_xmmm128
	Mnemonic::Vptest,// VEX_Vptest_xmm_xmmm128
	Mnemonic::Vptest,// VEX_Vptest_ymm_ymmm256
	Mnemonic::Vbroadcastss,// VEX_Vbroadcastss_xmm_m32
	Mnemonic::Vbroadcastss,// VEX_Vbroadcastss_ymm_m32
	Mnemonic::Vbroadcastss,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	Mnemonic::Vbroadcastss,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	Mnemonic::Vbroadcastss,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	Mnemonic::Vbroadcastsd,// VEX_Vbroadcastsd_ymm_m64
	Mnemonic::Vbroadcastf32x2,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	Mnemonic::Vbroadcastf32x2,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	Mnemonic::Vbroadcastsd,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	Mnemonic::Vbroadcastsd,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	Mnemonic::Vbroadcastf128,// VEX_Vbroadcastf128_ymm_m128
	Mnemonic::Vbroadcastf32x4,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	Mnemonic::Vbroadcastf32x4,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	Mnemonic::Vbroadcastf64x2,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	Mnemonic::Vbroadcastf64x2,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	Mnemonic::Vbroadcastf32x8,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	Mnemonic::Vbroadcastf64x4,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	Mnemonic::Pabsb,// Pabsb_mm_mmm64
	Mnemonic::Pabsb,// Pabsb_xmm_xmmm128
	Mnemonic::Vpabsb,// VEX_Vpabsb_xmm_xmmm128
	Mnemonic::Vpabsb,// VEX_Vpabsb_ymm_ymmm256
	Mnemonic::Vpabsb,// EVEX_Vpabsb_xmm_k1z_xmmm128
	Mnemonic::Vpabsb,// EVEX_Vpabsb_ymm_k1z_ymmm256
	Mnemonic::Vpabsb,// EVEX_Vpabsb_zmm_k1z_zmmm512
	Mnemonic::Pabsw,// Pabsw_mm_mmm64
	Mnemonic::Pabsw,// Pabsw_xmm_xmmm128
	Mnemonic::Vpabsw,// VEX_Vpabsw_xmm_xmmm128
	Mnemonic::Vpabsw,// VEX_Vpabsw_ymm_ymmm256
	Mnemonic::Vpabsw,// EVEX_Vpabsw_xmm_k1z_xmmm128
	Mnemonic::Vpabsw,// EVEX_Vpabsw_ymm_k1z_ymmm256
	Mnemonic::Vpabsw,// EVEX_Vpabsw_zmm_k1z_zmmm512
	Mnemonic::Pabsd,// Pabsd_mm_mmm64
	Mnemonic::Pabsd,// Pabsd_xmm_xmmm128
	Mnemonic::Vpabsd,// VEX_Vpabsd_xmm_xmmm128
	Mnemonic::Vpabsd,// VEX_Vpabsd_ymm_ymmm256
	Mnemonic::Vpabsd,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	Mnemonic::Vpabsd,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	Mnemonic::Vpabsd,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	Mnemonic::Vpabsq,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	Mnemonic::Vpabsq,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	Mnemonic::Vpabsq,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	Mnemonic::Pmovsxbw,// Pmovsxbw_xmm_xmmm64
	Mnemonic::Vpmovsxbw,// VEX_Vpmovsxbw_xmm_xmmm64
	Mnemonic::Vpmovsxbw,// VEX_Vpmovsxbw_ymm_xmmm128
	Mnemonic::Vpmovsxbw,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	Mnemonic::Vpmovsxbw,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	Mnemonic::Vpmovsxbw,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	Mnemonic::Vpmovswb,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	Mnemonic::Vpmovswb,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	Mnemonic::Vpmovswb,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	Mnemonic::Pmovsxbd,// Pmovsxbd_xmm_xmmm32
	Mnemonic::Vpmovsxbd,// VEX_Vpmovsxbd_xmm_xmmm32
	Mnemonic::Vpmovsxbd,// VEX_Vpmovsxbd_ymm_xmmm64
	Mnemonic::Vpmovsxbd,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	Mnemonic::Vpmovsxbd,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	Mnemonic::Vpmovsxbd,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	Mnemonic::Vpmovsdb,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	Mnemonic::Vpmovsdb,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	Mnemonic::Vpmovsdb,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	Mnemonic::Pmovsxbq,// Pmovsxbq_xmm_xmmm16
	Mnemonic::Vpmovsxbq,// VEX_Vpmovsxbq_xmm_xmmm16
	Mnemonic::Vpmovsxbq,// VEX_Vpmovsxbq_ymm_xmmm32
	Mnemonic::Vpmovsxbq,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	Mnemonic::Vpmovsxbq,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	Mnemonic::Vpmovsxbq,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	Mnemonic::Vpmovsqb,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	Mnemonic::Vpmovsqb,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	Mnemonic::Vpmovsqb,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	Mnemonic::Pmovsxwd,// Pmovsxwd_xmm_xmmm64
	Mnemonic::Vpmovsxwd,// VEX_Vpmovsxwd_xmm_xmmm64
	Mnemonic::Vpmovsxwd,// VEX_Vpmovsxwd_ymm_xmmm128
	Mnemonic::Vpmovsxwd,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	Mnemonic::Vpmovsxwd,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	Mnemonic::Vpmovsxwd,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	Mnemonic::Vpmovsdw,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	Mnemonic::Vpmovsdw,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	Mnemonic::Vpmovsdw,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	Mnemonic::Pmovsxwq,// Pmovsxwq_xmm_xmmm32
	Mnemonic::Vpmovsxwq,// VEX_Vpmovsxwq_xmm_xmmm32
	Mnemonic::Vpmovsxwq,// VEX_Vpmovsxwq_ymm_xmmm64
	Mnemonic::Vpmovsxwq,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	Mnemonic::Vpmovsxwq,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	Mnemonic::Vpmovsxwq,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	Mnemonic::Vpmovsqw,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	Mnemonic::Vpmovsqw,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	Mnemonic::Vpmovsqw,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	Mnemonic::Pmovsxdq,// Pmovsxdq_xmm_xmmm64
	Mnemonic::Vpmovsxdq,// VEX_Vpmovsxdq_xmm_xmmm64
	Mnemonic::Vpmovsxdq,// VEX_Vpmovsxdq_ymm_xmmm128
	Mnemonic::Vpmovsxdq,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	Mnemonic::Vpmovsxdq,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	Mnemonic::Vpmovsxdq,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	Mnemonic::Vpmovsqd,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	Mnemonic::Vpmovsqd,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	Mnemonic::Vpmovsqd,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	Mnemonic::Vptestmb,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	Mnemonic::Vptestmb,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	Mnemonic::Vptestmb,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	Mnemonic::Vptestmw,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	Mnemonic::Vptestmw,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	Mnemonic::Vptestmw,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	Mnemonic::Vptestnmb,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	Mnemonic::Vptestnmb,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	Mnemonic::Vptestnmb,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	Mnemonic::Vptestnmw,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	Mnemonic::Vptestnmw,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	Mnemonic::Vptestnmw,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	Mnemonic::Vptestmd,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	Mnemonic::Vptestmd,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	Mnemonic::Vptestmd,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	Mnemonic::Vptestmq,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	Mnemonic::Vptestmq,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	Mnemonic::Vptestmq,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	Mnemonic::Vptestnmd,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	Mnemonic::Vptestnmd,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	Mnemonic::Vptestnmd,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	Mnemonic::Vptestnmq,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	Mnemonic::Vptestnmq,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	Mnemonic::Vptestnmq,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	Mnemonic::Pmuldq,// Pmuldq_xmm_xmmm128
	Mnemonic::Vpmuldq,// VEX_Vpmuldq_xmm_xmm_xmmm128
	Mnemonic::Vpmuldq,// VEX_Vpmuldq_ymm_ymm_ymmm256
	Mnemonic::Vpmuldq,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmuldq,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmuldq,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpmovm2b,// EVEX_Vpmovm2b_xmm_kr
	Mnemonic::Vpmovm2b,// EVEX_Vpmovm2b_ymm_kr
	Mnemonic::Vpmovm2b,// EVEX_Vpmovm2b_zmm_kr
	Mnemonic::Vpmovm2w,// EVEX_Vpmovm2w_xmm_kr
	Mnemonic::Vpmovm2w,// EVEX_Vpmovm2w_ymm_kr
	Mnemonic::Vpmovm2w,// EVEX_Vpmovm2w_zmm_kr
	Mnemonic::Pcmpeqq,// Pcmpeqq_xmm_xmmm128
	Mnemonic::Vpcmpeqq,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	Mnemonic::Vpcmpeqq,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	Mnemonic::Vpcmpeqq,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	Mnemonic::Vpcmpeqq,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	Mnemonic::Vpcmpeqq,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	Mnemonic::Vpmovb2m,// EVEX_Vpmovb2m_kr_xmm
	Mnemonic::Vpmovb2m,// EVEX_Vpmovb2m_kr_ymm
	Mnemonic::Vpmovb2m,// EVEX_Vpmovb2m_kr_zmm
	Mnemonic::Vpmovw2m,// EVEX_Vpmovw2m_kr_xmm
	Mnemonic::Vpmovw2m,// EVEX_Vpmovw2m_kr_ymm
	Mnemonic::Vpmovw2m,// EVEX_Vpmovw2m_kr_zmm
	Mnemonic::Movntdqa,// Movntdqa_xmm_m128
	Mnemonic::Vmovntdqa,// VEX_Vmovntdqa_xmm_m128
	Mnemonic::Vmovntdqa,// VEX_Vmovntdqa_ymm_m256
	Mnemonic::Vmovntdqa,// EVEX_Vmovntdqa_xmm_m128
	Mnemonic::Vmovntdqa,// EVEX_Vmovntdqa_ymm_m256
	Mnemonic::Vmovntdqa,// EVEX_Vmovntdqa_zmm_m512
	Mnemonic::Vpbroadcastmb2q,// EVEX_Vpbroadcastmb2q_xmm_kr
	Mnemonic::Vpbroadcastmb2q,// EVEX_Vpbroadcastmb2q_ymm_kr
	Mnemonic::Vpbroadcastmb2q,// EVEX_Vpbroadcastmb2q_zmm_kr
	Mnemonic::Packusdw,// Packusdw_xmm_xmmm128
	Mnemonic::Vpackusdw,// VEX_Vpackusdw_xmm_xmm_xmmm128
	Mnemonic::Vpackusdw,// VEX_Vpackusdw_ymm_ymm_ymmm256
	Mnemonic::Vpackusdw,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpackusdw,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpackusdw,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vmaskmovps,// VEX_Vmaskmovps_xmm_xmm_m128
	Mnemonic::Vmaskmovps,// VEX_Vmaskmovps_ymm_ymm_m256
	Mnemonic::Vscalefps,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vscalefps,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vscalefps,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vscalefpd,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vscalefpd,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vscalefpd,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vmaskmovpd,// VEX_Vmaskmovpd_xmm_xmm_m128
	Mnemonic::Vmaskmovpd,// VEX_Vmaskmovpd_ymm_ymm_m256
	Mnemonic::Vscalefss,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vscalefsd,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vmaskmovps,// VEX_Vmaskmovps_m128_xmm_xmm
	Mnemonic::Vmaskmovps,// VEX_Vmaskmovps_m256_ymm_ymm
	Mnemonic::Vmaskmovpd,// VEX_Vmaskmovpd_m128_xmm_xmm
	Mnemonic::Vmaskmovpd,// VEX_Vmaskmovpd_m256_ymm_ymm
	Mnemonic::Pmovzxbw,// Pmovzxbw_xmm_xmmm64
	Mnemonic::Vpmovzxbw,// VEX_Vpmovzxbw_xmm_xmmm64
	Mnemonic::Vpmovzxbw,// VEX_Vpmovzxbw_ymm_xmmm128
	Mnemonic::Vpmovzxbw,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	Mnemonic::Vpmovzxbw,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	Mnemonic::Vpmovzxbw,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	Mnemonic::Vpmovwb,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	Mnemonic::Vpmovwb,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	Mnemonic::Vpmovwb,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	Mnemonic::Pmovzxbd,// Pmovzxbd_xmm_xmmm32
	Mnemonic::Vpmovzxbd,// VEX_Vpmovzxbd_xmm_xmmm32
	Mnemonic::Vpmovzxbd,// VEX_Vpmovzxbd_ymm_xmmm64
	Mnemonic::Vpmovzxbd,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	Mnemonic::Vpmovzxbd,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	Mnemonic::Vpmovzxbd,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	Mnemonic::Vpmovdb,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	Mnemonic::Vpmovdb,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	Mnemonic::Vpmovdb,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	Mnemonic::Pmovzxbq,// Pmovzxbq_xmm_xmmm16
	Mnemonic::Vpmovzxbq,// VEX_Vpmovzxbq_xmm_xmmm16
	Mnemonic::Vpmovzxbq,// VEX_Vpmovzxbq_ymm_xmmm32
	Mnemonic::Vpmovzxbq,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	Mnemonic::Vpmovzxbq,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	Mnemonic::Vpmovzxbq,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	Mnemonic::Vpmovqb,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	Mnemonic::Vpmovqb,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	Mnemonic::Vpmovqb,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	Mnemonic::Pmovzxwd,// Pmovzxwd_xmm_xmmm64
	Mnemonic::Vpmovzxwd,// VEX_Vpmovzxwd_xmm_xmmm64
	Mnemonic::Vpmovzxwd,// VEX_Vpmovzxwd_ymm_xmmm128
	Mnemonic::Vpmovzxwd,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	Mnemonic::Vpmovzxwd,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	Mnemonic::Vpmovzxwd,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	Mnemonic::Vpmovdw,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	Mnemonic::Vpmovdw,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	Mnemonic::Vpmovdw,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	Mnemonic::Pmovzxwq,// Pmovzxwq_xmm_xmmm32
	Mnemonic::Vpmovzxwq,// VEX_Vpmovzxwq_xmm_xmmm32
	Mnemonic::Vpmovzxwq,// VEX_Vpmovzxwq_ymm_xmmm64
	Mnemonic::Vpmovzxwq,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	Mnemonic::Vpmovzxwq,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	Mnemonic::Vpmovzxwq,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	Mnemonic::Vpmovqw,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	Mnemonic::Vpmovqw,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	Mnemonic::Vpmovqw,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	Mnemonic::Pmovzxdq,// Pmovzxdq_xmm_xmmm64
	Mnemonic::Vpmovzxdq,// VEX_Vpmovzxdq_xmm_xmmm64
	Mnemonic::Vpmovzxdq,// VEX_Vpmovzxdq_ymm_xmmm128
	Mnemonic::Vpmovzxdq,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	Mnemonic::Vpmovzxdq,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	Mnemonic::Vpmovzxdq,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	Mnemonic::Vpmovqd,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	Mnemonic::Vpmovqd,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	Mnemonic::Vpmovqd,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	Mnemonic::Vpermd,// VEX_Vpermd_ymm_ymm_ymmm256
	Mnemonic::Vpermd,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermd,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermq,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermq,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pcmpgtq,// Pcmpgtq_xmm_xmmm128
	Mnemonic::Vpcmpgtq,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	Mnemonic::Vpcmpgtq,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	Mnemonic::Vpcmpgtq,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	Mnemonic::Vpcmpgtq,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	Mnemonic::Vpcmpgtq,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	Mnemonic::Pminsb,// Pminsb_xmm_xmmm128
	Mnemonic::Vpminsb,// VEX_Vpminsb_xmm_xmm_xmmm128
	Mnemonic::Vpminsb,// VEX_Vpminsb_ymm_ymm_ymmm256
	Mnemonic::Vpminsb,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpminsb,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpminsb,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpmovm2d,// EVEX_Vpmovm2d_xmm_kr
	Mnemonic::Vpmovm2d,// EVEX_Vpmovm2d_ymm_kr
	Mnemonic::Vpmovm2d,// EVEX_Vpmovm2d_zmm_kr
	Mnemonic::Vpmovm2q,// EVEX_Vpmovm2q_xmm_kr
	Mnemonic::Vpmovm2q,// EVEX_Vpmovm2q_ymm_kr
	Mnemonic::Vpmovm2q,// EVEX_Vpmovm2q_zmm_kr
	Mnemonic::Pminsd,// Pminsd_xmm_xmmm128
	Mnemonic::Vpminsd,// VEX_Vpminsd_xmm_xmm_xmmm128
	Mnemonic::Vpminsd,// VEX_Vpminsd_ymm_ymm_ymmm256
	Mnemonic::Vpminsd,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpminsd,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpminsd,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpminsq,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpminsq,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpminsq,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpmovd2m,// EVEX_Vpmovd2m_kr_xmm
	Mnemonic::Vpmovd2m,// EVEX_Vpmovd2m_kr_ymm
	Mnemonic::Vpmovd2m,// EVEX_Vpmovd2m_kr_zmm
	Mnemonic::Vpmovq2m,// EVEX_Vpmovq2m_kr_xmm
	Mnemonic::Vpmovq2m,// EVEX_Vpmovq2m_kr_ymm
	Mnemonic::Vpmovq2m,// EVEX_Vpmovq2m_kr_zmm
	Mnemonic::Pminuw,// Pminuw_xmm_xmmm128
	Mnemonic::Vpminuw,// VEX_Vpminuw_xmm_xmm_xmmm128
	Mnemonic::Vpminuw,// VEX_Vpminuw_ymm_ymm_ymmm256
	Mnemonic::Vpminuw,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpminuw,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpminuw,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpbroadcastmw2d,// EVEX_Vpbroadcastmw2d_xmm_kr
	Mnemonic::Vpbroadcastmw2d,// EVEX_Vpbroadcastmw2d_ymm_kr
	Mnemonic::Vpbroadcastmw2d,// EVEX_Vpbroadcastmw2d_zmm_kr
	Mnemonic::Pminud,// Pminud_xmm_xmmm128
	Mnemonic::Vpminud,// VEX_Vpminud_xmm_xmm_xmmm128
	Mnemonic::Vpminud,// VEX_Vpminud_ymm_ymm_ymmm256
	Mnemonic::Vpminud,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpminud,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpminud,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpminuq,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpminuq,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpminuq,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pmaxsb,// Pmaxsb_xmm_xmmm128
	Mnemonic::Vpmaxsb,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	Mnemonic::Vpmaxsb,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	Mnemonic::Vpmaxsb,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaxsb,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaxsb,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmaxsd,// Pmaxsd_xmm_xmmm128
	Mnemonic::Vpmaxsd,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	Mnemonic::Vpmaxsd,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	Mnemonic::Vpmaxsd,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpmaxsd,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpmaxsd,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpmaxsq,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmaxsq,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmaxsq,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pmaxuw,// Pmaxuw_xmm_xmmm128
	Mnemonic::Vpmaxuw,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	Mnemonic::Vpmaxuw,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	Mnemonic::Vpmaxuw,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpmaxuw,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpmaxuw,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	Mnemonic::Pmaxud,// Pmaxud_xmm_xmmm128
	Mnemonic::Vpmaxud,// VEX_Vpmaxud_xmm_xmm_xmmm128
	Mnemonic::Vpmaxud,// VEX_Vpmaxud_ymm_ymm_ymmm256
	Mnemonic::Vpmaxud,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpmaxud,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpmaxud,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpmaxuq,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmaxuq,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmaxuq,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Pmulld,// Pmulld_xmm_xmmm128
	Mnemonic::Vpmulld,// VEX_Vpmulld_xmm_xmm_xmmm128
	Mnemonic::Vpmulld,// VEX_Vpmulld_ymm_ymm_ymmm256
	Mnemonic::Vpmulld,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpmulld,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpmulld,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpmullq,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmullq,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmullq,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Phminposuw,// Phminposuw_xmm_xmmm128
	Mnemonic::Vphminposuw,// VEX_Vphminposuw_xmm_xmmm128
	Mnemonic::Vgetexpps,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	Mnemonic::Vgetexpps,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	Mnemonic::Vgetexpps,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	Mnemonic::Vgetexppd,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	Mnemonic::Vgetexppd,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	Mnemonic::Vgetexppd,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	Mnemonic::Vgetexpss,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Vgetexpsd,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	Mnemonic::Vplzcntd,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	Mnemonic::Vplzcntd,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	Mnemonic::Vplzcntd,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	Mnemonic::Vplzcntq,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	Mnemonic::Vplzcntq,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	Mnemonic::Vplzcntq,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	Mnemonic::Vpsrlvd,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	Mnemonic::Vpsrlvd,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	Mnemonic::Vpsrlvq,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	Mnemonic::Vpsrlvq,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	Mnemonic::Vpsrlvd,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpsrlvd,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpsrlvd,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpsrlvq,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpsrlvq,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpsrlvq,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpsravd,// VEX_Vpsravd_xmm_xmm_xmmm128
	Mnemonic::Vpsravd,// VEX_Vpsravd_ymm_ymm_ymmm256
	Mnemonic::Vpsravd,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpsravd,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpsravd,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpsravq,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpsravq,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpsravq,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpsllvd,// VEX_Vpsllvd_xmm_xmm_xmmm128
	Mnemonic::Vpsllvd,// VEX_Vpsllvd_ymm_ymm_ymmm256
	Mnemonic::Vpsllvq,// VEX_Vpsllvq_xmm_xmm_xmmm128
	Mnemonic::Vpsllvq,// VEX_Vpsllvq_ymm_ymm_ymmm256
	Mnemonic::Vpsllvd,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpsllvd,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpsllvd,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpsllvq,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpsllvq,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpsllvq,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vrcp14ps,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	Mnemonic::Vrcp14ps,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	Mnemonic::Vrcp14ps,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	Mnemonic::Vrcp14pd,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	Mnemonic::Vrcp14pd,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	Mnemonic::Vrcp14pd,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	Mnemonic::Vrcp14ss,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	Mnemonic::Vrcp14sd,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	Mnemonic::Vrsqrt14ps,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	Mnemonic::Vrsqrt14ps,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	Mnemonic::Vrsqrt14ps,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	Mnemonic::Vrsqrt14pd,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	Mnemonic::Vrsqrt14pd,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	Mnemonic::Vrsqrt14pd,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	Mnemonic::Vrsqrt14ss,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	Mnemonic::Vrsqrt14sd,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	Mnemonic::Vpdpbusd,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpdpbusd,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpdpbusd,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpdpbusds,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpdpbusds,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpdpbusds,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpdpwssd,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpdpwssd,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpdpwssd,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vdpbf16ps,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vdpbf16ps,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vdpbf16ps,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vp4dpwssd,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	Mnemonic::Vpdpwssds,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpdpwssds,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpdpwssds,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vp4dpwssds,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	Mnemonic::Vpopcntb,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	Mnemonic::Vpopcntb,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	Mnemonic::Vpopcntb,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	Mnemonic::Vpopcntw,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	Mnemonic::Vpopcntw,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	Mnemonic::Vpopcntw,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	Mnemonic::Vpopcntd,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	Mnemonic::Vpopcntd,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	Mnemonic::Vpopcntd,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	Mnemonic::Vpopcntq,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	Mnemonic::Vpopcntq,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	Mnemonic::Vpopcntq,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	Mnemonic::Vpbroadcastd,// VEX_Vpbroadcastd_xmm_xmmm32
	Mnemonic::Vpbroadcastd,// VEX_Vpbroadcastd_ymm_xmmm32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	Mnemonic::Vpbroadcastq,// VEX_Vpbroadcastq_xmm_xmmm64
	Mnemonic::Vpbroadcastq,// VEX_Vpbroadcastq_ymm_xmmm64
	Mnemonic::Vbroadcasti32x2,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	Mnemonic::Vbroadcasti32x2,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	Mnemonic::Vbroadcasti32x2,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	Mnemonic::Vbroadcasti128,// VEX_Vbroadcasti128_ymm_m128
	Mnemonic::Vbroadcasti32x4,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	Mnemonic::Vbroadcasti32x4,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	Mnemonic::Vbroadcasti64x2,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	Mnemonic::Vbroadcasti64x2,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	Mnemonic::Vbroadcasti32x8,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	Mnemonic::Vbroadcasti64x4,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	Mnemonic::Vpexpandb,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	Mnemonic::Vpexpandb,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	Mnemonic::Vpexpandb,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	Mnemonic::Vpexpandw,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	Mnemonic::Vpexpandw,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	Mnemonic::Vpexpandw,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	Mnemonic::Vpcompressb,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	Mnemonic::Vpcompressb,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	Mnemonic::Vpcompressb,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	Mnemonic::Vpcompressw,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	Mnemonic::Vpcompressw,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	Mnemonic::Vpcompressw,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	Mnemonic::Vpblendmd,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpblendmd,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpblendmd,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpblendmq,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpblendmq,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpblendmq,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vblendmps,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vblendmps,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vblendmps,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vblendmpd,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vblendmpd,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vblendmpd,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpblendmb,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpblendmb,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpblendmb,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpblendmw,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpblendmw,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpblendmw,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vp2intersectd,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	Mnemonic::Vp2intersectd,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	Mnemonic::Vp2intersectd,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	Mnemonic::Vp2intersectq,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	Mnemonic::Vp2intersectq,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	Mnemonic::Vp2intersectq,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	Mnemonic::Vpshldvw,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpshldvw,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpshldvw,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpshldvd,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpshldvd,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpshldvd,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpshldvq,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpshldvq,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpshldvq,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpshrdvw,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpshrdvw,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpshrdvw,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vcvtneps2bf16,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtneps2bf16,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	Mnemonic::Vcvtneps2bf16,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	Mnemonic::Vcvtne2ps2bf16,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vcvtne2ps2bf16,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vcvtne2ps2bf16,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpshrdvd,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpshrdvd,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpshrdvd,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpshrdvq,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpshrdvq,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpshrdvq,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpermi2b,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermi2b,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermi2b,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermi2w,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermi2w,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermi2w,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermi2d,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpermi2d,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermi2d,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermi2q,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpermi2q,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermi2q,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpermi2ps,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpermi2ps,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermi2ps,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermi2pd,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpermi2pd,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermi2pd,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpbroadcastb,// VEX_Vpbroadcastb_xmm_xmmm8
	Mnemonic::Vpbroadcastb,// VEX_Vpbroadcastb_ymm_xmmm8
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	Mnemonic::Vpbroadcastw,// VEX_Vpbroadcastw_xmm_xmmm16
	Mnemonic::Vpbroadcastw,// VEX_Vpbroadcastw_ymm_xmmm16
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_xmm_k1z_r32
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_ymm_k1z_r32
	Mnemonic::Vpbroadcastb,// EVEX_Vpbroadcastb_zmm_k1z_r32
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_xmm_k1z_r32
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_ymm_k1z_r32
	Mnemonic::Vpbroadcastw,// EVEX_Vpbroadcastw_zmm_k1z_r32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_xmm_k1z_r32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_ymm_k1z_r32
	Mnemonic::Vpbroadcastd,// EVEX_Vpbroadcastd_zmm_k1z_r32
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_xmm_k1z_r64
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_ymm_k1z_r64
	Mnemonic::Vpbroadcastq,// EVEX_Vpbroadcastq_zmm_k1z_r64
	Mnemonic::Vpermt2b,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermt2b,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermt2b,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermt2w,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermt2w,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermt2w,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermt2d,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpermt2d,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermt2d,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermt2q,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpermt2q,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermt2q,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpermt2ps,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vpermt2ps,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vpermt2ps,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	Mnemonic::Vpermt2pd,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpermt2pd,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpermt2pd,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Invept,// Invept_r32_m128
	Mnemonic::Invept,// Invept_r64_m128
	Mnemonic::Invvpid,// Invvpid_r32_m128
	Mnemonic::Invvpid,// Invvpid_r64_m128
	Mnemonic::Invpcid,// Invpcid_r32_m128
	Mnemonic::Invpcid,// Invpcid_r64_m128
	Mnemonic::Vpmultishiftqb,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmultishiftqb,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmultishiftqb,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vexpandps,// EVEX_Vexpandps_xmm_k1z_xmmm128
	Mnemonic::Vexpandps,// EVEX_Vexpandps_ymm_k1z_ymmm256
	Mnemonic::Vexpandps,// EVEX_Vexpandps_zmm_k1z_zmmm512
	Mnemonic::Vexpandpd,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	Mnemonic::Vexpandpd,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	Mnemonic::Vexpandpd,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	Mnemonic::Vpexpandd,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	Mnemonic::Vpexpandd,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	Mnemonic::Vpexpandd,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	Mnemonic::Vpexpandq,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	Mnemonic::Vpexpandq,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	Mnemonic::Vpexpandq,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	Mnemonic::Vcompressps,// EVEX_Vcompressps_xmmm128_k1z_xmm
	Mnemonic::Vcompressps,// EVEX_Vcompressps_ymmm256_k1z_ymm
	Mnemonic::Vcompressps,// EVEX_Vcompressps_zmmm512_k1z_zmm
	Mnemonic::Vcompresspd,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	Mnemonic::Vcompresspd,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	Mnemonic::Vcompresspd,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	Mnemonic::Vpcompressd,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	Mnemonic::Vpcompressd,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	Mnemonic::Vpcompressd,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	Mnemonic::Vpcompressq,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	Mnemonic::Vpcompressq,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	Mnemonic::Vpcompressq,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	Mnemonic::Vpmaskmovd,// VEX_Vpmaskmovd_xmm_xmm_m128
	Mnemonic::Vpmaskmovd,// VEX_Vpmaskmovd_ymm_ymm_m256
	Mnemonic::Vpmaskmovq,// VEX_Vpmaskmovq_xmm_xmm_m128
	Mnemonic::Vpmaskmovq,// VEX_Vpmaskmovq_ymm_ymm_m256
	Mnemonic::Vpermb,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermb,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermb,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpermw,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	Mnemonic::Vpermw,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	Mnemonic::Vpermw,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	Mnemonic::Vpmaskmovd,// VEX_Vpmaskmovd_m128_xmm_xmm
	Mnemonic::Vpmaskmovd,// VEX_Vpmaskmovd_m256_ymm_ymm
	Mnemonic::Vpmaskmovq,// VEX_Vpmaskmovq_m128_xmm_xmm
	Mnemonic::Vpmaskmovq,// VEX_Vpmaskmovq_m256_ymm_ymm
	Mnemonic::Vpshufbitqmb,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	Mnemonic::Vpshufbitqmb,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	Mnemonic::Vpshufbitqmb,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	Mnemonic::Vpgatherdd,// VEX_Vpgatherdd_xmm_vm32x_xmm
	Mnemonic::Vpgatherdd,// VEX_Vpgatherdd_ymm_vm32y_ymm
	Mnemonic::Vpgatherdq,// VEX_Vpgatherdq_xmm_vm32x_xmm
	Mnemonic::Vpgatherdq,// VEX_Vpgatherdq_ymm_vm32x_ymm
	Mnemonic::Vpgatherdd,// EVEX_Vpgatherdd_xmm_k1_vm32x
	Mnemonic::Vpgatherdd,// EVEX_Vpgatherdd_ymm_k1_vm32y
	Mnemonic::Vpgatherdd,// EVEX_Vpgatherdd_zmm_k1_vm32z
	Mnemonic::Vpgatherdq,// EVEX_Vpgatherdq_xmm_k1_vm32x
	Mnemonic::Vpgatherdq,// EVEX_Vpgatherdq_ymm_k1_vm32x
	Mnemonic::Vpgatherdq,// EVEX_Vpgatherdq_zmm_k1_vm32y
	Mnemonic::Vpgatherqd,// VEX_Vpgatherqd_xmm_vm64x_xmm
	Mnemonic::Vpgatherqd,// VEX_Vpgatherqd_xmm_vm64y_xmm
	Mnemonic::Vpgatherqq,// VEX_Vpgatherqq_xmm_vm64x_xmm
	Mnemonic::Vpgatherqq,// VEX_Vpgatherqq_ymm_vm64y_ymm
	Mnemonic::Vpgatherqd,// EVEX_Vpgatherqd_xmm_k1_vm64x
	Mnemonic::Vpgatherqd,// EVEX_Vpgatherqd_xmm_k1_vm64y
	Mnemonic::Vpgatherqd,// EVEX_Vpgatherqd_ymm_k1_vm64z
	Mnemonic::Vpgatherqq,// EVEX_Vpgatherqq_xmm_k1_vm64x
	Mnemonic::Vpgatherqq,// EVEX_Vpgatherqq_ymm_k1_vm64y
	Mnemonic::Vpgatherqq,// EVEX_Vpgatherqq_zmm_k1_vm64z
	Mnemonic::Vgatherdps,// VEX_Vgatherdps_xmm_vm32x_xmm
	Mnemonic::Vgatherdps,// VEX_Vgatherdps_ymm_vm32y_ymm
	Mnemonic::Vgatherdpd,// VEX_Vgatherdpd_xmm_vm32x_xmm
	Mnemonic::Vgatherdpd,// VEX_Vgatherdpd_ymm_vm32x_ymm
	Mnemonic::Vgatherdps,// EVEX_Vgatherdps_xmm_k1_vm32x
	Mnemonic::Vgatherdps,// EVEX_Vgatherdps_ymm_k1_vm32y
	Mnemonic::Vgatherdps,// EVEX_Vgatherdps_zmm_k1_vm32z
	Mnemonic::Vgatherdpd,// EVEX_Vgatherdpd_xmm_k1_vm32x
	Mnemonic::Vgatherdpd,// EVEX_Vgatherdpd_ymm_k1_vm32x
	Mnemonic::Vgatherdpd,// EVEX_Vgatherdpd_zmm_k1_vm32y
	Mnemonic::Vgatherqps,// VEX_Vgatherqps_xmm_vm64x_xmm
	Mnemonic::Vgatherqps,// VEX_Vgatherqps_xmm_vm64y_xmm
	Mnemonic::Vgatherqpd,// VEX_Vgatherqpd_xmm_vm64x_xmm
	Mnemonic::Vgatherqpd,// VEX_Vgatherqpd_ymm_vm64y_ymm
	Mnemonic::Vgatherqps,// EVEX_Vgatherqps_xmm_k1_vm64x
	Mnemonic::Vgatherqps,// EVEX_Vgatherqps_xmm_k1_vm64y
	Mnemonic::Vgatherqps,// EVEX_Vgatherqps_ymm_k1_vm64z
	Mnemonic::Vgatherqpd,// EVEX_Vgatherqpd_xmm_k1_vm64x
	Mnemonic::Vgatherqpd,// EVEX_Vgatherqpd_ymm_k1_vm64y
	Mnemonic::Vgatherqpd,// EVEX_Vgatherqpd_zmm_k1_vm64z
	Mnemonic::Vfmaddsub132ps,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub132ps,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub132pd,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub132pd,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub132ps,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmaddsub132ps,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmaddsub132ps,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmaddsub132pd,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmaddsub132pd,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmaddsub132pd,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmsubadd132ps,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd132ps,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd132pd,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd132pd,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd132ps,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsubadd132ps,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsubadd132ps,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsubadd132pd,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsubadd132pd,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsubadd132pd,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd132ps,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	Mnemonic::Vfmadd132ps,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	Mnemonic::Vfmadd132pd,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	Mnemonic::Vfmadd132pd,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	Mnemonic::Vfmadd132ps,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmadd132ps,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmadd132ps,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmadd132pd,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmadd132pd,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmadd132pd,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd132ss,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	Mnemonic::Vfmadd132sd,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	Mnemonic::Vfmadd132ss,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmadd132sd,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfmsub132ps,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsub132ps,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsub132pd,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsub132pd,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsub132ps,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsub132ps,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsub132ps,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsub132pd,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsub132pd,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsub132pd,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::V4fmaddps,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	Mnemonic::Vfmsub132ss,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	Mnemonic::Vfmsub132sd,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	Mnemonic::Vfmsub132ss,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmsub132sd,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::V4fmaddss,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	Mnemonic::Vfnmadd132ps,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd132ps,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd132pd,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd132pd,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd132ps,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmadd132ps,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmadd132ps,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmadd132pd,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmadd132pd,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmadd132pd,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmadd132ss,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmadd132sd,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmadd132ss,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmadd132sd,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfnmsub132ps,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub132ps,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub132pd,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub132pd,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub132ps,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmsub132ps,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmsub132ps,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmsub132pd,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmsub132pd,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmsub132pd,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmsub132ss,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmsub132sd,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmsub132ss,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmsub132sd,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vpscatterdd,// EVEX_Vpscatterdd_vm32x_k1_xmm
	Mnemonic::Vpscatterdd,// EVEX_Vpscatterdd_vm32y_k1_ymm
	Mnemonic::Vpscatterdd,// EVEX_Vpscatterdd_vm32z_k1_zmm
	Mnemonic::Vpscatterdq,// EVEX_Vpscatterdq_vm32x_k1_xmm
	Mnemonic::Vpscatterdq,// EVEX_Vpscatterdq_vm32x_k1_ymm
	Mnemonic::Vpscatterdq,// EVEX_Vpscatterdq_vm32y_k1_zmm
	Mnemonic::Vpscatterqd,// EVEX_Vpscatterqd_vm64x_k1_xmm
	Mnemonic::Vpscatterqd,// EVEX_Vpscatterqd_vm64y_k1_xmm
	Mnemonic::Vpscatterqd,// EVEX_Vpscatterqd_vm64z_k1_ymm
	Mnemonic::Vpscatterqq,// EVEX_Vpscatterqq_vm64x_k1_xmm
	Mnemonic::Vpscatterqq,// EVEX_Vpscatterqq_vm64y_k1_ymm
	Mnemonic::Vpscatterqq,// EVEX_Vpscatterqq_vm64z_k1_zmm
	Mnemonic::Vscatterdps,// EVEX_Vscatterdps_vm32x_k1_xmm
	Mnemonic::Vscatterdps,// EVEX_Vscatterdps_vm32y_k1_ymm
	Mnemonic::Vscatterdps,// EVEX_Vscatterdps_vm32z_k1_zmm
	Mnemonic::Vscatterdpd,// EVEX_Vscatterdpd_vm32x_k1_xmm
	Mnemonic::Vscatterdpd,// EVEX_Vscatterdpd_vm32x_k1_ymm
	Mnemonic::Vscatterdpd,// EVEX_Vscatterdpd_vm32y_k1_zmm
	Mnemonic::Vscatterqps,// EVEX_Vscatterqps_vm64x_k1_xmm
	Mnemonic::Vscatterqps,// EVEX_Vscatterqps_vm64y_k1_xmm
	Mnemonic::Vscatterqps,// EVEX_Vscatterqps_vm64z_k1_ymm
	Mnemonic::Vscatterqpd,// EVEX_Vscatterqpd_vm64x_k1_xmm
	Mnemonic::Vscatterqpd,// EVEX_Vscatterqpd_vm64y_k1_ymm
	Mnemonic::Vscatterqpd,// EVEX_Vscatterqpd_vm64z_k1_zmm
	Mnemonic::Vfmaddsub213ps,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub213ps,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub213pd,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub213pd,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub213ps,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmaddsub213ps,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmaddsub213ps,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmaddsub213pd,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmaddsub213pd,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmaddsub213pd,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmsubadd213ps,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd213ps,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd213pd,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd213pd,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd213ps,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsubadd213ps,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsubadd213ps,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsubadd213pd,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsubadd213pd,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsubadd213pd,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd213ps,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	Mnemonic::Vfmadd213ps,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	Mnemonic::Vfmadd213pd,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	Mnemonic::Vfmadd213pd,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	Mnemonic::Vfmadd213ps,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmadd213ps,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmadd213ps,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmadd213pd,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmadd213pd,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmadd213pd,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd213ss,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	Mnemonic::Vfmadd213sd,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	Mnemonic::Vfmadd213ss,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmadd213sd,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfmsub213ps,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsub213ps,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsub213pd,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsub213pd,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsub213ps,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsub213ps,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsub213ps,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsub213pd,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsub213pd,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsub213pd,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::V4fnmaddps,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	Mnemonic::Vfmsub213ss,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	Mnemonic::Vfmsub213sd,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	Mnemonic::Vfmsub213ss,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmsub213sd,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::V4fnmaddss,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	Mnemonic::Vfnmadd213ps,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd213ps,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd213pd,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd213pd,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd213ps,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmadd213ps,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmadd213ps,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmadd213pd,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmadd213pd,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmadd213pd,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmadd213ss,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmadd213sd,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmadd213ss,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmadd213sd,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfnmsub213ps,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub213ps,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub213pd,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub213pd,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub213ps,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmsub213ps,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmsub213ps,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmsub213pd,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmsub213pd,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmsub213pd,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmsub213ss,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmsub213sd,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmsub213ss,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmsub213sd,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vpmadd52luq,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmadd52luq,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmadd52luq,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vpmadd52huq,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vpmadd52huq,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vpmadd52huq,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	Mnemonic::Vfmaddsub231ps,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub231ps,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub231pd,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsub231pd,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsub231ps,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmaddsub231ps,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmaddsub231ps,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmaddsub231pd,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmaddsub231pd,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmaddsub231pd,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmsubadd231ps,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd231ps,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd231pd,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsubadd231pd,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsubadd231ps,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsubadd231ps,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsubadd231ps,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsubadd231pd,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsubadd231pd,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsubadd231pd,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd231ps,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	Mnemonic::Vfmadd231ps,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	Mnemonic::Vfmadd231pd,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	Mnemonic::Vfmadd231pd,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	Mnemonic::Vfmadd231ps,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmadd231ps,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmadd231ps,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmadd231pd,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmadd231pd,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmadd231pd,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmadd231ss,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	Mnemonic::Vfmadd231sd,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	Mnemonic::Vfmadd231ss,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmadd231sd,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfmsub231ps,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	Mnemonic::Vfmsub231ps,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	Mnemonic::Vfmsub231pd,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	Mnemonic::Vfmsub231pd,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	Mnemonic::Vfmsub231ps,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmsub231ps,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmsub231ps,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmsub231pd,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfmsub231pd,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfmsub231pd,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfmsub231ss,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	Mnemonic::Vfmsub231sd,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	Mnemonic::Vfmsub231ss,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmsub231sd,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfnmadd231ps,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd231ps,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd231pd,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmadd231pd,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmadd231ps,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmadd231ps,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmadd231ps,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmadd231pd,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmadd231pd,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmadd231pd,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmadd231ss,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmadd231sd,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmadd231ss,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmadd231sd,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vfnmsub231ps,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub231ps,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub231pd,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	Mnemonic::Vfnmsub231pd,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	Mnemonic::Vfnmsub231ps,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfnmsub231ps,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfnmsub231ps,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfnmsub231pd,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	Mnemonic::Vfnmsub231pd,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	Mnemonic::Vfnmsub231pd,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	Mnemonic::Vfnmsub231ss,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	Mnemonic::Vfnmsub231sd,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	Mnemonic::Vfnmsub231ss,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfnmsub231sd,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vpconflictd,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	Mnemonic::Vpconflictd,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	Mnemonic::Vpconflictd,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	Mnemonic::Vpconflictq,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	Mnemonic::Vpconflictq,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	Mnemonic::Vpconflictq,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	Mnemonic::Vgatherpf0dps,// EVEX_Vgatherpf0dps_vm32z_k1
	Mnemonic::Vgatherpf0dpd,// EVEX_Vgatherpf0dpd_vm32y_k1
	Mnemonic::Vgatherpf1dps,// EVEX_Vgatherpf1dps_vm32z_k1
	Mnemonic::Vgatherpf1dpd,// EVEX_Vgatherpf1dpd_vm32y_k1
	Mnemonic::Vscatterpf0dps,// EVEX_Vscatterpf0dps_vm32z_k1
	Mnemonic::Vscatterpf0dpd,// EVEX_Vscatterpf0dpd_vm32y_k1
	Mnemonic::Vscatterpf1dps,// EVEX_Vscatterpf1dps_vm32z_k1
	Mnemonic::Vscatterpf1dpd,// EVEX_Vscatterpf1dpd_vm32y_k1
	Mnemonic::Vgatherpf0qps,// EVEX_Vgatherpf0qps_vm64z_k1
	Mnemonic::Vgatherpf0qpd,// EVEX_Vgatherpf0qpd_vm64z_k1
	Mnemonic::Vgatherpf1qps,// EVEX_Vgatherpf1qps_vm64z_k1
	Mnemonic::Vgatherpf1qpd,// EVEX_Vgatherpf1qpd_vm64z_k1
	Mnemonic::Vscatterpf0qps,// EVEX_Vscatterpf0qps_vm64z_k1
	Mnemonic::Vscatterpf0qpd,// EVEX_Vscatterpf0qpd_vm64z_k1
	Mnemonic::Vscatterpf1qps,// EVEX_Vscatterpf1qps_vm64z_k1
	Mnemonic::Vscatterpf1qpd,// EVEX_Vscatterpf1qpd_vm64z_k1
	Mnemonic::Sha1nexte,// Sha1nexte_xmm_xmmm128
	Mnemonic::Vexp2ps,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	Mnemonic::Vexp2pd,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	Mnemonic::Sha1msg1,// Sha1msg1_xmm_xmmm128
	Mnemonic::Sha1msg2,// Sha1msg2_xmm_xmmm128
	Mnemonic::Vrcp28ps,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	Mnemonic::Vrcp28pd,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	Mnemonic::Sha256rnds2,// Sha256rnds2_xmm_xmmm128
	Mnemonic::Vrcp28ss,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Vrcp28sd,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	Mnemonic::Sha256msg1,// Sha256msg1_xmm_xmmm128
	Mnemonic::Vrsqrt28ps,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	Mnemonic::Vrsqrt28pd,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	Mnemonic::Sha256msg2,// Sha256msg2_xmm_xmmm128
	Mnemonic::Vrsqrt28ss,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	Mnemonic::Vrsqrt28sd,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	Mnemonic::Gf2p8mulb,// Gf2p8mulb_xmm_xmmm128
	Mnemonic::Vgf2p8mulb,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	Mnemonic::Vgf2p8mulb,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	Mnemonic::Vgf2p8mulb,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	Mnemonic::Vgf2p8mulb,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	Mnemonic::Vgf2p8mulb,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	Mnemonic::Aesimc,// Aesimc_xmm_xmmm128
	Mnemonic::Vaesimc,// VEX_Vaesimc_xmm_xmmm128
	Mnemonic::Aesenc,// Aesenc_xmm_xmmm128
	Mnemonic::Vaesenc,// VEX_Vaesenc_xmm_xmm_xmmm128
	Mnemonic::Vaesenc,// VEX_Vaesenc_ymm_ymm_ymmm256
	Mnemonic::Vaesenc,// EVEX_Vaesenc_xmm_xmm_xmmm128
	Mnemonic::Vaesenc,// EVEX_Vaesenc_ymm_ymm_ymmm256
	Mnemonic::Vaesenc,// EVEX_Vaesenc_zmm_zmm_zmmm512
	Mnemonic::Aesenclast,// Aesenclast_xmm_xmmm128
	Mnemonic::Vaesenclast,// VEX_Vaesenclast_xmm_xmm_xmmm128
	Mnemonic::Vaesenclast,// VEX_Vaesenclast_ymm_ymm_ymmm256
	Mnemonic::Vaesenclast,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	Mnemonic::Vaesenclast,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	Mnemonic::Vaesenclast,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	Mnemonic::Aesdec,// Aesdec_xmm_xmmm128
	Mnemonic::Vaesdec,// VEX_Vaesdec_xmm_xmm_xmmm128
	Mnemonic::Vaesdec,// VEX_Vaesdec_ymm_ymm_ymmm256
	Mnemonic::Vaesdec,// EVEX_Vaesdec_xmm_xmm_xmmm128
	Mnemonic::Vaesdec,// EVEX_Vaesdec_ymm_ymm_ymmm256
	Mnemonic::Vaesdec,// EVEX_Vaesdec_zmm_zmm_zmmm512
	Mnemonic::Aesdeclast,// Aesdeclast_xmm_xmmm128
	Mnemonic::Vaesdeclast,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	Mnemonic::Vaesdeclast,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	Mnemonic::Vaesdeclast,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	Mnemonic::Vaesdeclast,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	Mnemonic::Vaesdeclast,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	Mnemonic::Movbe,// Movbe_r16_m16
	Mnemonic::Movbe,// Movbe_r32_m32
	Mnemonic::Movbe,// Movbe_r64_m64
	Mnemonic::Crc32,// Crc32_r32_rm8
	Mnemonic::Crc32,// Crc32_r64_rm8
	Mnemonic::Movbe,// Movbe_m16_r16
	Mnemonic::Movbe,// Movbe_m32_r32
	Mnemonic::Movbe,// Movbe_m64_r64
	Mnemonic::Crc32,// Crc32_r32_rm16
	Mnemonic::Crc32,// Crc32_r32_rm32
	Mnemonic::Crc32,// Crc32_r64_rm64
	Mnemonic::Andn,// VEX_Andn_r32_r32_rm32
	Mnemonic::Andn,// VEX_Andn_r64_r64_rm64
	Mnemonic::Blsr,// VEX_Blsr_r32_rm32
	Mnemonic::Blsr,// VEX_Blsr_r64_rm64
	Mnemonic::Blsmsk,// VEX_Blsmsk_r32_rm32
	Mnemonic::Blsmsk,// VEX_Blsmsk_r64_rm64
	Mnemonic::Blsi,// VEX_Blsi_r32_rm32
	Mnemonic::Blsi,// VEX_Blsi_r64_rm64
	Mnemonic::Bzhi,// VEX_Bzhi_r32_rm32_r32
	Mnemonic::Bzhi,// VEX_Bzhi_r64_rm64_r64
	Mnemonic::Wrussd,// Wrussd_m32_r32
	Mnemonic::Wrussq,// Wrussq_m64_r64
	Mnemonic::Pext,// VEX_Pext_r32_r32_rm32
	Mnemonic::Pext,// VEX_Pext_r64_r64_rm64
	Mnemonic::Pdep,// VEX_Pdep_r32_r32_rm32
	Mnemonic::Pdep,// VEX_Pdep_r64_r64_rm64
	Mnemonic::Wrssd,// Wrssd_m32_r32
	Mnemonic::Wrssq,// Wrssq_m64_r64
	Mnemonic::Adcx,// Adcx_r32_rm32
	Mnemonic::Adcx,// Adcx_r64_rm64
	Mnemonic::Adox,// Adox_r32_rm32
	Mnemonic::Adox,// Adox_r64_rm64
	Mnemonic::Mulx,// VEX_Mulx_r32_r32_rm32
	Mnemonic::Mulx,// VEX_Mulx_r64_r64_rm64
	Mnemonic::Bextr,// VEX_Bextr_r32_rm32_r32
	Mnemonic::Bextr,// VEX_Bextr_r64_rm64_r64
	Mnemonic::Shlx,// VEX_Shlx_r32_rm32_r32
	Mnemonic::Shlx,// VEX_Shlx_r64_rm64_r64
	Mnemonic::Sarx,// VEX_Sarx_r32_rm32_r32
	Mnemonic::Sarx,// VEX_Sarx_r64_rm64_r64
	Mnemonic::Shrx,// VEX_Shrx_r32_rm32_r32
	Mnemonic::Shrx,// VEX_Shrx_r64_rm64_r64
	Mnemonic::Movdir64b,// Movdir64b_r16_m512
	Mnemonic::Movdir64b,// Movdir64b_r32_m512
	Mnemonic::Movdir64b,// Movdir64b_r64_m512
	Mnemonic::Enqcmds,// Enqcmds_r16_m512
	Mnemonic::Enqcmds,// Enqcmds_r32_m512
	Mnemonic::Enqcmds,// Enqcmds_r64_m512
	Mnemonic::Enqcmd,// Enqcmd_r16_m512
	Mnemonic::Enqcmd,// Enqcmd_r32_m512
	Mnemonic::Enqcmd,// Enqcmd_r64_m512
	Mnemonic::Movdiri,// Movdiri_m32_r32
	Mnemonic::Movdiri,// Movdiri_m64_r64
	Mnemonic::Vpermq,// VEX_Vpermq_ymm_ymmm256_imm8
	Mnemonic::Vpermq,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpermq,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Vpermpd,// VEX_Vpermpd_ymm_ymmm256_imm8
	Mnemonic::Vpermpd,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpermpd,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Vpblendd,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpblendd,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	Mnemonic::Valignd,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Valignd,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Valignd,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Valignq,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Valignq,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Valignq,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Vpermilps,// VEX_Vpermilps_xmm_xmmm128_imm8
	Mnemonic::Vpermilps,// VEX_Vpermilps_ymm_ymmm256_imm8
	Mnemonic::Vpermilps,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vpermilps,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vpermilps,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	Mnemonic::Vpermilpd,// VEX_Vpermilpd_xmm_xmmm128_imm8
	Mnemonic::Vpermilpd,// VEX_Vpermilpd_ymm_ymmm256_imm8
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vpermilpd,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	Mnemonic::Vperm2f128,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	Mnemonic::Roundps,// Roundps_xmm_xmmm128_imm8
	Mnemonic::Vroundps,// VEX_Vroundps_xmm_xmmm128_imm8
	Mnemonic::Vroundps,// VEX_Vroundps_ymm_ymmm256_imm8
	Mnemonic::Vrndscaleps,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vrndscaleps,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vrndscaleps,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	Mnemonic::Roundpd,// Roundpd_xmm_xmmm128_imm8
	Mnemonic::Vroundpd,// VEX_Vroundpd_xmm_xmmm128_imm8
	Mnemonic::Vroundpd,// VEX_Vroundpd_ymm_ymmm256_imm8
	Mnemonic::Vrndscalepd,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vrndscalepd,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vrndscalepd,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	Mnemonic::Roundss,// Roundss_xmm_xmmm32_imm8
	Mnemonic::Vroundss,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	Mnemonic::Vrndscaless,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	Mnemonic::Roundsd,// Roundsd_xmm_xmmm64_imm8
	Mnemonic::Vroundsd,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	Mnemonic::Vrndscalesd,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	Mnemonic::Blendps,// Blendps_xmm_xmmm128_imm8
	Mnemonic::Vblendps,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	Mnemonic::Vblendps,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	Mnemonic::Blendpd,// Blendpd_xmm_xmmm128_imm8
	Mnemonic::Vblendpd,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	Mnemonic::Vblendpd,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	Mnemonic::Pblendw,// Pblendw_xmm_xmmm128_imm8
	Mnemonic::Vpblendw,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpblendw,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	Mnemonic::Palignr,// Palignr_mm_mmm64_imm8
	Mnemonic::Palignr,// Palignr_xmm_xmmm128_imm8
	Mnemonic::Vpalignr,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpalignr,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	Mnemonic::Vpalignr,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	Mnemonic::Vpalignr,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	Mnemonic::Vpalignr,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	Mnemonic::Pextrb,// Pextrb_r32m8_xmm_imm8
	Mnemonic::Pextrb,// Pextrb_r64m8_xmm_imm8
	Mnemonic::Vpextrb,// VEX_Vpextrb_r32m8_xmm_imm8
	Mnemonic::Vpextrb,// VEX_Vpextrb_r64m8_xmm_imm8
	Mnemonic::Vpextrb,// EVEX_Vpextrb_r32m8_xmm_imm8
	Mnemonic::Vpextrb,// EVEX_Vpextrb_r64m8_xmm_imm8
	Mnemonic::Pextrw,// Pextrw_r32m16_xmm_imm8
	Mnemonic::Pextrw,// Pextrw_r64m16_xmm_imm8
	Mnemonic::Vpextrw,// VEX_Vpextrw_r32m16_xmm_imm8
	Mnemonic::Vpextrw,// VEX_Vpextrw_r64m16_xmm_imm8
	Mnemonic::Vpextrw,// EVEX_Vpextrw_r32m16_xmm_imm8
	Mnemonic::Vpextrw,// EVEX_Vpextrw_r64m16_xmm_imm8
	Mnemonic::Pextrd,// Pextrd_rm32_xmm_imm8
	Mnemonic::Pextrq,// Pextrq_rm64_xmm_imm8
	Mnemonic::Vpextrd,// VEX_Vpextrd_rm32_xmm_imm8
	Mnemonic::Vpextrq,// VEX_Vpextrq_rm64_xmm_imm8
	Mnemonic::Vpextrd,// EVEX_Vpextrd_rm32_xmm_imm8
	Mnemonic::Vpextrq,// EVEX_Vpextrq_rm64_xmm_imm8
	Mnemonic::Extractps,// Extractps_rm32_xmm_imm8
	Mnemonic::Extractps,// Extractps_r64m32_xmm_imm8
	Mnemonic::Vextractps,// VEX_Vextractps_rm32_xmm_imm8
	Mnemonic::Vextractps,// VEX_Vextractps_r64m32_xmm_imm8
	Mnemonic::Vextractps,// EVEX_Vextractps_rm32_xmm_imm8
	Mnemonic::Vextractps,// EVEX_Vextractps_r64m32_xmm_imm8
	Mnemonic::Vinsertf128,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	Mnemonic::Vinsertf32x4,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	Mnemonic::Vinsertf32x4,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	Mnemonic::Vinsertf64x2,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	Mnemonic::Vinsertf64x2,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	Mnemonic::Vextractf128,// VEX_Vextractf128_xmmm128_ymm_imm8
	Mnemonic::Vextractf32x4,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	Mnemonic::Vextractf32x4,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	Mnemonic::Vextractf64x2,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	Mnemonic::Vextractf64x2,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	Mnemonic::Vinsertf32x8,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	Mnemonic::Vinsertf64x4,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	Mnemonic::Vextractf32x8,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	Mnemonic::Vextractf64x4,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	Mnemonic::Vcvtps2ph,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	Mnemonic::Vcvtps2ph,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	Mnemonic::Vcvtps2ph,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	Mnemonic::Vcvtps2ph,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	Mnemonic::Vcvtps2ph,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	Mnemonic::Vpcmpud,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	Mnemonic::Vpcmpud,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	Mnemonic::Vpcmpud,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	Mnemonic::Vpcmpuq,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	Mnemonic::Vpcmpuq,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	Mnemonic::Vpcmpuq,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	Mnemonic::Vpcmpd,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	Mnemonic::Vpcmpd,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	Mnemonic::Vpcmpd,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	Mnemonic::Vpcmpq,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	Mnemonic::Vpcmpq,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	Mnemonic::Vpcmpq,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	Mnemonic::Pinsrb,// Pinsrb_xmm_r32m8_imm8
	Mnemonic::Pinsrb,// Pinsrb_xmm_r64m8_imm8
	Mnemonic::Vpinsrb,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	Mnemonic::Vpinsrb,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	Mnemonic::Vpinsrb,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	Mnemonic::Vpinsrb,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	Mnemonic::Insertps,// Insertps_xmm_xmmm32_imm8
	Mnemonic::Vinsertps,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	Mnemonic::Vinsertps,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	Mnemonic::Pinsrd,// Pinsrd_xmm_rm32_imm8
	Mnemonic::Pinsrq,// Pinsrq_xmm_rm64_imm8
	Mnemonic::Vpinsrd,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	Mnemonic::Vpinsrq,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	Mnemonic::Vpinsrd,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	Mnemonic::Vpinsrq,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	Mnemonic::Vshuff32x4,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vshuff32x4,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Vshuff64x2,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vshuff64x2,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Vpternlogd,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vpternlogd,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vpternlogd,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Vpternlogq,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vpternlogq,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vpternlogq,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Vgetmantps,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vgetmantps,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vgetmantps,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	Mnemonic::Vgetmantpd,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vgetmantpd,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vgetmantpd,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	Mnemonic::Vgetmantss,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	Mnemonic::Vgetmantsd,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	Mnemonic::Kshiftrb,// VEX_Kshiftrb_kr_kr_imm8
	Mnemonic::Kshiftrw,// VEX_Kshiftrw_kr_kr_imm8
	Mnemonic::Kshiftrd,// VEX_Kshiftrd_kr_kr_imm8
	Mnemonic::Kshiftrq,// VEX_Kshiftrq_kr_kr_imm8
	Mnemonic::Kshiftlb,// VEX_Kshiftlb_kr_kr_imm8
	Mnemonic::Kshiftlw,// VEX_Kshiftlw_kr_kr_imm8
	Mnemonic::Kshiftld,// VEX_Kshiftld_kr_kr_imm8
	Mnemonic::Kshiftlq,// VEX_Kshiftlq_kr_kr_imm8
	Mnemonic::Vinserti128,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	Mnemonic::Vinserti32x4,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	Mnemonic::Vinserti32x4,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	Mnemonic::Vinserti64x2,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	Mnemonic::Vinserti64x2,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	Mnemonic::Vextracti128,// VEX_Vextracti128_xmmm128_ymm_imm8
	Mnemonic::Vextracti32x4,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	Mnemonic::Vextracti32x4,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	Mnemonic::Vextracti64x2,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	Mnemonic::Vextracti64x2,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	Mnemonic::Vinserti32x8,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	Mnemonic::Vinserti64x4,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	Mnemonic::Vextracti32x8,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	Mnemonic::Vextracti64x4,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	Mnemonic::Vpcmpub,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	Mnemonic::Vpcmpub,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	Mnemonic::Vpcmpub,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	Mnemonic::Vpcmpuw,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	Mnemonic::Vpcmpuw,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	Mnemonic::Vpcmpuw,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	Mnemonic::Vpcmpb,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	Mnemonic::Vpcmpb,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	Mnemonic::Vpcmpb,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	Mnemonic::Vpcmpw,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	Mnemonic::Vpcmpw,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	Mnemonic::Vpcmpw,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	Mnemonic::Dpps,// Dpps_xmm_xmmm128_imm8
	Mnemonic::Vdpps,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	Mnemonic::Vdpps,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	Mnemonic::Dppd,// Dppd_xmm_xmmm128_imm8
	Mnemonic::Vdppd,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	Mnemonic::Mpsadbw,// Mpsadbw_xmm_xmmm128_imm8
	Mnemonic::Vmpsadbw,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	Mnemonic::Vmpsadbw,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	Mnemonic::Vdbpsadbw,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	Mnemonic::Vdbpsadbw,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	Mnemonic::Vdbpsadbw,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	Mnemonic::Vshufi32x4,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vshufi32x4,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Vshufi64x2,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vshufi64x2,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Pclmulqdq,// Pclmulqdq_xmm_xmmm128_imm8
	Mnemonic::Vpclmulqdq,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpclmulqdq,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	Mnemonic::Vpclmulqdq,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpclmulqdq,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	Mnemonic::Vpclmulqdq,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	Mnemonic::Vperm2i128,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	Mnemonic::Vpermil2ps,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	Mnemonic::Vpermil2ps,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	Mnemonic::Vpermil2ps,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	Mnemonic::Vpermil2ps,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	Mnemonic::Vpermil2pd,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	Mnemonic::Vpermil2pd,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	Mnemonic::Vpermil2pd,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	Mnemonic::Vpermil2pd,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	Mnemonic::Vblendvps,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vblendvps,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vblendvpd,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vblendvpd,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vpblendvb,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpblendvb,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	Mnemonic::Vrangeps,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vrangeps,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vrangeps,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	Mnemonic::Vrangepd,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vrangepd,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vrangepd,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	Mnemonic::Vrangess,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	Mnemonic::Vrangesd,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	Mnemonic::Vfixupimmps,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vfixupimmps,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vfixupimmps,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	Mnemonic::Vfixupimmpd,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vfixupimmpd,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vfixupimmpd,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	Mnemonic::Vfixupimmss,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	Mnemonic::Vfixupimmsd,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	Mnemonic::Vreduceps,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	Mnemonic::Vreduceps,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	Mnemonic::Vreduceps,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	Mnemonic::Vreducepd,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	Mnemonic::Vreducepd,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	Mnemonic::Vreducepd,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	Mnemonic::Vreducess,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	Mnemonic::Vreducesd,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	Mnemonic::Vfmaddsubps,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmaddsubps,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmaddsubps,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsubps,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmaddsubpd,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmaddsubpd,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmaddsubpd,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmaddsubpd,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmsubaddps,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmsubaddps,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmsubaddps,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmsubaddps,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmsubaddpd,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmsubaddpd,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmsubaddpd,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmsubaddpd,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Pcmpestrm,// Pcmpestrm_xmm_xmmm128_imm8
	Mnemonic::Pcmpestrm64,// Pcmpestrm64_xmm_xmmm128_imm8
	Mnemonic::Vpcmpestrm,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	Mnemonic::Vpcmpestrm64,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	Mnemonic::Pcmpestri,// Pcmpestri_xmm_xmmm128_imm8
	Mnemonic::Pcmpestri64,// Pcmpestri64_xmm_xmmm128_imm8
	Mnemonic::Vpcmpestri,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	Mnemonic::Vpcmpestri64,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	Mnemonic::Pcmpistrm,// Pcmpistrm_xmm_xmmm128_imm8
	Mnemonic::Vpcmpistrm,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	Mnemonic::Pcmpistri,// Pcmpistri_xmm_xmmm128_imm8
	Mnemonic::Vpcmpistri,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	Mnemonic::Vfpclassps,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	Mnemonic::Vfpclassps,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	Mnemonic::Vfpclassps,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	Mnemonic::Vfpclasspd,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	Mnemonic::Vfpclasspd,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	Mnemonic::Vfpclasspd,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	Mnemonic::Vfpclassss,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	Mnemonic::Vfpclasssd,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	Mnemonic::Vfmaddps,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmaddps,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmaddps,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmaddps,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmaddpd,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmaddpd,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmaddpd,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmaddpd,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmaddss,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	Mnemonic::Vfmaddss,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	Mnemonic::Vfmaddsd,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	Mnemonic::Vfmaddsd,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	Mnemonic::Vfmsubps,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmsubps,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmsubps,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmsubps,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmsubpd,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfmsubpd,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfmsubpd,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfmsubpd,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfmsubss,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	Mnemonic::Vfmsubss,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	Mnemonic::Vfmsubsd,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	Mnemonic::Vfmsubsd,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	Mnemonic::Vpshldw,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	Mnemonic::Vpshldw,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	Mnemonic::Vpshldw,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	Mnemonic::Vpshldd,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vpshldd,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vpshldd,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Vpshldq,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vpshldq,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vpshldq,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Vpshrdw,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	Mnemonic::Vpshrdw,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	Mnemonic::Vpshrdw,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	Mnemonic::Vpshrdd,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	Mnemonic::Vpshrdd,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	Mnemonic::Vpshrdd,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	Mnemonic::Vpshrdq,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vpshrdq,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vpshrdq,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Vfnmaddps,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfnmaddps,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfnmaddps,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfnmaddps,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfnmaddpd,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfnmaddpd,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfnmaddpd,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfnmaddpd,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfnmaddss,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	Mnemonic::Vfnmaddss,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	Mnemonic::Vfnmaddsd,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	Mnemonic::Vfnmaddsd,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	Mnemonic::Vfnmsubps,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfnmsubps,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfnmsubps,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfnmsubps,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfnmsubpd,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vfnmsubpd,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	Mnemonic::Vfnmsubpd,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	Mnemonic::Vfnmsubpd,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	Mnemonic::Vfnmsubss,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	Mnemonic::Vfnmsubss,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	Mnemonic::Vfnmsubsd,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	Mnemonic::Vfnmsubsd,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	Mnemonic::Sha1rnds4,// Sha1rnds4_xmm_xmmm128_imm8
	Mnemonic::Gf2p8affineqb,// Gf2p8affineqb_xmm_xmmm128_imm8
	Mnemonic::Vgf2p8affineqb,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	Mnemonic::Vgf2p8affineqb,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	Mnemonic::Vgf2p8affineqb,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vgf2p8affineqb,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vgf2p8affineqb,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Gf2p8affineinvqb,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	Mnemonic::Vgf2p8affineinvqb,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	Mnemonic::Vgf2p8affineinvqb,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	Mnemonic::Vgf2p8affineinvqb,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	Mnemonic::Vgf2p8affineinvqb,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	Mnemonic::Vgf2p8affineinvqb,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	Mnemonic::Aeskeygenassist,// Aeskeygenassist_xmm_xmmm128_imm8
	Mnemonic::Vaeskeygenassist,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	Mnemonic::Rorx,// VEX_Rorx_r32_rm32_imm8
	Mnemonic::Rorx,// VEX_Rorx_r64_rm64_imm8
	Mnemonic::Vpmacssww,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacsswd,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacssdql,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacssdd,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacssdqh,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacsww,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacswd,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacsdql,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacsdd,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmacsdqh,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpcmov,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpcmov,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	Mnemonic::Vpcmov,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	Mnemonic::Vpcmov,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	Mnemonic::Vpperm,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpperm,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	Mnemonic::Vpmadcsswd,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vpmadcswd,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	Mnemonic::Vprotb,// XOP_Vprotb_xmm_xmmm128_imm8
	Mnemonic::Vprotw,// XOP_Vprotw_xmm_xmmm128_imm8
	Mnemonic::Vprotd,// XOP_Vprotd_xmm_xmmm128_imm8
	Mnemonic::Vprotq,// XOP_Vprotq_xmm_xmmm128_imm8
	Mnemonic::Vpcomb,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomw,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomd,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomq,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomub,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomuw,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomud,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	Mnemonic::Vpcomuq,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	Mnemonic::Blcfill,// XOP_Blcfill_r32_rm32
	Mnemonic::Blcfill,// XOP_Blcfill_r64_rm64
	Mnemonic::Blsfill,// XOP_Blsfill_r32_rm32
	Mnemonic::Blsfill,// XOP_Blsfill_r64_rm64
	Mnemonic::Blcs,// XOP_Blcs_r32_rm32
	Mnemonic::Blcs,// XOP_Blcs_r64_rm64
	Mnemonic::Tzmsk,// XOP_Tzmsk_r32_rm32
	Mnemonic::Tzmsk,// XOP_Tzmsk_r64_rm64
	Mnemonic::Blcic,// XOP_Blcic_r32_rm32
	Mnemonic::Blcic,// XOP_Blcic_r64_rm64
	Mnemonic::Blsic,// XOP_Blsic_r32_rm32
	Mnemonic::Blsic,// XOP_Blsic_r64_rm64
	Mnemonic::T1mskc,// XOP_T1mskc_r32_rm32
	Mnemonic::T1mskc,// XOP_T1mskc_r64_rm64
	Mnemonic::Blcmsk,// XOP_Blcmsk_r32_rm32
	Mnemonic::Blcmsk,// XOP_Blcmsk_r64_rm64
	Mnemonic::Blci,// XOP_Blci_r32_rm32
	Mnemonic::Blci,// XOP_Blci_r64_rm64
	Mnemonic::Llwpcb,// XOP_Llwpcb_r32
	Mnemonic::Llwpcb,// XOP_Llwpcb_r64
	Mnemonic::Slwpcb,// XOP_Slwpcb_r32
	Mnemonic::Slwpcb,// XOP_Slwpcb_r64
	Mnemonic::Vfrczps,// XOP_Vfrczps_xmm_xmmm128
	Mnemonic::Vfrczps,// XOP_Vfrczps_ymm_ymmm256
	Mnemonic::Vfrczpd,// XOP_Vfrczpd_xmm_xmmm128
	Mnemonic::Vfrczpd,// XOP_Vfrczpd_ymm_ymmm256
	Mnemonic::Vfrczss,// XOP_Vfrczss_xmm_xmmm32
	Mnemonic::Vfrczsd,// XOP_Vfrczsd_xmm_xmmm64
	Mnemonic::Vprotb,// XOP_Vprotb_xmm_xmmm128_xmm
	Mnemonic::Vprotb,// XOP_Vprotb_xmm_xmm_xmmm128
	Mnemonic::Vprotw,// XOP_Vprotw_xmm_xmmm128_xmm
	Mnemonic::Vprotw,// XOP_Vprotw_xmm_xmm_xmmm128
	Mnemonic::Vprotd,// XOP_Vprotd_xmm_xmmm128_xmm
	Mnemonic::Vprotd,// XOP_Vprotd_xmm_xmm_xmmm128
	Mnemonic::Vprotq,// XOP_Vprotq_xmm_xmmm128_xmm
	Mnemonic::Vprotq,// XOP_Vprotq_xmm_xmm_xmmm128
	Mnemonic::Vpshlb,// XOP_Vpshlb_xmm_xmmm128_xmm
	Mnemonic::Vpshlb,// XOP_Vpshlb_xmm_xmm_xmmm128
	Mnemonic::Vpshlw,// XOP_Vpshlw_xmm_xmmm128_xmm
	Mnemonic::Vpshlw,// XOP_Vpshlw_xmm_xmm_xmmm128
	Mnemonic::Vpshld,// XOP_Vpshld_xmm_xmmm128_xmm
	Mnemonic::Vpshld,// XOP_Vpshld_xmm_xmm_xmmm128
	Mnemonic::Vpshlq,// XOP_Vpshlq_xmm_xmmm128_xmm
	Mnemonic::Vpshlq,// XOP_Vpshlq_xmm_xmm_xmmm128
	Mnemonic::Vpshab,// XOP_Vpshab_xmm_xmmm128_xmm
	Mnemonic::Vpshab,// XOP_Vpshab_xmm_xmm_xmmm128
	Mnemonic::Vpshaw,// XOP_Vpshaw_xmm_xmmm128_xmm
	Mnemonic::Vpshaw,// XOP_Vpshaw_xmm_xmm_xmmm128
	Mnemonic::Vpshad,// XOP_Vpshad_xmm_xmmm128_xmm
	Mnemonic::Vpshad,// XOP_Vpshad_xmm_xmm_xmmm128
	Mnemonic::Vpshaq,// XOP_Vpshaq_xmm_xmmm128_xmm
	Mnemonic::Vpshaq,// XOP_Vpshaq_xmm_xmm_xmmm128
	Mnemonic::Vphaddbw,// XOP_Vphaddbw_xmm_xmmm128
	Mnemonic::Vphaddbd,// XOP_Vphaddbd_xmm_xmmm128
	Mnemonic::Vphaddbq,// XOP_Vphaddbq_xmm_xmmm128
	Mnemonic::Vphaddwd,// XOP_Vphaddwd_xmm_xmmm128
	Mnemonic::Vphaddwq,// XOP_Vphaddwq_xmm_xmmm128
	Mnemonic::Vphadddq,// XOP_Vphadddq_xmm_xmmm128
	Mnemonic::Vphaddubw,// XOP_Vphaddubw_xmm_xmmm128
	Mnemonic::Vphaddubd,// XOP_Vphaddubd_xmm_xmmm128
	Mnemonic::Vphaddubq,// XOP_Vphaddubq_xmm_xmmm128
	Mnemonic::Vphadduwd,// XOP_Vphadduwd_xmm_xmmm128
	Mnemonic::Vphadduwq,// XOP_Vphadduwq_xmm_xmmm128
	Mnemonic::Vphaddudq,// XOP_Vphaddudq_xmm_xmmm128
	Mnemonic::Vphsubbw,// XOP_Vphsubbw_xmm_xmmm128
	Mnemonic::Vphsubwd,// XOP_Vphsubwd_xmm_xmmm128
	Mnemonic::Vphsubdq,// XOP_Vphsubdq_xmm_xmmm128
	Mnemonic::Bextr,// XOP_Bextr_r32_rm32_imm32
	Mnemonic::Bextr,// XOP_Bextr_r64_rm64_imm32
	Mnemonic::Lwpins,// XOP_Lwpins_r32_rm32_imm32
	Mnemonic::Lwpins,// XOP_Lwpins_r64_rm32_imm32
	Mnemonic::Lwpval,// XOP_Lwpval_r32_rm32_imm32
	Mnemonic::Lwpval,// XOP_Lwpval_r64_rm32_imm32
	Mnemonic::Pi2fw,// D3NOW_Pi2fw_mm_mmm64
	Mnemonic::Pi2fd,// D3NOW_Pi2fd_mm_mmm64
	Mnemonic::Pf2iw,// D3NOW_Pf2iw_mm_mmm64
	Mnemonic::Pf2id,// D3NOW_Pf2id_mm_mmm64
	Mnemonic::Pfrcpv,// D3NOW_Pfrcpv_mm_mmm64
	Mnemonic::Pfrsqrtv,// D3NOW_Pfrsqrtv_mm_mmm64
	Mnemonic::Pfnacc,// D3NOW_Pfnacc_mm_mmm64
	Mnemonic::Pfpnacc,// D3NOW_Pfpnacc_mm_mmm64
	Mnemonic::Pfcmpge,// D3NOW_Pfcmpge_mm_mmm64
	Mnemonic::Pfmin,// D3NOW_Pfmin_mm_mmm64
	Mnemonic::Pfrcp,// D3NOW_Pfrcp_mm_mmm64
	Mnemonic::Pfrsqrt,// D3NOW_Pfrsqrt_mm_mmm64
	Mnemonic::Pfsub,// D3NOW_Pfsub_mm_mmm64
	Mnemonic::Pfadd,// D3NOW_Pfadd_mm_mmm64
	Mnemonic::Pfcmpgt,// D3NOW_Pfcmpgt_mm_mmm64
	Mnemonic::Pfmax,// D3NOW_Pfmax_mm_mmm64
	Mnemonic::Pfrcpit1,// D3NOW_Pfrcpit1_mm_mmm64
	Mnemonic::Pfrsqit1,// D3NOW_Pfrsqit1_mm_mmm64
	Mnemonic::Pfsubr,// D3NOW_Pfsubr_mm_mmm64
	Mnemonic::Pfacc,// D3NOW_Pfacc_mm_mmm64
	Mnemonic::Pfcmpeq,// D3NOW_Pfcmpeq_mm_mmm64
	Mnemonic::Pfmul,// D3NOW_Pfmul_mm_mmm64
	Mnemonic::Pfrcpit2,// D3NOW_Pfrcpit2_mm_mmm64
	Mnemonic::Pmulhrw,// D3NOW_Pmulhrw_mm_mmm64
	Mnemonic::Pswapd,// D3NOW_Pswapd_mm_mmm64
	Mnemonic::Pavgusb,// D3NOW_Pavgusb_mm_mmm64
	Mnemonic::Rmpadjust,// Rmpadjust
	Mnemonic::Rmpupdate,// Rmpupdate
	Mnemonic::Psmash,// Psmash
	Mnemonic::Pvalidate,// Pvalidatew
	Mnemonic::Pvalidate,// Pvalidated
	Mnemonic::Pvalidate,// Pvalidateq
	Mnemonic::Serialize,// Serialize
	Mnemonic::Xsusldtrk,// Xsusldtrk
	Mnemonic::Xresldtrk,// Xresldtrk
	Mnemonic::Invlpgb,// Invlpgbw
	Mnemonic::Invlpgb,// Invlpgbd
	Mnemonic::Invlpgb,// Invlpgbq
	Mnemonic::Tlbsync,// Tlbsync
	Mnemonic::Prefetchw,// Prefetchreserved3_m8
	Mnemonic::Prefetch,// Prefetchreserved4_m8
	Mnemonic::Prefetch,// Prefetchreserved5_m8
	Mnemonic::Prefetch,// Prefetchreserved6_m8
	Mnemonic::Prefetch,// Prefetchreserved7_m8
	Mnemonic::Ud0,// Ud0
	Mnemonic::Vmgexit,// Vmgexit
	Mnemonic::Getsecq,// Getsecq
	Mnemonic::Ldtilecfg,// VEX_Ldtilecfg_m512
	Mnemonic::Tilerelease,// VEX_Tilerelease
	Mnemonic::Sttilecfg,// VEX_Sttilecfg_m512
	Mnemonic::Tilezero,// VEX_Tilezero_tmm
	Mnemonic::Tileloaddt1,// VEX_Tileloaddt1_tmm_sibmem
	Mnemonic::Tilestored,// VEX_Tilestored_sibmem_tmm
	Mnemonic::Tileloadd,// VEX_Tileloadd_tmm_sibmem
	Mnemonic::Tdpbf16ps,// VEX_Tdpbf16ps_tmm_tmm_tmm
	Mnemonic::Tdpbuud,// VEX_Tdpbuud_tmm_tmm_tmm
	Mnemonic::Tdpbusd,// VEX_Tdpbusd_tmm_tmm_tmm
	Mnemonic::Tdpbsud,// VEX_Tdpbsud_tmm_tmm_tmm
	Mnemonic::Tdpbssd,// VEX_Tdpbssd_tmm_tmm_tmm
	Mnemonic::Fnstdw,// Fnstdw_AX
	Mnemonic::Fnstsg,// Fnstsg_AX
	Mnemonic::Rdshr,// Rdshr_rm32
	Mnemonic::Wrshr,// Wrshr_rm32
	Mnemonic::Smint,// Smint
	Mnemonic::Dmint,// Dmint
	Mnemonic::Rdm,// Rdm
	Mnemonic::Svdc,// Svdc_m80_Sreg
	Mnemonic::Rsdc,// Rsdc_Sreg_m80
	Mnemonic::Svldt,// Svldt_m80
	Mnemonic::Rsldt,// Rsldt_m80
	Mnemonic::Svts,// Svts_m80
	Mnemonic::Rsts,// Rsts_m80
	Mnemonic::Smint,// Smint_0F7E
	Mnemonic::Bb0_reset,// Bb0_reset
	Mnemonic::Bb1_reset,// Bb1_reset
	Mnemonic::Cpu_write,// Cpu_write
	Mnemonic::Cpu_read,// Cpu_read
	Mnemonic::Altinst,// Altinst
	Mnemonic::Paveb,// Paveb_mm_mmm64
	Mnemonic::Paddsiw,// Paddsiw_mm_mmm64
	Mnemonic::Pmagw,// Pmagw_mm_mmm64
	Mnemonic::Pdistib,// Pdistib_mm_m64
	Mnemonic::Psubsiw,// Psubsiw_mm_mmm64
	Mnemonic::Pmvzb,// Pmvzb_mm_m64
	Mnemonic::Pmulhrw,// Pmulhrw_mm_mmm64
	Mnemonic::Pmvnzb,// Pmvnzb_mm_m64
	Mnemonic::Pmvlzb,// Pmvlzb_mm_m64
	Mnemonic::Pmvgezb,// Pmvgezb_mm_m64
	Mnemonic::Pmulhriw,// Pmulhriw_mm_mmm64
	Mnemonic::Pmachriw,// Pmachriw_mm_m64
	Mnemonic::Undoc,// Cyrix_D9D7
	Mnemonic::Undoc,// Cyrix_D9E2
	Mnemonic::Ftstp,// Ftstp
	Mnemonic::Undoc,// Cyrix_D9E7
	Mnemonic::Frint2,// Frint2
	Mnemonic::Frichop,// Frichop
	Mnemonic::Undoc,// Cyrix_DED8
	Mnemonic::Undoc,// Cyrix_DEDA
	Mnemonic::Undoc,// Cyrix_DEDC
	Mnemonic::Undoc,// Cyrix_DEDD
	Mnemonic::Undoc,// Cyrix_DEDE
	Mnemonic::Frinear,// Frinear
	Mnemonic::Tdcall,// Tdcall
	Mnemonic::Seamret,// Seamret
	Mnemonic::Seamops,// Seamops
	Mnemonic::Seamcall,// Seamcall
	Mnemonic::Aesencwide128kl,// Aesencwide128kl_m384
	Mnemonic::Aesdecwide128kl,// Aesdecwide128kl_m384
	Mnemonic::Aesencwide256kl,// Aesencwide256kl_m512
	Mnemonic::Aesdecwide256kl,// Aesdecwide256kl_m512
	Mnemonic::Loadiwkey,// Loadiwkey_xmm_xmm
	Mnemonic::Aesenc128kl,// Aesenc128kl_xmm_m384
	Mnemonic::Aesdec128kl,// Aesdec128kl_xmm_m384
	Mnemonic::Aesenc256kl,// Aesenc256kl_xmm_m512
	Mnemonic::Aesdec256kl,// Aesdec256kl_xmm_m512
	Mnemonic::Encodekey128,// Encodekey128_r32_r32
	Mnemonic::Encodekey256,// Encodekey256_r32_r32
	Mnemonic::Vbroadcastss,// VEX_Vbroadcastss_xmm_xmm
	Mnemonic::Vbroadcastss,// VEX_Vbroadcastss_ymm_xmm
	Mnemonic::Vbroadcastsd,// VEX_Vbroadcastsd_ymm_xmm
	Mnemonic::Vmgexit,// Vmgexit_F2
	Mnemonic::Uiret,// Uiret
	Mnemonic::Testui,// Testui
	Mnemonic::Clui,// Clui
	Mnemonic::Stui,// Stui
	Mnemonic::Senduipi,// Senduipi_r64
	Mnemonic::Hreset,// Hreset_imm8
	Mnemonic::Vpdpbusd,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	Mnemonic::Vpdpbusd,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	Mnemonic::Vpdpbusds,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	Mnemonic::Vpdpbusds,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	Mnemonic::Vpdpwssd,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	Mnemonic::Vpdpwssd,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	Mnemonic::Vpdpwssds,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	Mnemonic::Vpdpwssds,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	Mnemonic::Ccs_hash,// Ccs_hash_16
	Mnemonic::Ccs_hash,// Ccs_hash_32
	Mnemonic::Ccs_hash,// Ccs_hash_64
	Mnemonic::Ccs_encrypt,// Ccs_encrypt_16
	Mnemonic::Ccs_encrypt,// Ccs_encrypt_32
	Mnemonic::Ccs_encrypt,// Ccs_encrypt_64
	Mnemonic::Lkgs,// Lkgs_rm16
	Mnemonic::Lkgs,// Lkgs_r32m16
	Mnemonic::Lkgs,// Lkgs_r64m16
	Mnemonic::Eretu,// Eretu
	Mnemonic::Erets,// Erets
	Mnemonic::Vaddph,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vaddph,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vaddph,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vaddsh,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vcmpph,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	Mnemonic::Vcmpph,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	Mnemonic::Vcmpph,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	Mnemonic::Vcmpsh,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	Mnemonic::Vcomish,// EVEX_Vcomish_xmm_xmmm16_sae
	Mnemonic::Vcvtdq2ph,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtdq2ph,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	Mnemonic::Vcvtdq2ph,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	Mnemonic::Vcvtpd2ph,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtpd2ph,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtpd2ph,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtph2dq,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	Mnemonic::Vcvtph2dq,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	Mnemonic::Vcvtph2dq,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	Mnemonic::Vcvtph2pd,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	Mnemonic::Vcvtph2pd,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	Mnemonic::Vcvtph2pd,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	Mnemonic::Vcvtph2psx,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	Mnemonic::Vcvtph2psx,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	Mnemonic::Vcvtph2psx,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	Mnemonic::Vcvtph2qq,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	Mnemonic::Vcvtph2qq,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	Mnemonic::Vcvtph2qq,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	Mnemonic::Vcvtph2udq,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	Mnemonic::Vcvtph2udq,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	Mnemonic::Vcvtph2udq,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	Mnemonic::Vcvtph2uqq,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	Mnemonic::Vcvtph2uqq,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	Mnemonic::Vcvtph2uqq,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	Mnemonic::Vcvtph2uw,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	Mnemonic::Vcvtph2uw,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	Mnemonic::Vcvtph2uw,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	Mnemonic::Vcvtph2w,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	Mnemonic::Vcvtph2w,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	Mnemonic::Vcvtph2w,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	Mnemonic::Vcvtps2phx,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtps2phx,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	Mnemonic::Vcvtps2phx,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	Mnemonic::Vcvtqq2ph,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtqq2ph,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtqq2ph,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtsd2sh,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	Mnemonic::Vcvtsh2sd,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	Mnemonic::Vcvtsh2si,// EVEX_Vcvtsh2si_r32_xmmm16_er
	Mnemonic::Vcvtsh2si,// EVEX_Vcvtsh2si_r64_xmmm16_er
	Mnemonic::Vcvtsh2ss,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	Mnemonic::Vcvtsh2usi,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	Mnemonic::Vcvtsh2usi,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	Mnemonic::Vcvtsi2sh,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	Mnemonic::Vcvtsi2sh,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	Mnemonic::Vcvtss2sh,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vcvttph2dq,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	Mnemonic::Vcvttph2dq,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	Mnemonic::Vcvttph2dq,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	Mnemonic::Vcvttph2qq,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	Mnemonic::Vcvttph2qq,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	Mnemonic::Vcvttph2qq,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	Mnemonic::Vcvttph2udq,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	Mnemonic::Vcvttph2udq,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	Mnemonic::Vcvttph2udq,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	Mnemonic::Vcvttph2uqq,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	Mnemonic::Vcvttph2uqq,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	Mnemonic::Vcvttph2uqq,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	Mnemonic::Vcvttph2uw,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	Mnemonic::Vcvttph2uw,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	Mnemonic::Vcvttph2uw,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	Mnemonic::Vcvttph2w,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	Mnemonic::Vcvttph2w,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	Mnemonic::Vcvttph2w,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	Mnemonic::Vcvttsh2si,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	Mnemonic::Vcvttsh2si,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	Mnemonic::Vcvttsh2usi,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	Mnemonic::Vcvttsh2usi,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	Mnemonic::Vcvtudq2ph,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	Mnemonic::Vcvtudq2ph,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	Mnemonic::Vcvtudq2ph,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	Mnemonic::Vcvtuqq2ph,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	Mnemonic::Vcvtuqq2ph,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	Mnemonic::Vcvtuqq2ph,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	Mnemonic::Vcvtusi2sh,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	Mnemonic::Vcvtusi2sh,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	Mnemonic::Vcvtuw2ph,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	Mnemonic::Vcvtuw2ph,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	Mnemonic::Vcvtuw2ph,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	Mnemonic::Vcvtw2ph,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	Mnemonic::Vcvtw2ph,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	Mnemonic::Vcvtw2ph,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	Mnemonic::Vdivph,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vdivph,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vdivph,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vdivsh,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfcmaddcph,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfcmaddcph,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfcmaddcph,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmaddcph,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmaddcph,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmaddcph,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfcmaddcsh,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmaddcsh,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfcmulcph,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfcmulcph,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfcmulcph,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfmulcph,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	Mnemonic::Vfmulcph,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	Mnemonic::Vfmulcph,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	Mnemonic::Vfcmulcsh,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmulcsh,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	Mnemonic::Vfmaddsub132ph,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmaddsub132ph,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmaddsub132ph,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmaddsub213ph,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmaddsub213ph,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmaddsub213ph,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmaddsub231ph,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmaddsub231ph,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmaddsub231ph,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsubadd132ph,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsubadd132ph,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsubadd132ph,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsubadd213ph,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsubadd213ph,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsubadd213ph,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsubadd231ph,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsubadd231ph,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsubadd231ph,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmadd132ph,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmadd132ph,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmadd132ph,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmadd213ph,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmadd213ph,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmadd213ph,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmadd231ph,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmadd231ph,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmadd231ph,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmadd132ph,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmadd132ph,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmadd132ph,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmadd213ph,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmadd213ph,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmadd213ph,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmadd231ph,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmadd231ph,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmadd231ph,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmadd132sh,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfmadd213sh,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfmadd231sh,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmadd132sh,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmadd213sh,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmadd231sh,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfmsub132ph,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsub132ph,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsub132ph,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsub213ph,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsub213ph,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsub213ph,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsub231ph,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfmsub231ph,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfmsub231ph,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmsub132ph,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmsub132ph,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmsub132ph,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmsub213ph,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmsub213ph,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmsub213ph,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfnmsub231ph,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vfnmsub231ph,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vfnmsub231ph,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vfmsub132sh,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfmsub213sh,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfmsub231sh,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmsub132sh,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmsub213sh,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfnmsub231sh,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vfpclassph,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	Mnemonic::Vfpclassph,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	Mnemonic::Vfpclassph,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	Mnemonic::Vfpclasssh,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	Mnemonic::Vgetexpph,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	Mnemonic::Vgetexpph,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	Mnemonic::Vgetexpph,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	Mnemonic::Vgetexpsh,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	Mnemonic::Vgetmantph,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	Mnemonic::Vgetmantph,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	Mnemonic::Vgetmantph,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	Mnemonic::Vgetmantsh,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	Mnemonic::Vmaxph,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vmaxph,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vmaxph,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	Mnemonic::Vmaxsh,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	Mnemonic::Vminph,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vminph,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vminph,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	Mnemonic::Vminsh,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	Mnemonic::Vmovsh,// EVEX_Vmovsh_xmm_k1z_m16
	Mnemonic::Vmovsh,// EVEX_Vmovsh_m16_k1_xmm
	Mnemonic::Vmovsh,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	Mnemonic::Vmovsh,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	Mnemonic::Vmovw,// EVEX_Vmovw_xmm_r32m16
	Mnemonic::Vmovw,// EVEX_Vmovw_xmm_r64m16
	Mnemonic::Vmovw,// EVEX_Vmovw_r32m16_xmm
	Mnemonic::Vmovw,// EVEX_Vmovw_r64m16_xmm
	Mnemonic::Vmulph,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vmulph,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vmulph,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vmulsh,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vrcpph,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	Mnemonic::Vrcpph,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	Mnemonic::Vrcpph,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	Mnemonic::Vrcpsh,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	Mnemonic::Vreduceph,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	Mnemonic::Vreduceph,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	Mnemonic::Vreduceph,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	Mnemonic::Vreducesh,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	Mnemonic::Vrndscaleph,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	Mnemonic::Vrndscaleph,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	Mnemonic::Vrndscaleph,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	Mnemonic::Vrndscalesh,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	Mnemonic::Vrsqrtph,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	Mnemonic::Vrsqrtph,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	Mnemonic::Vrsqrtph,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	Mnemonic::Vrsqrtsh,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	Mnemonic::Vscalefph,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vscalefph,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vscalefph,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vscalefsh,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vsqrtph,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	Mnemonic::Vsqrtph,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	Mnemonic::Vsqrtph,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	Mnemonic::Vsqrtsh,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vsubph,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	Mnemonic::Vsubph,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	Mnemonic::Vsubph,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	Mnemonic::Vsubsh,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	Mnemonic::Vucomish,// EVEX_Vucomish_xmm_xmmm16_sae
	Mnemonic::Rdudbg,// Rdudbg
	Mnemonic::Wrudbg,// Wrudbg
	Mnemonic::Jkzd,// VEX_KNC_Jkzd_kr_rel8_64
	Mnemonic::Jknzd,// VEX_KNC_Jknzd_kr_rel8_64
	Mnemonic::Vprefetchnta,// VEX_KNC_Vprefetchnta_m8
	Mnemonic::Vprefetch0,// VEX_KNC_Vprefetch0_m8
	Mnemonic::Vprefetch1,// VEX_KNC_Vprefetch1_m8
	Mnemonic::Vprefetch2,// VEX_KNC_Vprefetch2_m8
	Mnemonic::Vprefetchenta,// VEX_KNC_Vprefetchenta_m8
	Mnemonic::Vprefetche0,// VEX_KNC_Vprefetche0_m8
	Mnemonic::Vprefetche1,// VEX_KNC_Vprefetche1_m8
	Mnemonic::Vprefetche2,// VEX_KNC_Vprefetche2_m8
	Mnemonic::Kand,// VEX_KNC_Kand_kr_kr
	Mnemonic::Kandn,// VEX_KNC_Kandn_kr_kr
	Mnemonic::Kandnr,// VEX_KNC_Kandnr_kr_kr
	Mnemonic::Knot,// VEX_KNC_Knot_kr_kr
	Mnemonic::Kor,// VEX_KNC_Kor_kr_kr
	Mnemonic::Kxnor,// VEX_KNC_Kxnor_kr_kr
	Mnemonic::Kxor,// VEX_KNC_Kxor_kr_kr
	Mnemonic::Kmerge2l1h,// VEX_KNC_Kmerge2l1h_kr_kr
	Mnemonic::Kmerge2l1l,// VEX_KNC_Kmerge2l1l_kr_kr
	Mnemonic::Jkzd,// VEX_KNC_Jkzd_kr_rel32_64
	Mnemonic::Jknzd,// VEX_KNC_Jknzd_kr_rel32_64
	Mnemonic::Kmov,// VEX_KNC_Kmov_kr_kr
	Mnemonic::Kmov,// VEX_KNC_Kmov_kr_r32
	Mnemonic::Kmov,// VEX_KNC_Kmov_r32_kr
	Mnemonic::Kconcath,// VEX_KNC_Kconcath_r64_kr_kr
	Mnemonic::Kconcatl,// VEX_KNC_Kconcatl_r64_kr_kr
	Mnemonic::Kortest,// VEX_KNC_Kortest_kr_kr
	Mnemonic::Delay,// VEX_KNC_Delay_r32
	Mnemonic::Delay,// VEX_KNC_Delay_r64
	Mnemonic::Spflt,// VEX_KNC_Spflt_r32
	Mnemonic::Spflt,// VEX_KNC_Spflt_r64
	Mnemonic::Clevict1,// VEX_KNC_Clevict1_m8
	Mnemonic::Clevict0,// VEX_KNC_Clevict0_m8
	Mnemonic::Popcnt,// VEX_KNC_Popcnt_r32_r32
	Mnemonic::Popcnt,// VEX_KNC_Popcnt_r64_r64
	Mnemonic::Tzcnt,// VEX_KNC_Tzcnt_r32_r32
	Mnemonic::Tzcnt,// VEX_KNC_Tzcnt_r64_r64
	Mnemonic::Tzcnti,// VEX_KNC_Tzcnti_r32_r32
	Mnemonic::Tzcnti,// VEX_KNC_Tzcnti_r64_r64
	Mnemonic::Lzcnt,// VEX_KNC_Lzcnt_r32_r32
	Mnemonic::Lzcnt,// VEX_KNC_Lzcnt_r64_r64
	Mnemonic::Undoc,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	Mnemonic::Undoc,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	Mnemonic::Undoc,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	Mnemonic::Undoc,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	Mnemonic::Undoc,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	Mnemonic::Undoc,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	Mnemonic::Kextract,// VEX_KNC_Kextract_kr_r64_imm8
	Mnemonic::Vprefetchnta,// MVEX_Vprefetchnta_m
	Mnemonic::Vprefetch0,// MVEX_Vprefetch0_m
	Mnemonic::Vprefetch1,// MVEX_Vprefetch1_m
	Mnemonic::Vprefetch2,// MVEX_Vprefetch2_m
	Mnemonic::Vprefetchenta,// MVEX_Vprefetchenta_m
	Mnemonic::Vprefetche0,// MVEX_Vprefetche0_m
	Mnemonic::Vprefetche1,// MVEX_Vprefetche1_m
	Mnemonic::Vprefetche2,// MVEX_Vprefetche2_m
	Mnemonic::Vmovaps,// MVEX_Vmovaps_zmm_k1_zmmmt
	Mnemonic::Vmovapd,// MVEX_Vmovapd_zmm_k1_zmmmt
	Mnemonic::Vmovaps,// MVEX_Vmovaps_mt_k1_zmm
	Mnemonic::Vmovapd,// MVEX_Vmovapd_mt_k1_zmm
	Mnemonic::Vmovnrapd,// MVEX_Vmovnrapd_m_k1_zmm
	Mnemonic::Vmovnrngoapd,// MVEX_Vmovnrngoapd_m_k1_zmm
	Mnemonic::Vmovnraps,// MVEX_Vmovnraps_m_k1_zmm
	Mnemonic::Vmovnrngoaps,// MVEX_Vmovnrngoaps_m_k1_zmm
	Mnemonic::Vaddps,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	Mnemonic::Vaddpd,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vmulps,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	Mnemonic::Vmulpd,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vcvtps2pd,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	Mnemonic::Vcvtpd2ps,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	Mnemonic::Vsubps,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	Mnemonic::Vsubpd,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpcmpgtd,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	Mnemonic::Vmovdqa32,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	Mnemonic::Vmovdqa64,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	Mnemonic::Vpshufd,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	Mnemonic::Vpsrld,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	Mnemonic::Vpsrad,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	Mnemonic::Vpslld,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	Mnemonic::Vpcmpeqd,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	Mnemonic::Vcvtudq2pd,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	Mnemonic::Vmovdqa32,// MVEX_Vmovdqa32_mt_k1_zmm
	Mnemonic::Vmovdqa64,// MVEX_Vmovdqa64_mt_k1_zmm
	Mnemonic::Clevict1,// MVEX_Clevict1_m
	Mnemonic::Clevict0,// MVEX_Clevict0_m
	Mnemonic::Vcmpps,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	Mnemonic::Vcmppd,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	Mnemonic::Vpandd,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpandq,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	Mnemonic::Vpandnd,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpandnq,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	Mnemonic::Vcvtdq2pd,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	Mnemonic::Vpord,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	Mnemonic::Vporq,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	Mnemonic::Vpxord,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	Mnemonic::Vpxorq,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	Mnemonic::Vpsubd,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpaddd,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	Mnemonic::Vbroadcastss,// MVEX_Vbroadcastss_zmm_k1_mt
	Mnemonic::Vbroadcastsd,// MVEX_Vbroadcastsd_zmm_k1_mt
	Mnemonic::Vbroadcastf32x4,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	Mnemonic::Vbroadcastf64x4,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	Mnemonic::Vptestmd,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	Mnemonic::Vpermd,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpminsd,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpminud,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmaxsd,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmaxud,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmulld,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	Mnemonic::Vgetexpps,// MVEX_Vgetexpps_zmm_k1_zmmmt
	Mnemonic::Vgetexppd,// MVEX_Vgetexppd_zmm_k1_zmmmt
	Mnemonic::Vpsrlvd,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpsravd,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpsllvd,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	Mnemonic::Vaddnps,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	Mnemonic::Vaddnpd,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vgmaxabsps,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	Mnemonic::Vgminps,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	Mnemonic::Vgminpd,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vgmaxps,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	Mnemonic::Vgmaxpd,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	Mnemonic::Vfixupnanps,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfixupnanpd,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	Mnemonic::Vpbroadcastd,// MVEX_Vpbroadcastd_zmm_k1_mt
	Mnemonic::Vpbroadcastq,// MVEX_Vpbroadcastq_zmm_k1_mt
	Mnemonic::Vbroadcasti32x4,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	Mnemonic::Vbroadcasti64x4,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	Mnemonic::Vpadcd,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	Mnemonic::Vpaddsetcd,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	Mnemonic::Vpsbbd,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	Mnemonic::Vpsubsetbd,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	Mnemonic::Vpblendmd,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpblendmq,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	Mnemonic::Vblendmps,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	Mnemonic::Vblendmpd,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	Mnemonic::Vpsubrd,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	Mnemonic::Vsubrps,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	Mnemonic::Vsubrpd,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpsbbrd,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	Mnemonic::Vpsubrsetbd,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	Mnemonic::Vpcmpltd,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	Mnemonic::Vscaleps,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmulhud,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmulhd,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpgatherdd,// MVEX_Vpgatherdd_zmm_k1_mvt
	Mnemonic::Vpgatherdq,// MVEX_Vpgatherdq_zmm_k1_mvt
	Mnemonic::Vgatherdps,// MVEX_Vgatherdps_zmm_k1_mvt
	Mnemonic::Vgatherdpd,// MVEX_Vgatherdpd_zmm_k1_mvt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	Mnemonic::Vfmadd132ps,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmadd132pd,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub132ps,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub132pd,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd132ps,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd132pd,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub132ps,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub132pd,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vpscatterdd,// MVEX_Vpscatterdd_mvt_k1_zmm
	Mnemonic::Vpscatterdq,// MVEX_Vpscatterdq_mvt_k1_zmm
	Mnemonic::Vscatterdps,// MVEX_Vscatterdps_mvt_k1_zmm
	Mnemonic::Vscatterdpd,// MVEX_Vscatterdpd_mvt_k1_zmm
	Mnemonic::Vfmadd233ps,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmadd213ps,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmadd213pd,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub213ps,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub213pd,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd213ps,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd213pd,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub213ps,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub213pd,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	Mnemonic::Vpmadd233d,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	Mnemonic::Vpmadd231d,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmadd231ps,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmadd231pd,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub231ps,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfmsub231pd,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd231ps,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmadd231pd,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub231ps,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	Mnemonic::Vfnmsub231pd,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	Mnemonic::Vgatherpf0hintdps,// MVEX_Vgatherpf0hintdps_mvt_k1
	Mnemonic::Vgatherpf0hintdpd,// MVEX_Vgatherpf0hintdpd_mvt_k1
	Mnemonic::Vgatherpf0dps,// MVEX_Vgatherpf0dps_mvt_k1
	Mnemonic::Vgatherpf1dps,// MVEX_Vgatherpf1dps_mvt_k1
	Mnemonic::Vscatterpf0hintdps,// MVEX_Vscatterpf0hintdps_mvt_k1
	Mnemonic::Vscatterpf0hintdpd,// MVEX_Vscatterpf0hintdpd_mvt_k1
	Mnemonic::Vscatterpf0dps,// MVEX_Vscatterpf0dps_mvt_k1
	Mnemonic::Vscatterpf1dps,// MVEX_Vscatterpf1dps_mvt_k1
	Mnemonic::Vexp223ps,// MVEX_Vexp223ps_zmm_k1_zmmmt
	Mnemonic::Vlog2ps,// MVEX_Vlog2ps_zmm_k1_zmmmt
	Mnemonic::Vrcp23ps,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	Mnemonic::Vrsqrt23ps,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	Mnemonic::Vaddsetsps,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	Mnemonic::Vpaddsetsd,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	Mnemonic::Vloadunpackld,// MVEX_Vloadunpackld_zmm_k1_mt
	Mnemonic::Vloadunpacklq,// MVEX_Vloadunpacklq_zmm_k1_mt
	Mnemonic::Vpackstoreld,// MVEX_Vpackstoreld_mt_k1_zmm
	Mnemonic::Vpackstorelq,// MVEX_Vpackstorelq_mt_k1_zmm
	Mnemonic::Vloadunpacklps,// MVEX_Vloadunpacklps_zmm_k1_mt
	Mnemonic::Vloadunpacklpd,// MVEX_Vloadunpacklpd_zmm_k1_mt
	Mnemonic::Vpackstorelps,// MVEX_Vpackstorelps_mt_k1_zmm
	Mnemonic::Vpackstorelpd,// MVEX_Vpackstorelpd_mt_k1_zmm
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	Mnemonic::Vloadunpackhd,// MVEX_Vloadunpackhd_zmm_k1_mt
	Mnemonic::Vloadunpackhq,// MVEX_Vloadunpackhq_zmm_k1_mt
	Mnemonic::Vpackstorehd,// MVEX_Vpackstorehd_mt_k1_zmm
	Mnemonic::Vpackstorehq,// MVEX_Vpackstorehq_mt_k1_zmm
	Mnemonic::Vloadunpackhps,// MVEX_Vloadunpackhps_zmm_k1_mt
	Mnemonic::Vloadunpackhpd,// MVEX_Vloadunpackhpd_zmm_k1_mt
	Mnemonic::Vpackstorehps,// MVEX_Vpackstorehps_mt_k1_zmm
	Mnemonic::Vpackstorehpd,// MVEX_Vpackstorehpd_mt_k1_zmm
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	Mnemonic::Valignd,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	Mnemonic::Vpermf32x4,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	Mnemonic::Vpcmpud,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	Mnemonic::Vpcmpd,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	Mnemonic::Vgetmantps,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	Mnemonic::Vgetmantpd,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	Mnemonic::Vrndfxpntps,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	Mnemonic::Vrndfxpntpd,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	Mnemonic::Vcvtfxpntudq2ps,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	Mnemonic::Vcvtfxpntps2udq,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	Mnemonic::Vcvtfxpntpd2udq,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	Mnemonic::Vcvtfxpntdq2ps,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	Mnemonic::Vcvtfxpntps2dq,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	Mnemonic::Undoc,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	Mnemonic::Vcvtfxpntpd2dq,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	Mnemonic::Undoc,// Via_undoc_F30FA6F0_16
	Mnemonic::Undoc,// Via_undoc_F30FA6F0_32
	Mnemonic::Undoc,// Via_undoc_F30FA6F0_64
	Mnemonic::Undoc,// Via_undoc_F30FA6F8_16
	Mnemonic::Undoc,// Via_undoc_F30FA6F8_32
	Mnemonic::Undoc,// Via_undoc_F30FA6F8_64
	Mnemonic::Xsha512,// Xsha512_16
	Mnemonic::Xsha512,// Xsha512_32
	Mnemonic::Xsha512,// Xsha512_64
	Mnemonic::Xstore_alt,// Xstore_alt_16
	Mnemonic::Xstore_alt,// Xstore_alt_32
	Mnemonic::Xstore_alt,// Xstore_alt_64
	Mnemonic::Xsha512_alt,// Xsha512_alt_16
	Mnemonic::Xsha512_alt,// Xsha512_alt_32
	Mnemonic::Xsha512_alt,// Xsha512_alt_64
	Mnemonic::Zero_bytes,// Zero_bytes
	Mnemonic::Wrmsrns,// Wrmsrns
	Mnemonic::Wrmsrlist,// Wrmsrlist
	Mnemonic::Rdmsrlist,// Rdmsrlist
	Mnemonic::Rmpquery,// Rmpquery
	Mnemonic::Prefetchit1,// Prefetchit1_m8
	Mnemonic::Prefetchit0,// Prefetchit0_m8
	Mnemonic::Aadd,// Aadd_m32_r32
	Mnemonic::Aadd,// Aadd_m64_r64
	Mnemonic::Aand,// Aand_m32_r32
	Mnemonic::Aand,// Aand_m64_r64
	Mnemonic::Axor,// Axor_m32_r32
	Mnemonic::Axor,// Axor_m64_r64
	Mnemonic::Aor,// Aor_m32_r32
	Mnemonic::Aor,// Aor_m64_r64
	Mnemonic::Vpdpbuud,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	Mnemonic::Vpdpbuud,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	Mnemonic::Vpdpbsud,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	Mnemonic::Vpdpbsud,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	Mnemonic::Vpdpbssd,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	Mnemonic::Vpdpbssd,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	Mnemonic::Vpdpbuuds,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	Mnemonic::Vpdpbuuds,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	Mnemonic::Vpdpbsuds,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	Mnemonic::Vpdpbsuds,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	Mnemonic::Vpdpbssds,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	Mnemonic::Vpdpbssds,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	Mnemonic::Tdpfp16ps,// VEX_Tdpfp16ps_tmm_tmm_tmm
	Mnemonic::Vcvtneps2bf16,// VEX_Vcvtneps2bf16_xmm_xmmm128
	Mnemonic::Vcvtneps2bf16,// VEX_Vcvtneps2bf16_xmm_ymmm256
	Mnemonic::Vcvtneoph2ps,// VEX_Vcvtneoph2ps_xmm_m128
	Mnemonic::Vcvtneoph2ps,// VEX_Vcvtneoph2ps_ymm_m256
	Mnemonic::Vcvtneeph2ps,// VEX_Vcvtneeph2ps_xmm_m128
	Mnemonic::Vcvtneeph2ps,// VEX_Vcvtneeph2ps_ymm_m256
	Mnemonic::Vcvtneebf162ps,// VEX_Vcvtneebf162ps_xmm_m128
	Mnemonic::Vcvtneebf162ps,// VEX_Vcvtneebf162ps_ymm_m256
	Mnemonic::Vcvtneobf162ps,// VEX_Vcvtneobf162ps_xmm_m128
	Mnemonic::Vcvtneobf162ps,// VEX_Vcvtneobf162ps_ymm_m256
	Mnemonic::Vbcstnesh2ps,// VEX_Vbcstnesh2ps_xmm_m16
	Mnemonic::Vbcstnesh2ps,// VEX_Vbcstnesh2ps_ymm_m16
	Mnemonic::Vbcstnebf162ps,// VEX_Vbcstnebf162ps_xmm_m16
	Mnemonic::Vbcstnebf162ps,// VEX_Vbcstnebf162ps_ymm_m16
	Mnemonic::Vpmadd52luq,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	Mnemonic::Vpmadd52luq,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	Mnemonic::Vpmadd52huq,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	Mnemonic::Vpmadd52huq,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	Mnemonic::Cmpoxadd,// VEX_Cmpoxadd_m32_r32_r32
	Mnemonic::Cmpoxadd,// VEX_Cmpoxadd_m64_r64_r64
	Mnemonic::Cmpnoxadd,// VEX_Cmpnoxadd_m32_r32_r32
	Mnemonic::Cmpnoxadd,// VEX_Cmpnoxadd_m64_r64_r64
	Mnemonic::Cmpbxadd,// VEX_Cmpbxadd_m32_r32_r32
	Mnemonic::Cmpbxadd,// VEX_Cmpbxadd_m64_r64_r64
	Mnemonic::Cmpnbxadd,// VEX_Cmpnbxadd_m32_r32_r32
	Mnemonic::Cmpnbxadd,// VEX_Cmpnbxadd_m64_r64_r64
	Mnemonic::Cmpzxadd,// VEX_Cmpzxadd_m32_r32_r32
	Mnemonic::Cmpzxadd,// VEX_Cmpzxadd_m64_r64_r64
	Mnemonic::Cmpnzxadd,// VEX_Cmpnzxadd_m32_r32_r32
	Mnemonic::Cmpnzxadd,// VEX_Cmpnzxadd_m64_r64_r64
	Mnemonic::Cmpbexadd,// VEX_Cmpbexadd_m32_r32_r32
	Mnemonic::Cmpbexadd,// VEX_Cmpbexadd_m64_r64_r64
	Mnemonic::Cmpnbexadd,// VEX_Cmpnbexadd_m32_r32_r32
	Mnemonic::Cmpnbexadd,// VEX_Cmpnbexadd_m64_r64_r64
	Mnemonic::Cmpsxadd,// VEX_Cmpsxadd_m32_r32_r32
	Mnemonic::Cmpsxadd,// VEX_Cmpsxadd_m64_r64_r64
	Mnemonic::Cmpnsxadd,// VEX_Cmpnsxadd_m32_r32_r32
	Mnemonic::Cmpnsxadd,// VEX_Cmpnsxadd_m64_r64_r64
	Mnemonic::Cmppxadd,// VEX_Cmppxadd_m32_r32_r32
	Mnemonic::Cmppxadd,// VEX_Cmppxadd_m64_r64_r64
	Mnemonic::Cmpnpxadd,// VEX_Cmpnpxadd_m32_r32_r32
	Mnemonic::Cmpnpxadd,// VEX_Cmpnpxadd_m64_r64_r64
	Mnemonic::Cmplxadd,// VEX_Cmplxadd_m32_r32_r32
	Mnemonic::Cmplxadd,// VEX_Cmplxadd_m64_r64_r64
	Mnemonic::Cmpnlxadd,// VEX_Cmpnlxadd_m32_r32_r32
	Mnemonic::Cmpnlxadd,// VEX_Cmpnlxadd_m64_r64_r64
	Mnemonic::Cmplexadd,// VEX_Cmplexadd_m32_r32_r32
	Mnemonic::Cmplexadd,// VEX_Cmplexadd_m64_r64_r64
	Mnemonic::Cmpnlexadd,// VEX_Cmpnlexadd_m32_r32_r32
	Mnemonic::Cmpnlexadd,// VEX_Cmpnlexadd_m64_r64_r64
	Mnemonic::Tcmmrlfp16ps,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	Mnemonic::Tcmmimfp16ps,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	Mnemonic::Pbndkb,// Pbndkb
	Mnemonic::Vsha512rnds2,// VEX_Vsha512rnds2_ymm_ymm_xmm
	Mnemonic::Vsha512msg1,// VEX_Vsha512msg1_ymm_xmm
	Mnemonic::Vsha512msg2,// VEX_Vsha512msg2_ymm_ymm
	Mnemonic::Vpdpwuud,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	Mnemonic::Vpdpwuud,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	Mnemonic::Vpdpwusd,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	Mnemonic::Vpdpwusd,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	Mnemonic::Vpdpwsud,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	Mnemonic::Vpdpwsud,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	Mnemonic::Vpdpwuuds,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	Mnemonic::Vpdpwuuds,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	Mnemonic::Vpdpwusds,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	Mnemonic::Vpdpwusds,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	Mnemonic::Vpdpwsuds,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	Mnemonic::Vpdpwsuds,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	Mnemonic::Vsm3msg1,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	Mnemonic::Vsm3msg2,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	Mnemonic::Vsm4key4,// VEX_Vsm4key4_xmm_xmm_xmmm128
	Mnemonic::Vsm4key4,// VEX_Vsm4key4_ymm_ymm_ymmm256
	Mnemonic::Vsm4rnds4,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	Mnemonic::Vsm4rnds4,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	Mnemonic::Vsm3rnds2,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];
