// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::iced_constants::IcedConstants;
use crate::MemorySize;

#[rustfmt::skip]
pub(super) static SIZES_NORMAL: [MemorySize; IcedConstants::CODE_ENUM_COUNT] = [
	MemorySize::Unknown,// INVALID
	MemorySize::Unknown,// DeclareByte
	MemorySize::Unknown,// DeclareWord
	MemorySize::Unknown,// DeclareDword
	MemorySize::Unknown,// DeclareQword
	MemorySize::UInt8,// Add_rm8_r8
	MemorySize::UInt16,// Add_rm16_r16
	MemorySize::UInt32,// Add_rm32_r32
	MemorySize::UInt64,// Add_rm64_r64
	MemorySize::UInt8,// Add_r8_rm8
	MemorySize::UInt16,// Add_r16_rm16
	MemorySize::UInt32,// Add_r32_rm32
	MemorySize::UInt64,// Add_r64_rm64
	MemorySize::Unknown,// Add_AL_imm8
	MemorySize::Unknown,// Add_AX_imm16
	MemorySize::Unknown,// Add_EAX_imm32
	MemorySize::Unknown,// Add_RAX_imm32
	MemorySize::Unknown,// Pushw_ES
	MemorySize::Unknown,// Pushd_ES
	MemorySize::Unknown,// Popw_ES
	MemorySize::Unknown,// Popd_ES
	MemorySize::UInt8,// Or_rm8_r8
	MemorySize::UInt16,// Or_rm16_r16
	MemorySize::UInt32,// Or_rm32_r32
	MemorySize::UInt64,// Or_rm64_r64
	MemorySize::UInt8,// Or_r8_rm8
	MemorySize::UInt16,// Or_r16_rm16
	MemorySize::UInt32,// Or_r32_rm32
	MemorySize::UInt64,// Or_r64_rm64
	MemorySize::Unknown,// Or_AL_imm8
	MemorySize::Unknown,// Or_AX_imm16
	MemorySize::Unknown,// Or_EAX_imm32
	MemorySize::Unknown,// Or_RAX_imm32
	MemorySize::Unknown,// Pushw_CS
	MemorySize::Unknown,// Pushd_CS
	MemorySize::Unknown,// Popw_CS
	MemorySize::UInt8,// Adc_rm8_r8
	MemorySize::UInt16,// Adc_rm16_r16
	MemorySize::UInt32,// Adc_rm32_r32
	MemorySize::UInt64,// Adc_rm64_r64
	MemorySize::UInt8,// Adc_r8_rm8
	MemorySize::UInt16,// Adc_r16_rm16
	MemorySize::UInt32,// Adc_r32_rm32
	MemorySize::UInt64,// Adc_r64_rm64
	MemorySize::Unknown,// Adc_AL_imm8
	MemorySize::Unknown,// Adc_AX_imm16
	MemorySize::Unknown,// Adc_EAX_imm32
	MemorySize::Unknown,// Adc_RAX_imm32
	MemorySize::Unknown,// Pushw_SS
	MemorySize::Unknown,// Pushd_SS
	MemorySize::Unknown,// Popw_SS
	MemorySize::Unknown,// Popd_SS
	MemorySize::UInt8,// Sbb_rm8_r8
	MemorySize::UInt16,// Sbb_rm16_r16
	MemorySize::UInt32,// Sbb_rm32_r32
	MemorySize::UInt64,// Sbb_rm64_r64
	MemorySize::UInt8,// Sbb_r8_rm8
	MemorySize::UInt16,// Sbb_r16_rm16
	MemorySize::UInt32,// Sbb_r32_rm32
	MemorySize::UInt64,// Sbb_r64_rm64
	MemorySize::Unknown,// Sbb_AL_imm8
	MemorySize::Unknown,// Sbb_AX_imm16
	MemorySize::Unknown,// Sbb_EAX_imm32
	MemorySize::Unknown,// Sbb_RAX_imm32
	MemorySize::Unknown,// Pushw_DS
	MemorySize::Unknown,// Pushd_DS
	MemorySize::Unknown,// Popw_DS
	MemorySize::Unknown,// Popd_DS
	MemorySize::UInt8,// And_rm8_r8
	MemorySize::UInt16,// And_rm16_r16
	MemorySize::UInt32,// And_rm32_r32
	MemorySize::UInt64,// And_rm64_r64
	MemorySize::UInt8,// And_r8_rm8
	MemorySize::UInt16,// And_r16_rm16
	MemorySize::UInt32,// And_r32_rm32
	MemorySize::UInt64,// And_r64_rm64
	MemorySize::Unknown,// And_AL_imm8
	MemorySize::Unknown,// And_AX_imm16
	MemorySize::Unknown,// And_EAX_imm32
	MemorySize::Unknown,// And_RAX_imm32
	MemorySize::Unknown,// Daa
	MemorySize::UInt8,// Sub_rm8_r8
	MemorySize::UInt16,// Sub_rm16_r16
	MemorySize::UInt32,// Sub_rm32_r32
	MemorySize::UInt64,// Sub_rm64_r64
	MemorySize::UInt8,// Sub_r8_rm8
	MemorySize::UInt16,// Sub_r16_rm16
	MemorySize::UInt32,// Sub_r32_rm32
	MemorySize::UInt64,// Sub_r64_rm64
	MemorySize::Unknown,// Sub_AL_imm8
	MemorySize::Unknown,// Sub_AX_imm16
	MemorySize::Unknown,// Sub_EAX_imm32
	MemorySize::Unknown,// Sub_RAX_imm32
	MemorySize::Unknown,// Das
	MemorySize::UInt8,// Xor_rm8_r8
	MemorySize::UInt16,// Xor_rm16_r16
	MemorySize::UInt32,// Xor_rm32_r32
	MemorySize::UInt64,// Xor_rm64_r64
	MemorySize::UInt8,// Xor_r8_rm8
	MemorySize::UInt16,// Xor_r16_rm16
	MemorySize::UInt32,// Xor_r32_rm32
	MemorySize::UInt64,// Xor_r64_rm64
	MemorySize::Unknown,// Xor_AL_imm8
	MemorySize::Unknown,// Xor_AX_imm16
	MemorySize::Unknown,// Xor_EAX_imm32
	MemorySize::Unknown,// Xor_RAX_imm32
	MemorySize::Unknown,// Aaa
	MemorySize::UInt8,// Cmp_rm8_r8
	MemorySize::UInt16,// Cmp_rm16_r16
	MemorySize::UInt32,// Cmp_rm32_r32
	MemorySize::UInt64,// Cmp_rm64_r64
	MemorySize::UInt8,// Cmp_r8_rm8
	MemorySize::UInt16,// Cmp_r16_rm16
	MemorySize::UInt32,// Cmp_r32_rm32
	MemorySize::UInt64,// Cmp_r64_rm64
	MemorySize::Unknown,// Cmp_AL_imm8
	MemorySize::Unknown,// Cmp_AX_imm16
	MemorySize::Unknown,// Cmp_EAX_imm32
	MemorySize::Unknown,// Cmp_RAX_imm32
	MemorySize::Unknown,// Aas
	MemorySize::Unknown,// Inc_r16
	MemorySize::Unknown,// Inc_r32
	MemorySize::Unknown,// Dec_r16
	MemorySize::Unknown,// Dec_r32
	MemorySize::Unknown,// Push_r16
	MemorySize::Unknown,// Push_r32
	MemorySize::Unknown,// Push_r64
	MemorySize::Unknown,// Pop_r16
	MemorySize::Unknown,// Pop_r32
	MemorySize::Unknown,// Pop_r64
	MemorySize::Unknown,// Pushaw
	MemorySize::Unknown,// Pushad
	MemorySize::Unknown,// Popaw
	MemorySize::Unknown,// Popad
	MemorySize::Bound16_WordWord,// Bound_r16_m1616
	MemorySize::Bound32_DwordDword,// Bound_r32_m3232
	MemorySize::UInt16,// Arpl_rm16_r16
	MemorySize::UInt16,// Arpl_r32m16_r32
	MemorySize::Int16,// Movsxd_r16_rm16
	MemorySize::Int32,// Movsxd_r32_rm32
	MemorySize::Int32,// Movsxd_r64_rm32
	MemorySize::Unknown,// Push_imm16
	MemorySize::Unknown,// Pushd_imm32
	MemorySize::Unknown,// Pushq_imm32
	MemorySize::Int16,// Imul_r16_rm16_imm16
	MemorySize::Int32,// Imul_r32_rm32_imm32
	MemorySize::Int64,// Imul_r64_rm64_imm32
	MemorySize::Unknown,// Pushw_imm8
	MemorySize::Unknown,// Pushd_imm8
	MemorySize::Unknown,// Pushq_imm8
	MemorySize::Int16,// Imul_r16_rm16_imm8
	MemorySize::Int32,// Imul_r32_rm32_imm8
	MemorySize::Int64,// Imul_r64_rm64_imm8
	MemorySize::UInt8,// Insb_m8_DX
	MemorySize::UInt16,// Insw_m16_DX
	MemorySize::UInt32,// Insd_m32_DX
	MemorySize::UInt8,// Outsb_DX_m8
	MemorySize::UInt16,// Outsw_DX_m16
	MemorySize::UInt32,// Outsd_DX_m32
	MemorySize::Unknown,// Jo_rel8_16
	MemorySize::Unknown,// Jo_rel8_32
	MemorySize::Unknown,// Jo_rel8_64
	MemorySize::Unknown,// Jno_rel8_16
	MemorySize::Unknown,// Jno_rel8_32
	MemorySize::Unknown,// Jno_rel8_64
	MemorySize::Unknown,// Jb_rel8_16
	MemorySize::Unknown,// Jb_rel8_32
	MemorySize::Unknown,// Jb_rel8_64
	MemorySize::Unknown,// Jae_rel8_16
	MemorySize::Unknown,// Jae_rel8_32
	MemorySize::Unknown,// Jae_rel8_64
	MemorySize::Unknown,// Je_rel8_16
	MemorySize::Unknown,// Je_rel8_32
	MemorySize::Unknown,// Je_rel8_64
	MemorySize::Unknown,// Jne_rel8_16
	MemorySize::Unknown,// Jne_rel8_32
	MemorySize::Unknown,// Jne_rel8_64
	MemorySize::Unknown,// Jbe_rel8_16
	MemorySize::Unknown,// Jbe_rel8_32
	MemorySize::Unknown,// Jbe_rel8_64
	MemorySize::Unknown,// Ja_rel8_16
	MemorySize::Unknown,// Ja_rel8_32
	MemorySize::Unknown,// Ja_rel8_64
	MemorySize::Unknown,// Js_rel8_16
	MemorySize::Unknown,// Js_rel8_32
	MemorySize::Unknown,// Js_rel8_64
	MemorySize::Unknown,// Jns_rel8_16
	MemorySize::Unknown,// Jns_rel8_32
	MemorySize::Unknown,// Jns_rel8_64
	MemorySize::Unknown,// Jp_rel8_16
	MemorySize::Unknown,// Jp_rel8_32
	MemorySize::Unknown,// Jp_rel8_64
	MemorySize::Unknown,// Jnp_rel8_16
	MemorySize::Unknown,// Jnp_rel8_32
	MemorySize::Unknown,// Jnp_rel8_64
	MemorySize::Unknown,// Jl_rel8_16
	MemorySize::Unknown,// Jl_rel8_32
	MemorySize::Unknown,// Jl_rel8_64
	MemorySize::Unknown,// Jge_rel8_16
	MemorySize::Unknown,// Jge_rel8_32
	MemorySize::Unknown,// Jge_rel8_64
	MemorySize::Unknown,// Jle_rel8_16
	MemorySize::Unknown,// Jle_rel8_32
	MemorySize::Unknown,// Jle_rel8_64
	MemorySize::Unknown,// Jg_rel8_16
	MemorySize::Unknown,// Jg_rel8_32
	MemorySize::Unknown,// Jg_rel8_64
	MemorySize::UInt8,// Add_rm8_imm8
	MemorySize::UInt8,// Or_rm8_imm8
	MemorySize::UInt8,// Adc_rm8_imm8
	MemorySize::UInt8,// Sbb_rm8_imm8
	MemorySize::UInt8,// And_rm8_imm8
	MemorySize::UInt8,// Sub_rm8_imm8
	MemorySize::UInt8,// Xor_rm8_imm8
	MemorySize::UInt8,// Cmp_rm8_imm8
	MemorySize::UInt16,// Add_rm16_imm16
	MemorySize::UInt32,// Add_rm32_imm32
	MemorySize::UInt64,// Add_rm64_imm32
	MemorySize::UInt16,// Or_rm16_imm16
	MemorySize::UInt32,// Or_rm32_imm32
	MemorySize::UInt64,// Or_rm64_imm32
	MemorySize::UInt16,// Adc_rm16_imm16
	MemorySize::UInt32,// Adc_rm32_imm32
	MemorySize::UInt64,// Adc_rm64_imm32
	MemorySize::UInt16,// Sbb_rm16_imm16
	MemorySize::UInt32,// Sbb_rm32_imm32
	MemorySize::UInt64,// Sbb_rm64_imm32
	MemorySize::UInt16,// And_rm16_imm16
	MemorySize::UInt32,// And_rm32_imm32
	MemorySize::UInt64,// And_rm64_imm32
	MemorySize::UInt16,// Sub_rm16_imm16
	MemorySize::UInt32,// Sub_rm32_imm32
	MemorySize::UInt64,// Sub_rm64_imm32
	MemorySize::UInt16,// Xor_rm16_imm16
	MemorySize::UInt32,// Xor_rm32_imm32
	MemorySize::UInt64,// Xor_rm64_imm32
	MemorySize::UInt16,// Cmp_rm16_imm16
	MemorySize::UInt32,// Cmp_rm32_imm32
	MemorySize::UInt64,// Cmp_rm64_imm32
	MemorySize::UInt8,// Add_rm8_imm8_82
	MemorySize::UInt8,// Or_rm8_imm8_82
	MemorySize::UInt8,// Adc_rm8_imm8_82
	MemorySize::UInt8,// Sbb_rm8_imm8_82
	MemorySize::UInt8,// And_rm8_imm8_82
	MemorySize::UInt8,// Sub_rm8_imm8_82
	MemorySize::UInt8,// Xor_rm8_imm8_82
	MemorySize::UInt8,// Cmp_rm8_imm8_82
	MemorySize::UInt16,// Add_rm16_imm8
	MemorySize::UInt32,// Add_rm32_imm8
	MemorySize::UInt64,// Add_rm64_imm8
	MemorySize::UInt16,// Or_rm16_imm8
	MemorySize::UInt32,// Or_rm32_imm8
	MemorySize::UInt64,// Or_rm64_imm8
	MemorySize::UInt16,// Adc_rm16_imm8
	MemorySize::UInt32,// Adc_rm32_imm8
	MemorySize::UInt64,// Adc_rm64_imm8
	MemorySize::UInt16,// Sbb_rm16_imm8
	MemorySize::UInt32,// Sbb_rm32_imm8
	MemorySize::UInt64,// Sbb_rm64_imm8
	MemorySize::UInt16,// And_rm16_imm8
	MemorySize::UInt32,// And_rm32_imm8
	MemorySize::UInt64,// And_rm64_imm8
	MemorySize::UInt16,// Sub_rm16_imm8
	MemorySize::UInt32,// Sub_rm32_imm8
	MemorySize::UInt64,// Sub_rm64_imm8
	MemorySize::UInt16,// Xor_rm16_imm8
	MemorySize::UInt32,// Xor_rm32_imm8
	MemorySize::UInt64,// Xor_rm64_imm8
	MemorySize::UInt16,// Cmp_rm16_imm8
	MemorySize::UInt32,// Cmp_rm32_imm8
	MemorySize::UInt64,// Cmp_rm64_imm8
	MemorySize::UInt8,// Test_rm8_r8
	MemorySize::UInt16,// Test_rm16_r16
	MemorySize::UInt32,// Test_rm32_r32
	MemorySize::UInt64,// Test_rm64_r64
	MemorySize::UInt8,// Xchg_rm8_r8
	MemorySize::UInt16,// Xchg_rm16_r16
	MemorySize::UInt32,// Xchg_rm32_r32
	MemorySize::UInt64,// Xchg_rm64_r64
	MemorySize::UInt8,// Mov_rm8_r8
	MemorySize::UInt16,// Mov_rm16_r16
	MemorySize::UInt32,// Mov_rm32_r32
	MemorySize::UInt64,// Mov_rm64_r64
	MemorySize::UInt8,// Mov_r8_rm8
	MemorySize::UInt16,// Mov_r16_rm16
	MemorySize::UInt32,// Mov_r32_rm32
	MemorySize::UInt64,// Mov_r64_rm64
	MemorySize::UInt16,// Mov_rm16_Sreg
	MemorySize::UInt16,// Mov_r32m16_Sreg
	MemorySize::UInt16,// Mov_r64m16_Sreg
	MemorySize::Unknown,// Lea_r16_m
	MemorySize::Unknown,// Lea_r32_m
	MemorySize::Unknown,// Lea_r64_m
	MemorySize::UInt16,// Mov_Sreg_rm16
	MemorySize::UInt16,// Mov_Sreg_r32m16
	MemorySize::UInt16,// Mov_Sreg_r64m16
	MemorySize::UInt16,// Pop_rm16
	MemorySize::UInt32,// Pop_rm32
	MemorySize::UInt64,// Pop_rm64
	MemorySize::Unknown,// Nopw
	MemorySize::Unknown,// Nopd
	MemorySize::Unknown,// Nopq
	MemorySize::Unknown,// Xchg_r16_AX
	MemorySize::Unknown,// Xchg_r32_EAX
	MemorySize::Unknown,// Xchg_r64_RAX
	MemorySize::Unknown,// Pause
	MemorySize::Unknown,// Cbw
	MemorySize::Unknown,// Cwde
	MemorySize::Unknown,// Cdqe
	MemorySize::Unknown,// Cwd
	MemorySize::Unknown,// Cdq
	MemorySize::Unknown,// Cqo
	MemorySize::Unknown,// Call_ptr1616
	MemorySize::Unknown,// Call_ptr1632
	MemorySize::Unknown,// Wait
	MemorySize::Unknown,// Pushfw
	MemorySize::Unknown,// Pushfd
	MemorySize::Unknown,// Pushfq
	MemorySize::Unknown,// Popfw
	MemorySize::Unknown,// Popfd
	MemorySize::Unknown,// Popfq
	MemorySize::Unknown,// Sahf
	MemorySize::Unknown,// Lahf
	MemorySize::UInt8,// Mov_AL_moffs8
	MemorySize::UInt16,// Mov_AX_moffs16
	MemorySize::UInt32,// Mov_EAX_moffs32
	MemorySize::UInt64,// Mov_RAX_moffs64
	MemorySize::UInt8,// Mov_moffs8_AL
	MemorySize::UInt16,// Mov_moffs16_AX
	MemorySize::UInt32,// Mov_moffs32_EAX
	MemorySize::UInt64,// Mov_moffs64_RAX
	MemorySize::UInt8,// Movsb_m8_m8
	MemorySize::UInt16,// Movsw_m16_m16
	MemorySize::UInt32,// Movsd_m32_m32
	MemorySize::UInt64,// Movsq_m64_m64
	MemorySize::UInt8,// Cmpsb_m8_m8
	MemorySize::UInt16,// Cmpsw_m16_m16
	MemorySize::UInt32,// Cmpsd_m32_m32
	MemorySize::UInt64,// Cmpsq_m64_m64
	MemorySize::Unknown,// Test_AL_imm8
	MemorySize::Unknown,// Test_AX_imm16
	MemorySize::Unknown,// Test_EAX_imm32
	MemorySize::Unknown,// Test_RAX_imm32
	MemorySize::UInt8,// Stosb_m8_AL
	MemorySize::UInt16,// Stosw_m16_AX
	MemorySize::UInt32,// Stosd_m32_EAX
	MemorySize::UInt64,// Stosq_m64_RAX
	MemorySize::UInt8,// Lodsb_AL_m8
	MemorySize::UInt16,// Lodsw_AX_m16
	MemorySize::UInt32,// Lodsd_EAX_m32
	MemorySize::UInt64,// Lodsq_RAX_m64
	MemorySize::UInt8,// Scasb_AL_m8
	MemorySize::UInt16,// Scasw_AX_m16
	MemorySize::UInt32,// Scasd_EAX_m32
	MemorySize::UInt64,// Scasq_RAX_m64
	MemorySize::Unknown,// Mov_r8_imm8
	MemorySize::Unknown,// Mov_r16_imm16
	MemorySize::Unknown,// Mov_r32_imm32
	MemorySize::Unknown,// Mov_r64_imm64
	MemorySize::UInt8,// Rol_rm8_imm8
	MemorySize::UInt8,// Ror_rm8_imm8
	MemorySize::UInt8,// Rcl_rm8_imm8
	MemorySize::UInt8,// Rcr_rm8_imm8
	MemorySize::UInt8,// Shl_rm8_imm8
	MemorySize::UInt8,// Shr_rm8_imm8
	MemorySize::UInt8,// Sal_rm8_imm8
	MemorySize::Int8,// Sar_rm8_imm8
	MemorySize::UInt16,// Rol_rm16_imm8
	MemorySize::UInt32,// Rol_rm32_imm8
	MemorySize::UInt64,// Rol_rm64_imm8
	MemorySize::UInt16,// Ror_rm16_imm8
	MemorySize::UInt32,// Ror_rm32_imm8
	MemorySize::UInt64,// Ror_rm64_imm8
	MemorySize::UInt16,// Rcl_rm16_imm8
	MemorySize::UInt32,// Rcl_rm32_imm8
	MemorySize::UInt64,// Rcl_rm64_imm8
	MemorySize::UInt16,// Rcr_rm16_imm8
	MemorySize::UInt32,// Rcr_rm32_imm8
	MemorySize::UInt64,// Rcr_rm64_imm8
	MemorySize::UInt16,// Shl_rm16_imm8
	MemorySize::UInt32,// Shl_rm32_imm8
	MemorySize::UInt64,// Shl_rm64_imm8
	MemorySize::UInt16,// Shr_rm16_imm8
	MemorySize::UInt32,// Shr_rm32_imm8
	MemorySize::UInt64,// Shr_rm64_imm8
	MemorySize::UInt16,// Sal_rm16_imm8
	MemorySize::UInt32,// Sal_rm32_imm8
	MemorySize::UInt64,// Sal_rm64_imm8
	MemorySize::Int16,// Sar_rm16_imm8
	MemorySize::Int32,// Sar_rm32_imm8
	MemorySize::Int64,// Sar_rm64_imm8
	MemorySize::Unknown,// Retnw_imm16
	MemorySize::Unknown,// Retnd_imm16
	MemorySize::Unknown,// Retnq_imm16
	MemorySize::Unknown,// Retnw
	MemorySize::Unknown,// Retnd
	MemorySize::Unknown,// Retnq
	MemorySize::SegPtr16,// Les_r16_m1616
	MemorySize::SegPtr32,// Les_r32_m1632
	MemorySize::SegPtr16,// Lds_r16_m1616
	MemorySize::SegPtr32,// Lds_r32_m1632
	MemorySize::UInt8,// Mov_rm8_imm8
	MemorySize::Unknown,// Xabort_imm8
	MemorySize::UInt16,// Mov_rm16_imm16
	MemorySize::UInt32,// Mov_rm32_imm32
	MemorySize::UInt64,// Mov_rm64_imm32
	MemorySize::Unknown,// Xbegin_rel16
	MemorySize::Unknown,// Xbegin_rel32
	MemorySize::Unknown,// Enterw_imm16_imm8
	MemorySize::Unknown,// Enterd_imm16_imm8
	MemorySize::Unknown,// Enterq_imm16_imm8
	MemorySize::Unknown,// Leavew
	MemorySize::Unknown,// Leaved
	MemorySize::Unknown,// Leaveq
	MemorySize::Unknown,// Retfw_imm16
	MemorySize::Unknown,// Retfd_imm16
	MemorySize::Unknown,// Retfq_imm16
	MemorySize::Unknown,// Retfw
	MemorySize::Unknown,// Retfd
	MemorySize::Unknown,// Retfq
	MemorySize::Unknown,// Int3
	MemorySize::Unknown,// Int_imm8
	MemorySize::Unknown,// Into
	MemorySize::Unknown,// Iretw
	MemorySize::Unknown,// Iretd
	MemorySize::Unknown,// Iretq
	MemorySize::UInt8,// Rol_rm8_1
	MemorySize::UInt8,// Ror_rm8_1
	MemorySize::UInt8,// Rcl_rm8_1
	MemorySize::UInt8,// Rcr_rm8_1
	MemorySize::UInt8,// Shl_rm8_1
	MemorySize::UInt8,// Shr_rm8_1
	MemorySize::UInt8,// Sal_rm8_1
	MemorySize::Int8,// Sar_rm8_1
	MemorySize::UInt16,// Rol_rm16_1
	MemorySize::UInt32,// Rol_rm32_1
	MemorySize::UInt64,// Rol_rm64_1
	MemorySize::UInt16,// Ror_rm16_1
	MemorySize::UInt32,// Ror_rm32_1
	MemorySize::UInt64,// Ror_rm64_1
	MemorySize::UInt16,// Rcl_rm16_1
	MemorySize::UInt32,// Rcl_rm32_1
	MemorySize::UInt64,// Rcl_rm64_1
	MemorySize::UInt16,// Rcr_rm16_1
	MemorySize::UInt32,// Rcr_rm32_1
	MemorySize::UInt64,// Rcr_rm64_1
	MemorySize::UInt16,// Shl_rm16_1
	MemorySize::UInt32,// Shl_rm32_1
	MemorySize::UInt64,// Shl_rm64_1
	MemorySize::UInt16,// Shr_rm16_1
	MemorySize::UInt32,// Shr_rm32_1
	MemorySize::UInt64,// Shr_rm64_1
	MemorySize::UInt16,// Sal_rm16_1
	MemorySize::UInt32,// Sal_rm32_1
	MemorySize::UInt64,// Sal_rm64_1
	MemorySize::Int16,// Sar_rm16_1
	MemorySize::Int32,// Sar_rm32_1
	MemorySize::Int64,// Sar_rm64_1
	MemorySize::UInt8,// Rol_rm8_CL
	MemorySize::UInt8,// Ror_rm8_CL
	MemorySize::UInt8,// Rcl_rm8_CL
	MemorySize::UInt8,// Rcr_rm8_CL
	MemorySize::UInt8,// Shl_rm8_CL
	MemorySize::UInt8,// Shr_rm8_CL
	MemorySize::UInt8,// Sal_rm8_CL
	MemorySize::Int8,// Sar_rm8_CL
	MemorySize::UInt16,// Rol_rm16_CL
	MemorySize::UInt32,// Rol_rm32_CL
	MemorySize::UInt64,// Rol_rm64_CL
	MemorySize::UInt16,// Ror_rm16_CL
	MemorySize::UInt32,// Ror_rm32_CL
	MemorySize::UInt64,// Ror_rm64_CL
	MemorySize::UInt16,// Rcl_rm16_CL
	MemorySize::UInt32,// Rcl_rm32_CL
	MemorySize::UInt64,// Rcl_rm64_CL
	MemorySize::UInt16,// Rcr_rm16_CL
	MemorySize::UInt32,// Rcr_rm32_CL
	MemorySize::UInt64,// Rcr_rm64_CL
	MemorySize::UInt16,// Shl_rm16_CL
	MemorySize::UInt32,// Shl_rm32_CL
	MemorySize::UInt64,// Shl_rm64_CL
	MemorySize::UInt16,// Shr_rm16_CL
	MemorySize::UInt32,// Shr_rm32_CL
	MemorySize::UInt64,// Shr_rm64_CL
	MemorySize::UInt16,// Sal_rm16_CL
	MemorySize::UInt32,// Sal_rm32_CL
	MemorySize::UInt64,// Sal_rm64_CL
	MemorySize::Int16,// Sar_rm16_CL
	MemorySize::Int32,// Sar_rm32_CL
	MemorySize::Int64,// Sar_rm64_CL
	MemorySize::Unknown,// Aam_imm8
	MemorySize::Unknown,// Aad_imm8
	MemorySize::Unknown,// Salc
	MemorySize::UInt8,// Xlat_m8
	MemorySize::Float32,// Fadd_m32fp
	MemorySize::Float32,// Fmul_m32fp
	MemorySize::Float32,// Fcom_m32fp
	MemorySize::Float32,// Fcomp_m32fp
	MemorySize::Float32,// Fsub_m32fp
	MemorySize::Float32,// Fsubr_m32fp
	MemorySize::Float32,// Fdiv_m32fp
	MemorySize::Float32,// Fdivr_m32fp
	MemorySize::Unknown,// Fadd_st0_sti
	MemorySize::Unknown,// Fmul_st0_sti
	MemorySize::Unknown,// Fcom_st0_sti
	MemorySize::Unknown,// Fcomp_st0_sti
	MemorySize::Unknown,// Fsub_st0_sti
	MemorySize::Unknown,// Fsubr_st0_sti
	MemorySize::Unknown,// Fdiv_st0_sti
	MemorySize::Unknown,// Fdivr_st0_sti
	MemorySize::Float32,// Fld_m32fp
	MemorySize::Float32,// Fst_m32fp
	MemorySize::Float32,// Fstp_m32fp
	MemorySize::FpuEnv14,// Fldenv_m14byte
	MemorySize::FpuEnv28,// Fldenv_m28byte
	MemorySize::UInt16,// Fldcw_m2byte
	MemorySize::FpuEnv14,// Fnstenv_m14byte
	MemorySize::FpuEnv14,// Fstenv_m14byte
	MemorySize::FpuEnv28,// Fnstenv_m28byte
	MemorySize::FpuEnv28,// Fstenv_m28byte
	MemorySize::UInt16,// Fnstcw_m2byte
	MemorySize::UInt16,// Fstcw_m2byte
	MemorySize::Unknown,// Fld_sti
	MemorySize::Unknown,// Fxch_st0_sti
	MemorySize::Unknown,// Fnop
	MemorySize::Unknown,// Fstpnce_sti
	MemorySize::Unknown,// Fchs
	MemorySize::Unknown,// Fabs
	MemorySize::Unknown,// Ftst
	MemorySize::Unknown,// Fxam
	MemorySize::Unknown,// Fld1
	MemorySize::Unknown,// Fldl2t
	MemorySize::Unknown,// Fldl2e
	MemorySize::Unknown,// Fldpi
	MemorySize::Unknown,// Fldlg2
	MemorySize::Unknown,// Fldln2
	MemorySize::Unknown,// Fldz
	MemorySize::Unknown,// F2xm1
	MemorySize::Unknown,// Fyl2x
	MemorySize::Unknown,// Fptan
	MemorySize::Unknown,// Fpatan
	MemorySize::Unknown,// Fxtract
	MemorySize::Unknown,// Fprem1
	MemorySize::Unknown,// Fdecstp
	MemorySize::Unknown,// Fincstp
	MemorySize::Unknown,// Fprem
	MemorySize::Unknown,// Fyl2xp1
	MemorySize::Unknown,// Fsqrt
	MemorySize::Unknown,// Fsincos
	MemorySize::Unknown,// Frndint
	MemorySize::Unknown,// Fscale
	MemorySize::Unknown,// Fsin
	MemorySize::Unknown,// Fcos
	MemorySize::Int32,// Fiadd_m32int
	MemorySize::Int32,// Fimul_m32int
	MemorySize::Int32,// Ficom_m32int
	MemorySize::Int32,// Ficomp_m32int
	MemorySize::Int32,// Fisub_m32int
	MemorySize::Int32,// Fisubr_m32int
	MemorySize::Int32,// Fidiv_m32int
	MemorySize::Int32,// Fidivr_m32int
	MemorySize::Unknown,// Fcmovb_st0_sti
	MemorySize::Unknown,// Fcmove_st0_sti
	MemorySize::Unknown,// Fcmovbe_st0_sti
	MemorySize::Unknown,// Fcmovu_st0_sti
	MemorySize::Unknown,// Fucompp
	MemorySize::Int32,// Fild_m32int
	MemorySize::Int32,// Fisttp_m32int
	MemorySize::Int32,// Fist_m32int
	MemorySize::Int32,// Fistp_m32int
	MemorySize::Float80,// Fld_m80fp
	MemorySize::Float80,// Fstp_m80fp
	MemorySize::Unknown,// Fcmovnb_st0_sti
	MemorySize::Unknown,// Fcmovne_st0_sti
	MemorySize::Unknown,// Fcmovnbe_st0_sti
	MemorySize::Unknown,// Fcmovnu_st0_sti
	MemorySize::Unknown,// Fneni
	MemorySize::Unknown,// Feni
	MemorySize::Unknown,// Fndisi
	MemorySize::Unknown,// Fdisi
	MemorySize::Unknown,// Fnclex
	MemorySize::Unknown,// Fclex
	MemorySize::Unknown,// Fninit
	MemorySize::Unknown,// Finit
	MemorySize::Unknown,// Fnsetpm
	MemorySize::Unknown,// Fsetpm
	MemorySize::Unknown,// Frstpm
	MemorySize::Unknown,// Fucomi_st0_sti
	MemorySize::Unknown,// Fcomi_st0_sti
	MemorySize::Float64,// Fadd_m64fp
	MemorySize::Float64,// Fmul_m64fp
	MemorySize::Float64,// Fcom_m64fp
	MemorySize::Float64,// Fcomp_m64fp
	MemorySize::Float64,// Fsub_m64fp
	MemorySize::Float64,// Fsubr_m64fp
	MemorySize::Float64,// Fdiv_m64fp
	MemorySize::Float64,// Fdivr_m64fp
	MemorySize::Unknown,// Fadd_sti_st0
	MemorySize::Unknown,// Fmul_sti_st0
	MemorySize::Unknown,// Fcom_st0_sti_DCD0
	MemorySize::Unknown,// Fcomp_st0_sti_DCD8
	MemorySize::Unknown,// Fsubr_sti_st0
	MemorySize::Unknown,// Fsub_sti_st0
	MemorySize::Unknown,// Fdivr_sti_st0
	MemorySize::Unknown,// Fdiv_sti_st0
	MemorySize::Float64,// Fld_m64fp
	MemorySize::Int64,// Fisttp_m64int
	MemorySize::Float64,// Fst_m64fp
	MemorySize::Float64,// Fstp_m64fp
	MemorySize::FpuState94,// Frstor_m94byte
	MemorySize::FpuState108,// Frstor_m108byte
	MemorySize::FpuState94,// Fnsave_m94byte
	MemorySize::FpuState94,// Fsave_m94byte
	MemorySize::FpuState108,// Fnsave_m108byte
	MemorySize::FpuState108,// Fsave_m108byte
	MemorySize::UInt16,// Fnstsw_m2byte
	MemorySize::UInt16,// Fstsw_m2byte
	MemorySize::Unknown,// Ffree_sti
	MemorySize::Unknown,// Fxch_st0_sti_DDC8
	MemorySize::Unknown,// Fst_sti
	MemorySize::Unknown,// Fstp_sti
	MemorySize::Unknown,// Fucom_st0_sti
	MemorySize::Unknown,// Fucomp_st0_sti
	MemorySize::Int16,// Fiadd_m16int
	MemorySize::Int16,// Fimul_m16int
	MemorySize::Int16,// Ficom_m16int
	MemorySize::Int16,// Ficomp_m16int
	MemorySize::Int16,// Fisub_m16int
	MemorySize::Int16,// Fisubr_m16int
	MemorySize::Int16,// Fidiv_m16int
	MemorySize::Int16,// Fidivr_m16int
	MemorySize::Unknown,// Faddp_sti_st0
	MemorySize::Unknown,// Fmulp_sti_st0
	MemorySize::Unknown,// Fcomp_st0_sti_DED0
	MemorySize::Unknown,// Fcompp
	MemorySize::Unknown,// Fsubrp_sti_st0
	MemorySize::Unknown,// Fsubp_sti_st0
	MemorySize::Unknown,// Fdivrp_sti_st0
	MemorySize::Unknown,// Fdivp_sti_st0
	MemorySize::Int16,// Fild_m16int
	MemorySize::Int16,// Fisttp_m16int
	MemorySize::Int16,// Fist_m16int
	MemorySize::Int16,// Fistp_m16int
	MemorySize::Bcd,// Fbld_m80bcd
	MemorySize::Int64,// Fild_m64int
	MemorySize::Bcd,// Fbstp_m80bcd
	MemorySize::Int64,// Fistp_m64int
	MemorySize::Unknown,// Ffreep_sti
	MemorySize::Unknown,// Fxch_st0_sti_DFC8
	MemorySize::Unknown,// Fstp_sti_DFD0
	MemorySize::Unknown,// Fstp_sti_DFD8
	MemorySize::Unknown,// Fnstsw_AX
	MemorySize::Unknown,// Fstsw_AX
	MemorySize::Unknown,// Fstdw_AX
	MemorySize::Unknown,// Fstsg_AX
	MemorySize::Unknown,// Fucomip_st0_sti
	MemorySize::Unknown,// Fcomip_st0_sti
	MemorySize::Unknown,// Loopne_rel8_16_CX
	MemorySize::Unknown,// Loopne_rel8_32_CX
	MemorySize::Unknown,// Loopne_rel8_16_ECX
	MemorySize::Unknown,// Loopne_rel8_32_ECX
	MemorySize::Unknown,// Loopne_rel8_64_ECX
	MemorySize::Unknown,// Loopne_rel8_16_RCX
	MemorySize::Unknown,// Loopne_rel8_64_RCX
	MemorySize::Unknown,// Loope_rel8_16_CX
	MemorySize::Unknown,// Loope_rel8_32_CX
	MemorySize::Unknown,// Loope_rel8_16_ECX
	MemorySize::Unknown,// Loope_rel8_32_ECX
	MemorySize::Unknown,// Loope_rel8_64_ECX
	MemorySize::Unknown,// Loope_rel8_16_RCX
	MemorySize::Unknown,// Loope_rel8_64_RCX
	MemorySize::Unknown,// Loop_rel8_16_CX
	MemorySize::Unknown,// Loop_rel8_32_CX
	MemorySize::Unknown,// Loop_rel8_16_ECX
	MemorySize::Unknown,// Loop_rel8_32_ECX
	MemorySize::Unknown,// Loop_rel8_64_ECX
	MemorySize::Unknown,// Loop_rel8_16_RCX
	MemorySize::Unknown,// Loop_rel8_64_RCX
	MemorySize::Unknown,// Jcxz_rel8_16
	MemorySize::Unknown,// Jcxz_rel8_32
	MemorySize::Unknown,// Jecxz_rel8_16
	MemorySize::Unknown,// Jecxz_rel8_32
	MemorySize::Unknown,// Jecxz_rel8_64
	MemorySize::Unknown,// Jrcxz_rel8_16
	MemorySize::Unknown,// Jrcxz_rel8_64
	MemorySize::Unknown,// In_AL_imm8
	MemorySize::Unknown,// In_AX_imm8
	MemorySize::Unknown,// In_EAX_imm8
	MemorySize::Unknown,// Out_imm8_AL
	MemorySize::Unknown,// Out_imm8_AX
	MemorySize::Unknown,// Out_imm8_EAX
	MemorySize::Unknown,// Call_rel16
	MemorySize::Unknown,// Call_rel32_32
	MemorySize::Unknown,// Call_rel32_64
	MemorySize::Unknown,// Jmp_rel16
	MemorySize::Unknown,// Jmp_rel32_32
	MemorySize::Unknown,// Jmp_rel32_64
	MemorySize::Unknown,// Jmp_ptr1616
	MemorySize::Unknown,// Jmp_ptr1632
	MemorySize::Unknown,// Jmp_rel8_16
	MemorySize::Unknown,// Jmp_rel8_32
	MemorySize::Unknown,// Jmp_rel8_64
	MemorySize::Unknown,// In_AL_DX
	MemorySize::Unknown,// In_AX_DX
	MemorySize::Unknown,// In_EAX_DX
	MemorySize::Unknown,// Out_DX_AL
	MemorySize::Unknown,// Out_DX_AX
	MemorySize::Unknown,// Out_DX_EAX
	MemorySize::Unknown,// Int1
	MemorySize::Unknown,// Hlt
	MemorySize::Unknown,// Cmc
	MemorySize::UInt8,// Test_rm8_imm8
	MemorySize::UInt8,// Test_rm8_imm8_F6r1
	MemorySize::UInt8,// Not_rm8
	MemorySize::Int8,// Neg_rm8
	MemorySize::UInt8,// Mul_rm8
	MemorySize::Int8,// Imul_rm8
	MemorySize::UInt8,// Div_rm8
	MemorySize::Int8,// Idiv_rm8
	MemorySize::UInt16,// Test_rm16_imm16
	MemorySize::UInt32,// Test_rm32_imm32
	MemorySize::UInt64,// Test_rm64_imm32
	MemorySize::UInt16,// Test_rm16_imm16_F7r1
	MemorySize::UInt32,// Test_rm32_imm32_F7r1
	MemorySize::UInt64,// Test_rm64_imm32_F7r1
	MemorySize::UInt16,// Not_rm16
	MemorySize::UInt32,// Not_rm32
	MemorySize::UInt64,// Not_rm64
	MemorySize::Int16,// Neg_rm16
	MemorySize::Int32,// Neg_rm32
	MemorySize::Int64,// Neg_rm64
	MemorySize::UInt16,// Mul_rm16
	MemorySize::UInt32,// Mul_rm32
	MemorySize::UInt64,// Mul_rm64
	MemorySize::Int16,// Imul_rm16
	MemorySize::Int32,// Imul_rm32
	MemorySize::Int64,// Imul_rm64
	MemorySize::UInt16,// Div_rm16
	MemorySize::UInt32,// Div_rm32
	MemorySize::UInt64,// Div_rm64
	MemorySize::Int16,// Idiv_rm16
	MemorySize::Int32,// Idiv_rm32
	MemorySize::Int64,// Idiv_rm64
	MemorySize::Unknown,// Clc
	MemorySize::Unknown,// Stc
	MemorySize::Unknown,// Cli
	MemorySize::Unknown,// Sti
	MemorySize::Unknown,// Cld
	MemorySize::Unknown,// Std
	MemorySize::UInt8,// Inc_rm8
	MemorySize::UInt8,// Dec_rm8
	MemorySize::UInt16,// Inc_rm16
	MemorySize::UInt32,// Inc_rm32
	MemorySize::UInt64,// Inc_rm64
	MemorySize::UInt16,// Dec_rm16
	MemorySize::UInt32,// Dec_rm32
	MemorySize::UInt64,// Dec_rm64
	MemorySize::WordOffset,// Call_rm16
	MemorySize::DwordOffset,// Call_rm32
	MemorySize::QwordOffset,// Call_rm64
	MemorySize::SegPtr16,// Call_m1616
	MemorySize::SegPtr32,// Call_m1632
	MemorySize::SegPtr64,// Call_m1664
	MemorySize::WordOffset,// Jmp_rm16
	MemorySize::DwordOffset,// Jmp_rm32
	MemorySize::QwordOffset,// Jmp_rm64
	MemorySize::SegPtr16,// Jmp_m1616
	MemorySize::SegPtr32,// Jmp_m1632
	MemorySize::SegPtr64,// Jmp_m1664
	MemorySize::UInt16,// Push_rm16
	MemorySize::UInt32,// Push_rm32
	MemorySize::UInt64,// Push_rm64
	MemorySize::UInt16,// Sldt_rm16
	MemorySize::UInt16,// Sldt_r32m16
	MemorySize::UInt16,// Sldt_r64m16
	MemorySize::UInt16,// Str_rm16
	MemorySize::UInt16,// Str_r32m16
	MemorySize::UInt16,// Str_r64m16
	MemorySize::UInt16,// Lldt_rm16
	MemorySize::UInt16,// Lldt_r32m16
	MemorySize::UInt16,// Lldt_r64m16
	MemorySize::UInt16,// Ltr_rm16
	MemorySize::UInt16,// Ltr_r32m16
	MemorySize::UInt16,// Ltr_r64m16
	MemorySize::UInt16,// Verr_rm16
	MemorySize::UInt16,// Verr_r32m16
	MemorySize::UInt16,// Verr_r64m16
	MemorySize::UInt16,// Verw_rm16
	MemorySize::UInt16,// Verw_r32m16
	MemorySize::UInt16,// Verw_r64m16
	MemorySize::WordOffset,// Jmpe_rm16
	MemorySize::DwordOffset,// Jmpe_rm32
	MemorySize::Fword6,// Sgdt_m1632_16
	MemorySize::Fword6,// Sgdt_m1632
	MemorySize::Fword10,// Sgdt_m1664
	MemorySize::Fword6,// Sidt_m1632_16
	MemorySize::Fword6,// Sidt_m1632
	MemorySize::Fword10,// Sidt_m1664
	MemorySize::Fword6,// Lgdt_m1632_16
	MemorySize::Fword6,// Lgdt_m1632
	MemorySize::Fword10,// Lgdt_m1664
	MemorySize::Fword6,// Lidt_m1632_16
	MemorySize::Fword6,// Lidt_m1632
	MemorySize::Fword10,// Lidt_m1664
	MemorySize::UInt16,// Smsw_rm16
	MemorySize::UInt16,// Smsw_r32m16
	MemorySize::UInt16,// Smsw_r64m16
	MemorySize::UInt64,// Rstorssp_m64
	MemorySize::UInt16,// Lmsw_rm16
	MemorySize::UInt16,// Lmsw_r32m16
	MemorySize::UInt16,// Lmsw_r64m16
	MemorySize::Unknown,// Invlpg_m
	MemorySize::Unknown,// Enclv
	MemorySize::Unknown,// Vmcall
	MemorySize::Unknown,// Vmlaunch
	MemorySize::Unknown,// Vmresume
	MemorySize::Unknown,// Vmxoff
	MemorySize::Unknown,// Pconfig
	MemorySize::Unknown,// Monitorw
	MemorySize::Unknown,// Monitord
	MemorySize::Unknown,// Monitorq
	MemorySize::Unknown,// Mwait
	MemorySize::Unknown,// Clac
	MemorySize::Unknown,// Stac
	MemorySize::Unknown,// Encls
	MemorySize::Unknown,// Xgetbv
	MemorySize::Unknown,// Xsetbv
	MemorySize::Unknown,// Vmfunc
	MemorySize::Unknown,// Xend
	MemorySize::Unknown,// Xtest
	MemorySize::Unknown,// Enclu
	MemorySize::Unknown,// Vmrunw
	MemorySize::Unknown,// Vmrund
	MemorySize::Unknown,// Vmrunq
	MemorySize::Unknown,// Vmmcall
	MemorySize::Unknown,// Vmloadw
	MemorySize::Unknown,// Vmloadd
	MemorySize::Unknown,// Vmloadq
	MemorySize::Unknown,// Vmsavew
	MemorySize::Unknown,// Vmsaved
	MemorySize::Unknown,// Vmsaveq
	MemorySize::Unknown,// Stgi
	MemorySize::Unknown,// Clgi
	MemorySize::Unknown,// Skinit
	MemorySize::Unknown,// Invlpgaw
	MemorySize::Unknown,// Invlpgad
	MemorySize::Unknown,// Invlpgaq
	MemorySize::Unknown,// Setssbsy
	MemorySize::Unknown,// Saveprevssp
	MemorySize::Unknown,// Rdpkru
	MemorySize::Unknown,// Wrpkru
	MemorySize::Unknown,// Swapgs
	MemorySize::Unknown,// Rdtscp
	MemorySize::Unknown,// Monitorxw
	MemorySize::Unknown,// Monitorxd
	MemorySize::Unknown,// Monitorxq
	MemorySize::Unknown,// Mcommit
	MemorySize::Unknown,// Mwaitx
	MemorySize::Unknown,// Clzerow
	MemorySize::Unknown,// Clzerod
	MemorySize::Unknown,// Clzeroq
	MemorySize::Unknown,// Rdpru
	MemorySize::UInt16,// Lar_r16_rm16
	MemorySize::UInt16,// Lar_r32_r32m16
	MemorySize::UInt16,// Lar_r64_r64m16
	MemorySize::UInt16,// Lsl_r16_rm16
	MemorySize::UInt16,// Lsl_r32_r32m16
	MemorySize::UInt16,// Lsl_r64_r64m16
	MemorySize::Unknown,// Storeall
	MemorySize::Unknown,// Loadall286
	MemorySize::Unknown,// Syscall
	MemorySize::Unknown,// Clts
	MemorySize::Unknown,// Loadall386
	MemorySize::Unknown,// Sysretd
	MemorySize::Unknown,// Sysretq
	MemorySize::Unknown,// Invd
	MemorySize::Unknown,// Wbinvd
	MemorySize::Unknown,// Wbnoinvd
	MemorySize::Unknown,// Cl1invmb
	MemorySize::Unknown,// Ud2
	MemorySize::UInt16,// Reservednop_rm16_r16_0F0D
	MemorySize::UInt32,// Reservednop_rm32_r32_0F0D
	MemorySize::UInt64,// Reservednop_rm64_r64_0F0D
	MemorySize::UInt8,// Prefetch_m8
	MemorySize::UInt8,// Prefetchw_m8
	MemorySize::UInt8,// Prefetchwt1_m8
	MemorySize::Unknown,// Femms
	MemorySize::UInt8,// Umov_rm8_r8
	MemorySize::UInt16,// Umov_rm16_r16
	MemorySize::UInt32,// Umov_rm32_r32
	MemorySize::UInt8,// Umov_r8_rm8
	MemorySize::UInt16,// Umov_r16_rm16
	MemorySize::UInt32,// Umov_r32_rm32
	MemorySize::Packed128_Float32,// Movups_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmovups_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmovups_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmovups_xmm_k1z_xmmm128
	MemorySize::Packed256_Float32,// EVEX_Vmovups_ymm_k1z_ymmm256
	MemorySize::Packed512_Float32,// EVEX_Vmovups_zmm_k1z_zmmm512
	MemorySize::Packed128_Float64,// Movupd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vmovupd_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vmovupd_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vmovupd_xmm_k1z_xmmm128
	MemorySize::Packed256_Float64,// EVEX_Vmovupd_ymm_k1z_ymmm256
	MemorySize::Packed512_Float64,// EVEX_Vmovupd_zmm_k1z_zmmm512
	MemorySize::Float32,// Movss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vmovss_xmm_xmm_xmm
	MemorySize::Float32,// VEX_Vmovss_xmm_m32
	MemorySize::Unknown,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	MemorySize::Float32,// EVEX_Vmovss_xmm_k1z_m32
	MemorySize::Float64,// Movsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmovsd_xmm_xmm_xmm
	MemorySize::Float64,// VEX_Vmovsd_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	MemorySize::Float64,// EVEX_Vmovsd_xmm_k1z_m64
	MemorySize::Packed128_Float32,// Movups_xmmm128_xmm
	MemorySize::Packed128_Float32,// VEX_Vmovups_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vmovups_ymmm256_ymm
	MemorySize::Packed128_Float32,// EVEX_Vmovups_xmmm128_k1z_xmm
	MemorySize::Packed256_Float32,// EVEX_Vmovups_ymmm256_k1z_ymm
	MemorySize::Packed512_Float32,// EVEX_Vmovups_zmmm512_k1z_zmm
	MemorySize::Packed128_Float64,// Movupd_xmmm128_xmm
	MemorySize::Packed128_Float64,// VEX_Vmovupd_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vmovupd_ymmm256_ymm
	MemorySize::Packed128_Float64,// EVEX_Vmovupd_xmmm128_k1z_xmm
	MemorySize::Packed256_Float64,// EVEX_Vmovupd_ymmm256_k1z_ymm
	MemorySize::Packed512_Float64,// EVEX_Vmovupd_zmmm512_k1z_zmm
	MemorySize::Float32,// Movss_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vmovss_xmm_xmm_xmm_0F11
	MemorySize::Float32,// VEX_Vmovss_m32_xmm
	MemorySize::Unknown,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	MemorySize::Float32,// EVEX_Vmovss_m32_k1_xmm
	MemorySize::Float64,// Movsd_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	MemorySize::Float64,// VEX_Vmovsd_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	MemorySize::Float64,// EVEX_Vmovsd_m64_k1_xmm
	MemorySize::Unknown,// Movhlps_xmm_xmm
	MemorySize::Packed64_Float32,// Movlps_xmm_m64
	MemorySize::Unknown,// VEX_Vmovhlps_xmm_xmm_xmm
	MemorySize::Packed64_Float32,// VEX_Vmovlps_xmm_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovhlps_xmm_xmm_xmm
	MemorySize::Packed64_Float32,// EVEX_Vmovlps_xmm_xmm_m64
	MemorySize::Float64,// Movlpd_xmm_m64
	MemorySize::Float64,// VEX_Vmovlpd_xmm_xmm_m64
	MemorySize::Float64,// EVEX_Vmovlpd_xmm_xmm_m64
	MemorySize::Packed128_Float32,// Movsldup_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmovsldup_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmovsldup_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	MemorySize::Packed256_Float32,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	MemorySize::Packed512_Float32,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	MemorySize::Float64,// Movddup_xmm_xmmm64
	MemorySize::Float64,// VEX_Vmovddup_xmm_xmmm64
	MemorySize::Packed256_Float64,// VEX_Vmovddup_ymm_ymmm256
	MemorySize::Float64,// EVEX_Vmovddup_xmm_k1z_xmmm64
	MemorySize::Packed256_Float64,// EVEX_Vmovddup_ymm_k1z_ymmm256
	MemorySize::Packed512_Float64,// EVEX_Vmovddup_zmm_k1z_zmmm512
	MemorySize::Packed64_Float32,// Movlps_m64_xmm
	MemorySize::Packed64_Float32,// VEX_Vmovlps_m64_xmm
	MemorySize::Packed64_Float32,// EVEX_Vmovlps_m64_xmm
	MemorySize::Float64,// Movlpd_m64_xmm
	MemorySize::Float64,// VEX_Vmovlpd_m64_xmm
	MemorySize::Float64,// EVEX_Vmovlpd_m64_xmm
	MemorySize::Packed128_Float32,// Unpcklps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vunpcklps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vunpcklps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Unpcklpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// Unpckhps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vunpckhps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vunpckhps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Unpckhpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Movlhps_xmm_xmm
	MemorySize::Unknown,// VEX_Vmovlhps_xmm_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovlhps_xmm_xmm_xmm
	MemorySize::Packed64_Float32,// Movhps_xmm_m64
	MemorySize::Packed64_Float32,// VEX_Vmovhps_xmm_xmm_m64
	MemorySize::Packed64_Float32,// EVEX_Vmovhps_xmm_xmm_m64
	MemorySize::Float64,// Movhpd_xmm_m64
	MemorySize::Float64,// VEX_Vmovhpd_xmm_xmm_m64
	MemorySize::Float64,// EVEX_Vmovhpd_xmm_xmm_m64
	MemorySize::Packed128_Float32,// Movshdup_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmovshdup_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmovshdup_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	MemorySize::Packed256_Float32,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	MemorySize::Packed512_Float32,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	MemorySize::Packed64_Float32,// Movhps_m64_xmm
	MemorySize::Packed64_Float32,// VEX_Vmovhps_m64_xmm
	MemorySize::Packed64_Float32,// EVEX_Vmovhps_m64_xmm
	MemorySize::Float64,// Movhpd_m64_xmm
	MemorySize::Float64,// VEX_Vmovhpd_m64_xmm
	MemorySize::Float64,// EVEX_Vmovhpd_m64_xmm
	MemorySize::UInt16,// Reservednop_rm16_r16_0F18
	MemorySize::UInt32,// Reservednop_rm32_r32_0F18
	MemorySize::UInt64,// Reservednop_rm64_r64_0F18
	MemorySize::UInt16,// Reservednop_rm16_r16_0F19
	MemorySize::UInt32,// Reservednop_rm32_r32_0F19
	MemorySize::UInt64,// Reservednop_rm64_r64_0F19
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1A
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1A
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1A
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1B
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1B
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1B
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1C
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1C
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1C
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1D
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1D
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1D
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1E
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1E
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1E
	MemorySize::UInt16,// Reservednop_rm16_r16_0F1F
	MemorySize::UInt32,// Reservednop_rm32_r32_0F1F
	MemorySize::UInt64,// Reservednop_rm64_r64_0F1F
	MemorySize::UInt8,// Prefetchnta_m8
	MemorySize::UInt8,// Prefetcht0_m8
	MemorySize::UInt8,// Prefetcht1_m8
	MemorySize::UInt8,// Prefetcht2_m8
	MemorySize::Unknown,// Bndldx_bnd_mib
	MemorySize::Bnd32,// Bndmov_bnd_bndm64
	MemorySize::Bnd64,// Bndmov_bnd_bndm128
	MemorySize::UInt32,// Bndcl_bnd_rm32
	MemorySize::UInt64,// Bndcl_bnd_rm64
	MemorySize::UInt32,// Bndcu_bnd_rm32
	MemorySize::UInt64,// Bndcu_bnd_rm64
	MemorySize::Unknown,// Bndstx_mib_bnd
	MemorySize::Bnd32,// Bndmov_bndm64_bnd
	MemorySize::Bnd64,// Bndmov_bndm128_bnd
	MemorySize::UInt32,// Bndmk_bnd_m32
	MemorySize::UInt64,// Bndmk_bnd_m64
	MemorySize::UInt32,// Bndcn_bnd_rm32
	MemorySize::UInt64,// Bndcn_bnd_rm64
	MemorySize::UInt8,// Cldemote_m8
	MemorySize::Unknown,// Rdsspd_r32
	MemorySize::Unknown,// Rdsspq_r64
	MemorySize::Unknown,// Endbr64
	MemorySize::Unknown,// Endbr32
	MemorySize::UInt16,// Nop_rm16
	MemorySize::UInt32,// Nop_rm32
	MemorySize::UInt64,// Nop_rm64
	MemorySize::Unknown,// Mov_r32_cr
	MemorySize::Unknown,// Mov_r64_cr
	MemorySize::Unknown,// Mov_r32_dr
	MemorySize::Unknown,// Mov_r64_dr
	MemorySize::Unknown,// Mov_cr_r32
	MemorySize::Unknown,// Mov_cr_r64
	MemorySize::Unknown,// Mov_dr_r32
	MemorySize::Unknown,// Mov_dr_r64
	MemorySize::Unknown,// Mov_r32_tr
	MemorySize::Unknown,// Mov_tr_r32
	MemorySize::Packed128_Float32,// Movaps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmovaps_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmovaps_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmovaps_xmm_k1z_xmmm128
	MemorySize::Packed256_Float32,// EVEX_Vmovaps_ymm_k1z_ymmm256
	MemorySize::Packed512_Float32,// EVEX_Vmovaps_zmm_k1z_zmmm512
	MemorySize::Packed128_Float64,// Movapd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vmovapd_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vmovapd_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vmovapd_xmm_k1z_xmmm128
	MemorySize::Packed256_Float64,// EVEX_Vmovapd_ymm_k1z_ymmm256
	MemorySize::Packed512_Float64,// EVEX_Vmovapd_zmm_k1z_zmmm512
	MemorySize::Packed128_Float32,// Movaps_xmmm128_xmm
	MemorySize::Packed128_Float32,// VEX_Vmovaps_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vmovaps_ymmm256_ymm
	MemorySize::Packed128_Float32,// EVEX_Vmovaps_xmmm128_k1z_xmm
	MemorySize::Packed256_Float32,// EVEX_Vmovaps_ymmm256_k1z_ymm
	MemorySize::Packed512_Float32,// EVEX_Vmovaps_zmmm512_k1z_zmm
	MemorySize::Packed128_Float64,// Movapd_xmmm128_xmm
	MemorySize::Packed128_Float64,// VEX_Vmovapd_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vmovapd_ymmm256_ymm
	MemorySize::Packed128_Float64,// EVEX_Vmovapd_xmmm128_k1z_xmm
	MemorySize::Packed256_Float64,// EVEX_Vmovapd_ymmm256_k1z_ymm
	MemorySize::Packed512_Float64,// EVEX_Vmovapd_zmmm512_k1z_zmm
	MemorySize::Packed64_Int32,// Cvtpi2ps_xmm_mmm64
	MemorySize::Packed64_Int32,// Cvtpi2pd_xmm_mmm64
	MemorySize::Int32,// Cvtsi2ss_xmm_rm32
	MemorySize::Int64,// Cvtsi2ss_xmm_rm64
	MemorySize::Int32,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	MemorySize::Int64,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	MemorySize::Int32,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	MemorySize::Int64,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	MemorySize::Int32,// Cvtsi2sd_xmm_rm32
	MemorySize::Int64,// Cvtsi2sd_xmm_rm64
	MemorySize::Int32,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	MemorySize::Int64,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	MemorySize::Int32,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	MemorySize::Int64,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	MemorySize::Packed128_Float32,// Movntps_m128_xmm
	MemorySize::Packed128_Float32,// VEX_Vmovntps_m128_xmm
	MemorySize::Packed256_Float32,// VEX_Vmovntps_m256_ymm
	MemorySize::Packed128_Float32,// EVEX_Vmovntps_m128_xmm
	MemorySize::Packed256_Float32,// EVEX_Vmovntps_m256_ymm
	MemorySize::Packed512_Float32,// EVEX_Vmovntps_m512_zmm
	MemorySize::Packed128_Float64,// Movntpd_m128_xmm
	MemorySize::Packed128_Float64,// VEX_Vmovntpd_m128_xmm
	MemorySize::Packed256_Float64,// VEX_Vmovntpd_m256_ymm
	MemorySize::Packed128_Float64,// EVEX_Vmovntpd_m128_xmm
	MemorySize::Packed256_Float64,// EVEX_Vmovntpd_m256_ymm
	MemorySize::Packed512_Float64,// EVEX_Vmovntpd_m512_zmm
	MemorySize::Float32,// Movntss_m32_xmm
	MemorySize::Float64,// Movntsd_m64_xmm
	MemorySize::Packed64_Float32,// Cvttps2pi_mm_xmmm64
	MemorySize::Packed128_Float64,// Cvttpd2pi_mm_xmmm128
	MemorySize::Float32,// Cvttss2si_r32_xmmm32
	MemorySize::Float32,// Cvttss2si_r64_xmmm32
	MemorySize::Float32,// VEX_Vcvttss2si_r32_xmmm32
	MemorySize::Float32,// VEX_Vcvttss2si_r64_xmmm32
	MemorySize::Float32,// EVEX_Vcvttss2si_r32_xmmm32_sae
	MemorySize::Float32,// EVEX_Vcvttss2si_r64_xmmm32_sae
	MemorySize::Float64,// Cvttsd2si_r32_xmmm64
	MemorySize::Float64,// Cvttsd2si_r64_xmmm64
	MemorySize::Float64,// VEX_Vcvttsd2si_r32_xmmm64
	MemorySize::Float64,// VEX_Vcvttsd2si_r64_xmmm64
	MemorySize::Float64,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	MemorySize::Float64,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	MemorySize::Packed64_Float32,// Cvtps2pi_mm_xmmm64
	MemorySize::Packed128_Float64,// Cvtpd2pi_mm_xmmm128
	MemorySize::Float32,// Cvtss2si_r32_xmmm32
	MemorySize::Float32,// Cvtss2si_r64_xmmm32
	MemorySize::Float32,// VEX_Vcvtss2si_r32_xmmm32
	MemorySize::Float32,// VEX_Vcvtss2si_r64_xmmm32
	MemorySize::Float32,// EVEX_Vcvtss2si_r32_xmmm32_er
	MemorySize::Float32,// EVEX_Vcvtss2si_r64_xmmm32_er
	MemorySize::Float64,// Cvtsd2si_r32_xmmm64
	MemorySize::Float64,// Cvtsd2si_r64_xmmm64
	MemorySize::Float64,// VEX_Vcvtsd2si_r32_xmmm64
	MemorySize::Float64,// VEX_Vcvtsd2si_r64_xmmm64
	MemorySize::Float64,// EVEX_Vcvtsd2si_r32_xmmm64_er
	MemorySize::Float64,// EVEX_Vcvtsd2si_r64_xmmm64_er
	MemorySize::Float32,// Ucomiss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vucomiss_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vucomiss_xmm_xmmm32_sae
	MemorySize::Float64,// Ucomisd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vucomisd_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vucomisd_xmm_xmmm64_sae
	MemorySize::Float32,// Comiss_xmm_xmmm32
	MemorySize::Float64,// Comisd_xmm_xmmm64
	MemorySize::Float32,// VEX_Vcomiss_xmm_xmmm32
	MemorySize::Float64,// VEX_Vcomisd_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vcomiss_xmm_xmmm32_sae
	MemorySize::Float64,// EVEX_Vcomisd_xmm_xmmm64_sae
	MemorySize::Unknown,// Wrmsr
	MemorySize::Unknown,// Rdtsc
	MemorySize::Unknown,// Rdmsr
	MemorySize::Unknown,// Rdpmc
	MemorySize::Unknown,// Sysenter
	MemorySize::Unknown,// Sysexitd
	MemorySize::Unknown,// Sysexitq
	MemorySize::Unknown,// Getsecd
	MemorySize::UInt16,// Cmovo_r16_rm16
	MemorySize::UInt32,// Cmovo_r32_rm32
	MemorySize::UInt64,// Cmovo_r64_rm64
	MemorySize::UInt16,// Cmovno_r16_rm16
	MemorySize::UInt32,// Cmovno_r32_rm32
	MemorySize::UInt64,// Cmovno_r64_rm64
	MemorySize::UInt16,// Cmovb_r16_rm16
	MemorySize::UInt32,// Cmovb_r32_rm32
	MemorySize::UInt64,// Cmovb_r64_rm64
	MemorySize::UInt16,// Cmovae_r16_rm16
	MemorySize::UInt32,// Cmovae_r32_rm32
	MemorySize::UInt64,// Cmovae_r64_rm64
	MemorySize::UInt16,// Cmove_r16_rm16
	MemorySize::UInt32,// Cmove_r32_rm32
	MemorySize::UInt64,// Cmove_r64_rm64
	MemorySize::UInt16,// Cmovne_r16_rm16
	MemorySize::UInt32,// Cmovne_r32_rm32
	MemorySize::UInt64,// Cmovne_r64_rm64
	MemorySize::UInt16,// Cmovbe_r16_rm16
	MemorySize::UInt32,// Cmovbe_r32_rm32
	MemorySize::UInt64,// Cmovbe_r64_rm64
	MemorySize::UInt16,// Cmova_r16_rm16
	MemorySize::UInt32,// Cmova_r32_rm32
	MemorySize::UInt64,// Cmova_r64_rm64
	MemorySize::UInt16,// Cmovs_r16_rm16
	MemorySize::UInt32,// Cmovs_r32_rm32
	MemorySize::UInt64,// Cmovs_r64_rm64
	MemorySize::UInt16,// Cmovns_r16_rm16
	MemorySize::UInt32,// Cmovns_r32_rm32
	MemorySize::UInt64,// Cmovns_r64_rm64
	MemorySize::UInt16,// Cmovp_r16_rm16
	MemorySize::UInt32,// Cmovp_r32_rm32
	MemorySize::UInt64,// Cmovp_r64_rm64
	MemorySize::UInt16,// Cmovnp_r16_rm16
	MemorySize::UInt32,// Cmovnp_r32_rm32
	MemorySize::UInt64,// Cmovnp_r64_rm64
	MemorySize::UInt16,// Cmovl_r16_rm16
	MemorySize::UInt32,// Cmovl_r32_rm32
	MemorySize::UInt64,// Cmovl_r64_rm64
	MemorySize::UInt16,// Cmovge_r16_rm16
	MemorySize::UInt32,// Cmovge_r32_rm32
	MemorySize::UInt64,// Cmovge_r64_rm64
	MemorySize::UInt16,// Cmovle_r16_rm16
	MemorySize::UInt32,// Cmovle_r32_rm32
	MemorySize::UInt64,// Cmovle_r64_rm64
	MemorySize::UInt16,// Cmovg_r16_rm16
	MemorySize::UInt32,// Cmovg_r32_rm32
	MemorySize::UInt64,// Cmovg_r64_rm64
	MemorySize::Unknown,// VEX_Kandw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnd_kr_kr_kr
	MemorySize::Unknown,// VEX_Knotw_kr_kr
	MemorySize::Unknown,// VEX_Knotq_kr_kr
	MemorySize::Unknown,// VEX_Knotb_kr_kr
	MemorySize::Unknown,// VEX_Knotd_kr_kr
	MemorySize::Unknown,// VEX_Korw_kr_kr_kr
	MemorySize::Unknown,// VEX_Korq_kr_kr_kr
	MemorySize::Unknown,// VEX_Korb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckwd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckdq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckbw_kr_kr_kr
	MemorySize::Unknown,// Movmskps_r32_xmm
	MemorySize::Unknown,// Movmskps_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r32_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r32_ymm
	MemorySize::Unknown,// VEX_Vmovmskps_r64_ymm
	MemorySize::Unknown,// Movmskpd_r32_xmm
	MemorySize::Unknown,// Movmskpd_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r32_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r32_ymm
	MemorySize::Unknown,// VEX_Vmovmskpd_r64_ymm
	MemorySize::Packed128_Float32,// Sqrtps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vsqrtps_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vsqrtps_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	MemorySize::Packed128_Float64,// Sqrtpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vsqrtpd_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vsqrtpd_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	MemorySize::Float32,// Sqrtss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vsqrtss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// Sqrtsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// Rsqrtps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vrsqrtps_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vrsqrtps_ymm_ymmm256
	MemorySize::Float32,// Rsqrtss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	MemorySize::Packed128_Float32,// Rcpps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vrcpps_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vrcpps_ymm_ymmm256
	MemorySize::Float32,// Rcpss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vrcpss_xmm_xmm_xmmm32
	MemorySize::Packed128_Float32,// Andps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vandps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vandps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Andpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vandpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vandpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// Andnps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vandnps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vandnps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Andnpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vandnpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vandnpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// Orps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vorps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vorps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Orpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vorpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vorpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// Xorps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vxorps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vxorps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// Xorpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vxorpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vxorpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// Addps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vaddps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vaddps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// Addpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vaddpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vaddpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// Addss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vaddss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// Addsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vaddsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// Mulps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmulps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmulps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// Mulpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vmulpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vmulpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// Mulss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vmulss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// Mulsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vmulsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed64_Float32,// Cvtps2pd_xmm_xmmm64
	MemorySize::Packed64_Float32,// VEX_Vcvtps2pd_xmm_xmmm64
	MemorySize::Packed128_Float32,// VEX_Vcvtps2pd_ymm_xmmm128
	MemorySize::Packed64_Float32,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	MemorySize::Packed128_Float64,// Cvtpd2ps_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vcvtpd2ps_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vcvtpd2ps_xmm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Float32,// Cvtss2sd_xmm_xmmm32
	MemorySize::Float32,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// Cvtsd2ss_xmm_xmmm64
	MemorySize::Float64,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Int32,// Cvtdq2ps_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vcvtdq2ps_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vcvtdq2ps_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	MemorySize::Packed128_Int64,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Packed128_Float32,// Cvtps2dq_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vcvtps2dq_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vcvtps2dq_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	MemorySize::Packed128_Float32,// Cvttps2dq_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vcvttps2dq_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vcvttps2dq_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed128_Float32,// Subps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vsubps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vsubps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// Subpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vsubpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vsubpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// Subss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vsubss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// Subsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vsubsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// Minps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vminps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vminps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	MemorySize::Packed128_Float64,// Minpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vminpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vminpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	MemorySize::Float32,// Minss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vminss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// Minsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vminsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Packed128_Float32,// Divps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vdivps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vdivps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// Divpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vdivpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vdivpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// Divss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vdivss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// Divsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vdivsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// Maxps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vmaxps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vmaxps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	MemorySize::Packed128_Float64,// Maxpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vmaxpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vmaxpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	MemorySize::Float32,// Maxss_xmm_xmmm32
	MemorySize::Float32,// VEX_Vmaxss_xmm_xmm_xmmm32
	MemorySize::Float32,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// Maxsd_xmm_xmmm64
	MemorySize::Float64,// VEX_Vmaxsd_xmm_xmm_xmmm64
	MemorySize::Float64,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Packed32_UInt8,// Punpcklbw_mm_mmm32
	MemorySize::Packed128_UInt8,// Punpcklbw_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed32_UInt16,// Punpcklwd_mm_mmm32
	MemorySize::Packed128_UInt16,// Punpcklwd_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	MemorySize::Int32,// Punpckldq_mm_mmm32
	MemorySize::Packed128_UInt32,// Punpckldq_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed64_Int16,// Packsswb_mm_mmm64
	MemorySize::Packed128_Int16,// Packsswb_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpacksswb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpacksswb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int8,// Pcmpgtb_mm_mmm64
	MemorySize::Packed128_Int8,// Pcmpgtb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	MemorySize::Packed64_Int16,// Pcmpgtw_mm_mmm64
	MemorySize::Packed128_Int16,// Pcmpgtw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	MemorySize::Packed64_Int32,// Pcmpgtd_mm_mmm64
	MemorySize::Packed128_Int32,// Pcmpgtd_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	MemorySize::Packed64_Int16,// Packuswb_mm_mmm64
	MemorySize::Packed128_Int16,// Packuswb_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpackuswb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpackuswb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt8,// Punpckhbw_mm_mmm64
	MemorySize::Packed128_UInt8,// Punpckhbw_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Punpckhwd_mm_mmm64
	MemorySize::Packed128_UInt16,// Punpckhwd_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt32,// Punpckhdq_mm_mmm64
	MemorySize::Packed128_UInt32,// Punpckhdq_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed64_Int32,// Packssdw_mm_mmm64
	MemorySize::Packed128_Int32,// Packssdw_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpackssdw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpackssdw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// Punpcklqdq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt64,// Punpckhqdq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	MemorySize::UInt32,// Movd_mm_rm32
	MemorySize::UInt64,// Movq_mm_rm64
	MemorySize::UInt32,// Movd_xmm_rm32
	MemorySize::UInt64,// Movq_xmm_rm64
	MemorySize::UInt32,// VEX_Vmovd_xmm_rm32
	MemorySize::UInt64,// VEX_Vmovq_xmm_rm64
	MemorySize::UInt32,// EVEX_Vmovd_xmm_rm32
	MemorySize::UInt64,// EVEX_Vmovq_xmm_rm64
	MemorySize::UInt64,// Movq_mm_mmm64
	MemorySize::Packed128_UInt32,// Movdqa_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vmovdqa_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vmovdqa_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt32,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt32,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt64,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt64,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt64,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt32,// Movdqu_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vmovdqu_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vmovdqu_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt32,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt32,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt64,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt64,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt64,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt8,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	MemorySize::Packed64_UInt16,// Pshufw_mm_mmm64_imm8
	MemorySize::Packed128_UInt32,// Pshufd_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt32,// VEX_Vpshufd_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt32,// VEX_Vpshufd_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	MemorySize::Packed128_UInt16,// Pshufhw_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt16,// VEX_Vpshufhw_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// VEX_Vpshufhw_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	MemorySize::Packed128_UInt16,// Pshuflw_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt16,// VEX_Vpshuflw_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// VEX_Vpshuflw_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psrlw_mm_imm8
	MemorySize::Unknown,// Psrlw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlw_ymm_ymm_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psraw_mm_imm8
	MemorySize::Unknown,// Psraw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsraw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsraw_ymm_ymm_imm8
	MemorySize::Packed128_Int16,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	MemorySize::Packed256_Int16,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	MemorySize::Packed512_Int16,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psllw_mm_imm8
	MemorySize::Unknown,// Psllw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllw_ymm_ymm_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Psrld_mm_imm8
	MemorySize::Unknown,// Psrld_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrld_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrld_ymm_ymm_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// Psrad_mm_imm8
	MemorySize::Unknown,// Psrad_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrad_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrad_ymm_ymm_imm8
	MemorySize::Packed128_Int32,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_Int32,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_Int32,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	MemorySize::Packed128_Int64,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_Int64,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Int64,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Pslld_mm_imm8
	MemorySize::Unknown,// Pslld_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslld_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslld_ymm_ymm_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// Psrlq_mm_imm8
	MemorySize::Unknown,// Psrlq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlq_ymm_ymm_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Psrldq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrldq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrldq_ymm_ymm_imm8
	MemorySize::UInt128,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt128,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt128,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	MemorySize::Unknown,// Psllq_mm_imm8
	MemorySize::Unknown,// Psllq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllq_ymm_ymm_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Pslldq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslldq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslldq_ymm_ymm_imm8
	MemorySize::UInt128,// EVEX_Vpslldq_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt128,// EVEX_Vpslldq_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt128,// EVEX_Vpslldq_zmm_zmmm512_imm8
	MemorySize::Packed64_UInt8,// Pcmpeqb_mm_mmm64
	MemorySize::Packed128_UInt8,// Pcmpeqb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Pcmpeqw_mm_mmm64
	MemorySize::Packed128_UInt16,// Pcmpeqw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	MemorySize::Packed64_UInt32,// Pcmpeqd_mm_mmm64
	MemorySize::Packed128_UInt32,// Pcmpeqd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	MemorySize::Unknown,// Emms
	MemorySize::Unknown,// VEX_Vzeroupper
	MemorySize::Unknown,// VEX_Vzeroall
	MemorySize::UInt32,// Vmread_rm32_r32
	MemorySize::UInt64,// Vmread_rm64_r64
	MemorySize::Packed128_Float32,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed128_Float64,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Extrq_xmm_imm8_imm8
	MemorySize::Packed64_Float32,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Float32,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	MemorySize::Packed128_Float64,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	MemorySize::Float32,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	MemorySize::Float32,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	MemorySize::Unknown,// Insertq_xmm_xmm_imm8_imm8
	MemorySize::Float64,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	MemorySize::Float64,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	MemorySize::UInt32,// Vmwrite_r32_rm32
	MemorySize::UInt64,// Vmwrite_r64_rm64
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Extrq_xmm_xmm
	MemorySize::Packed64_Float32,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	MemorySize::Float32,// EVEX_Vcvtss2usi_r32_xmmm32_er
	MemorySize::Float32,// EVEX_Vcvtss2usi_r64_xmmm32_er
	MemorySize::Unknown,// Insertq_xmm_xmm
	MemorySize::Float64,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	MemorySize::Float64,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	MemorySize::Packed64_Float32,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Float32,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	MemorySize::Packed128_Float64,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	MemorySize::Packed64_UInt32,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	MemorySize::Packed128_UInt32,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	MemorySize::Packed128_UInt64,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	MemorySize::Packed128_UInt32,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	MemorySize::Packed128_UInt64,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Packed64_Float32,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	MemorySize::UInt32,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	MemorySize::UInt64,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	MemorySize::UInt32,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	MemorySize::UInt64,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	MemorySize::Packed128_Float64,// Haddpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vhaddpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vhaddpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// Haddps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vhaddps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vhaddps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// Hsubpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vhsubpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vhsubpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// Hsubps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vhsubps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vhsubps_ymm_ymm_ymmm256
	MemorySize::UInt32,// Movd_rm32_mm
	MemorySize::UInt64,// Movq_rm64_mm
	MemorySize::UInt32,// Movd_rm32_xmm
	MemorySize::UInt64,// Movq_rm64_xmm
	MemorySize::UInt32,// VEX_Vmovd_rm32_xmm
	MemorySize::UInt64,// VEX_Vmovq_rm64_xmm
	MemorySize::UInt32,// EVEX_Vmovd_rm32_xmm
	MemorySize::UInt64,// EVEX_Vmovq_rm64_xmm
	MemorySize::UInt64,// Movq_xmm_xmmm64
	MemorySize::UInt64,// VEX_Vmovq_xmm_xmmm64
	MemorySize::UInt64,// EVEX_Vmovq_xmm_xmmm64
	MemorySize::UInt64,// Movq_mmm64_mm
	MemorySize::Packed128_UInt32,// Movdqa_xmmm128_xmm
	MemorySize::Packed128_UInt32,// VEX_Vmovdqa_xmmm128_xmm
	MemorySize::Packed256_UInt32,// VEX_Vmovdqa_ymmm256_ymm
	MemorySize::Packed128_UInt32,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt32,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt32,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt64,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt64,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt64,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt32,// Movdqu_xmmm128_xmm
	MemorySize::Packed128_UInt32,// VEX_Vmovdqu_xmmm128_xmm
	MemorySize::Packed256_UInt32,// VEX_Vmovdqu_ymmm256_ymm
	MemorySize::Packed128_UInt32,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt32,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt32,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt64,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt64,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt64,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt8,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt8,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt8,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt16,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt16,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt16,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	MemorySize::Unknown,// Jo_rel16
	MemorySize::Unknown,// Jo_rel32_32
	MemorySize::Unknown,// Jo_rel32_64
	MemorySize::Unknown,// Jno_rel16
	MemorySize::Unknown,// Jno_rel32_32
	MemorySize::Unknown,// Jno_rel32_64
	MemorySize::Unknown,// Jb_rel16
	MemorySize::Unknown,// Jb_rel32_32
	MemorySize::Unknown,// Jb_rel32_64
	MemorySize::Unknown,// Jae_rel16
	MemorySize::Unknown,// Jae_rel32_32
	MemorySize::Unknown,// Jae_rel32_64
	MemorySize::Unknown,// Je_rel16
	MemorySize::Unknown,// Je_rel32_32
	MemorySize::Unknown,// Je_rel32_64
	MemorySize::Unknown,// Jne_rel16
	MemorySize::Unknown,// Jne_rel32_32
	MemorySize::Unknown,// Jne_rel32_64
	MemorySize::Unknown,// Jbe_rel16
	MemorySize::Unknown,// Jbe_rel32_32
	MemorySize::Unknown,// Jbe_rel32_64
	MemorySize::Unknown,// Ja_rel16
	MemorySize::Unknown,// Ja_rel32_32
	MemorySize::Unknown,// Ja_rel32_64
	MemorySize::Unknown,// Js_rel16
	MemorySize::Unknown,// Js_rel32_32
	MemorySize::Unknown,// Js_rel32_64
	MemorySize::Unknown,// Jns_rel16
	MemorySize::Unknown,// Jns_rel32_32
	MemorySize::Unknown,// Jns_rel32_64
	MemorySize::Unknown,// Jp_rel16
	MemorySize::Unknown,// Jp_rel32_32
	MemorySize::Unknown,// Jp_rel32_64
	MemorySize::Unknown,// Jnp_rel16
	MemorySize::Unknown,// Jnp_rel32_32
	MemorySize::Unknown,// Jnp_rel32_64
	MemorySize::Unknown,// Jl_rel16
	MemorySize::Unknown,// Jl_rel32_32
	MemorySize::Unknown,// Jl_rel32_64
	MemorySize::Unknown,// Jge_rel16
	MemorySize::Unknown,// Jge_rel32_32
	MemorySize::Unknown,// Jge_rel32_64
	MemorySize::Unknown,// Jle_rel16
	MemorySize::Unknown,// Jle_rel32_32
	MemorySize::Unknown,// Jle_rel32_64
	MemorySize::Unknown,// Jg_rel16
	MemorySize::Unknown,// Jg_rel32_32
	MemorySize::Unknown,// Jg_rel32_64
	MemorySize::UInt8,// Seto_rm8
	MemorySize::UInt8,// Setno_rm8
	MemorySize::UInt8,// Setb_rm8
	MemorySize::UInt8,// Setae_rm8
	MemorySize::UInt8,// Sete_rm8
	MemorySize::UInt8,// Setne_rm8
	MemorySize::UInt8,// Setbe_rm8
	MemorySize::UInt8,// Seta_rm8
	MemorySize::UInt8,// Sets_rm8
	MemorySize::UInt8,// Setns_rm8
	MemorySize::UInt8,// Setp_rm8
	MemorySize::UInt8,// Setnp_rm8
	MemorySize::UInt8,// Setl_rm8
	MemorySize::UInt8,// Setge_rm8
	MemorySize::UInt8,// Setle_rm8
	MemorySize::UInt8,// Setg_rm8
	MemorySize::UInt16,// VEX_Kmovw_kr_km16
	MemorySize::UInt64,// VEX_Kmovq_kr_km64
	MemorySize::UInt8,// VEX_Kmovb_kr_km8
	MemorySize::UInt32,// VEX_Kmovd_kr_km32
	MemorySize::UInt16,// VEX_Kmovw_m16_kr
	MemorySize::UInt64,// VEX_Kmovq_m64_kr
	MemorySize::UInt8,// VEX_Kmovb_m8_kr
	MemorySize::UInt32,// VEX_Kmovd_m32_kr
	MemorySize::Unknown,// VEX_Kmovw_kr_r32
	MemorySize::Unknown,// VEX_Kmovb_kr_r32
	MemorySize::Unknown,// VEX_Kmovd_kr_r32
	MemorySize::Unknown,// VEX_Kmovq_kr_r64
	MemorySize::Unknown,// VEX_Kmovw_r32_kr
	MemorySize::Unknown,// VEX_Kmovb_r32_kr
	MemorySize::Unknown,// VEX_Kmovd_r32_kr
	MemorySize::Unknown,// VEX_Kmovq_r64_kr
	MemorySize::Unknown,// VEX_Kortestw_kr_kr
	MemorySize::Unknown,// VEX_Kortestq_kr_kr
	MemorySize::Unknown,// VEX_Kortestb_kr_kr
	MemorySize::Unknown,// VEX_Kortestd_kr_kr
	MemorySize::Unknown,// VEX_Ktestw_kr_kr
	MemorySize::Unknown,// VEX_Ktestq_kr_kr
	MemorySize::Unknown,// VEX_Ktestb_kr_kr
	MemorySize::Unknown,// VEX_Ktestd_kr_kr
	MemorySize::Unknown,// Pushw_FS
	MemorySize::Unknown,// Pushd_FS
	MemorySize::Unknown,// Pushq_FS
	MemorySize::Unknown,// Popw_FS
	MemorySize::Unknown,// Popd_FS
	MemorySize::Unknown,// Popq_FS
	MemorySize::Unknown,// Cpuid
	MemorySize::UInt16,// Bt_rm16_r16
	MemorySize::UInt32,// Bt_rm32_r32
	MemorySize::UInt64,// Bt_rm64_r64
	MemorySize::UInt16,// Shld_rm16_r16_imm8
	MemorySize::UInt32,// Shld_rm32_r32_imm8
	MemorySize::UInt64,// Shld_rm64_r64_imm8
	MemorySize::UInt16,// Shld_rm16_r16_CL
	MemorySize::UInt32,// Shld_rm32_r32_CL
	MemorySize::UInt64,// Shld_rm64_r64_CL
	MemorySize::Unknown,// Montmul_16
	MemorySize::Unknown,// Montmul_32
	MemorySize::Unknown,// Montmul_64
	MemorySize::Unknown,// Xsha1_16
	MemorySize::Unknown,// Xsha1_32
	MemorySize::Unknown,// Xsha1_64
	MemorySize::Unknown,// Xsha256_16
	MemorySize::Unknown,// Xsha256_32
	MemorySize::Unknown,// Xsha256_64
	MemorySize::UInt16,// Xbts_r16_rm16
	MemorySize::UInt32,// Xbts_r32_rm32
	MemorySize::Unknown,// Xstore_16
	MemorySize::Unknown,// Xstore_32
	MemorySize::Unknown,// Xstore_64
	MemorySize::Unknown,// Xcryptecb_16
	MemorySize::Unknown,// Xcryptecb_32
	MemorySize::Unknown,// Xcryptecb_64
	MemorySize::Unknown,// Xcryptcbc_16
	MemorySize::Unknown,// Xcryptcbc_32
	MemorySize::Unknown,// Xcryptcbc_64
	MemorySize::Unknown,// Xcryptctr_16
	MemorySize::Unknown,// Xcryptctr_32
	MemorySize::Unknown,// Xcryptctr_64
	MemorySize::Unknown,// Xcryptcfb_16
	MemorySize::Unknown,// Xcryptcfb_32
	MemorySize::Unknown,// Xcryptcfb_64
	MemorySize::Unknown,// Xcryptofb_16
	MemorySize::Unknown,// Xcryptofb_32
	MemorySize::Unknown,// Xcryptofb_64
	MemorySize::UInt16,// Ibts_rm16_r16
	MemorySize::UInt32,// Ibts_rm32_r32
	MemorySize::UInt8,// Cmpxchg486_rm8_r8
	MemorySize::UInt16,// Cmpxchg486_rm16_r16
	MemorySize::UInt32,// Cmpxchg486_rm32_r32
	MemorySize::Unknown,// Pushw_GS
	MemorySize::Unknown,// Pushd_GS
	MemorySize::Unknown,// Pushq_GS
	MemorySize::Unknown,// Popw_GS
	MemorySize::Unknown,// Popd_GS
	MemorySize::Unknown,// Popq_GS
	MemorySize::Unknown,// Rsm
	MemorySize::UInt16,// Bts_rm16_r16
	MemorySize::UInt32,// Bts_rm32_r32
	MemorySize::UInt64,// Bts_rm64_r64
	MemorySize::UInt16,// Shrd_rm16_r16_imm8
	MemorySize::UInt32,// Shrd_rm32_r32_imm8
	MemorySize::UInt64,// Shrd_rm64_r64_imm8
	MemorySize::UInt16,// Shrd_rm16_r16_CL
	MemorySize::UInt32,// Shrd_rm32_r32_CL
	MemorySize::UInt64,// Shrd_rm64_r64_CL
	MemorySize::Fxsave_512Byte,// Fxsave_m512byte
	MemorySize::Fxsave64_512Byte,// Fxsave64_m512byte
	MemorySize::Unknown,// Rdfsbase_r32
	MemorySize::Unknown,// Rdfsbase_r64
	MemorySize::Fxsave_512Byte,// Fxrstor_m512byte
	MemorySize::Fxsave64_512Byte,// Fxrstor64_m512byte
	MemorySize::Unknown,// Rdgsbase_r32
	MemorySize::Unknown,// Rdgsbase_r64
	MemorySize::UInt32,// Ldmxcsr_m32
	MemorySize::Unknown,// Wrfsbase_r32
	MemorySize::Unknown,// Wrfsbase_r64
	MemorySize::UInt32,// VEX_Vldmxcsr_m32
	MemorySize::UInt32,// Stmxcsr_m32
	MemorySize::Unknown,// Wrgsbase_r32
	MemorySize::Unknown,// Wrgsbase_r64
	MemorySize::UInt32,// VEX_Vstmxcsr_m32
	MemorySize::Xsave,// Xsave_mem
	MemorySize::Xsave64,// Xsave64_mem
	MemorySize::UInt32,// Ptwrite_rm32
	MemorySize::UInt64,// Ptwrite_rm64
	MemorySize::Xsave,// Xrstor_mem
	MemorySize::Xsave64,// Xrstor64_mem
	MemorySize::Unknown,// Incsspd_r32
	MemorySize::Unknown,// Incsspq_r64
	MemorySize::Xsave,// Xsaveopt_mem
	MemorySize::Xsave64,// Xsaveopt64_mem
	MemorySize::UInt8,// Clwb_m8
	MemorySize::Unknown,// Tpause_r32
	MemorySize::Unknown,// Tpause_r64
	MemorySize::UInt64,// Clrssbsy_m64
	MemorySize::Unknown,// Umonitor_r16
	MemorySize::Unknown,// Umonitor_r32
	MemorySize::Unknown,// Umonitor_r64
	MemorySize::Unknown,// Umwait_r32
	MemorySize::Unknown,// Umwait_r64
	MemorySize::UInt8,// Clflush_m8
	MemorySize::UInt8,// Clflushopt_m8
	MemorySize::Unknown,// Lfence
	MemorySize::Unknown,// Lfence_E9
	MemorySize::Unknown,// Lfence_EA
	MemorySize::Unknown,// Lfence_EB
	MemorySize::Unknown,// Lfence_EC
	MemorySize::Unknown,// Lfence_ED
	MemorySize::Unknown,// Lfence_EE
	MemorySize::Unknown,// Lfence_EF
	MemorySize::Unknown,// Mfence
	MemorySize::Unknown,// Mfence_F1
	MemorySize::Unknown,// Mfence_F2
	MemorySize::Unknown,// Mfence_F3
	MemorySize::Unknown,// Mfence_F4
	MemorySize::Unknown,// Mfence_F5
	MemorySize::Unknown,// Mfence_F6
	MemorySize::Unknown,// Mfence_F7
	MemorySize::Unknown,// Sfence
	MemorySize::Unknown,// Sfence_F9
	MemorySize::Unknown,// Sfence_FA
	MemorySize::Unknown,// Sfence_FB
	MemorySize::Unknown,// Sfence_FC
	MemorySize::Unknown,// Sfence_FD
	MemorySize::Unknown,// Sfence_FE
	MemorySize::Unknown,// Sfence_FF
	MemorySize::Unknown,// Pcommit
	MemorySize::Int16,// Imul_r16_rm16
	MemorySize::Int32,// Imul_r32_rm32
	MemorySize::Int64,// Imul_r64_rm64
	MemorySize::UInt8,// Cmpxchg_rm8_r8
	MemorySize::UInt16,// Cmpxchg_rm16_r16
	MemorySize::UInt32,// Cmpxchg_rm32_r32
	MemorySize::UInt64,// Cmpxchg_rm64_r64
	MemorySize::SegPtr16,// Lss_r16_m1616
	MemorySize::SegPtr32,// Lss_r32_m1632
	MemorySize::SegPtr64,// Lss_r64_m1664
	MemorySize::UInt16,// Btr_rm16_r16
	MemorySize::UInt32,// Btr_rm32_r32
	MemorySize::UInt64,// Btr_rm64_r64
	MemorySize::SegPtr16,// Lfs_r16_m1616
	MemorySize::SegPtr32,// Lfs_r32_m1632
	MemorySize::SegPtr64,// Lfs_r64_m1664
	MemorySize::SegPtr16,// Lgs_r16_m1616
	MemorySize::SegPtr32,// Lgs_r32_m1632
	MemorySize::SegPtr64,// Lgs_r64_m1664
	MemorySize::UInt8,// Movzx_r16_rm8
	MemorySize::UInt8,// Movzx_r32_rm8
	MemorySize::UInt8,// Movzx_r64_rm8
	MemorySize::UInt16,// Movzx_r16_rm16
	MemorySize::UInt16,// Movzx_r32_rm16
	MemorySize::UInt16,// Movzx_r64_rm16
	MemorySize::Unknown,// Jmpe_disp16
	MemorySize::Unknown,// Jmpe_disp32
	MemorySize::UInt16,// Popcnt_r16_rm16
	MemorySize::UInt32,// Popcnt_r32_rm32
	MemorySize::UInt64,// Popcnt_r64_rm64
	MemorySize::UInt16,// Ud1_r16_rm16
	MemorySize::UInt32,// Ud1_r32_rm32
	MemorySize::UInt64,// Ud1_r64_rm64
	MemorySize::UInt16,// Bt_rm16_imm8
	MemorySize::UInt32,// Bt_rm32_imm8
	MemorySize::UInt64,// Bt_rm64_imm8
	MemorySize::UInt16,// Bts_rm16_imm8
	MemorySize::UInt32,// Bts_rm32_imm8
	MemorySize::UInt64,// Bts_rm64_imm8
	MemorySize::UInt16,// Btr_rm16_imm8
	MemorySize::UInt32,// Btr_rm32_imm8
	MemorySize::UInt64,// Btr_rm64_imm8
	MemorySize::UInt16,// Btc_rm16_imm8
	MemorySize::UInt32,// Btc_rm32_imm8
	MemorySize::UInt64,// Btc_rm64_imm8
	MemorySize::UInt16,// Btc_rm16_r16
	MemorySize::UInt32,// Btc_rm32_r32
	MemorySize::UInt64,// Btc_rm64_r64
	MemorySize::UInt16,// Bsf_r16_rm16
	MemorySize::UInt32,// Bsf_r32_rm32
	MemorySize::UInt64,// Bsf_r64_rm64
	MemorySize::UInt16,// Tzcnt_r16_rm16
	MemorySize::UInt32,// Tzcnt_r32_rm32
	MemorySize::UInt64,// Tzcnt_r64_rm64
	MemorySize::UInt16,// Bsr_r16_rm16
	MemorySize::UInt32,// Bsr_r32_rm32
	MemorySize::UInt64,// Bsr_r64_rm64
	MemorySize::UInt16,// Lzcnt_r16_rm16
	MemorySize::UInt32,// Lzcnt_r32_rm32
	MemorySize::UInt64,// Lzcnt_r64_rm64
	MemorySize::Int8,// Movsx_r16_rm8
	MemorySize::Int8,// Movsx_r32_rm8
	MemorySize::Int8,// Movsx_r64_rm8
	MemorySize::Int16,// Movsx_r16_rm16
	MemorySize::Int16,// Movsx_r32_rm16
	MemorySize::Int16,// Movsx_r64_rm16
	MemorySize::UInt8,// Xadd_rm8_r8
	MemorySize::UInt16,// Xadd_rm16_r16
	MemorySize::UInt32,// Xadd_rm32_r32
	MemorySize::UInt64,// Xadd_rm64_r64
	MemorySize::Packed128_Float32,// Cmpps_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// Cmppd_xmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float64,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	MemorySize::Float32,// Cmpss_xmm_xmmm32_imm8
	MemorySize::Float32,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	MemorySize::Float32,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// Cmpsd_xmm_xmmm64_imm8
	MemorySize::Float64,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	MemorySize::Float64,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	MemorySize::UInt32,// Movnti_m32_r32
	MemorySize::UInt64,// Movnti_m64_r64
	MemorySize::UInt16,// Pinsrw_mm_r32m16_imm8
	MemorySize::UInt16,// Pinsrw_mm_r64m16_imm8
	MemorySize::UInt16,// Pinsrw_xmm_r32m16_imm8
	MemorySize::UInt16,// Pinsrw_xmm_r64m16_imm8
	MemorySize::UInt16,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	MemorySize::UInt16,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	MemorySize::UInt16,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	MemorySize::UInt16,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	MemorySize::Unknown,// Pextrw_r32_mm_imm8
	MemorySize::Unknown,// Pextrw_r64_mm_imm8
	MemorySize::Unknown,// Pextrw_r32_xmm_imm8
	MemorySize::Unknown,// Pextrw_r64_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r32_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r64_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r32_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r64_xmm_imm8
	MemorySize::Packed128_Float32,// Shufps_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed128_Float64,// Shufpd_xmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float64,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::UInt64,// Cmpxchg8b_m64
	MemorySize::UInt128,// Cmpxchg16b_m128
	MemorySize::Xsave,// Xrstors_mem
	MemorySize::Xsave64,// Xrstors64_mem
	MemorySize::Xsave,// Xsavec_mem
	MemorySize::Xsave64,// Xsavec64_mem
	MemorySize::Xsave,// Xsaves_mem
	MemorySize::Xsave64,// Xsaves64_mem
	MemorySize::UInt64,// Vmptrld_m64
	MemorySize::UInt64,// Vmclear_m64
	MemorySize::UInt64,// Vmxon_m64
	MemorySize::Unknown,// Rdrand_r16
	MemorySize::Unknown,// Rdrand_r32
	MemorySize::Unknown,// Rdrand_r64
	MemorySize::UInt64,// Vmptrst_m64
	MemorySize::Unknown,// Rdseed_r16
	MemorySize::Unknown,// Rdseed_r32
	MemorySize::Unknown,// Rdseed_r64
	MemorySize::Unknown,// Rdpid_r32
	MemorySize::Unknown,// Rdpid_r64
	MemorySize::Unknown,// Bswap_r16
	MemorySize::Unknown,// Bswap_r32
	MemorySize::Unknown,// Bswap_r64
	MemorySize::Packed128_Float64,// Addsubpd_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// Addsubps_xmm_xmmm128
	MemorySize::Packed128_Float32,// VEX_Vaddsubps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vaddsubps_ymm_ymm_ymmm256
	MemorySize::Packed64_UInt16,// Psrlw_mm_mmm64
	MemorySize::Packed128_UInt16,// Psrlw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsrlw_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsrlw_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	MemorySize::Packed64_UInt32,// Psrld_mm_mmm64
	MemorySize::Packed128_UInt32,// Psrld_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpsrld_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpsrld_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	MemorySize::UInt64,// Psrlq_mm_mmm64
	MemorySize::Packed128_UInt64,// Psrlq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpsrlq_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpsrlq_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	MemorySize::UInt64,// Paddq_mm_mmm64
	MemorySize::Packed128_UInt64,// Paddq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpaddq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpaddq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_Int16,// Pmullw_mm_mmm64
	MemorySize::Packed128_Int16,// Pmullw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpmullw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpmullw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	MemorySize::UInt64,// Movq_xmmm64_xmm
	MemorySize::UInt64,// VEX_Vmovq_xmmm64_xmm
	MemorySize::UInt64,// EVEX_Vmovq_xmmm64_xmm
	MemorySize::Unknown,// Movq2dq_xmm_mm
	MemorySize::Unknown,// Movdq2q_mm_xmm
	MemorySize::Unknown,// Pmovmskb_r32_mm
	MemorySize::Unknown,// Pmovmskb_r64_mm
	MemorySize::Unknown,// Pmovmskb_r32_xmm
	MemorySize::Unknown,// Pmovmskb_r64_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r32_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r64_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r32_ymm
	MemorySize::Unknown,// VEX_Vpmovmskb_r64_ymm
	MemorySize::Packed64_UInt8,// Psubusb_mm_mmm64
	MemorySize::Packed128_UInt8,// Psubusb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpsubusb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpsubusb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Psubusw_mm_mmm64
	MemorySize::Packed128_UInt16,// Psubusw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsubusw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpsubusw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt8,// Pminub_mm_mmm64
	MemorySize::Packed128_UInt8,// Pminub_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpminub_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpminub_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	MemorySize::UInt64,// Pand_mm_mmm64
	MemorySize::UInt128,// Pand_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vpand_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vpand_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_UInt8,// Paddusb_mm_mmm64
	MemorySize::Packed128_UInt8,// Paddusb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpaddusb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpaddusb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Paddusw_mm_mmm64
	MemorySize::Packed128_UInt16,// Paddusw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpaddusw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpaddusw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt8,// Pmaxub_mm_mmm64
	MemorySize::Packed128_UInt8,// Pmaxub_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpmaxub_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpmaxub_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	MemorySize::UInt64,// Pandn_mm_mmm64
	MemorySize::UInt128,// Pandn_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vpandn_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vpandn_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_UInt8,// Pavgb_mm_mmm64
	MemorySize::Packed128_UInt8,// Pavgb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpavgb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpavgb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Psraw_mm_mmm64
	MemorySize::Packed128_Int16,// Psraw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpsraw_xmm_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpsraw_ymm_ymm_xmmm128
	MemorySize::Packed128_Int16,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_Int16,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_Int16,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	MemorySize::Packed64_Int32,// Psrad_mm_mmm64
	MemorySize::Packed128_Int32,// Psrad_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpsrad_xmm_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpsrad_ymm_ymm_xmmm128
	MemorySize::Packed128_Int32,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_Int32,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_Int32,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	MemorySize::Packed128_Int64,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_Int64,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_Int64,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	MemorySize::Packed64_UInt16,// Pavgw_mm_mmm64
	MemorySize::Packed128_UInt16,// Pavgw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpavgw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpavgw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Pmulhuw_mm_mmm64
	MemorySize::Packed128_UInt16,// Pmulhuw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Pmulhw_mm_mmm64
	MemorySize::Packed128_Int16,// Pmulhw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpmulhw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpmulhw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_Float64,// Cvttpd2dq_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vcvttpd2dq_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vcvttpd2dq_xmm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	MemorySize::Packed64_Int32,// Cvtdq2pd_xmm_xmmm64
	MemorySize::Packed64_Int32,// VEX_Vcvtdq2pd_xmm_xmmm64
	MemorySize::Packed128_Int32,// VEX_Vcvtdq2pd_ymm_xmmm128
	MemorySize::Packed64_Int32,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	MemorySize::Packed128_Int32,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	MemorySize::Packed128_Int64,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	MemorySize::Packed128_Float64,// Cvtpd2dq_xmm_xmmm128
	MemorySize::Packed128_Float64,// VEX_Vcvtpd2dq_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vcvtpd2dq_xmm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	MemorySize::Packed64_UInt32,// Movntq_m64_mm
	MemorySize::Packed128_UInt32,// Movntdq_m128_xmm
	MemorySize::Packed128_UInt32,// VEX_Vmovntdq_m128_xmm
	MemorySize::Packed256_UInt32,// VEX_Vmovntdq_m256_ymm
	MemorySize::Packed128_UInt32,// EVEX_Vmovntdq_m128_xmm
	MemorySize::Packed256_UInt32,// EVEX_Vmovntdq_m256_ymm
	MemorySize::Packed512_UInt32,// EVEX_Vmovntdq_m512_zmm
	MemorySize::Packed64_Int8,// Psubsb_mm_mmm64
	MemorySize::Packed128_Int8,// Psubsb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpsubsb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpsubsb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Psubsw_mm_mmm64
	MemorySize::Packed128_Int16,// Psubsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpsubsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpsubsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Pminsw_mm_mmm64
	MemorySize::Packed128_Int16,// Pminsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpminsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpminsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	MemorySize::UInt64,// Por_mm_mmm64
	MemorySize::UInt128,// Por_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vpor_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vpor_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_Int8,// Paddsb_mm_mmm64
	MemorySize::Packed128_Int8,// Paddsb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpaddsb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpaddsb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Paddsw_mm_mmm64
	MemorySize::Packed128_Int16,// Paddsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpaddsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpaddsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_Int16,// Pmaxsw_mm_mmm64
	MemorySize::Packed128_Int16,// Pmaxsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	MemorySize::UInt64,// Pxor_mm_mmm64
	MemorySize::UInt128,// Pxor_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vpxor_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vpxor_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	MemorySize::UInt128,// Lddqu_xmm_m128
	MemorySize::UInt128,// VEX_Vlddqu_xmm_m128
	MemorySize::UInt256,// VEX_Vlddqu_ymm_m256
	MemorySize::Packed64_UInt16,// Psllw_mm_mmm64
	MemorySize::Packed128_UInt16,// Psllw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsllw_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsllw_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	MemorySize::Packed64_UInt32,// Pslld_mm_mmm64
	MemorySize::Packed128_UInt32,// Pslld_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpslld_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpslld_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	MemorySize::UInt64,// Psllq_mm_mmm64
	MemorySize::Packed128_UInt64,// Psllq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpsllq_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpsllq_ymm_ymm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	MemorySize::Packed128_UInt64,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	MemorySize::Packed64_UInt32,// Pmuludq_mm_mmm64
	MemorySize::Packed128_UInt32,// Pmuludq_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpmuludq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpmuludq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt32,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt32,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_Int16,// Pmaddwd_mm_mmm64
	MemorySize::Packed128_Int16,// Pmaddwd_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt8,// Psadbw_mm_mmm64
	MemorySize::Packed128_UInt8,// Psadbw_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpsadbw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpsadbw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	MemorySize::UInt64,// Maskmovq_rDI_mm_mm
	MemorySize::UInt128,// Maskmovdqu_rDI_xmm_xmm
	MemorySize::UInt128,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	MemorySize::Packed64_UInt8,// Psubb_mm_mmm64
	MemorySize::Packed128_UInt8,// Psubb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpsubb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpsubb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Psubw_mm_mmm64
	MemorySize::Packed128_UInt16,// Psubw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpsubw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpsubw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt32,// Psubd_mm_mmm64
	MemorySize::Packed128_UInt32,// Psubd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpsubd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpsubd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Int64,// Psubq_mm_mmm64
	MemorySize::Packed128_UInt64,// Psubq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpsubq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpsubq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_UInt8,// Paddb_mm_mmm64
	MemorySize::Packed128_UInt8,// Paddb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpaddb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpaddb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Paddw_mm_mmm64
	MemorySize::Packed128_UInt16,// Paddw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpaddw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpaddw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt32,// Paddd_mm_mmm64
	MemorySize::Packed128_UInt32,// Paddd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpaddd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpaddd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	MemorySize::UInt16,// Ud0_r16_rm16
	MemorySize::UInt32,// Ud0_r32_rm32
	MemorySize::UInt64,// Ud0_r64_rm64
	MemorySize::Packed64_UInt8,// Pshufb_mm_mmm64
	MemorySize::Packed128_UInt8,// Pshufb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vpshufb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpshufb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Phaddw_mm_mmm64
	MemorySize::Packed128_UInt16,// Phaddw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vphaddw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vphaddw_ymm_ymm_ymmm256
	MemorySize::Packed64_UInt32,// Phaddd_mm_mmm64
	MemorySize::Packed128_UInt32,// Phaddd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vphaddd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vphaddd_ymm_ymm_ymmm256
	MemorySize::Packed64_Int16,// Phaddsw_mm_mmm64
	MemorySize::Packed128_Int16,// Phaddsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vphaddsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vphaddsw_ymm_ymm_ymmm256
	MemorySize::Packed64_Int8,// Pmaddubsw_mm_mmm64
	MemorySize::Packed128_Int8,// Pmaddubsw_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt16,// Phsubw_mm_mmm64
	MemorySize::Packed128_UInt16,// Phsubw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vphsubw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vphsubw_ymm_ymm_ymmm256
	MemorySize::Packed64_UInt32,// Phsubd_mm_mmm64
	MemorySize::Packed128_UInt32,// Phsubd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vphsubd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vphsubd_ymm_ymm_ymmm256
	MemorySize::Packed64_Int16,// Phsubsw_mm_mmm64
	MemorySize::Packed128_Int16,// Phsubsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vphsubsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vphsubsw_ymm_ymm_ymmm256
	MemorySize::Packed64_Int8,// Psignb_mm_mmm64
	MemorySize::Packed128_Int8,// Psignb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpsignb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpsignb_ymm_ymm_ymmm256
	MemorySize::Packed64_Int16,// Psignw_mm_mmm64
	MemorySize::Packed128_Int16,// Psignw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpsignw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpsignw_ymm_ymm_ymmm256
	MemorySize::Packed64_Int32,// Psignd_mm_mmm64
	MemorySize::Packed128_Int32,// Psignd_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpsignd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpsignd_ymm_ymm_ymmm256
	MemorySize::Packed64_Int16,// Pmulhrsw_mm_mmm64
	MemorySize::Packed128_Int16,// Pmulhrsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_Float32,// VEX_Vpermilps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vpermilps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// VEX_Vpermilpd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vpermilpd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// VEX_Vtestps_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vtestps_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vtestpd_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vtestpd_ymm_ymmm256
	MemorySize::Packed128_UInt8,// Pblendvb_xmm_xmmm128
	MemorySize::Packed128_UInt16,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed64_UInt8,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt8,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt8,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	MemorySize::Packed128_UInt16,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed32_UInt8,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	MemorySize::Packed64_UInt8,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	MemorySize::Packed128_UInt8,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	MemorySize::Packed128_UInt16,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed16_UInt8,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	MemorySize::Packed32_UInt8,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	MemorySize::Packed64_UInt8,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	MemorySize::Packed64_Float16,// VEX_Vcvtph2ps_xmm_xmmm64
	MemorySize::Packed128_Float16,// VEX_Vcvtph2ps_ymm_xmmm128
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	MemorySize::Packed64_UInt16,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt16,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt16,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	MemorySize::Packed128_Float32,// Blendvps_xmm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed32_UInt16,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	MemorySize::Packed64_UInt16,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	MemorySize::Packed128_UInt16,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	MemorySize::Packed128_Float64,// Blendvpd_xmm_xmmm128
	MemorySize::Packed128_UInt32,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed64_UInt32,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt32,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt32,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	MemorySize::Packed256_Float32,// VEX_Vpermps_ymm_ymm_ymmm256
	MemorySize::Packed256_Float32,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed256_Float64,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::UInt128,// Ptest_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vptest_xmm_xmmm128
	MemorySize::UInt256,// VEX_Vptest_ymm_ymmm256
	MemorySize::Float32,// VEX_Vbroadcastss_xmm_m32
	MemorySize::Float32,// VEX_Vbroadcastss_ymm_m32
	MemorySize::Float32,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	MemorySize::Float32,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	MemorySize::Float32,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	MemorySize::Float64,// VEX_Vbroadcastsd_ymm_m64
	MemorySize::Packed64_Float32,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	MemorySize::Packed64_Float32,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	MemorySize::Float64,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	MemorySize::Float64,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	MemorySize::Float128,// VEX_Vbroadcastf128_ymm_m128
	MemorySize::Packed128_Float32,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	MemorySize::Packed128_Float32,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	MemorySize::Packed128_Float64,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	MemorySize::Packed128_Float64,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	MemorySize::Packed256_Float32,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	MemorySize::Packed256_Float64,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	MemorySize::Packed64_Int8,// Pabsb_mm_mmm64
	MemorySize::Packed128_Int8,// Pabsb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpabsb_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpabsb_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpabsb_xmm_k1z_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpabsb_ymm_k1z_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpabsb_zmm_k1z_zmmm512
	MemorySize::Packed64_Int16,// Pabsw_mm_mmm64
	MemorySize::Packed128_Int16,// Pabsw_xmm_xmmm128
	MemorySize::Packed128_Int16,// VEX_Vpabsw_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpabsw_ymm_ymmm256
	MemorySize::Packed128_Int16,// EVEX_Vpabsw_xmm_k1z_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpabsw_ymm_k1z_ymmm256
	MemorySize::Packed512_Int16,// EVEX_Vpabsw_zmm_k1z_zmmm512
	MemorySize::Packed64_Int32,// Pabsd_mm_mmm64
	MemorySize::Packed128_Int32,// Pabsd_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpabsd_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpabsd_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	MemorySize::Packed128_Int64,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	MemorySize::Packed64_Int8,// Pmovsxbw_xmm_xmmm64
	MemorySize::Packed64_Int8,// VEX_Vpmovsxbw_xmm_xmmm64
	MemorySize::Packed128_Int8,// VEX_Vpmovsxbw_ymm_xmmm128
	MemorySize::Packed64_Int8,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	MemorySize::Packed128_Int8,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	MemorySize::Packed64_Int8,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	MemorySize::Packed128_Int8,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	MemorySize::Packed256_Int8,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	MemorySize::Packed32_Int8,// Pmovsxbd_xmm_xmmm32
	MemorySize::Packed32_Int8,// VEX_Vpmovsxbd_xmm_xmmm32
	MemorySize::Packed64_Int8,// VEX_Vpmovsxbd_ymm_xmmm64
	MemorySize::Packed32_Int8,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	MemorySize::Packed64_Int8,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	MemorySize::Packed128_Int8,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	MemorySize::Packed32_Int8,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	MemorySize::Packed64_Int8,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	MemorySize::Packed128_Int8,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	MemorySize::Packed16_Int8,// Pmovsxbq_xmm_xmmm16
	MemorySize::Packed16_Int8,// VEX_Vpmovsxbq_xmm_xmmm16
	MemorySize::Packed32_Int8,// VEX_Vpmovsxbq_ymm_xmmm32
	MemorySize::Packed16_Int8,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	MemorySize::Packed32_Int8,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	MemorySize::Packed64_Int8,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	MemorySize::Packed16_Int8,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	MemorySize::Packed32_Int8,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	MemorySize::Packed64_Int8,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	MemorySize::Packed64_Int16,// Pmovsxwd_xmm_xmmm64
	MemorySize::Packed64_Int16,// VEX_Vpmovsxwd_xmm_xmmm64
	MemorySize::Packed128_Int16,// VEX_Vpmovsxwd_ymm_xmmm128
	MemorySize::Packed64_Int16,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	MemorySize::Packed128_Int16,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	MemorySize::Packed256_Int16,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	MemorySize::Packed64_Int16,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	MemorySize::Packed128_Int16,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	MemorySize::Packed256_Int16,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	MemorySize::Packed32_Int16,// Pmovsxwq_xmm_xmmm32
	MemorySize::Packed32_Int16,// VEX_Vpmovsxwq_xmm_xmmm32
	MemorySize::Packed64_Int16,// VEX_Vpmovsxwq_ymm_xmmm64
	MemorySize::Packed32_Int16,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	MemorySize::Packed64_Int16,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	MemorySize::Packed128_Int16,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	MemorySize::Packed32_Int16,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	MemorySize::Packed64_Int16,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	MemorySize::Packed128_Int16,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	MemorySize::Packed64_Int32,// Pmovsxdq_xmm_xmmm64
	MemorySize::Packed64_Int32,// VEX_Vpmovsxdq_xmm_xmmm64
	MemorySize::Packed128_Int32,// VEX_Vpmovsxdq_ymm_xmmm128
	MemorySize::Packed64_Int32,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	MemorySize::Packed128_Int32,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	MemorySize::Packed256_Int32,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	MemorySize::Packed64_Int32,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	MemorySize::Packed128_Int32,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	MemorySize::Packed256_Int32,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	MemorySize::Packed128_UInt8,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	MemorySize::Packed128_UInt8,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	MemorySize::Packed128_UInt32,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	MemorySize::Packed128_Int32,// Pmuldq_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpmuldq_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpmuldq_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Int32,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Int32,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovm2b_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2b_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2b_zmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_zmm_kr
	MemorySize::Packed128_UInt64,// Pcmpeqq_xmm_xmmm128
	MemorySize::Packed128_UInt64,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_zmm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_zmm
	MemorySize::UInt128,// Movntdqa_xmm_m128
	MemorySize::UInt128,// VEX_Vmovntdqa_xmm_m128
	MemorySize::UInt256,// VEX_Vmovntdqa_ymm_m256
	MemorySize::UInt128,// EVEX_Vmovntdqa_xmm_m128
	MemorySize::UInt256,// EVEX_Vmovntdqa_ymm_m256
	MemorySize::UInt512,// EVEX_Vmovntdqa_zmm_m512
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_xmm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_ymm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_zmm_kr
	MemorySize::Packed128_Int32,// Packusdw_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpackusdw_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpackusdw_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float32,// VEX_Vmaskmovps_xmm_xmm_m128
	MemorySize::Packed256_Float32,// VEX_Vmaskmovps_ymm_ymm_m256
	MemorySize::Packed128_Float32,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float64,// VEX_Vmaskmovpd_xmm_xmm_m128
	MemorySize::Packed256_Float64,// VEX_Vmaskmovpd_ymm_ymm_m256
	MemorySize::Float32,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vmaskmovps_m128_xmm_xmm
	MemorySize::Packed256_Float32,// VEX_Vmaskmovps_m256_ymm_ymm
	MemorySize::Packed128_Float64,// VEX_Vmaskmovpd_m128_xmm_xmm
	MemorySize::Packed256_Float64,// VEX_Vmaskmovpd_m256_ymm_ymm
	MemorySize::Packed64_UInt8,// Pmovzxbw_xmm_xmmm64
	MemorySize::Packed64_UInt8,// VEX_Vpmovzxbw_xmm_xmmm64
	MemorySize::Packed128_UInt8,// VEX_Vpmovzxbw_ymm_xmmm128
	MemorySize::Packed64_UInt8,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	MemorySize::Packed128_UInt8,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	MemorySize::Packed64_UInt8,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt8,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt8,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	MemorySize::Packed32_UInt8,// Pmovzxbd_xmm_xmmm32
	MemorySize::Packed32_UInt8,// VEX_Vpmovzxbd_xmm_xmmm32
	MemorySize::Packed64_UInt8,// VEX_Vpmovzxbd_ymm_xmmm64
	MemorySize::Packed32_UInt8,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	MemorySize::Packed64_UInt8,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	MemorySize::Packed128_UInt8,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	MemorySize::Packed32_UInt8,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	MemorySize::Packed64_UInt8,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	MemorySize::Packed128_UInt8,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	MemorySize::Packed16_UInt8,// Pmovzxbq_xmm_xmmm16
	MemorySize::Packed16_UInt8,// VEX_Vpmovzxbq_xmm_xmmm16
	MemorySize::Packed32_UInt8,// VEX_Vpmovzxbq_ymm_xmmm32
	MemorySize::Packed16_UInt8,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	MemorySize::Packed32_UInt8,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	MemorySize::Packed64_UInt8,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	MemorySize::Packed16_UInt8,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	MemorySize::Packed32_UInt8,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	MemorySize::Packed64_UInt8,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	MemorySize::Packed64_UInt16,// Pmovzxwd_xmm_xmmm64
	MemorySize::Packed64_UInt16,// VEX_Vpmovzxwd_xmm_xmmm64
	MemorySize::Packed128_UInt16,// VEX_Vpmovzxwd_ymm_xmmm128
	MemorySize::Packed64_UInt16,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	MemorySize::Packed128_UInt16,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	MemorySize::Packed64_UInt16,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt16,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt16,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	MemorySize::Packed32_UInt16,// Pmovzxwq_xmm_xmmm32
	MemorySize::Packed32_UInt16,// VEX_Vpmovzxwq_xmm_xmmm32
	MemorySize::Packed64_UInt16,// VEX_Vpmovzxwq_ymm_xmmm64
	MemorySize::Packed32_UInt16,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	MemorySize::Packed64_UInt16,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	MemorySize::Packed128_UInt16,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	MemorySize::Packed32_UInt16,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	MemorySize::Packed64_UInt16,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	MemorySize::Packed128_UInt16,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	MemorySize::Packed64_UInt32,// Pmovzxdq_xmm_xmmm64
	MemorySize::Packed64_UInt32,// VEX_Vpmovzxdq_xmm_xmmm64
	MemorySize::Packed128_UInt32,// VEX_Vpmovzxdq_ymm_xmmm128
	MemorySize::Packed64_UInt32,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	MemorySize::Packed128_UInt32,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	MemorySize::Packed256_UInt32,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	MemorySize::Packed64_UInt32,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	MemorySize::Packed128_UInt32,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	MemorySize::Packed256_UInt32,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	MemorySize::Packed256_UInt32,// VEX_Vpermd_ymm_ymm_ymmm256
	MemorySize::Packed256_UInt32,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed256_UInt64,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Int64,// Pcmpgtq_xmm_xmmm128
	MemorySize::Packed128_Int64,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	MemorySize::Packed256_Int64,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	MemorySize::Packed128_Int64,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	MemorySize::Packed128_Int8,// Pminsb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpminsb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpminsb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpmovm2d_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2d_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2d_zmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_zmm_kr
	MemorySize::Packed128_Int32,// Pminsd_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpminsd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpminsd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int64,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_zmm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_zmm
	MemorySize::Packed128_UInt16,// Pminuw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpminuw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpminuw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_xmm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_ymm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_zmm_kr
	MemorySize::Packed128_UInt32,// Pminud_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpminud_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpminud_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Int8,// Pmaxsb_xmm_xmmm128
	MemorySize::Packed128_Int8,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_Int8,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_Int8,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_Int32,// Pmaxsd_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int64,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt16,// Pmaxuw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// Pmaxud_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vpmaxud_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpmaxud_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Int32,// Pmulld_xmm_xmmm128
	MemorySize::Packed128_Int32,// VEX_Vpmulld_xmm_xmm_xmmm128
	MemorySize::Packed256_Int32,// VEX_Vpmulld_ymm_ymm_ymmm256
	MemorySize::Packed128_Int32,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int64,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt16,// Phminposuw_xmm_xmmm128
	MemorySize::Packed128_UInt16,// VEX_Vphminposuw_xmm_xmmm128
	MemorySize::Packed128_Float32,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed128_Float64,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	MemorySize::Float32,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Packed128_UInt32,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	MemorySize::Packed128_UInt32,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt32,// VEX_Vpsravd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpsravd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt32,// VEX_Vpsllvd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vpsllvd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt64,// VEX_Vpsllvq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt64,// VEX_Vpsllvq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	MemorySize::Packed128_Float64,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	MemorySize::Float32,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	MemorySize::Float64,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	MemorySize::Packed128_Float32,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	MemorySize::Packed128_Float64,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	MemorySize::Float32,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	MemorySize::Float64,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	MemorySize::Packed128_Int8,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int8,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int8,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int8,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int8,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int8,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int16,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int16,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int16,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_2xBFloat16,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_2xBFloat16,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_2xBFloat16,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int16,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	MemorySize::Packed128_Int16,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Int16,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Int16,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Int16,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	MemorySize::Packed128_UInt8,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	MemorySize::Int32,// VEX_Vpbroadcastd_xmm_xmmm32
	MemorySize::Int32,// VEX_Vpbroadcastd_ymm_xmmm32
	MemorySize::Int32,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	MemorySize::Int32,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	MemorySize::Int32,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	MemorySize::Int64,// VEX_Vpbroadcastq_xmm_xmmm64
	MemorySize::Int64,// VEX_Vpbroadcastq_ymm_xmmm64
	MemorySize::Packed64_UInt32,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	MemorySize::Packed64_UInt32,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	MemorySize::Packed64_UInt32,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	MemorySize::Int64,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	MemorySize::Int64,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	MemorySize::Int64,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	MemorySize::Int128,// VEX_Vbroadcasti128_ymm_m128
	MemorySize::Packed128_UInt32,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	MemorySize::Packed128_UInt32,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	MemorySize::Packed128_UInt64,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	MemorySize::Packed128_UInt64,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	MemorySize::Packed256_UInt32,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	MemorySize::Packed256_UInt64,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	MemorySize::Packed128_UInt8,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt8,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt8,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt8,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt16,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt16,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt16,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt32,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt8,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	MemorySize::Packed128_UInt16,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt16,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_Float32,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	MemorySize::Packed128_Float32,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt32,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt8,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Int8,// VEX_Vpbroadcastb_xmm_xmmm8
	MemorySize::Int8,// VEX_Vpbroadcastb_ymm_xmmm8
	MemorySize::Int8,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	MemorySize::Int8,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	MemorySize::Int8,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	MemorySize::Int16,// VEX_Vpbroadcastw_xmm_xmmm16
	MemorySize::Int16,// VEX_Vpbroadcastw_ymm_xmmm16
	MemorySize::Int16,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	MemorySize::Int16,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	MemorySize::Int16,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpbroadcastb_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastb_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastb_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastq_xmm_k1z_r64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_ymm_k1z_r64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_zmm_k1z_r64
	MemorySize::Packed128_UInt8,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Packed128_Float64,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	MemorySize::UInt128,// Invept_r32_m128
	MemorySize::UInt128,// Invept_r64_m128
	MemorySize::UInt128,// Invvpid_r32_m128
	MemorySize::UInt128,// Invvpid_r64_m128
	MemorySize::UInt128,// Invpcid_r32_m128
	MemorySize::UInt128,// Invpcid_r64_m128
	MemorySize::Packed128_UInt64,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// EVEX_Vexpandps_xmm_k1z_xmmm128
	MemorySize::Packed256_Float32,// EVEX_Vexpandps_ymm_k1z_ymmm256
	MemorySize::Packed512_Float32,// EVEX_Vexpandps_zmm_k1z_zmmm512
	MemorySize::Packed128_Float64,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	MemorySize::Packed256_Float64,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	MemorySize::Packed512_Float64,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt32,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt32,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt32,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	MemorySize::Packed128_UInt64,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	MemorySize::Packed256_UInt64,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	MemorySize::Packed512_UInt64,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	MemorySize::Packed128_Float32,// EVEX_Vcompressps_xmmm128_k1z_xmm
	MemorySize::Packed256_Float32,// EVEX_Vcompressps_ymmm256_k1z_ymm
	MemorySize::Packed512_Float32,// EVEX_Vcompressps_zmmm512_k1z_zmm
	MemorySize::Packed128_Float64,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	MemorySize::Packed256_Float64,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	MemorySize::Packed512_Float64,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt32,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt32,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt32,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt64,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	MemorySize::Packed256_UInt64,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	MemorySize::Packed512_UInt64,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	MemorySize::Packed128_UInt32,// VEX_Vpmaskmovd_xmm_xmm_m128
	MemorySize::Packed256_UInt32,// VEX_Vpmaskmovd_ymm_ymm_m256
	MemorySize::Packed128_UInt64,// VEX_Vpmaskmovq_xmm_xmm_m128
	MemorySize::Packed256_UInt64,// VEX_Vpmaskmovq_ymm_ymm_m256
	MemorySize::Packed128_UInt8,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt16,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt16,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt16,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	MemorySize::Packed128_UInt32,// VEX_Vpmaskmovd_m128_xmm_xmm
	MemorySize::Packed256_UInt32,// VEX_Vpmaskmovd_m256_ymm_ymm
	MemorySize::Packed128_UInt64,// VEX_Vpmaskmovq_m128_xmm_xmm
	MemorySize::Packed256_UInt64,// VEX_Vpmaskmovq_m256_ymm_ymm
	MemorySize::Packed128_UInt8,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	MemorySize::Int32,// VEX_Vpgatherdd_xmm_vm32x_xmm
	MemorySize::Int32,// VEX_Vpgatherdd_ymm_vm32y_ymm
	MemorySize::Int64,// VEX_Vpgatherdq_xmm_vm32x_xmm
	MemorySize::Int64,// VEX_Vpgatherdq_ymm_vm32x_ymm
	MemorySize::Int32,// EVEX_Vpgatherdd_xmm_k1_vm32x
	MemorySize::Int32,// EVEX_Vpgatherdd_ymm_k1_vm32y
	MemorySize::Int32,// EVEX_Vpgatherdd_zmm_k1_vm32z
	MemorySize::Int64,// EVEX_Vpgatherdq_xmm_k1_vm32x
	MemorySize::Int64,// EVEX_Vpgatherdq_ymm_k1_vm32x
	MemorySize::Int64,// EVEX_Vpgatherdq_zmm_k1_vm32y
	MemorySize::Int32,// VEX_Vpgatherqd_xmm_vm64x_xmm
	MemorySize::Int32,// VEX_Vpgatherqd_xmm_vm64y_xmm
	MemorySize::Int64,// VEX_Vpgatherqq_xmm_vm64x_xmm
	MemorySize::Int64,// VEX_Vpgatherqq_ymm_vm64y_ymm
	MemorySize::Int32,// EVEX_Vpgatherqd_xmm_k1_vm64x
	MemorySize::Int32,// EVEX_Vpgatherqd_xmm_k1_vm64y
	MemorySize::Int32,// EVEX_Vpgatherqd_ymm_k1_vm64z
	MemorySize::Int64,// EVEX_Vpgatherqq_xmm_k1_vm64x
	MemorySize::Int64,// EVEX_Vpgatherqq_ymm_k1_vm64y
	MemorySize::Int64,// EVEX_Vpgatherqq_zmm_k1_vm64z
	MemorySize::Float32,// VEX_Vgatherdps_xmm_vm32x_xmm
	MemorySize::Float32,// VEX_Vgatherdps_ymm_vm32y_ymm
	MemorySize::Float64,// VEX_Vgatherdpd_xmm_vm32x_xmm
	MemorySize::Float64,// VEX_Vgatherdpd_ymm_vm32x_ymm
	MemorySize::Float32,// EVEX_Vgatherdps_xmm_k1_vm32x
	MemorySize::Float32,// EVEX_Vgatherdps_ymm_k1_vm32y
	MemorySize::Float32,// EVEX_Vgatherdps_zmm_k1_vm32z
	MemorySize::Float64,// EVEX_Vgatherdpd_xmm_k1_vm32x
	MemorySize::Float64,// EVEX_Vgatherdpd_ymm_k1_vm32x
	MemorySize::Float64,// EVEX_Vgatherdpd_zmm_k1_vm32y
	MemorySize::Float32,// VEX_Vgatherqps_xmm_vm64x_xmm
	MemorySize::Float32,// VEX_Vgatherqps_xmm_vm64y_xmm
	MemorySize::Float64,// VEX_Vgatherqpd_xmm_vm64x_xmm
	MemorySize::Float64,// VEX_Vgatherqpd_ymm_vm64y_ymm
	MemorySize::Float32,// EVEX_Vgatherqps_xmm_k1_vm64x
	MemorySize::Float32,// EVEX_Vgatherqps_xmm_k1_vm64y
	MemorySize::Float32,// EVEX_Vgatherqps_ymm_k1_vm64z
	MemorySize::Float64,// EVEX_Vgatherqpd_xmm_k1_vm64x
	MemorySize::Float64,// EVEX_Vgatherqpd_ymm_k1_vm64y
	MemorySize::Float64,// EVEX_Vgatherqpd_zmm_k1_vm64z
	MemorySize::Packed128_Float32,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	MemorySize::Float32,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	MemorySize::Packed128_Float32,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Int32,// EVEX_Vpscatterdd_vm32x_k1_xmm
	MemorySize::Int32,// EVEX_Vpscatterdd_vm32y_k1_ymm
	MemorySize::Int32,// EVEX_Vpscatterdd_vm32z_k1_zmm
	MemorySize::Int64,// EVEX_Vpscatterdq_vm32x_k1_xmm
	MemorySize::Int64,// EVEX_Vpscatterdq_vm32x_k1_ymm
	MemorySize::Int64,// EVEX_Vpscatterdq_vm32y_k1_zmm
	MemorySize::Int32,// EVEX_Vpscatterqd_vm64x_k1_xmm
	MemorySize::Int32,// EVEX_Vpscatterqd_vm64y_k1_xmm
	MemorySize::Int32,// EVEX_Vpscatterqd_vm64z_k1_ymm
	MemorySize::Int64,// EVEX_Vpscatterqq_vm64x_k1_xmm
	MemorySize::Int64,// EVEX_Vpscatterqq_vm64y_k1_ymm
	MemorySize::Int64,// EVEX_Vpscatterqq_vm64z_k1_zmm
	MemorySize::Float32,// EVEX_Vscatterdps_vm32x_k1_xmm
	MemorySize::Float32,// EVEX_Vscatterdps_vm32y_k1_ymm
	MemorySize::Float32,// EVEX_Vscatterdps_vm32z_k1_zmm
	MemorySize::Float64,// EVEX_Vscatterdpd_vm32x_k1_xmm
	MemorySize::Float64,// EVEX_Vscatterdpd_vm32x_k1_ymm
	MemorySize::Float64,// EVEX_Vscatterdpd_vm32y_k1_zmm
	MemorySize::Float32,// EVEX_Vscatterqps_vm64x_k1_xmm
	MemorySize::Float32,// EVEX_Vscatterqps_vm64y_k1_xmm
	MemorySize::Float32,// EVEX_Vscatterqps_vm64z_k1_ymm
	MemorySize::Float64,// EVEX_Vscatterqpd_vm64x_k1_xmm
	MemorySize::Float64,// EVEX_Vscatterqpd_vm64y_k1_ymm
	MemorySize::Float64,// EVEX_Vscatterqpd_vm64z_k1_zmm
	MemorySize::Packed128_Float32,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	MemorySize::Float32,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	MemorySize::Packed128_Float32,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_UInt52,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt52,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt52,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_UInt52,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_UInt52,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_UInt52,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Packed128_Float32,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Packed128_Float32,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_Float32,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Float32,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	MemorySize::Float32,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Float64,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Packed128_UInt32,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	MemorySize::Packed128_UInt64,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	MemorySize::Float32,// EVEX_Vgatherpf0dps_vm32z_k1
	MemorySize::Float64,// EVEX_Vgatherpf0dpd_vm32y_k1
	MemorySize::Float32,// EVEX_Vgatherpf1dps_vm32z_k1
	MemorySize::Float64,// EVEX_Vgatherpf1dpd_vm32y_k1
	MemorySize::Float32,// EVEX_Vscatterpf0dps_vm32z_k1
	MemorySize::Float64,// EVEX_Vscatterpf0dpd_vm32y_k1
	MemorySize::Float32,// EVEX_Vscatterpf1dps_vm32z_k1
	MemorySize::Float64,// EVEX_Vscatterpf1dpd_vm32y_k1
	MemorySize::Float32,// EVEX_Vgatherpf0qps_vm64z_k1
	MemorySize::Float64,// EVEX_Vgatherpf0qpd_vm64z_k1
	MemorySize::Float32,// EVEX_Vgatherpf1qps_vm64z_k1
	MemorySize::Float64,// EVEX_Vgatherpf1qpd_vm64z_k1
	MemorySize::Float32,// EVEX_Vscatterpf0qps_vm64z_k1
	MemorySize::Float64,// EVEX_Vscatterpf0qpd_vm64z_k1
	MemorySize::Float32,// EVEX_Vscatterpf1qps_vm64z_k1
	MemorySize::Float64,// EVEX_Vscatterpf1qpd_vm64z_k1
	MemorySize::Packed128_UInt32,// Sha1nexte_xmm_xmmm128
	MemorySize::Packed512_Float32,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed512_Float64,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Packed128_UInt32,// Sha1msg1_xmm_xmmm128
	MemorySize::Packed128_UInt32,// Sha1msg2_xmm_xmmm128
	MemorySize::Packed512_Float32,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed512_Float64,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Packed128_UInt32,// Sha256rnds2_xmm_xmmm128
	MemorySize::Float32,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Packed128_UInt32,// Sha256msg1_xmm_xmmm128
	MemorySize::Packed512_Float32,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Packed512_Float64,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Packed128_UInt32,// Sha256msg2_xmm_xmmm128
	MemorySize::Float32,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Float64,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Packed128_UInt8,// Gf2p8mulb_xmm_xmmm128
	MemorySize::Packed128_UInt8,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	MemorySize::Packed256_UInt8,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	MemorySize::Packed512_UInt8,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	MemorySize::UInt128,// Aesimc_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vaesimc_xmm_xmmm128
	MemorySize::UInt128,// Aesenc_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vaesenc_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vaesenc_ymm_ymm_ymmm256
	MemorySize::UInt128,// EVEX_Vaesenc_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// EVEX_Vaesenc_ymm_ymm_ymmm256
	MemorySize::Packed512_UInt128,// EVEX_Vaesenc_zmm_zmm_zmmm512
	MemorySize::UInt128,// Aesenclast_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vaesenclast_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vaesenclast_ymm_ymm_ymmm256
	MemorySize::UInt128,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	MemorySize::Packed512_UInt128,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	MemorySize::UInt128,// Aesdec_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vaesdec_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vaesdec_ymm_ymm_ymmm256
	MemorySize::UInt128,// EVEX_Vaesdec_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// EVEX_Vaesdec_ymm_ymm_ymmm256
	MemorySize::Packed512_UInt128,// EVEX_Vaesdec_zmm_zmm_zmmm512
	MemorySize::UInt128,// Aesdeclast_xmm_xmmm128
	MemorySize::UInt128,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	MemorySize::UInt128,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt128,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	MemorySize::Packed512_UInt128,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	MemorySize::UInt16,// Movbe_r16_m16
	MemorySize::UInt32,// Movbe_r32_m32
	MemorySize::UInt64,// Movbe_r64_m64
	MemorySize::UInt8,// Crc32_r32_rm8
	MemorySize::UInt8,// Crc32_r64_rm8
	MemorySize::UInt16,// Movbe_m16_r16
	MemorySize::UInt32,// Movbe_m32_r32
	MemorySize::UInt64,// Movbe_m64_r64
	MemorySize::UInt16,// Crc32_r32_rm16
	MemorySize::UInt32,// Crc32_r32_rm32
	MemorySize::UInt64,// Crc32_r64_rm64
	MemorySize::UInt32,// VEX_Andn_r32_r32_rm32
	MemorySize::UInt64,// VEX_Andn_r64_r64_rm64
	MemorySize::UInt32,// VEX_Blsr_r32_rm32
	MemorySize::UInt64,// VEX_Blsr_r64_rm64
	MemorySize::UInt32,// VEX_Blsmsk_r32_rm32
	MemorySize::UInt64,// VEX_Blsmsk_r64_rm64
	MemorySize::UInt32,// VEX_Blsi_r32_rm32
	MemorySize::UInt64,// VEX_Blsi_r64_rm64
	MemorySize::UInt32,// VEX_Bzhi_r32_rm32_r32
	MemorySize::UInt64,// VEX_Bzhi_r64_rm64_r64
	MemorySize::UInt32,// Wrussd_m32_r32
	MemorySize::UInt64,// Wrussq_m64_r64
	MemorySize::UInt32,// VEX_Pext_r32_r32_rm32
	MemorySize::UInt64,// VEX_Pext_r64_r64_rm64
	MemorySize::UInt32,// VEX_Pdep_r32_r32_rm32
	MemorySize::UInt64,// VEX_Pdep_r64_r64_rm64
	MemorySize::UInt32,// Wrssd_m32_r32
	MemorySize::UInt64,// Wrssq_m64_r64
	MemorySize::UInt32,// Adcx_r32_rm32
	MemorySize::UInt64,// Adcx_r64_rm64
	MemorySize::UInt32,// Adox_r32_rm32
	MemorySize::UInt64,// Adox_r64_rm64
	MemorySize::UInt32,// VEX_Mulx_r32_r32_rm32
	MemorySize::UInt64,// VEX_Mulx_r64_r64_rm64
	MemorySize::UInt32,// VEX_Bextr_r32_rm32_r32
	MemorySize::UInt64,// VEX_Bextr_r64_rm64_r64
	MemorySize::UInt32,// VEX_Shlx_r32_rm32_r32
	MemorySize::UInt64,// VEX_Shlx_r64_rm64_r64
	MemorySize::Int32,// VEX_Sarx_r32_rm32_r32
	MemorySize::Int64,// VEX_Sarx_r64_rm64_r64
	MemorySize::UInt32,// VEX_Shrx_r32_rm32_r32
	MemorySize::UInt64,// VEX_Shrx_r64_rm64_r64
	MemorySize::UInt512,// Movdir64b_r16_m512
	MemorySize::UInt512,// Movdir64b_r32_m512
	MemorySize::UInt512,// Movdir64b_r64_m512
	MemorySize::UInt512,// Enqcmds_r16_m512
	MemorySize::UInt512,// Enqcmds_r32_m512
	MemorySize::UInt512,// Enqcmds_r64_m512
	MemorySize::UInt512,// Enqcmd_r16_m512
	MemorySize::UInt512,// Enqcmd_r32_m512
	MemorySize::UInt512,// Enqcmd_r64_m512
	MemorySize::UInt32,// Movdiri_m32_r32
	MemorySize::UInt64,// Movdiri_m64_r64
	MemorySize::Packed256_UInt64,// VEX_Vpermq_ymm_ymmm256_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Packed256_Float64,// VEX_Vpermpd_ymm_ymmm256_imm8
	MemorySize::Packed256_Float64,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	MemorySize::Packed128_UInt32,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt32,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt32,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_Float32,// VEX_Vpermilps_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vpermilps_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	MemorySize::Packed128_Float64,// VEX_Vpermilpd_xmm_xmmm128_imm8
	MemorySize::Packed256_Float64,// VEX_Vpermilpd_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	MemorySize::Packed256_Float128,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// Roundps_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// VEX_Vroundps_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vroundps_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// Roundpd_xmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// VEX_Vroundpd_xmm_xmmm128_imm8
	MemorySize::Packed256_Float64,// VEX_Vroundpd_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Float32,// Roundss_xmm_xmmm32_imm8
	MemorySize::Float32,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	MemorySize::Float32,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// Roundsd_xmm_xmmm64_imm8
	MemorySize::Float64,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	MemorySize::Float64,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Packed128_Float32,// Blendps_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// Blendpd_xmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float64,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt16,// Pblendw_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt16,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	MemorySize::Packed64_UInt8,// Palignr_mm_mmm64_imm8
	MemorySize::Packed128_UInt8,// Palignr_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt8,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt8,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::UInt8,// Pextrb_r32m8_xmm_imm8
	MemorySize::UInt8,// Pextrb_r64m8_xmm_imm8
	MemorySize::UInt8,// VEX_Vpextrb_r32m8_xmm_imm8
	MemorySize::UInt8,// VEX_Vpextrb_r64m8_xmm_imm8
	MemorySize::UInt8,// EVEX_Vpextrb_r32m8_xmm_imm8
	MemorySize::UInt8,// EVEX_Vpextrb_r64m8_xmm_imm8
	MemorySize::UInt16,// Pextrw_r32m16_xmm_imm8
	MemorySize::UInt16,// Pextrw_r64m16_xmm_imm8
	MemorySize::UInt16,// VEX_Vpextrw_r32m16_xmm_imm8
	MemorySize::UInt16,// VEX_Vpextrw_r64m16_xmm_imm8
	MemorySize::UInt16,// EVEX_Vpextrw_r32m16_xmm_imm8
	MemorySize::UInt16,// EVEX_Vpextrw_r64m16_xmm_imm8
	MemorySize::UInt32,// Pextrd_rm32_xmm_imm8
	MemorySize::UInt64,// Pextrq_rm64_xmm_imm8
	MemorySize::UInt32,// VEX_Vpextrd_rm32_xmm_imm8
	MemorySize::UInt64,// VEX_Vpextrq_rm64_xmm_imm8
	MemorySize::UInt32,// EVEX_Vpextrd_rm32_xmm_imm8
	MemorySize::UInt64,// EVEX_Vpextrq_rm64_xmm_imm8
	MemorySize::Float32,// Extractps_rm32_xmm_imm8
	MemorySize::Float32,// Extractps_r64m32_xmm_imm8
	MemorySize::Float32,// VEX_Vextractps_rm32_xmm_imm8
	MemorySize::Float32,// VEX_Vextractps_r64m32_xmm_imm8
	MemorySize::Float32,// EVEX_Vextractps_rm32_xmm_imm8
	MemorySize::Float32,// EVEX_Vextractps_r64m32_xmm_imm8
	MemorySize::Float128,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	MemorySize::Packed128_Float32,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Packed128_Float32,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Packed128_Float64,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Float128,// VEX_Vextractf128_xmmm128_ymm_imm8
	MemorySize::Packed128_Float32,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	MemorySize::Packed128_Float32,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	MemorySize::Packed128_Float64,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	MemorySize::Packed128_Float64,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	MemorySize::Packed256_Float32,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Packed256_Float64,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Packed256_Float32,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	MemorySize::Packed256_Float64,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	MemorySize::Packed64_Float16,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	MemorySize::Packed128_Float16,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	MemorySize::Packed64_Float16,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	MemorySize::Packed128_Float16,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	MemorySize::Packed256_Float16,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	MemorySize::Packed128_UInt32,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	MemorySize::Packed128_Int32,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Packed256_Int32,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Int32,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	MemorySize::Packed128_Int64,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Packed256_Int64,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Int64,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	MemorySize::UInt8,// Pinsrb_xmm_r32m8_imm8
	MemorySize::UInt8,// Pinsrb_xmm_r64m8_imm8
	MemorySize::UInt8,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	MemorySize::UInt8,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	MemorySize::UInt8,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	MemorySize::UInt8,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	MemorySize::Float32,// Insertps_xmm_xmmm32_imm8
	MemorySize::Float32,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	MemorySize::Float32,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	MemorySize::UInt32,// Pinsrd_xmm_rm32_imm8
	MemorySize::UInt64,// Pinsrq_xmm_rm64_imm8
	MemorySize::UInt32,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	MemorySize::UInt64,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	MemorySize::UInt32,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	MemorySize::UInt64,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	MemorySize::Packed256_Float32,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed256_Float64,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_Float32,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Float32,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Unknown,// VEX_Kshiftrb_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrw_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrd_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrq_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlb_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlw_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftld_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlq_kr_kr_imm8
	MemorySize::Int128,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Int128,// VEX_Vextracti128_xmmm128_ymm_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	MemorySize::Packed128_UInt8,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt8,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	MemorySize::Packed128_Int8,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	MemorySize::Packed256_Int8,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	MemorySize::Packed512_Int8,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	MemorySize::Packed128_Int16,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	MemorySize::Packed256_Int16,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	MemorySize::Packed512_Int16,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	MemorySize::Packed128_Float32,// Dpps_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_Float32,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float64,// Dppd_xmm_xmmm128_imm8
	MemorySize::Packed128_Float64,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Mpsadbw_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt8,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt8,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_UInt64,// Pclmulqdq_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt64,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt64,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	MemorySize::Packed256_UInt128,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_Float32,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	MemorySize::Packed256_Float32,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	MemorySize::Packed128_Float32,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	MemorySize::Packed256_Float32,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	MemorySize::Packed128_Float64,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	MemorySize::Packed256_Float64,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	MemorySize::Packed128_Float64,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	MemorySize::Packed256_Float64,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	MemorySize::Packed128_Float32,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_UInt8,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_UInt8,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	MemorySize::Float32,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Packed128_Float32,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	MemorySize::Float32,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Packed128_Float32,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Packed128_Float64,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Float32,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Float64,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Packed128_Float32,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float32,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// Pcmpestrm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Pcmpestrm64_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Pcmpestri_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Pcmpestri64_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Pcmpistrm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Pcmpistri_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	MemorySize::Packed128_Float32,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	MemorySize::Packed256_Float32,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	MemorySize::Packed512_Float32,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	MemorySize::Packed128_Float64,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	MemorySize::Packed256_Float64,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	MemorySize::Packed512_Float64,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	MemorySize::Float32,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	MemorySize::Float64,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	MemorySize::Packed128_Float32,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Float32,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	MemorySize::Float32,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	MemorySize::Float64,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	MemorySize::Packed128_Float32,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Float32,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	MemorySize::Float32,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	MemorySize::Float64,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	MemorySize::Packed128_UInt16,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_UInt16,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt16,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Packed512_UInt16,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Packed128_UInt32,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Packed256_UInt32,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Packed512_UInt32,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Packed128_UInt64,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt64,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt64,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_Float32,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Float32,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	MemorySize::Float32,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	MemorySize::Float64,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	MemorySize::Packed128_Float32,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float32,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float32,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_Float64,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed256_Float64,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Packed128_Float64,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Packed256_Float64,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Float32,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	MemorySize::Float32,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	MemorySize::Float64,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	MemorySize::Float64,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	MemorySize::Packed128_UInt32,// Sha1rnds4_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// Gf2p8affineqb_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt8,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt8,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt8,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Packed128_UInt8,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	MemorySize::Packed256_UInt8,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	MemorySize::Packed128_UInt8,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Packed256_UInt8,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Packed512_UInt8,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::UInt128,// Aeskeygenassist_xmm_xmmm128_imm8
	MemorySize::UInt128,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	MemorySize::UInt32,// VEX_Rorx_r32_rm32_imm8
	MemorySize::UInt64,// VEX_Rorx_r64_rm64_imm8
	MemorySize::Packed128_Int16,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int16,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int16,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int16,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	MemorySize::UInt128,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	MemorySize::UInt256,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	MemorySize::UInt128,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	MemorySize::UInt256,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt8,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	MemorySize::Packed128_Int16,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_Int16,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt8,// XOP_Vprotb_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt16,// XOP_Vprotw_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt32,// XOP_Vprotd_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt64,// XOP_Vprotq_xmm_xmmm128_imm8
	MemorySize::Packed128_Int8,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_Int16,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_Int32,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_Int64,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt8,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt16,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt32,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	MemorySize::Packed128_UInt64,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	MemorySize::UInt32,// XOP_Blcfill_r32_rm32
	MemorySize::UInt64,// XOP_Blcfill_r64_rm64
	MemorySize::UInt32,// XOP_Blsfill_r32_rm32
	MemorySize::UInt64,// XOP_Blsfill_r64_rm64
	MemorySize::UInt32,// XOP_Blcs_r32_rm32
	MemorySize::UInt64,// XOP_Blcs_r64_rm64
	MemorySize::UInt32,// XOP_Tzmsk_r32_rm32
	MemorySize::UInt64,// XOP_Tzmsk_r64_rm64
	MemorySize::UInt32,// XOP_Blcic_r32_rm32
	MemorySize::UInt64,// XOP_Blcic_r64_rm64
	MemorySize::UInt32,// XOP_Blsic_r32_rm32
	MemorySize::UInt64,// XOP_Blsic_r64_rm64
	MemorySize::UInt32,// XOP_T1mskc_r32_rm32
	MemorySize::UInt64,// XOP_T1mskc_r64_rm64
	MemorySize::UInt32,// XOP_Blcmsk_r32_rm32
	MemorySize::UInt64,// XOP_Blcmsk_r64_rm64
	MemorySize::UInt32,// XOP_Blci_r32_rm32
	MemorySize::UInt64,// XOP_Blci_r64_rm64
	MemorySize::Unknown,// XOP_Llwpcb_r32
	MemorySize::Unknown,// XOP_Llwpcb_r64
	MemorySize::Unknown,// XOP_Slwpcb_r32
	MemorySize::Unknown,// XOP_Slwpcb_r64
	MemorySize::Packed128_Float32,// XOP_Vfrczps_xmm_xmmm128
	MemorySize::Packed256_Float32,// XOP_Vfrczps_ymm_ymmm256
	MemorySize::Packed128_Float64,// XOP_Vfrczpd_xmm_xmmm128
	MemorySize::Packed256_Float64,// XOP_Vfrczpd_ymm_ymmm256
	MemorySize::Float32,// XOP_Vfrczss_xmm_xmmm32
	MemorySize::Float64,// XOP_Vfrczsd_xmm_xmmm64
	MemorySize::Packed128_UInt8,// XOP_Vprotb_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt8,// XOP_Vprotb_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt16,// XOP_Vprotw_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt16,// XOP_Vprotw_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// XOP_Vprotd_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt32,// XOP_Vprotd_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt64,// XOP_Vprotq_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt64,// XOP_Vprotq_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt8,// XOP_Vpshlb_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt8,// XOP_Vpshlb_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt16,// XOP_Vpshlw_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt16,// XOP_Vpshlw_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// XOP_Vpshld_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt32,// XOP_Vpshld_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt64,// XOP_Vpshlq_xmm_xmmm128_xmm
	MemorySize::Packed128_UInt64,// XOP_Vpshlq_xmm_xmm_xmmm128
	MemorySize::Packed128_Int8,// XOP_Vpshab_xmm_xmmm128_xmm
	MemorySize::Packed128_Int8,// XOP_Vpshab_xmm_xmm_xmmm128
	MemorySize::Packed128_Int16,// XOP_Vpshaw_xmm_xmmm128_xmm
	MemorySize::Packed128_Int16,// XOP_Vpshaw_xmm_xmm_xmmm128
	MemorySize::Packed128_Int32,// XOP_Vpshad_xmm_xmmm128_xmm
	MemorySize::Packed128_Int32,// XOP_Vpshad_xmm_xmm_xmmm128
	MemorySize::Packed128_Int64,// XOP_Vpshaq_xmm_xmmm128_xmm
	MemorySize::Packed128_Int64,// XOP_Vpshaq_xmm_xmm_xmmm128
	MemorySize::Packed128_Int8,// XOP_Vphaddbw_xmm_xmmm128
	MemorySize::Packed128_Int8,// XOP_Vphaddbd_xmm_xmmm128
	MemorySize::Packed128_Int8,// XOP_Vphaddbq_xmm_xmmm128
	MemorySize::Packed128_Int16,// XOP_Vphaddwd_xmm_xmmm128
	MemorySize::Packed128_Int16,// XOP_Vphaddwq_xmm_xmmm128
	MemorySize::Packed128_Int32,// XOP_Vphadddq_xmm_xmmm128
	MemorySize::Packed128_UInt8,// XOP_Vphaddubw_xmm_xmmm128
	MemorySize::Packed128_UInt8,// XOP_Vphaddubd_xmm_xmmm128
	MemorySize::Packed128_UInt8,// XOP_Vphaddubq_xmm_xmmm128
	MemorySize::Packed128_UInt16,// XOP_Vphadduwd_xmm_xmmm128
	MemorySize::Packed128_UInt16,// XOP_Vphadduwq_xmm_xmmm128
	MemorySize::Packed128_UInt32,// XOP_Vphaddudq_xmm_xmmm128
	MemorySize::Packed128_Int8,// XOP_Vphsubbw_xmm_xmmm128
	MemorySize::Packed128_Int16,// XOP_Vphsubwd_xmm_xmmm128
	MemorySize::Packed128_Int32,// XOP_Vphsubdq_xmm_xmmm128
	MemorySize::UInt32,// XOP_Bextr_r32_rm32_imm32
	MemorySize::UInt64,// XOP_Bextr_r64_rm64_imm32
	MemorySize::UInt32,// XOP_Lwpins_r32_rm32_imm32
	MemorySize::UInt32,// XOP_Lwpins_r64_rm32_imm32
	MemorySize::UInt32,// XOP_Lwpval_r32_rm32_imm32
	MemorySize::UInt32,// XOP_Lwpval_r64_rm32_imm32
	MemorySize::Packed64_Int16,// D3NOW_Pi2fw_mm_mmm64
	MemorySize::Packed64_Int32,// D3NOW_Pi2fd_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pf2iw_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pf2id_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrcpv_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrsqrtv_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfnacc_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfpnacc_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfcmpge_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfmin_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrcp_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrsqrt_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfsub_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfadd_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfcmpgt_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfmax_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrcpit1_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrsqit1_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfsubr_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfacc_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfcmpeq_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfmul_mm_mmm64
	MemorySize::Packed64_Float32,// D3NOW_Pfrcpit2_mm_mmm64
	MemorySize::Packed64_Int16,// D3NOW_Pmulhrw_mm_mmm64
	MemorySize::Packed64_UInt32,// D3NOW_Pswapd_mm_mmm64
	MemorySize::Packed64_UInt8,// D3NOW_Pavgusb_mm_mmm64
	MemorySize::Unknown,// Rmpadjust
	MemorySize::Unknown,// Rmpupdate
	MemorySize::Unknown,// Psmash
	MemorySize::Unknown,// Pvalidatew
	MemorySize::Unknown,// Pvalidated
	MemorySize::Unknown,// Pvalidateq
	MemorySize::Unknown,// Serialize
	MemorySize::Unknown,// Xsusldtrk
	MemorySize::Unknown,// Xresldtrk
	MemorySize::Unknown,// Invlpgbw
	MemorySize::Unknown,// Invlpgbd
	MemorySize::Unknown,// Invlpgbq
	MemorySize::Unknown,// Tlbsync
	MemorySize::UInt8,// Prefetchreserved3_m8
	MemorySize::UInt8,// Prefetchreserved4_m8
	MemorySize::UInt8,// Prefetchreserved5_m8
	MemorySize::UInt8,// Prefetchreserved6_m8
	MemorySize::UInt8,// Prefetchreserved7_m8
	MemorySize::Unknown,// Ud0
	MemorySize::Unknown,// Vmgexit
	MemorySize::Unknown,// Getsecq
	MemorySize::Tilecfg,// VEX_Ldtilecfg_m512
	MemorySize::Unknown,// VEX_Tilerelease
	MemorySize::Tilecfg,// VEX_Sttilecfg_m512
	MemorySize::Unknown,// VEX_Tilezero_tmm
	MemorySize::Tile,// VEX_Tileloaddt1_tmm_sibmem
	MemorySize::Tile,// VEX_Tilestored_sibmem_tmm
	MemorySize::Tile,// VEX_Tileloadd_tmm_sibmem
	MemorySize::Unknown,// VEX_Tdpbf16ps_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbuud_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbusd_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbsud_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbssd_tmm_tmm_tmm
	MemorySize::Unknown,// Fnstdw_AX
	MemorySize::Unknown,// Fnstsg_AX
	MemorySize::UInt32,// Rdshr_rm32
	MemorySize::UInt32,// Wrshr_rm32
	MemorySize::Unknown,// Smint
	MemorySize::Unknown,// Dmint
	MemorySize::Unknown,// Rdm
	MemorySize::SegmentDescSelector,// Svdc_m80_Sreg
	MemorySize::SegmentDescSelector,// Rsdc_Sreg_m80
	MemorySize::SegmentDescSelector,// Svldt_m80
	MemorySize::SegmentDescSelector,// Rsldt_m80
	MemorySize::SegmentDescSelector,// Svts_m80
	MemorySize::SegmentDescSelector,// Rsts_m80
	MemorySize::Unknown,// Smint_0F7E
	MemorySize::Unknown,// Bb0_reset
	MemorySize::Unknown,// Bb1_reset
	MemorySize::Unknown,// Cpu_write
	MemorySize::Unknown,// Cpu_read
	MemorySize::Unknown,// Altinst
	MemorySize::Packed64_UInt8,// Paveb_mm_mmm64
	MemorySize::Packed64_Int16,// Paddsiw_mm_mmm64
	MemorySize::Packed64_UInt16,// Pmagw_mm_mmm64
	MemorySize::Packed64_UInt8,// Pdistib_mm_m64
	MemorySize::Packed64_Int16,// Psubsiw_mm_mmm64
	MemorySize::Packed64_UInt8,// Pmvzb_mm_m64
	MemorySize::Packed64_Int16,// Pmulhrw_mm_mmm64
	MemorySize::Packed64_UInt8,// Pmvnzb_mm_m64
	MemorySize::Packed64_Int8,// Pmvlzb_mm_m64
	MemorySize::Packed64_Int8,// Pmvgezb_mm_m64
	MemorySize::Packed64_Int16,// Pmulhriw_mm_mmm64
	MemorySize::Packed64_UInt16,// Pmachriw_mm_m64
	MemorySize::Unknown,// Cyrix_D9D7
	MemorySize::Unknown,// Cyrix_D9E2
	MemorySize::Unknown,// Ftstp
	MemorySize::Unknown,// Cyrix_D9E7
	MemorySize::Unknown,// Frint2
	MemorySize::Unknown,// Frichop
	MemorySize::Unknown,// Cyrix_DED8
	MemorySize::Unknown,// Cyrix_DEDA
	MemorySize::Unknown,// Cyrix_DEDC
	MemorySize::Unknown,// Cyrix_DEDD
	MemorySize::Unknown,// Cyrix_DEDE
	MemorySize::Unknown,// Frinear
	MemorySize::Unknown,// Tdcall
	MemorySize::Unknown,// Seamret
	MemorySize::Unknown,// Seamops
	MemorySize::Unknown,// Seamcall
	MemorySize::KLHandleAes128,// Aesencwide128kl_m384
	MemorySize::KLHandleAes128,// Aesdecwide128kl_m384
	MemorySize::KLHandleAes256,// Aesencwide256kl_m512
	MemorySize::KLHandleAes256,// Aesdecwide256kl_m512
	MemorySize::Unknown,// Loadiwkey_xmm_xmm
	MemorySize::KLHandleAes128,// Aesenc128kl_xmm_m384
	MemorySize::KLHandleAes128,// Aesdec128kl_xmm_m384
	MemorySize::KLHandleAes256,// Aesenc256kl_xmm_m512
	MemorySize::KLHandleAes256,// Aesdec256kl_xmm_m512
	MemorySize::Unknown,// Encodekey128_r32_r32
	MemorySize::Unknown,// Encodekey256_r32_r32
	MemorySize::Unknown,// VEX_Vbroadcastss_xmm_xmm
	MemorySize::Unknown,// VEX_Vbroadcastss_ymm_xmm
	MemorySize::Unknown,// VEX_Vbroadcastsd_ymm_xmm
	MemorySize::Unknown,// Vmgexit_F2
	MemorySize::Unknown,// Uiret
	MemorySize::Unknown,// Testui
	MemorySize::Unknown,// Clui
	MemorySize::Unknown,// Stui
	MemorySize::Unknown,// Senduipi_r64
	MemorySize::Unknown,// Hreset_imm8
	MemorySize::Packed128_Int8,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	MemorySize::Unknown,// Ccs_hash_16
	MemorySize::Unknown,// Ccs_hash_32
	MemorySize::Unknown,// Ccs_hash_64
	MemorySize::Unknown,// Ccs_encrypt_16
	MemorySize::Unknown,// Ccs_encrypt_32
	MemorySize::Unknown,// Ccs_encrypt_64
	MemorySize::UInt16,// Lkgs_rm16
	MemorySize::UInt16,// Lkgs_r32m16
	MemorySize::UInt16,// Lkgs_r64m16
	MemorySize::Unknown,// Eretu
	MemorySize::Unknown,// Erets
	MemorySize::Packed128_Float16,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	MemorySize::Packed256_Float16,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	MemorySize::Packed512_Float16,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	MemorySize::Float16,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	MemorySize::Float16,// EVEX_Vcomish_xmm_xmmm16_sae
	MemorySize::Packed128_Int32,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Int32,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	MemorySize::Packed512_Int32,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	MemorySize::Packed128_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	MemorySize::Packed32_Float16,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	MemorySize::Packed32_Float16,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	MemorySize::Packed32_Float16,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	MemorySize::Packed64_Float16,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	MemorySize::Packed128_Float32,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	MemorySize::Packed256_Float32,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	MemorySize::Packed512_Float32,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	MemorySize::Packed128_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	MemorySize::Packed256_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	MemorySize::Packed512_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	MemorySize::Float64,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	MemorySize::Float16,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Float16,// EVEX_Vcvtsh2si_r32_xmmm16_er
	MemorySize::Float16,// EVEX_Vcvtsh2si_r64_xmmm16_er
	MemorySize::Float16,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Float16,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	MemorySize::Float16,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	MemorySize::Int32,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	MemorySize::Int64,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	MemorySize::Float32,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Packed64_Float16,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	MemorySize::Packed32_Float16,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	MemorySize::Packed64_Float16,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	MemorySize::Packed64_Float16,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	MemorySize::Packed32_Float16,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	MemorySize::Packed64_Float16,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	MemorySize::Packed128_Float16,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	MemorySize::Float16,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	MemorySize::Float16,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	MemorySize::Float16,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	MemorySize::Float16,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	MemorySize::Packed128_UInt32,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	MemorySize::Packed256_UInt32,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	MemorySize::Packed512_UInt32,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	MemorySize::Packed128_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	MemorySize::Packed256_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	MemorySize::Packed512_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	MemorySize::UInt32,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	MemorySize::UInt64,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	MemorySize::Packed128_UInt16,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_UInt16,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_UInt16,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	MemorySize::Packed128_Int16,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Int16,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Int16,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_2xFloat16,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_2xFloat16,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_2xFloat16,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_2xFloat16,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_2xFloat16,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_2xFloat16,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed32_Float16,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Packed32_Float16,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Packed128_2xFloat16,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_2xFloat16,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_2xFloat16,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed128_2xFloat16,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Packed256_2xFloat16,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Packed512_2xFloat16,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Packed32_Float16,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Packed32_Float16,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Packed128_Float16,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Packed128_Float16,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	MemorySize::Packed256_Float16,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	MemorySize::Packed512_Float16,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	MemorySize::Float16,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	MemorySize::Packed128_Float16,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	MemorySize::Float16,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Packed128_Float16,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Packed256_Float16,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Packed512_Float16,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Float16,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Packed128_Float16,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	MemorySize::Float16,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Packed128_Float16,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	MemorySize::Float16,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Float16,// EVEX_Vmovsh_xmm_k1z_m16
	MemorySize::Float16,// EVEX_Vmovsh_m16_k1_xmm
	MemorySize::Unknown,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	MemorySize::UInt16,// EVEX_Vmovw_xmm_r32m16
	MemorySize::UInt16,// EVEX_Vmovw_xmm_r64m16
	MemorySize::UInt16,// EVEX_Vmovw_r32m16_xmm
	MemorySize::UInt16,// EVEX_Vmovw_r64m16_xmm
	MemorySize::Packed128_Float16,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	MemorySize::Float16,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	MemorySize::Packed128_Float16,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Packed256_Float16,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Packed512_Float16,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Float16,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Packed128_Float16,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Packed256_Float16,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Packed512_Float16,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Float16,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Packed128_Float16,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	MemorySize::Float16,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	MemorySize::Packed128_Float16,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Packed128_Float16,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Packed256_Float16,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Packed512_Float16,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Float16,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Float16,// EVEX_Vucomish_xmm_xmmm16_sae
	MemorySize::Unknown,// Rdudbg
	MemorySize::Unknown,// Wrudbg
	MemorySize::Unknown,// VEX_KNC_Jkzd_kr_rel8_64
	MemorySize::Unknown,// VEX_KNC_Jknzd_kr_rel8_64
	MemorySize::UInt8,// VEX_KNC_Vprefetchnta_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetch0_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetch1_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetch2_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetchenta_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetche0_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetche1_m8
	MemorySize::UInt8,// VEX_KNC_Vprefetche2_m8
	MemorySize::Unknown,// VEX_KNC_Kand_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kandn_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kandnr_kr_kr
	MemorySize::Unknown,// VEX_KNC_Knot_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kxnor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kxor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmerge2l1h_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmerge2l1l_kr_kr
	MemorySize::Unknown,// VEX_KNC_Jkzd_kr_rel32_64
	MemorySize::Unknown,// VEX_KNC_Jknzd_kr_rel32_64
	MemorySize::Unknown,// VEX_KNC_Kmov_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmov_kr_r32
	MemorySize::Unknown,// VEX_KNC_Kmov_r32_kr
	MemorySize::Unknown,// VEX_KNC_Kconcath_r64_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kconcatl_r64_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kortest_kr_kr
	MemorySize::Unknown,// VEX_KNC_Delay_r32
	MemorySize::Unknown,// VEX_KNC_Delay_r64
	MemorySize::Unknown,// VEX_KNC_Spflt_r32
	MemorySize::Unknown,// VEX_KNC_Spflt_r64
	MemorySize::UInt8,// VEX_KNC_Clevict1_m8
	MemorySize::UInt8,// VEX_KNC_Clevict0_m8
	MemorySize::Unknown,// VEX_KNC_Popcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Popcnt_r64_r64
	MemorySize::Unknown,// VEX_KNC_Tzcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Tzcnt_r64_r64
	MemorySize::Unknown,// VEX_KNC_Tzcnti_r32_r32
	MemorySize::Unknown,// VEX_KNC_Tzcnti_r64_r64
	MemorySize::Unknown,// VEX_KNC_Lzcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Lzcnt_r64_r64
	MemorySize::UInt32,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	MemorySize::UInt64,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	MemorySize::UInt32,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	MemorySize::UInt64,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	MemorySize::UInt32,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	MemorySize::UInt64,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	MemorySize::Unknown,// VEX_KNC_Kextract_kr_r64_imm8
	MemorySize::Unknown,// MVEX_Vprefetchnta_m
	MemorySize::Unknown,// MVEX_Vprefetch0_m
	MemorySize::Unknown,// MVEX_Vprefetch1_m
	MemorySize::Unknown,// MVEX_Vprefetch2_m
	MemorySize::Unknown,// MVEX_Vprefetchenta_m
	MemorySize::Unknown,// MVEX_Vprefetche0_m
	MemorySize::Unknown,// MVEX_Vprefetche1_m
	MemorySize::Unknown,// MVEX_Vprefetche2_m
	MemorySize::Unknown,// MVEX_Vmovaps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovapd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovaps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovapd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrapd_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrngoapd_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnraps_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrngoaps_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa32_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovdqa64_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Clevict1_m
	MemorySize::Unknown,// MVEX_Clevict0_m
	MemorySize::Unknown,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vbroadcastss_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastsd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgetexpps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vgetexppd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	MemorySize::Unknown,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	MemorySize::Unknown,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	MemorySize::Unknown,// MVEX_Vpbroadcastd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpbroadcastq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	MemorySize::Unknown,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	MemorySize::Unknown,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpgatherdd_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vpgatherdq_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vgatherdps_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vgatherdpd_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	MemorySize::Unknown,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpscatterdd_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpscatterdq_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vscatterdps_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vscatterdpd_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	MemorySize::Unknown,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	MemorySize::Unknown,// MVEX_Vgatherpf0hintdps_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf0hintdpd_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf0dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf1dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0hintdps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0hintdpd_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf1dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vexp223ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vlog2ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	MemorySize::Unknown,// MVEX_Vloadunpackld_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpacklq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstoreld_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorelq_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vloadunpacklps_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpacklpd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorelps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorelpd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	MemorySize::Unknown,// MVEX_Vloadunpackhd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpackhq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorehd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorehq_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vloadunpackhps_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpackhpd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorehps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorehpd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	MemorySize::Unknown,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	MemorySize::Unknown,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// Via_undoc_F30FA6F0_16
	MemorySize::Unknown,// Via_undoc_F30FA6F0_32
	MemorySize::Unknown,// Via_undoc_F30FA6F0_64
	MemorySize::Unknown,// Via_undoc_F30FA6F8_16
	MemorySize::Unknown,// Via_undoc_F30FA6F8_32
	MemorySize::Unknown,// Via_undoc_F30FA6F8_64
	MemorySize::Unknown,// Xsha512_16
	MemorySize::Unknown,// Xsha512_32
	MemorySize::Unknown,// Xsha512_64
	MemorySize::Unknown,// Xstore_alt_16
	MemorySize::Unknown,// Xstore_alt_32
	MemorySize::Unknown,// Xstore_alt_64
	MemorySize::Unknown,// Xsha512_alt_16
	MemorySize::Unknown,// Xsha512_alt_32
	MemorySize::Unknown,// Xsha512_alt_64
	MemorySize::Unknown,// Zero_bytes
	MemorySize::Unknown,// Wrmsrns
	MemorySize::Unknown,// Wrmsrlist
	MemorySize::Unknown,// Rdmsrlist
	MemorySize::Unknown,// Rmpquery
	MemorySize::UInt8,// Prefetchit1_m8
	MemorySize::UInt8,// Prefetchit0_m8
	MemorySize::UInt32,// Aadd_m32_r32
	MemorySize::UInt64,// Aadd_m64_r64
	MemorySize::UInt32,// Aand_m32_r32
	MemorySize::UInt64,// Aand_m64_r64
	MemorySize::UInt32,// Axor_m32_r32
	MemorySize::UInt64,// Axor_m64_r64
	MemorySize::UInt32,// Aor_m32_r32
	MemorySize::UInt64,// Aor_m64_r64
	MemorySize::Packed128_UInt8,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt8,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt8,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	MemorySize::Packed128_Int8,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	MemorySize::Packed256_Int8,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Tdpfp16ps_tmm_tmm_tmm
	MemorySize::Packed128_Float32,// VEX_Vcvtneps2bf16_xmm_xmmm128
	MemorySize::Packed256_Float32,// VEX_Vcvtneps2bf16_xmm_ymmm256
	MemorySize::Packed128_Float16,// VEX_Vcvtneoph2ps_xmm_m128
	MemorySize::Packed256_Float16,// VEX_Vcvtneoph2ps_ymm_m256
	MemorySize::Packed128_Float16,// VEX_Vcvtneeph2ps_xmm_m128
	MemorySize::Packed256_Float16,// VEX_Vcvtneeph2ps_ymm_m256
	MemorySize::Packed128_BFloat16,// VEX_Vcvtneebf162ps_xmm_m128
	MemorySize::Packed256_BFloat16,// VEX_Vcvtneebf162ps_ymm_m256
	MemorySize::Packed128_BFloat16,// VEX_Vcvtneobf162ps_xmm_m128
	MemorySize::Packed256_BFloat16,// VEX_Vcvtneobf162ps_ymm_m256
	MemorySize::Float16,// VEX_Vbcstnesh2ps_xmm_m16
	MemorySize::Float16,// VEX_Vbcstnesh2ps_ymm_m16
	MemorySize::BFloat16,// VEX_Vbcstnebf162ps_xmm_m16
	MemorySize::BFloat16,// VEX_Vbcstnebf162ps_ymm_m16
	MemorySize::Packed128_UInt52,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt52,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt52,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt52,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	MemorySize::UInt32,// VEX_Cmpoxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpoxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnoxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnoxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpbxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpbxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnbxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnbxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpzxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpzxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnzxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnzxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpbexadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpbexadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnbexadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnbexadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpsxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpsxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnsxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnsxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmppxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmppxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnpxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnpxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmplxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmplxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnlxadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnlxadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmplexadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmplexadd_m64_r64_r64
	MemorySize::UInt32,// VEX_Cmpnlexadd_m32_r32_r32
	MemorySize::UInt64,// VEX_Cmpnlexadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	MemorySize::Unknown,// Pbndkb
	MemorySize::Unknown,// VEX_Vsha512rnds2_ymm_ymm_xmm
	MemorySize::Unknown,// VEX_Vsha512msg1_ymm_xmm
	MemorySize::Unknown,// VEX_Vsha512msg2_ymm_ymm
	MemorySize::Packed128_UInt16,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt16,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt16,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	MemorySize::Packed128_Int16,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	MemorySize::Packed256_Int16,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	MemorySize::Packed128_UInt32,// VEX_Vsm4key4_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vsm4key4_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	MemorySize::Packed256_UInt32,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	MemorySize::Packed128_UInt32,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];

#[rustfmt::skip]
pub(super) static SIZES_BCST: [MemorySize; IcedConstants::CODE_ENUM_COUNT] = [
	MemorySize::Unknown,// INVALID
	MemorySize::Unknown,// DeclareByte
	MemorySize::Unknown,// DeclareWord
	MemorySize::Unknown,// DeclareDword
	MemorySize::Unknown,// DeclareQword
	MemorySize::Unknown,// Add_rm8_r8
	MemorySize::Unknown,// Add_rm16_r16
	MemorySize::Unknown,// Add_rm32_r32
	MemorySize::Unknown,// Add_rm64_r64
	MemorySize::Unknown,// Add_r8_rm8
	MemorySize::Unknown,// Add_r16_rm16
	MemorySize::Unknown,// Add_r32_rm32
	MemorySize::Unknown,// Add_r64_rm64
	MemorySize::Unknown,// Add_AL_imm8
	MemorySize::Unknown,// Add_AX_imm16
	MemorySize::Unknown,// Add_EAX_imm32
	MemorySize::Unknown,// Add_RAX_imm32
	MemorySize::Unknown,// Pushw_ES
	MemorySize::Unknown,// Pushd_ES
	MemorySize::Unknown,// Popw_ES
	MemorySize::Unknown,// Popd_ES
	MemorySize::Unknown,// Or_rm8_r8
	MemorySize::Unknown,// Or_rm16_r16
	MemorySize::Unknown,// Or_rm32_r32
	MemorySize::Unknown,// Or_rm64_r64
	MemorySize::Unknown,// Or_r8_rm8
	MemorySize::Unknown,// Or_r16_rm16
	MemorySize::Unknown,// Or_r32_rm32
	MemorySize::Unknown,// Or_r64_rm64
	MemorySize::Unknown,// Or_AL_imm8
	MemorySize::Unknown,// Or_AX_imm16
	MemorySize::Unknown,// Or_EAX_imm32
	MemorySize::Unknown,// Or_RAX_imm32
	MemorySize::Unknown,// Pushw_CS
	MemorySize::Unknown,// Pushd_CS
	MemorySize::Unknown,// Popw_CS
	MemorySize::Unknown,// Adc_rm8_r8
	MemorySize::Unknown,// Adc_rm16_r16
	MemorySize::Unknown,// Adc_rm32_r32
	MemorySize::Unknown,// Adc_rm64_r64
	MemorySize::Unknown,// Adc_r8_rm8
	MemorySize::Unknown,// Adc_r16_rm16
	MemorySize::Unknown,// Adc_r32_rm32
	MemorySize::Unknown,// Adc_r64_rm64
	MemorySize::Unknown,// Adc_AL_imm8
	MemorySize::Unknown,// Adc_AX_imm16
	MemorySize::Unknown,// Adc_EAX_imm32
	MemorySize::Unknown,// Adc_RAX_imm32
	MemorySize::Unknown,// Pushw_SS
	MemorySize::Unknown,// Pushd_SS
	MemorySize::Unknown,// Popw_SS
	MemorySize::Unknown,// Popd_SS
	MemorySize::Unknown,// Sbb_rm8_r8
	MemorySize::Unknown,// Sbb_rm16_r16
	MemorySize::Unknown,// Sbb_rm32_r32
	MemorySize::Unknown,// Sbb_rm64_r64
	MemorySize::Unknown,// Sbb_r8_rm8
	MemorySize::Unknown,// Sbb_r16_rm16
	MemorySize::Unknown,// Sbb_r32_rm32
	MemorySize::Unknown,// Sbb_r64_rm64
	MemorySize::Unknown,// Sbb_AL_imm8
	MemorySize::Unknown,// Sbb_AX_imm16
	MemorySize::Unknown,// Sbb_EAX_imm32
	MemorySize::Unknown,// Sbb_RAX_imm32
	MemorySize::Unknown,// Pushw_DS
	MemorySize::Unknown,// Pushd_DS
	MemorySize::Unknown,// Popw_DS
	MemorySize::Unknown,// Popd_DS
	MemorySize::Unknown,// And_rm8_r8
	MemorySize::Unknown,// And_rm16_r16
	MemorySize::Unknown,// And_rm32_r32
	MemorySize::Unknown,// And_rm64_r64
	MemorySize::Unknown,// And_r8_rm8
	MemorySize::Unknown,// And_r16_rm16
	MemorySize::Unknown,// And_r32_rm32
	MemorySize::Unknown,// And_r64_rm64
	MemorySize::Unknown,// And_AL_imm8
	MemorySize::Unknown,// And_AX_imm16
	MemorySize::Unknown,// And_EAX_imm32
	MemorySize::Unknown,// And_RAX_imm32
	MemorySize::Unknown,// Daa
	MemorySize::Unknown,// Sub_rm8_r8
	MemorySize::Unknown,// Sub_rm16_r16
	MemorySize::Unknown,// Sub_rm32_r32
	MemorySize::Unknown,// Sub_rm64_r64
	MemorySize::Unknown,// Sub_r8_rm8
	MemorySize::Unknown,// Sub_r16_rm16
	MemorySize::Unknown,// Sub_r32_rm32
	MemorySize::Unknown,// Sub_r64_rm64
	MemorySize::Unknown,// Sub_AL_imm8
	MemorySize::Unknown,// Sub_AX_imm16
	MemorySize::Unknown,// Sub_EAX_imm32
	MemorySize::Unknown,// Sub_RAX_imm32
	MemorySize::Unknown,// Das
	MemorySize::Unknown,// Xor_rm8_r8
	MemorySize::Unknown,// Xor_rm16_r16
	MemorySize::Unknown,// Xor_rm32_r32
	MemorySize::Unknown,// Xor_rm64_r64
	MemorySize::Unknown,// Xor_r8_rm8
	MemorySize::Unknown,// Xor_r16_rm16
	MemorySize::Unknown,// Xor_r32_rm32
	MemorySize::Unknown,// Xor_r64_rm64
	MemorySize::Unknown,// Xor_AL_imm8
	MemorySize::Unknown,// Xor_AX_imm16
	MemorySize::Unknown,// Xor_EAX_imm32
	MemorySize::Unknown,// Xor_RAX_imm32
	MemorySize::Unknown,// Aaa
	MemorySize::Unknown,// Cmp_rm8_r8
	MemorySize::Unknown,// Cmp_rm16_r16
	MemorySize::Unknown,// Cmp_rm32_r32
	MemorySize::Unknown,// Cmp_rm64_r64
	MemorySize::Unknown,// Cmp_r8_rm8
	MemorySize::Unknown,// Cmp_r16_rm16
	MemorySize::Unknown,// Cmp_r32_rm32
	MemorySize::Unknown,// Cmp_r64_rm64
	MemorySize::Unknown,// Cmp_AL_imm8
	MemorySize::Unknown,// Cmp_AX_imm16
	MemorySize::Unknown,// Cmp_EAX_imm32
	MemorySize::Unknown,// Cmp_RAX_imm32
	MemorySize::Unknown,// Aas
	MemorySize::Unknown,// Inc_r16
	MemorySize::Unknown,// Inc_r32
	MemorySize::Unknown,// Dec_r16
	MemorySize::Unknown,// Dec_r32
	MemorySize::Unknown,// Push_r16
	MemorySize::Unknown,// Push_r32
	MemorySize::Unknown,// Push_r64
	MemorySize::Unknown,// Pop_r16
	MemorySize::Unknown,// Pop_r32
	MemorySize::Unknown,// Pop_r64
	MemorySize::Unknown,// Pushaw
	MemorySize::Unknown,// Pushad
	MemorySize::Unknown,// Popaw
	MemorySize::Unknown,// Popad
	MemorySize::Unknown,// Bound_r16_m1616
	MemorySize::Unknown,// Bound_r32_m3232
	MemorySize::Unknown,// Arpl_rm16_r16
	MemorySize::Unknown,// Arpl_r32m16_r32
	MemorySize::Unknown,// Movsxd_r16_rm16
	MemorySize::Unknown,// Movsxd_r32_rm32
	MemorySize::Unknown,// Movsxd_r64_rm32
	MemorySize::Unknown,// Push_imm16
	MemorySize::Unknown,// Pushd_imm32
	MemorySize::Unknown,// Pushq_imm32
	MemorySize::Unknown,// Imul_r16_rm16_imm16
	MemorySize::Unknown,// Imul_r32_rm32_imm32
	MemorySize::Unknown,// Imul_r64_rm64_imm32
	MemorySize::Unknown,// Pushw_imm8
	MemorySize::Unknown,// Pushd_imm8
	MemorySize::Unknown,// Pushq_imm8
	MemorySize::Unknown,// Imul_r16_rm16_imm8
	MemorySize::Unknown,// Imul_r32_rm32_imm8
	MemorySize::Unknown,// Imul_r64_rm64_imm8
	MemorySize::Unknown,// Insb_m8_DX
	MemorySize::Unknown,// Insw_m16_DX
	MemorySize::Unknown,// Insd_m32_DX
	MemorySize::Unknown,// Outsb_DX_m8
	MemorySize::Unknown,// Outsw_DX_m16
	MemorySize::Unknown,// Outsd_DX_m32
	MemorySize::Unknown,// Jo_rel8_16
	MemorySize::Unknown,// Jo_rel8_32
	MemorySize::Unknown,// Jo_rel8_64
	MemorySize::Unknown,// Jno_rel8_16
	MemorySize::Unknown,// Jno_rel8_32
	MemorySize::Unknown,// Jno_rel8_64
	MemorySize::Unknown,// Jb_rel8_16
	MemorySize::Unknown,// Jb_rel8_32
	MemorySize::Unknown,// Jb_rel8_64
	MemorySize::Unknown,// Jae_rel8_16
	MemorySize::Unknown,// Jae_rel8_32
	MemorySize::Unknown,// Jae_rel8_64
	MemorySize::Unknown,// Je_rel8_16
	MemorySize::Unknown,// Je_rel8_32
	MemorySize::Unknown,// Je_rel8_64
	MemorySize::Unknown,// Jne_rel8_16
	MemorySize::Unknown,// Jne_rel8_32
	MemorySize::Unknown,// Jne_rel8_64
	MemorySize::Unknown,// Jbe_rel8_16
	MemorySize::Unknown,// Jbe_rel8_32
	MemorySize::Unknown,// Jbe_rel8_64
	MemorySize::Unknown,// Ja_rel8_16
	MemorySize::Unknown,// Ja_rel8_32
	MemorySize::Unknown,// Ja_rel8_64
	MemorySize::Unknown,// Js_rel8_16
	MemorySize::Unknown,// Js_rel8_32
	MemorySize::Unknown,// Js_rel8_64
	MemorySize::Unknown,// Jns_rel8_16
	MemorySize::Unknown,// Jns_rel8_32
	MemorySize::Unknown,// Jns_rel8_64
	MemorySize::Unknown,// Jp_rel8_16
	MemorySize::Unknown,// Jp_rel8_32
	MemorySize::Unknown,// Jp_rel8_64
	MemorySize::Unknown,// Jnp_rel8_16
	MemorySize::Unknown,// Jnp_rel8_32
	MemorySize::Unknown,// Jnp_rel8_64
	MemorySize::Unknown,// Jl_rel8_16
	MemorySize::Unknown,// Jl_rel8_32
	MemorySize::Unknown,// Jl_rel8_64
	MemorySize::Unknown,// Jge_rel8_16
	MemorySize::Unknown,// Jge_rel8_32
	MemorySize::Unknown,// Jge_rel8_64
	MemorySize::Unknown,// Jle_rel8_16
	MemorySize::Unknown,// Jle_rel8_32
	MemorySize::Unknown,// Jle_rel8_64
	MemorySize::Unknown,// Jg_rel8_16
	MemorySize::Unknown,// Jg_rel8_32
	MemorySize::Unknown,// Jg_rel8_64
	MemorySize::Unknown,// Add_rm8_imm8
	MemorySize::Unknown,// Or_rm8_imm8
	MemorySize::Unknown,// Adc_rm8_imm8
	MemorySize::Unknown,// Sbb_rm8_imm8
	MemorySize::Unknown,// And_rm8_imm8
	MemorySize::Unknown,// Sub_rm8_imm8
	MemorySize::Unknown,// Xor_rm8_imm8
	MemorySize::Unknown,// Cmp_rm8_imm8
	MemorySize::Unknown,// Add_rm16_imm16
	MemorySize::Unknown,// Add_rm32_imm32
	MemorySize::Unknown,// Add_rm64_imm32
	MemorySize::Unknown,// Or_rm16_imm16
	MemorySize::Unknown,// Or_rm32_imm32
	MemorySize::Unknown,// Or_rm64_imm32
	MemorySize::Unknown,// Adc_rm16_imm16
	MemorySize::Unknown,// Adc_rm32_imm32
	MemorySize::Unknown,// Adc_rm64_imm32
	MemorySize::Unknown,// Sbb_rm16_imm16
	MemorySize::Unknown,// Sbb_rm32_imm32
	MemorySize::Unknown,// Sbb_rm64_imm32
	MemorySize::Unknown,// And_rm16_imm16
	MemorySize::Unknown,// And_rm32_imm32
	MemorySize::Unknown,// And_rm64_imm32
	MemorySize::Unknown,// Sub_rm16_imm16
	MemorySize::Unknown,// Sub_rm32_imm32
	MemorySize::Unknown,// Sub_rm64_imm32
	MemorySize::Unknown,// Xor_rm16_imm16
	MemorySize::Unknown,// Xor_rm32_imm32
	MemorySize::Unknown,// Xor_rm64_imm32
	MemorySize::Unknown,// Cmp_rm16_imm16
	MemorySize::Unknown,// Cmp_rm32_imm32
	MemorySize::Unknown,// Cmp_rm64_imm32
	MemorySize::Unknown,// Add_rm8_imm8_82
	MemorySize::Unknown,// Or_rm8_imm8_82
	MemorySize::Unknown,// Adc_rm8_imm8_82
	MemorySize::Unknown,// Sbb_rm8_imm8_82
	MemorySize::Unknown,// And_rm8_imm8_82
	MemorySize::Unknown,// Sub_rm8_imm8_82
	MemorySize::Unknown,// Xor_rm8_imm8_82
	MemorySize::Unknown,// Cmp_rm8_imm8_82
	MemorySize::Unknown,// Add_rm16_imm8
	MemorySize::Unknown,// Add_rm32_imm8
	MemorySize::Unknown,// Add_rm64_imm8
	MemorySize::Unknown,// Or_rm16_imm8
	MemorySize::Unknown,// Or_rm32_imm8
	MemorySize::Unknown,// Or_rm64_imm8
	MemorySize::Unknown,// Adc_rm16_imm8
	MemorySize::Unknown,// Adc_rm32_imm8
	MemorySize::Unknown,// Adc_rm64_imm8
	MemorySize::Unknown,// Sbb_rm16_imm8
	MemorySize::Unknown,// Sbb_rm32_imm8
	MemorySize::Unknown,// Sbb_rm64_imm8
	MemorySize::Unknown,// And_rm16_imm8
	MemorySize::Unknown,// And_rm32_imm8
	MemorySize::Unknown,// And_rm64_imm8
	MemorySize::Unknown,// Sub_rm16_imm8
	MemorySize::Unknown,// Sub_rm32_imm8
	MemorySize::Unknown,// Sub_rm64_imm8
	MemorySize::Unknown,// Xor_rm16_imm8
	MemorySize::Unknown,// Xor_rm32_imm8
	MemorySize::Unknown,// Xor_rm64_imm8
	MemorySize::Unknown,// Cmp_rm16_imm8
	MemorySize::Unknown,// Cmp_rm32_imm8
	MemorySize::Unknown,// Cmp_rm64_imm8
	MemorySize::Unknown,// Test_rm8_r8
	MemorySize::Unknown,// Test_rm16_r16
	MemorySize::Unknown,// Test_rm32_r32
	MemorySize::Unknown,// Test_rm64_r64
	MemorySize::Unknown,// Xchg_rm8_r8
	MemorySize::Unknown,// Xchg_rm16_r16
	MemorySize::Unknown,// Xchg_rm32_r32
	MemorySize::Unknown,// Xchg_rm64_r64
	MemorySize::Unknown,// Mov_rm8_r8
	MemorySize::Unknown,// Mov_rm16_r16
	MemorySize::Unknown,// Mov_rm32_r32
	MemorySize::Unknown,// Mov_rm64_r64
	MemorySize::Unknown,// Mov_r8_rm8
	MemorySize::Unknown,// Mov_r16_rm16
	MemorySize::Unknown,// Mov_r32_rm32
	MemorySize::Unknown,// Mov_r64_rm64
	MemorySize::Unknown,// Mov_rm16_Sreg
	MemorySize::Unknown,// Mov_r32m16_Sreg
	MemorySize::Unknown,// Mov_r64m16_Sreg
	MemorySize::Unknown,// Lea_r16_m
	MemorySize::Unknown,// Lea_r32_m
	MemorySize::Unknown,// Lea_r64_m
	MemorySize::Unknown,// Mov_Sreg_rm16
	MemorySize::Unknown,// Mov_Sreg_r32m16
	MemorySize::Unknown,// Mov_Sreg_r64m16
	MemorySize::Unknown,// Pop_rm16
	MemorySize::Unknown,// Pop_rm32
	MemorySize::Unknown,// Pop_rm64
	MemorySize::Unknown,// Nopw
	MemorySize::Unknown,// Nopd
	MemorySize::Unknown,// Nopq
	MemorySize::Unknown,// Xchg_r16_AX
	MemorySize::Unknown,// Xchg_r32_EAX
	MemorySize::Unknown,// Xchg_r64_RAX
	MemorySize::Unknown,// Pause
	MemorySize::Unknown,// Cbw
	MemorySize::Unknown,// Cwde
	MemorySize::Unknown,// Cdqe
	MemorySize::Unknown,// Cwd
	MemorySize::Unknown,// Cdq
	MemorySize::Unknown,// Cqo
	MemorySize::Unknown,// Call_ptr1616
	MemorySize::Unknown,// Call_ptr1632
	MemorySize::Unknown,// Wait
	MemorySize::Unknown,// Pushfw
	MemorySize::Unknown,// Pushfd
	MemorySize::Unknown,// Pushfq
	MemorySize::Unknown,// Popfw
	MemorySize::Unknown,// Popfd
	MemorySize::Unknown,// Popfq
	MemorySize::Unknown,// Sahf
	MemorySize::Unknown,// Lahf
	MemorySize::Unknown,// Mov_AL_moffs8
	MemorySize::Unknown,// Mov_AX_moffs16
	MemorySize::Unknown,// Mov_EAX_moffs32
	MemorySize::Unknown,// Mov_RAX_moffs64
	MemorySize::Unknown,// Mov_moffs8_AL
	MemorySize::Unknown,// Mov_moffs16_AX
	MemorySize::Unknown,// Mov_moffs32_EAX
	MemorySize::Unknown,// Mov_moffs64_RAX
	MemorySize::Unknown,// Movsb_m8_m8
	MemorySize::Unknown,// Movsw_m16_m16
	MemorySize::Unknown,// Movsd_m32_m32
	MemorySize::Unknown,// Movsq_m64_m64
	MemorySize::Unknown,// Cmpsb_m8_m8
	MemorySize::Unknown,// Cmpsw_m16_m16
	MemorySize::Unknown,// Cmpsd_m32_m32
	MemorySize::Unknown,// Cmpsq_m64_m64
	MemorySize::Unknown,// Test_AL_imm8
	MemorySize::Unknown,// Test_AX_imm16
	MemorySize::Unknown,// Test_EAX_imm32
	MemorySize::Unknown,// Test_RAX_imm32
	MemorySize::Unknown,// Stosb_m8_AL
	MemorySize::Unknown,// Stosw_m16_AX
	MemorySize::Unknown,// Stosd_m32_EAX
	MemorySize::Unknown,// Stosq_m64_RAX
	MemorySize::Unknown,// Lodsb_AL_m8
	MemorySize::Unknown,// Lodsw_AX_m16
	MemorySize::Unknown,// Lodsd_EAX_m32
	MemorySize::Unknown,// Lodsq_RAX_m64
	MemorySize::Unknown,// Scasb_AL_m8
	MemorySize::Unknown,// Scasw_AX_m16
	MemorySize::Unknown,// Scasd_EAX_m32
	MemorySize::Unknown,// Scasq_RAX_m64
	MemorySize::Unknown,// Mov_r8_imm8
	MemorySize::Unknown,// Mov_r16_imm16
	MemorySize::Unknown,// Mov_r32_imm32
	MemorySize::Unknown,// Mov_r64_imm64
	MemorySize::Unknown,// Rol_rm8_imm8
	MemorySize::Unknown,// Ror_rm8_imm8
	MemorySize::Unknown,// Rcl_rm8_imm8
	MemorySize::Unknown,// Rcr_rm8_imm8
	MemorySize::Unknown,// Shl_rm8_imm8
	MemorySize::Unknown,// Shr_rm8_imm8
	MemorySize::Unknown,// Sal_rm8_imm8
	MemorySize::Unknown,// Sar_rm8_imm8
	MemorySize::Unknown,// Rol_rm16_imm8
	MemorySize::Unknown,// Rol_rm32_imm8
	MemorySize::Unknown,// Rol_rm64_imm8
	MemorySize::Unknown,// Ror_rm16_imm8
	MemorySize::Unknown,// Ror_rm32_imm8
	MemorySize::Unknown,// Ror_rm64_imm8
	MemorySize::Unknown,// Rcl_rm16_imm8
	MemorySize::Unknown,// Rcl_rm32_imm8
	MemorySize::Unknown,// Rcl_rm64_imm8
	MemorySize::Unknown,// Rcr_rm16_imm8
	MemorySize::Unknown,// Rcr_rm32_imm8
	MemorySize::Unknown,// Rcr_rm64_imm8
	MemorySize::Unknown,// Shl_rm16_imm8
	MemorySize::Unknown,// Shl_rm32_imm8
	MemorySize::Unknown,// Shl_rm64_imm8
	MemorySize::Unknown,// Shr_rm16_imm8
	MemorySize::Unknown,// Shr_rm32_imm8
	MemorySize::Unknown,// Shr_rm64_imm8
	MemorySize::Unknown,// Sal_rm16_imm8
	MemorySize::Unknown,// Sal_rm32_imm8
	MemorySize::Unknown,// Sal_rm64_imm8
	MemorySize::Unknown,// Sar_rm16_imm8
	MemorySize::Unknown,// Sar_rm32_imm8
	MemorySize::Unknown,// Sar_rm64_imm8
	MemorySize::Unknown,// Retnw_imm16
	MemorySize::Unknown,// Retnd_imm16
	MemorySize::Unknown,// Retnq_imm16
	MemorySize::Unknown,// Retnw
	MemorySize::Unknown,// Retnd
	MemorySize::Unknown,// Retnq
	MemorySize::Unknown,// Les_r16_m1616
	MemorySize::Unknown,// Les_r32_m1632
	MemorySize::Unknown,// Lds_r16_m1616
	MemorySize::Unknown,// Lds_r32_m1632
	MemorySize::Unknown,// Mov_rm8_imm8
	MemorySize::Unknown,// Xabort_imm8
	MemorySize::Unknown,// Mov_rm16_imm16
	MemorySize::Unknown,// Mov_rm32_imm32
	MemorySize::Unknown,// Mov_rm64_imm32
	MemorySize::Unknown,// Xbegin_rel16
	MemorySize::Unknown,// Xbegin_rel32
	MemorySize::Unknown,// Enterw_imm16_imm8
	MemorySize::Unknown,// Enterd_imm16_imm8
	MemorySize::Unknown,// Enterq_imm16_imm8
	MemorySize::Unknown,// Leavew
	MemorySize::Unknown,// Leaved
	MemorySize::Unknown,// Leaveq
	MemorySize::Unknown,// Retfw_imm16
	MemorySize::Unknown,// Retfd_imm16
	MemorySize::Unknown,// Retfq_imm16
	MemorySize::Unknown,// Retfw
	MemorySize::Unknown,// Retfd
	MemorySize::Unknown,// Retfq
	MemorySize::Unknown,// Int3
	MemorySize::Unknown,// Int_imm8
	MemorySize::Unknown,// Into
	MemorySize::Unknown,// Iretw
	MemorySize::Unknown,// Iretd
	MemorySize::Unknown,// Iretq
	MemorySize::Unknown,// Rol_rm8_1
	MemorySize::Unknown,// Ror_rm8_1
	MemorySize::Unknown,// Rcl_rm8_1
	MemorySize::Unknown,// Rcr_rm8_1
	MemorySize::Unknown,// Shl_rm8_1
	MemorySize::Unknown,// Shr_rm8_1
	MemorySize::Unknown,// Sal_rm8_1
	MemorySize::Unknown,// Sar_rm8_1
	MemorySize::Unknown,// Rol_rm16_1
	MemorySize::Unknown,// Rol_rm32_1
	MemorySize::Unknown,// Rol_rm64_1
	MemorySize::Unknown,// Ror_rm16_1
	MemorySize::Unknown,// Ror_rm32_1
	MemorySize::Unknown,// Ror_rm64_1
	MemorySize::Unknown,// Rcl_rm16_1
	MemorySize::Unknown,// Rcl_rm32_1
	MemorySize::Unknown,// Rcl_rm64_1
	MemorySize::Unknown,// Rcr_rm16_1
	MemorySize::Unknown,// Rcr_rm32_1
	MemorySize::Unknown,// Rcr_rm64_1
	MemorySize::Unknown,// Shl_rm16_1
	MemorySize::Unknown,// Shl_rm32_1
	MemorySize::Unknown,// Shl_rm64_1
	MemorySize::Unknown,// Shr_rm16_1
	MemorySize::Unknown,// Shr_rm32_1
	MemorySize::Unknown,// Shr_rm64_1
	MemorySize::Unknown,// Sal_rm16_1
	MemorySize::Unknown,// Sal_rm32_1
	MemorySize::Unknown,// Sal_rm64_1
	MemorySize::Unknown,// Sar_rm16_1
	MemorySize::Unknown,// Sar_rm32_1
	MemorySize::Unknown,// Sar_rm64_1
	MemorySize::Unknown,// Rol_rm8_CL
	MemorySize::Unknown,// Ror_rm8_CL
	MemorySize::Unknown,// Rcl_rm8_CL
	MemorySize::Unknown,// Rcr_rm8_CL
	MemorySize::Unknown,// Shl_rm8_CL
	MemorySize::Unknown,// Shr_rm8_CL
	MemorySize::Unknown,// Sal_rm8_CL
	MemorySize::Unknown,// Sar_rm8_CL
	MemorySize::Unknown,// Rol_rm16_CL
	MemorySize::Unknown,// Rol_rm32_CL
	MemorySize::Unknown,// Rol_rm64_CL
	MemorySize::Unknown,// Ror_rm16_CL
	MemorySize::Unknown,// Ror_rm32_CL
	MemorySize::Unknown,// Ror_rm64_CL
	MemorySize::Unknown,// Rcl_rm16_CL
	MemorySize::Unknown,// Rcl_rm32_CL
	MemorySize::Unknown,// Rcl_rm64_CL
	MemorySize::Unknown,// Rcr_rm16_CL
	MemorySize::Unknown,// Rcr_rm32_CL
	MemorySize::Unknown,// Rcr_rm64_CL
	MemorySize::Unknown,// Shl_rm16_CL
	MemorySize::Unknown,// Shl_rm32_CL
	MemorySize::Unknown,// Shl_rm64_CL
	MemorySize::Unknown,// Shr_rm16_CL
	MemorySize::Unknown,// Shr_rm32_CL
	MemorySize::Unknown,// Shr_rm64_CL
	MemorySize::Unknown,// Sal_rm16_CL
	MemorySize::Unknown,// Sal_rm32_CL
	MemorySize::Unknown,// Sal_rm64_CL
	MemorySize::Unknown,// Sar_rm16_CL
	MemorySize::Unknown,// Sar_rm32_CL
	MemorySize::Unknown,// Sar_rm64_CL
	MemorySize::Unknown,// Aam_imm8
	MemorySize::Unknown,// Aad_imm8
	MemorySize::Unknown,// Salc
	MemorySize::Unknown,// Xlat_m8
	MemorySize::Unknown,// Fadd_m32fp
	MemorySize::Unknown,// Fmul_m32fp
	MemorySize::Unknown,// Fcom_m32fp
	MemorySize::Unknown,// Fcomp_m32fp
	MemorySize::Unknown,// Fsub_m32fp
	MemorySize::Unknown,// Fsubr_m32fp
	MemorySize::Unknown,// Fdiv_m32fp
	MemorySize::Unknown,// Fdivr_m32fp
	MemorySize::Unknown,// Fadd_st0_sti
	MemorySize::Unknown,// Fmul_st0_sti
	MemorySize::Unknown,// Fcom_st0_sti
	MemorySize::Unknown,// Fcomp_st0_sti
	MemorySize::Unknown,// Fsub_st0_sti
	MemorySize::Unknown,// Fsubr_st0_sti
	MemorySize::Unknown,// Fdiv_st0_sti
	MemorySize::Unknown,// Fdivr_st0_sti
	MemorySize::Unknown,// Fld_m32fp
	MemorySize::Unknown,// Fst_m32fp
	MemorySize::Unknown,// Fstp_m32fp
	MemorySize::Unknown,// Fldenv_m14byte
	MemorySize::Unknown,// Fldenv_m28byte
	MemorySize::Unknown,// Fldcw_m2byte
	MemorySize::Unknown,// Fnstenv_m14byte
	MemorySize::Unknown,// Fstenv_m14byte
	MemorySize::Unknown,// Fnstenv_m28byte
	MemorySize::Unknown,// Fstenv_m28byte
	MemorySize::Unknown,// Fnstcw_m2byte
	MemorySize::Unknown,// Fstcw_m2byte
	MemorySize::Unknown,// Fld_sti
	MemorySize::Unknown,// Fxch_st0_sti
	MemorySize::Unknown,// Fnop
	MemorySize::Unknown,// Fstpnce_sti
	MemorySize::Unknown,// Fchs
	MemorySize::Unknown,// Fabs
	MemorySize::Unknown,// Ftst
	MemorySize::Unknown,// Fxam
	MemorySize::Unknown,// Fld1
	MemorySize::Unknown,// Fldl2t
	MemorySize::Unknown,// Fldl2e
	MemorySize::Unknown,// Fldpi
	MemorySize::Unknown,// Fldlg2
	MemorySize::Unknown,// Fldln2
	MemorySize::Unknown,// Fldz
	MemorySize::Unknown,// F2xm1
	MemorySize::Unknown,// Fyl2x
	MemorySize::Unknown,// Fptan
	MemorySize::Unknown,// Fpatan
	MemorySize::Unknown,// Fxtract
	MemorySize::Unknown,// Fprem1
	MemorySize::Unknown,// Fdecstp
	MemorySize::Unknown,// Fincstp
	MemorySize::Unknown,// Fprem
	MemorySize::Unknown,// Fyl2xp1
	MemorySize::Unknown,// Fsqrt
	MemorySize::Unknown,// Fsincos
	MemorySize::Unknown,// Frndint
	MemorySize::Unknown,// Fscale
	MemorySize::Unknown,// Fsin
	MemorySize::Unknown,// Fcos
	MemorySize::Unknown,// Fiadd_m32int
	MemorySize::Unknown,// Fimul_m32int
	MemorySize::Unknown,// Ficom_m32int
	MemorySize::Unknown,// Ficomp_m32int
	MemorySize::Unknown,// Fisub_m32int
	MemorySize::Unknown,// Fisubr_m32int
	MemorySize::Unknown,// Fidiv_m32int
	MemorySize::Unknown,// Fidivr_m32int
	MemorySize::Unknown,// Fcmovb_st0_sti
	MemorySize::Unknown,// Fcmove_st0_sti
	MemorySize::Unknown,// Fcmovbe_st0_sti
	MemorySize::Unknown,// Fcmovu_st0_sti
	MemorySize::Unknown,// Fucompp
	MemorySize::Unknown,// Fild_m32int
	MemorySize::Unknown,// Fisttp_m32int
	MemorySize::Unknown,// Fist_m32int
	MemorySize::Unknown,// Fistp_m32int
	MemorySize::Unknown,// Fld_m80fp
	MemorySize::Unknown,// Fstp_m80fp
	MemorySize::Unknown,// Fcmovnb_st0_sti
	MemorySize::Unknown,// Fcmovne_st0_sti
	MemorySize::Unknown,// Fcmovnbe_st0_sti
	MemorySize::Unknown,// Fcmovnu_st0_sti
	MemorySize::Unknown,// Fneni
	MemorySize::Unknown,// Feni
	MemorySize::Unknown,// Fndisi
	MemorySize::Unknown,// Fdisi
	MemorySize::Unknown,// Fnclex
	MemorySize::Unknown,// Fclex
	MemorySize::Unknown,// Fninit
	MemorySize::Unknown,// Finit
	MemorySize::Unknown,// Fnsetpm
	MemorySize::Unknown,// Fsetpm
	MemorySize::Unknown,// Frstpm
	MemorySize::Unknown,// Fucomi_st0_sti
	MemorySize::Unknown,// Fcomi_st0_sti
	MemorySize::Unknown,// Fadd_m64fp
	MemorySize::Unknown,// Fmul_m64fp
	MemorySize::Unknown,// Fcom_m64fp
	MemorySize::Unknown,// Fcomp_m64fp
	MemorySize::Unknown,// Fsub_m64fp
	MemorySize::Unknown,// Fsubr_m64fp
	MemorySize::Unknown,// Fdiv_m64fp
	MemorySize::Unknown,// Fdivr_m64fp
	MemorySize::Unknown,// Fadd_sti_st0
	MemorySize::Unknown,// Fmul_sti_st0
	MemorySize::Unknown,// Fcom_st0_sti_DCD0
	MemorySize::Unknown,// Fcomp_st0_sti_DCD8
	MemorySize::Unknown,// Fsubr_sti_st0
	MemorySize::Unknown,// Fsub_sti_st0
	MemorySize::Unknown,// Fdivr_sti_st0
	MemorySize::Unknown,// Fdiv_sti_st0
	MemorySize::Unknown,// Fld_m64fp
	MemorySize::Unknown,// Fisttp_m64int
	MemorySize::Unknown,// Fst_m64fp
	MemorySize::Unknown,// Fstp_m64fp
	MemorySize::Unknown,// Frstor_m94byte
	MemorySize::Unknown,// Frstor_m108byte
	MemorySize::Unknown,// Fnsave_m94byte
	MemorySize::Unknown,// Fsave_m94byte
	MemorySize::Unknown,// Fnsave_m108byte
	MemorySize::Unknown,// Fsave_m108byte
	MemorySize::Unknown,// Fnstsw_m2byte
	MemorySize::Unknown,// Fstsw_m2byte
	MemorySize::Unknown,// Ffree_sti
	MemorySize::Unknown,// Fxch_st0_sti_DDC8
	MemorySize::Unknown,// Fst_sti
	MemorySize::Unknown,// Fstp_sti
	MemorySize::Unknown,// Fucom_st0_sti
	MemorySize::Unknown,// Fucomp_st0_sti
	MemorySize::Unknown,// Fiadd_m16int
	MemorySize::Unknown,// Fimul_m16int
	MemorySize::Unknown,// Ficom_m16int
	MemorySize::Unknown,// Ficomp_m16int
	MemorySize::Unknown,// Fisub_m16int
	MemorySize::Unknown,// Fisubr_m16int
	MemorySize::Unknown,// Fidiv_m16int
	MemorySize::Unknown,// Fidivr_m16int
	MemorySize::Unknown,// Faddp_sti_st0
	MemorySize::Unknown,// Fmulp_sti_st0
	MemorySize::Unknown,// Fcomp_st0_sti_DED0
	MemorySize::Unknown,// Fcompp
	MemorySize::Unknown,// Fsubrp_sti_st0
	MemorySize::Unknown,// Fsubp_sti_st0
	MemorySize::Unknown,// Fdivrp_sti_st0
	MemorySize::Unknown,// Fdivp_sti_st0
	MemorySize::Unknown,// Fild_m16int
	MemorySize::Unknown,// Fisttp_m16int
	MemorySize::Unknown,// Fist_m16int
	MemorySize::Unknown,// Fistp_m16int
	MemorySize::Unknown,// Fbld_m80bcd
	MemorySize::Unknown,// Fild_m64int
	MemorySize::Unknown,// Fbstp_m80bcd
	MemorySize::Unknown,// Fistp_m64int
	MemorySize::Unknown,// Ffreep_sti
	MemorySize::Unknown,// Fxch_st0_sti_DFC8
	MemorySize::Unknown,// Fstp_sti_DFD0
	MemorySize::Unknown,// Fstp_sti_DFD8
	MemorySize::Unknown,// Fnstsw_AX
	MemorySize::Unknown,// Fstsw_AX
	MemorySize::Unknown,// Fstdw_AX
	MemorySize::Unknown,// Fstsg_AX
	MemorySize::Unknown,// Fucomip_st0_sti
	MemorySize::Unknown,// Fcomip_st0_sti
	MemorySize::Unknown,// Loopne_rel8_16_CX
	MemorySize::Unknown,// Loopne_rel8_32_CX
	MemorySize::Unknown,// Loopne_rel8_16_ECX
	MemorySize::Unknown,// Loopne_rel8_32_ECX
	MemorySize::Unknown,// Loopne_rel8_64_ECX
	MemorySize::Unknown,// Loopne_rel8_16_RCX
	MemorySize::Unknown,// Loopne_rel8_64_RCX
	MemorySize::Unknown,// Loope_rel8_16_CX
	MemorySize::Unknown,// Loope_rel8_32_CX
	MemorySize::Unknown,// Loope_rel8_16_ECX
	MemorySize::Unknown,// Loope_rel8_32_ECX
	MemorySize::Unknown,// Loope_rel8_64_ECX
	MemorySize::Unknown,// Loope_rel8_16_RCX
	MemorySize::Unknown,// Loope_rel8_64_RCX
	MemorySize::Unknown,// Loop_rel8_16_CX
	MemorySize::Unknown,// Loop_rel8_32_CX
	MemorySize::Unknown,// Loop_rel8_16_ECX
	MemorySize::Unknown,// Loop_rel8_32_ECX
	MemorySize::Unknown,// Loop_rel8_64_ECX
	MemorySize::Unknown,// Loop_rel8_16_RCX
	MemorySize::Unknown,// Loop_rel8_64_RCX
	MemorySize::Unknown,// Jcxz_rel8_16
	MemorySize::Unknown,// Jcxz_rel8_32
	MemorySize::Unknown,// Jecxz_rel8_16
	MemorySize::Unknown,// Jecxz_rel8_32
	MemorySize::Unknown,// Jecxz_rel8_64
	MemorySize::Unknown,// Jrcxz_rel8_16
	MemorySize::Unknown,// Jrcxz_rel8_64
	MemorySize::Unknown,// In_AL_imm8
	MemorySize::Unknown,// In_AX_imm8
	MemorySize::Unknown,// In_EAX_imm8
	MemorySize::Unknown,// Out_imm8_AL
	MemorySize::Unknown,// Out_imm8_AX
	MemorySize::Unknown,// Out_imm8_EAX
	MemorySize::Unknown,// Call_rel16
	MemorySize::Unknown,// Call_rel32_32
	MemorySize::Unknown,// Call_rel32_64
	MemorySize::Unknown,// Jmp_rel16
	MemorySize::Unknown,// Jmp_rel32_32
	MemorySize::Unknown,// Jmp_rel32_64
	MemorySize::Unknown,// Jmp_ptr1616
	MemorySize::Unknown,// Jmp_ptr1632
	MemorySize::Unknown,// Jmp_rel8_16
	MemorySize::Unknown,// Jmp_rel8_32
	MemorySize::Unknown,// Jmp_rel8_64
	MemorySize::Unknown,// In_AL_DX
	MemorySize::Unknown,// In_AX_DX
	MemorySize::Unknown,// In_EAX_DX
	MemorySize::Unknown,// Out_DX_AL
	MemorySize::Unknown,// Out_DX_AX
	MemorySize::Unknown,// Out_DX_EAX
	MemorySize::Unknown,// Int1
	MemorySize::Unknown,// Hlt
	MemorySize::Unknown,// Cmc
	MemorySize::Unknown,// Test_rm8_imm8
	MemorySize::Unknown,// Test_rm8_imm8_F6r1
	MemorySize::Unknown,// Not_rm8
	MemorySize::Unknown,// Neg_rm8
	MemorySize::Unknown,// Mul_rm8
	MemorySize::Unknown,// Imul_rm8
	MemorySize::Unknown,// Div_rm8
	MemorySize::Unknown,// Idiv_rm8
	MemorySize::Unknown,// Test_rm16_imm16
	MemorySize::Unknown,// Test_rm32_imm32
	MemorySize::Unknown,// Test_rm64_imm32
	MemorySize::Unknown,// Test_rm16_imm16_F7r1
	MemorySize::Unknown,// Test_rm32_imm32_F7r1
	MemorySize::Unknown,// Test_rm64_imm32_F7r1
	MemorySize::Unknown,// Not_rm16
	MemorySize::Unknown,// Not_rm32
	MemorySize::Unknown,// Not_rm64
	MemorySize::Unknown,// Neg_rm16
	MemorySize::Unknown,// Neg_rm32
	MemorySize::Unknown,// Neg_rm64
	MemorySize::Unknown,// Mul_rm16
	MemorySize::Unknown,// Mul_rm32
	MemorySize::Unknown,// Mul_rm64
	MemorySize::Unknown,// Imul_rm16
	MemorySize::Unknown,// Imul_rm32
	MemorySize::Unknown,// Imul_rm64
	MemorySize::Unknown,// Div_rm16
	MemorySize::Unknown,// Div_rm32
	MemorySize::Unknown,// Div_rm64
	MemorySize::Unknown,// Idiv_rm16
	MemorySize::Unknown,// Idiv_rm32
	MemorySize::Unknown,// Idiv_rm64
	MemorySize::Unknown,// Clc
	MemorySize::Unknown,// Stc
	MemorySize::Unknown,// Cli
	MemorySize::Unknown,// Sti
	MemorySize::Unknown,// Cld
	MemorySize::Unknown,// Std
	MemorySize::Unknown,// Inc_rm8
	MemorySize::Unknown,// Dec_rm8
	MemorySize::Unknown,// Inc_rm16
	MemorySize::Unknown,// Inc_rm32
	MemorySize::Unknown,// Inc_rm64
	MemorySize::Unknown,// Dec_rm16
	MemorySize::Unknown,// Dec_rm32
	MemorySize::Unknown,// Dec_rm64
	MemorySize::Unknown,// Call_rm16
	MemorySize::Unknown,// Call_rm32
	MemorySize::Unknown,// Call_rm64
	MemorySize::Unknown,// Call_m1616
	MemorySize::Unknown,// Call_m1632
	MemorySize::Unknown,// Call_m1664
	MemorySize::Unknown,// Jmp_rm16
	MemorySize::Unknown,// Jmp_rm32
	MemorySize::Unknown,// Jmp_rm64
	MemorySize::Unknown,// Jmp_m1616
	MemorySize::Unknown,// Jmp_m1632
	MemorySize::Unknown,// Jmp_m1664
	MemorySize::Unknown,// Push_rm16
	MemorySize::Unknown,// Push_rm32
	MemorySize::Unknown,// Push_rm64
	MemorySize::Unknown,// Sldt_rm16
	MemorySize::Unknown,// Sldt_r32m16
	MemorySize::Unknown,// Sldt_r64m16
	MemorySize::Unknown,// Str_rm16
	MemorySize::Unknown,// Str_r32m16
	MemorySize::Unknown,// Str_r64m16
	MemorySize::Unknown,// Lldt_rm16
	MemorySize::Unknown,// Lldt_r32m16
	MemorySize::Unknown,// Lldt_r64m16
	MemorySize::Unknown,// Ltr_rm16
	MemorySize::Unknown,// Ltr_r32m16
	MemorySize::Unknown,// Ltr_r64m16
	MemorySize::Unknown,// Verr_rm16
	MemorySize::Unknown,// Verr_r32m16
	MemorySize::Unknown,// Verr_r64m16
	MemorySize::Unknown,// Verw_rm16
	MemorySize::Unknown,// Verw_r32m16
	MemorySize::Unknown,// Verw_r64m16
	MemorySize::Unknown,// Jmpe_rm16
	MemorySize::Unknown,// Jmpe_rm32
	MemorySize::Unknown,// Sgdt_m1632_16
	MemorySize::Unknown,// Sgdt_m1632
	MemorySize::Unknown,// Sgdt_m1664
	MemorySize::Unknown,// Sidt_m1632_16
	MemorySize::Unknown,// Sidt_m1632
	MemorySize::Unknown,// Sidt_m1664
	MemorySize::Unknown,// Lgdt_m1632_16
	MemorySize::Unknown,// Lgdt_m1632
	MemorySize::Unknown,// Lgdt_m1664
	MemorySize::Unknown,// Lidt_m1632_16
	MemorySize::Unknown,// Lidt_m1632
	MemorySize::Unknown,// Lidt_m1664
	MemorySize::Unknown,// Smsw_rm16
	MemorySize::Unknown,// Smsw_r32m16
	MemorySize::Unknown,// Smsw_r64m16
	MemorySize::Unknown,// Rstorssp_m64
	MemorySize::Unknown,// Lmsw_rm16
	MemorySize::Unknown,// Lmsw_r32m16
	MemorySize::Unknown,// Lmsw_r64m16
	MemorySize::Unknown,// Invlpg_m
	MemorySize::Unknown,// Enclv
	MemorySize::Unknown,// Vmcall
	MemorySize::Unknown,// Vmlaunch
	MemorySize::Unknown,// Vmresume
	MemorySize::Unknown,// Vmxoff
	MemorySize::Unknown,// Pconfig
	MemorySize::Unknown,// Monitorw
	MemorySize::Unknown,// Monitord
	MemorySize::Unknown,// Monitorq
	MemorySize::Unknown,// Mwait
	MemorySize::Unknown,// Clac
	MemorySize::Unknown,// Stac
	MemorySize::Unknown,// Encls
	MemorySize::Unknown,// Xgetbv
	MemorySize::Unknown,// Xsetbv
	MemorySize::Unknown,// Vmfunc
	MemorySize::Unknown,// Xend
	MemorySize::Unknown,// Xtest
	MemorySize::Unknown,// Enclu
	MemorySize::Unknown,// Vmrunw
	MemorySize::Unknown,// Vmrund
	MemorySize::Unknown,// Vmrunq
	MemorySize::Unknown,// Vmmcall
	MemorySize::Unknown,// Vmloadw
	MemorySize::Unknown,// Vmloadd
	MemorySize::Unknown,// Vmloadq
	MemorySize::Unknown,// Vmsavew
	MemorySize::Unknown,// Vmsaved
	MemorySize::Unknown,// Vmsaveq
	MemorySize::Unknown,// Stgi
	MemorySize::Unknown,// Clgi
	MemorySize::Unknown,// Skinit
	MemorySize::Unknown,// Invlpgaw
	MemorySize::Unknown,// Invlpgad
	MemorySize::Unknown,// Invlpgaq
	MemorySize::Unknown,// Setssbsy
	MemorySize::Unknown,// Saveprevssp
	MemorySize::Unknown,// Rdpkru
	MemorySize::Unknown,// Wrpkru
	MemorySize::Unknown,// Swapgs
	MemorySize::Unknown,// Rdtscp
	MemorySize::Unknown,// Monitorxw
	MemorySize::Unknown,// Monitorxd
	MemorySize::Unknown,// Monitorxq
	MemorySize::Unknown,// Mcommit
	MemorySize::Unknown,// Mwaitx
	MemorySize::Unknown,// Clzerow
	MemorySize::Unknown,// Clzerod
	MemorySize::Unknown,// Clzeroq
	MemorySize::Unknown,// Rdpru
	MemorySize::Unknown,// Lar_r16_rm16
	MemorySize::Unknown,// Lar_r32_r32m16
	MemorySize::Unknown,// Lar_r64_r64m16
	MemorySize::Unknown,// Lsl_r16_rm16
	MemorySize::Unknown,// Lsl_r32_r32m16
	MemorySize::Unknown,// Lsl_r64_r64m16
	MemorySize::Unknown,// Storeall
	MemorySize::Unknown,// Loadall286
	MemorySize::Unknown,// Syscall
	MemorySize::Unknown,// Clts
	MemorySize::Unknown,// Loadall386
	MemorySize::Unknown,// Sysretd
	MemorySize::Unknown,// Sysretq
	MemorySize::Unknown,// Invd
	MemorySize::Unknown,// Wbinvd
	MemorySize::Unknown,// Wbnoinvd
	MemorySize::Unknown,// Cl1invmb
	MemorySize::Unknown,// Ud2
	MemorySize::Unknown,// Reservednop_rm16_r16_0F0D
	MemorySize::Unknown,// Reservednop_rm32_r32_0F0D
	MemorySize::Unknown,// Reservednop_rm64_r64_0F0D
	MemorySize::Unknown,// Prefetch_m8
	MemorySize::Unknown,// Prefetchw_m8
	MemorySize::Unknown,// Prefetchwt1_m8
	MemorySize::Unknown,// Femms
	MemorySize::Unknown,// Umov_rm8_r8
	MemorySize::Unknown,// Umov_rm16_r16
	MemorySize::Unknown,// Umov_rm32_r32
	MemorySize::Unknown,// Umov_r8_rm8
	MemorySize::Unknown,// Umov_r16_rm16
	MemorySize::Unknown,// Umov_r32_rm32
	MemorySize::Unknown,// Movups_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovups_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovups_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovups_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovups_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovups_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movupd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovupd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovupd_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovupd_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovupd_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovupd_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vmovss_xmm_xmm_xmm
	MemorySize::Unknown,// VEX_Vmovss_xmm_m32
	MemorySize::Unknown,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovss_xmm_k1z_m32
	MemorySize::Unknown,// Movsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmovsd_xmm_xmm_xmm
	MemorySize::Unknown,// VEX_Vmovsd_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovsd_xmm_k1z_m64
	MemorySize::Unknown,// Movups_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovups_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovups_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovups_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovups_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovups_zmmm512_k1z_zmm
	MemorySize::Unknown,// Movupd_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovupd_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovupd_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovupd_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovupd_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovupd_zmmm512_k1z_zmm
	MemorySize::Unknown,// Movss_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vmovss_xmm_xmm_xmm_0F11
	MemorySize::Unknown,// VEX_Vmovss_m32_xmm
	MemorySize::Unknown,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	MemorySize::Unknown,// EVEX_Vmovss_m32_k1_xmm
	MemorySize::Unknown,// Movsd_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	MemorySize::Unknown,// VEX_Vmovsd_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	MemorySize::Unknown,// EVEX_Vmovsd_m64_k1_xmm
	MemorySize::Unknown,// Movhlps_xmm_xmm
	MemorySize::Unknown,// Movlps_xmm_m64
	MemorySize::Unknown,// VEX_Vmovhlps_xmm_xmm_xmm
	MemorySize::Unknown,// VEX_Vmovlps_xmm_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovhlps_xmm_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovlps_xmm_xmm_m64
	MemorySize::Unknown,// Movlpd_xmm_m64
	MemorySize::Unknown,// VEX_Vmovlpd_xmm_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovlpd_xmm_xmm_m64
	MemorySize::Unknown,// Movsldup_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovsldup_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovsldup_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movddup_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmovddup_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmovddup_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovddup_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vmovddup_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovddup_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movlps_m64_xmm
	MemorySize::Unknown,// VEX_Vmovlps_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovlps_m64_xmm
	MemorySize::Unknown,// Movlpd_m64_xmm
	MemorySize::Unknown,// VEX_Vmovlpd_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovlpd_m64_xmm
	MemorySize::Unknown,// Unpcklps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpcklps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpcklps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Unpcklpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Unpckhps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpckhps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpckhps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Unpckhpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Movlhps_xmm_xmm
	MemorySize::Unknown,// VEX_Vmovlhps_xmm_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovlhps_xmm_xmm_xmm
	MemorySize::Unknown,// Movhps_xmm_m64
	MemorySize::Unknown,// VEX_Vmovhps_xmm_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovhps_xmm_xmm_m64
	MemorySize::Unknown,// Movhpd_xmm_m64
	MemorySize::Unknown,// VEX_Vmovhpd_xmm_xmm_m64
	MemorySize::Unknown,// EVEX_Vmovhpd_xmm_xmm_m64
	MemorySize::Unknown,// Movshdup_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovshdup_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovshdup_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movhps_m64_xmm
	MemorySize::Unknown,// VEX_Vmovhps_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovhps_m64_xmm
	MemorySize::Unknown,// Movhpd_m64_xmm
	MemorySize::Unknown,// VEX_Vmovhpd_m64_xmm
	MemorySize::Unknown,// EVEX_Vmovhpd_m64_xmm
	MemorySize::Unknown,// Reservednop_rm16_r16_0F18
	MemorySize::Unknown,// Reservednop_rm32_r32_0F18
	MemorySize::Unknown,// Reservednop_rm64_r64_0F18
	MemorySize::Unknown,// Reservednop_rm16_r16_0F19
	MemorySize::Unknown,// Reservednop_rm32_r32_0F19
	MemorySize::Unknown,// Reservednop_rm64_r64_0F19
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1A
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1A
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1A
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1B
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1B
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1B
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1C
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1C
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1C
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1D
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1D
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1D
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1E
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1E
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1E
	MemorySize::Unknown,// Reservednop_rm16_r16_0F1F
	MemorySize::Unknown,// Reservednop_rm32_r32_0F1F
	MemorySize::Unknown,// Reservednop_rm64_r64_0F1F
	MemorySize::Unknown,// Prefetchnta_m8
	MemorySize::Unknown,// Prefetcht0_m8
	MemorySize::Unknown,// Prefetcht1_m8
	MemorySize::Unknown,// Prefetcht2_m8
	MemorySize::Unknown,// Bndldx_bnd_mib
	MemorySize::Unknown,// Bndmov_bnd_bndm64
	MemorySize::Unknown,// Bndmov_bnd_bndm128
	MemorySize::Unknown,// Bndcl_bnd_rm32
	MemorySize::Unknown,// Bndcl_bnd_rm64
	MemorySize::Unknown,// Bndcu_bnd_rm32
	MemorySize::Unknown,// Bndcu_bnd_rm64
	MemorySize::Unknown,// Bndstx_mib_bnd
	MemorySize::Unknown,// Bndmov_bndm64_bnd
	MemorySize::Unknown,// Bndmov_bndm128_bnd
	MemorySize::Unknown,// Bndmk_bnd_m32
	MemorySize::Unknown,// Bndmk_bnd_m64
	MemorySize::Unknown,// Bndcn_bnd_rm32
	MemorySize::Unknown,// Bndcn_bnd_rm64
	MemorySize::Unknown,// Cldemote_m8
	MemorySize::Unknown,// Rdsspd_r32
	MemorySize::Unknown,// Rdsspq_r64
	MemorySize::Unknown,// Endbr64
	MemorySize::Unknown,// Endbr32
	MemorySize::Unknown,// Nop_rm16
	MemorySize::Unknown,// Nop_rm32
	MemorySize::Unknown,// Nop_rm64
	MemorySize::Unknown,// Mov_r32_cr
	MemorySize::Unknown,// Mov_r64_cr
	MemorySize::Unknown,// Mov_r32_dr
	MemorySize::Unknown,// Mov_r64_dr
	MemorySize::Unknown,// Mov_cr_r32
	MemorySize::Unknown,// Mov_cr_r64
	MemorySize::Unknown,// Mov_dr_r32
	MemorySize::Unknown,// Mov_dr_r64
	MemorySize::Unknown,// Mov_r32_tr
	MemorySize::Unknown,// Mov_tr_r32
	MemorySize::Unknown,// Movaps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovaps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovaps_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovaps_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovaps_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovaps_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movapd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovapd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovapd_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovapd_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovapd_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovapd_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movaps_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovaps_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovaps_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovaps_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovaps_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovaps_zmmm512_k1z_zmm
	MemorySize::Unknown,// Movapd_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovapd_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovapd_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovapd_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovapd_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovapd_zmmm512_k1z_zmm
	MemorySize::Unknown,// Cvtpi2ps_xmm_mmm64
	MemorySize::Unknown,// Cvtpi2pd_xmm_mmm64
	MemorySize::Unknown,// Cvtsi2ss_xmm_rm32
	MemorySize::Unknown,// Cvtsi2ss_xmm_rm64
	MemorySize::Unknown,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	MemorySize::Unknown,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	MemorySize::Unknown,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	MemorySize::Unknown,// Cvtsi2sd_xmm_rm32
	MemorySize::Unknown,// Cvtsi2sd_xmm_rm64
	MemorySize::Unknown,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	MemorySize::Unknown,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	MemorySize::Unknown,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	MemorySize::Unknown,// Movntps_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntps_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntps_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntps_m128_xmm
	MemorySize::Unknown,// EVEX_Vmovntps_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntps_m512_zmm
	MemorySize::Unknown,// Movntpd_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntpd_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntpd_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntpd_m128_xmm
	MemorySize::Unknown,// EVEX_Vmovntpd_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntpd_m512_zmm
	MemorySize::Unknown,// Movntss_m32_xmm
	MemorySize::Unknown,// Movntsd_m64_xmm
	MemorySize::Unknown,// Cvttps2pi_mm_xmmm64
	MemorySize::Unknown,// Cvttpd2pi_mm_xmmm128
	MemorySize::Unknown,// Cvttss2si_r32_xmmm32
	MemorySize::Unknown,// Cvttss2si_r64_xmmm32
	MemorySize::Unknown,// VEX_Vcvttss2si_r32_xmmm32
	MemorySize::Unknown,// VEX_Vcvttss2si_r64_xmmm32
	MemorySize::Unknown,// EVEX_Vcvttss2si_r32_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vcvttss2si_r64_xmmm32_sae
	MemorySize::Unknown,// Cvttsd2si_r32_xmmm64
	MemorySize::Unknown,// Cvttsd2si_r64_xmmm64
	MemorySize::Unknown,// VEX_Vcvttsd2si_r32_xmmm64
	MemorySize::Unknown,// VEX_Vcvttsd2si_r64_xmmm64
	MemorySize::Unknown,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	MemorySize::Unknown,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	MemorySize::Unknown,// Cvtps2pi_mm_xmmm64
	MemorySize::Unknown,// Cvtpd2pi_mm_xmmm128
	MemorySize::Unknown,// Cvtss2si_r32_xmmm32
	MemorySize::Unknown,// Cvtss2si_r64_xmmm32
	MemorySize::Unknown,// VEX_Vcvtss2si_r32_xmmm32
	MemorySize::Unknown,// VEX_Vcvtss2si_r64_xmmm32
	MemorySize::Unknown,// EVEX_Vcvtss2si_r32_xmmm32_er
	MemorySize::Unknown,// EVEX_Vcvtss2si_r64_xmmm32_er
	MemorySize::Unknown,// Cvtsd2si_r32_xmmm64
	MemorySize::Unknown,// Cvtsd2si_r64_xmmm64
	MemorySize::Unknown,// VEX_Vcvtsd2si_r32_xmmm64
	MemorySize::Unknown,// VEX_Vcvtsd2si_r64_xmmm64
	MemorySize::Unknown,// EVEX_Vcvtsd2si_r32_xmmm64_er
	MemorySize::Unknown,// EVEX_Vcvtsd2si_r64_xmmm64_er
	MemorySize::Unknown,// Ucomiss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vucomiss_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vucomiss_xmm_xmmm32_sae
	MemorySize::Unknown,// Ucomisd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vucomisd_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vucomisd_xmm_xmmm64_sae
	MemorySize::Unknown,// Comiss_xmm_xmmm32
	MemorySize::Unknown,// Comisd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcomiss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vcomisd_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vcomiss_xmm_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vcomisd_xmm_xmmm64_sae
	MemorySize::Unknown,// Wrmsr
	MemorySize::Unknown,// Rdtsc
	MemorySize::Unknown,// Rdmsr
	MemorySize::Unknown,// Rdpmc
	MemorySize::Unknown,// Sysenter
	MemorySize::Unknown,// Sysexitd
	MemorySize::Unknown,// Sysexitq
	MemorySize::Unknown,// Getsecd
	MemorySize::Unknown,// Cmovo_r16_rm16
	MemorySize::Unknown,// Cmovo_r32_rm32
	MemorySize::Unknown,// Cmovo_r64_rm64
	MemorySize::Unknown,// Cmovno_r16_rm16
	MemorySize::Unknown,// Cmovno_r32_rm32
	MemorySize::Unknown,// Cmovno_r64_rm64
	MemorySize::Unknown,// Cmovb_r16_rm16
	MemorySize::Unknown,// Cmovb_r32_rm32
	MemorySize::Unknown,// Cmovb_r64_rm64
	MemorySize::Unknown,// Cmovae_r16_rm16
	MemorySize::Unknown,// Cmovae_r32_rm32
	MemorySize::Unknown,// Cmovae_r64_rm64
	MemorySize::Unknown,// Cmove_r16_rm16
	MemorySize::Unknown,// Cmove_r32_rm32
	MemorySize::Unknown,// Cmove_r64_rm64
	MemorySize::Unknown,// Cmovne_r16_rm16
	MemorySize::Unknown,// Cmovne_r32_rm32
	MemorySize::Unknown,// Cmovne_r64_rm64
	MemorySize::Unknown,// Cmovbe_r16_rm16
	MemorySize::Unknown,// Cmovbe_r32_rm32
	MemorySize::Unknown,// Cmovbe_r64_rm64
	MemorySize::Unknown,// Cmova_r16_rm16
	MemorySize::Unknown,// Cmova_r32_rm32
	MemorySize::Unknown,// Cmova_r64_rm64
	MemorySize::Unknown,// Cmovs_r16_rm16
	MemorySize::Unknown,// Cmovs_r32_rm32
	MemorySize::Unknown,// Cmovs_r64_rm64
	MemorySize::Unknown,// Cmovns_r16_rm16
	MemorySize::Unknown,// Cmovns_r32_rm32
	MemorySize::Unknown,// Cmovns_r64_rm64
	MemorySize::Unknown,// Cmovp_r16_rm16
	MemorySize::Unknown,// Cmovp_r32_rm32
	MemorySize::Unknown,// Cmovp_r64_rm64
	MemorySize::Unknown,// Cmovnp_r16_rm16
	MemorySize::Unknown,// Cmovnp_r32_rm32
	MemorySize::Unknown,// Cmovnp_r64_rm64
	MemorySize::Unknown,// Cmovl_r16_rm16
	MemorySize::Unknown,// Cmovl_r32_rm32
	MemorySize::Unknown,// Cmovl_r64_rm64
	MemorySize::Unknown,// Cmovge_r16_rm16
	MemorySize::Unknown,// Cmovge_r32_rm32
	MemorySize::Unknown,// Cmovge_r64_rm64
	MemorySize::Unknown,// Cmovle_r16_rm16
	MemorySize::Unknown,// Cmovle_r32_rm32
	MemorySize::Unknown,// Cmovle_r64_rm64
	MemorySize::Unknown,// Cmovg_r16_rm16
	MemorySize::Unknown,// Cmovg_r32_rm32
	MemorySize::Unknown,// Cmovg_r64_rm64
	MemorySize::Unknown,// VEX_Kandw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kandnd_kr_kr_kr
	MemorySize::Unknown,// VEX_Knotw_kr_kr
	MemorySize::Unknown,// VEX_Knotq_kr_kr
	MemorySize::Unknown,// VEX_Knotb_kr_kr
	MemorySize::Unknown,// VEX_Knotd_kr_kr
	MemorySize::Unknown,// VEX_Korw_kr_kr_kr
	MemorySize::Unknown,// VEX_Korq_kr_kr_kr
	MemorySize::Unknown,// VEX_Korb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnorb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxnord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxorb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kxord_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddw_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddb_kr_kr_kr
	MemorySize::Unknown,// VEX_Kaddd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckwd_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckdq_kr_kr_kr
	MemorySize::Unknown,// VEX_Kunpckbw_kr_kr_kr
	MemorySize::Unknown,// Movmskps_r32_xmm
	MemorySize::Unknown,// Movmskps_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r32_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskps_r32_ymm
	MemorySize::Unknown,// VEX_Vmovmskps_r64_ymm
	MemorySize::Unknown,// Movmskpd_r32_xmm
	MemorySize::Unknown,// Movmskpd_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r32_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r64_xmm
	MemorySize::Unknown,// VEX_Vmovmskpd_r32_ymm
	MemorySize::Unknown,// VEX_Vmovmskpd_r64_ymm
	MemorySize::Unknown,// Sqrtps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsqrtps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsqrtps_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	MemorySize::Unknown,// Sqrtpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsqrtpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsqrtpd_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Sqrtss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vsqrtss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// Sqrtsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Rsqrtps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vrsqrtps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vrsqrtps_ymm_ymmm256
	MemorySize::Unknown,// Rsqrtss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	MemorySize::Unknown,// Rcpps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vrcpps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vrcpps_ymm_ymmm256
	MemorySize::Unknown,// Rcpss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vrcpss_xmm_xmm_xmmm32
	MemorySize::Unknown,// Andps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Andpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Andnps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandnps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandnps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Andnpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandnpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vandnpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Orps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vorps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vorps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Orpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vorpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vorpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Xorps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vxorps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vxorps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Xorpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vxorpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vxorpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Addps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// Addpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// Addss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vaddss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// Addsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vaddsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Mulps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmulps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmulps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// Mulpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmulpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmulpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// Mulss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vmulss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// Mulsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmulsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Cvtps2pd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtps2pd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtps2pd_ymm_xmmm128
	MemorySize::Broadcast64_Float32,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	MemorySize::Unknown,// Cvtpd2ps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtpd2ps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtpd2ps_xmm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Cvtss2sd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// Cvtsd2ss_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Cvtdq2ps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtdq2ps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtdq2ps_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_Int64,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Cvtps2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtps2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtps2dq_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	MemorySize::Unknown,// Cvttps2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvttps2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvttps2dq_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	MemorySize::Unknown,// Subps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsubps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsubps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// Subpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsubpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsubpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// Subss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vsubss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// Subsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vsubsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Minps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vminps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vminps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	MemorySize::Unknown,// Minpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vminpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vminpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	MemorySize::Unknown,// Minss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vminss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// Minsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vminsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Unknown,// Divps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vdivps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vdivps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// Divpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vdivpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vdivpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// Divss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vdivss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// Divsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vdivsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// Maxps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmaxps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmaxps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	MemorySize::Unknown,// Maxpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmaxpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmaxpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	MemorySize::Unknown,// Maxss_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vmaxss_xmm_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// Maxsd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmaxsd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Unknown,// Punpcklbw_mm_mmm32
	MemorySize::Unknown,// Punpcklbw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Punpcklwd_mm_mmm32
	MemorySize::Unknown,// Punpcklwd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Punpckldq_mm_mmm32
	MemorySize::Unknown,// Punpckldq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Packsswb_mm_mmm64
	MemorySize::Unknown,// Packsswb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpacksswb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpacksswb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pcmpgtb_mm_mmm64
	MemorySize::Unknown,// Pcmpgtb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// Pcmpgtw_mm_mmm64
	MemorySize::Unknown,// Pcmpgtw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// Pcmpgtd_mm_mmm64
	MemorySize::Unknown,// Pcmpgtd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	MemorySize::Unknown,// Packuswb_mm_mmm64
	MemorySize::Unknown,// Packuswb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackuswb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackuswb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Punpckhbw_mm_mmm64
	MemorySize::Unknown,// Punpckhbw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Punpckhwd_mm_mmm64
	MemorySize::Unknown,// Punpckhwd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Punpckhdq_mm_mmm64
	MemorySize::Unknown,// Punpckhdq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Packssdw_mm_mmm64
	MemorySize::Unknown,// Packssdw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackssdw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackssdw_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Punpcklqdq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt64,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Punpckhqdq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt64,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Movd_mm_rm32
	MemorySize::Unknown,// Movq_mm_rm64
	MemorySize::Unknown,// Movd_xmm_rm32
	MemorySize::Unknown,// Movq_xmm_rm64
	MemorySize::Unknown,// VEX_Vmovd_xmm_rm32
	MemorySize::Unknown,// VEX_Vmovq_xmm_rm64
	MemorySize::Unknown,// EVEX_Vmovd_xmm_rm32
	MemorySize::Unknown,// EVEX_Vmovq_xmm_rm64
	MemorySize::Unknown,// Movq_mm_mmm64
	MemorySize::Unknown,// Movdqa_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovdqa_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovdqa_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	MemorySize::Unknown,// Movdqu_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovdqu_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vmovdqu_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	MemorySize::Unknown,// Pshufw_mm_mmm64_imm8
	MemorySize::Unknown,// Pshufd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshufd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshufd_ymm_ymmm256_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// Pshufhw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshufhw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshufhw_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Pshuflw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshuflw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpshuflw_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psrlw_mm_imm8
	MemorySize::Unknown,// Psrlw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlw_ymm_ymm_imm8
	MemorySize::Unknown,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psraw_mm_imm8
	MemorySize::Unknown,// Psraw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsraw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsraw_ymm_ymm_imm8
	MemorySize::Unknown,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	MemorySize::Unknown,// Psllw_mm_imm8
	MemorySize::Unknown,// Psllw_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllw_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllw_ymm_ymm_imm8
	MemorySize::Unknown,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Psrld_mm_imm8
	MemorySize::Unknown,// Psrld_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrld_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrld_ymm_ymm_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// Psrad_mm_imm8
	MemorySize::Unknown,// Psrad_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrad_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrad_ymm_ymm_imm8
	MemorySize::Broadcast128_Int32,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_Int32,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_Int32,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	MemorySize::Broadcast128_Int64,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_Int64,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Int64,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Pslld_mm_imm8
	MemorySize::Unknown,// Pslld_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslld_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslld_ymm_ymm_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// Psrlq_mm_imm8
	MemorySize::Unknown,// Psrlq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrlq_ymm_ymm_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Psrldq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrldq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsrldq_ymm_ymm_imm8
	MemorySize::Unknown,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	MemorySize::Unknown,// Psllq_mm_imm8
	MemorySize::Unknown,// Psllq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpsllq_ymm_ymm_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// Pslldq_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslldq_xmm_xmm_imm8
	MemorySize::Unknown,// VEX_Vpslldq_ymm_ymm_imm8
	MemorySize::Unknown,// EVEX_Vpslldq_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpslldq_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpslldq_zmm_zmmm512_imm8
	MemorySize::Unknown,// Pcmpeqb_mm_mmm64
	MemorySize::Unknown,// Pcmpeqb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// Pcmpeqw_mm_mmm64
	MemorySize::Unknown,// Pcmpeqw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// Pcmpeqd_mm_mmm64
	MemorySize::Unknown,// Pcmpeqd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	MemorySize::Unknown,// Emms
	MemorySize::Unknown,// VEX_Vzeroupper
	MemorySize::Unknown,// VEX_Vzeroall
	MemorySize::Unknown,// Vmread_rm32_r32
	MemorySize::Unknown,// Vmread_rm64_r64
	MemorySize::Broadcast128_Float32,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Extrq_xmm_imm8_imm8
	MemorySize::Broadcast64_Float32,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	MemorySize::Unknown,// Insertq_xmm_xmm_imm8_imm8
	MemorySize::Unknown,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	MemorySize::Unknown,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	MemorySize::Unknown,// Vmwrite_r32_rm32
	MemorySize::Unknown,// Vmwrite_r64_rm64
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Extrq_xmm_xmm
	MemorySize::Broadcast64_Float32,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// EVEX_Vcvtss2usi_r32_xmmm32_er
	MemorySize::Unknown,// EVEX_Vcvtss2usi_r64_xmmm32_er
	MemorySize::Unknown,// Insertq_xmm_xmm
	MemorySize::Unknown,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	MemorySize::Unknown,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	MemorySize::Broadcast64_Float32,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	MemorySize::Broadcast64_UInt32,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_UInt32,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	MemorySize::Broadcast128_UInt64,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	MemorySize::Broadcast128_UInt32,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_UInt64,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	MemorySize::Broadcast64_Float32,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	MemorySize::Unknown,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	MemorySize::Unknown,// Haddpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhaddpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhaddpd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Haddps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhaddps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhaddps_ymm_ymm_ymmm256
	MemorySize::Unknown,// Hsubpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhsubpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhsubpd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Hsubps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhsubps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vhsubps_ymm_ymm_ymmm256
	MemorySize::Unknown,// Movd_rm32_mm
	MemorySize::Unknown,// Movq_rm64_mm
	MemorySize::Unknown,// Movd_rm32_xmm
	MemorySize::Unknown,// Movq_rm64_xmm
	MemorySize::Unknown,// VEX_Vmovd_rm32_xmm
	MemorySize::Unknown,// VEX_Vmovq_rm64_xmm
	MemorySize::Unknown,// EVEX_Vmovd_rm32_xmm
	MemorySize::Unknown,// EVEX_Vmovq_rm64_xmm
	MemorySize::Unknown,// Movq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vmovq_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vmovq_xmm_xmmm64
	MemorySize::Unknown,// Movq_mmm64_mm
	MemorySize::Unknown,// Movdqa_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovdqa_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovdqa_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	MemorySize::Unknown,// Movdqu_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovdqu_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vmovdqu_ymmm256_ymm
	MemorySize::Unknown,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	MemorySize::Unknown,// Jo_rel16
	MemorySize::Unknown,// Jo_rel32_32
	MemorySize::Unknown,// Jo_rel32_64
	MemorySize::Unknown,// Jno_rel16
	MemorySize::Unknown,// Jno_rel32_32
	MemorySize::Unknown,// Jno_rel32_64
	MemorySize::Unknown,// Jb_rel16
	MemorySize::Unknown,// Jb_rel32_32
	MemorySize::Unknown,// Jb_rel32_64
	MemorySize::Unknown,// Jae_rel16
	MemorySize::Unknown,// Jae_rel32_32
	MemorySize::Unknown,// Jae_rel32_64
	MemorySize::Unknown,// Je_rel16
	MemorySize::Unknown,// Je_rel32_32
	MemorySize::Unknown,// Je_rel32_64
	MemorySize::Unknown,// Jne_rel16
	MemorySize::Unknown,// Jne_rel32_32
	MemorySize::Unknown,// Jne_rel32_64
	MemorySize::Unknown,// Jbe_rel16
	MemorySize::Unknown,// Jbe_rel32_32
	MemorySize::Unknown,// Jbe_rel32_64
	MemorySize::Unknown,// Ja_rel16
	MemorySize::Unknown,// Ja_rel32_32
	MemorySize::Unknown,// Ja_rel32_64
	MemorySize::Unknown,// Js_rel16
	MemorySize::Unknown,// Js_rel32_32
	MemorySize::Unknown,// Js_rel32_64
	MemorySize::Unknown,// Jns_rel16
	MemorySize::Unknown,// Jns_rel32_32
	MemorySize::Unknown,// Jns_rel32_64
	MemorySize::Unknown,// Jp_rel16
	MemorySize::Unknown,// Jp_rel32_32
	MemorySize::Unknown,// Jp_rel32_64
	MemorySize::Unknown,// Jnp_rel16
	MemorySize::Unknown,// Jnp_rel32_32
	MemorySize::Unknown,// Jnp_rel32_64
	MemorySize::Unknown,// Jl_rel16
	MemorySize::Unknown,// Jl_rel32_32
	MemorySize::Unknown,// Jl_rel32_64
	MemorySize::Unknown,// Jge_rel16
	MemorySize::Unknown,// Jge_rel32_32
	MemorySize::Unknown,// Jge_rel32_64
	MemorySize::Unknown,// Jle_rel16
	MemorySize::Unknown,// Jle_rel32_32
	MemorySize::Unknown,// Jle_rel32_64
	MemorySize::Unknown,// Jg_rel16
	MemorySize::Unknown,// Jg_rel32_32
	MemorySize::Unknown,// Jg_rel32_64
	MemorySize::Unknown,// Seto_rm8
	MemorySize::Unknown,// Setno_rm8
	MemorySize::Unknown,// Setb_rm8
	MemorySize::Unknown,// Setae_rm8
	MemorySize::Unknown,// Sete_rm8
	MemorySize::Unknown,// Setne_rm8
	MemorySize::Unknown,// Setbe_rm8
	MemorySize::Unknown,// Seta_rm8
	MemorySize::Unknown,// Sets_rm8
	MemorySize::Unknown,// Setns_rm8
	MemorySize::Unknown,// Setp_rm8
	MemorySize::Unknown,// Setnp_rm8
	MemorySize::Unknown,// Setl_rm8
	MemorySize::Unknown,// Setge_rm8
	MemorySize::Unknown,// Setle_rm8
	MemorySize::Unknown,// Setg_rm8
	MemorySize::Unknown,// VEX_Kmovw_kr_km16
	MemorySize::Unknown,// VEX_Kmovq_kr_km64
	MemorySize::Unknown,// VEX_Kmovb_kr_km8
	MemorySize::Unknown,// VEX_Kmovd_kr_km32
	MemorySize::Unknown,// VEX_Kmovw_m16_kr
	MemorySize::Unknown,// VEX_Kmovq_m64_kr
	MemorySize::Unknown,// VEX_Kmovb_m8_kr
	MemorySize::Unknown,// VEX_Kmovd_m32_kr
	MemorySize::Unknown,// VEX_Kmovw_kr_r32
	MemorySize::Unknown,// VEX_Kmovb_kr_r32
	MemorySize::Unknown,// VEX_Kmovd_kr_r32
	MemorySize::Unknown,// VEX_Kmovq_kr_r64
	MemorySize::Unknown,// VEX_Kmovw_r32_kr
	MemorySize::Unknown,// VEX_Kmovb_r32_kr
	MemorySize::Unknown,// VEX_Kmovd_r32_kr
	MemorySize::Unknown,// VEX_Kmovq_r64_kr
	MemorySize::Unknown,// VEX_Kortestw_kr_kr
	MemorySize::Unknown,// VEX_Kortestq_kr_kr
	MemorySize::Unknown,// VEX_Kortestb_kr_kr
	MemorySize::Unknown,// VEX_Kortestd_kr_kr
	MemorySize::Unknown,// VEX_Ktestw_kr_kr
	MemorySize::Unknown,// VEX_Ktestq_kr_kr
	MemorySize::Unknown,// VEX_Ktestb_kr_kr
	MemorySize::Unknown,// VEX_Ktestd_kr_kr
	MemorySize::Unknown,// Pushw_FS
	MemorySize::Unknown,// Pushd_FS
	MemorySize::Unknown,// Pushq_FS
	MemorySize::Unknown,// Popw_FS
	MemorySize::Unknown,// Popd_FS
	MemorySize::Unknown,// Popq_FS
	MemorySize::Unknown,// Cpuid
	MemorySize::Unknown,// Bt_rm16_r16
	MemorySize::Unknown,// Bt_rm32_r32
	MemorySize::Unknown,// Bt_rm64_r64
	MemorySize::Unknown,// Shld_rm16_r16_imm8
	MemorySize::Unknown,// Shld_rm32_r32_imm8
	MemorySize::Unknown,// Shld_rm64_r64_imm8
	MemorySize::Unknown,// Shld_rm16_r16_CL
	MemorySize::Unknown,// Shld_rm32_r32_CL
	MemorySize::Unknown,// Shld_rm64_r64_CL
	MemorySize::Unknown,// Montmul_16
	MemorySize::Unknown,// Montmul_32
	MemorySize::Unknown,// Montmul_64
	MemorySize::Unknown,// Xsha1_16
	MemorySize::Unknown,// Xsha1_32
	MemorySize::Unknown,// Xsha1_64
	MemorySize::Unknown,// Xsha256_16
	MemorySize::Unknown,// Xsha256_32
	MemorySize::Unknown,// Xsha256_64
	MemorySize::Unknown,// Xbts_r16_rm16
	MemorySize::Unknown,// Xbts_r32_rm32
	MemorySize::Unknown,// Xstore_16
	MemorySize::Unknown,// Xstore_32
	MemorySize::Unknown,// Xstore_64
	MemorySize::Unknown,// Xcryptecb_16
	MemorySize::Unknown,// Xcryptecb_32
	MemorySize::Unknown,// Xcryptecb_64
	MemorySize::Unknown,// Xcryptcbc_16
	MemorySize::Unknown,// Xcryptcbc_32
	MemorySize::Unknown,// Xcryptcbc_64
	MemorySize::Unknown,// Xcryptctr_16
	MemorySize::Unknown,// Xcryptctr_32
	MemorySize::Unknown,// Xcryptctr_64
	MemorySize::Unknown,// Xcryptcfb_16
	MemorySize::Unknown,// Xcryptcfb_32
	MemorySize::Unknown,// Xcryptcfb_64
	MemorySize::Unknown,// Xcryptofb_16
	MemorySize::Unknown,// Xcryptofb_32
	MemorySize::Unknown,// Xcryptofb_64
	MemorySize::Unknown,// Ibts_rm16_r16
	MemorySize::Unknown,// Ibts_rm32_r32
	MemorySize::Unknown,// Cmpxchg486_rm8_r8
	MemorySize::Unknown,// Cmpxchg486_rm16_r16
	MemorySize::Unknown,// Cmpxchg486_rm32_r32
	MemorySize::Unknown,// Pushw_GS
	MemorySize::Unknown,// Pushd_GS
	MemorySize::Unknown,// Pushq_GS
	MemorySize::Unknown,// Popw_GS
	MemorySize::Unknown,// Popd_GS
	MemorySize::Unknown,// Popq_GS
	MemorySize::Unknown,// Rsm
	MemorySize::Unknown,// Bts_rm16_r16
	MemorySize::Unknown,// Bts_rm32_r32
	MemorySize::Unknown,// Bts_rm64_r64
	MemorySize::Unknown,// Shrd_rm16_r16_imm8
	MemorySize::Unknown,// Shrd_rm32_r32_imm8
	MemorySize::Unknown,// Shrd_rm64_r64_imm8
	MemorySize::Unknown,// Shrd_rm16_r16_CL
	MemorySize::Unknown,// Shrd_rm32_r32_CL
	MemorySize::Unknown,// Shrd_rm64_r64_CL
	MemorySize::Unknown,// Fxsave_m512byte
	MemorySize::Unknown,// Fxsave64_m512byte
	MemorySize::Unknown,// Rdfsbase_r32
	MemorySize::Unknown,// Rdfsbase_r64
	MemorySize::Unknown,// Fxrstor_m512byte
	MemorySize::Unknown,// Fxrstor64_m512byte
	MemorySize::Unknown,// Rdgsbase_r32
	MemorySize::Unknown,// Rdgsbase_r64
	MemorySize::Unknown,// Ldmxcsr_m32
	MemorySize::Unknown,// Wrfsbase_r32
	MemorySize::Unknown,// Wrfsbase_r64
	MemorySize::Unknown,// VEX_Vldmxcsr_m32
	MemorySize::Unknown,// Stmxcsr_m32
	MemorySize::Unknown,// Wrgsbase_r32
	MemorySize::Unknown,// Wrgsbase_r64
	MemorySize::Unknown,// VEX_Vstmxcsr_m32
	MemorySize::Unknown,// Xsave_mem
	MemorySize::Unknown,// Xsave64_mem
	MemorySize::Unknown,// Ptwrite_rm32
	MemorySize::Unknown,// Ptwrite_rm64
	MemorySize::Unknown,// Xrstor_mem
	MemorySize::Unknown,// Xrstor64_mem
	MemorySize::Unknown,// Incsspd_r32
	MemorySize::Unknown,// Incsspq_r64
	MemorySize::Unknown,// Xsaveopt_mem
	MemorySize::Unknown,// Xsaveopt64_mem
	MemorySize::Unknown,// Clwb_m8
	MemorySize::Unknown,// Tpause_r32
	MemorySize::Unknown,// Tpause_r64
	MemorySize::Unknown,// Clrssbsy_m64
	MemorySize::Unknown,// Umonitor_r16
	MemorySize::Unknown,// Umonitor_r32
	MemorySize::Unknown,// Umonitor_r64
	MemorySize::Unknown,// Umwait_r32
	MemorySize::Unknown,// Umwait_r64
	MemorySize::Unknown,// Clflush_m8
	MemorySize::Unknown,// Clflushopt_m8
	MemorySize::Unknown,// Lfence
	MemorySize::Unknown,// Lfence_E9
	MemorySize::Unknown,// Lfence_EA
	MemorySize::Unknown,// Lfence_EB
	MemorySize::Unknown,// Lfence_EC
	MemorySize::Unknown,// Lfence_ED
	MemorySize::Unknown,// Lfence_EE
	MemorySize::Unknown,// Lfence_EF
	MemorySize::Unknown,// Mfence
	MemorySize::Unknown,// Mfence_F1
	MemorySize::Unknown,// Mfence_F2
	MemorySize::Unknown,// Mfence_F3
	MemorySize::Unknown,// Mfence_F4
	MemorySize::Unknown,// Mfence_F5
	MemorySize::Unknown,// Mfence_F6
	MemorySize::Unknown,// Mfence_F7
	MemorySize::Unknown,// Sfence
	MemorySize::Unknown,// Sfence_F9
	MemorySize::Unknown,// Sfence_FA
	MemorySize::Unknown,// Sfence_FB
	MemorySize::Unknown,// Sfence_FC
	MemorySize::Unknown,// Sfence_FD
	MemorySize::Unknown,// Sfence_FE
	MemorySize::Unknown,// Sfence_FF
	MemorySize::Unknown,// Pcommit
	MemorySize::Unknown,// Imul_r16_rm16
	MemorySize::Unknown,// Imul_r32_rm32
	MemorySize::Unknown,// Imul_r64_rm64
	MemorySize::Unknown,// Cmpxchg_rm8_r8
	MemorySize::Unknown,// Cmpxchg_rm16_r16
	MemorySize::Unknown,// Cmpxchg_rm32_r32
	MemorySize::Unknown,// Cmpxchg_rm64_r64
	MemorySize::Unknown,// Lss_r16_m1616
	MemorySize::Unknown,// Lss_r32_m1632
	MemorySize::Unknown,// Lss_r64_m1664
	MemorySize::Unknown,// Btr_rm16_r16
	MemorySize::Unknown,// Btr_rm32_r32
	MemorySize::Unknown,// Btr_rm64_r64
	MemorySize::Unknown,// Lfs_r16_m1616
	MemorySize::Unknown,// Lfs_r32_m1632
	MemorySize::Unknown,// Lfs_r64_m1664
	MemorySize::Unknown,// Lgs_r16_m1616
	MemorySize::Unknown,// Lgs_r32_m1632
	MemorySize::Unknown,// Lgs_r64_m1664
	MemorySize::Unknown,// Movzx_r16_rm8
	MemorySize::Unknown,// Movzx_r32_rm8
	MemorySize::Unknown,// Movzx_r64_rm8
	MemorySize::Unknown,// Movzx_r16_rm16
	MemorySize::Unknown,// Movzx_r32_rm16
	MemorySize::Unknown,// Movzx_r64_rm16
	MemorySize::Unknown,// Jmpe_disp16
	MemorySize::Unknown,// Jmpe_disp32
	MemorySize::Unknown,// Popcnt_r16_rm16
	MemorySize::Unknown,// Popcnt_r32_rm32
	MemorySize::Unknown,// Popcnt_r64_rm64
	MemorySize::Unknown,// Ud1_r16_rm16
	MemorySize::Unknown,// Ud1_r32_rm32
	MemorySize::Unknown,// Ud1_r64_rm64
	MemorySize::Unknown,// Bt_rm16_imm8
	MemorySize::Unknown,// Bt_rm32_imm8
	MemorySize::Unknown,// Bt_rm64_imm8
	MemorySize::Unknown,// Bts_rm16_imm8
	MemorySize::Unknown,// Bts_rm32_imm8
	MemorySize::Unknown,// Bts_rm64_imm8
	MemorySize::Unknown,// Btr_rm16_imm8
	MemorySize::Unknown,// Btr_rm32_imm8
	MemorySize::Unknown,// Btr_rm64_imm8
	MemorySize::Unknown,// Btc_rm16_imm8
	MemorySize::Unknown,// Btc_rm32_imm8
	MemorySize::Unknown,// Btc_rm64_imm8
	MemorySize::Unknown,// Btc_rm16_r16
	MemorySize::Unknown,// Btc_rm32_r32
	MemorySize::Unknown,// Btc_rm64_r64
	MemorySize::Unknown,// Bsf_r16_rm16
	MemorySize::Unknown,// Bsf_r32_rm32
	MemorySize::Unknown,// Bsf_r64_rm64
	MemorySize::Unknown,// Tzcnt_r16_rm16
	MemorySize::Unknown,// Tzcnt_r32_rm32
	MemorySize::Unknown,// Tzcnt_r64_rm64
	MemorySize::Unknown,// Bsr_r16_rm16
	MemorySize::Unknown,// Bsr_r32_rm32
	MemorySize::Unknown,// Bsr_r64_rm64
	MemorySize::Unknown,// Lzcnt_r16_rm16
	MemorySize::Unknown,// Lzcnt_r32_rm32
	MemorySize::Unknown,// Lzcnt_r64_rm64
	MemorySize::Unknown,// Movsx_r16_rm8
	MemorySize::Unknown,// Movsx_r32_rm8
	MemorySize::Unknown,// Movsx_r64_rm8
	MemorySize::Unknown,// Movsx_r16_rm16
	MemorySize::Unknown,// Movsx_r32_rm16
	MemorySize::Unknown,// Movsx_r64_rm16
	MemorySize::Unknown,// Xadd_rm8_r8
	MemorySize::Unknown,// Xadd_rm16_r16
	MemorySize::Unknown,// Xadd_rm32_r32
	MemorySize::Unknown,// Xadd_rm64_r64
	MemorySize::Unknown,// Cmpps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	MemorySize::Unknown,// Cmppd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float64,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	MemorySize::Unknown,// Cmpss_xmm_xmmm32_imm8
	MemorySize::Unknown,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	MemorySize::Unknown,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// Cmpsd_xmm_xmmm64_imm8
	MemorySize::Unknown,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	MemorySize::Unknown,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	MemorySize::Unknown,// Movnti_m32_r32
	MemorySize::Unknown,// Movnti_m64_r64
	MemorySize::Unknown,// Pinsrw_mm_r32m16_imm8
	MemorySize::Unknown,// Pinsrw_mm_r64m16_imm8
	MemorySize::Unknown,// Pinsrw_xmm_r32m16_imm8
	MemorySize::Unknown,// Pinsrw_xmm_r64m16_imm8
	MemorySize::Unknown,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	MemorySize::Unknown,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	MemorySize::Unknown,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	MemorySize::Unknown,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	MemorySize::Unknown,// Pextrw_r32_mm_imm8
	MemorySize::Unknown,// Pextrw_r64_mm_imm8
	MemorySize::Unknown,// Pextrw_r32_xmm_imm8
	MemorySize::Unknown,// Pextrw_r64_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r32_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r64_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r32_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r64_xmm_imm8
	MemorySize::Unknown,// Shufps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Unknown,// Shufpd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float64,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// Cmpxchg8b_m64
	MemorySize::Unknown,// Cmpxchg16b_m128
	MemorySize::Unknown,// Xrstors_mem
	MemorySize::Unknown,// Xrstors64_mem
	MemorySize::Unknown,// Xsavec_mem
	MemorySize::Unknown,// Xsavec64_mem
	MemorySize::Unknown,// Xsaves_mem
	MemorySize::Unknown,// Xsaves64_mem
	MemorySize::Unknown,// Vmptrld_m64
	MemorySize::Unknown,// Vmclear_m64
	MemorySize::Unknown,// Vmxon_m64
	MemorySize::Unknown,// Rdrand_r16
	MemorySize::Unknown,// Rdrand_r32
	MemorySize::Unknown,// Rdrand_r64
	MemorySize::Unknown,// Vmptrst_m64
	MemorySize::Unknown,// Rdseed_r16
	MemorySize::Unknown,// Rdseed_r32
	MemorySize::Unknown,// Rdseed_r64
	MemorySize::Unknown,// Rdpid_r32
	MemorySize::Unknown,// Rdpid_r64
	MemorySize::Unknown,// Bswap_r16
	MemorySize::Unknown,// Bswap_r32
	MemorySize::Unknown,// Bswap_r64
	MemorySize::Unknown,// Addsubpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Addsubps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddsubps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaddsubps_ymm_ymm_ymmm256
	MemorySize::Unknown,// Psrlw_mm_mmm64
	MemorySize::Unknown,// Psrlw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlw_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Psrld_mm_mmm64
	MemorySize::Unknown,// Psrld_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrld_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrld_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Psrlq_mm_mmm64
	MemorySize::Unknown,// Psrlq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlq_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Paddq_mm_mmm64
	MemorySize::Unknown,// Paddq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt64,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pmullw_mm_mmm64
	MemorySize::Unknown,// Pmullw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmullw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmullw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Movq_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vmovq_xmmm64_xmm
	MemorySize::Unknown,// EVEX_Vmovq_xmmm64_xmm
	MemorySize::Unknown,// Movq2dq_xmm_mm
	MemorySize::Unknown,// Movdq2q_mm_xmm
	MemorySize::Unknown,// Pmovmskb_r32_mm
	MemorySize::Unknown,// Pmovmskb_r64_mm
	MemorySize::Unknown,// Pmovmskb_r32_xmm
	MemorySize::Unknown,// Pmovmskb_r64_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r32_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r64_xmm
	MemorySize::Unknown,// VEX_Vpmovmskb_r32_ymm
	MemorySize::Unknown,// VEX_Vpmovmskb_r64_ymm
	MemorySize::Unknown,// Psubusb_mm_mmm64
	MemorySize::Unknown,// Psubusb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubusb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubusb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psubusw_mm_mmm64
	MemorySize::Unknown,// Psubusw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubusw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubusw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pminub_mm_mmm64
	MemorySize::Unknown,// Pminub_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminub_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminub_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pand_mm_mmm64
	MemorySize::Unknown,// Pand_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpand_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpand_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Paddusb_mm_mmm64
	MemorySize::Unknown,// Paddusb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddusb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddusb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Paddusw_mm_mmm64
	MemorySize::Unknown,// Paddusw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddusw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddusw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmaxub_mm_mmm64
	MemorySize::Unknown,// Pmaxub_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxub_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxub_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pandn_mm_mmm64
	MemorySize::Unknown,// Pandn_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpandn_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpandn_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pavgb_mm_mmm64
	MemorySize::Unknown,// Pavgb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpavgb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpavgb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psraw_mm_mmm64
	MemorySize::Unknown,// Psraw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsraw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsraw_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Psrad_mm_mmm64
	MemorySize::Unknown,// Psrad_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrad_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrad_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Pavgw_mm_mmm64
	MemorySize::Unknown,// Pavgw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpavgw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpavgw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmulhuw_mm_mmm64
	MemorySize::Unknown,// Pmulhuw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmulhw_mm_mmm64
	MemorySize::Unknown,// Pmulhw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Cvttpd2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvttpd2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvttpd2dq_xmm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Cvtdq2pd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtdq2pd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtdq2pd_ymm_xmmm128
	MemorySize::Broadcast64_Int32,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	MemorySize::Broadcast128_Int32,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	MemorySize::Broadcast128_Int64,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Cvtpd2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtpd2dq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtpd2dq_xmm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	MemorySize::Unknown,// Movntq_m64_mm
	MemorySize::Unknown,// Movntdq_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntdq_m128_xmm
	MemorySize::Unknown,// VEX_Vmovntdq_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntdq_m128_xmm
	MemorySize::Unknown,// EVEX_Vmovntdq_m256_ymm
	MemorySize::Unknown,// EVEX_Vmovntdq_m512_zmm
	MemorySize::Unknown,// Psubsb_mm_mmm64
	MemorySize::Unknown,// Psubsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubsb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubsb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psubsw_mm_mmm64
	MemorySize::Unknown,// Psubsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pminsw_mm_mmm64
	MemorySize::Unknown,// Pminsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Por_mm_mmm64
	MemorySize::Unknown,// Por_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpor_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpor_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Paddsb_mm_mmm64
	MemorySize::Unknown,// Paddsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddsb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddsb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Paddsw_mm_mmm64
	MemorySize::Unknown,// Paddsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmaxsw_mm_mmm64
	MemorySize::Unknown,// Pmaxsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pxor_mm_mmm64
	MemorySize::Unknown,// Pxor_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpxor_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpxor_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Lddqu_xmm_m128
	MemorySize::Unknown,// VEX_Vlddqu_xmm_m128
	MemorySize::Unknown,// VEX_Vlddqu_ymm_m256
	MemorySize::Unknown,// Psllw_mm_mmm64
	MemorySize::Unknown,// Psllw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllw_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Pslld_mm_mmm64
	MemorySize::Unknown,// Pslld_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpslld_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpslld_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Psllq_mm_mmm64
	MemorySize::Unknown,// Psllq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllq_ymm_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	MemorySize::Unknown,// Pmuludq_mm_mmm64
	MemorySize::Unknown,// Pmuludq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmuludq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmuludq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_2xUInt32,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_2xUInt32,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_2xUInt32,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pmaddwd_mm_mmm64
	MemorySize::Unknown,// Pmaddwd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psadbw_mm_mmm64
	MemorySize::Unknown,// Psadbw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsadbw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsadbw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	MemorySize::Unknown,// Maskmovq_rDI_mm_mm
	MemorySize::Unknown,// Maskmovdqu_rDI_xmm_xmm
	MemorySize::Unknown,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	MemorySize::Unknown,// Psubb_mm_mmm64
	MemorySize::Unknown,// Psubb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psubw_mm_mmm64
	MemorySize::Unknown,// Psubw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Psubd_mm_mmm64
	MemorySize::Unknown,// Psubd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Psubq_mm_mmm64
	MemorySize::Unknown,// Psubq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsubq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Paddb_mm_mmm64
	MemorySize::Unknown,// Paddb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Paddw_mm_mmm64
	MemorySize::Unknown,// Paddw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Paddd_mm_mmm64
	MemorySize::Unknown,// Paddd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpaddd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// Ud0_r16_rm16
	MemorySize::Unknown,// Ud0_r32_rm32
	MemorySize::Unknown,// Ud0_r64_rm64
	MemorySize::Unknown,// Pshufb_mm_mmm64
	MemorySize::Unknown,// Pshufb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpshufb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpshufb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Phaddw_mm_mmm64
	MemorySize::Unknown,// Phaddw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddw_ymm_ymm_ymmm256
	MemorySize::Unknown,// Phaddd_mm_mmm64
	MemorySize::Unknown,// Phaddd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Phaddsw_mm_mmm64
	MemorySize::Unknown,// Phaddsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphaddsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// Pmaddubsw_mm_mmm64
	MemorySize::Unknown,// Pmaddubsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Phsubw_mm_mmm64
	MemorySize::Unknown,// Phsubw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubw_ymm_ymm_ymmm256
	MemorySize::Unknown,// Phsubd_mm_mmm64
	MemorySize::Unknown,// Phsubd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Phsubsw_mm_mmm64
	MemorySize::Unknown,// Phsubsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphsubsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// Psignb_mm_mmm64
	MemorySize::Unknown,// Psignb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignb_ymm_ymm_ymmm256
	MemorySize::Unknown,// Psignw_mm_mmm64
	MemorySize::Unknown,// Psignw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignw_ymm_ymm_ymmm256
	MemorySize::Unknown,// Psignd_mm_mmm64
	MemorySize::Unknown,// Psignd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsignd_ymm_ymm_ymmm256
	MemorySize::Unknown,// Pmulhrsw_mm_mmm64
	MemorySize::Unknown,// Pmulhrsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// VEX_Vpermilps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpermilps_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// VEX_Vpermilpd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpermilpd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float64,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// VEX_Vtestps_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vtestps_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vtestpd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vtestpd_ymm_ymmm256
	MemorySize::Unknown,// Pblendvb_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	MemorySize::Unknown,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	MemorySize::Unknown,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	MemorySize::Unknown,// VEX_Vcvtph2ps_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vcvtph2ps_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	MemorySize::Unknown,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	MemorySize::Unknown,// Blendvps_xmm_xmmm128
	MemorySize::Broadcast128_UInt32,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	MemorySize::Unknown,// Blendvpd_xmm_xmmm128
	MemorySize::Broadcast128_UInt32,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	MemorySize::Unknown,// VEX_Vpermps_ymm_ymm_ymmm256
	MemorySize::Broadcast256_Float32,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast256_Float64,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Ptest_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vptest_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vptest_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vbroadcastss_xmm_m32
	MemorySize::Unknown,// VEX_Vbroadcastss_ymm_m32
	MemorySize::Unknown,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	MemorySize::Unknown,// VEX_Vbroadcastsd_ymm_m64
	MemorySize::Unknown,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	MemorySize::Unknown,// VEX_Vbroadcastf128_ymm_m128
	MemorySize::Unknown,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	MemorySize::Unknown,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	MemorySize::Unknown,// Pabsb_mm_mmm64
	MemorySize::Unknown,// Pabsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsb_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpabsb_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpabsb_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpabsb_zmm_k1z_zmmm512
	MemorySize::Unknown,// Pabsw_mm_mmm64
	MemorySize::Unknown,// Pabsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsw_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpabsw_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpabsw_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpabsw_zmm_k1z_zmmm512
	MemorySize::Unknown,// Pabsd_mm_mmm64
	MemorySize::Unknown,// Pabsd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpabsd_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_Int64,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// Pmovsxbw_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxbw_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxbw_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	MemorySize::Unknown,// Pmovsxbd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovsxbd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovsxbd_ymm_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	MemorySize::Unknown,// Pmovsxbq_xmm_xmmm16
	MemorySize::Unknown,// VEX_Vpmovsxbq_xmm_xmmm16
	MemorySize::Unknown,// VEX_Vpmovsxbq_ymm_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	MemorySize::Unknown,// Pmovsxwd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxwd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxwd_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	MemorySize::Unknown,// Pmovsxwq_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovsxwq_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovsxwq_ymm_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	MemorySize::Unknown,// Pmovsxdq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxdq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovsxdq_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	MemorySize::Unknown,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	MemorySize::Broadcast128_UInt32,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	MemorySize::Unknown,// Pmuldq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmuldq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmuldq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_2xInt32,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_2xInt32,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_2xInt32,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovm2b_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2b_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2b_zmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2w_zmm_kr
	MemorySize::Unknown,// Pcmpeqq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt64,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovb2m_kr_zmm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovw2m_kr_zmm
	MemorySize::Unknown,// Movntdqa_xmm_m128
	MemorySize::Unknown,// VEX_Vmovntdqa_xmm_m128
	MemorySize::Unknown,// VEX_Vmovntdqa_ymm_m256
	MemorySize::Unknown,// EVEX_Vmovntdqa_xmm_m128
	MemorySize::Unknown,// EVEX_Vmovntdqa_ymm_m256
	MemorySize::Unknown,// EVEX_Vmovntdqa_zmm_m512
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_xmm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_ymm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmb2q_zmm_kr
	MemorySize::Unknown,// Packusdw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackusdw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpackusdw_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// VEX_Vmaskmovps_xmm_xmm_m128
	MemorySize::Unknown,// VEX_Vmaskmovps_ymm_ymm_m256
	MemorySize::Broadcast128_Float32,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vmaskmovpd_xmm_xmm_m128
	MemorySize::Unknown,// VEX_Vmaskmovpd_ymm_ymm_m256
	MemorySize::Unknown,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vmaskmovps_m128_xmm_xmm
	MemorySize::Unknown,// VEX_Vmaskmovps_m256_ymm_ymm
	MemorySize::Unknown,// VEX_Vmaskmovpd_m128_xmm_xmm
	MemorySize::Unknown,// VEX_Vmaskmovpd_m256_ymm_ymm
	MemorySize::Unknown,// Pmovzxbw_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxbw_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxbw_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	MemorySize::Unknown,// Pmovzxbd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovzxbd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovzxbd_ymm_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	MemorySize::Unknown,// Pmovzxbq_xmm_xmmm16
	MemorySize::Unknown,// VEX_Vpmovzxbq_xmm_xmmm16
	MemorySize::Unknown,// VEX_Vpmovzxbq_ymm_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	MemorySize::Unknown,// Pmovzxwd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxwd_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxwd_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	MemorySize::Unknown,// Pmovzxwq_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovzxwq_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpmovzxwq_ymm_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	MemorySize::Unknown,// Pmovzxdq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxdq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpmovzxdq_ymm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	MemorySize::Unknown,// VEX_Vpermd_ymm_ymm_ymmm256
	MemorySize::Broadcast256_UInt32,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast256_UInt64,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pcmpgtq_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int64,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	MemorySize::Unknown,// Pminsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpmovm2d_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2d_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2d_zmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_xmm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_ymm_kr
	MemorySize::Unknown,// EVEX_Vpmovm2q_zmm_kr
	MemorySize::Unknown,// Pminsd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminsd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Int64,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovd2m_kr_zmm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_xmm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_ymm
	MemorySize::Unknown,// EVEX_Vpmovq2m_kr_zmm
	MemorySize::Unknown,// Pminuw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminuw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminuw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_xmm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_ymm_kr
	MemorySize::Unknown,// EVEX_Vpbroadcastmw2d_zmm_kr
	MemorySize::Unknown,// Pminud_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpminud_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pmaxsb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmaxsd_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Int64,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pmaxuw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Pmaxud_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmaxud_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Pmulld_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulld_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmulld_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Int32,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Int64,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Phminposuw_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vphminposuw_xmm_xmmm128
	MemorySize::Broadcast128_Float32,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Broadcast128_UInt32,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// VEX_Vpsravd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsravd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// VEX_Vpsllvd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllvd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpsllvq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpsllvq_ymm_ymm_ymmm256
	MemorySize::Broadcast128_UInt32,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Broadcast128_Float32,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_Float64,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	MemorySize::Broadcast128_Float32,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_Float64,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	MemorySize::Unknown,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	MemorySize::Broadcast128_UInt32,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt32,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_2xInt16,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xInt16,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xInt16,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_2xBFloat16,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xBFloat16,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xBFloat16,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	MemorySize::Broadcast128_2xInt16,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xInt16,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xInt16,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	MemorySize::Unknown,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	MemorySize::Unknown,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// VEX_Vpbroadcastd_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vpbroadcastd_ymm_xmmm32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	MemorySize::Unknown,// VEX_Vpbroadcastq_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vpbroadcastq_ymm_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	MemorySize::Unknown,// VEX_Vbroadcasti128_ymm_m128
	MemorySize::Unknown,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	MemorySize::Unknown,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	MemorySize::Unknown,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	MemorySize::Unknown,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	MemorySize::Broadcast128_UInt32,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Broadcast128_Float32,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Float64,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt32,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	MemorySize::Broadcast128_Float32,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Float64,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// VEX_Vpbroadcastb_xmm_xmmm8
	MemorySize::Unknown,// VEX_Vpbroadcastb_ymm_xmmm8
	MemorySize::Unknown,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	MemorySize::Unknown,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	MemorySize::Unknown,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	MemorySize::Unknown,// VEX_Vpbroadcastw_xmm_xmmm16
	MemorySize::Unknown,// VEX_Vpbroadcastw_ymm_xmmm16
	MemorySize::Unknown,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	MemorySize::Unknown,// EVEX_Vpbroadcastb_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastb_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastb_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastw_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_xmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_ymm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastd_zmm_k1z_r32
	MemorySize::Unknown,// EVEX_Vpbroadcastq_xmm_k1z_r64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_ymm_k1z_r64
	MemorySize::Unknown,// EVEX_Vpbroadcastq_zmm_k1z_r64
	MemorySize::Unknown,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	MemorySize::Broadcast128_UInt32,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	MemorySize::Broadcast128_Float32,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	MemorySize::Broadcast128_Float64,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// Invept_r32_m128
	MemorySize::Unknown,// Invept_r64_m128
	MemorySize::Unknown,// Invvpid_r32_m128
	MemorySize::Unknown,// Invvpid_r64_m128
	MemorySize::Unknown,// Invpcid_r32_m128
	MemorySize::Unknown,// Invpcid_r64_m128
	MemorySize::Broadcast128_UInt64,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// EVEX_Vexpandps_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vexpandps_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vexpandps_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	MemorySize::Unknown,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	MemorySize::Unknown,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	MemorySize::Unknown,// EVEX_Vcompressps_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vcompressps_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vcompressps_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	MemorySize::Unknown,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	MemorySize::Unknown,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	MemorySize::Unknown,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	MemorySize::Unknown,// VEX_Vpmaskmovd_xmm_xmm_m128
	MemorySize::Unknown,// VEX_Vpmaskmovd_ymm_ymm_m256
	MemorySize::Unknown,// VEX_Vpmaskmovq_xmm_xmm_m128
	MemorySize::Unknown,// VEX_Vpmaskmovq_ymm_ymm_m256
	MemorySize::Unknown,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// VEX_Vpmaskmovd_m128_xmm_xmm
	MemorySize::Unknown,// VEX_Vpmaskmovd_m256_ymm_ymm
	MemorySize::Unknown,// VEX_Vpmaskmovq_m128_xmm_xmm
	MemorySize::Unknown,// VEX_Vpmaskmovq_m256_ymm_ymm
	MemorySize::Unknown,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	MemorySize::Unknown,// VEX_Vpgatherdd_xmm_vm32x_xmm
	MemorySize::Unknown,// VEX_Vpgatherdd_ymm_vm32y_ymm
	MemorySize::Unknown,// VEX_Vpgatherdq_xmm_vm32x_xmm
	MemorySize::Unknown,// VEX_Vpgatherdq_ymm_vm32x_ymm
	MemorySize::Unknown,// EVEX_Vpgatherdd_xmm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vpgatherdd_ymm_k1_vm32y
	MemorySize::Unknown,// EVEX_Vpgatherdd_zmm_k1_vm32z
	MemorySize::Unknown,// EVEX_Vpgatherdq_xmm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vpgatherdq_ymm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vpgatherdq_zmm_k1_vm32y
	MemorySize::Unknown,// VEX_Vpgatherqd_xmm_vm64x_xmm
	MemorySize::Unknown,// VEX_Vpgatherqd_xmm_vm64y_xmm
	MemorySize::Unknown,// VEX_Vpgatherqq_xmm_vm64x_xmm
	MemorySize::Unknown,// VEX_Vpgatherqq_ymm_vm64y_ymm
	MemorySize::Unknown,// EVEX_Vpgatherqd_xmm_k1_vm64x
	MemorySize::Unknown,// EVEX_Vpgatherqd_xmm_k1_vm64y
	MemorySize::Unknown,// EVEX_Vpgatherqd_ymm_k1_vm64z
	MemorySize::Unknown,// EVEX_Vpgatherqq_xmm_k1_vm64x
	MemorySize::Unknown,// EVEX_Vpgatherqq_ymm_k1_vm64y
	MemorySize::Unknown,// EVEX_Vpgatherqq_zmm_k1_vm64z
	MemorySize::Unknown,// VEX_Vgatherdps_xmm_vm32x_xmm
	MemorySize::Unknown,// VEX_Vgatherdps_ymm_vm32y_ymm
	MemorySize::Unknown,// VEX_Vgatherdpd_xmm_vm32x_xmm
	MemorySize::Unknown,// VEX_Vgatherdpd_ymm_vm32x_ymm
	MemorySize::Unknown,// EVEX_Vgatherdps_xmm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vgatherdps_ymm_k1_vm32y
	MemorySize::Unknown,// EVEX_Vgatherdps_zmm_k1_vm32z
	MemorySize::Unknown,// EVEX_Vgatherdpd_xmm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vgatherdpd_ymm_k1_vm32x
	MemorySize::Unknown,// EVEX_Vgatherdpd_zmm_k1_vm32y
	MemorySize::Unknown,// VEX_Vgatherqps_xmm_vm64x_xmm
	MemorySize::Unknown,// VEX_Vgatherqps_xmm_vm64y_xmm
	MemorySize::Unknown,// VEX_Vgatherqpd_xmm_vm64x_xmm
	MemorySize::Unknown,// VEX_Vgatherqpd_ymm_vm64y_ymm
	MemorySize::Unknown,// EVEX_Vgatherqps_xmm_k1_vm64x
	MemorySize::Unknown,// EVEX_Vgatherqps_xmm_k1_vm64y
	MemorySize::Unknown,// EVEX_Vgatherqps_ymm_k1_vm64z
	MemorySize::Unknown,// EVEX_Vgatherqpd_xmm_k1_vm64x
	MemorySize::Unknown,// EVEX_Vgatherqpd_ymm_k1_vm64y
	MemorySize::Unknown,// EVEX_Vgatherqpd_zmm_k1_vm64z
	MemorySize::Unknown,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	MemorySize::Unknown,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	MemorySize::Unknown,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// EVEX_Vpscatterdd_vm32x_k1_xmm
	MemorySize::Unknown,// EVEX_Vpscatterdd_vm32y_k1_ymm
	MemorySize::Unknown,// EVEX_Vpscatterdd_vm32z_k1_zmm
	MemorySize::Unknown,// EVEX_Vpscatterdq_vm32x_k1_xmm
	MemorySize::Unknown,// EVEX_Vpscatterdq_vm32x_k1_ymm
	MemorySize::Unknown,// EVEX_Vpscatterdq_vm32y_k1_zmm
	MemorySize::Unknown,// EVEX_Vpscatterqd_vm64x_k1_xmm
	MemorySize::Unknown,// EVEX_Vpscatterqd_vm64y_k1_xmm
	MemorySize::Unknown,// EVEX_Vpscatterqd_vm64z_k1_ymm
	MemorySize::Unknown,// EVEX_Vpscatterqq_vm64x_k1_xmm
	MemorySize::Unknown,// EVEX_Vpscatterqq_vm64y_k1_ymm
	MemorySize::Unknown,// EVEX_Vpscatterqq_vm64z_k1_zmm
	MemorySize::Unknown,// EVEX_Vscatterdps_vm32x_k1_xmm
	MemorySize::Unknown,// EVEX_Vscatterdps_vm32y_k1_ymm
	MemorySize::Unknown,// EVEX_Vscatterdps_vm32z_k1_zmm
	MemorySize::Unknown,// EVEX_Vscatterdpd_vm32x_k1_xmm
	MemorySize::Unknown,// EVEX_Vscatterdpd_vm32x_k1_ymm
	MemorySize::Unknown,// EVEX_Vscatterdpd_vm32y_k1_zmm
	MemorySize::Unknown,// EVEX_Vscatterqps_vm64x_k1_xmm
	MemorySize::Unknown,// EVEX_Vscatterqps_vm64y_k1_xmm
	MemorySize::Unknown,// EVEX_Vscatterqps_vm64z_k1_ymm
	MemorySize::Unknown,// EVEX_Vscatterqpd_vm64x_k1_xmm
	MemorySize::Unknown,// EVEX_Vscatterqpd_vm64y_k1_ymm
	MemorySize::Unknown,// EVEX_Vscatterqpd_vm64z_k1_zmm
	MemorySize::Unknown,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	MemorySize::Unknown,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	MemorySize::Unknown,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Broadcast128_UInt52,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt52,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt52,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Broadcast128_UInt52,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_UInt52,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_UInt52,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	MemorySize::Unknown,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	MemorySize::Broadcast128_Float32,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	MemorySize::Unknown,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	MemorySize::Broadcast128_UInt32,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	MemorySize::Broadcast128_UInt64,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	MemorySize::Unknown,// EVEX_Vgatherpf0dps_vm32z_k1
	MemorySize::Unknown,// EVEX_Vgatherpf0dpd_vm32y_k1
	MemorySize::Unknown,// EVEX_Vgatherpf1dps_vm32z_k1
	MemorySize::Unknown,// EVEX_Vgatherpf1dpd_vm32y_k1
	MemorySize::Unknown,// EVEX_Vscatterpf0dps_vm32z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf0dpd_vm32y_k1
	MemorySize::Unknown,// EVEX_Vscatterpf1dps_vm32z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf1dpd_vm32y_k1
	MemorySize::Unknown,// EVEX_Vgatherpf0qps_vm64z_k1
	MemorySize::Unknown,// EVEX_Vgatherpf0qpd_vm64z_k1
	MemorySize::Unknown,// EVEX_Vgatherpf1qps_vm64z_k1
	MemorySize::Unknown,// EVEX_Vgatherpf1qpd_vm64z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf0qps_vm64z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf0qpd_vm64z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf1qps_vm64z_k1
	MemorySize::Unknown,// EVEX_Vscatterpf1qpd_vm64z_k1
	MemorySize::Unknown,// Sha1nexte_xmm_xmmm128
	MemorySize::Broadcast512_Float32,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Broadcast512_Float64,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Sha1msg1_xmm_xmmm128
	MemorySize::Unknown,// Sha1msg2_xmm_xmmm128
	MemorySize::Broadcast512_Float32,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Broadcast512_Float64,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Sha256rnds2_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Unknown,// Sha256msg1_xmm_xmmm128
	MemorySize::Broadcast512_Float32,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	MemorySize::Broadcast512_Float64,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	MemorySize::Unknown,// Sha256msg2_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	MemorySize::Unknown,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	MemorySize::Unknown,// Gf2p8mulb_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	MemorySize::Unknown,// Aesimc_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesimc_xmm_xmmm128
	MemorySize::Unknown,// Aesenc_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesenc_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesenc_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesenc_xmm_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vaesenc_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesenc_zmm_zmm_zmmm512
	MemorySize::Unknown,// Aesenclast_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesenclast_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesenclast_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	MemorySize::Unknown,// Aesdec_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesdec_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesdec_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesdec_xmm_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vaesdec_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesdec_zmm_zmm_zmmm512
	MemorySize::Unknown,// Aesdeclast_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	MemorySize::Unknown,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	MemorySize::Unknown,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	MemorySize::Unknown,// Movbe_r16_m16
	MemorySize::Unknown,// Movbe_r32_m32
	MemorySize::Unknown,// Movbe_r64_m64
	MemorySize::Unknown,// Crc32_r32_rm8
	MemorySize::Unknown,// Crc32_r64_rm8
	MemorySize::Unknown,// Movbe_m16_r16
	MemorySize::Unknown,// Movbe_m32_r32
	MemorySize::Unknown,// Movbe_m64_r64
	MemorySize::Unknown,// Crc32_r32_rm16
	MemorySize::Unknown,// Crc32_r32_rm32
	MemorySize::Unknown,// Crc32_r64_rm64
	MemorySize::Unknown,// VEX_Andn_r32_r32_rm32
	MemorySize::Unknown,// VEX_Andn_r64_r64_rm64
	MemorySize::Unknown,// VEX_Blsr_r32_rm32
	MemorySize::Unknown,// VEX_Blsr_r64_rm64
	MemorySize::Unknown,// VEX_Blsmsk_r32_rm32
	MemorySize::Unknown,// VEX_Blsmsk_r64_rm64
	MemorySize::Unknown,// VEX_Blsi_r32_rm32
	MemorySize::Unknown,// VEX_Blsi_r64_rm64
	MemorySize::Unknown,// VEX_Bzhi_r32_rm32_r32
	MemorySize::Unknown,// VEX_Bzhi_r64_rm64_r64
	MemorySize::Unknown,// Wrussd_m32_r32
	MemorySize::Unknown,// Wrussq_m64_r64
	MemorySize::Unknown,// VEX_Pext_r32_r32_rm32
	MemorySize::Unknown,// VEX_Pext_r64_r64_rm64
	MemorySize::Unknown,// VEX_Pdep_r32_r32_rm32
	MemorySize::Unknown,// VEX_Pdep_r64_r64_rm64
	MemorySize::Unknown,// Wrssd_m32_r32
	MemorySize::Unknown,// Wrssq_m64_r64
	MemorySize::Unknown,// Adcx_r32_rm32
	MemorySize::Unknown,// Adcx_r64_rm64
	MemorySize::Unknown,// Adox_r32_rm32
	MemorySize::Unknown,// Adox_r64_rm64
	MemorySize::Unknown,// VEX_Mulx_r32_r32_rm32
	MemorySize::Unknown,// VEX_Mulx_r64_r64_rm64
	MemorySize::Unknown,// VEX_Bextr_r32_rm32_r32
	MemorySize::Unknown,// VEX_Bextr_r64_rm64_r64
	MemorySize::Unknown,// VEX_Shlx_r32_rm32_r32
	MemorySize::Unknown,// VEX_Shlx_r64_rm64_r64
	MemorySize::Unknown,// VEX_Sarx_r32_rm32_r32
	MemorySize::Unknown,// VEX_Sarx_r64_rm64_r64
	MemorySize::Unknown,// VEX_Shrx_r32_rm32_r32
	MemorySize::Unknown,// VEX_Shrx_r64_rm64_r64
	MemorySize::Unknown,// Movdir64b_r16_m512
	MemorySize::Unknown,// Movdir64b_r32_m512
	MemorySize::Unknown,// Movdir64b_r64_m512
	MemorySize::Unknown,// Enqcmds_r16_m512
	MemorySize::Unknown,// Enqcmds_r32_m512
	MemorySize::Unknown,// Enqcmds_r64_m512
	MemorySize::Unknown,// Enqcmd_r16_m512
	MemorySize::Unknown,// Enqcmd_r32_m512
	MemorySize::Unknown,// Enqcmd_r64_m512
	MemorySize::Unknown,// Movdiri_m32_r32
	MemorySize::Unknown,// Movdiri_m64_r64
	MemorySize::Unknown,// VEX_Vpermq_ymm_ymmm256_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// VEX_Vpermpd_ymm_ymmm256_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// VEX_Vpermilps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpermilps_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	MemorySize::Unknown,// VEX_Vpermilpd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpermilpd_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float64,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	MemorySize::Unknown,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// Roundps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vroundps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vroundps_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Unknown,// Roundpd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vroundpd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vroundpd_ymm_ymmm256_imm8
	MemorySize::Broadcast128_Float64,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Unknown,// Roundss_xmm_xmmm32_imm8
	MemorySize::Unknown,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	MemorySize::Unknown,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// Roundsd_xmm_xmmm64_imm8
	MemorySize::Unknown,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	MemorySize::Unknown,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Unknown,// Blendps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// Blendpd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// Pblendw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// Palignr_mm_mmm64_imm8
	MemorySize::Unknown,// Palignr_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Unknown,// Pextrb_r32m8_xmm_imm8
	MemorySize::Unknown,// Pextrb_r64m8_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrb_r32m8_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrb_r64m8_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrb_r32m8_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrb_r64m8_xmm_imm8
	MemorySize::Unknown,// Pextrw_r32m16_xmm_imm8
	MemorySize::Unknown,// Pextrw_r64m16_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r32m16_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrw_r64m16_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r32m16_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrw_r64m16_xmm_imm8
	MemorySize::Unknown,// Pextrd_rm32_xmm_imm8
	MemorySize::Unknown,// Pextrq_rm64_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrd_rm32_xmm_imm8
	MemorySize::Unknown,// VEX_Vpextrq_rm64_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrd_rm32_xmm_imm8
	MemorySize::Unknown,// EVEX_Vpextrq_rm64_xmm_imm8
	MemorySize::Unknown,// Extractps_rm32_xmm_imm8
	MemorySize::Unknown,// Extractps_r64m32_xmm_imm8
	MemorySize::Unknown,// VEX_Vextractps_rm32_xmm_imm8
	MemorySize::Unknown,// VEX_Vextractps_r64m32_xmm_imm8
	MemorySize::Unknown,// EVEX_Vextractps_rm32_xmm_imm8
	MemorySize::Unknown,// EVEX_Vextractps_r64m32_xmm_imm8
	MemorySize::Unknown,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vextractf128_xmmm128_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	MemorySize::Unknown,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	MemorySize::Unknown,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	MemorySize::Unknown,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	MemorySize::Unknown,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	MemorySize::Unknown,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	MemorySize::Broadcast128_UInt32,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	MemorySize::Broadcast128_Int32,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_Int32,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Int32,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_Int64,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_Int64,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Int64,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// Pinsrb_xmm_r32m8_imm8
	MemorySize::Unknown,// Pinsrb_xmm_r64m8_imm8
	MemorySize::Unknown,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	MemorySize::Unknown,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	MemorySize::Unknown,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	MemorySize::Unknown,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	MemorySize::Unknown,// Insertps_xmm_xmmm32_imm8
	MemorySize::Unknown,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	MemorySize::Unknown,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	MemorySize::Unknown,// Pinsrd_xmm_rm32_imm8
	MemorySize::Unknown,// Pinsrq_xmm_rm64_imm8
	MemorySize::Unknown,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	MemorySize::Unknown,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	MemorySize::Unknown,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	MemorySize::Unknown,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Unknown,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Unknown,// VEX_Kshiftrb_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrw_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrd_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftrq_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlb_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlw_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftld_kr_kr_imm8
	MemorySize::Unknown,// VEX_Kshiftlq_kr_kr_imm8
	MemorySize::Unknown,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vextracti128_xmmm128_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	MemorySize::Unknown,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	MemorySize::Unknown,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	MemorySize::Unknown,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	MemorySize::Unknown,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	MemorySize::Unknown,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	MemorySize::Unknown,// Dpps_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// Dppd_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// Mpsadbw_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// Pclmulqdq_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	MemorySize::Unknown,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	MemorySize::Unknown,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	MemorySize::Unknown,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	MemorySize::Unknown,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	MemorySize::Unknown,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	MemorySize::Unknown,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	MemorySize::Unknown,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	MemorySize::Unknown,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	MemorySize::Unknown,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	MemorySize::Unknown,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	MemorySize::Broadcast128_Float32,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	MemorySize::Unknown,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Broadcast128_Float32,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	MemorySize::Unknown,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Broadcast128_Float32,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	MemorySize::Broadcast128_Float64,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	MemorySize::Unknown,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	MemorySize::Unknown,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	MemorySize::Unknown,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// Pcmpestrm_xmm_xmmm128_imm8
	MemorySize::Unknown,// Pcmpestrm64_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	MemorySize::Unknown,// Pcmpestri_xmm_xmmm128_imm8
	MemorySize::Unknown,// Pcmpestri64_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	MemorySize::Unknown,// Pcmpistrm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	MemorySize::Unknown,// Pcmpistri_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	MemorySize::Broadcast128_Float32,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	MemorySize::Broadcast256_Float32,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	MemorySize::Broadcast512_Float32,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	MemorySize::Broadcast128_Float64,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	MemorySize::Broadcast256_Float64,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	MemorySize::Broadcast512_Float64,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	MemorySize::Unknown,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	MemorySize::Unknown,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	MemorySize::Unknown,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	MemorySize::Unknown,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	MemorySize::Unknown,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	MemorySize::Unknown,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	MemorySize::Broadcast128_UInt32,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	MemorySize::Broadcast256_UInt32,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	MemorySize::Broadcast512_UInt32,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	MemorySize::Unknown,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	MemorySize::Unknown,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	MemorySize::Unknown,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	MemorySize::Unknown,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	MemorySize::Unknown,// Sha1rnds4_xmm_xmmm128_imm8
	MemorySize::Unknown,// Gf2p8affineqb_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	MemorySize::Broadcast128_UInt64,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	MemorySize::Broadcast256_UInt64,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	MemorySize::Broadcast512_UInt64,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	MemorySize::Unknown,// Aeskeygenassist_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	MemorySize::Unknown,// VEX_Rorx_r32_rm32_imm8
	MemorySize::Unknown,// VEX_Rorx_r64_rm64_imm8
	MemorySize::Unknown,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	MemorySize::Unknown,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	MemorySize::Unknown,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vprotb_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vprotw_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vprotd_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vprotq_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	MemorySize::Unknown,// XOP_Blcfill_r32_rm32
	MemorySize::Unknown,// XOP_Blcfill_r64_rm64
	MemorySize::Unknown,// XOP_Blsfill_r32_rm32
	MemorySize::Unknown,// XOP_Blsfill_r64_rm64
	MemorySize::Unknown,// XOP_Blcs_r32_rm32
	MemorySize::Unknown,// XOP_Blcs_r64_rm64
	MemorySize::Unknown,// XOP_Tzmsk_r32_rm32
	MemorySize::Unknown,// XOP_Tzmsk_r64_rm64
	MemorySize::Unknown,// XOP_Blcic_r32_rm32
	MemorySize::Unknown,// XOP_Blcic_r64_rm64
	MemorySize::Unknown,// XOP_Blsic_r32_rm32
	MemorySize::Unknown,// XOP_Blsic_r64_rm64
	MemorySize::Unknown,// XOP_T1mskc_r32_rm32
	MemorySize::Unknown,// XOP_T1mskc_r64_rm64
	MemorySize::Unknown,// XOP_Blcmsk_r32_rm32
	MemorySize::Unknown,// XOP_Blcmsk_r64_rm64
	MemorySize::Unknown,// XOP_Blci_r32_rm32
	MemorySize::Unknown,// XOP_Blci_r64_rm64
	MemorySize::Unknown,// XOP_Llwpcb_r32
	MemorySize::Unknown,// XOP_Llwpcb_r64
	MemorySize::Unknown,// XOP_Slwpcb_r32
	MemorySize::Unknown,// XOP_Slwpcb_r64
	MemorySize::Unknown,// XOP_Vfrczps_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vfrczps_ymm_ymmm256
	MemorySize::Unknown,// XOP_Vfrczpd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vfrczpd_ymm_ymmm256
	MemorySize::Unknown,// XOP_Vfrczss_xmm_xmmm32
	MemorySize::Unknown,// XOP_Vfrczsd_xmm_xmmm64
	MemorySize::Unknown,// XOP_Vprotb_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vprotb_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vprotw_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vprotw_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vprotd_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vprotd_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vprotq_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vprotq_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshlb_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshlb_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshlw_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshlw_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshld_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshld_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshlq_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshlq_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshab_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshab_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshaw_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshaw_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshad_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshad_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vpshaq_xmm_xmmm128_xmm
	MemorySize::Unknown,// XOP_Vpshaq_xmm_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddbw_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddbd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddbq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddwd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddwq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphadddq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddubw_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddubd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddubq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphadduwd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphadduwq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphaddudq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphsubbw_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphsubwd_xmm_xmmm128
	MemorySize::Unknown,// XOP_Vphsubdq_xmm_xmmm128
	MemorySize::Unknown,// XOP_Bextr_r32_rm32_imm32
	MemorySize::Unknown,// XOP_Bextr_r64_rm64_imm32
	MemorySize::Unknown,// XOP_Lwpins_r32_rm32_imm32
	MemorySize::Unknown,// XOP_Lwpins_r64_rm32_imm32
	MemorySize::Unknown,// XOP_Lwpval_r32_rm32_imm32
	MemorySize::Unknown,// XOP_Lwpval_r64_rm32_imm32
	MemorySize::Unknown,// D3NOW_Pi2fw_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pi2fd_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pf2iw_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pf2id_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrcpv_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrsqrtv_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfnacc_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfpnacc_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfcmpge_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfmin_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrcp_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrsqrt_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfsub_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfadd_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfcmpgt_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfmax_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrcpit1_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrsqit1_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfsubr_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfacc_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfcmpeq_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfmul_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pfrcpit2_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pmulhrw_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pswapd_mm_mmm64
	MemorySize::Unknown,// D3NOW_Pavgusb_mm_mmm64
	MemorySize::Unknown,// Rmpadjust
	MemorySize::Unknown,// Rmpupdate
	MemorySize::Unknown,// Psmash
	MemorySize::Unknown,// Pvalidatew
	MemorySize::Unknown,// Pvalidated
	MemorySize::Unknown,// Pvalidateq
	MemorySize::Unknown,// Serialize
	MemorySize::Unknown,// Xsusldtrk
	MemorySize::Unknown,// Xresldtrk
	MemorySize::Unknown,// Invlpgbw
	MemorySize::Unknown,// Invlpgbd
	MemorySize::Unknown,// Invlpgbq
	MemorySize::Unknown,// Tlbsync
	MemorySize::Unknown,// Prefetchreserved3_m8
	MemorySize::Unknown,// Prefetchreserved4_m8
	MemorySize::Unknown,// Prefetchreserved5_m8
	MemorySize::Unknown,// Prefetchreserved6_m8
	MemorySize::Unknown,// Prefetchreserved7_m8
	MemorySize::Unknown,// Ud0
	MemorySize::Unknown,// Vmgexit
	MemorySize::Unknown,// Getsecq
	MemorySize::Unknown,// VEX_Ldtilecfg_m512
	MemorySize::Unknown,// VEX_Tilerelease
	MemorySize::Unknown,// VEX_Sttilecfg_m512
	MemorySize::Unknown,// VEX_Tilezero_tmm
	MemorySize::Unknown,// VEX_Tileloaddt1_tmm_sibmem
	MemorySize::Unknown,// VEX_Tilestored_sibmem_tmm
	MemorySize::Unknown,// VEX_Tileloadd_tmm_sibmem
	MemorySize::Unknown,// VEX_Tdpbf16ps_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbuud_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbusd_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbsud_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tdpbssd_tmm_tmm_tmm
	MemorySize::Unknown,// Fnstdw_AX
	MemorySize::Unknown,// Fnstsg_AX
	MemorySize::Unknown,// Rdshr_rm32
	MemorySize::Unknown,// Wrshr_rm32
	MemorySize::Unknown,// Smint
	MemorySize::Unknown,// Dmint
	MemorySize::Unknown,// Rdm
	MemorySize::Unknown,// Svdc_m80_Sreg
	MemorySize::Unknown,// Rsdc_Sreg_m80
	MemorySize::Unknown,// Svldt_m80
	MemorySize::Unknown,// Rsldt_m80
	MemorySize::Unknown,// Svts_m80
	MemorySize::Unknown,// Rsts_m80
	MemorySize::Unknown,// Smint_0F7E
	MemorySize::Unknown,// Bb0_reset
	MemorySize::Unknown,// Bb1_reset
	MemorySize::Unknown,// Cpu_write
	MemorySize::Unknown,// Cpu_read
	MemorySize::Unknown,// Altinst
	MemorySize::Unknown,// Paveb_mm_mmm64
	MemorySize::Unknown,// Paddsiw_mm_mmm64
	MemorySize::Unknown,// Pmagw_mm_mmm64
	MemorySize::Unknown,// Pdistib_mm_m64
	MemorySize::Unknown,// Psubsiw_mm_mmm64
	MemorySize::Unknown,// Pmvzb_mm_m64
	MemorySize::Unknown,// Pmulhrw_mm_mmm64
	MemorySize::Unknown,// Pmvnzb_mm_m64
	MemorySize::Unknown,// Pmvlzb_mm_m64
	MemorySize::Unknown,// Pmvgezb_mm_m64
	MemorySize::Unknown,// Pmulhriw_mm_mmm64
	MemorySize::Unknown,// Pmachriw_mm_m64
	MemorySize::Unknown,// Cyrix_D9D7
	MemorySize::Unknown,// Cyrix_D9E2
	MemorySize::Unknown,// Ftstp
	MemorySize::Unknown,// Cyrix_D9E7
	MemorySize::Unknown,// Frint2
	MemorySize::Unknown,// Frichop
	MemorySize::Unknown,// Cyrix_DED8
	MemorySize::Unknown,// Cyrix_DEDA
	MemorySize::Unknown,// Cyrix_DEDC
	MemorySize::Unknown,// Cyrix_DEDD
	MemorySize::Unknown,// Cyrix_DEDE
	MemorySize::Unknown,// Frinear
	MemorySize::Unknown,// Tdcall
	MemorySize::Unknown,// Seamret
	MemorySize::Unknown,// Seamops
	MemorySize::Unknown,// Seamcall
	MemorySize::Unknown,// Aesencwide128kl_m384
	MemorySize::Unknown,// Aesdecwide128kl_m384
	MemorySize::Unknown,// Aesencwide256kl_m512
	MemorySize::Unknown,// Aesdecwide256kl_m512
	MemorySize::Unknown,// Loadiwkey_xmm_xmm
	MemorySize::Unknown,// Aesenc128kl_xmm_m384
	MemorySize::Unknown,// Aesdec128kl_xmm_m384
	MemorySize::Unknown,// Aesenc256kl_xmm_m512
	MemorySize::Unknown,// Aesdec256kl_xmm_m512
	MemorySize::Unknown,// Encodekey128_r32_r32
	MemorySize::Unknown,// Encodekey256_r32_r32
	MemorySize::Unknown,// VEX_Vbroadcastss_xmm_xmm
	MemorySize::Unknown,// VEX_Vbroadcastss_ymm_xmm
	MemorySize::Unknown,// VEX_Vbroadcastsd_ymm_xmm
	MemorySize::Unknown,// Vmgexit_F2
	MemorySize::Unknown,// Uiret
	MemorySize::Unknown,// Testui
	MemorySize::Unknown,// Clui
	MemorySize::Unknown,// Stui
	MemorySize::Unknown,// Senduipi_r64
	MemorySize::Unknown,// Hreset_imm8
	MemorySize::Unknown,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	MemorySize::Unknown,// Ccs_hash_16
	MemorySize::Unknown,// Ccs_hash_32
	MemorySize::Unknown,// Ccs_hash_64
	MemorySize::Unknown,// Ccs_encrypt_16
	MemorySize::Unknown,// Ccs_encrypt_32
	MemorySize::Unknown,// Ccs_encrypt_64
	MemorySize::Unknown,// Lkgs_rm16
	MemorySize::Unknown,// Lkgs_r32m16
	MemorySize::Unknown,// Lkgs_r64m16
	MemorySize::Unknown,// Eretu
	MemorySize::Unknown,// Erets
	MemorySize::Broadcast128_Float16,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	MemorySize::Broadcast256_Float16,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	MemorySize::Broadcast512_Float16,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	MemorySize::Unknown,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	MemorySize::Unknown,// EVEX_Vcomish_xmm_xmmm16_sae
	MemorySize::Broadcast128_Int32,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Int32,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	MemorySize::Broadcast512_Int32,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Float64,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	MemorySize::Broadcast32_Float16,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	MemorySize::Broadcast32_Float16,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	MemorySize::Broadcast32_Float16,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	MemorySize::Broadcast64_Float16,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	MemorySize::Broadcast128_Float32,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_Float32,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	MemorySize::Broadcast512_Float32,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_Int64,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	MemorySize::Unknown,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vcvtsh2si_r32_xmmm16_er
	MemorySize::Unknown,// EVEX_Vcvtsh2si_r64_xmmm16_er
	MemorySize::Unknown,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	MemorySize::Unknown,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	MemorySize::Unknown,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	MemorySize::Unknown,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Broadcast64_Float16,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	MemorySize::Broadcast32_Float16,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	MemorySize::Broadcast64_Float16,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	MemorySize::Broadcast64_Float16,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	MemorySize::Broadcast32_Float16,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	MemorySize::Broadcast64_Float16,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	MemorySize::Unknown,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	MemorySize::Broadcast128_UInt32,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	MemorySize::Broadcast256_UInt32,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	MemorySize::Broadcast512_UInt32,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	MemorySize::Broadcast128_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	MemorySize::Broadcast256_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	MemorySize::Broadcast512_UInt64,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	MemorySize::Unknown,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	MemorySize::Unknown,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	MemorySize::Broadcast128_UInt16,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_UInt16,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_UInt16,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	MemorySize::Broadcast128_Int16,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Int16,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Int16,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_2xFloat16,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xFloat16,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xFloat16,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_2xFloat16,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xFloat16,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xFloat16,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Broadcast128_2xFloat16,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xFloat16,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xFloat16,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Broadcast128_2xFloat16,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	MemorySize::Broadcast256_2xFloat16,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	MemorySize::Broadcast512_2xFloat16,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	MemorySize::Unknown,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Unknown,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	MemorySize::Broadcast256_Float16,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	MemorySize::Broadcast512_Float16,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	MemorySize::Unknown,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	MemorySize::Broadcast128_Float16,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	MemorySize::Unknown,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Broadcast256_Float16,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Broadcast512_Float16,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Unknown,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	MemorySize::Unknown,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	MemorySize::Unknown,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	MemorySize::Unknown,// EVEX_Vmovsh_xmm_k1z_m16
	MemorySize::Unknown,// EVEX_Vmovsh_m16_k1_xmm
	MemorySize::Unknown,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	MemorySize::Unknown,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	MemorySize::Unknown,// EVEX_Vmovw_xmm_r32m16
	MemorySize::Unknown,// EVEX_Vmovw_xmm_r64m16
	MemorySize::Unknown,// EVEX_Vmovw_r32m16_xmm
	MemorySize::Unknown,// EVEX_Vmovw_r64m16_xmm
	MemorySize::Broadcast128_Float16,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	MemorySize::Unknown,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	MemorySize::Broadcast128_Float16,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Broadcast256_Float16,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Broadcast512_Float16,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Unknown,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	MemorySize::Broadcast256_Float16,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	MemorySize::Broadcast512_Float16,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	MemorySize::Unknown,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	MemorySize::Broadcast128_Float16,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	MemorySize::Unknown,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	MemorySize::Broadcast128_Float16,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Broadcast128_Float16,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	MemorySize::Broadcast256_Float16,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	MemorySize::Broadcast512_Float16,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	MemorySize::Unknown,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	MemorySize::Unknown,// EVEX_Vucomish_xmm_xmmm16_sae
	MemorySize::Unknown,// Rdudbg
	MemorySize::Unknown,// Wrudbg
	MemorySize::Unknown,// VEX_KNC_Jkzd_kr_rel8_64
	MemorySize::Unknown,// VEX_KNC_Jknzd_kr_rel8_64
	MemorySize::Unknown,// VEX_KNC_Vprefetchnta_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetch0_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetch1_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetch2_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetchenta_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetche0_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetche1_m8
	MemorySize::Unknown,// VEX_KNC_Vprefetche2_m8
	MemorySize::Unknown,// VEX_KNC_Kand_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kandn_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kandnr_kr_kr
	MemorySize::Unknown,// VEX_KNC_Knot_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kxnor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kxor_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmerge2l1h_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmerge2l1l_kr_kr
	MemorySize::Unknown,// VEX_KNC_Jkzd_kr_rel32_64
	MemorySize::Unknown,// VEX_KNC_Jknzd_kr_rel32_64
	MemorySize::Unknown,// VEX_KNC_Kmov_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kmov_kr_r32
	MemorySize::Unknown,// VEX_KNC_Kmov_r32_kr
	MemorySize::Unknown,// VEX_KNC_Kconcath_r64_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kconcatl_r64_kr_kr
	MemorySize::Unknown,// VEX_KNC_Kortest_kr_kr
	MemorySize::Unknown,// VEX_KNC_Delay_r32
	MemorySize::Unknown,// VEX_KNC_Delay_r64
	MemorySize::Unknown,// VEX_KNC_Spflt_r32
	MemorySize::Unknown,// VEX_KNC_Spflt_r64
	MemorySize::Unknown,// VEX_KNC_Clevict1_m8
	MemorySize::Unknown,// VEX_KNC_Clevict0_m8
	MemorySize::Unknown,// VEX_KNC_Popcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Popcnt_r64_r64
	MemorySize::Unknown,// VEX_KNC_Tzcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Tzcnt_r64_r64
	MemorySize::Unknown,// VEX_KNC_Tzcnti_r32_r32
	MemorySize::Unknown,// VEX_KNC_Tzcnti_r64_r64
	MemorySize::Unknown,// VEX_KNC_Lzcnt_r32_r32
	MemorySize::Unknown,// VEX_KNC_Lzcnt_r64_r64
	MemorySize::Unknown,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	MemorySize::Unknown,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	MemorySize::Unknown,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	MemorySize::Unknown,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	MemorySize::Unknown,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	MemorySize::Unknown,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	MemorySize::Unknown,// VEX_KNC_Kextract_kr_r64_imm8
	MemorySize::Unknown,// MVEX_Vprefetchnta_m
	MemorySize::Unknown,// MVEX_Vprefetch0_m
	MemorySize::Unknown,// MVEX_Vprefetch1_m
	MemorySize::Unknown,// MVEX_Vprefetch2_m
	MemorySize::Unknown,// MVEX_Vprefetchenta_m
	MemorySize::Unknown,// MVEX_Vprefetche0_m
	MemorySize::Unknown,// MVEX_Vprefetche1_m
	MemorySize::Unknown,// MVEX_Vprefetche2_m
	MemorySize::Unknown,// MVEX_Vmovaps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovapd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovaps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovapd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrapd_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrngoapd_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnraps_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovnrngoaps_m_k1_zmm
	MemorySize::Unknown,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vmovdqa32_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vmovdqa64_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Clevict1_m
	MemorySize::Unknown,// MVEX_Clevict0_m
	MemorySize::Unknown,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vbroadcastss_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastsd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgetexpps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vgetexppd_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	MemorySize::Unknown,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	MemorySize::Unknown,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	MemorySize::Unknown,// MVEX_Vpbroadcastd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpbroadcastq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	MemorySize::Unknown,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	MemorySize::Unknown,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpgatherdd_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vpgatherdq_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vgatherdps_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Vgatherdpd_zmm_k1_mvt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	MemorySize::Unknown,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpscatterdd_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpscatterdq_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vscatterdps_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vscatterdpd_mvt_k1_zmm
	MemorySize::Unknown,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	MemorySize::Unknown,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	MemorySize::Unknown,// MVEX_Vgatherpf0hintdps_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf0hintdpd_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf0dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vgatherpf1dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0hintdps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0hintdpd_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf0dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vscatterpf1dps_mvt_k1
	MemorySize::Unknown,// MVEX_Vexp223ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vlog2ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	MemorySize::Unknown,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	MemorySize::Unknown,// MVEX_Vloadunpackld_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpacklq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstoreld_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorelq_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vloadunpacklps_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpacklpd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorelps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorelpd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	MemorySize::Unknown,// MVEX_Vloadunpackhd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpackhq_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorehd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorehq_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vloadunpackhps_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vloadunpackhpd_zmm_k1_mt
	MemorySize::Unknown,// MVEX_Vpackstorehps_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Vpackstorehpd_mt_k1_zmm
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	MemorySize::Unknown,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	MemorySize::Unknown,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	MemorySize::Unknown,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	MemorySize::Unknown,// Via_undoc_F30FA6F0_16
	MemorySize::Unknown,// Via_undoc_F30FA6F0_32
	MemorySize::Unknown,// Via_undoc_F30FA6F0_64
	MemorySize::Unknown,// Via_undoc_F30FA6F8_16
	MemorySize::Unknown,// Via_undoc_F30FA6F8_32
	MemorySize::Unknown,// Via_undoc_F30FA6F8_64
	MemorySize::Unknown,// Xsha512_16
	MemorySize::Unknown,// Xsha512_32
	MemorySize::Unknown,// Xsha512_64
	MemorySize::Unknown,// Xstore_alt_16
	MemorySize::Unknown,// Xstore_alt_32
	MemorySize::Unknown,// Xstore_alt_64
	MemorySize::Unknown,// Xsha512_alt_16
	MemorySize::Unknown,// Xsha512_alt_32
	MemorySize::Unknown,// Xsha512_alt_64
	MemorySize::Unknown,// Zero_bytes
	MemorySize::Unknown,// Wrmsrns
	MemorySize::Unknown,// Wrmsrlist
	MemorySize::Unknown,// Rdmsrlist
	MemorySize::Unknown,// Rmpquery
	MemorySize::Unknown,// Prefetchit1_m8
	MemorySize::Unknown,// Prefetchit0_m8
	MemorySize::Unknown,// Aadd_m32_r32
	MemorySize::Unknown,// Aadd_m64_r64
	MemorySize::Unknown,// Aand_m32_r32
	MemorySize::Unknown,// Aand_m64_r64
	MemorySize::Unknown,// Axor_m32_r32
	MemorySize::Unknown,// Axor_m64_r64
	MemorySize::Unknown,// Aor_m32_r32
	MemorySize::Unknown,// Aor_m64_r64
	MemorySize::Unknown,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Tdpfp16ps_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Vcvtneps2bf16_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vcvtneps2bf16_xmm_ymmm256
	MemorySize::Unknown,// VEX_Vcvtneoph2ps_xmm_m128
	MemorySize::Unknown,// VEX_Vcvtneoph2ps_ymm_m256
	MemorySize::Unknown,// VEX_Vcvtneeph2ps_xmm_m128
	MemorySize::Unknown,// VEX_Vcvtneeph2ps_ymm_m256
	MemorySize::Unknown,// VEX_Vcvtneebf162ps_xmm_m128
	MemorySize::Unknown,// VEX_Vcvtneebf162ps_ymm_m256
	MemorySize::Unknown,// VEX_Vcvtneobf162ps_xmm_m128
	MemorySize::Unknown,// VEX_Vcvtneobf162ps_ymm_m256
	MemorySize::Unknown,// VEX_Vbcstnesh2ps_xmm_m16
	MemorySize::Unknown,// VEX_Vbcstnesh2ps_ymm_m16
	MemorySize::Unknown,// VEX_Vbcstnebf162ps_xmm_m16
	MemorySize::Unknown,// VEX_Vbcstnebf162ps_ymm_m16
	MemorySize::Unknown,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Cmpoxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpoxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnoxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnoxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpbxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpbxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnbxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnbxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpzxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpzxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnzxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnzxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpbexadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpbexadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnbexadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnbexadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpsxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpsxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnsxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnsxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmppxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmppxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnpxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnpxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmplxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmplxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnlxadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnlxadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmplexadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmplexadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Cmpnlexadd_m32_r32_r32
	MemorySize::Unknown,// VEX_Cmpnlexadd_m64_r64_r64
	MemorySize::Unknown,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	MemorySize::Unknown,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	MemorySize::Unknown,// Pbndkb
	MemorySize::Unknown,// VEX_Vsha512rnds2_ymm_ymm_xmm
	MemorySize::Unknown,// VEX_Vsha512msg1_ymm_xmm
	MemorySize::Unknown,// VEX_Vsha512msg2_ymm_ymm
	MemorySize::Unknown,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsm4key4_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsm4key4_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	MemorySize::Unknown,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	MemorySize::Unknown,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];
