// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

#[rustfmt::skip]
pub(super) static OPC_FLAGS1: [u32; 4936] = [
	0x0000_0000,// INVALID
	0x0000_0000,// DeclareByte
	0x0000_0000,// DeclareWord
	0x0000_0000,// DeclareDword
	0x0000_0000,// DeclareQword
	0x0000_0000,// Add_rm8_r8
	0x0000_0000,// Add_rm16_r16
	0x0000_0000,// Add_rm32_r32
	0x0000_0000,// Add_rm64_r64
	0x0000_0000,// Add_r8_rm8
	0x0000_0000,// Add_r16_rm16
	0x0000_0000,// Add_r32_rm32
	0x0000_0000,// Add_r64_rm64
	0x0000_0000,// Add_AL_imm8
	0x0000_0000,// Add_AX_imm16
	0x0000_0000,// Add_EAX_imm32
	0x0000_0000,// Add_RAX_imm32
	0x0002_0000,// Pushw_ES
	0x0002_0000,// Pushd_ES
	0x0002_0000,// Popw_ES
	0x0002_0000,// Popd_ES
	0x0000_0000,// Or_rm8_r8
	0x0000_0000,// Or_rm16_r16
	0x0000_0000,// Or_rm32_r32
	0x0000_0000,// Or_rm64_r64
	0x0000_0000,// Or_r8_rm8
	0x0000_0000,// Or_r16_rm16
	0x0000_0000,// Or_r32_rm32
	0x0000_0000,// Or_r64_rm64
	0x0000_0000,// Or_AL_imm8
	0x0000_0000,// Or_AX_imm16
	0x0000_0000,// Or_EAX_imm32
	0x0000_0000,// Or_RAX_imm32
	0x0002_0000,// Pushw_CS
	0x0002_0000,// Pushd_CS
	0x0002_0000,// Popw_CS
	0x0000_0000,// Adc_rm8_r8
	0x0000_0000,// Adc_rm16_r16
	0x0000_0000,// Adc_rm32_r32
	0x0000_0000,// Adc_rm64_r64
	0x0000_0000,// Adc_r8_rm8
	0x0000_0000,// Adc_r16_rm16
	0x0000_0000,// Adc_r32_rm32
	0x0000_0000,// Adc_r64_rm64
	0x0000_0000,// Adc_AL_imm8
	0x0000_0000,// Adc_AX_imm16
	0x0000_0000,// Adc_EAX_imm32
	0x0000_0000,// Adc_RAX_imm32
	0x0002_0000,// Pushw_SS
	0x0002_0000,// Pushd_SS
	0x0002_0000,// Popw_SS
	0x0002_0000,// Popd_SS
	0x0000_0000,// Sbb_rm8_r8
	0x0000_0000,// Sbb_rm16_r16
	0x0000_0000,// Sbb_rm32_r32
	0x0000_0000,// Sbb_rm64_r64
	0x0000_0000,// Sbb_r8_rm8
	0x0000_0000,// Sbb_r16_rm16
	0x0000_0000,// Sbb_r32_rm32
	0x0000_0000,// Sbb_r64_rm64
	0x0000_0000,// Sbb_AL_imm8
	0x0000_0000,// Sbb_AX_imm16
	0x0000_0000,// Sbb_EAX_imm32
	0x0000_0000,// Sbb_RAX_imm32
	0x0002_0000,// Pushw_DS
	0x0002_0000,// Pushd_DS
	0x0002_0000,// Popw_DS
	0x0002_0000,// Popd_DS
	0x0000_0000,// And_rm8_r8
	0x0000_0000,// And_rm16_r16
	0x0000_0000,// And_rm32_r32
	0x0000_0000,// And_rm64_r64
	0x0000_0000,// And_r8_rm8
	0x0000_0000,// And_r16_rm16
	0x0000_0000,// And_r32_rm32
	0x0000_0000,// And_r64_rm64
	0x0000_0000,// And_AL_imm8
	0x0000_0000,// And_AX_imm16
	0x0000_0000,// And_EAX_imm32
	0x0000_0000,// And_RAX_imm32
	0x0000_0000,// Daa
	0x0000_0000,// Sub_rm8_r8
	0x0000_0000,// Sub_rm16_r16
	0x0000_0000,// Sub_rm32_r32
	0x0000_0000,// Sub_rm64_r64
	0x0000_0000,// Sub_r8_rm8
	0x0000_0000,// Sub_r16_rm16
	0x0000_0000,// Sub_r32_rm32
	0x0000_0000,// Sub_r64_rm64
	0x0000_0000,// Sub_AL_imm8
	0x0000_0000,// Sub_AX_imm16
	0x0000_0000,// Sub_EAX_imm32
	0x0000_0000,// Sub_RAX_imm32
	0x0000_0000,// Das
	0x0000_0000,// Xor_rm8_r8
	0x0000_0000,// Xor_rm16_r16
	0x0000_0000,// Xor_rm32_r32
	0x0000_0000,// Xor_rm64_r64
	0x0000_0000,// Xor_r8_rm8
	0x0000_0000,// Xor_r16_rm16
	0x0000_0000,// Xor_r32_rm32
	0x0000_0000,// Xor_r64_rm64
	0x0000_0000,// Xor_AL_imm8
	0x0000_0000,// Xor_AX_imm16
	0x0000_0000,// Xor_EAX_imm32
	0x0000_0000,// Xor_RAX_imm32
	0x0000_0000,// Aaa
	0x0000_0000,// Cmp_rm8_r8
	0x0000_0000,// Cmp_rm16_r16
	0x0000_0000,// Cmp_rm32_r32
	0x0000_0000,// Cmp_rm64_r64
	0x0000_0000,// Cmp_r8_rm8
	0x0000_0000,// Cmp_r16_rm16
	0x0000_0000,// Cmp_r32_rm32
	0x0000_0000,// Cmp_r64_rm64
	0x0000_0000,// Cmp_AL_imm8
	0x0000_0000,// Cmp_AX_imm16
	0x0000_0000,// Cmp_EAX_imm32
	0x0000_0000,// Cmp_RAX_imm32
	0x0000_0000,// Aas
	0x0000_0000,// Inc_r16
	0x0000_0000,// Inc_r32
	0x0000_0000,// Dec_r16
	0x0000_0000,// Dec_r32
	0x0002_0000,// Push_r16
	0x0002_0000,// Push_r32
	0x0002_0000,// Push_r64
	0x0002_0000,// Pop_r16
	0x0002_0000,// Pop_r32
	0x0002_0000,// Pop_r64
	0x0002_0000,// Pushaw
	0x0002_0000,// Pushad
	0x0002_0000,// Popaw
	0x0002_0000,// Popad
	0x0000_0000,// Bound_r16_m1616
	0x0000_0000,// Bound_r32_m3232
	0x0000_0000,// Arpl_rm16_r16
	0x0000_0000,// Arpl_r32m16_r32
	0x0000_0000,// Movsxd_r16_rm16
	0x0000_0000,// Movsxd_r32_rm32
	0x0000_0000,// Movsxd_r64_rm32
	0x0002_0000,// Push_imm16
	0x0002_0000,// Pushd_imm32
	0x0002_0000,// Pushq_imm32
	0x0000_0000,// Imul_r16_rm16_imm16
	0x0000_0000,// Imul_r32_rm32_imm32
	0x0000_0000,// Imul_r64_rm64_imm32
	0x0002_0000,// Pushw_imm8
	0x0002_0000,// Pushd_imm8
	0x0002_0000,// Pushq_imm8
	0x0000_0000,// Imul_r16_rm16_imm8
	0x0000_0000,// Imul_r32_rm32_imm8
	0x0000_0000,// Imul_r64_rm64_imm8
	0x0000_8004,// Insb_m8_DX
	0x0000_8004,// Insw_m16_DX
	0x0000_8004,// Insd_m32_DX
	0x0000_8004,// Outsb_DX_m8
	0x0000_8004,// Outsw_DX_m16
	0x0000_8004,// Outsd_DX_m32
	0x0000_0000,// Jo_rel8_16
	0x0000_0000,// Jo_rel8_32
	0x0000_0000,// Jo_rel8_64
	0x0000_0000,// Jno_rel8_16
	0x0000_0000,// Jno_rel8_32
	0x0000_0000,// Jno_rel8_64
	0x0000_0000,// Jb_rel8_16
	0x0000_0000,// Jb_rel8_32
	0x0000_0000,// Jb_rel8_64
	0x0000_0000,// Jae_rel8_16
	0x0000_0000,// Jae_rel8_32
	0x0000_0000,// Jae_rel8_64
	0x0000_0000,// Je_rel8_16
	0x0000_0000,// Je_rel8_32
	0x0000_0000,// Je_rel8_64
	0x0000_0000,// Jne_rel8_16
	0x0000_0000,// Jne_rel8_32
	0x0000_0000,// Jne_rel8_64
	0x0000_0000,// Jbe_rel8_16
	0x0000_0000,// Jbe_rel8_32
	0x0000_0000,// Jbe_rel8_64
	0x0000_0000,// Ja_rel8_16
	0x0000_0000,// Ja_rel8_32
	0x0000_0000,// Ja_rel8_64
	0x0000_0000,// Js_rel8_16
	0x0000_0000,// Js_rel8_32
	0x0000_0000,// Js_rel8_64
	0x0000_0000,// Jns_rel8_16
	0x0000_0000,// Jns_rel8_32
	0x0000_0000,// Jns_rel8_64
	0x0000_0000,// Jp_rel8_16
	0x0000_0000,// Jp_rel8_32
	0x0000_0000,// Jp_rel8_64
	0x0000_0000,// Jnp_rel8_16
	0x0000_0000,// Jnp_rel8_32
	0x0000_0000,// Jnp_rel8_64
	0x0000_0000,// Jl_rel8_16
	0x0000_0000,// Jl_rel8_32
	0x0000_0000,// Jl_rel8_64
	0x0000_0000,// Jge_rel8_16
	0x0000_0000,// Jge_rel8_32
	0x0000_0000,// Jge_rel8_64
	0x0000_0000,// Jle_rel8_16
	0x0000_0000,// Jle_rel8_32
	0x0000_0000,// Jle_rel8_64
	0x0000_0000,// Jg_rel8_16
	0x0000_0000,// Jg_rel8_32
	0x0000_0000,// Jg_rel8_64
	0x0000_0000,// Add_rm8_imm8
	0x0000_0000,// Or_rm8_imm8
	0x0000_0000,// Adc_rm8_imm8
	0x0000_0000,// Sbb_rm8_imm8
	0x0000_0000,// And_rm8_imm8
	0x0000_0000,// Sub_rm8_imm8
	0x0000_0000,// Xor_rm8_imm8
	0x0000_0000,// Cmp_rm8_imm8
	0x0000_0000,// Add_rm16_imm16
	0x0000_0000,// Add_rm32_imm32
	0x0000_0000,// Add_rm64_imm32
	0x0000_0000,// Or_rm16_imm16
	0x0000_0000,// Or_rm32_imm32
	0x0000_0000,// Or_rm64_imm32
	0x0000_0000,// Adc_rm16_imm16
	0x0000_0000,// Adc_rm32_imm32
	0x0000_0000,// Adc_rm64_imm32
	0x0000_0000,// Sbb_rm16_imm16
	0x0000_0000,// Sbb_rm32_imm32
	0x0000_0000,// Sbb_rm64_imm32
	0x0000_0000,// And_rm16_imm16
	0x0000_0000,// And_rm32_imm32
	0x0000_0000,// And_rm64_imm32
	0x0000_0000,// Sub_rm16_imm16
	0x0000_0000,// Sub_rm32_imm32
	0x0000_0000,// Sub_rm64_imm32
	0x0000_0000,// Xor_rm16_imm16
	0x0000_0000,// Xor_rm32_imm32
	0x0000_0000,// Xor_rm64_imm32
	0x0000_0000,// Cmp_rm16_imm16
	0x0000_0000,// Cmp_rm32_imm32
	0x0000_0000,// Cmp_rm64_imm32
	0x0000_0000,// Add_rm8_imm8_82
	0x0000_0000,// Or_rm8_imm8_82
	0x0000_0000,// Adc_rm8_imm8_82
	0x0000_0000,// Sbb_rm8_imm8_82
	0x0000_0000,// And_rm8_imm8_82
	0x0000_0000,// Sub_rm8_imm8_82
	0x0000_0000,// Xor_rm8_imm8_82
	0x0000_0000,// Cmp_rm8_imm8_82
	0x0000_0000,// Add_rm16_imm8
	0x0000_0000,// Add_rm32_imm8
	0x0000_0000,// Add_rm64_imm8
	0x0000_0000,// Or_rm16_imm8
	0x0000_0000,// Or_rm32_imm8
	0x0000_0000,// Or_rm64_imm8
	0x0000_0000,// Adc_rm16_imm8
	0x0000_0000,// Adc_rm32_imm8
	0x0000_0000,// Adc_rm64_imm8
	0x0000_0000,// Sbb_rm16_imm8
	0x0000_0000,// Sbb_rm32_imm8
	0x0000_0000,// Sbb_rm64_imm8
	0x0000_0000,// And_rm16_imm8
	0x0000_0000,// And_rm32_imm8
	0x0000_0000,// And_rm64_imm8
	0x0000_0000,// Sub_rm16_imm8
	0x0000_0000,// Sub_rm32_imm8
	0x0000_0000,// Sub_rm64_imm8
	0x0000_0000,// Xor_rm16_imm8
	0x0000_0000,// Xor_rm32_imm8
	0x0000_0000,// Xor_rm64_imm8
	0x0000_0000,// Cmp_rm16_imm8
	0x0000_0000,// Cmp_rm32_imm8
	0x0000_0000,// Cmp_rm64_imm8
	0x0000_0000,// Test_rm8_r8
	0x0000_0000,// Test_rm16_r16
	0x0000_0000,// Test_rm32_r32
	0x0000_0000,// Test_rm64_r64
	0x0000_0000,// Xchg_rm8_r8
	0x0000_0000,// Xchg_rm16_r16
	0x0000_0000,// Xchg_rm32_r32
	0x0000_0000,// Xchg_rm64_r64
	0x0000_0000,// Mov_rm8_r8
	0x0000_0000,// Mov_rm16_r16
	0x0000_0000,// Mov_rm32_r32
	0x0000_0000,// Mov_rm64_r64
	0x0000_0000,// Mov_r8_rm8
	0x0000_0000,// Mov_r16_rm16
	0x0000_0000,// Mov_r32_rm32
	0x0000_0000,// Mov_r64_rm64
	0x0000_0000,// Mov_rm16_Sreg
	0x0000_0000,// Mov_r32m16_Sreg
	0x0000_0000,// Mov_r64m16_Sreg
	0x0004_0000,// Lea_r16_m
	0x0004_0000,// Lea_r32_m
	0x0004_0000,// Lea_r64_m
	0x0000_0000,// Mov_Sreg_rm16
	0x0000_0000,// Mov_Sreg_r32m16
	0x0000_0000,// Mov_Sreg_r64m16
	0x0002_0000,// Pop_rm16
	0x0002_0000,// Pop_rm32
	0x0002_0000,// Pop_rm64
	0x0000_0008,// Nopw
	0x0000_0008,// Nopd
	0x0000_0008,// Nopq
	0x0000_0000,// Xchg_r16_AX
	0x0000_0000,// Xchg_r32_EAX
	0x0000_0000,// Xchg_r64_RAX
	0x0000_0000,// Pause
	0x0000_0000,// Cbw
	0x0000_0000,// Cwde
	0x0000_0000,// Cdqe
	0x0000_0000,// Cwd
	0x0000_0000,// Cdq
	0x0000_0000,// Cqo
	0x0002_0000,// Call_ptr1616
	0x0002_0000,// Call_ptr1632
	0x0000_0000,// Wait
	0x0002_0000,// Pushfw
	0x0002_0000,// Pushfd
	0x0002_0000,// Pushfq
	0x0002_0000,// Popfw
	0x0002_0000,// Popfd
	0x0002_0000,// Popfq
	0x0000_0000,// Sahf
	0x0000_0000,// Lahf
	0x0000_0000,// Mov_AL_moffs8
	0x0000_0000,// Mov_AX_moffs16
	0x0000_0000,// Mov_EAX_moffs32
	0x0000_0000,// Mov_RAX_moffs64
	0x0000_0000,// Mov_moffs8_AL
	0x0000_0000,// Mov_moffs16_AX
	0x0000_0000,// Mov_moffs32_EAX
	0x0000_0000,// Mov_moffs64_RAX
	0x0000_0000,// Movsb_m8_m8
	0x0000_0000,// Movsw_m16_m16
	0x0000_0000,// Movsd_m32_m32
	0x0000_0000,// Movsq_m64_m64
	0x0000_0000,// Cmpsb_m8_m8
	0x0000_0000,// Cmpsw_m16_m16
	0x0000_0000,// Cmpsd_m32_m32
	0x0000_0000,// Cmpsq_m64_m64
	0x0000_0000,// Test_AL_imm8
	0x0000_0000,// Test_AX_imm16
	0x0000_0000,// Test_EAX_imm32
	0x0000_0000,// Test_RAX_imm32
	0x0000_0000,// Stosb_m8_AL
	0x0000_0000,// Stosw_m16_AX
	0x0000_0000,// Stosd_m32_EAX
	0x0000_0000,// Stosq_m64_RAX
	0x0000_0000,// Lodsb_AL_m8
	0x0000_0000,// Lodsw_AX_m16
	0x0000_0000,// Lodsd_EAX_m32
	0x0000_0000,// Lodsq_RAX_m64
	0x0000_0000,// Scasb_AL_m8
	0x0000_0000,// Scasw_AX_m16
	0x0000_0000,// Scasd_EAX_m32
	0x0000_0000,// Scasq_RAX_m64
	0x0000_0000,// Mov_r8_imm8
	0x0000_0000,// Mov_r16_imm16
	0x0000_0000,// Mov_r32_imm32
	0x0000_0000,// Mov_r64_imm64
	0x0000_0000,// Rol_rm8_imm8
	0x0000_0000,// Ror_rm8_imm8
	0x0000_0000,// Rcl_rm8_imm8
	0x0000_0000,// Rcr_rm8_imm8
	0x0000_0000,// Shl_rm8_imm8
	0x0000_0000,// Shr_rm8_imm8
	0x0000_0000,// Sal_rm8_imm8
	0x0000_0000,// Sar_rm8_imm8
	0x0000_0000,// Rol_rm16_imm8
	0x0000_0000,// Rol_rm32_imm8
	0x0000_0000,// Rol_rm64_imm8
	0x0000_0000,// Ror_rm16_imm8
	0x0000_0000,// Ror_rm32_imm8
	0x0000_0000,// Ror_rm64_imm8
	0x0000_0000,// Rcl_rm16_imm8
	0x0000_0000,// Rcl_rm32_imm8
	0x0000_0000,// Rcl_rm64_imm8
	0x0000_0000,// Rcr_rm16_imm8
	0x0000_0000,// Rcr_rm32_imm8
	0x0000_0000,// Rcr_rm64_imm8
	0x0000_0000,// Shl_rm16_imm8
	0x0000_0000,// Shl_rm32_imm8
	0x0000_0000,// Shl_rm64_imm8
	0x0000_0000,// Shr_rm16_imm8
	0x0000_0000,// Shr_rm32_imm8
	0x0000_0000,// Shr_rm64_imm8
	0x0000_0000,// Sal_rm16_imm8
	0x0000_0000,// Sal_rm32_imm8
	0x0000_0000,// Sal_rm64_imm8
	0x0000_0000,// Sar_rm16_imm8
	0x0000_0000,// Sar_rm32_imm8
	0x0000_0000,// Sar_rm64_imm8
	0x0002_0000,// Retnw_imm16
	0x0002_0000,// Retnd_imm16
	0x0002_0000,// Retnq_imm16
	0x0002_0000,// Retnw
	0x0002_0000,// Retnd
	0x0002_0000,// Retnq
	0x0000_0000,// Les_r16_m1616
	0x0000_0000,// Les_r32_m1632
	0x0000_0000,// Lds_r16_m1616
	0x0000_0000,// Lds_r32_m1632
	0x0000_0000,// Mov_rm8_imm8
	0x0001_0000,// Xabort_imm8
	0x0000_0000,// Mov_rm16_imm16
	0x0000_0000,// Mov_rm32_imm32
	0x0000_0000,// Mov_rm64_imm32
	0x0000_0000,// Xbegin_rel16
	0x0000_0000,// Xbegin_rel32
	0x0002_0000,// Enterw_imm16_imm8
	0x0002_0000,// Enterd_imm16_imm8
	0x0002_0000,// Enterq_imm16_imm8
	0x0002_0000,// Leavew
	0x0002_0000,// Leaved
	0x0002_0000,// Leaveq
	0x0002_0000,// Retfw_imm16
	0x0002_0000,// Retfd_imm16
	0x0002_0000,// Retfq_imm16
	0x0002_0000,// Retfw
	0x0002_0000,// Retfd
	0x0002_0000,// Retfq
	0x0000_0000,// Int3
	0x0000_0000,// Int_imm8
	0x0000_0000,// Into
	0x0002_0060,// Iretw
	0x0002_0060,// Iretd
	0x0002_0060,// Iretq
	0x0000_0000,// Rol_rm8_1
	0x0000_0000,// Ror_rm8_1
	0x0000_0000,// Rcl_rm8_1
	0x0000_0000,// Rcr_rm8_1
	0x0000_0000,// Shl_rm8_1
	0x0000_0000,// Shr_rm8_1
	0x0000_0000,// Sal_rm8_1
	0x0000_0000,// Sar_rm8_1
	0x0000_0000,// Rol_rm16_1
	0x0000_0000,// Rol_rm32_1
	0x0000_0000,// Rol_rm64_1
	0x0000_0000,// Ror_rm16_1
	0x0000_0000,// Ror_rm32_1
	0x0000_0000,// Ror_rm64_1
	0x0000_0000,// Rcl_rm16_1
	0x0000_0000,// Rcl_rm32_1
	0x0000_0000,// Rcl_rm64_1
	0x0000_0000,// Rcr_rm16_1
	0x0000_0000,// Rcr_rm32_1
	0x0000_0000,// Rcr_rm64_1
	0x0000_0000,// Shl_rm16_1
	0x0000_0000,// Shl_rm32_1
	0x0000_0000,// Shl_rm64_1
	0x0000_0000,// Shr_rm16_1
	0x0000_0000,// Shr_rm32_1
	0x0000_0000,// Shr_rm64_1
	0x0000_0000,// Sal_rm16_1
	0x0000_0000,// Sal_rm32_1
	0x0000_0000,// Sal_rm64_1
	0x0000_0000,// Sar_rm16_1
	0x0000_0000,// Sar_rm32_1
	0x0000_0000,// Sar_rm64_1
	0x0000_0000,// Rol_rm8_CL
	0x0000_0000,// Ror_rm8_CL
	0x0000_0000,// Rcl_rm8_CL
	0x0000_0000,// Rcr_rm8_CL
	0x0000_0000,// Shl_rm8_CL
	0x0000_0000,// Shr_rm8_CL
	0x0000_0000,// Sal_rm8_CL
	0x0000_0000,// Sar_rm8_CL
	0x0000_0000,// Rol_rm16_CL
	0x0000_0000,// Rol_rm32_CL
	0x0000_0000,// Rol_rm64_CL
	0x0000_0000,// Ror_rm16_CL
	0x0000_0000,// Ror_rm32_CL
	0x0000_0000,// Ror_rm64_CL
	0x0000_0000,// Rcl_rm16_CL
	0x0000_0000,// Rcl_rm32_CL
	0x0000_0000,// Rcl_rm64_CL
	0x0000_0000,// Rcr_rm16_CL
	0x0000_0000,// Rcr_rm32_CL
	0x0000_0000,// Rcr_rm64_CL
	0x0000_0000,// Shl_rm16_CL
	0x0000_0000,// Shl_rm32_CL
	0x0000_0000,// Shl_rm64_CL
	0x0000_0000,// Shr_rm16_CL
	0x0000_0000,// Shr_rm32_CL
	0x0000_0000,// Shr_rm64_CL
	0x0000_0000,// Sal_rm16_CL
	0x0000_0000,// Sal_rm32_CL
	0x0000_0000,// Sal_rm64_CL
	0x0000_0000,// Sar_rm16_CL
	0x0000_0000,// Sar_rm32_CL
	0x0000_0000,// Sar_rm64_CL
	0x0000_0000,// Aam_imm8
	0x0000_0000,// Aad_imm8
	0x0000_0000,// Salc
	0x0000_0000,// Xlat_m8
	0x0000_0000,// Fadd_m32fp
	0x0000_0000,// Fmul_m32fp
	0x0000_0000,// Fcom_m32fp
	0x0000_0000,// Fcomp_m32fp
	0x0000_0000,// Fsub_m32fp
	0x0000_0000,// Fsubr_m32fp
	0x0000_0000,// Fdiv_m32fp
	0x0000_0000,// Fdivr_m32fp
	0x0000_0000,// Fadd_st0_sti
	0x0000_0000,// Fmul_st0_sti
	0x0000_0000,// Fcom_st0_sti
	0x0000_0000,// Fcomp_st0_sti
	0x0000_0000,// Fsub_st0_sti
	0x0000_0000,// Fsubr_st0_sti
	0x0000_0000,// Fdiv_st0_sti
	0x0000_0000,// Fdivr_st0_sti
	0x0000_0000,// Fld_m32fp
	0x0000_0000,// Fst_m32fp
	0x0000_0000,// Fstp_m32fp
	0x0000_0000,// Fldenv_m14byte
	0x0000_0000,// Fldenv_m28byte
	0x0000_0000,// Fldcw_m2byte
	0x0000_0400,// Fnstenv_m14byte
	0x0000_0000,// Fstenv_m14byte
	0x0000_0400,// Fnstenv_m28byte
	0x0000_0000,// Fstenv_m28byte
	0x0000_0400,// Fnstcw_m2byte
	0x0000_0000,// Fstcw_m2byte
	0x0000_0000,// Fld_sti
	0x0000_0000,// Fxch_st0_sti
	0x0000_0000,// Fnop
	0x0000_0000,// Fstpnce_sti
	0x0000_0000,// Fchs
	0x0000_0000,// Fabs
	0x0000_0000,// Ftst
	0x0000_0000,// Fxam
	0x0000_0000,// Fld1
	0x0000_0000,// Fldl2t
	0x0000_0000,// Fldl2e
	0x0000_0000,// Fldpi
	0x0000_0000,// Fldlg2
	0x0000_0000,// Fldln2
	0x0000_0000,// Fldz
	0x0000_0000,// F2xm1
	0x0000_0000,// Fyl2x
	0x0000_0000,// Fptan
	0x0000_0000,// Fpatan
	0x0000_0000,// Fxtract
	0x0000_0000,// Fprem1
	0x0000_0000,// Fdecstp
	0x0000_0000,// Fincstp
	0x0000_0000,// Fprem
	0x0000_0000,// Fyl2xp1
	0x0000_0000,// Fsqrt
	0x0000_0000,// Fsincos
	0x0000_0000,// Frndint
	0x0000_0000,// Fscale
	0x0000_0000,// Fsin
	0x0000_0000,// Fcos
	0x0000_0000,// Fiadd_m32int
	0x0000_0000,// Fimul_m32int
	0x0000_0000,// Ficom_m32int
	0x0000_0000,// Ficomp_m32int
	0x0000_0000,// Fisub_m32int
	0x0000_0000,// Fisubr_m32int
	0x0000_0000,// Fidiv_m32int
	0x0000_0000,// Fidivr_m32int
	0x0000_0000,// Fcmovb_st0_sti
	0x0000_0000,// Fcmove_st0_sti
	0x0000_0000,// Fcmovbe_st0_sti
	0x0000_0000,// Fcmovu_st0_sti
	0x0000_0000,// Fucompp
	0x0000_0000,// Fild_m32int
	0x0000_0000,// Fisttp_m32int
	0x0000_0000,// Fist_m32int
	0x0000_0000,// Fistp_m32int
	0x0000_0000,// Fld_m80fp
	0x0000_0000,// Fstp_m80fp
	0x0000_0000,// Fcmovnb_st0_sti
	0x0000_0000,// Fcmovne_st0_sti
	0x0000_0000,// Fcmovnbe_st0_sti
	0x0000_0000,// Fcmovnu_st0_sti
	0x0000_0400,// Fneni
	0x0000_0000,// Feni
	0x0000_0400,// Fndisi
	0x0000_0000,// Fdisi
	0x0000_0400,// Fnclex
	0x0000_0000,// Fclex
	0x0000_0400,// Fninit
	0x0000_0000,// Finit
	0x0000_0400,// Fnsetpm
	0x0000_0000,// Fsetpm
	0x0180_0000,// Frstpm
	0x0000_0000,// Fucomi_st0_sti
	0x0000_0000,// Fcomi_st0_sti
	0x0000_0000,// Fadd_m64fp
	0x0000_0000,// Fmul_m64fp
	0x0000_0000,// Fcom_m64fp
	0x0000_0000,// Fcomp_m64fp
	0x0000_0000,// Fsub_m64fp
	0x0000_0000,// Fsubr_m64fp
	0x0000_0000,// Fdiv_m64fp
	0x0000_0000,// Fdivr_m64fp
	0x0000_0000,// Fadd_sti_st0
	0x0000_0000,// Fmul_sti_st0
	0x0000_0000,// Fcom_st0_sti_DCD0
	0x0000_0000,// Fcomp_st0_sti_DCD8
	0x0000_0000,// Fsubr_sti_st0
	0x0000_0000,// Fsub_sti_st0
	0x0000_0000,// Fdivr_sti_st0
	0x0000_0000,// Fdiv_sti_st0
	0x0000_0000,// Fld_m64fp
	0x0000_0000,// Fisttp_m64int
	0x0000_0000,// Fst_m64fp
	0x0000_0000,// Fstp_m64fp
	0x0000_0000,// Frstor_m94byte
	0x0000_0000,// Frstor_m108byte
	0x0000_0400,// Fnsave_m94byte
	0x0000_0000,// Fsave_m94byte
	0x0000_0400,// Fnsave_m108byte
	0x0000_0000,// Fsave_m108byte
	0x0000_0400,// Fnstsw_m2byte
	0x0000_0000,// Fstsw_m2byte
	0x0000_0000,// Ffree_sti
	0x0000_0000,// Fxch_st0_sti_DDC8
	0x0000_0000,// Fst_sti
	0x0000_0000,// Fstp_sti
	0x0000_0000,// Fucom_st0_sti
	0x0000_0000,// Fucomp_st0_sti
	0x0000_0000,// Fiadd_m16int
	0x0000_0000,// Fimul_m16int
	0x0000_0000,// Ficom_m16int
	0x0000_0000,// Ficomp_m16int
	0x0000_0000,// Fisub_m16int
	0x0000_0000,// Fisubr_m16int
	0x0000_0000,// Fidiv_m16int
	0x0000_0000,// Fidivr_m16int
	0x0000_0000,// Faddp_sti_st0
	0x0000_0000,// Fmulp_sti_st0
	0x0000_0000,// Fcomp_st0_sti_DED0
	0x0000_0000,// Fcompp
	0x0000_0000,// Fsubrp_sti_st0
	0x0000_0000,// Fsubp_sti_st0
	0x0000_0000,// Fdivrp_sti_st0
	0x0000_0000,// Fdivp_sti_st0
	0x0000_0000,// Fild_m16int
	0x0000_0000,// Fisttp_m16int
	0x0000_0000,// Fist_m16int
	0x0000_0000,// Fistp_m16int
	0x0000_0000,// Fbld_m80bcd
	0x0000_0000,// Fild_m64int
	0x0000_0000,// Fbstp_m80bcd
	0x0000_0000,// Fistp_m64int
	0x0000_0000,// Ffreep_sti
	0x0000_0000,// Fxch_st0_sti_DFC8
	0x0000_0000,// Fstp_sti_DFD0
	0x0000_0000,// Fstp_sti_DFD8
	0x0000_0400,// Fnstsw_AX
	0x0000_0000,// Fstsw_AX
	0x0180_0000,// Fstdw_AX
	0x0180_0000,// Fstsg_AX
	0x0000_0000,// Fucomip_st0_sti
	0x0000_0000,// Fcomip_st0_sti
	0x0000_0000,// Loopne_rel8_16_CX
	0x0000_0000,// Loopne_rel8_32_CX
	0x0000_0000,// Loopne_rel8_16_ECX
	0x0000_0000,// Loopne_rel8_32_ECX
	0x0000_0000,// Loopne_rel8_64_ECX
	0x0000_0000,// Loopne_rel8_16_RCX
	0x0000_0000,// Loopne_rel8_64_RCX
	0x0000_0000,// Loope_rel8_16_CX
	0x0000_0000,// Loope_rel8_32_CX
	0x0000_0000,// Loope_rel8_16_ECX
	0x0000_0000,// Loope_rel8_32_ECX
	0x0000_0000,// Loope_rel8_64_ECX
	0x0000_0000,// Loope_rel8_16_RCX
	0x0000_0000,// Loope_rel8_64_RCX
	0x0000_0000,// Loop_rel8_16_CX
	0x0000_0000,// Loop_rel8_32_CX
	0x0000_0000,// Loop_rel8_16_ECX
	0x0000_0000,// Loop_rel8_32_ECX
	0x0000_0000,// Loop_rel8_64_ECX
	0x0000_0000,// Loop_rel8_16_RCX
	0x0000_0000,// Loop_rel8_64_RCX
	0x0000_0000,// Jcxz_rel8_16
	0x0000_0000,// Jcxz_rel8_32
	0x0000_0000,// Jecxz_rel8_16
	0x0000_0000,// Jecxz_rel8_32
	0x0000_0000,// Jecxz_rel8_64
	0x0000_0000,// Jrcxz_rel8_16
	0x0000_0000,// Jrcxz_rel8_64
	0x0000_8004,// In_AL_imm8
	0x0000_8004,// In_AX_imm8
	0x0000_8004,// In_EAX_imm8
	0x0000_8004,// Out_imm8_AL
	0x0000_8004,// Out_imm8_AX
	0x0000_8004,// Out_imm8_EAX
	0x0002_0000,// Call_rel16
	0x0002_0000,// Call_rel32_32
	0x0002_0000,// Call_rel32_64
	0x0000_0000,// Jmp_rel16
	0x0000_0000,// Jmp_rel32_32
	0x0000_0000,// Jmp_rel32_64
	0x0000_0000,// Jmp_ptr1616
	0x0000_0000,// Jmp_ptr1632
	0x0000_0000,// Jmp_rel8_16
	0x0000_0000,// Jmp_rel8_32
	0x0000_0000,// Jmp_rel8_64
	0x0000_8004,// In_AL_DX
	0x0000_8004,// In_AX_DX
	0x0000_8004,// In_EAX_DX
	0x0000_8004,// Out_DX_AL
	0x0000_8004,// Out_DX_AX
	0x0000_8004,// Out_DX_EAX
	0x0000_0000,// Int1
	0x0000_8001,// Hlt
	0x0000_0000,// Cmc
	0x0000_0000,// Test_rm8_imm8
	0x0000_0000,// Test_rm8_imm8_F6r1
	0x0000_0000,// Not_rm8
	0x0000_0000,// Neg_rm8
	0x0000_0000,// Mul_rm8
	0x0000_0000,// Imul_rm8
	0x0000_0000,// Div_rm8
	0x0000_0000,// Idiv_rm8
	0x0000_0000,// Test_rm16_imm16
	0x0000_0000,// Test_rm32_imm32
	0x0000_0000,// Test_rm64_imm32
	0x0000_0000,// Test_rm16_imm16_F7r1
	0x0000_0000,// Test_rm32_imm32_F7r1
	0x0000_0000,// Test_rm64_imm32_F7r1
	0x0000_0000,// Not_rm16
	0x0000_0000,// Not_rm32
	0x0000_0000,// Not_rm64
	0x0000_0000,// Neg_rm16
	0x0000_0000,// Neg_rm32
	0x0000_0000,// Neg_rm64
	0x0000_0000,// Mul_rm16
	0x0000_0000,// Mul_rm32
	0x0000_0000,// Mul_rm64
	0x0000_0000,// Imul_rm16
	0x0000_0000,// Imul_rm32
	0x0000_0000,// Imul_rm64
	0x0000_0000,// Div_rm16
	0x0000_0000,// Div_rm32
	0x0000_0000,// Div_rm64
	0x0000_0000,// Idiv_rm16
	0x0000_0000,// Idiv_rm32
	0x0000_0000,// Idiv_rm64
	0x0000_0000,// Clc
	0x0000_0000,// Stc
	0x0000_8000,// Cli
	0x0000_8000,// Sti
	0x0000_0000,// Cld
	0x0000_0000,// Std
	0x0000_0000,// Inc_rm8
	0x0000_0000,// Dec_rm8
	0x0000_0000,// Inc_rm16
	0x0000_0000,// Inc_rm32
	0x0000_0000,// Inc_rm64
	0x0000_0000,// Dec_rm16
	0x0000_0000,// Dec_rm32
	0x0000_0000,// Dec_rm64
	0x0002_0100,// Call_rm16
	0x0002_0100,// Call_rm32
	0x0002_0100,// Call_rm64
	0x0002_0100,// Call_m1616
	0x0002_0100,// Call_m1632
	0x0002_0100,// Call_m1664
	0x0000_0100,// Jmp_rm16
	0x0000_0100,// Jmp_rm32
	0x0000_0100,// Jmp_rm64
	0x0000_0100,// Jmp_m1616
	0x0000_0100,// Jmp_m1632
	0x0000_0100,// Jmp_m1664
	0x0002_0000,// Push_rm16
	0x0002_0000,// Push_rm32
	0x0002_0000,// Push_rm64
	0x0000_0080,// Sldt_rm16
	0x0000_0080,// Sldt_r32m16
	0x0000_0080,// Sldt_r64m16
	0x0000_0080,// Str_rm16
	0x0000_0080,// Str_r32m16
	0x0000_0080,// Str_r64m16
	0x0000_8061,// Lldt_rm16
	0x0000_8061,// Lldt_r32m16
	0x0000_8061,// Lldt_r64m16
	0x0000_8061,// Ltr_rm16
	0x0000_8061,// Ltr_r32m16
	0x0000_8061,// Ltr_r64m16
	0x0000_0000,// Verr_rm16
	0x0000_0000,// Verr_r32m16
	0x0000_0000,// Verr_r64m16
	0x0000_0000,// Verw_rm16
	0x0000_0000,// Verw_r32m16
	0x0000_0000,// Verw_r64m16
	0x00E0_0000,// Jmpe_rm16
	0x00E0_0000,// Jmpe_rm32
	0x0000_0080,// Sgdt_m1632_16
	0x0000_0080,// Sgdt_m1632
	0x4000_0080,// Sgdt_m1664
	0x0000_0080,// Sidt_m1632_16
	0x0000_0080,// Sidt_m1632
	0x4000_0080,// Sidt_m1664
	0x0000_8061,// Lgdt_m1632_16
	0x0000_8061,// Lgdt_m1632
	0x4000_8061,// Lgdt_m1664
	0x0000_8061,// Lidt_m1632_16
	0x0000_8061,// Lidt_m1632
	0x4000_8061,// Lidt_m1664
	0x0000_0080,// Smsw_rm16
	0x0000_0080,// Smsw_r32m16
	0x0000_0080,// Smsw_r64m16
	0x0000_0000,// Rstorssp_m64
	0x0000_8021,// Lmsw_rm16
	0x0000_8021,// Lmsw_r32m16
	0x0000_8021,// Lmsw_r64m16
	0x0000_8061,// Invlpg_m
	0x0000_8001,// Enclv
	0x0000_0001,// Vmcall
	0x0000_8001,// Vmlaunch
	0x0000_8001,// Vmresume
	0x0000_8001,// Vmxoff
	0x0000_8001,// Pconfig
	0x0000_0080,// Monitorw
	0x0000_0080,// Monitord
	0x0000_0080,// Monitorq
	0x0000_0080,// Mwait
	0x0000_8001,// Clac
	0x0000_8001,// Stac
	0x0000_8001,// Encls
	0x0000_0000,// Xgetbv
	0x0000_8001,// Xsetbv
	0x0000_0000,// Vmfunc
	0x0000_0000,// Xend
	0x0000_0000,// Xtest
	0x0000_0002,// Enclu
	0x0001_8001,// Vmrunw
	0x0001_8001,// Vmrund
	0x0001_8001,// Vmrunq
	0x0000_0000,// Vmmcall
	0x0000_8001,// Vmloadw
	0x0000_8001,// Vmloadd
	0x0000_8001,// Vmloadq
	0x0000_8001,// Vmsavew
	0x0000_8001,// Vmsaved
	0x0000_8001,// Vmsaveq
	0x0000_8001,// Stgi
	0x0000_8001,// Clgi
	0x0000_8001,// Skinit
	0x0000_8041,// Invlpgaw
	0x0000_8041,// Invlpgad
	0x0000_8041,// Invlpgaq
	0x0000_8001,// Setssbsy
	0x0000_0000,// Saveprevssp
	0x0000_0000,// Rdpkru
	0x0000_0000,// Wrpkru
	0x0000_8041,// Swapgs
	0x0000_0080,// Rdtscp
	0x0000_0000,// Monitorxw
	0x0000_0000,// Monitorxd
	0x0000_0000,// Monitorxq
	0x0000_0000,// Mcommit
	0x0000_0000,// Mwaitx
	0x0000_0200,// Clzerow
	0x0000_0200,// Clzerod
	0x0000_0200,// Clzeroq
	0x0000_0080,// Rdpru
	0x0000_0000,// Lar_r16_rm16
	0x0000_0000,// Lar_r32_r32m16
	0x0000_0000,// Lar_r64_r64m16
	0x0000_0000,// Lsl_r16_rm16
	0x0000_0000,// Lsl_r32_r32m16
	0x0000_0000,// Lsl_r64_r64m16
	0x0101_8001,// Storeall
	0x0101_8001,// Loadall286
	0x0000_0000,// Syscall
	0x0000_8001,// Clts
	0x0121_8001,// Loadall386
	0x0000_8001,// Sysretd
	0x0000_8001,// Sysretq
	0x0000_8061,// Invd
	0x0000_8061,// Wbinvd
	0x0000_8061,// Wbnoinvd
	0x0040_8001,// Cl1invmb
	0x0000_0000,// Ud2
	0x0000_0018,// Reservednop_rm16_r16_0F0D
	0x0000_0018,// Reservednop_rm32_r32_0F0D
	0x0000_0018,// Reservednop_rm64_r64_0F0D
	0x0000_0000,// Prefetch_m8
	0x0000_0000,// Prefetchw_m8
	0x0000_0000,// Prefetchwt1_m8
	0x0000_0000,// Femms
	0x01C0_0000,// Umov_rm8_r8
	0x01C0_0000,// Umov_rm16_r16
	0x01C0_0000,// Umov_rm32_r32
	0x01C0_0000,// Umov_r8_rm8
	0x01C0_0000,// Umov_r16_rm16
	0x01C0_0000,// Umov_r32_rm32
	0x0000_0000,// Movups_xmm_xmmm128
	0x0000_0000,// VEX_Vmovups_xmm_xmmm128
	0x0000_0000,// VEX_Vmovups_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovups_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovups_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovups_zmm_k1z_zmmm512
	0x0000_0000,// Movupd_xmm_xmmm128
	0x0000_0000,// VEX_Vmovupd_xmm_xmmm128
	0x0000_0000,// VEX_Vmovupd_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovupd_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovupd_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovupd_zmm_k1z_zmmm512
	0x0000_0000,// Movss_xmm_xmmm32
	0x0000_0000,// VEX_Vmovss_xmm_xmm_xmm
	0x0000_0000,// VEX_Vmovss_xmm_m32
	0x0000_0000,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	0x0000_0000,// EVEX_Vmovss_xmm_k1z_m32
	0x0000_0000,// Movsd_xmm_xmmm64
	0x0000_0000,// VEX_Vmovsd_xmm_xmm_xmm
	0x0000_0000,// VEX_Vmovsd_xmm_m64
	0x0000_0000,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	0x0000_0000,// EVEX_Vmovsd_xmm_k1z_m64
	0x0000_0000,// Movups_xmmm128_xmm
	0x0000_0000,// VEX_Vmovups_xmmm128_xmm
	0x0000_0000,// VEX_Vmovups_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovups_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovups_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovups_zmmm512_k1z_zmm
	0x0000_0000,// Movupd_xmmm128_xmm
	0x0000_0000,// VEX_Vmovupd_xmmm128_xmm
	0x0000_0000,// VEX_Vmovupd_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovupd_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovupd_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovupd_zmmm512_k1z_zmm
	0x0000_0000,// Movss_xmmm32_xmm
	0x0000_0000,// VEX_Vmovss_xmm_xmm_xmm_0F11
	0x0000_0000,// VEX_Vmovss_m32_xmm
	0x0000_0000,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	0x0000_0000,// EVEX_Vmovss_m32_k1_xmm
	0x0000_0000,// Movsd_xmmm64_xmm
	0x0000_0000,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	0x0000_0000,// VEX_Vmovsd_m64_xmm
	0x0000_0000,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	0x0000_0000,// EVEX_Vmovsd_m64_k1_xmm
	0x0000_0000,// Movhlps_xmm_xmm
	0x0000_0000,// Movlps_xmm_m64
	0x0000_0000,// VEX_Vmovhlps_xmm_xmm_xmm
	0x0000_0000,// VEX_Vmovlps_xmm_xmm_m64
	0x0000_0000,// EVEX_Vmovhlps_xmm_xmm_xmm
	0x0000_0000,// EVEX_Vmovlps_xmm_xmm_m64
	0x0000_0000,// Movlpd_xmm_m64
	0x0000_0000,// VEX_Vmovlpd_xmm_xmm_m64
	0x0000_0000,// EVEX_Vmovlpd_xmm_xmm_m64
	0x0000_0000,// Movsldup_xmm_xmmm128
	0x0000_0000,// VEX_Vmovsldup_xmm_xmmm128
	0x0000_0000,// VEX_Vmovsldup_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	0x0000_0000,// Movddup_xmm_xmmm64
	0x0000_0000,// VEX_Vmovddup_xmm_xmmm64
	0x0000_0000,// VEX_Vmovddup_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovddup_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vmovddup_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovddup_zmm_k1z_zmmm512
	0x0000_0000,// Movlps_m64_xmm
	0x0000_0000,// VEX_Vmovlps_m64_xmm
	0x0000_0000,// EVEX_Vmovlps_m64_xmm
	0x0000_0000,// Movlpd_m64_xmm
	0x0000_0000,// VEX_Vmovlpd_m64_xmm
	0x0000_0000,// EVEX_Vmovlpd_m64_xmm
	0x0000_0000,// Unpcklps_xmm_xmmm128
	0x0000_0000,// VEX_Vunpcklps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vunpcklps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Unpcklpd_xmm_xmmm128
	0x0000_0000,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Unpckhps_xmm_xmmm128
	0x0000_0000,// VEX_Vunpckhps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vunpckhps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Unpckhpd_xmm_xmmm128
	0x0000_0000,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Movlhps_xmm_xmm
	0x0000_0000,// VEX_Vmovlhps_xmm_xmm_xmm
	0x0000_0000,// EVEX_Vmovlhps_xmm_xmm_xmm
	0x0000_0000,// Movhps_xmm_m64
	0x0000_0000,// VEX_Vmovhps_xmm_xmm_m64
	0x0000_0000,// EVEX_Vmovhps_xmm_xmm_m64
	0x0000_0000,// Movhpd_xmm_m64
	0x0000_0000,// VEX_Vmovhpd_xmm_xmm_m64
	0x0000_0000,// EVEX_Vmovhpd_xmm_xmm_m64
	0x0000_0000,// Movshdup_xmm_xmmm128
	0x0000_0000,// VEX_Vmovshdup_xmm_xmmm128
	0x0000_0000,// VEX_Vmovshdup_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	0x0000_0000,// Movhps_m64_xmm
	0x0000_0000,// VEX_Vmovhps_m64_xmm
	0x0000_0000,// EVEX_Vmovhps_m64_xmm
	0x0000_0000,// Movhpd_m64_xmm
	0x0000_0000,// VEX_Vmovhpd_m64_xmm
	0x0000_0000,// EVEX_Vmovhpd_m64_xmm
	0x0000_0018,// Reservednop_rm16_r16_0F18
	0x0000_0018,// Reservednop_rm32_r32_0F18
	0x0000_0018,// Reservednop_rm64_r64_0F18
	0x0000_0018,// Reservednop_rm16_r16_0F19
	0x0000_0018,// Reservednop_rm32_r32_0F19
	0x0000_0018,// Reservednop_rm64_r64_0F19
	0x0000_0018,// Reservednop_rm16_r16_0F1A
	0x0000_0018,// Reservednop_rm32_r32_0F1A
	0x0000_0018,// Reservednop_rm64_r64_0F1A
	0x0000_0018,// Reservednop_rm16_r16_0F1B
	0x0000_0018,// Reservednop_rm32_r32_0F1B
	0x0000_0018,// Reservednop_rm64_r64_0F1B
	0x0000_0018,// Reservednop_rm16_r16_0F1C
	0x0000_0018,// Reservednop_rm32_r32_0F1C
	0x0000_0018,// Reservednop_rm64_r64_0F1C
	0x0000_0018,// Reservednop_rm16_r16_0F1D
	0x0000_0018,// Reservednop_rm32_r32_0F1D
	0x0000_0018,// Reservednop_rm64_r64_0F1D
	0x0000_0018,// Reservednop_rm16_r16_0F1E
	0x0000_0018,// Reservednop_rm32_r32_0F1E
	0x0000_0018,// Reservednop_rm64_r64_0F1E
	0x0000_0018,// Reservednop_rm16_r16_0F1F
	0x0000_0018,// Reservednop_rm32_r32_0F1F
	0x0000_0018,// Reservednop_rm64_r64_0F1F
	0x0000_0200,// Prefetchnta_m8
	0x0000_0000,// Prefetcht0_m8
	0x0000_0000,// Prefetcht1_m8
	0x0000_0000,// Prefetcht2_m8
	0x0160_0000,// Bndldx_bnd_mib
	0x0160_0000,// Bndmov_bnd_bndm64
	0x0160_0000,// Bndmov_bnd_bndm128
	0x0164_0000,// Bndcl_bnd_rm32
	0x0164_0000,// Bndcl_bnd_rm64
	0x0164_0000,// Bndcu_bnd_rm32
	0x0164_0000,// Bndcu_bnd_rm64
	0x0160_0000,// Bndstx_mib_bnd
	0x0160_0000,// Bndmov_bndm64_bnd
	0x0160_0000,// Bndmov_bndm128_bnd
	0x0164_0000,// Bndmk_bnd_m32
	0x0164_0000,// Bndmk_bnd_m64
	0x0164_0000,// Bndcn_bnd_rm32
	0x0164_0000,// Bndcn_bnd_rm64
	0x0000_0000,// Cldemote_m8
	0x0000_0000,// Rdsspd_r32
	0x0000_0000,// Rdsspq_r64
	0x0000_0000,// Endbr64
	0x0000_0000,// Endbr32
	0x0000_0008,// Nop_rm16
	0x0000_0008,// Nop_rm32
	0x0000_0008,// Nop_rm64
	0x0000_8801,// Mov_r32_cr
	0x4000_8801,// Mov_r64_cr
	0x0000_8801,// Mov_r32_dr
	0x4000_8801,// Mov_r64_dr
	0x0000_8861,// Mov_cr_r32
	0x4000_8861,// Mov_cr_r64
	0x0000_8861,// Mov_dr_r32
	0x4000_8861,// Mov_dr_r64
	0x0140_8801,// Mov_r32_tr
	0x0140_8801,// Mov_tr_r32
	0x0000_0000,// Movaps_xmm_xmmm128
	0x0000_0000,// VEX_Vmovaps_xmm_xmmm128
	0x0000_0000,// VEX_Vmovaps_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovaps_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovaps_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovaps_zmm_k1z_zmmm512
	0x0000_0000,// Movapd_xmm_xmmm128
	0x0000_0000,// VEX_Vmovapd_xmm_xmmm128
	0x0000_0000,// VEX_Vmovapd_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovapd_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovapd_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovapd_zmm_k1z_zmmm512
	0x0000_0000,// Movaps_xmmm128_xmm
	0x0000_0000,// VEX_Vmovaps_xmmm128_xmm
	0x0000_0000,// VEX_Vmovaps_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovaps_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovaps_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovaps_zmmm512_k1z_zmm
	0x0000_0000,// Movapd_xmmm128_xmm
	0x0000_0000,// VEX_Vmovapd_xmmm128_xmm
	0x0000_0000,// VEX_Vmovapd_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovapd_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovapd_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovapd_zmmm512_k1z_zmm
	0x0000_0000,// Cvtpi2ps_xmm_mmm64
	0x0000_0000,// Cvtpi2pd_xmm_mmm64
	0x0000_0000,// Cvtsi2ss_xmm_rm32
	0x0000_0000,// Cvtsi2ss_xmm_rm64
	0x0000_0000,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	0x0000_0000,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	0x0000_0000,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	0x0000_0000,// Cvtsi2sd_xmm_rm32
	0x0000_0000,// Cvtsi2sd_xmm_rm64
	0x0000_0000,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	0x0000_0000,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	0x0000_0000,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	0x0000_0200,// Movntps_m128_xmm
	0x0000_0200,// VEX_Vmovntps_m128_xmm
	0x0000_0200,// VEX_Vmovntps_m256_ymm
	0x0000_0200,// EVEX_Vmovntps_m128_xmm
	0x0000_0200,// EVEX_Vmovntps_m256_ymm
	0x0000_0200,// EVEX_Vmovntps_m512_zmm
	0x0000_0200,// Movntpd_m128_xmm
	0x0000_0200,// VEX_Vmovntpd_m128_xmm
	0x0000_0200,// VEX_Vmovntpd_m256_ymm
	0x0000_0200,// EVEX_Vmovntpd_m128_xmm
	0x0000_0200,// EVEX_Vmovntpd_m256_ymm
	0x0000_0200,// EVEX_Vmovntpd_m512_zmm
	0x0000_0200,// Movntss_m32_xmm
	0x0000_0200,// Movntsd_m64_xmm
	0x0000_0000,// Cvttps2pi_mm_xmmm64
	0x0000_0000,// Cvttpd2pi_mm_xmmm128
	0x0000_0000,// Cvttss2si_r32_xmmm32
	0x0000_0000,// Cvttss2si_r64_xmmm32
	0x0000_0000,// VEX_Vcvttss2si_r32_xmmm32
	0x0000_0000,// VEX_Vcvttss2si_r64_xmmm32
	0x0000_0000,// EVEX_Vcvttss2si_r32_xmmm32_sae
	0x0000_0000,// EVEX_Vcvttss2si_r64_xmmm32_sae
	0x0000_0000,// Cvttsd2si_r32_xmmm64
	0x0000_0000,// Cvttsd2si_r64_xmmm64
	0x0000_0000,// VEX_Vcvttsd2si_r32_xmmm64
	0x0000_0000,// VEX_Vcvttsd2si_r64_xmmm64
	0x0000_0000,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	0x0000_0000,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	0x0000_0000,// Cvtps2pi_mm_xmmm64
	0x0000_0000,// Cvtpd2pi_mm_xmmm128
	0x0000_0000,// Cvtss2si_r32_xmmm32
	0x0000_0000,// Cvtss2si_r64_xmmm32
	0x0000_0000,// VEX_Vcvtss2si_r32_xmmm32
	0x0000_0000,// VEX_Vcvtss2si_r64_xmmm32
	0x0000_0000,// EVEX_Vcvtss2si_r32_xmmm32_er
	0x0000_0000,// EVEX_Vcvtss2si_r64_xmmm32_er
	0x0000_0000,// Cvtsd2si_r32_xmmm64
	0x0000_0000,// Cvtsd2si_r64_xmmm64
	0x0000_0000,// VEX_Vcvtsd2si_r32_xmmm64
	0x0000_0000,// VEX_Vcvtsd2si_r64_xmmm64
	0x0000_0000,// EVEX_Vcvtsd2si_r32_xmmm64_er
	0x0000_0000,// EVEX_Vcvtsd2si_r64_xmmm64_er
	0x0000_0000,// Ucomiss_xmm_xmmm32
	0x0000_0000,// VEX_Vucomiss_xmm_xmmm32
	0x0000_0000,// EVEX_Vucomiss_xmm_xmmm32_sae
	0x0000_0000,// Ucomisd_xmm_xmmm64
	0x0000_0000,// VEX_Vucomisd_xmm_xmmm64
	0x0000_0000,// EVEX_Vucomisd_xmm_xmmm64_sae
	0x0000_0000,// Comiss_xmm_xmmm32
	0x0000_0000,// Comisd_xmm_xmmm64
	0x0000_0000,// VEX_Vcomiss_xmm_xmmm32
	0x0000_0000,// VEX_Vcomisd_xmm_xmmm64
	0x0000_0000,// EVEX_Vcomiss_xmm_xmmm32_sae
	0x0000_0000,// EVEX_Vcomisd_xmm_xmmm64_sae
	0x0000_8061,// Wrmsr
	0x0000_0080,// Rdtsc
	0x0000_8001,// Rdmsr
	0x0000_0080,// Rdpmc
	0x0000_0000,// Sysenter
	0x0000_8001,// Sysexitd
	0x0000_8001,// Sysexitq
	0x0001_0080,// Getsecd
	0x0000_0000,// Cmovo_r16_rm16
	0x0000_0000,// Cmovo_r32_rm32
	0x0000_0000,// Cmovo_r64_rm64
	0x0000_0000,// Cmovno_r16_rm16
	0x0000_0000,// Cmovno_r32_rm32
	0x0000_0000,// Cmovno_r64_rm64
	0x0000_0000,// Cmovb_r16_rm16
	0x0000_0000,// Cmovb_r32_rm32
	0x0000_0000,// Cmovb_r64_rm64
	0x0000_0000,// Cmovae_r16_rm16
	0x0000_0000,// Cmovae_r32_rm32
	0x0000_0000,// Cmovae_r64_rm64
	0x0000_0000,// Cmove_r16_rm16
	0x0000_0000,// Cmove_r32_rm32
	0x0000_0000,// Cmove_r64_rm64
	0x0000_0000,// Cmovne_r16_rm16
	0x0000_0000,// Cmovne_r32_rm32
	0x0000_0000,// Cmovne_r64_rm64
	0x0000_0000,// Cmovbe_r16_rm16
	0x0000_0000,// Cmovbe_r32_rm32
	0x0000_0000,// Cmovbe_r64_rm64
	0x0000_0000,// Cmova_r16_rm16
	0x0000_0000,// Cmova_r32_rm32
	0x0000_0000,// Cmova_r64_rm64
	0x0000_0000,// Cmovs_r16_rm16
	0x0000_0000,// Cmovs_r32_rm32
	0x0000_0000,// Cmovs_r64_rm64
	0x0000_0000,// Cmovns_r16_rm16
	0x0000_0000,// Cmovns_r32_rm32
	0x0000_0000,// Cmovns_r64_rm64
	0x0000_0000,// Cmovp_r16_rm16
	0x0000_0000,// Cmovp_r32_rm32
	0x0000_0000,// Cmovp_r64_rm64
	0x0000_0000,// Cmovnp_r16_rm16
	0x0000_0000,// Cmovnp_r32_rm32
	0x0000_0000,// Cmovnp_r64_rm64
	0x0000_0000,// Cmovl_r16_rm16
	0x0000_0000,// Cmovl_r32_rm32
	0x0000_0000,// Cmovl_r64_rm64
	0x0000_0000,// Cmovge_r16_rm16
	0x0000_0000,// Cmovge_r32_rm32
	0x0000_0000,// Cmovge_r64_rm64
	0x0000_0000,// Cmovle_r16_rm16
	0x0000_0000,// Cmovle_r32_rm32
	0x0000_0000,// Cmovle_r64_rm64
	0x0000_0000,// Cmovg_r16_rm16
	0x0000_0000,// Cmovg_r32_rm32
	0x0000_0000,// Cmovg_r64_rm64
	0x0000_0000,// VEX_Kandw_kr_kr_kr
	0x0000_0000,// VEX_Kandq_kr_kr_kr
	0x0000_0000,// VEX_Kandb_kr_kr_kr
	0x0000_0000,// VEX_Kandd_kr_kr_kr
	0x0000_0000,// VEX_Kandnw_kr_kr_kr
	0x0000_0000,// VEX_Kandnq_kr_kr_kr
	0x0000_0000,// VEX_Kandnb_kr_kr_kr
	0x0000_0000,// VEX_Kandnd_kr_kr_kr
	0x0000_0000,// VEX_Knotw_kr_kr
	0x0000_0000,// VEX_Knotq_kr_kr
	0x0000_0000,// VEX_Knotb_kr_kr
	0x0000_0000,// VEX_Knotd_kr_kr
	0x0000_0000,// VEX_Korw_kr_kr_kr
	0x0000_0000,// VEX_Korq_kr_kr_kr
	0x0000_0000,// VEX_Korb_kr_kr_kr
	0x0000_0000,// VEX_Kord_kr_kr_kr
	0x0000_0000,// VEX_Kxnorw_kr_kr_kr
	0x0000_0000,// VEX_Kxnorq_kr_kr_kr
	0x0000_0000,// VEX_Kxnorb_kr_kr_kr
	0x0000_0000,// VEX_Kxnord_kr_kr_kr
	0x0000_0000,// VEX_Kxorw_kr_kr_kr
	0x0000_0000,// VEX_Kxorq_kr_kr_kr
	0x0000_0000,// VEX_Kxorb_kr_kr_kr
	0x0000_0000,// VEX_Kxord_kr_kr_kr
	0x0000_0000,// VEX_Kaddw_kr_kr_kr
	0x0000_0000,// VEX_Kaddq_kr_kr_kr
	0x0000_0000,// VEX_Kaddb_kr_kr_kr
	0x0000_0000,// VEX_Kaddd_kr_kr_kr
	0x0000_0000,// VEX_Kunpckwd_kr_kr_kr
	0x0000_0000,// VEX_Kunpckdq_kr_kr_kr
	0x0000_0000,// VEX_Kunpckbw_kr_kr_kr
	0x0000_0000,// Movmskps_r32_xmm
	0x0000_0000,// Movmskps_r64_xmm
	0x0000_0000,// VEX_Vmovmskps_r32_xmm
	0x0000_0000,// VEX_Vmovmskps_r64_xmm
	0x0000_0000,// VEX_Vmovmskps_r32_ymm
	0x0000_0000,// VEX_Vmovmskps_r64_ymm
	0x0000_0000,// Movmskpd_r32_xmm
	0x0000_0000,// Movmskpd_r64_xmm
	0x0000_0000,// VEX_Vmovmskpd_r32_xmm
	0x0000_0000,// VEX_Vmovmskpd_r64_xmm
	0x0000_0000,// VEX_Vmovmskpd_r32_ymm
	0x0000_0000,// VEX_Vmovmskpd_r64_ymm
	0x0000_0000,// Sqrtps_xmm_xmmm128
	0x0000_0000,// VEX_Vsqrtps_xmm_xmmm128
	0x0000_0000,// VEX_Vsqrtps_ymm_ymmm256
	0x0000_0000,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	0x0000_0000,// Sqrtpd_xmm_xmmm128
	0x0000_0000,// VEX_Vsqrtpd_xmm_xmmm128
	0x0000_0000,// VEX_Vsqrtpd_ymm_ymmm256
	0x0000_0000,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	0x0000_0000,// Sqrtss_xmm_xmmm32
	0x0000_0000,// VEX_Vsqrtss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// Sqrtsd_xmm_xmmm64
	0x0000_0000,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Rsqrtps_xmm_xmmm128
	0x0000_0000,// VEX_Vrsqrtps_xmm_xmmm128
	0x0000_0000,// VEX_Vrsqrtps_ymm_ymmm256
	0x0000_0000,// Rsqrtss_xmm_xmmm32
	0x0000_0000,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	0x0000_0000,// Rcpps_xmm_xmmm128
	0x0000_0000,// VEX_Vrcpps_xmm_xmmm128
	0x0000_0000,// VEX_Vrcpps_ymm_ymmm256
	0x0000_0000,// Rcpss_xmm_xmmm32
	0x0000_0000,// VEX_Vrcpss_xmm_xmm_xmmm32
	0x0000_0000,// Andps_xmm_xmmm128
	0x0000_0000,// VEX_Vandps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vandps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Andpd_xmm_xmmm128
	0x0000_0000,// VEX_Vandpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vandpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Andnps_xmm_xmmm128
	0x0000_0000,// VEX_Vandnps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vandnps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Andnpd_xmm_xmmm128
	0x0000_0000,// VEX_Vandnpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vandnpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Orps_xmm_xmmm128
	0x0000_0000,// VEX_Vorps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vorps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Orpd_xmm_xmmm128
	0x0000_0000,// VEX_Vorpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vorpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Xorps_xmm_xmmm128
	0x0000_0000,// VEX_Vxorps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vxorps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Xorpd_xmm_xmmm128
	0x0000_0000,// VEX_Vxorpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vxorpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Addps_xmm_xmmm128
	0x0000_0000,// VEX_Vaddps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaddps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// Addpd_xmm_xmmm128
	0x0000_0000,// VEX_Vaddpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaddpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// Addss_xmm_xmmm32
	0x0000_0000,// VEX_Vaddss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// Addsd_xmm_xmmm64
	0x0000_0000,// VEX_Vaddsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Mulps_xmm_xmmm128
	0x0000_0000,// VEX_Vmulps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vmulps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// Mulpd_xmm_xmmm128
	0x0000_0000,// VEX_Vmulpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vmulpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// Mulss_xmm_xmmm32
	0x0000_0000,// VEX_Vmulss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// Mulsd_xmm_xmmm64
	0x0000_0000,// VEX_Vmulsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Cvtps2pd_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtps2pd_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtps2pd_ymm_xmmm128
	0x0000_0000,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	0x0000_0000,// Cvtpd2ps_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtpd2ps_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtpd2ps_xmm_ymmm256
	0x0000_0000,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	0x0000_0000,// Cvtss2sd_xmm_xmmm32
	0x0000_0000,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// Cvtsd2ss_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Cvtdq2ps_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtdq2ps_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtdq2ps_ymm_ymmm256
	0x0000_0000,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	0x0000_0000,// Cvtps2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtps2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtps2dq_ymm_ymmm256
	0x0000_0000,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	0x0000_0000,// Cvttps2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvttps2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvttps2dq_ymm_ymmm256
	0x0000_0000,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// Subps_xmm_xmmm128
	0x0000_0000,// VEX_Vsubps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsubps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// Subpd_xmm_xmmm128
	0x0000_0000,// VEX_Vsubpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsubpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// Subss_xmm_xmmm32
	0x0000_0000,// VEX_Vsubss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// Subsd_xmm_xmmm64
	0x0000_0000,// VEX_Vsubsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Minps_xmm_xmmm128
	0x0000_0000,// VEX_Vminps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vminps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	0x0000_0000,// Minpd_xmm_xmmm128
	0x0000_0000,// VEX_Vminpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vminpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	0x0000_0000,// Minss_xmm_xmmm32
	0x0000_0000,// VEX_Vminss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// Minsd_xmm_xmmm64
	0x0000_0000,// VEX_Vminsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	0x0000_0000,// Divps_xmm_xmmm128
	0x0000_0000,// VEX_Vdivps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vdivps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// Divpd_xmm_xmmm128
	0x0000_0000,// VEX_Vdivpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vdivpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// Divss_xmm_xmmm32
	0x0000_0000,// VEX_Vdivss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// Divsd_xmm_xmmm64
	0x0000_0000,// VEX_Vdivsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// Maxps_xmm_xmmm128
	0x0000_0000,// VEX_Vmaxps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vmaxps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	0x0000_0000,// Maxpd_xmm_xmmm128
	0x0000_0000,// VEX_Vmaxpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vmaxpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	0x0000_0000,// Maxss_xmm_xmmm32
	0x0000_0000,// VEX_Vmaxss_xmm_xmm_xmmm32
	0x0000_0000,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// Maxsd_xmm_xmmm64
	0x0000_0000,// VEX_Vmaxsd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	0x0000_0000,// Punpcklbw_mm_mmm32
	0x0000_0000,// Punpcklbw_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Punpcklwd_mm_mmm32
	0x0000_0000,// Punpcklwd_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Punpckldq_mm_mmm32
	0x0000_0000,// Punpckldq_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Packsswb_mm_mmm64
	0x0000_0000,// Packsswb_xmm_xmmm128
	0x0000_0000,// VEX_Vpacksswb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpacksswb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pcmpgtb_mm_mmm64
	0x0000_0000,// Pcmpgtb_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	0x0000_0000,// Pcmpgtw_mm_mmm64
	0x0000_0000,// Pcmpgtw_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	0x0000_0000,// Pcmpgtd_mm_mmm64
	0x0000_0000,// Pcmpgtd_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	0x0000_0000,// Packuswb_mm_mmm64
	0x0000_0000,// Packuswb_xmm_xmmm128
	0x0000_0000,// VEX_Vpackuswb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpackuswb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Punpckhbw_mm_mmm64
	0x0000_0000,// Punpckhbw_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Punpckhwd_mm_mmm64
	0x0000_0000,// Punpckhwd_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Punpckhdq_mm_mmm64
	0x0000_0000,// Punpckhdq_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Packssdw_mm_mmm64
	0x0000_0000,// Packssdw_xmm_xmmm128
	0x0000_0000,// VEX_Vpackssdw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpackssdw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Punpcklqdq_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Punpckhqdq_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Movd_mm_rm32
	0x0000_0000,// Movq_mm_rm64
	0x0000_0000,// Movd_xmm_rm32
	0x0000_0000,// Movq_xmm_rm64
	0x0000_0000,// VEX_Vmovd_xmm_rm32
	0x0000_0000,// VEX_Vmovq_xmm_rm64
	0x0000_0000,// EVEX_Vmovd_xmm_rm32
	0x0000_0000,// EVEX_Vmovq_xmm_rm64
	0x0000_0000,// Movq_mm_mmm64
	0x0000_0000,// Movdqa_xmm_xmmm128
	0x0000_0000,// VEX_Vmovdqa_xmm_xmmm128
	0x0000_0000,// VEX_Vmovdqa_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	0x0000_0000,// Movdqu_xmm_xmmm128
	0x0000_0000,// VEX_Vmovdqu_xmm_xmmm128
	0x0000_0000,// VEX_Vmovdqu_ymm_ymmm256
	0x0000_0000,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	0x0000_0000,// Pshufw_mm_mmm64_imm8
	0x0000_0000,// Pshufd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshufd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshufd_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// Pshufhw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshufhw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshufhw_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	0x0000_0000,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	0x0000_0000,// Pshuflw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshuflw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpshuflw_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	0x0000_0000,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	0x0000_0000,// Psrlw_mm_imm8
	0x0000_0000,// Psrlw_xmm_imm8
	0x0000_0000,// VEX_Vpsrlw_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsrlw_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	0x0000_0000,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	0x0000_0000,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	0x0000_0000,// Psraw_mm_imm8
	0x0000_0000,// Psraw_xmm_imm8
	0x0000_0000,// VEX_Vpsraw_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsraw_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	0x0000_0000,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	0x0000_0000,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	0x0000_0000,// Psllw_mm_imm8
	0x0000_0000,// Psllw_xmm_imm8
	0x0000_0000,// VEX_Vpsllw_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsllw_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	0x0000_0000,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	0x0000_0000,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	0x0000_0000,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// Psrld_mm_imm8
	0x0000_0000,// Psrld_xmm_imm8
	0x0000_0000,// VEX_Vpsrld_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsrld_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// Psrad_mm_imm8
	0x0000_0000,// Psrad_xmm_imm8
	0x0000_0000,// VEX_Vpsrad_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsrad_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// Pslld_mm_imm8
	0x0000_0000,// Pslld_xmm_imm8
	0x0000_0000,// VEX_Vpslld_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpslld_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// Psrlq_mm_imm8
	0x0000_0000,// Psrlq_xmm_imm8
	0x0000_0000,// VEX_Vpsrlq_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsrlq_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// Psrldq_xmm_imm8
	0x0000_0000,// VEX_Vpsrldq_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsrldq_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	0x0000_0000,// Psllq_mm_imm8
	0x0000_0000,// Psllq_xmm_imm8
	0x0000_0000,// VEX_Vpsllq_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpsllq_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// Pslldq_xmm_imm8
	0x0000_0000,// VEX_Vpslldq_xmm_xmm_imm8
	0x0000_0000,// VEX_Vpslldq_ymm_ymm_imm8
	0x0000_0000,// EVEX_Vpslldq_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpslldq_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpslldq_zmm_zmmm512_imm8
	0x0000_0000,// Pcmpeqb_mm_mmm64
	0x0000_0000,// Pcmpeqb_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	0x0000_0000,// Pcmpeqw_mm_mmm64
	0x0000_0000,// Pcmpeqw_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	0x0000_0000,// Pcmpeqd_mm_mmm64
	0x0000_0000,// Pcmpeqd_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	0x0000_0000,// Emms
	0x0000_0000,// VEX_Vzeroupper
	0x0000_0000,// VEX_Vzeroall
	0x0000_8001,// Vmread_rm32_r32
	0x0000_8001,// Vmread_rm64_r64
	0x0000_0000,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	0x0000_0000,// Extrq_xmm_imm8_imm8
	0x0000_0000,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	0x0000_0000,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	0x0000_0000,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	0x0000_0000,// Insertq_xmm_xmm_imm8_imm8
	0x0000_0000,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	0x0000_0000,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	0x0000_8001,// Vmwrite_r32_rm32
	0x0000_8001,// Vmwrite_r64_rm64
	0x0000_0000,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	0x0000_0000,// Extrq_xmm_xmm
	0x0000_0000,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	0x0000_0000,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtss2usi_r32_xmmm32_er
	0x0000_0000,// EVEX_Vcvtss2usi_r64_xmmm32_er
	0x0000_0000,// Insertq_xmm_xmm
	0x0000_0000,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	0x0000_0000,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	0x0000_0000,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	0x0000_0000,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	0x0000_0000,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	0x0000_0000,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	0x0000_0000,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	0x0000_0000,// Haddpd_xmm_xmmm128
	0x0000_0000,// VEX_Vhaddpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vhaddpd_ymm_ymm_ymmm256
	0x0000_0000,// Haddps_xmm_xmmm128
	0x0000_0000,// VEX_Vhaddps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vhaddps_ymm_ymm_ymmm256
	0x0000_0000,// Hsubpd_xmm_xmmm128
	0x0000_0000,// VEX_Vhsubpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vhsubpd_ymm_ymm_ymmm256
	0x0000_0000,// Hsubps_xmm_xmmm128
	0x0000_0000,// VEX_Vhsubps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vhsubps_ymm_ymm_ymmm256
	0x0000_0000,// Movd_rm32_mm
	0x0000_0000,// Movq_rm64_mm
	0x0000_0000,// Movd_rm32_xmm
	0x0000_0000,// Movq_rm64_xmm
	0x0000_0000,// VEX_Vmovd_rm32_xmm
	0x0000_0000,// VEX_Vmovq_rm64_xmm
	0x0000_0000,// EVEX_Vmovd_rm32_xmm
	0x0000_0000,// EVEX_Vmovq_rm64_xmm
	0x0000_0000,// Movq_xmm_xmmm64
	0x0000_0000,// VEX_Vmovq_xmm_xmmm64
	0x0000_0000,// EVEX_Vmovq_xmm_xmmm64
	0x0000_0000,// Movq_mmm64_mm
	0x0000_0000,// Movdqa_xmmm128_xmm
	0x0000_0000,// VEX_Vmovdqa_xmmm128_xmm
	0x0000_0000,// VEX_Vmovdqa_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	0x0000_0000,// Movdqu_xmmm128_xmm
	0x0000_0000,// VEX_Vmovdqu_xmmm128_xmm
	0x0000_0000,// VEX_Vmovdqu_ymmm256_ymm
	0x0000_0000,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	0x0000_0000,// Jo_rel16
	0x0000_0000,// Jo_rel32_32
	0x0000_0000,// Jo_rel32_64
	0x0000_0000,// Jno_rel16
	0x0000_0000,// Jno_rel32_32
	0x0000_0000,// Jno_rel32_64
	0x0000_0000,// Jb_rel16
	0x0000_0000,// Jb_rel32_32
	0x0000_0000,// Jb_rel32_64
	0x0000_0000,// Jae_rel16
	0x0000_0000,// Jae_rel32_32
	0x0000_0000,// Jae_rel32_64
	0x0000_0000,// Je_rel16
	0x0000_0000,// Je_rel32_32
	0x0000_0000,// Je_rel32_64
	0x0000_0000,// Jne_rel16
	0x0000_0000,// Jne_rel32_32
	0x0000_0000,// Jne_rel32_64
	0x0000_0000,// Jbe_rel16
	0x0000_0000,// Jbe_rel32_32
	0x0000_0000,// Jbe_rel32_64
	0x0000_0000,// Ja_rel16
	0x0000_0000,// Ja_rel32_32
	0x0000_0000,// Ja_rel32_64
	0x0000_0000,// Js_rel16
	0x0000_0000,// Js_rel32_32
	0x0000_0000,// Js_rel32_64
	0x0000_0000,// Jns_rel16
	0x0000_0000,// Jns_rel32_32
	0x0000_0000,// Jns_rel32_64
	0x0000_0000,// Jp_rel16
	0x0000_0000,// Jp_rel32_32
	0x0000_0000,// Jp_rel32_64
	0x0000_0000,// Jnp_rel16
	0x0000_0000,// Jnp_rel32_32
	0x0000_0000,// Jnp_rel32_64
	0x0000_0000,// Jl_rel16
	0x0000_0000,// Jl_rel32_32
	0x0000_0000,// Jl_rel32_64
	0x0000_0000,// Jge_rel16
	0x0000_0000,// Jge_rel32_32
	0x0000_0000,// Jge_rel32_64
	0x0000_0000,// Jle_rel16
	0x0000_0000,// Jle_rel32_32
	0x0000_0000,// Jle_rel32_64
	0x0000_0000,// Jg_rel16
	0x0000_0000,// Jg_rel32_32
	0x0000_0000,// Jg_rel32_64
	0x0000_0000,// Seto_rm8
	0x0000_0000,// Setno_rm8
	0x0000_0000,// Setb_rm8
	0x0000_0000,// Setae_rm8
	0x0000_0000,// Sete_rm8
	0x0000_0000,// Setne_rm8
	0x0000_0000,// Setbe_rm8
	0x0000_0000,// Seta_rm8
	0x0000_0000,// Sets_rm8
	0x0000_0000,// Setns_rm8
	0x0000_0000,// Setp_rm8
	0x0000_0000,// Setnp_rm8
	0x0000_0000,// Setl_rm8
	0x0000_0000,// Setge_rm8
	0x0000_0000,// Setle_rm8
	0x0000_0000,// Setg_rm8
	0x0000_0000,// VEX_Kmovw_kr_km16
	0x0000_0000,// VEX_Kmovq_kr_km64
	0x0000_0000,// VEX_Kmovb_kr_km8
	0x0000_0000,// VEX_Kmovd_kr_km32
	0x0000_0000,// VEX_Kmovw_m16_kr
	0x0000_0000,// VEX_Kmovq_m64_kr
	0x0000_0000,// VEX_Kmovb_m8_kr
	0x0000_0000,// VEX_Kmovd_m32_kr
	0x0000_0000,// VEX_Kmovw_kr_r32
	0x0000_0000,// VEX_Kmovb_kr_r32
	0x0000_0000,// VEX_Kmovd_kr_r32
	0x0000_0000,// VEX_Kmovq_kr_r64
	0x0000_0000,// VEX_Kmovw_r32_kr
	0x0000_0000,// VEX_Kmovb_r32_kr
	0x0000_0000,// VEX_Kmovd_r32_kr
	0x0000_0000,// VEX_Kmovq_r64_kr
	0x0000_0000,// VEX_Kortestw_kr_kr
	0x0000_0000,// VEX_Kortestq_kr_kr
	0x0000_0000,// VEX_Kortestb_kr_kr
	0x0000_0000,// VEX_Kortestd_kr_kr
	0x0000_0000,// VEX_Ktestw_kr_kr
	0x0000_0000,// VEX_Ktestq_kr_kr
	0x0000_0000,// VEX_Ktestb_kr_kr
	0x0000_0000,// VEX_Ktestd_kr_kr
	0x0002_0000,// Pushw_FS
	0x0002_0000,// Pushd_FS
	0x0002_0000,// Pushq_FS
	0x0002_0000,// Popw_FS
	0x0002_0000,// Popd_FS
	0x0002_0000,// Popq_FS
	0x0000_00E0,// Cpuid
	0x0000_0000,// Bt_rm16_r16
	0x0000_0000,// Bt_rm32_r32
	0x0000_0000,// Bt_rm64_r64
	0x0000_0000,// Shld_rm16_r16_imm8
	0x0000_0000,// Shld_rm32_r32_imm8
	0x0000_0000,// Shld_rm64_r64_imm8
	0x0000_0000,// Shld_rm16_r16_CL
	0x0000_0000,// Shld_rm32_r32_CL
	0x0000_0000,// Shld_rm64_r64_CL
	0x0000_0000,// Montmul_16
	0x0000_0000,// Montmul_32
	0x0000_0000,// Montmul_64
	0x0000_0000,// Xsha1_16
	0x0000_0000,// Xsha1_32
	0x0000_0000,// Xsha1_64
	0x0000_0000,// Xsha256_16
	0x0000_0000,// Xsha256_32
	0x0000_0000,// Xsha256_64
	0x01E0_0000,// Xbts_r16_rm16
	0x01E0_0000,// Xbts_r32_rm32
	0x0000_0000,// Xstore_16
	0x0000_0000,// Xstore_32
	0x0000_0000,// Xstore_64
	0x0000_0000,// Xcryptecb_16
	0x0000_0000,// Xcryptecb_32
	0x0000_0000,// Xcryptecb_64
	0x0000_0000,// Xcryptcbc_16
	0x0000_0000,// Xcryptcbc_32
	0x0000_0000,// Xcryptcbc_64
	0x0000_0000,// Xcryptctr_16
	0x0000_0000,// Xcryptctr_32
	0x0000_0000,// Xcryptctr_64
	0x0000_0000,// Xcryptcfb_16
	0x0000_0000,// Xcryptcfb_32
	0x0000_0000,// Xcryptcfb_64
	0x0000_0000,// Xcryptofb_16
	0x0000_0000,// Xcryptofb_32
	0x0000_0000,// Xcryptofb_64
	0x01E0_0000,// Ibts_rm16_r16
	0x01E0_0000,// Ibts_rm32_r32
	0x0060_0000,// Cmpxchg486_rm8_r8
	0x0060_0000,// Cmpxchg486_rm16_r16
	0x0060_0000,// Cmpxchg486_rm32_r32
	0x0002_0000,// Pushw_GS
	0x0002_0000,// Pushd_GS
	0x0002_0000,// Pushq_GS
	0x0002_0000,// Popw_GS
	0x0002_0000,// Popd_GS
	0x0002_0000,// Popq_GS
	0x0001_8060,// Rsm
	0x0000_0000,// Bts_rm16_r16
	0x0000_0000,// Bts_rm32_r32
	0x0000_0000,// Bts_rm64_r64
	0x0000_0000,// Shrd_rm16_r16_imm8
	0x0000_0000,// Shrd_rm32_r32_imm8
	0x0000_0000,// Shrd_rm64_r64_imm8
	0x0000_0000,// Shrd_rm16_r16_CL
	0x0000_0000,// Shrd_rm32_r32_CL
	0x0000_0000,// Shrd_rm64_r64_CL
	0x0001_0400,// Fxsave_m512byte
	0x0001_0400,// Fxsave64_m512byte
	0x0000_0000,// Rdfsbase_r32
	0x0000_0000,// Rdfsbase_r64
	0x0001_0400,// Fxrstor_m512byte
	0x0001_0400,// Fxrstor64_m512byte
	0x0000_0000,// Rdgsbase_r32
	0x0000_0000,// Rdgsbase_r64
	0x0000_0000,// Ldmxcsr_m32
	0x0000_0000,// Wrfsbase_r32
	0x0000_0000,// Wrfsbase_r64
	0x0000_0000,// VEX_Vldmxcsr_m32
	0x0000_0000,// Stmxcsr_m32
	0x0000_0000,// Wrgsbase_r32
	0x0000_0000,// Wrgsbase_r64
	0x0000_0000,// VEX_Vstmxcsr_m32
	0x0001_0000,// Xsave_mem
	0x0001_0000,// Xsave64_mem
	0x0000_1000,// Ptwrite_rm32
	0x0000_1000,// Ptwrite_rm64
	0x0001_0000,// Xrstor_mem
	0x0001_0000,// Xrstor64_mem
	0x0000_0000,// Incsspd_r32
	0x0000_0000,// Incsspq_r64
	0x0001_0000,// Xsaveopt_mem
	0x0001_0000,// Xsaveopt64_mem
	0x0000_0000,// Clwb_m8
	0x0000_0000,// Tpause_r32
	0x0000_0000,// Tpause_r64
	0x0000_8001,// Clrssbsy_m64
	0x0000_0000,// Umonitor_r16
	0x0000_0000,// Umonitor_r32
	0x0000_0000,// Umonitor_r64
	0x0000_0000,// Umwait_r32
	0x0000_0000,// Umwait_r64
	0x0000_0000,// Clflush_m8
	0x0000_0000,// Clflushopt_m8
	0x0000_0000,// Lfence
	0x0000_0000,// Lfence_E9
	0x0000_0000,// Lfence_EA
	0x0000_0000,// Lfence_EB
	0x0000_0000,// Lfence_EC
	0x0000_0000,// Lfence_ED
	0x0000_0000,// Lfence_EE
	0x0000_0000,// Lfence_EF
	0x0000_0040,// Mfence
	0x0000_0040,// Mfence_F1
	0x0000_0040,// Mfence_F2
	0x0000_0040,// Mfence_F3
	0x0000_0040,// Mfence_F4
	0x0000_0040,// Mfence_F5
	0x0000_0040,// Mfence_F6
	0x0000_0040,// Mfence_F7
	0x0000_0000,// Sfence
	0x0000_0000,// Sfence_F9
	0x0000_0000,// Sfence_FA
	0x0000_0000,// Sfence_FB
	0x0000_0000,// Sfence_FC
	0x0000_0000,// Sfence_FD
	0x0000_0000,// Sfence_FE
	0x0000_0000,// Sfence_FF
	0x01A0_0000,// Pcommit
	0x0000_0000,// Imul_r16_rm16
	0x0000_0000,// Imul_r32_rm32
	0x0000_0000,// Imul_r64_rm64
	0x0000_0000,// Cmpxchg_rm8_r8
	0x0000_0000,// Cmpxchg_rm16_r16
	0x0000_0000,// Cmpxchg_rm32_r32
	0x0000_0000,// Cmpxchg_rm64_r64
	0x0000_0000,// Lss_r16_m1616
	0x0000_0000,// Lss_r32_m1632
	0x0000_0000,// Lss_r64_m1664
	0x0000_0000,// Btr_rm16_r16
	0x0000_0000,// Btr_rm32_r32
	0x0000_0000,// Btr_rm64_r64
	0x0000_0000,// Lfs_r16_m1616
	0x0000_0000,// Lfs_r32_m1632
	0x0000_0000,// Lfs_r64_m1664
	0x0000_0000,// Lgs_r16_m1616
	0x0000_0000,// Lgs_r32_m1632
	0x0000_0000,// Lgs_r64_m1664
	0x0000_0000,// Movzx_r16_rm8
	0x0000_0000,// Movzx_r32_rm8
	0x0000_0000,// Movzx_r64_rm8
	0x0000_0000,// Movzx_r16_rm16
	0x0000_0000,// Movzx_r32_rm16
	0x0000_0000,// Movzx_r64_rm16
	0x00E0_0000,// Jmpe_disp16
	0x00E0_0000,// Jmpe_disp32
	0x0000_0000,// Popcnt_r16_rm16
	0x0000_0000,// Popcnt_r32_rm32
	0x0000_0000,// Popcnt_r64_rm64
	0x0000_0000,// Ud1_r16_rm16
	0x0000_0000,// Ud1_r32_rm32
	0x0000_0000,// Ud1_r64_rm64
	0x0000_0000,// Bt_rm16_imm8
	0x0000_0000,// Bt_rm32_imm8
	0x0000_0000,// Bt_rm64_imm8
	0x0000_0000,// Bts_rm16_imm8
	0x0000_0000,// Bts_rm32_imm8
	0x0000_0000,// Bts_rm64_imm8
	0x0000_0000,// Btr_rm16_imm8
	0x0000_0000,// Btr_rm32_imm8
	0x0000_0000,// Btr_rm64_imm8
	0x0000_0000,// Btc_rm16_imm8
	0x0000_0000,// Btc_rm32_imm8
	0x0000_0000,// Btc_rm64_imm8
	0x0000_0000,// Btc_rm16_r16
	0x0000_0000,// Btc_rm32_r32
	0x0000_0000,// Btc_rm64_r64
	0x0000_0000,// Bsf_r16_rm16
	0x0000_0000,// Bsf_r32_rm32
	0x0000_0000,// Bsf_r64_rm64
	0x0000_0000,// Tzcnt_r16_rm16
	0x0000_0000,// Tzcnt_r32_rm32
	0x0000_0000,// Tzcnt_r64_rm64
	0x0000_0000,// Bsr_r16_rm16
	0x0000_0000,// Bsr_r32_rm32
	0x0000_0000,// Bsr_r64_rm64
	0x0000_0000,// Lzcnt_r16_rm16
	0x0000_0000,// Lzcnt_r32_rm32
	0x0000_0000,// Lzcnt_r64_rm64
	0x0000_0000,// Movsx_r16_rm8
	0x0000_0000,// Movsx_r32_rm8
	0x0000_0000,// Movsx_r64_rm8
	0x0000_0000,// Movsx_r16_rm16
	0x0000_0000,// Movsx_r32_rm16
	0x0000_0000,// Movsx_r64_rm16
	0x0000_0000,// Xadd_rm8_r8
	0x0000_0000,// Xadd_rm16_r16
	0x0000_0000,// Xadd_rm32_r32
	0x0000_0000,// Xadd_rm64_r64
	0x0000_0000,// Cmpps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	0x0000_0000,// Cmppd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	0x0000_0000,// Cmpss_xmm_xmmm32_imm8
	0x0000_0000,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	0x0000_0000,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	0x0000_0000,// Cmpsd_xmm_xmmm64_imm8
	0x0000_0000,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	0x0000_0000,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	0x0000_0200,// Movnti_m32_r32
	0x0000_0200,// Movnti_m64_r64
	0x0000_0000,// Pinsrw_mm_r32m16_imm8
	0x0000_0000,// Pinsrw_mm_r64m16_imm8
	0x0000_0000,// Pinsrw_xmm_r32m16_imm8
	0x0000_0000,// Pinsrw_xmm_r64m16_imm8
	0x0000_0000,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x0000_0000,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x0000_0000,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x0000_0000,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x0000_0000,// Pextrw_r32_mm_imm8
	0x0000_0000,// Pextrw_r64_mm_imm8
	0x0000_0000,// Pextrw_r32_xmm_imm8
	0x0000_0000,// Pextrw_r64_xmm_imm8
	0x0000_0000,// VEX_Vpextrw_r32_xmm_imm8
	0x0000_0000,// VEX_Vpextrw_r64_xmm_imm8
	0x0000_0000,// EVEX_Vpextrw_r32_xmm_imm8
	0x0000_0000,// EVEX_Vpextrw_r64_xmm_imm8
	0x0000_0000,// Shufps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// Shufpd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// Cmpxchg8b_m64
	0x0000_0000,// Cmpxchg16b_m128
	0x0001_8001,// Xrstors_mem
	0x0001_8001,// Xrstors64_mem
	0x0001_0000,// Xsavec_mem
	0x0001_0000,// Xsavec64_mem
	0x0001_8001,// Xsaves_mem
	0x0001_8001,// Xsaves64_mem
	0x0000_8001,// Vmptrld_m64
	0x0000_8001,// Vmclear_m64
	0x0000_8001,// Vmxon_m64
	0x0000_2000,// Rdrand_r16
	0x0000_2000,// Rdrand_r32
	0x0000_2000,// Rdrand_r64
	0x0000_8001,// Vmptrst_m64
	0x0000_2000,// Rdseed_r16
	0x0000_2000,// Rdseed_r32
	0x0000_2000,// Rdseed_r64
	0x0000_0000,// Rdpid_r32
	0x0000_0000,// Rdpid_r64
	0x0000_0000,// Bswap_r16
	0x0000_0000,// Bswap_r32
	0x0000_0000,// Bswap_r64
	0x0000_0000,// Addsubpd_xmm_xmmm128
	0x0000_0000,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	0x0000_0000,// Addsubps_xmm_xmmm128
	0x0000_0000,// VEX_Vaddsubps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaddsubps_ymm_ymm_ymmm256
	0x0000_0000,// Psrlw_mm_mmm64
	0x0000_0000,// Psrlw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlw_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Psrld_mm_mmm64
	0x0000_0000,// Psrld_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrld_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrld_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Psrlq_mm_mmm64
	0x0000_0000,// Psrlq_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlq_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Paddq_mm_mmm64
	0x0000_0000,// Paddq_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pmullw_mm_mmm64
	0x0000_0000,// Pmullw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmullw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmullw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Movq_xmmm64_xmm
	0x0000_0000,// VEX_Vmovq_xmmm64_xmm
	0x0000_0000,// EVEX_Vmovq_xmmm64_xmm
	0x0000_0000,// Movq2dq_xmm_mm
	0x0000_0000,// Movdq2q_mm_xmm
	0x0000_0000,// Pmovmskb_r32_mm
	0x0000_0000,// Pmovmskb_r64_mm
	0x0000_0000,// Pmovmskb_r32_xmm
	0x0000_0000,// Pmovmskb_r64_xmm
	0x0000_0000,// VEX_Vpmovmskb_r32_xmm
	0x0000_0000,// VEX_Vpmovmskb_r64_xmm
	0x0000_0000,// VEX_Vpmovmskb_r32_ymm
	0x0000_0000,// VEX_Vpmovmskb_r64_ymm
	0x0000_0000,// Psubusb_mm_mmm64
	0x0000_0000,// Psubusb_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubusb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubusb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psubusw_mm_mmm64
	0x0000_0000,// Psubusw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubusw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubusw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pminub_mm_mmm64
	0x0000_0000,// Pminub_xmm_xmmm128
	0x0000_0000,// VEX_Vpminub_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminub_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pand_mm_mmm64
	0x0000_0000,// Pand_xmm_xmmm128
	0x0000_0000,// VEX_Vpand_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpand_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Paddusb_mm_mmm64
	0x0000_0000,// Paddusb_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddusb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddusb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Paddusw_mm_mmm64
	0x0000_0000,// Paddusw_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddusw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddusw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmaxub_mm_mmm64
	0x0000_0000,// Pmaxub_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxub_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxub_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pandn_mm_mmm64
	0x0000_0000,// Pandn_xmm_xmmm128
	0x0000_0000,// VEX_Vpandn_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpandn_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pavgb_mm_mmm64
	0x0000_0000,// Pavgb_xmm_xmmm128
	0x0000_0000,// VEX_Vpavgb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpavgb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psraw_mm_mmm64
	0x0000_0000,// Psraw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsraw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsraw_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Psrad_mm_mmm64
	0x0000_0000,// Psrad_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrad_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrad_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	0x0000_0000,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Pavgw_mm_mmm64
	0x0000_0000,// Pavgw_xmm_xmmm128
	0x0000_0000,// VEX_Vpavgw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpavgw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmulhuw_mm_mmm64
	0x0000_0000,// Pmulhuw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmulhw_mm_mmm64
	0x0000_0000,// Pmulhw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Cvttpd2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvttpd2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvttpd2dq_xmm_ymmm256
	0x0000_0000,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	0x0000_0000,// Cvtdq2pd_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtdq2pd_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtdq2pd_ymm_xmmm128
	0x0000_0000,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	0x0000_0000,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	0x0000_0000,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	0x0000_0000,// Cvtpd2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtpd2dq_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtpd2dq_xmm_ymmm256
	0x0000_0000,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	0x0000_0200,// Movntq_m64_mm
	0x0000_0200,// Movntdq_m128_xmm
	0x0000_0200,// VEX_Vmovntdq_m128_xmm
	0x0000_0200,// VEX_Vmovntdq_m256_ymm
	0x0000_0200,// EVEX_Vmovntdq_m128_xmm
	0x0000_0200,// EVEX_Vmovntdq_m256_ymm
	0x0000_0200,// EVEX_Vmovntdq_m512_zmm
	0x0000_0000,// Psubsb_mm_mmm64
	0x0000_0000,// Psubsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubsb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubsb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psubsw_mm_mmm64
	0x0000_0000,// Psubsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pminsw_mm_mmm64
	0x0000_0000,// Pminsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Por_mm_mmm64
	0x0000_0000,// Por_xmm_xmmm128
	0x0000_0000,// VEX_Vpor_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpor_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Paddsb_mm_mmm64
	0x0000_0000,// Paddsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddsb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddsb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Paddsw_mm_mmm64
	0x0000_0000,// Paddsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmaxsw_mm_mmm64
	0x0000_0000,// Pmaxsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pxor_mm_mmm64
	0x0000_0000,// Pxor_xmm_xmmm128
	0x0000_0000,// VEX_Vpxor_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpxor_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Lddqu_xmm_m128
	0x0000_0000,// VEX_Vlddqu_xmm_m128
	0x0000_0000,// VEX_Vlddqu_ymm_m256
	0x0000_0000,// Psllw_mm_mmm64
	0x0000_0000,// Psllw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllw_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Pslld_mm_mmm64
	0x0000_0000,// Pslld_xmm_xmmm128
	0x0000_0000,// VEX_Vpslld_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpslld_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Psllq_mm_mmm64
	0x0000_0000,// Psllq_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllq_ymm_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	0x0000_0000,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	0x0000_0000,// Pmuludq_mm_mmm64
	0x0000_0000,// Pmuludq_xmm_xmmm128
	0x0000_0000,// VEX_Vpmuludq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmuludq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pmaddwd_mm_mmm64
	0x0000_0000,// Pmaddwd_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psadbw_mm_mmm64
	0x0000_0000,// Psadbw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsadbw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsadbw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	0x0000_0200,// Maskmovq_rDI_mm_mm
	0x0000_0200,// Maskmovdqu_rDI_xmm_xmm
	0x0000_0200,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	0x0000_0000,// Psubb_mm_mmm64
	0x0000_0000,// Psubb_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psubw_mm_mmm64
	0x0000_0000,// Psubw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Psubd_mm_mmm64
	0x0000_0000,// Psubd_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Psubq_mm_mmm64
	0x0000_0000,// Psubq_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsubq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Paddb_mm_mmm64
	0x0000_0000,// Paddb_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Paddw_mm_mmm64
	0x0000_0000,// Paddw_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Paddd_mm_mmm64
	0x0000_0000,// Paddd_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpaddd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// Ud0_r16_rm16
	0x0000_0000,// Ud0_r32_rm32
	0x0000_0000,// Ud0_r64_rm64
	0x0000_0000,// Pshufb_mm_mmm64
	0x0000_0000,// Pshufb_xmm_xmmm128
	0x0000_0000,// VEX_Vpshufb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpshufb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Phaddw_mm_mmm64
	0x0000_0000,// Phaddw_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddw_ymm_ymm_ymmm256
	0x0000_0000,// Phaddd_mm_mmm64
	0x0000_0000,// Phaddd_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddd_ymm_ymm_ymmm256
	0x0000_0000,// Phaddsw_mm_mmm64
	0x0000_0000,// Phaddsw_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphaddsw_ymm_ymm_ymmm256
	0x0000_0000,// Pmaddubsw_mm_mmm64
	0x0000_0000,// Pmaddubsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Phsubw_mm_mmm64
	0x0000_0000,// Phsubw_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubw_ymm_ymm_ymmm256
	0x0000_0000,// Phsubd_mm_mmm64
	0x0000_0000,// Phsubd_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubd_ymm_ymm_ymmm256
	0x0000_0000,// Phsubsw_mm_mmm64
	0x0000_0000,// Phsubsw_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vphsubsw_ymm_ymm_ymmm256
	0x0000_0000,// Psignb_mm_mmm64
	0x0000_0000,// Psignb_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignb_ymm_ymm_ymmm256
	0x0000_0000,// Psignw_mm_mmm64
	0x0000_0000,// Psignw_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignw_ymm_ymm_ymmm256
	0x0000_0000,// Psignd_mm_mmm64
	0x0000_0000,// Psignd_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsignd_ymm_ymm_ymmm256
	0x0000_0000,// Pmulhrsw_mm_mmm64
	0x0000_0000,// Pmulhrsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// VEX_Vpermilps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpermilps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// VEX_Vpermilpd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpermilpd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// VEX_Vtestps_xmm_xmmm128
	0x0000_0000,// VEX_Vtestps_ymm_ymmm256
	0x0000_0000,// VEX_Vtestpd_xmm_xmmm128
	0x0000_0000,// VEX_Vtestpd_ymm_ymmm256
	0x0000_0000,// Pblendvb_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	0x0000_0000,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	0x0000_0000,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	0x0000_0000,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	0x0000_0000,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	0x0000_0000,// VEX_Vcvtph2ps_xmm_xmmm64
	0x0000_0000,// VEX_Vcvtph2ps_ymm_xmmm128
	0x0000_0000,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	0x0000_0000,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	0x0000_0000,// Blendvps_xmm_xmmm128
	0x0000_0000,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	0x0000_0000,// Blendvpd_xmm_xmmm128
	0x0000_0000,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	0x0000_0000,// VEX_Vpermps_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Ptest_xmm_xmmm128
	0x0000_0000,// VEX_Vptest_xmm_xmmm128
	0x0000_0000,// VEX_Vptest_ymm_ymmm256
	0x0000_0000,// VEX_Vbroadcastss_xmm_m32
	0x0000_0000,// VEX_Vbroadcastss_ymm_m32
	0x0000_0000,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	0x0000_0000,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	0x0000_0000,// VEX_Vbroadcastsd_ymm_m64
	0x0000_0000,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	0x0000_0000,// VEX_Vbroadcastf128_ymm_m128
	0x0000_0000,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	0x0000_0000,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	0x0000_0000,// Pabsb_mm_mmm64
	0x0000_0000,// Pabsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsb_ymm_ymmm256
	0x0000_0000,// EVEX_Vpabsb_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpabsb_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpabsb_zmm_k1z_zmmm512
	0x0000_0000,// Pabsw_mm_mmm64
	0x0000_0000,// Pabsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsw_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsw_ymm_ymmm256
	0x0000_0000,// EVEX_Vpabsw_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpabsw_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpabsw_zmm_k1z_zmmm512
	0x0000_0000,// Pabsd_mm_mmm64
	0x0000_0000,// Pabsd_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsd_xmm_xmmm128
	0x0000_0000,// VEX_Vpabsd_ymm_ymmm256
	0x0000_0000,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	0x0000_0000,// Pmovsxbw_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxbw_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxbw_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	0x0000_0000,// Pmovsxbd_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovsxbd_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovsxbd_ymm_xmmm64
	0x0000_0000,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	0x0000_0000,// Pmovsxbq_xmm_xmmm16
	0x0000_0000,// VEX_Vpmovsxbq_xmm_xmmm16
	0x0000_0000,// VEX_Vpmovsxbq_ymm_xmmm32
	0x0000_0000,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	0x0000_0000,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	0x0000_0000,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	0x0000_0000,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	0x0000_0000,// Pmovsxwd_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxwd_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxwd_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	0x0000_0000,// Pmovsxwq_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovsxwq_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovsxwq_ymm_xmmm64
	0x0000_0000,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	0x0000_0000,// Pmovsxdq_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxdq_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovsxdq_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	0x0000_0000,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	0x0000_0000,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	0x0000_0000,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	0x0000_0000,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	0x0000_0000,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	0x0000_0000,// Pmuldq_xmm_xmmm128
	0x0000_0000,// VEX_Vpmuldq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmuldq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmovm2b_xmm_kr
	0x0000_0000,// EVEX_Vpmovm2b_ymm_kr
	0x0000_0000,// EVEX_Vpmovm2b_zmm_kr
	0x0000_0000,// EVEX_Vpmovm2w_xmm_kr
	0x0000_0000,// EVEX_Vpmovm2w_ymm_kr
	0x0000_0000,// EVEX_Vpmovm2w_zmm_kr
	0x0000_0000,// Pcmpeqq_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmovb2m_kr_xmm
	0x0000_0000,// EVEX_Vpmovb2m_kr_ymm
	0x0000_0000,// EVEX_Vpmovb2m_kr_zmm
	0x0000_0000,// EVEX_Vpmovw2m_kr_xmm
	0x0000_0000,// EVEX_Vpmovw2m_kr_ymm
	0x0000_0000,// EVEX_Vpmovw2m_kr_zmm
	0x0000_0200,// Movntdqa_xmm_m128
	0x0000_0200,// VEX_Vmovntdqa_xmm_m128
	0x0000_0200,// VEX_Vmovntdqa_ymm_m256
	0x0000_0200,// EVEX_Vmovntdqa_xmm_m128
	0x0000_0200,// EVEX_Vmovntdqa_ymm_m256
	0x0000_0200,// EVEX_Vmovntdqa_zmm_m512
	0x0000_0000,// EVEX_Vpbroadcastmb2q_xmm_kr
	0x0000_0000,// EVEX_Vpbroadcastmb2q_ymm_kr
	0x0000_0000,// EVEX_Vpbroadcastmb2q_zmm_kr
	0x0000_0000,// Packusdw_xmm_xmmm128
	0x0000_0000,// VEX_Vpackusdw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpackusdw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// VEX_Vmaskmovps_xmm_xmm_m128
	0x0000_0000,// VEX_Vmaskmovps_ymm_ymm_m256
	0x0000_0000,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vmaskmovpd_xmm_xmm_m128
	0x0000_0000,// VEX_Vmaskmovpd_ymm_ymm_m256
	0x0000_0000,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vmaskmovps_m128_xmm_xmm
	0x0000_0000,// VEX_Vmaskmovps_m256_ymm_ymm
	0x0000_0000,// VEX_Vmaskmovpd_m128_xmm_xmm
	0x0000_0000,// VEX_Vmaskmovpd_m256_ymm_ymm
	0x0000_0000,// Pmovzxbw_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxbw_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxbw_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	0x0000_0000,// Pmovzxbd_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovzxbd_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovzxbd_ymm_xmmm64
	0x0000_0000,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	0x0000_0000,// Pmovzxbq_xmm_xmmm16
	0x0000_0000,// VEX_Vpmovzxbq_xmm_xmmm16
	0x0000_0000,// VEX_Vpmovzxbq_ymm_xmmm32
	0x0000_0000,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	0x0000_0000,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	0x0000_0000,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	0x0000_0000,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	0x0000_0000,// Pmovzxwd_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxwd_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxwd_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	0x0000_0000,// Pmovzxwq_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovzxwq_xmm_xmmm32
	0x0000_0000,// VEX_Vpmovzxwq_ymm_xmmm64
	0x0000_0000,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	0x0000_0000,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	0x0000_0000,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	0x0000_0000,// Pmovzxdq_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxdq_xmm_xmmm64
	0x0000_0000,// VEX_Vpmovzxdq_ymm_xmmm128
	0x0000_0000,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	0x0000_0000,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	0x0000_0000,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	0x0000_0000,// VEX_Vpermd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pcmpgtq_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	0x0000_0000,// Pminsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpmovm2d_xmm_kr
	0x0000_0000,// EVEX_Vpmovm2d_ymm_kr
	0x0000_0000,// EVEX_Vpmovm2d_zmm_kr
	0x0000_0000,// EVEX_Vpmovm2q_xmm_kr
	0x0000_0000,// EVEX_Vpmovm2q_ymm_kr
	0x0000_0000,// EVEX_Vpmovm2q_zmm_kr
	0x0000_0000,// Pminsd_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminsd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmovd2m_kr_xmm
	0x0000_0000,// EVEX_Vpmovd2m_kr_ymm
	0x0000_0000,// EVEX_Vpmovd2m_kr_zmm
	0x0000_0000,// EVEX_Vpmovq2m_kr_xmm
	0x0000_0000,// EVEX_Vpmovq2m_kr_ymm
	0x0000_0000,// EVEX_Vpmovq2m_kr_zmm
	0x0000_0000,// Pminuw_xmm_xmmm128
	0x0000_0000,// VEX_Vpminuw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminuw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpbroadcastmw2d_xmm_kr
	0x0000_0000,// EVEX_Vpbroadcastmw2d_ymm_kr
	0x0000_0000,// EVEX_Vpbroadcastmw2d_zmm_kr
	0x0000_0000,// Pminud_xmm_xmmm128
	0x0000_0000,// VEX_Vpminud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpminud_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pmaxsb_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmaxsd_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pmaxuw_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Pmaxud_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmaxud_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Pmulld_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulld_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmulld_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// Phminposuw_xmm_xmmm128
	0x0000_0000,// VEX_Vphminposuw_xmm_xmmm128
	0x0000_0000,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	0x0000_0000,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	0x0000_0000,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// VEX_Vpsravd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsravd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// VEX_Vpsllvd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllvd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpsllvq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpsllvq_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	0x0000_0000,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	0x0000_0000,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	0x0000_0000,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	0x0000_0000,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	0x0000_0000,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	0x0000_0000,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	0x0000_0000,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	0x0000_0000,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	0x0000_0000,// VEX_Vpbroadcastd_xmm_xmmm32
	0x0000_0000,// VEX_Vpbroadcastd_ymm_xmmm32
	0x0000_0000,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	0x0000_0000,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	0x0000_0000,// VEX_Vpbroadcastq_xmm_xmmm64
	0x0000_0000,// VEX_Vpbroadcastq_ymm_xmmm64
	0x0000_0000,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	0x0000_0000,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	0x0000_0000,// VEX_Vbroadcasti128_ymm_m128
	0x0000_0000,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	0x0000_0000,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	0x0000_0000,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	0x0000_0000,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// VEX_Vpbroadcastb_xmm_xmmm8
	0x0000_0000,// VEX_Vpbroadcastb_ymm_xmmm8
	0x0000_0000,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	0x0000_0000,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	0x0000_0000,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	0x0000_0000,// VEX_Vpbroadcastw_xmm_xmmm16
	0x0000_0000,// VEX_Vpbroadcastw_ymm_xmmm16
	0x0000_0000,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	0x0000_0000,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	0x0000_0000,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	0x0000_0000,// EVEX_Vpbroadcastb_xmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastb_ymm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastb_zmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastw_xmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastw_ymm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastw_zmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastd_xmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastd_ymm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastd_zmm_k1z_r32
	0x0000_0000,// EVEX_Vpbroadcastq_xmm_k1z_r64
	0x0000_0000,// EVEX_Vpbroadcastq_ymm_k1z_r64
	0x0000_0000,// EVEX_Vpbroadcastq_zmm_k1z_r64
	0x0000_0000,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	0x0000_0000,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	0x0000_8021,// Invept_r32_m128
	0x0000_8021,// Invept_r64_m128
	0x0000_8021,// Invvpid_r32_m128
	0x0000_8021,// Invvpid_r64_m128
	0x0000_8061,// Invpcid_r32_m128
	0x0000_8061,// Invpcid_r64_m128
	0x0000_0000,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vexpandps_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vexpandps_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vexpandps_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	0x0000_0000,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	0x0000_0000,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	0x0000_0000,// EVEX_Vcompressps_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vcompressps_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vcompressps_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	0x0000_0000,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	0x0000_0000,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	0x0000_0000,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	0x0000_0000,// VEX_Vpmaskmovd_xmm_xmm_m128
	0x0000_0000,// VEX_Vpmaskmovd_ymm_ymm_m256
	0x0000_0000,// VEX_Vpmaskmovq_xmm_xmm_m128
	0x0000_0000,// VEX_Vpmaskmovq_ymm_ymm_m256
	0x0000_0000,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	0x0000_0000,// VEX_Vpmaskmovd_m128_xmm_xmm
	0x0000_0000,// VEX_Vpmaskmovd_m256_ymm_ymm
	0x0000_0000,// VEX_Vpmaskmovq_m128_xmm_xmm
	0x0000_0000,// VEX_Vpmaskmovq_m256_ymm_ymm
	0x0000_0000,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	0x0000_0000,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	0x0000_0000,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	0x0000_4000,// VEX_Vpgatherdd_xmm_vm32x_xmm
	0x0000_4000,// VEX_Vpgatherdd_ymm_vm32y_ymm
	0x0000_4000,// VEX_Vpgatherdq_xmm_vm32x_xmm
	0x0000_4000,// VEX_Vpgatherdq_ymm_vm32x_ymm
	0x0008_4000,// EVEX_Vpgatherdd_xmm_k1_vm32x
	0x0008_4000,// EVEX_Vpgatherdd_ymm_k1_vm32y
	0x0008_4000,// EVEX_Vpgatherdd_zmm_k1_vm32z
	0x0008_4000,// EVEX_Vpgatherdq_xmm_k1_vm32x
	0x0008_4000,// EVEX_Vpgatherdq_ymm_k1_vm32x
	0x0008_4000,// EVEX_Vpgatherdq_zmm_k1_vm32y
	0x0000_4000,// VEX_Vpgatherqd_xmm_vm64x_xmm
	0x0000_4000,// VEX_Vpgatherqd_xmm_vm64y_xmm
	0x0000_4000,// VEX_Vpgatherqq_xmm_vm64x_xmm
	0x0000_4000,// VEX_Vpgatherqq_ymm_vm64y_ymm
	0x0008_4000,// EVEX_Vpgatherqd_xmm_k1_vm64x
	0x0008_4000,// EVEX_Vpgatherqd_xmm_k1_vm64y
	0x0008_4000,// EVEX_Vpgatherqd_ymm_k1_vm64z
	0x0008_4000,// EVEX_Vpgatherqq_xmm_k1_vm64x
	0x0008_4000,// EVEX_Vpgatherqq_ymm_k1_vm64y
	0x0008_4000,// EVEX_Vpgatherqq_zmm_k1_vm64z
	0x0000_4000,// VEX_Vgatherdps_xmm_vm32x_xmm
	0x0000_4000,// VEX_Vgatherdps_ymm_vm32y_ymm
	0x0000_4000,// VEX_Vgatherdpd_xmm_vm32x_xmm
	0x0000_4000,// VEX_Vgatherdpd_ymm_vm32x_ymm
	0x0008_4000,// EVEX_Vgatherdps_xmm_k1_vm32x
	0x0008_4000,// EVEX_Vgatherdps_ymm_k1_vm32y
	0x0008_4000,// EVEX_Vgatherdps_zmm_k1_vm32z
	0x0008_4000,// EVEX_Vgatherdpd_xmm_k1_vm32x
	0x0008_4000,// EVEX_Vgatherdpd_ymm_k1_vm32x
	0x0008_4000,// EVEX_Vgatherdpd_zmm_k1_vm32y
	0x0000_4000,// VEX_Vgatherqps_xmm_vm64x_xmm
	0x0000_4000,// VEX_Vgatherqps_xmm_vm64y_xmm
	0x0000_4000,// VEX_Vgatherqpd_xmm_vm64x_xmm
	0x0000_4000,// VEX_Vgatherqpd_ymm_vm64y_ymm
	0x0008_4000,// EVEX_Vgatherqps_xmm_k1_vm64x
	0x0008_4000,// EVEX_Vgatherqps_xmm_k1_vm64y
	0x0008_4000,// EVEX_Vgatherqps_ymm_k1_vm64z
	0x0008_4000,// EVEX_Vgatherqpd_xmm_k1_vm64x
	0x0008_4000,// EVEX_Vgatherqpd_ymm_k1_vm64y
	0x0008_4000,// EVEX_Vgatherqpd_zmm_k1_vm64z
	0x0000_0000,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	0x0000_0000,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	0x0000_0000,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x0008_0000,// EVEX_Vpscatterdd_vm32x_k1_xmm
	0x0008_0000,// EVEX_Vpscatterdd_vm32y_k1_ymm
	0x0008_0000,// EVEX_Vpscatterdd_vm32z_k1_zmm
	0x0008_0000,// EVEX_Vpscatterdq_vm32x_k1_xmm
	0x0008_0000,// EVEX_Vpscatterdq_vm32x_k1_ymm
	0x0008_0000,// EVEX_Vpscatterdq_vm32y_k1_zmm
	0x0008_0000,// EVEX_Vpscatterqd_vm64x_k1_xmm
	0x0008_0000,// EVEX_Vpscatterqd_vm64y_k1_xmm
	0x0008_0000,// EVEX_Vpscatterqd_vm64z_k1_ymm
	0x0008_0000,// EVEX_Vpscatterqq_vm64x_k1_xmm
	0x0008_0000,// EVEX_Vpscatterqq_vm64y_k1_ymm
	0x0008_0000,// EVEX_Vpscatterqq_vm64z_k1_zmm
	0x0008_0000,// EVEX_Vscatterdps_vm32x_k1_xmm
	0x0008_0000,// EVEX_Vscatterdps_vm32y_k1_ymm
	0x0008_0000,// EVEX_Vscatterdps_vm32z_k1_zmm
	0x0008_0000,// EVEX_Vscatterdpd_vm32x_k1_xmm
	0x0008_0000,// EVEX_Vscatterdpd_vm32x_k1_ymm
	0x0008_0000,// EVEX_Vscatterdpd_vm32y_k1_zmm
	0x0008_0000,// EVEX_Vscatterqps_vm64x_k1_xmm
	0x0008_0000,// EVEX_Vscatterqps_vm64y_k1_xmm
	0x0008_0000,// EVEX_Vscatterqps_vm64z_k1_ymm
	0x0008_0000,// EVEX_Vscatterqpd_vm64x_k1_xmm
	0x0008_0000,// EVEX_Vscatterqpd_vm64y_k1_ymm
	0x0008_0000,// EVEX_Vscatterqpd_vm64z_k1_zmm
	0x0000_0000,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	0x0000_0000,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	0x0000_0000,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	0x0000_0000,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x0000_0000,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x0000_0000,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x0000_0000,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x0000_0000,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x0000_0000,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x0000_0000,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	0x0000_0000,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	0x0000_0000,// EVEX_Vgatherpf0dps_vm32z_k1
	0x0000_0000,// EVEX_Vgatherpf0dpd_vm32y_k1
	0x0000_0000,// EVEX_Vgatherpf1dps_vm32z_k1
	0x0000_0000,// EVEX_Vgatherpf1dpd_vm32y_k1
	0x0000_0000,// EVEX_Vscatterpf0dps_vm32z_k1
	0x0000_0000,// EVEX_Vscatterpf0dpd_vm32y_k1
	0x0000_0000,// EVEX_Vscatterpf1dps_vm32z_k1
	0x0000_0000,// EVEX_Vscatterpf1dpd_vm32y_k1
	0x0000_0000,// EVEX_Vgatherpf0qps_vm64z_k1
	0x0000_0000,// EVEX_Vgatherpf0qpd_vm64z_k1
	0x0000_0000,// EVEX_Vgatherpf1qps_vm64z_k1
	0x0000_0000,// EVEX_Vgatherpf1qpd_vm64z_k1
	0x0000_0000,// EVEX_Vscatterpf0qps_vm64z_k1
	0x0000_0000,// EVEX_Vscatterpf0qpd_vm64z_k1
	0x0000_0000,// EVEX_Vscatterpf1qps_vm64z_k1
	0x0000_0000,// EVEX_Vscatterpf1qpd_vm64z_k1
	0x0000_0000,// Sha1nexte_xmm_xmmm128
	0x0000_0000,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// Sha1msg1_xmm_xmmm128
	0x0000_0000,// Sha1msg2_xmm_xmmm128
	0x0000_0000,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// Sha256rnds2_xmm_xmmm128
	0x0000_0000,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	0x0000_0000,// Sha256msg1_xmm_xmmm128
	0x0000_0000,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	0x0000_0000,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	0x0000_0000,// Sha256msg2_xmm_xmmm128
	0x0000_0000,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	0x0000_0000,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	0x0000_0000,// Gf2p8mulb_xmm_xmmm128
	0x0000_0000,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	0x0000_0000,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	0x0000_0000,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	0x0000_0000,// Aesimc_xmm_xmmm128
	0x0000_0000,// VEX_Vaesimc_xmm_xmmm128
	0x0000_0000,// Aesenc_xmm_xmmm128
	0x0000_0000,// VEX_Vaesenc_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaesenc_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesenc_xmm_xmm_xmmm128
	0x0000_0000,// EVEX_Vaesenc_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesenc_zmm_zmm_zmmm512
	0x0000_0000,// Aesenclast_xmm_xmmm128
	0x0000_0000,// VEX_Vaesenclast_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaesenclast_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	0x0000_0000,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	0x0000_0000,// Aesdec_xmm_xmmm128
	0x0000_0000,// VEX_Vaesdec_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaesdec_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesdec_xmm_xmm_xmmm128
	0x0000_0000,// EVEX_Vaesdec_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesdec_zmm_zmm_zmmm512
	0x0000_0000,// Aesdeclast_xmm_xmmm128
	0x0000_0000,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	0x0000_0000,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	0x0000_0000,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	0x0000_2000,// Movbe_r16_m16
	0x0000_2000,// Movbe_r32_m32
	0x0000_2000,// Movbe_r64_m64
	0x0000_0000,// Crc32_r32_rm8
	0x0000_0000,// Crc32_r64_rm8
	0x0000_2000,// Movbe_m16_r16
	0x0000_2000,// Movbe_m32_r32
	0x0000_2000,// Movbe_m64_r64
	0x0000_0000,// Crc32_r32_rm16
	0x0000_0000,// Crc32_r32_rm32
	0x0000_0000,// Crc32_r64_rm64
	0x0000_0000,// VEX_Andn_r32_r32_rm32
	0x0000_0000,// VEX_Andn_r64_r64_rm64
	0x0000_0000,// VEX_Blsr_r32_rm32
	0x0000_0000,// VEX_Blsr_r64_rm64
	0x0000_0000,// VEX_Blsmsk_r32_rm32
	0x0000_0000,// VEX_Blsmsk_r64_rm64
	0x0000_0000,// VEX_Blsi_r32_rm32
	0x0000_0000,// VEX_Blsi_r64_rm64
	0x0000_0000,// VEX_Bzhi_r32_rm32_r32
	0x0000_0000,// VEX_Bzhi_r64_rm64_r64
	0x0000_8001,// Wrussd_m32_r32
	0x0000_8001,// Wrussq_m64_r64
	0x0000_0000,// VEX_Pext_r32_r32_rm32
	0x0000_0000,// VEX_Pext_r64_r64_rm64
	0x0000_0000,// VEX_Pdep_r32_r32_rm32
	0x0000_0000,// VEX_Pdep_r64_r64_rm64
	0x0000_0000,// Wrssd_m32_r32
	0x0000_0000,// Wrssq_m64_r64
	0x0000_0000,// Adcx_r32_rm32
	0x0000_0000,// Adcx_r64_rm64
	0x0000_0000,// Adox_r32_rm32
	0x0000_0000,// Adox_r64_rm64
	0x0000_0000,// VEX_Mulx_r32_r32_rm32
	0x0000_0000,// VEX_Mulx_r64_r64_rm64
	0x0000_0000,// VEX_Bextr_r32_rm32_r32
	0x0000_0000,// VEX_Bextr_r64_rm64_r64
	0x0000_0000,// VEX_Shlx_r32_rm32_r32
	0x0000_0000,// VEX_Shlx_r64_rm64_r64
	0x0000_0000,// VEX_Sarx_r32_rm32_r32
	0x0000_0000,// VEX_Sarx_r64_rm64_r64
	0x0000_0000,// VEX_Shrx_r32_rm32_r32
	0x0000_0000,// VEX_Shrx_r64_rm64_r64
	0x0000_0000,// Movdir64b_r16_m512
	0x0000_0000,// Movdir64b_r32_m512
	0x0000_0000,// Movdir64b_r64_m512
	0x0010_8001,// Enqcmds_r16_m512
	0x0010_8001,// Enqcmds_r32_m512
	0x0010_8001,// Enqcmds_r64_m512
	0x0010_0000,// Enqcmd_r16_m512
	0x0010_0000,// Enqcmd_r32_m512
	0x0010_0000,// Enqcmd_r64_m512
	0x0000_0000,// Movdiri_m32_r32
	0x0000_0000,// Movdiri_m64_r64
	0x0000_0000,// VEX_Vpermq_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// VEX_Vpermpd_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// VEX_Vpermilps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpermilps_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	0x0000_0000,// VEX_Vpermilpd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpermilpd_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	0x0000_0000,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	0x0000_0000,// Roundps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vroundps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vroundps_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	0x0000_0000,// Roundpd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vroundpd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vroundpd_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	0x0000_0000,// Roundss_xmm_xmmm32_imm8
	0x0000_0000,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	0x0000_0000,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	0x0000_0000,// Roundsd_xmm_xmmm64_imm8
	0x0000_0000,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	0x0000_0000,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x0000_0000,// Blendps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	0x0000_0000,// Blendpd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	0x0000_0000,// Pblendw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	0x0000_0000,// Palignr_mm_mmm64_imm8
	0x0000_0000,// Palignr_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	0x0000_0000,// Pextrb_r32m8_xmm_imm8
	0x0000_0000,// Pextrb_r64m8_xmm_imm8
	0x0000_0000,// VEX_Vpextrb_r32m8_xmm_imm8
	0x0000_0000,// VEX_Vpextrb_r64m8_xmm_imm8
	0x0000_0000,// EVEX_Vpextrb_r32m8_xmm_imm8
	0x0000_0000,// EVEX_Vpextrb_r64m8_xmm_imm8
	0x0000_0000,// Pextrw_r32m16_xmm_imm8
	0x0000_0000,// Pextrw_r64m16_xmm_imm8
	0x0000_0000,// VEX_Vpextrw_r32m16_xmm_imm8
	0x0000_0000,// VEX_Vpextrw_r64m16_xmm_imm8
	0x0000_0000,// EVEX_Vpextrw_r32m16_xmm_imm8
	0x0000_0000,// EVEX_Vpextrw_r64m16_xmm_imm8
	0x0000_0000,// Pextrd_rm32_xmm_imm8
	0x0000_0000,// Pextrq_rm64_xmm_imm8
	0x0000_0000,// VEX_Vpextrd_rm32_xmm_imm8
	0x0000_0000,// VEX_Vpextrq_rm64_xmm_imm8
	0x0000_0000,// EVEX_Vpextrd_rm32_xmm_imm8
	0x0000_0000,// EVEX_Vpextrq_rm64_xmm_imm8
	0x0000_0000,// Extractps_rm32_xmm_imm8
	0x0000_0000,// Extractps_r64m32_xmm_imm8
	0x0000_0000,// VEX_Vextractps_rm32_xmm_imm8
	0x0000_0000,// VEX_Vextractps_r64m32_xmm_imm8
	0x0000_0000,// EVEX_Vextractps_rm32_xmm_imm8
	0x0000_0000,// EVEX_Vextractps_r64m32_xmm_imm8
	0x0000_0000,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	0x0000_0000,// VEX_Vextractf128_xmmm128_ymm_imm8
	0x0000_0000,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	0x0000_0000,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	0x0000_0000,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	0x0000_0000,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	0x0000_0000,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	0x0000_0000,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	0x0000_0000,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	0x0000_0000,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	0x0000_0000,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	0x0000_0000,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	0x0000_0000,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	0x0000_0000,// Pinsrb_xmm_r32m8_imm8
	0x0000_0000,// Pinsrb_xmm_r64m8_imm8
	0x0000_0000,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x0000_0000,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x0000_0000,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x0000_0000,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x0000_0000,// Insertps_xmm_xmmm32_imm8
	0x0000_0000,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x0000_0000,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x0000_0000,// Pinsrd_xmm_rm32_imm8
	0x0000_0000,// Pinsrq_xmm_rm64_imm8
	0x0000_0000,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x0000_0000,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x0000_0000,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x0000_0000,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x0000_0000,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	0x0000_0000,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	0x0000_0000,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x0000_0000,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x0000_0000,// VEX_Kshiftrb_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftrw_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftrd_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftrq_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftlb_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftlw_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftld_kr_kr_imm8
	0x0000_0000,// VEX_Kshiftlq_kr_kr_imm8
	0x0000_0000,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	0x0000_0000,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	0x0000_0000,// VEX_Vextracti128_xmmm128_ymm_imm8
	0x0000_0000,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	0x0000_0000,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	0x0000_0000,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	0x0000_0000,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	0x0000_0000,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	0x0000_0000,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	0x0000_0000,// Dpps_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	0x0000_0000,// Dppd_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// Mpsadbw_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// Pclmulqdq_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	0x0000_0000,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	0x0000_0000,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	0x0000_0000,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	0x0000_0000,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	0x0000_0000,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	0x0000_0000,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	0x0000_0000,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	0x0000_0000,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	0x0000_0000,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	0x0000_0000,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	0x0000_0000,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x0000_0000,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x0000_0000,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x0000_0000,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x0000_0000,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x0000_0000,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x0000_0000,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x0000_0000,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x0000_0000,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	0x0000_0000,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	0x0000_0000,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x0000_0000,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x0000_0000,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// Pcmpestrm_xmm_xmmm128_imm8
	0x0000_0000,// Pcmpestrm64_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	0x0000_0000,// Pcmpestri_xmm_xmmm128_imm8
	0x0000_0000,// Pcmpestri64_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	0x0000_0000,// Pcmpistrm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	0x0000_0000,// Pcmpistri_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	0x0000_0000,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	0x0000_0000,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	0x0000_0000,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	0x0000_0000,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	0x0000_0000,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	0x0000_0000,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	0x0000_0000,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	0x0000_0000,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	0x0000_0000,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	0x0000_0000,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	0x0000_0000,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	0x0000_0000,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	0x0000_0000,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	0x0000_0000,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	0x0000_0000,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	0x0000_0000,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	0x0000_0000,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	0x0000_0000,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	0x0000_0000,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	0x0000_0000,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	0x0000_0000,// Sha1rnds4_xmm_xmmm128_imm8
	0x0000_0000,// Gf2p8affineqb_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	0x0000_0000,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x0000_0000,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x0000_0000,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x0000_0000,// Aeskeygenassist_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	0x0000_0000,// VEX_Rorx_r32_rm32_imm8
	0x0000_0000,// VEX_Rorx_r64_rm64_imm8
	0x0000_0000,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	0x0000_0000,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	0x0000_0000,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vprotb_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vprotw_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vprotd_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vprotq_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	0x0000_0000,// XOP_Blcfill_r32_rm32
	0x0000_0000,// XOP_Blcfill_r64_rm64
	0x0000_0000,// XOP_Blsfill_r32_rm32
	0x0000_0000,// XOP_Blsfill_r64_rm64
	0x0000_0000,// XOP_Blcs_r32_rm32
	0x0000_0000,// XOP_Blcs_r64_rm64
	0x0000_0000,// XOP_Tzmsk_r32_rm32
	0x0000_0000,// XOP_Tzmsk_r64_rm64
	0x0000_0000,// XOP_Blcic_r32_rm32
	0x0000_0000,// XOP_Blcic_r64_rm64
	0x0000_0000,// XOP_Blsic_r32_rm32
	0x0000_0000,// XOP_Blsic_r64_rm64
	0x0000_0000,// XOP_T1mskc_r32_rm32
	0x0000_0000,// XOP_T1mskc_r64_rm64
	0x0000_0000,// XOP_Blcmsk_r32_rm32
	0x0000_0000,// XOP_Blcmsk_r64_rm64
	0x0000_0000,// XOP_Blci_r32_rm32
	0x0000_0000,// XOP_Blci_r64_rm64
	0x0000_0000,// XOP_Llwpcb_r32
	0x0000_0000,// XOP_Llwpcb_r64
	0x0000_0000,// XOP_Slwpcb_r32
	0x0000_0000,// XOP_Slwpcb_r64
	0x0000_0000,// XOP_Vfrczps_xmm_xmmm128
	0x0000_0000,// XOP_Vfrczps_ymm_ymmm256
	0x0000_0000,// XOP_Vfrczpd_xmm_xmmm128
	0x0000_0000,// XOP_Vfrczpd_ymm_ymmm256
	0x0000_0000,// XOP_Vfrczss_xmm_xmmm32
	0x0000_0000,// XOP_Vfrczsd_xmm_xmmm64
	0x0000_0000,// XOP_Vprotb_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vprotb_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vprotw_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vprotw_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vprotd_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vprotd_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vprotq_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vprotq_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshlb_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshlb_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshlw_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshlw_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshld_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshld_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshlq_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshlq_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshab_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshab_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshaw_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshaw_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshad_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshad_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vpshaq_xmm_xmmm128_xmm
	0x0000_0000,// XOP_Vpshaq_xmm_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddbw_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddbd_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddbq_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddwd_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddwq_xmm_xmmm128
	0x0000_0000,// XOP_Vphadddq_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddubw_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddubd_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddubq_xmm_xmmm128
	0x0000_0000,// XOP_Vphadduwd_xmm_xmmm128
	0x0000_0000,// XOP_Vphadduwq_xmm_xmmm128
	0x0000_0000,// XOP_Vphaddudq_xmm_xmmm128
	0x0000_0000,// XOP_Vphsubbw_xmm_xmmm128
	0x0000_0000,// XOP_Vphsubwd_xmm_xmmm128
	0x0000_0000,// XOP_Vphsubdq_xmm_xmmm128
	0x0000_0000,// XOP_Bextr_r32_rm32_imm32
	0x0000_0000,// XOP_Bextr_r64_rm64_imm32
	0x0000_0000,// XOP_Lwpins_r32_rm32_imm32
	0x0000_0000,// XOP_Lwpins_r64_rm32_imm32
	0x0000_0000,// XOP_Lwpval_r32_rm32_imm32
	0x0000_0000,// XOP_Lwpval_r64_rm32_imm32
	0x0000_0000,// D3NOW_Pi2fw_mm_mmm64
	0x0000_0000,// D3NOW_Pi2fd_mm_mmm64
	0x0000_0000,// D3NOW_Pf2iw_mm_mmm64
	0x0000_0000,// D3NOW_Pf2id_mm_mmm64
	0x0080_0000,// D3NOW_Pfrcpv_mm_mmm64
	0x0080_0000,// D3NOW_Pfrsqrtv_mm_mmm64
	0x0000_0000,// D3NOW_Pfnacc_mm_mmm64
	0x0000_0000,// D3NOW_Pfpnacc_mm_mmm64
	0x0000_0000,// D3NOW_Pfcmpge_mm_mmm64
	0x0000_0000,// D3NOW_Pfmin_mm_mmm64
	0x0000_0000,// D3NOW_Pfrcp_mm_mmm64
	0x0000_0000,// D3NOW_Pfrsqrt_mm_mmm64
	0x0000_0000,// D3NOW_Pfsub_mm_mmm64
	0x0000_0000,// D3NOW_Pfadd_mm_mmm64
	0x0000_0000,// D3NOW_Pfcmpgt_mm_mmm64
	0x0000_0000,// D3NOW_Pfmax_mm_mmm64
	0x0000_0000,// D3NOW_Pfrcpit1_mm_mmm64
	0x0000_0000,// D3NOW_Pfrsqit1_mm_mmm64
	0x0000_0000,// D3NOW_Pfsubr_mm_mmm64
	0x0000_0000,// D3NOW_Pfacc_mm_mmm64
	0x0000_0000,// D3NOW_Pfcmpeq_mm_mmm64
	0x0000_0000,// D3NOW_Pfmul_mm_mmm64
	0x0000_0000,// D3NOW_Pfrcpit2_mm_mmm64
	0x0000_0000,// D3NOW_Pmulhrw_mm_mmm64
	0x0000_0000,// D3NOW_Pswapd_mm_mmm64
	0x0000_0000,// D3NOW_Pavgusb_mm_mmm64
	0x0000_8001,// Rmpadjust
	0x0000_8001,// Rmpupdate
	0x0000_8001,// Psmash
	0x0000_8001,// Pvalidatew
	0x0000_8001,// Pvalidated
	0x0000_8001,// Pvalidateq
	0x0000_0060,// Serialize
	0x0000_0000,// Xsusldtrk
	0x0000_0000,// Xresldtrk
	0x0000_8001,// Invlpgbw
	0x0000_8001,// Invlpgbd
	0x0000_8001,// Invlpgbq
	0x0000_8041,// Tlbsync
	0x0000_0000,// Prefetchreserved3_m8
	0x0000_0000,// Prefetchreserved4_m8
	0x0000_0000,// Prefetchreserved5_m8
	0x0000_0000,// Prefetchreserved6_m8
	0x0000_0000,// Prefetchreserved7_m8
	0x0000_0000,// Ud0
	0x0000_0000,// Vmgexit
	0x0001_0080,// Getsecq
	0x0010_0000,// VEX_Ldtilecfg_m512
	0x0000_0000,// VEX_Tilerelease
	0x0010_0000,// VEX_Sttilecfg_m512
	0x0010_0000,// VEX_Tilezero_tmm
	0x0010_0000,// VEX_Tileloaddt1_tmm_sibmem
	0x0010_0000,// VEX_Tilestored_sibmem_tmm
	0x0010_0000,// VEX_Tileloadd_tmm_sibmem
	0x0010_4000,// VEX_Tdpbf16ps_tmm_tmm_tmm
	0x0010_4000,// VEX_Tdpbuud_tmm_tmm_tmm
	0x0010_4000,// VEX_Tdpbusd_tmm_tmm_tmm
	0x0010_4000,// VEX_Tdpbsud_tmm_tmm_tmm
	0x0010_4000,// VEX_Tdpbssd_tmm_tmm_tmm
	0x0180_0400,// Fnstdw_AX
	0x0180_0400,// Fnstsg_AX
	0x0080_8001,// Rdshr_rm32
	0x0080_8001,// Wrshr_rm32
	0x0080_8001,// Smint
	0x00A0_8001,// Dmint
	0x00A1_8001,// Rdm
	0x0080_8001,// Svdc_m80_Sreg
	0x0080_8001,// Rsdc_Sreg_m80
	0x0080_8001,// Svldt_m80
	0x0080_8001,// Rsldt_m80
	0x0080_8001,// Svts_m80
	0x0080_8001,// Rsts_m80
	0x00C0_8001,// Smint_0F7E
	0x0080_8001,// Bb0_reset
	0x0080_8001,// Bb1_reset
	0x0080_8001,// Cpu_write
	0x0080_8001,// Cpu_read
	0x0020_0000,// Altinst
	0x0080_0000,// Paveb_mm_mmm64
	0x0080_0000,// Paddsiw_mm_mmm64
	0x0080_0000,// Pmagw_mm_mmm64
	0x0080_0000,// Pdistib_mm_m64
	0x0080_0000,// Psubsiw_mm_mmm64
	0x0080_0000,// Pmvzb_mm_m64
	0x0080_0000,// Pmulhrw_mm_mmm64
	0x0080_0000,// Pmvnzb_mm_m64
	0x0080_0000,// Pmvlzb_mm_m64
	0x0080_0000,// Pmvgezb_mm_m64
	0x0080_0000,// Pmulhriw_mm_mmm64
	0x0080_0000,// Pmachriw_mm_m64
	0x0080_0000,// Cyrix_D9D7
	0x0080_0000,// Cyrix_D9E2
	0x0080_0000,// Ftstp
	0x0080_0000,// Cyrix_D9E7
	0x0080_0000,// Frint2
	0x0080_0000,// Frichop
	0x0080_0000,// Cyrix_DED8
	0x0080_0000,// Cyrix_DEDA
	0x0080_0000,// Cyrix_DEDC
	0x0080_0000,// Cyrix_DEDD
	0x0080_0000,// Cyrix_DEDE
	0x0080_0000,// Frinear
	0x0000_8001,// Tdcall
	0x0000_8001,// Seamret
	0x0000_8001,// Seamops
	0x0000_8001,// Seamcall
	0x0010_0000,// Aesencwide128kl_m384
	0x0010_0000,// Aesdecwide128kl_m384
	0x0010_0000,// Aesencwide256kl_m512
	0x0010_0000,// Aesdecwide256kl_m512
	0x0010_8001,// Loadiwkey_xmm_xmm
	0x0010_0000,// Aesenc128kl_xmm_m384
	0x0010_0000,// Aesdec128kl_xmm_m384
	0x0010_0000,// Aesenc256kl_xmm_m512
	0x0010_0000,// Aesdec256kl_xmm_m512
	0x0010_0000,// Encodekey128_r32_r32
	0x0010_0000,// Encodekey256_r32_r32
	0x0000_0000,// VEX_Vbroadcastss_xmm_xmm
	0x0000_0000,// VEX_Vbroadcastss_ymm_xmm
	0x0000_0000,// VEX_Vbroadcastsd_ymm_xmm
	0x0000_0000,// Vmgexit_F2
	0x0002_0000,// Uiret
	0x0000_0000,// Testui
	0x0000_0000,// Clui
	0x0000_0000,// Stui
	0x4000_0000,// Senduipi_r64
	0x0000_8001,// Hreset_imm8
	0x0000_0000,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	0x0000_0000,// Ccs_hash_16
	0x0000_0000,// Ccs_hash_32
	0x0000_0000,// Ccs_hash_64
	0x0000_0000,// Ccs_encrypt_16
	0x0000_0000,// Ccs_encrypt_32
	0x0000_0000,// Ccs_encrypt_64
	0x0000_8001,// Lkgs_rm16
	0x0000_8001,// Lkgs_r32m16
	0x0000_8001,// Lkgs_r64m16
	0x0002_8001,// Eretu
	0x0002_8001,// Erets
	0x0000_0000,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	0x0000_0000,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	0x0000_0000,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	0x0000_0000,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	0x0000_0000,// EVEX_Vcomish_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	0x0000_0000,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	0x0000_0000,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	0x0000_0000,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	0x0000_0000,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	0x0000_0000,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	0x0000_0000,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	0x0000_0000,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	0x0000_0000,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	0x0000_0000,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	0x0000_0000,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	0x0000_0000,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	0x0000_0000,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vcvtsh2si_r32_xmmm16_er
	0x0000_0000,// EVEX_Vcvtsh2si_r64_xmmm16_er
	0x0000_0000,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	0x0000_0000,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	0x0000_0000,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	0x0000_0000,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	0x0000_0000,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	0x0000_0000,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	0x0000_0000,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	0x0000_0000,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	0x0000_0000,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	0x0000_0000,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	0x0000_0000,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	0x0000_0000,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	0x0000_0000,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	0x0000_0000,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	0x0000_0000,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	0x0000_0000,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	0x0000_0000,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	0x0000_0000,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	0x0000_0000,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	0x0000_0000,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	0x0000_0000,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	0x0000_0000,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	0x0000_0000,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	0x0000_0000,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	0x0000_0000,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	0x0000_0000,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	0x0000_0000,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	0x8000_0000,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	0x8000_0000,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	0x8000_0000,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x8000_0000,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	0x8000_0000,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	0x8000_0000,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x8000_0000,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x8000_0000,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x8000_0000,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	0x8000_0000,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	0x8000_0000,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x8000_0000,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	0x8000_0000,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	0x8000_0000,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x8000_0000,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	0x8000_0000,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	0x0000_0000,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	0x0000_0000,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	0x0000_0000,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	0x0000_0000,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	0x0000_0000,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	0x0000_0000,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	0x0000_0000,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	0x0000_0000,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	0x0000_0000,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x0000_0000,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	0x0000_0000,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	0x0000_0000,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	0x0000_0000,// EVEX_Vmovsh_xmm_k1z_m16
	0x0000_0000,// EVEX_Vmovsh_m16_k1_xmm
	0x0000_0000,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	0x0000_0000,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	0x0000_0000,// EVEX_Vmovw_xmm_r32m16
	0x0000_0000,// EVEX_Vmovw_xmm_r64m16
	0x0000_0000,// EVEX_Vmovw_r32m16_xmm
	0x0000_0000,// EVEX_Vmovw_r64m16_xmm
	0x0000_0000,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	0x0000_0000,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	0x0000_0000,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	0x0000_0000,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	0x0000_0000,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	0x0000_0000,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x0000_0000,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	0x0000_0000,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	0x0000_0000,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	0x0000_0000,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x0000_0000,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	0x0000_0000,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	0x0000_0000,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	0x0000_0000,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	0x0000_0000,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	0x0000_0000,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	0x0000_0000,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	0x0000_0000,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	0x0000_0000,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	0x0000_0000,// EVEX_Vucomish_xmm_xmmm16_sae
	0x0200_8000,// Rdudbg
	0x0200_8000,// Wrudbg
	0x0220_0000,// VEX_KNC_Jkzd_kr_rel8_64
	0x0220_0000,// VEX_KNC_Jknzd_kr_rel8_64
	0x0220_0200,// VEX_KNC_Vprefetchnta_m8
	0x0220_0000,// VEX_KNC_Vprefetch0_m8
	0x0220_0000,// VEX_KNC_Vprefetch1_m8
	0x0220_0200,// VEX_KNC_Vprefetch2_m8
	0x0220_0200,// VEX_KNC_Vprefetchenta_m8
	0x0220_0000,// VEX_KNC_Vprefetche0_m8
	0x0220_0000,// VEX_KNC_Vprefetche1_m8
	0x0220_0200,// VEX_KNC_Vprefetche2_m8
	0x0220_0000,// VEX_KNC_Kand_kr_kr
	0x0220_0000,// VEX_KNC_Kandn_kr_kr
	0x0220_0000,// VEX_KNC_Kandnr_kr_kr
	0x0220_0000,// VEX_KNC_Knot_kr_kr
	0x0220_0000,// VEX_KNC_Kor_kr_kr
	0x0220_0000,// VEX_KNC_Kxnor_kr_kr
	0x0220_0000,// VEX_KNC_Kxor_kr_kr
	0x0220_0000,// VEX_KNC_Kmerge2l1h_kr_kr
	0x0220_0000,// VEX_KNC_Kmerge2l1l_kr_kr
	0x0220_0000,// VEX_KNC_Jkzd_kr_rel32_64
	0x0220_0000,// VEX_KNC_Jknzd_kr_rel32_64
	0x0220_0000,// VEX_KNC_Kmov_kr_kr
	0x0220_0000,// VEX_KNC_Kmov_kr_r32
	0x0220_0000,// VEX_KNC_Kmov_r32_kr
	0x0220_0000,// VEX_KNC_Kconcath_r64_kr_kr
	0x0220_0000,// VEX_KNC_Kconcatl_r64_kr_kr
	0x0220_0000,// VEX_KNC_Kortest_kr_kr
	0x0220_0000,// VEX_KNC_Delay_r32
	0x0220_0000,// VEX_KNC_Delay_r64
	0x0220_0000,// VEX_KNC_Spflt_r32
	0x0220_0000,// VEX_KNC_Spflt_r64
	0x0220_0000,// VEX_KNC_Clevict1_m8
	0x0220_0000,// VEX_KNC_Clevict0_m8
	0x0220_0000,// VEX_KNC_Popcnt_r32_r32
	0x0220_0000,// VEX_KNC_Popcnt_r64_r64
	0x0220_0000,// VEX_KNC_Tzcnt_r32_r32
	0x0220_0000,// VEX_KNC_Tzcnt_r64_r64
	0x0220_0000,// VEX_KNC_Tzcnti_r32_r32
	0x0220_0000,// VEX_KNC_Tzcnti_r64_r64
	0x0220_0000,// VEX_KNC_Lzcnt_r32_r32
	0x0220_0000,// VEX_KNC_Lzcnt_r64_r64
	0x0221_0000,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	0x0221_0000,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	0x0221_0000,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	0x0221_0000,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	0x0221_0000,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	0x0221_0000,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	0x0220_0000,// VEX_KNC_Kextract_kr_r64_imm8
	0x0220_0200,// MVEX_Vprefetchnta_m
	0x0220_0000,// MVEX_Vprefetch0_m
	0x0220_0000,// MVEX_Vprefetch1_m
	0x0220_0200,// MVEX_Vprefetch2_m
	0x0220_0200,// MVEX_Vprefetchenta_m
	0x0220_0000,// MVEX_Vprefetche0_m
	0x0220_0000,// MVEX_Vprefetche1_m
	0x0220_0200,// MVEX_Vprefetche2_m
	0x0220_0000,// MVEX_Vmovaps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vmovapd_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vmovaps_mt_k1_zmm
	0x0220_0000,// MVEX_Vmovapd_mt_k1_zmm
	0x0220_0000,// MVEX_Vmovnrapd_m_k1_zmm
	0x0220_0000,// MVEX_Vmovnrngoapd_m_k1_zmm
	0x0220_0000,// MVEX_Vmovnraps_m_k1_zmm
	0x0220_0000,// MVEX_Vmovnrngoaps_m_k1_zmm
	0x0220_0000,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vmovdqa32_mt_k1_zmm
	0x0220_0000,// MVEX_Vmovdqa64_mt_k1_zmm
	0x0220_0000,// MVEX_Clevict1_m
	0x0220_0000,// MVEX_Clevict0_m
	0x0220_0000,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	0x0220_0000,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	0x0220_0000,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vbroadcastss_zmm_k1_mt
	0x0220_0000,// MVEX_Vbroadcastsd_zmm_k1_mt
	0x0220_0000,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	0x0220_0000,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	0x0220_0000,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgetexpps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vgetexppd_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	0x0220_0000,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	0x0220_0000,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	0x0220_0000,// MVEX_Vpbroadcastd_zmm_k1_mt
	0x0220_0000,// MVEX_Vpbroadcastq_zmm_k1_mt
	0x0220_0000,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	0x0220_0000,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	0x0220_0000,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	0x0220_0000,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	0x0220_0000,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	0x0220_0000,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	0x0220_0000,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	0x0220_0000,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	0x0220_0000,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	0x0220_0000,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	0x0228_4000,// MVEX_Vpgatherdd_zmm_k1_mvt
	0x0228_4000,// MVEX_Vpgatherdq_zmm_k1_mvt
	0x0228_4000,// MVEX_Vgatherdps_zmm_k1_mvt
	0x0228_4000,// MVEX_Vgatherdpd_zmm_k1_mvt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	0x0220_0000,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	0x0228_0000,// MVEX_Vpscatterdd_mvt_k1_zmm
	0x0228_0000,// MVEX_Vpscatterdq_mvt_k1_zmm
	0x0228_0000,// MVEX_Vscatterdps_mvt_k1_zmm
	0x0228_0000,// MVEX_Vscatterdpd_mvt_k1_zmm
	0x0220_0000,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	0x0221_0000,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	0x0220_0000,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	0x0220_0000,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	0x0220_0000,// MVEX_Vgatherpf0hintdps_mvt_k1
	0x0220_0000,// MVEX_Vgatherpf0hintdpd_mvt_k1
	0x0228_0000,// MVEX_Vgatherpf0dps_mvt_k1
	0x0228_0000,// MVEX_Vgatherpf1dps_mvt_k1
	0x0220_0000,// MVEX_Vscatterpf0hintdps_mvt_k1
	0x0220_0000,// MVEX_Vscatterpf0hintdpd_mvt_k1
	0x0228_0000,// MVEX_Vscatterpf0dps_mvt_k1
	0x0228_0000,// MVEX_Vscatterpf1dps_mvt_k1
	0x0220_0000,// MVEX_Vexp223ps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vlog2ps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	0x0220_0000,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	0x0228_0000,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	0x0228_0000,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	0x0220_0000,// MVEX_Vloadunpackld_zmm_k1_mt
	0x0220_0000,// MVEX_Vloadunpacklq_zmm_k1_mt
	0x0220_0000,// MVEX_Vpackstoreld_mt_k1_zmm
	0x0220_0000,// MVEX_Vpackstorelq_mt_k1_zmm
	0x0220_0000,// MVEX_Vloadunpacklps_zmm_k1_mt
	0x0220_0000,// MVEX_Vloadunpacklpd_zmm_k1_mt
	0x0220_0000,// MVEX_Vpackstorelps_mt_k1_zmm
	0x0220_0000,// MVEX_Vpackstorelpd_mt_k1_zmm
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	0x0220_0000,// MVEX_Vloadunpackhd_zmm_k1_mt
	0x0220_0000,// MVEX_Vloadunpackhq_zmm_k1_mt
	0x0220_0000,// MVEX_Vpackstorehd_mt_k1_zmm
	0x0220_0000,// MVEX_Vpackstorehq_mt_k1_zmm
	0x0220_0000,// MVEX_Vloadunpackhps_zmm_k1_mt
	0x0220_0000,// MVEX_Vloadunpackhpd_zmm_k1_mt
	0x0220_0000,// MVEX_Vpackstorehps_mt_k1_zmm
	0x0220_0000,// MVEX_Vpackstorehpd_mt_k1_zmm
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	0x0220_0000,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	0x0220_0000,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	0x0220_0000,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	0x0220_0000,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	0x0220_0000,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	0x0221_0000,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	0x0220_0000,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	0x0001_0000,// Via_undoc_F30FA6F0_16
	0x0001_0000,// Via_undoc_F30FA6F0_32
	0x0001_0000,// Via_undoc_F30FA6F0_64
	0x0001_0000,// Via_undoc_F30FA6F8_16
	0x0001_0000,// Via_undoc_F30FA6F8_32
	0x0001_0000,// Via_undoc_F30FA6F8_64
	0x0000_0000,// Xsha512_16
	0x0000_0000,// Xsha512_32
	0x0000_0000,// Xsha512_64
	0x0000_0000,// Xstore_alt_16
	0x0000_0000,// Xstore_alt_32
	0x0000_0000,// Xstore_alt_64
	0x0000_0000,// Xsha512_alt_16
	0x0000_0000,// Xsha512_alt_32
	0x0000_0000,// Xsha512_alt_64
	0x0000_0000,// Zero_bytes
	0x0000_8001,// Wrmsrns
	0x0000_8001,// Wrmsrlist
	0x0000_8001,// Rdmsrlist
	0x0000_8001,// Rmpquery
	0x0000_0000,// Prefetchit1_m8
	0x0000_0000,// Prefetchit0_m8
	0x0010_0000,// Aadd_m32_r32
	0x0010_0000,// Aadd_m64_r64
	0x0010_0000,// Aand_m32_r32
	0x0010_0000,// Aand_m64_r64
	0x0010_0000,// Axor_m32_r32
	0x0010_0000,// Axor_m64_r64
	0x0010_0000,// Aor_m32_r32
	0x0010_0000,// Aor_m64_r64
	0x0000_0000,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	0x0010_4000,// VEX_Tdpfp16ps_tmm_tmm_tmm
	0x0000_0000,// VEX_Vcvtneps2bf16_xmm_xmmm128
	0x0000_0000,// VEX_Vcvtneps2bf16_xmm_ymmm256
	0x0010_0000,// VEX_Vcvtneoph2ps_xmm_m128
	0x0010_0000,// VEX_Vcvtneoph2ps_ymm_m256
	0x0010_0000,// VEX_Vcvtneeph2ps_xmm_m128
	0x0010_0000,// VEX_Vcvtneeph2ps_ymm_m256
	0x0010_0000,// VEX_Vcvtneebf162ps_xmm_m128
	0x0010_0000,// VEX_Vcvtneebf162ps_ymm_m256
	0x0010_0000,// VEX_Vcvtneobf162ps_xmm_m128
	0x0010_0000,// VEX_Vcvtneobf162ps_ymm_m256
	0x0010_0000,// VEX_Vbcstnesh2ps_xmm_m16
	0x0010_0000,// VEX_Vbcstnesh2ps_ymm_m16
	0x0010_0000,// VEX_Vbcstnebf162ps_xmm_m16
	0x0010_0000,// VEX_Vbcstnebf162ps_ymm_m16
	0x0000_0000,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	0x0010_0000,// VEX_Cmpoxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpoxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnoxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnoxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpbxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpbxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnbxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnbxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpzxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpzxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnzxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnzxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpbexadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpbexadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnbexadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnbexadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpsxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpsxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnsxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnsxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmppxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmppxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnpxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnpxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmplxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmplxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnlxadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnlxadd_m64_r64_r64
	0x0010_0000,// VEX_Cmplexadd_m32_r32_r32
	0x0010_0000,// VEX_Cmplexadd_m64_r64_r64
	0x0010_0000,// VEX_Cmpnlexadd_m32_r32_r32
	0x0010_0000,// VEX_Cmpnlexadd_m64_r64_r64
	0x0010_4000,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	0x0010_4000,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	0x0000_8001,// Pbndkb
	0x0010_0000,// VEX_Vsha512rnds2_ymm_ymm_xmm
	0x0010_0000,// VEX_Vsha512msg1_ymm_xmm
	0x0010_0000,// VEX_Vsha512msg2_ymm_ymm
	0x0000_0000,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsm4key4_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsm4key4_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	0x0000_0000,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	0x0000_0000,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];
#[rustfmt::skip]
pub(super) static OPC_FLAGS2: [u32; 4936] = [
	0x1E00_3FFF,// INVALID
	0x1E00_3FFF,// DeclareByte
	0x1E00_3FFF,// DeclareWord
	0x1E00_3FFF,// DeclareDword
	0x1E00_3FFF,// DeclareQword
	0x1E00_3FFF,// Add_rm8_r8
	0x1E00_3FFF,// Add_rm16_r16
	0x1E00_3FFF,// Add_rm32_r32
	0x1400_3FF0,// Add_rm64_r64
	0x1E00_3FFF,// Add_r8_rm8
	0x1E00_3FFF,// Add_r16_rm16
	0x1E00_3FFF,// Add_r32_rm32
	0x1400_3FF0,// Add_r64_rm64
	0x1E00_3FFF,// Add_AL_imm8
	0x1E00_3FFF,// Add_AX_imm16
	0x1E00_3FFF,// Add_EAX_imm32
	0x1400_3FF0,// Add_RAX_imm32
	0x0A00_3FFF,// Pushw_ES
	0x0A00_3FFF,// Pushd_ES
	0x0A80_3E7F,// Popw_ES
	0x0A80_3E7F,// Popd_ES
	0x1E00_3FFF,// Or_rm8_r8
	0x1E00_3FFF,// Or_rm16_r16
	0x1E00_3FFF,// Or_rm32_r32
	0x1400_3FF0,// Or_rm64_r64
	0x1E00_3FFF,// Or_r8_rm8
	0x1E00_3FFF,// Or_r16_rm16
	0x1E00_3FFF,// Or_r32_rm32
	0x1400_3FF0,// Or_r64_rm64
	0x1E00_3FFF,// Or_AL_imm8
	0x1E00_3FFF,// Or_AX_imm16
	0x1E00_3FFF,// Or_EAX_imm32
	0x1400_3FF0,// Or_RAX_imm32
	0x0A00_3FFF,// Pushw_CS
	0x0A00_3FFF,// Pushd_CS
	0x0A00_3FFF,// Popw_CS
	0x1E00_3FFF,// Adc_rm8_r8
	0x1E00_3FFF,// Adc_rm16_r16
	0x1E00_3FFF,// Adc_rm32_r32
	0x1400_3FF0,// Adc_rm64_r64
	0x1E00_3FFF,// Adc_r8_rm8
	0x1E00_3FFF,// Adc_r16_rm16
	0x1E00_3FFF,// Adc_r32_rm32
	0x1400_3FF0,// Adc_r64_rm64
	0x1E00_3FFF,// Adc_AL_imm8
	0x1E00_3FFF,// Adc_AX_imm16
	0x1E00_3FFF,// Adc_EAX_imm32
	0x1400_3FF0,// Adc_RAX_imm32
	0x0A00_3FFF,// Pushw_SS
	0x0A00_3FFF,// Pushd_SS
	0x0A80_3E7F,// Popw_SS
	0x0A80_3E7F,// Popd_SS
	0x1E00_3FFF,// Sbb_rm8_r8
	0x1E00_3FFF,// Sbb_rm16_r16
	0x1E00_3FFF,// Sbb_rm32_r32
	0x1400_3FF0,// Sbb_rm64_r64
	0x1E00_3FFF,// Sbb_r8_rm8
	0x1E00_3FFF,// Sbb_r16_rm16
	0x1E00_3FFF,// Sbb_r32_rm32
	0x1400_3FF0,// Sbb_r64_rm64
	0x1E00_3FFF,// Sbb_AL_imm8
	0x1E00_3FFF,// Sbb_AX_imm16
	0x1E00_3FFF,// Sbb_EAX_imm32
	0x1400_3FF0,// Sbb_RAX_imm32
	0x0A00_3FFF,// Pushw_DS
	0x0A00_3FFF,// Pushd_DS
	0x0A80_3E7F,// Popw_DS
	0x0A80_3E7F,// Popd_DS
	0x1E00_3FFF,// And_rm8_r8
	0x1E00_3FFF,// And_rm16_r16
	0x1E00_3FFF,// And_rm32_r32
	0x1400_3FF0,// And_rm64_r64
	0x1E00_3FFF,// And_r8_rm8
	0x1E00_3FFF,// And_r16_rm16
	0x1E00_3FFF,// And_r32_rm32
	0x1400_3FF0,// And_r64_rm64
	0x1E00_3FFF,// And_AL_imm8
	0x1E00_3FFF,// And_AX_imm16
	0x1E00_3FFF,// And_EAX_imm32
	0x1400_3FF0,// And_RAX_imm32
	0x0A00_3FFF,// Daa
	0x1E00_3FFF,// Sub_rm8_r8
	0x1E00_3FFF,// Sub_rm16_r16
	0x1E00_3FFF,// Sub_rm32_r32
	0x1400_3FF0,// Sub_rm64_r64
	0x1E00_3FFF,// Sub_r8_rm8
	0x1E00_3FFF,// Sub_r16_rm16
	0x1E00_3FFF,// Sub_r32_rm32
	0x1400_3FF0,// Sub_r64_rm64
	0x1E00_3FFF,// Sub_AL_imm8
	0x1E00_3FFF,// Sub_AX_imm16
	0x1E00_3FFF,// Sub_EAX_imm32
	0x1400_3FF0,// Sub_RAX_imm32
	0x0A00_3FFF,// Das
	0x1E00_3FFF,// Xor_rm8_r8
	0x1E00_3FFF,// Xor_rm16_r16
	0x1E00_3FFF,// Xor_rm32_r32
	0x1400_3FF0,// Xor_rm64_r64
	0x1E00_3FFF,// Xor_r8_rm8
	0x1E00_3FFF,// Xor_r16_rm16
	0x1E00_3FFF,// Xor_r32_rm32
	0x1400_3FF0,// Xor_r64_rm64
	0x1E00_3FFF,// Xor_AL_imm8
	0x1E00_3FFF,// Xor_AX_imm16
	0x1E00_3FFF,// Xor_EAX_imm32
	0x1400_3FF0,// Xor_RAX_imm32
	0x0A00_3FFF,// Aaa
	0x1E00_3FFF,// Cmp_rm8_r8
	0x1E00_3FFF,// Cmp_rm16_r16
	0x1E00_3FFF,// Cmp_rm32_r32
	0x1400_3FF0,// Cmp_rm64_r64
	0x1E00_3FFF,// Cmp_r8_rm8
	0x1E00_3FFF,// Cmp_r16_rm16
	0x1E00_3FFF,// Cmp_r32_rm32
	0x1400_3FF0,// Cmp_r64_rm64
	0x1E00_3FFF,// Cmp_AL_imm8
	0x1E00_3FFF,// Cmp_AX_imm16
	0x1E00_3FFF,// Cmp_EAX_imm32
	0x1400_3FF0,// Cmp_RAX_imm32
	0x0A00_3FFF,// Aas
	0x0A00_3FFF,// Inc_r16
	0x0A00_3FFF,// Inc_r32
	0x0A00_3FFF,// Dec_r16
	0x0A00_3FFF,// Dec_r32
	0x1E00_3FFF,// Push_r16
	0x0A00_3FFF,// Push_r32
	0x1400_3FF0,// Push_r64
	0x1E00_3FFF,// Pop_r16
	0x0A00_3FFF,// Pop_r32
	0x1400_3FF0,// Pop_r64
	0x0A00_3FFF,// Pushaw
	0x0A00_3FFF,// Pushad
	0x0A00_3FFF,// Popaw
	0x0A00_3FFF,// Popad
	0x0AA4_3FFF,// Bound_r16_m1616
	0x0AA4_3FFF,// Bound_r32_m3232
	0x0A00_3FFA,// Arpl_rm16_r16
	0x0A00_3FFA,// Arpl_r32m16_r32
	0x1400_3FF0,// Movsxd_r16_rm16
	0x1400_3FF0,// Movsxd_r32_rm32
	0x1400_3FF0,// Movsxd_r64_rm32
	0x1E00_3FFF,// Push_imm16
	0x0A00_3FFF,// Pushd_imm32
	0x1400_3FF0,// Pushq_imm32
	0x1E00_3FFF,// Imul_r16_rm16_imm16
	0x1E00_3FFF,// Imul_r32_rm32_imm32
	0x1400_3FF0,// Imul_r64_rm64_imm32
	0x1E00_3FFF,// Pushw_imm8
	0x0A00_3FFF,// Pushd_imm8
	0x1400_3FF0,// Pushq_imm8
	0x1E00_3FFF,// Imul_r16_rm16_imm8
	0x1E00_3FFF,// Imul_r32_rm32_imm8
	0x1400_3FF0,// Imul_r64_rm64_imm8
	0x1EA4_3E7F,// Insb_m8_DX
	0x1EA4_3E7F,// Insw_m16_DX
	0x1EA4_3E7F,// Insd_m32_DX
	0x1EA4_3E7F,// Outsb_DX_m8
	0x1EA4_3E7F,// Outsw_DX_m16
	0x1EA4_3E7F,// Outsd_DX_m32
	0x1A00_3FFF,// Jo_rel8_16
	0x0A00_3FFF,// Jo_rel8_32
	0x1400_3FF0,// Jo_rel8_64
	0x1A00_3FFF,// Jno_rel8_16
	0x0A00_3FFF,// Jno_rel8_32
	0x1400_3FF0,// Jno_rel8_64
	0x1A00_3FFF,// Jb_rel8_16
	0x0A00_3FFF,// Jb_rel8_32
	0x1400_3FF0,// Jb_rel8_64
	0x1A00_3FFF,// Jae_rel8_16
	0x0A00_3FFF,// Jae_rel8_32
	0x1400_3FF0,// Jae_rel8_64
	0x1A00_3FFF,// Je_rel8_16
	0x0A00_3FFF,// Je_rel8_32
	0x1400_3FF0,// Je_rel8_64
	0x1A00_3FFF,// Jne_rel8_16
	0x0A00_3FFF,// Jne_rel8_32
	0x1400_3FF0,// Jne_rel8_64
	0x1A00_3FFF,// Jbe_rel8_16
	0x0A00_3FFF,// Jbe_rel8_32
	0x1400_3FF0,// Jbe_rel8_64
	0x1A00_3FFF,// Ja_rel8_16
	0x0A00_3FFF,// Ja_rel8_32
	0x1400_3FF0,// Ja_rel8_64
	0x1A00_3FFF,// Js_rel8_16
	0x0A00_3FFF,// Js_rel8_32
	0x1400_3FF0,// Js_rel8_64
	0x1A00_3FFF,// Jns_rel8_16
	0x0A00_3FFF,// Jns_rel8_32
	0x1400_3FF0,// Jns_rel8_64
	0x1A00_3FFF,// Jp_rel8_16
	0x0A00_3FFF,// Jp_rel8_32
	0x1400_3FF0,// Jp_rel8_64
	0x1A00_3FFF,// Jnp_rel8_16
	0x0A00_3FFF,// Jnp_rel8_32
	0x1400_3FF0,// Jnp_rel8_64
	0x1A00_3FFF,// Jl_rel8_16
	0x0A00_3FFF,// Jl_rel8_32
	0x1400_3FF0,// Jl_rel8_64
	0x1A00_3FFF,// Jge_rel8_16
	0x0A00_3FFF,// Jge_rel8_32
	0x1400_3FF0,// Jge_rel8_64
	0x1A00_3FFF,// Jle_rel8_16
	0x0A00_3FFF,// Jle_rel8_32
	0x1400_3FF0,// Jle_rel8_64
	0x1A00_3FFF,// Jg_rel8_16
	0x0A00_3FFF,// Jg_rel8_32
	0x1400_3FF0,// Jg_rel8_64
	0x1E00_3FFF,// Add_rm8_imm8
	0x1E00_3FFF,// Or_rm8_imm8
	0x1E00_3FFF,// Adc_rm8_imm8
	0x1E00_3FFF,// Sbb_rm8_imm8
	0x1E00_3FFF,// And_rm8_imm8
	0x1E00_3FFF,// Sub_rm8_imm8
	0x1E00_3FFF,// Xor_rm8_imm8
	0x1E00_3FFF,// Cmp_rm8_imm8
	0x1E00_3FFF,// Add_rm16_imm16
	0x1E00_3FFF,// Add_rm32_imm32
	0x1400_3FF0,// Add_rm64_imm32
	0x1E00_3FFF,// Or_rm16_imm16
	0x1E00_3FFF,// Or_rm32_imm32
	0x1400_3FF0,// Or_rm64_imm32
	0x1E00_3FFF,// Adc_rm16_imm16
	0x1E00_3FFF,// Adc_rm32_imm32
	0x1400_3FF0,// Adc_rm64_imm32
	0x1E00_3FFF,// Sbb_rm16_imm16
	0x1E00_3FFF,// Sbb_rm32_imm32
	0x1400_3FF0,// Sbb_rm64_imm32
	0x1E00_3FFF,// And_rm16_imm16
	0x1E00_3FFF,// And_rm32_imm32
	0x1400_3FF0,// And_rm64_imm32
	0x1E00_3FFF,// Sub_rm16_imm16
	0x1E00_3FFF,// Sub_rm32_imm32
	0x1400_3FF0,// Sub_rm64_imm32
	0x1E00_3FFF,// Xor_rm16_imm16
	0x1E00_3FFF,// Xor_rm32_imm32
	0x1400_3FF0,// Xor_rm64_imm32
	0x1E00_3FFF,// Cmp_rm16_imm16
	0x1E00_3FFF,// Cmp_rm32_imm32
	0x1400_3FF0,// Cmp_rm64_imm32
	0x0A00_3FFF,// Add_rm8_imm8_82
	0x0A00_3FFF,// Or_rm8_imm8_82
	0x0A00_3FFF,// Adc_rm8_imm8_82
	0x0A00_3FFF,// Sbb_rm8_imm8_82
	0x0A00_3FFF,// And_rm8_imm8_82
	0x0A00_3FFF,// Sub_rm8_imm8_82
	0x0A00_3FFF,// Xor_rm8_imm8_82
	0x0A00_3FFF,// Cmp_rm8_imm8_82
	0x1E00_3FFF,// Add_rm16_imm8
	0x1E00_3FFF,// Add_rm32_imm8
	0x1400_3FF0,// Add_rm64_imm8
	0x1E00_3FFF,// Or_rm16_imm8
	0x1E00_3FFF,// Or_rm32_imm8
	0x1400_3FF0,// Or_rm64_imm8
	0x1E00_3FFF,// Adc_rm16_imm8
	0x1E00_3FFF,// Adc_rm32_imm8
	0x1400_3FF0,// Adc_rm64_imm8
	0x1E00_3FFF,// Sbb_rm16_imm8
	0x1E00_3FFF,// Sbb_rm32_imm8
	0x1400_3FF0,// Sbb_rm64_imm8
	0x1E00_3FFF,// And_rm16_imm8
	0x1E00_3FFF,// And_rm32_imm8
	0x1400_3FF0,// And_rm64_imm8
	0x1E00_3FFF,// Sub_rm16_imm8
	0x1E00_3FFF,// Sub_rm32_imm8
	0x1400_3FF0,// Sub_rm64_imm8
	0x1E00_3FFF,// Xor_rm16_imm8
	0x1E00_3FFF,// Xor_rm32_imm8
	0x1400_3FF0,// Xor_rm64_imm8
	0x1E00_3FFF,// Cmp_rm16_imm8
	0x1E00_3FFF,// Cmp_rm32_imm8
	0x1400_3FF0,// Cmp_rm64_imm8
	0x1E00_3FFF,// Test_rm8_r8
	0x1E00_3FFF,// Test_rm16_r16
	0x1E00_3FFF,// Test_rm32_r32
	0x1400_3FF0,// Test_rm64_r64
	0x1E00_3FFF,// Xchg_rm8_r8
	0x1E00_3FFF,// Xchg_rm16_r16
	0x1E00_3FFF,// Xchg_rm32_r32
	0x1400_3FF0,// Xchg_rm64_r64
	0x1E00_3FFF,// Mov_rm8_r8
	0x1E00_3FFF,// Mov_rm16_r16
	0x1E00_3FFF,// Mov_rm32_r32
	0x1400_3FF0,// Mov_rm64_r64
	0x1E00_3FFF,// Mov_r8_rm8
	0x1E00_3FFF,// Mov_r16_rm16
	0x1E00_3FFF,// Mov_r32_rm32
	0x1400_3FF0,// Mov_r64_rm64
	0x1E00_3FFF,// Mov_rm16_Sreg
	0x1E00_3FFF,// Mov_r32m16_Sreg
	0x1400_3FF0,// Mov_r64m16_Sreg
	0x1E00_3FFF,// Lea_r16_m
	0x1E00_3FFF,// Lea_r32_m
	0x1400_3FF0,// Lea_r64_m
	0x1E80_3E7F,// Mov_Sreg_rm16
	0x1E80_3E7F,// Mov_Sreg_r32m16
	0x1480_3E70,// Mov_Sreg_r64m16
	0x1E00_3FFF,// Pop_rm16
	0x0A00_3FFF,// Pop_rm32
	0x1400_3FF0,// Pop_rm64
	0x1E00_3FFF,// Nopw
	0x1E00_3FFF,// Nopd
	0x1400_3FF0,// Nopq
	0x1E00_3FFF,// Xchg_r16_AX
	0x1E00_3FFF,// Xchg_r32_EAX
	0x1400_3FF0,// Xchg_r64_RAX
	0x1E64_3FFF,// Pause
	0x1E00_3FFF,// Cbw
	0x1E00_3FFF,// Cwde
	0x1400_3FF0,// Cdqe
	0x1E00_3FFF,// Cwd
	0x1E00_3FFF,// Cdq
	0x1400_3FF0,// Cqo
	0x0AA4_3E7F,// Call_ptr1616
	0x0AA4_3E7F,// Call_ptr1632
	0x1E80_3FFF,// Wait
	0x1E20_3FFF,// Pushfw
	0x0A20_3FFF,// Pushfd
	0x1420_3FF0,// Pushfq
	0x1EA0_3FFF,// Popfw
	0x0AA0_3FFF,// Popfd
	0x14A0_3FF0,// Popfq
	0x1E00_3FFF,// Sahf
	0x1E00_3FFF,// Lahf
	0x1E00_3FFF,// Mov_AL_moffs8
	0x1E00_3FFF,// Mov_AX_moffs16
	0x1E00_3FFF,// Mov_EAX_moffs32
	0x1400_3FF0,// Mov_RAX_moffs64
	0x1E00_3FFF,// Mov_moffs8_AL
	0x1E00_3FFF,// Mov_moffs16_AX
	0x1E00_3FFF,// Mov_moffs32_EAX
	0x1400_3FF0,// Mov_moffs64_RAX
	0x1E00_3FFF,// Movsb_m8_m8
	0x1E00_3FFF,// Movsw_m16_m16
	0x1E00_3FFF,// Movsd_m32_m32
	0x1400_3FF0,// Movsq_m64_m64
	0x1E00_3FFF,// Cmpsb_m8_m8
	0x1E00_3FFF,// Cmpsw_m16_m16
	0x1E00_3FFF,// Cmpsd_m32_m32
	0x1400_3FF0,// Cmpsq_m64_m64
	0x1E00_3FFF,// Test_AL_imm8
	0x1E00_3FFF,// Test_AX_imm16
	0x1E00_3FFF,// Test_EAX_imm32
	0x1400_3FF0,// Test_RAX_imm32
	0x1E00_3FFF,// Stosb_m8_AL
	0x1E00_3FFF,// Stosw_m16_AX
	0x1E00_3FFF,// Stosd_m32_EAX
	0x1400_3FF0,// Stosq_m64_RAX
	0x1E00_3FFF,// Lodsb_AL_m8
	0x1E00_3FFF,// Lodsw_AX_m16
	0x1E00_3FFF,// Lodsd_EAX_m32
	0x1400_3FF0,// Lodsq_RAX_m64
	0x1E00_3FFF,// Scasb_AL_m8
	0x1E00_3FFF,// Scasw_AX_m16
	0x1E00_3FFF,// Scasd_EAX_m32
	0x1400_3FF0,// Scasq_RAX_m64
	0x1E00_3FFF,// Mov_r8_imm8
	0x1E00_3FFF,// Mov_r16_imm16
	0x1E00_3FFF,// Mov_r32_imm32
	0x1400_3FF0,// Mov_r64_imm64
	0x1E00_3FFF,// Rol_rm8_imm8
	0x1E00_3FFF,// Ror_rm8_imm8
	0x1E00_3FFF,// Rcl_rm8_imm8
	0x1E00_3FFF,// Rcr_rm8_imm8
	0x1E00_3FFF,// Shl_rm8_imm8
	0x1E00_3FFF,// Shr_rm8_imm8
	0x1E00_3FFF,// Sal_rm8_imm8
	0x1E00_3FFF,// Sar_rm8_imm8
	0x1E00_3FFF,// Rol_rm16_imm8
	0x1E00_3FFF,// Rol_rm32_imm8
	0x1400_3FF0,// Rol_rm64_imm8
	0x1E00_3FFF,// Ror_rm16_imm8
	0x1E00_3FFF,// Ror_rm32_imm8
	0x1400_3FF0,// Ror_rm64_imm8
	0x1E00_3FFF,// Rcl_rm16_imm8
	0x1E00_3FFF,// Rcl_rm32_imm8
	0x1400_3FF0,// Rcl_rm64_imm8
	0x1E00_3FFF,// Rcr_rm16_imm8
	0x1E00_3FFF,// Rcr_rm32_imm8
	0x1400_3FF0,// Rcr_rm64_imm8
	0x1E00_3FFF,// Shl_rm16_imm8
	0x1E00_3FFF,// Shl_rm32_imm8
	0x1400_3FF0,// Shl_rm64_imm8
	0x1E00_3FFF,// Shr_rm16_imm8
	0x1E00_3FFF,// Shr_rm32_imm8
	0x1400_3FF0,// Shr_rm64_imm8
	0x1E00_3FFF,// Sal_rm16_imm8
	0x1E00_3FFF,// Sal_rm32_imm8
	0x1400_3FF0,// Sal_rm64_imm8
	0x1E00_3FFF,// Sar_rm16_imm8
	0x1E00_3FFF,// Sar_rm32_imm8
	0x1400_3FF0,// Sar_rm64_imm8
	0x1A00_3FFF,// Retnw_imm16
	0x0A00_3FFF,// Retnd_imm16
	0x1400_3FF0,// Retnq_imm16
	0x1A00_3FFF,// Retnw
	0x0A00_3FFF,// Retnd
	0x1400_3FF0,// Retnq
	0x0A80_3E7F,// Les_r16_m1616
	0x0A80_3E7F,// Les_r32_m1632
	0x0A80_3E7F,// Lds_r16_m1616
	0x0A80_3E7F,// Lds_r32_m1632
	0x1E00_3FFF,// Mov_rm8_imm8
	0x1E40_3FFF,// Xabort_imm8
	0x1E00_3FFF,// Mov_rm16_imm16
	0x1E00_3FFF,// Mov_rm32_imm32
	0x1400_3FF0,// Mov_rm64_imm32
	0x1F00_3FFF,// Xbegin_rel16
	0x1F00_3FFF,// Xbegin_rel32
	0x1E00_3FFF,// Enterw_imm16_imm8
	0x0A00_3FFF,// Enterd_imm16_imm8
	0x1400_3FF0,// Enterq_imm16_imm8
	0x1E00_3FFF,// Leavew
	0x0A00_3FFF,// Leaved
	0x1400_3FF0,// Leaveq
	0x1E80_3E7F,// Retfw_imm16
	0x1E80_3E7F,// Retfd_imm16
	0x1480_3E70,// Retfq_imm16
	0x1E80_3E7F,// Retfw
	0x1E80_3E7F,// Retfd
	0x1480_3E70,// Retfq
	0x1E62_3FFF,// Int3
	0x1EA4_3E7F,// Int_imm8
	0x0AA4_3E7F,// Into
	0x1EA4_3E7F,// Iretw
	0x1EA4_3E7F,// Iretd
	0x14A4_3E70,// Iretq
	0x1E00_3FFF,// Rol_rm8_1
	0x1E00_3FFF,// Ror_rm8_1
	0x1E00_3FFF,// Rcl_rm8_1
	0x1E00_3FFF,// Rcr_rm8_1
	0x1E00_3FFF,// Shl_rm8_1
	0x1E00_3FFF,// Shr_rm8_1
	0x1E00_3FFF,// Sal_rm8_1
	0x1E00_3FFF,// Sar_rm8_1
	0x1E00_3FFF,// Rol_rm16_1
	0x1E00_3FFF,// Rol_rm32_1
	0x1400_3FF0,// Rol_rm64_1
	0x1E00_3FFF,// Ror_rm16_1
	0x1E00_3FFF,// Ror_rm32_1
	0x1400_3FF0,// Ror_rm64_1
	0x1E00_3FFF,// Rcl_rm16_1
	0x1E00_3FFF,// Rcl_rm32_1
	0x1400_3FF0,// Rcl_rm64_1
	0x1E00_3FFF,// Rcr_rm16_1
	0x1E00_3FFF,// Rcr_rm32_1
	0x1400_3FF0,// Rcr_rm64_1
	0x1E00_3FFF,// Shl_rm16_1
	0x1E00_3FFF,// Shl_rm32_1
	0x1400_3FF0,// Shl_rm64_1
	0x1E00_3FFF,// Shr_rm16_1
	0x1E00_3FFF,// Shr_rm32_1
	0x1400_3FF0,// Shr_rm64_1
	0x1E00_3FFF,// Sal_rm16_1
	0x1E00_3FFF,// Sal_rm32_1
	0x1400_3FF0,// Sal_rm64_1
	0x1E00_3FFF,// Sar_rm16_1
	0x1E00_3FFF,// Sar_rm32_1
	0x1400_3FF0,// Sar_rm64_1
	0x1E00_3FFF,// Rol_rm8_CL
	0x1E00_3FFF,// Ror_rm8_CL
	0x1E00_3FFF,// Rcl_rm8_CL
	0x1E00_3FFF,// Rcr_rm8_CL
	0x1E00_3FFF,// Shl_rm8_CL
	0x1E00_3FFF,// Shr_rm8_CL
	0x1E00_3FFF,// Sal_rm8_CL
	0x1E00_3FFF,// Sar_rm8_CL
	0x1E00_3FFF,// Rol_rm16_CL
	0x1E00_3FFF,// Rol_rm32_CL
	0x1400_3FF0,// Rol_rm64_CL
	0x1E00_3FFF,// Ror_rm16_CL
	0x1E00_3FFF,// Ror_rm32_CL
	0x1400_3FF0,// Ror_rm64_CL
	0x1E00_3FFF,// Rcl_rm16_CL
	0x1E00_3FFF,// Rcl_rm32_CL
	0x1400_3FF0,// Rcl_rm64_CL
	0x1E00_3FFF,// Rcr_rm16_CL
	0x1E00_3FFF,// Rcr_rm32_CL
	0x1400_3FF0,// Rcr_rm64_CL
	0x1E00_3FFF,// Shl_rm16_CL
	0x1E00_3FFF,// Shl_rm32_CL
	0x1400_3FF0,// Shl_rm64_CL
	0x1E00_3FFF,// Shr_rm16_CL
	0x1E00_3FFF,// Shr_rm32_CL
	0x1400_3FF0,// Shr_rm64_CL
	0x1E00_3FFF,// Sal_rm16_CL
	0x1E00_3FFF,// Sal_rm32_CL
	0x1400_3FF0,// Sal_rm64_CL
	0x1E00_3FFF,// Sar_rm16_CL
	0x1E00_3FFF,// Sar_rm32_CL
	0x1400_3FF0,// Sar_rm64_CL
	0x0A00_3FFF,// Aam_imm8
	0x0A00_3FFF,// Aad_imm8
	0x0A00_3FFF,// Salc
	0x1E00_3FFF,// Xlat_m8
	0x1E80_3FFF,// Fadd_m32fp
	0x1E80_3FFF,// Fmul_m32fp
	0x1E80_3FFF,// Fcom_m32fp
	0x1E80_3FFF,// Fcomp_m32fp
	0x1E80_3FFF,// Fsub_m32fp
	0x1E80_3FFF,// Fsubr_m32fp
	0x1E80_3FFF,// Fdiv_m32fp
	0x1E80_3FFF,// Fdivr_m32fp
	0x1E80_3FFF,// Fadd_st0_sti
	0x1E80_3FFF,// Fmul_st0_sti
	0xBE80_3FFF,// Fcom_st0_sti
	0xBE80_3FFF,// Fcomp_st0_sti
	0x1E80_3FFF,// Fsub_st0_sti
	0x1E80_3FFF,// Fsubr_st0_sti
	0x1E80_3FFF,// Fdiv_st0_sti
	0x1E80_3FFF,// Fdivr_st0_sti
	0x1E80_3FFF,// Fld_m32fp
	0x1E80_3FFF,// Fst_m32fp
	0x1E80_3FFF,// Fstp_m32fp
	0x1E80_3FFF,// Fldenv_m14byte
	0x1E80_3FFF,// Fldenv_m28byte
	0x1E80_3FFF,// Fldcw_m2byte
	0x1E80_3FFF,// Fnstenv_m14byte
	0x1E80_3FFF,// Fstenv_m14byte
	0x1E80_3FFF,// Fnstenv_m28byte
	0x1E80_3FFF,// Fstenv_m28byte
	0x1E80_3FFF,// Fnstcw_m2byte
	0x1E80_3FFF,// Fstcw_m2byte
	0x1E80_3FFF,// Fld_sti
	0xBE80_3FFF,// Fxch_st0_sti
	0x1E80_3FFF,// Fnop
	0x1E80_3FFF,// Fstpnce_sti
	0x1E80_3FFF,// Fchs
	0x1E80_3FFF,// Fabs
	0x1E80_3FFF,// Ftst
	0x1E80_3FFF,// Fxam
	0x1E80_3FFF,// Fld1
	0x1E80_3FFF,// Fldl2t
	0x1E80_3FFF,// Fldl2e
	0x1E80_3FFF,// Fldpi
	0x1E80_3FFF,// Fldlg2
	0x1E80_3FFF,// Fldln2
	0x1E80_3FFF,// Fldz
	0x1E80_3FFF,// F2xm1
	0x1E80_3FFF,// Fyl2x
	0x1E80_3FFF,// Fptan
	0x1E80_3FFF,// Fpatan
	0x1E80_3FFF,// Fxtract
	0x1E80_3FFF,// Fprem1
	0x1E80_3FFF,// Fdecstp
	0x1E80_3FFF,// Fincstp
	0x1E80_3FFF,// Fprem
	0x1E80_3FFF,// Fyl2xp1
	0x1E80_3FFF,// Fsqrt
	0x1E80_3FFF,// Fsincos
	0x1E80_3FFF,// Frndint
	0x1E80_3FFF,// Fscale
	0x1E80_3FFF,// Fsin
	0x1E80_3FFF,// Fcos
	0x1E80_3FFF,// Fiadd_m32int
	0x1E80_3FFF,// Fimul_m32int
	0x1E80_3FFF,// Ficom_m32int
	0x1E80_3FFF,// Ficomp_m32int
	0x1E80_3FFF,// Fisub_m32int
	0x1E80_3FFF,// Fisubr_m32int
	0x1E80_3FFF,// Fidiv_m32int
	0x1E80_3FFF,// Fidivr_m32int
	0x1E80_3FFF,// Fcmovb_st0_sti
	0x1E80_3FFF,// Fcmove_st0_sti
	0x1E80_3FFF,// Fcmovbe_st0_sti
	0x1E80_3FFF,// Fcmovu_st0_sti
	0x1E80_3FFF,// Fucompp
	0x1E80_3FFF,// Fild_m32int
	0x1E80_3FFF,// Fisttp_m32int
	0x1E80_3FFF,// Fist_m32int
	0x1E80_3FFF,// Fistp_m32int
	0x1E80_3FFF,// Fld_m80fp
	0x1E80_3FFF,// Fstp_m80fp
	0x1E80_3FFF,// Fcmovnb_st0_sti
	0x1E80_3FFF,// Fcmovne_st0_sti
	0x1E80_3FFF,// Fcmovnbe_st0_sti
	0x1E80_3FFF,// Fcmovnu_st0_sti
	0x1E80_3FFF,// Fneni
	0x1E80_3FFF,// Feni
	0x1E80_3FFF,// Fndisi
	0x1E80_3FFF,// Fdisi
	0x1E80_3FFF,// Fnclex
	0x1E80_3FFF,// Fclex
	0x1E80_3FFF,// Fninit
	0x1E80_3FFF,// Finit
	0x1E80_3FFF,// Fnsetpm
	0x1E80_3FFF,// Fsetpm
	0x0A80_3FFF,// Frstpm
	0x1E80_3FFF,// Fucomi_st0_sti
	0x1E80_3FFF,// Fcomi_st0_sti
	0x1E80_3FFF,// Fadd_m64fp
	0x1E80_3FFF,// Fmul_m64fp
	0x1E80_3FFF,// Fcom_m64fp
	0x1E80_3FFF,// Fcomp_m64fp
	0x1E80_3FFF,// Fsub_m64fp
	0x1E80_3FFF,// Fsubr_m64fp
	0x1E80_3FFF,// Fdiv_m64fp
	0x1E80_3FFF,// Fdivr_m64fp
	0x1E80_3FFF,// Fadd_sti_st0
	0x1E80_3FFF,// Fmul_sti_st0
	0xBE80_3FFF,// Fcom_st0_sti_DCD0
	0xBE80_3FFF,// Fcomp_st0_sti_DCD8
	0x1E80_3FFF,// Fsubr_sti_st0
	0x1E80_3FFF,// Fsub_sti_st0
	0x1E80_3FFF,// Fdivr_sti_st0
	0x1E80_3FFF,// Fdiv_sti_st0
	0x1E80_3FFF,// Fld_m64fp
	0x1E80_3FFF,// Fisttp_m64int
	0x1E80_3FFF,// Fst_m64fp
	0x1E80_3FFF,// Fstp_m64fp
	0x1E80_3FFF,// Frstor_m94byte
	0x1E80_3FFF,// Frstor_m108byte
	0x1E80_3FFF,// Fnsave_m94byte
	0x1E80_3FFF,// Fsave_m94byte
	0x1E80_3FFF,// Fnsave_m108byte
	0x1E80_3FFF,// Fsave_m108byte
	0x1E80_3FFF,// Fnstsw_m2byte
	0x1E80_3FFF,// Fstsw_m2byte
	0x1E80_3FFF,// Ffree_sti
	0xBE80_3FFF,// Fxch_st0_sti_DDC8
	0x1E80_3FFF,// Fst_sti
	0x1E80_3FFF,// Fstp_sti
	0xBE80_3FFF,// Fucom_st0_sti
	0xBE80_3FFF,// Fucomp_st0_sti
	0x1E80_3FFF,// Fiadd_m16int
	0x1E80_3FFF,// Fimul_m16int
	0x1E80_3FFF,// Ficom_m16int
	0x1E80_3FFF,// Ficomp_m16int
	0x1E80_3FFF,// Fisub_m16int
	0x1E80_3FFF,// Fisubr_m16int
	0x1E80_3FFF,// Fidiv_m16int
	0x1E80_3FFF,// Fidivr_m16int
	0x1E80_3FFF,// Faddp_sti_st0
	0x1E80_3FFF,// Fmulp_sti_st0
	0xBE80_3FFF,// Fcomp_st0_sti_DED0
	0x1E80_3FFF,// Fcompp
	0x1E80_3FFF,// Fsubrp_sti_st0
	0x1E80_3FFF,// Fsubp_sti_st0
	0x1E80_3FFF,// Fdivrp_sti_st0
	0x1E80_3FFF,// Fdivp_sti_st0
	0x1E80_3FFF,// Fild_m16int
	0x1E80_3FFF,// Fisttp_m16int
	0x1E80_3FFF,// Fist_m16int
	0x1E80_3FFF,// Fistp_m16int
	0x1E80_3FFF,// Fbld_m80bcd
	0x1E80_3FFF,// Fild_m64int
	0x1E80_3FFF,// Fbstp_m80bcd
	0x1E80_3FFF,// Fistp_m64int
	0x1E80_3FFF,// Ffreep_sti
	0xBE80_3FFF,// Fxch_st0_sti_DFC8
	0x1E80_3FFF,// Fstp_sti_DFD0
	0x1E80_3FFF,// Fstp_sti_DFD8
	0x1E80_3FFF,// Fnstsw_AX
	0x1E80_3FFF,// Fstsw_AX
	0x0A80_3FFF,// Fstdw_AX
	0x0A80_3FFF,// Fstsg_AX
	0x1E80_3FFF,// Fucomip_st0_sti
	0x1E80_3FFF,// Fcomip_st0_sti
	0x0A00_3FFF,// Loopne_rel8_16_CX
	0x0A00_3FFF,// Loopne_rel8_32_CX
	0x1A00_3FFF,// Loopne_rel8_16_ECX
	0x0A00_3FFF,// Loopne_rel8_32_ECX
	0x1400_3FF0,// Loopne_rel8_64_ECX
	0x1000_3FF0,// Loopne_rel8_16_RCX
	0x1400_3FF0,// Loopne_rel8_64_RCX
	0x0A00_3FFF,// Loope_rel8_16_CX
	0x0A00_3FFF,// Loope_rel8_32_CX
	0x1A00_3FFF,// Loope_rel8_16_ECX
	0x0A00_3FFF,// Loope_rel8_32_ECX
	0x1400_3FF0,// Loope_rel8_64_ECX
	0x1000_3FF0,// Loope_rel8_16_RCX
	0x1400_3FF0,// Loope_rel8_64_RCX
	0x0A00_3FFF,// Loop_rel8_16_CX
	0x0A00_3FFF,// Loop_rel8_32_CX
	0x1A00_3FFF,// Loop_rel8_16_ECX
	0x0A00_3FFF,// Loop_rel8_32_ECX
	0x1400_3FF0,// Loop_rel8_64_ECX
	0x1000_3FF0,// Loop_rel8_16_RCX
	0x1400_3FF0,// Loop_rel8_64_RCX
	0x0A00_3FFF,// Jcxz_rel8_16
	0x0A00_3FFF,// Jcxz_rel8_32
	0x1A00_3FFF,// Jecxz_rel8_16
	0x0A00_3FFF,// Jecxz_rel8_32
	0x1400_3FF0,// Jecxz_rel8_64
	0x1000_3FF0,// Jrcxz_rel8_16
	0x1400_3FF0,// Jrcxz_rel8_64
	0x1EA4_3E7F,// In_AL_imm8
	0x1EA4_3E7F,// In_AX_imm8
	0x1EA4_3E7F,// In_EAX_imm8
	0x1EA4_3E7F,// Out_imm8_AL
	0x1EA4_3E7F,// Out_imm8_AX
	0x1EA4_3E7F,// Out_imm8_EAX
	0x1A00_3FFF,// Call_rel16
	0x0A00_3FFF,// Call_rel32_32
	0x1400_3FF0,// Call_rel32_64
	0x1A00_3FFF,// Jmp_rel16
	0x0A00_3FFF,// Jmp_rel32_32
	0x1400_3FF0,// Jmp_rel32_64
	0x0AA4_3E7F,// Jmp_ptr1616
	0x0AA4_3E7F,// Jmp_ptr1632
	0x1A00_3FFF,// Jmp_rel8_16
	0x0A00_3FFF,// Jmp_rel8_32
	0x1400_3FF0,// Jmp_rel8_64
	0x1EA4_3E7F,// In_AL_DX
	0x1EA4_3E7F,// In_AX_DX
	0x1EA4_3E7F,// In_EAX_DX
	0x1EA4_3E7F,// Out_DX_AL
	0x1EA4_3E7F,// Out_DX_AX
	0x1EA4_3E7F,// Out_DX_EAX
	0x1E62_3FFF,// Int1
	0x1EA4_3E7B,// Hlt
	0x1E00_3FFF,// Cmc
	0x1E00_3FFF,// Test_rm8_imm8
	0x1E00_3FFF,// Test_rm8_imm8_F6r1
	0x1E00_3FFF,// Not_rm8
	0x1E00_3FFF,// Neg_rm8
	0x1E00_3FFF,// Mul_rm8
	0x1E00_3FFF,// Imul_rm8
	0x1E00_3FFF,// Div_rm8
	0x1E00_3FFF,// Idiv_rm8
	0x1E00_3FFF,// Test_rm16_imm16
	0x1E00_3FFF,// Test_rm32_imm32
	0x1400_3FF0,// Test_rm64_imm32
	0x1E00_3FFF,// Test_rm16_imm16_F7r1
	0x1E00_3FFF,// Test_rm32_imm32_F7r1
	0x1400_3FF0,// Test_rm64_imm32_F7r1
	0x1E00_3FFF,// Not_rm16
	0x1E00_3FFF,// Not_rm32
	0x1400_3FF0,// Not_rm64
	0x1E00_3FFF,// Neg_rm16
	0x1E00_3FFF,// Neg_rm32
	0x1400_3FF0,// Neg_rm64
	0x1E00_3FFF,// Mul_rm16
	0x1E00_3FFF,// Mul_rm32
	0x1400_3FF0,// Mul_rm64
	0x1E00_3FFF,// Imul_rm16
	0x1E00_3FFF,// Imul_rm32
	0x1400_3FF0,// Imul_rm64
	0x1E00_3FFF,// Div_rm16
	0x1E00_3FFF,// Div_rm32
	0x1400_3FF0,// Div_rm64
	0x1E00_3FFF,// Idiv_rm16
	0x1E00_3FFF,// Idiv_rm32
	0x1400_3FF0,// Idiv_rm64
	0x1E00_3FFF,// Clc
	0x1E00_3FFF,// Stc
	0x1E80_3FFF,// Cli
	0x1E80_3FFF,// Sti
	0x1E80_3FFF,// Cld
	0x1E80_3FFF,// Std
	0x1E00_3FFF,// Inc_rm8
	0x1E00_3FFF,// Dec_rm8
	0x1E00_3FFF,// Inc_rm16
	0x1E00_3FFF,// Inc_rm32
	0x1400_3FF0,// Inc_rm64
	0x1E00_3FFF,// Dec_rm16
	0x1E00_3FFF,// Dec_rm32
	0x1400_3FF0,// Dec_rm64
	0x1A00_3FFF,// Call_rm16
	0x0A00_3FFF,// Call_rm32
	0x1400_3FF0,// Call_rm64
	0x1EA4_3E7F,// Call_m1616
	0x1EA4_3E7F,// Call_m1632
	0x0484_3E70,// Call_m1664
	0x1A00_3FFF,// Jmp_rm16
	0x0A00_3FFF,// Jmp_rm32
	0x1400_3FF0,// Jmp_rm64
	0x1EA4_3E7F,// Jmp_m1616
	0x1EA4_3E7F,// Jmp_m1632
	0x0484_3E70,// Jmp_m1664
	0x1E00_3FFF,// Push_rm16
	0x0A00_3FFF,// Push_rm32
	0x1400_3FF0,// Push_rm64
	0x1EA4_3E7A,// Sldt_rm16
	0x1EA4_3E7A,// Sldt_r32m16
	0x14A4_3E70,// Sldt_r64m16
	0x1EA4_3E7A,// Str_rm16
	0x1EA4_3E7A,// Str_r32m16
	0x14A4_3E70,// Str_r64m16
	0x1EA4_3E7A,// Lldt_rm16
	0x1EA4_3E7A,// Lldt_r32m16
	0x14A4_3E70,// Lldt_r64m16
	0x1EA4_3E7A,// Ltr_rm16
	0x1EA4_3E7A,// Ltr_r32m16
	0x14A4_3E70,// Ltr_r64m16
	0x1E00_3FFA,// Verr_rm16
	0x1E00_3FFA,// Verr_r32m16
	0x1400_3FF0,// Verr_r64m16
	0x1E00_3FFA,// Verw_rm16
	0x1E00_3FFA,// Verw_r32m16
	0x1400_3FF0,// Verw_r64m16
	0x0A00_3FFF,// Jmpe_rm16
	0x0A00_3FFF,// Jmpe_rm32
	0x0AA4_3E7F,// Sgdt_m1632_16
	0x0AA4_3E7F,// Sgdt_m1632
	0x14A4_3E70,// Sgdt_m1664
	0x0AA4_3E7F,// Sidt_m1632_16
	0x0AA4_3E7F,// Sidt_m1632
	0x14A4_3E70,// Sidt_m1664
	0x0AA4_3E7B,// Lgdt_m1632_16
	0x0AA4_3E7B,// Lgdt_m1632
	0x14A4_3E70,// Lgdt_m1664
	0x0AA4_3E7B,// Lidt_m1632_16
	0x0AA4_3E7B,// Lidt_m1632
	0x14A4_3E70,// Lidt_m1664
	0x1EA4_3E7F,// Smsw_rm16
	0x1EA4_3E7F,// Smsw_r32m16
	0x14A4_3E70,// Smsw_r64m16
	0x1E00_3FFA,// Rstorssp_m64
	0x1EA4_3E7B,// Lmsw_rm16
	0x1EA4_3E7B,// Lmsw_r32m16
	0x14A4_3E70,// Lmsw_r64m16
	0x1EA4_3E7B,// Invlpg_m
	0x1E44_3C5A,// Enclv
	0x1E8A_3C7B,// Vmcall
	0x1E82_3C72,// Vmlaunch
	0x1E82_3C72,// Vmresume
	0x1E82_3C72,// Vmxoff
	0x1E84_3E7B,// Pconfig
	0x0AA4_3FFF,// Monitorw
	0x1EA4_3FFF,// Monitord
	0x14A4_3FF0,// Monitorq
	0x1EA4_3FFF,// Mwait
	0x1E80_3E7B,// Clac
	0x1E80_3E7B,// Stac
	0x1E44_3E5A,// Encls
	0x1E00_3FFF,// Xgetbv
	0x1EA2_3E7B,// Xsetbv
	0x1E84_387F,// Vmfunc
	0x1F00_3FFF,// Xend
	0x1E00_3FFF,// Xtest
	0x1E40_3FDA,// Enclu
	0x0A20_3E7A,// Vmrunw
	0x1E20_3E7A,// Vmrund
	0x1420_3E70,// Vmrunq
	0x1E20_3FFF,// Vmmcall
	0x0A20_3E7A,// Vmloadw
	0x1E20_3E7A,// Vmloadd
	0x1420_3E70,// Vmloadq
	0x0A20_3E7A,// Vmsavew
	0x1E20_3E7A,// Vmsaved
	0x1420_3E70,// Vmsaveq
	0x1E20_3E7A,// Stgi
	0x1E20_3E7A,// Clgi
	0x1E20_3E7A,// Skinit
	0x0A20_3E7A,// Invlpgaw
	0x1E20_3E7A,// Invlpgad
	0x1420_3E70,// Invlpgaq
	0x1E00_3E7A,// Setssbsy
	0x1E00_3FFA,// Saveprevssp
	0x1E00_3FFF,// Rdpkru
	0x1E80_3FFF,// Wrpkru
	0x1480_3E70,// Swapgs
	0x1EA4_3F7F,// Rdtscp
	0x0A20_3FFF,// Monitorxw
	0x1E20_3FFF,// Monitorxd
	0x1420_3FF0,// Monitorxq
	0x1E20_3FFF,// Mcommit
	0x1E20_3FFF,// Mwaitx
	0x0A00_3FFF,// Clzerow
	0x1E00_3FFF,// Clzerod
	0x1400_3FF0,// Clzeroq
	0x1E20_3FFF,// Rdpru
	0x1E00_3FFA,// Lar_r16_rm16
	0x1E00_3FFA,// Lar_r32_r32m16
	0x1400_3FF0,// Lar_r64_r64m16
	0x1E00_3FFA,// Lsl_r16_rm16
	0x1E00_3FFA,// Lsl_r32_r32m16
	0x1400_3FF0,// Lsl_r64_r64m16
	0x0A00_3E7B,// Storeall
	0x0A00_3E7B,// Loadall286
	0x1E80_3E7F,// Syscall
	0x1EA4_3E7B,// Clts
	0x0A00_3E7B,// Loadall386
	0x1E80_3E7B,// Sysretd
	0x1480_3E70,// Sysretq
	0x1EA2_3E7B,// Invd
	0x1EA4_3E7B,// Wbinvd
	0x1EA4_3E7B,// Wbnoinvd
	0x0A00_3E7B,// Cl1invmb
	0x1E82_3FFF,// Ud2
	0x1E00_3FFF,// Reservednop_rm16_r16_0F0D
	0x1E00_3FFF,// Reservednop_rm32_r32_0F0D
	0x1400_3FF0,// Reservednop_rm64_r64_0F0D
	0x1E00_3FFF,// Prefetch_m8
	0x1E00_3FFF,// Prefetchw_m8
	0x1E00_3FFF,// Prefetchwt1_m8
	0x1E00_3FFF,// Femms
	0x0A00_3FFF,// Umov_rm8_r8
	0x0A00_3FFF,// Umov_rm16_r16
	0x0A00_3FFF,// Umov_rm32_r32
	0x0A00_3FFF,// Umov_r8_rm8
	0x0A00_3FFF,// Umov_r16_rm16
	0x0A00_3FFF,// Umov_r32_rm32
	0x1E00_3FFF,// Movups_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovups_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovups_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovups_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovups_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovups_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movupd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovupd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovupd_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovupd_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovupd_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovupd_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vmovss_xmm_xmm_xmm
	0x1E00_3FFA,// VEX_Vmovss_xmm_m32
	0x1E00_3FFA,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	0x1E00_3FFA,// EVEX_Vmovss_xmm_k1z_m32
	0x1E00_3FFF,// Movsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmovsd_xmm_xmm_xmm
	0x1E00_3FFA,// VEX_Vmovsd_xmm_m64
	0x1E00_3FFA,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	0x1E00_3FFA,// EVEX_Vmovsd_xmm_k1z_m64
	0x9E00_3FFF,// Movups_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovups_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovups_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovups_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovups_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovups_zmmm512_k1z_zmm
	0x9E00_3FFF,// Movupd_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovupd_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovupd_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovupd_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovupd_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovupd_zmmm512_k1z_zmm
	0x9E00_3FFF,// Movss_xmmm32_xmm
	0x1E00_3FFA,// VEX_Vmovss_xmm_xmm_xmm_0F11
	0x1E00_3FFA,// VEX_Vmovss_m32_xmm
	0x1E00_3FFA,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	0x1E00_3FFA,// EVEX_Vmovss_m32_k1_xmm
	0x1E00_3FFF,// Movsd_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	0x1E00_3FFA,// VEX_Vmovsd_m64_xmm
	0x1E00_3FFA,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	0x1E00_3FFA,// EVEX_Vmovsd_m64_k1_xmm
	0x1E00_3FFF,// Movhlps_xmm_xmm
	0x1E00_3FFF,// Movlps_xmm_m64
	0x1E00_3FFA,// VEX_Vmovhlps_xmm_xmm_xmm
	0x9E00_3FFA,// VEX_Vmovlps_xmm_xmm_m64
	0x1E00_3FFA,// EVEX_Vmovhlps_xmm_xmm_xmm
	0x9E00_3FFA,// EVEX_Vmovlps_xmm_xmm_m64
	0x1E00_3FFF,// Movlpd_xmm_m64
	0x9E00_3FFA,// VEX_Vmovlpd_xmm_xmm_m64
	0x9E00_3FFA,// EVEX_Vmovlpd_xmm_xmm_m64
	0x1E00_3FFF,// Movsldup_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovsldup_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovsldup_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movddup_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmovddup_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmovddup_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovddup_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vmovddup_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovddup_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movlps_m64_xmm
	0x1E00_3FFA,// VEX_Vmovlps_m64_xmm
	0x1E00_3FFA,// EVEX_Vmovlps_m64_xmm
	0x1E00_3FFF,// Movlpd_m64_xmm
	0x1E00_3FFA,// VEX_Vmovlpd_m64_xmm
	0x1E00_3FFA,// EVEX_Vmovlpd_m64_xmm
	0x1E00_3FFF,// Unpcklps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpcklps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpcklps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Unpcklpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Unpckhps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpckhps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpckhps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Unpckhpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Movlhps_xmm_xmm
	0x1E00_3FFA,// VEX_Vmovlhps_xmm_xmm_xmm
	0x1E00_3FFA,// EVEX_Vmovlhps_xmm_xmm_xmm
	0x1E00_3FFF,// Movhps_xmm_m64
	0x9E00_3FFA,// VEX_Vmovhps_xmm_xmm_m64
	0x9E00_3FFA,// EVEX_Vmovhps_xmm_xmm_m64
	0x1E00_3FFF,// Movhpd_xmm_m64
	0x9E00_3FFA,// VEX_Vmovhpd_xmm_xmm_m64
	0x9E00_3FFA,// EVEX_Vmovhpd_xmm_xmm_m64
	0x1E00_3FFF,// Movshdup_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovshdup_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovshdup_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movhps_m64_xmm
	0x1E00_3FFA,// VEX_Vmovhps_m64_xmm
	0x1E00_3FFA,// EVEX_Vmovhps_m64_xmm
	0x1E00_3FFF,// Movhpd_m64_xmm
	0x1E00_3FFA,// VEX_Vmovhpd_m64_xmm
	0x1E00_3FFA,// EVEX_Vmovhpd_m64_xmm
	0x1E00_3FFF,// Reservednop_rm16_r16_0F18
	0x1E00_3FFF,// Reservednop_rm32_r32_0F18
	0x1400_3FF0,// Reservednop_rm64_r64_0F18
	0x1E00_3FFF,// Reservednop_rm16_r16_0F19
	0x1E00_3FFF,// Reservednop_rm32_r32_0F19
	0x1400_3FF0,// Reservednop_rm64_r64_0F19
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1A
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1A
	0x1400_3FF0,// Reservednop_rm64_r64_0F1A
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1B
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1B
	0x1400_3FF0,// Reservednop_rm64_r64_0F1B
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1C
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1C
	0x1400_3FF0,// Reservednop_rm64_r64_0F1C
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1D
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1D
	0x1400_3FF0,// Reservednop_rm64_r64_0F1D
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1E
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1E
	0x1400_3FF0,// Reservednop_rm64_r64_0F1E
	0x1E00_3FFF,// Reservednop_rm16_r16_0F1F
	0x1E00_3FFF,// Reservednop_rm32_r32_0F1F
	0x1400_3FF0,// Reservednop_rm64_r64_0F1F
	0x1E80_3FFF,// Prefetchnta_m8
	0x1E00_3FFF,// Prefetcht0_m8
	0x1E00_3FFF,// Prefetcht1_m8
	0x1E00_3FFF,// Prefetcht2_m8
	0x1E80_3FFF,// Bndldx_bnd_mib
	0x0A00_3FFF,// Bndmov_bnd_bndm64
	0x1400_3FF0,// Bndmov_bnd_bndm128
	0x0A00_3FFF,// Bndcl_bnd_rm32
	0x1400_3FF0,// Bndcl_bnd_rm64
	0x0A00_3FFF,// Bndcu_bnd_rm32
	0x1400_3FF0,// Bndcu_bnd_rm64
	0x1E80_3FFF,// Bndstx_mib_bnd
	0x0A00_3FFF,// Bndmov_bndm64_bnd
	0x1400_3FF0,// Bndmov_bndm128_bnd
	0x0A00_3FFF,// Bndmk_bnd_m32
	0x1400_3FF0,// Bndmk_bnd_m64
	0x0A00_3FFF,// Bndcn_bnd_rm32
	0x1400_3FF0,// Bndcn_bnd_rm64
	0x1E80_3FFF,// Cldemote_m8
	0x1E00_3FFF,// Rdsspd_r32
	0x1400_3FF0,// Rdsspq_r64
	0x1E00_3FFF,// Endbr64
	0x1E00_3FFF,// Endbr32
	0x1E00_3FFF,// Nop_rm16
	0x1E00_3FFF,// Nop_rm32
	0x1400_3FF0,// Nop_rm64
	0x0AA4_3E7B,// Mov_r32_cr
	0x14A4_3E70,// Mov_r64_cr
	0x0AA4_3E7B,// Mov_r32_dr
	0x14A4_3E70,// Mov_r64_dr
	0x0AA4_3E7B,// Mov_cr_r32
	0x14A4_3E70,// Mov_cr_r64
	0x0AA4_3E7B,// Mov_dr_r32
	0x14A4_3E70,// Mov_dr_r64
	0x0A00_3E7B,// Mov_r32_tr
	0x0A00_3E7B,// Mov_tr_r32
	0x1E00_3FFF,// Movaps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovaps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovaps_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovaps_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovaps_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovaps_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movapd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovapd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovapd_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovapd_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovapd_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovapd_zmm_k1z_zmmm512
	0x9E00_3FFF,// Movaps_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovaps_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovaps_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovaps_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovaps_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovaps_zmmm512_k1z_zmm
	0x9E00_3FFF,// Movapd_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovapd_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovapd_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovapd_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovapd_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovapd_zmmm512_k1z_zmm
	0x7E80_3FFF,// Cvtpi2ps_xmm_mmm64
	0x7E80_3FFF,// Cvtpi2pd_xmm_mmm64
	0x1E00_3FFF,// Cvtsi2ss_xmm_rm32
	0x1400_3FF0,// Cvtsi2ss_xmm_rm64
	0x1E00_3FFA,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	0x1400_3FF0,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	0x1E00_3FFA,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	0x1E00_3FFF,// Cvtsi2sd_xmm_rm32
	0x1400_3FF0,// Cvtsi2sd_xmm_rm64
	0x1E00_3FFA,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	0x1400_3FF0,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	0x1E00_3FFA,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	0x1E80_3FFF,// Movntps_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntps_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntps_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntps_m128_xmm
	0x1E80_3FFA,// EVEX_Vmovntps_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntps_m512_zmm
	0x1E80_3FFF,// Movntpd_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntpd_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntpd_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntpd_m128_xmm
	0x1E80_3FFA,// EVEX_Vmovntpd_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntpd_m512_zmm
	0x1E00_3FFF,// Movntss_m32_xmm
	0x1E00_3FFF,// Movntsd_m64_xmm
	0x7E80_3FFF,// Cvttps2pi_mm_xmmm64
	0x7E80_3FFF,// Cvttpd2pi_mm_xmmm128
	0x1E00_3FFF,// Cvttss2si_r32_xmmm32
	0x1400_3FF0,// Cvttss2si_r64_xmmm32
	0x1E00_3FFA,// VEX_Vcvttss2si_r32_xmmm32
	0x1400_3FF0,// VEX_Vcvttss2si_r64_xmmm32
	0x1E00_3FFA,// EVEX_Vcvttss2si_r32_xmmm32_sae
	0x1400_3FF0,// EVEX_Vcvttss2si_r64_xmmm32_sae
	0x1E00_3FFF,// Cvttsd2si_r32_xmmm64
	0x1400_3FF0,// Cvttsd2si_r64_xmmm64
	0x1E00_3FFA,// VEX_Vcvttsd2si_r32_xmmm64
	0x1400_3FF0,// VEX_Vcvttsd2si_r64_xmmm64
	0x1E00_3FFA,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	0x1400_3FF0,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	0x7E80_3FFF,// Cvtps2pi_mm_xmmm64
	0x7E80_3FFF,// Cvtpd2pi_mm_xmmm128
	0x1E00_3FFF,// Cvtss2si_r32_xmmm32
	0x1400_3FF0,// Cvtss2si_r64_xmmm32
	0x1E00_3FFA,// VEX_Vcvtss2si_r32_xmmm32
	0x1400_3FF0,// VEX_Vcvtss2si_r64_xmmm32
	0x1E00_3FFA,// EVEX_Vcvtss2si_r32_xmmm32_er
	0x1400_3FF0,// EVEX_Vcvtss2si_r64_xmmm32_er
	0x1E00_3FFF,// Cvtsd2si_r32_xmmm64
	0x1400_3FF0,// Cvtsd2si_r64_xmmm64
	0x1E00_3FFA,// VEX_Vcvtsd2si_r32_xmmm64
	0x1400_3FF0,// VEX_Vcvtsd2si_r64_xmmm64
	0x1E00_3FFA,// EVEX_Vcvtsd2si_r32_xmmm64_er
	0x1400_3FF0,// EVEX_Vcvtsd2si_r64_xmmm64_er
	0x1E00_3FFF,// Ucomiss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vucomiss_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vucomiss_xmm_xmmm32_sae
	0x1E00_3FFF,// Ucomisd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vucomisd_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vucomisd_xmm_xmmm64_sae
	0x1E00_3FFF,// Comiss_xmm_xmmm32
	0x1E00_3FFF,// Comisd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcomiss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vcomisd_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vcomiss_xmm_xmmm32_sae
	0x1E00_3FFA,// EVEX_Vcomisd_xmm_xmmm64_sae
	0x1EA4_3E7B,// Wrmsr
	0x1EA4_3F7F,// Rdtsc
	0x1EA4_3E7B,// Rdmsr
	0x1EA4_3E7F,// Rdpmc
	0x1E80_3E7F,// Sysenter
	0x1E80_3E7B,// Sysexitd
	0x1480_3E70,// Sysexitq
	0x1E82_3E7F,// Getsecd
	0x1E00_3FFF,// Cmovo_r16_rm16
	0x1E00_3FFF,// Cmovo_r32_rm32
	0x1400_3FF0,// Cmovo_r64_rm64
	0x1E00_3FFF,// Cmovno_r16_rm16
	0x1E00_3FFF,// Cmovno_r32_rm32
	0x1400_3FF0,// Cmovno_r64_rm64
	0x1E00_3FFF,// Cmovb_r16_rm16
	0x1E00_3FFF,// Cmovb_r32_rm32
	0x1400_3FF0,// Cmovb_r64_rm64
	0x1E00_3FFF,// Cmovae_r16_rm16
	0x1E00_3FFF,// Cmovae_r32_rm32
	0x1400_3FF0,// Cmovae_r64_rm64
	0x1E00_3FFF,// Cmove_r16_rm16
	0x1E00_3FFF,// Cmove_r32_rm32
	0x1400_3FF0,// Cmove_r64_rm64
	0x1E00_3FFF,// Cmovne_r16_rm16
	0x1E00_3FFF,// Cmovne_r32_rm32
	0x1400_3FF0,// Cmovne_r64_rm64
	0x1E00_3FFF,// Cmovbe_r16_rm16
	0x1E00_3FFF,// Cmovbe_r32_rm32
	0x1400_3FF0,// Cmovbe_r64_rm64
	0x1E00_3FFF,// Cmova_r16_rm16
	0x1E00_3FFF,// Cmova_r32_rm32
	0x1400_3FF0,// Cmova_r64_rm64
	0x1E00_3FFF,// Cmovs_r16_rm16
	0x1E00_3FFF,// Cmovs_r32_rm32
	0x1400_3FF0,// Cmovs_r64_rm64
	0x1E00_3FFF,// Cmovns_r16_rm16
	0x1E00_3FFF,// Cmovns_r32_rm32
	0x1400_3FF0,// Cmovns_r64_rm64
	0x1E00_3FFF,// Cmovp_r16_rm16
	0x1E00_3FFF,// Cmovp_r32_rm32
	0x1400_3FF0,// Cmovp_r64_rm64
	0x1E00_3FFF,// Cmovnp_r16_rm16
	0x1E00_3FFF,// Cmovnp_r32_rm32
	0x1400_3FF0,// Cmovnp_r64_rm64
	0x1E00_3FFF,// Cmovl_r16_rm16
	0x1E00_3FFF,// Cmovl_r32_rm32
	0x1400_3FF0,// Cmovl_r64_rm64
	0x1E00_3FFF,// Cmovge_r16_rm16
	0x1E00_3FFF,// Cmovge_r32_rm32
	0x1400_3FF0,// Cmovge_r64_rm64
	0x1E00_3FFF,// Cmovle_r16_rm16
	0x1E00_3FFF,// Cmovle_r32_rm32
	0x1400_3FF0,// Cmovle_r64_rm64
	0x1E00_3FFF,// Cmovg_r16_rm16
	0x1E00_3FFF,// Cmovg_r32_rm32
	0x1400_3FF0,// Cmovg_r64_rm64
	0x1E00_3FFA,// VEX_Kandw_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandd_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandnw_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandnq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandnb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kandnd_kr_kr_kr
	0x1E00_3FFA,// VEX_Knotw_kr_kr
	0x1E00_3FFA,// VEX_Knotq_kr_kr
	0x1E00_3FFA,// VEX_Knotb_kr_kr
	0x1E00_3FFA,// VEX_Knotd_kr_kr
	0x1E00_3FFA,// VEX_Korw_kr_kr_kr
	0x1E00_3FFA,// VEX_Korq_kr_kr_kr
	0x1E00_3FFA,// VEX_Korb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kord_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxnorw_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxnorq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxnorb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxnord_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxorw_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxorq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxorb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kxord_kr_kr_kr
	0x1E00_3FFA,// VEX_Kaddw_kr_kr_kr
	0x1E00_3FFA,// VEX_Kaddq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kaddb_kr_kr_kr
	0x1E00_3FFA,// VEX_Kaddd_kr_kr_kr
	0x1E00_3FFA,// VEX_Kunpckwd_kr_kr_kr
	0x1E00_3FFA,// VEX_Kunpckdq_kr_kr_kr
	0x1E00_3FFA,// VEX_Kunpckbw_kr_kr_kr
	0x7E00_3FFF,// Movmskps_r32_xmm
	0x7400_3FF0,// Movmskps_r64_xmm
	0x5E00_3FFA,// VEX_Vmovmskps_r32_xmm
	0x5400_3FF0,// VEX_Vmovmskps_r64_xmm
	0x5E00_3FFA,// VEX_Vmovmskps_r32_ymm
	0x5400_3FF0,// VEX_Vmovmskps_r64_ymm
	0x7E00_3FFF,// Movmskpd_r32_xmm
	0x7400_3FF0,// Movmskpd_r64_xmm
	0x5E00_3FFA,// VEX_Vmovmskpd_r32_xmm
	0x5400_3FF0,// VEX_Vmovmskpd_r64_xmm
	0x5E00_3FFA,// VEX_Vmovmskpd_r32_ymm
	0x5400_3FF0,// VEX_Vmovmskpd_r64_ymm
	0x1E00_3FFF,// Sqrtps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsqrtps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsqrtps_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	0x1E00_3FFF,// Sqrtpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsqrtpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsqrtpd_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	0x1E00_3FFF,// Sqrtss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vsqrtss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFF,// Sqrtsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Rsqrtps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vrsqrtps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vrsqrtps_ymm_ymmm256
	0x1E00_3FFF,// Rsqrtss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	0x1E00_3FFF,// Rcpps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vrcpps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vrcpps_ymm_ymmm256
	0x1E00_3FFF,// Rcpss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vrcpss_xmm_xmm_xmmm32
	0x1E00_3FFF,// Andps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Andpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Andnps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandnps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandnps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Andnpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandnpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vandnpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Orps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vorps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vorps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Orpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vorpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vorpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Xorps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vxorps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vxorps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Xorpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vxorpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vxorpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Addps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFF,// Addpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFF,// Addss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vaddss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFF,// Addsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vaddsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Mulps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmulps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmulps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFF,// Mulpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmulpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmulpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFF,// Mulss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vmulss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFF,// Mulsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmulsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Cvtps2pd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtps2pd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtps2pd_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	0x1E00_3FFF,// Cvtpd2ps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtpd2ps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtpd2ps_xmm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	0x1E00_3FFF,// Cvtss2sd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFF,// Cvtsd2ss_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Cvtdq2ps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtdq2ps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtdq2ps_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	0x1E00_3FFF,// Cvtps2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtps2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtps2dq_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	0x1E00_3FFF,// Cvttps2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvttps2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvttps2dq_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFF,// Subps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsubps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsubps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFF,// Subpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsubpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsubpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFF,// Subss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vsubss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFF,// Subsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vsubsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Minps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vminps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vminps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	0x1E00_3FFF,// Minpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vminpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vminpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	0x1E00_3FFF,// Minss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vminss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFF,// Minsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vminsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	0x1E00_3FFF,// Divps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vdivps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vdivps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFF,// Divpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vdivpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vdivpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFF,// Divss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vdivss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFF,// Divsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vdivsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFF,// Maxps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmaxps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmaxps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	0x1E00_3FFF,// Maxpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmaxpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmaxpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	0x1E00_3FFF,// Maxss_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vmaxss_xmm_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFF,// Maxsd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmaxsd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	0x7E80_3FFF,// Punpcklbw_mm_mmm32
	0x1E00_3FFF,// Punpcklbw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Punpcklwd_mm_mmm32
	0x1E00_3FFF,// Punpcklwd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Punpckldq_mm_mmm32
	0x1E00_3FFF,// Punpckldq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	0x1E80_3FFF,// Packsswb_mm_mmm64
	0x1E00_3FFF,// Packsswb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpacksswb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpacksswb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Pcmpgtb_mm_mmm64
	0x1E00_3FFF,// Pcmpgtb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	0x7E80_3FFF,// Pcmpgtw_mm_mmm64
	0x1E00_3FFF,// Pcmpgtw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	0x7E80_3FFF,// Pcmpgtd_mm_mmm64
	0x1E00_3FFF,// Pcmpgtd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	0x7E80_3FFF,// Packuswb_mm_mmm64
	0x1E00_3FFF,// Packuswb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackuswb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackuswb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Punpckhbw_mm_mmm64
	0x1E00_3FFF,// Punpckhbw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Punpckhwd_mm_mmm64
	0x1E00_3FFF,// Punpckhwd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Punpckhdq_mm_mmm64
	0x1E00_3FFF,// Punpckhdq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	0x1E80_3FFF,// Packssdw_mm_mmm64
	0x1E00_3FFF,// Packssdw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackssdw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackssdw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFF,// Punpcklqdq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Punpckhqdq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Movd_mm_rm32
	0x7480_3FF0,// Movq_mm_rm64
	0x7E00_3FFF,// Movd_xmm_rm32
	0x7400_3FF0,// Movq_xmm_rm64
	0x1E00_3FFA,// VEX_Vmovd_xmm_rm32
	0x1400_3FF0,// VEX_Vmovq_xmm_rm64
	0x1E00_3FFA,// EVEX_Vmovd_xmm_rm32
	0x1400_3FF0,// EVEX_Vmovq_xmm_rm64
	0x7E80_3FFF,// Movq_mm_mmm64
	0x1E00_3FFF,// Movdqa_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovdqa_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovdqa_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	0x1E00_3FFF,// Movdqu_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovdqu_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vmovdqu_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	0x1E80_3FFF,// Pshufw_mm_mmm64_imm8
	0x1E00_3FFF,// Pshufd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshufd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshufd_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	0x1E00_3FFF,// Pshufhw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshufhw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshufhw_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	0x1E00_3FFF,// Pshuflw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshuflw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpshuflw_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	0x7E80_3FFF,// Psrlw_mm_imm8
	0x1E00_3FFF,// Psrlw_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrlw_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrlw_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	0x7E80_3FFF,// Psraw_mm_imm8
	0x1E00_3FFF,// Psraw_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsraw_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsraw_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	0x1E80_3FFF,// Psllw_mm_imm8
	0x1E00_3FFF,// Psllw_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsllw_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsllw_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFA,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	0x7E80_3FFF,// Psrld_mm_imm8
	0x1E00_3FFF,// Psrld_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrld_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrld_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	0x7E80_3FFF,// Psrad_mm_imm8
	0x1E00_3FFF,// Psrad_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrad_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrad_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	0x7E80_3FFF,// Pslld_mm_imm8
	0x1E00_3FFF,// Pslld_xmm_imm8
	0x1E00_3FFA,// VEX_Vpslld_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpslld_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	0x7E80_3FFF,// Psrlq_mm_imm8
	0x1E00_3FFF,// Psrlq_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrlq_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrlq_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFF,// Psrldq_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrldq_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsrldq_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	0x7E80_3FFF,// Psllq_mm_imm8
	0x1E00_3FFF,// Psllq_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsllq_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpsllq_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFF,// Pslldq_xmm_imm8
	0x1E00_3FFA,// VEX_Vpslldq_xmm_xmm_imm8
	0x1E00_3FFA,// VEX_Vpslldq_ymm_ymm_imm8
	0x1E00_3FFA,// EVEX_Vpslldq_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpslldq_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpslldq_zmm_zmmm512_imm8
	0x7E80_3FFF,// Pcmpeqb_mm_mmm64
	0x1E00_3FFF,// Pcmpeqb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	0x7E80_3FFF,// Pcmpeqw_mm_mmm64
	0x1E00_3FFF,// Pcmpeqw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	0x7E80_3FFF,// Pcmpeqd_mm_mmm64
	0x1E00_3FFF,// Pcmpeqd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	0x1E80_3FFF,// Emms
	0x1E80_3FFA,// VEX_Vzeroupper
	0x1E00_3FFA,// VEX_Vzeroall
	0x0A84_3C72,// Vmread_rm32_r32
	0x1484_3C70,// Vmread_rm64_r64
	0x1E00_3FFA,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFA,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	0x1E00_3FFF,// Extrq_xmm_imm8_imm8
	0x1E00_3FFA,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	0x1E00_3FFA,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFA,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	0x1400_3FF0,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	0x1E00_3FFF,// Insertq_xmm_xmm_imm8_imm8
	0x1E00_3FFA,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	0x1400_3FF0,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	0x0A84_3C72,// Vmwrite_r32_rm32
	0x1484_3C70,// Vmwrite_r64_rm64
	0x1E00_3FFA,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	0x1E00_3FFF,// Extrq_xmm_xmm
	0x1E00_3FFA,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	0x1E00_3FFA,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtss2usi_r32_xmmm32_er
	0x1400_3FF0,// EVEX_Vcvtss2usi_r64_xmmm32_er
	0x1E00_3FFF,// Insertq_xmm_xmm
	0x1E00_3FFA,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	0x1400_3FF0,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	0x1E00_3FFA,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	0x1E00_3FFA,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFA,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	0x1E00_3FFA,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	0x1E00_3FFA,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	0x1E00_3FFA,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	0x1E00_3FFF,// Haddpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhaddpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhaddpd_ymm_ymm_ymmm256
	0x1E00_3FFF,// Haddps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhaddps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhaddps_ymm_ymm_ymmm256
	0x1E00_3FFF,// Hsubpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhsubpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhsubpd_ymm_ymm_ymmm256
	0x1E00_3FFF,// Hsubps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhsubps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vhsubps_ymm_ymm_ymmm256
	0x7E80_3FFF,// Movd_rm32_mm
	0x7480_3FF0,// Movq_rm64_mm
	0x7E00_3FFF,// Movd_rm32_xmm
	0x7400_3FF0,// Movq_rm64_xmm
	0x1E00_3FFA,// VEX_Vmovd_rm32_xmm
	0x1400_3FF0,// VEX_Vmovq_rm64_xmm
	0x1E00_3FFA,// EVEX_Vmovd_rm32_xmm
	0x1400_3FF0,// EVEX_Vmovq_rm64_xmm
	0x1E00_3FFF,// Movq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vmovq_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vmovq_xmm_xmmm64
	0x7E80_3FFF,// Movq_mmm64_mm
	0x9E00_3FFF,// Movdqa_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovdqa_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovdqa_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	0x9E00_3FFA,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	0x9E00_3FFF,// Movdqu_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovdqu_xmmm128_xmm
	0x9E00_3FFA,// VEX_Vmovdqu_ymmm256_ymm
	0x9E00_3FFA,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	0x9E00_3FFA,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	0x9E00_3FFA,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	0x9E00_3FFA,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	0x9E00_3FFA,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	0x9E00_3FFA,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	0x1A00_3FFF,// Jo_rel16
	0x0A00_3FFF,// Jo_rel32_32
	0x1400_3FF0,// Jo_rel32_64
	0x1A00_3FFF,// Jno_rel16
	0x0A00_3FFF,// Jno_rel32_32
	0x1400_3FF0,// Jno_rel32_64
	0x1A00_3FFF,// Jb_rel16
	0x0A00_3FFF,// Jb_rel32_32
	0x1400_3FF0,// Jb_rel32_64
	0x1A00_3FFF,// Jae_rel16
	0x0A00_3FFF,// Jae_rel32_32
	0x1400_3FF0,// Jae_rel32_64
	0x1A00_3FFF,// Je_rel16
	0x0A00_3FFF,// Je_rel32_32
	0x1400_3FF0,// Je_rel32_64
	0x1A00_3FFF,// Jne_rel16
	0x0A00_3FFF,// Jne_rel32_32
	0x1400_3FF0,// Jne_rel32_64
	0x1A00_3FFF,// Jbe_rel16
	0x0A00_3FFF,// Jbe_rel32_32
	0x1400_3FF0,// Jbe_rel32_64
	0x1A00_3FFF,// Ja_rel16
	0x0A00_3FFF,// Ja_rel32_32
	0x1400_3FF0,// Ja_rel32_64
	0x1A00_3FFF,// Js_rel16
	0x0A00_3FFF,// Js_rel32_32
	0x1400_3FF0,// Js_rel32_64
	0x1A00_3FFF,// Jns_rel16
	0x0A00_3FFF,// Jns_rel32_32
	0x1400_3FF0,// Jns_rel32_64
	0x1A00_3FFF,// Jp_rel16
	0x0A00_3FFF,// Jp_rel32_32
	0x1400_3FF0,// Jp_rel32_64
	0x1A00_3FFF,// Jnp_rel16
	0x0A00_3FFF,// Jnp_rel32_32
	0x1400_3FF0,// Jnp_rel32_64
	0x1A00_3FFF,// Jl_rel16
	0x0A00_3FFF,// Jl_rel32_32
	0x1400_3FF0,// Jl_rel32_64
	0x1A00_3FFF,// Jge_rel16
	0x0A00_3FFF,// Jge_rel32_32
	0x1400_3FF0,// Jge_rel32_64
	0x1A00_3FFF,// Jle_rel16
	0x0A00_3FFF,// Jle_rel32_32
	0x1400_3FF0,// Jle_rel32_64
	0x1A00_3FFF,// Jg_rel16
	0x0A00_3FFF,// Jg_rel32_32
	0x1400_3FF0,// Jg_rel32_64
	0x1E00_3FFF,// Seto_rm8
	0x1E00_3FFF,// Setno_rm8
	0x1E00_3FFF,// Setb_rm8
	0x1E00_3FFF,// Setae_rm8
	0x1E00_3FFF,// Sete_rm8
	0x1E00_3FFF,// Setne_rm8
	0x1E00_3FFF,// Setbe_rm8
	0x1E00_3FFF,// Seta_rm8
	0x1E00_3FFF,// Sets_rm8
	0x1E00_3FFF,// Setns_rm8
	0x1E00_3FFF,// Setp_rm8
	0x1E00_3FFF,// Setnp_rm8
	0x1E00_3FFF,// Setl_rm8
	0x1E00_3FFF,// Setge_rm8
	0x1E00_3FFF,// Setle_rm8
	0x1E00_3FFF,// Setg_rm8
	0x1E00_3FFA,// VEX_Kmovw_kr_km16
	0x1E00_3FFA,// VEX_Kmovq_kr_km64
	0x1E00_3FFA,// VEX_Kmovb_kr_km8
	0x1E00_3FFA,// VEX_Kmovd_kr_km32
	0x1E00_3FFA,// VEX_Kmovw_m16_kr
	0x1E00_3FFA,// VEX_Kmovq_m64_kr
	0x1E00_3FFA,// VEX_Kmovb_m8_kr
	0x1E00_3FFA,// VEX_Kmovd_m32_kr
	0x1E00_3FFA,// VEX_Kmovw_kr_r32
	0x1E00_3FFA,// VEX_Kmovb_kr_r32
	0x1E00_3FFA,// VEX_Kmovd_kr_r32
	0x1400_3FF0,// VEX_Kmovq_kr_r64
	0x1E00_3FFA,// VEX_Kmovw_r32_kr
	0x1E00_3FFA,// VEX_Kmovb_r32_kr
	0x1E00_3FFA,// VEX_Kmovd_r32_kr
	0x1400_3FF0,// VEX_Kmovq_r64_kr
	0x1E00_3FFA,// VEX_Kortestw_kr_kr
	0x1E00_3FFA,// VEX_Kortestq_kr_kr
	0x1E00_3FFA,// VEX_Kortestb_kr_kr
	0x1E00_3FFA,// VEX_Kortestd_kr_kr
	0x1E00_3FFA,// VEX_Ktestw_kr_kr
	0x1E00_3FFA,// VEX_Ktestq_kr_kr
	0x1E00_3FFA,// VEX_Ktestb_kr_kr
	0x1E00_3FFA,// VEX_Ktestd_kr_kr
	0x1E00_3FFF,// Pushw_FS
	0x0A00_3FFF,// Pushd_FS
	0x1400_3FF0,// Pushq_FS
	0x1E80_3E7F,// Popw_FS
	0x0A80_3E7F,// Popd_FS
	0x1480_3E70,// Popq_FS
	0x1E62_3E7F,// Cpuid
	0x1E00_3FFF,// Bt_rm16_r16
	0x1E00_3FFF,// Bt_rm32_r32
	0x1400_3FF0,// Bt_rm64_r64
	0x1E00_3FFF,// Shld_rm16_r16_imm8
	0x1E00_3FFF,// Shld_rm32_r32_imm8
	0x1400_3FF0,// Shld_rm64_r64_imm8
	0x1E00_3FFF,// Shld_rm16_r16_CL
	0x1E00_3FFF,// Shld_rm32_r32_CL
	0x1400_3FF0,// Shld_rm64_r64_CL
	0x0A00_3FFF,// Montmul_16
	0x1E00_3FFF,// Montmul_32
	0x1400_3FF0,// Montmul_64
	0x0A00_3FFF,// Xsha1_16
	0x1E00_3FFF,// Xsha1_32
	0x1400_3FF0,// Xsha1_64
	0x0A00_3FFF,// Xsha256_16
	0x1E00_3FFF,// Xsha256_32
	0x1400_3FF0,// Xsha256_64
	0x0A00_3FFF,// Xbts_r16_rm16
	0x0A00_3FFF,// Xbts_r32_rm32
	0x0A00_3FFF,// Xstore_16
	0x1E00_3FFF,// Xstore_32
	0x1400_3FF0,// Xstore_64
	0x0A00_3FFF,// Xcryptecb_16
	0x1E00_3FFF,// Xcryptecb_32
	0x1400_3FF0,// Xcryptecb_64
	0x0A00_3FFF,// Xcryptcbc_16
	0x1E00_3FFF,// Xcryptcbc_32
	0x1400_3FF0,// Xcryptcbc_64
	0x0A00_3FFF,// Xcryptctr_16
	0x1E00_3FFF,// Xcryptctr_32
	0x1400_3FF0,// Xcryptctr_64
	0x0A00_3FFF,// Xcryptcfb_16
	0x1E00_3FFF,// Xcryptcfb_32
	0x1400_3FF0,// Xcryptcfb_64
	0x0A00_3FFF,// Xcryptofb_16
	0x1E00_3FFF,// Xcryptofb_32
	0x1400_3FF0,// Xcryptofb_64
	0x0A00_3FFF,// Ibts_rm16_r16
	0x0A00_3FFF,// Ibts_rm32_r32
	0x0A00_3FFF,// Cmpxchg486_rm8_r8
	0x0A00_3FFF,// Cmpxchg486_rm16_r16
	0x0A00_3FFF,// Cmpxchg486_rm32_r32
	0x1E00_3FFF,// Pushw_GS
	0x0A00_3FFF,// Pushd_GS
	0x1400_3FF0,// Pushq_GS
	0x1E80_3E7F,// Popw_GS
	0x0A80_3E7F,// Popd_GS
	0x1480_3E70,// Popq_GS
	0x1EA2_3A6F,// Rsm
	0x1E00_3FFF,// Bts_rm16_r16
	0x1E00_3FFF,// Bts_rm32_r32
	0x1400_3FF0,// Bts_rm64_r64
	0x1E00_3FFF,// Shrd_rm16_r16_imm8
	0x1E00_3FFF,// Shrd_rm32_r32_imm8
	0x1400_3FF0,// Shrd_rm64_r64_imm8
	0x1E00_3FFF,// Shrd_rm16_r16_CL
	0x1E00_3FFF,// Shrd_rm32_r32_CL
	0x1400_3FF0,// Shrd_rm64_r64_CL
	0x1E80_3FFF,// Fxsave_m512byte
	0x1480_3FF0,// Fxsave64_m512byte
	0x1400_3FF0,// Rdfsbase_r32
	0x1400_3FF0,// Rdfsbase_r64
	0x1E80_3FFF,// Fxrstor_m512byte
	0x1480_3FF0,// Fxrstor64_m512byte
	0x1400_3FF0,// Rdgsbase_r32
	0x1400_3FF0,// Rdgsbase_r64
	0x1E00_3FFF,// Ldmxcsr_m32
	0x1480_3FF0,// Wrfsbase_r32
	0x1480_3FF0,// Wrfsbase_r64
	0x1E00_3FFA,// VEX_Vldmxcsr_m32
	0x1E00_3FFF,// Stmxcsr_m32
	0x1480_3FF0,// Wrgsbase_r32
	0x1480_3FF0,// Wrgsbase_r64
	0x1E00_3FFA,// VEX_Vstmxcsr_m32
	0x1E80_3FFF,// Xsave_mem
	0x1480_3FF0,// Xsave64_mem
	0x1E00_3FFF,// Ptwrite_rm32
	0x1400_3FF0,// Ptwrite_rm64
	0x1E80_3FFF,// Xrstor_mem
	0x1480_3FF0,// Xrstor64_mem
	0x1E00_3FFA,// Incsspd_r32
	0x1400_3FF0,// Incsspq_r64
	0x1E80_3FFF,// Xsaveopt_mem
	0x1480_3FF0,// Xsaveopt64_mem
	0x1E80_3FFF,// Clwb_m8
	0x1E84_3FFF,// Tpause_r32
	0x1484_3FF0,// Tpause_r64
	0x1E00_3E7A,// Clrssbsy_m64
	0x0A40_3FFF,// Umonitor_r16
	0x1E40_3FFF,// Umonitor_r32
	0x1440_3FF0,// Umonitor_r64
	0x1E44_3FFF,// Umwait_r32
	0x1444_3FF0,// Umwait_r64
	0x1E80_3FFF,// Clflush_m8
	0x1E80_3FFF,// Clflushopt_m8
	0x1E00_3FFF,// Lfence
	0x1E00_3FFF,// Lfence_E9
	0x1E00_3FFF,// Lfence_EA
	0x1E00_3FFF,// Lfence_EB
	0x1E00_3FFF,// Lfence_EC
	0x1E00_3FFF,// Lfence_ED
	0x1E00_3FFF,// Lfence_EE
	0x1E00_3FFF,// Lfence_EF
	0x1E00_3FFF,// Mfence
	0x1E00_3FFF,// Mfence_F1
	0x1E00_3FFF,// Mfence_F2
	0x1E00_3FFF,// Mfence_F3
	0x1E00_3FFF,// Mfence_F4
	0x1E00_3FFF,// Mfence_F5
	0x1E00_3FFF,// Mfence_F6
	0x1E00_3FFF,// Mfence_F7
	0x1E00_3FFF,// Sfence
	0x1E00_3FFF,// Sfence_F9
	0x1E00_3FFF,// Sfence_FA
	0x1E00_3FFF,// Sfence_FB
	0x1E00_3FFF,// Sfence_FC
	0x1E00_3FFF,// Sfence_FD
	0x1E00_3FFF,// Sfence_FE
	0x1E00_3FFF,// Sfence_FF
	0x1E84_3E7F,// Pcommit
	0x1E00_3FFF,// Imul_r16_rm16
	0x1E00_3FFF,// Imul_r32_rm32
	0x1400_3FF0,// Imul_r64_rm64
	0x1E00_3FFF,// Cmpxchg_rm8_r8
	0x1E00_3FFF,// Cmpxchg_rm16_r16
	0x1E00_3FFF,// Cmpxchg_rm32_r32
	0x1400_3FF0,// Cmpxchg_rm64_r64
	0x1E80_3E7F,// Lss_r16_m1616
	0x1E80_3E7F,// Lss_r32_m1632
	0x0480_3E70,// Lss_r64_m1664
	0x1E00_3FFF,// Btr_rm16_r16
	0x1E00_3FFF,// Btr_rm32_r32
	0x1400_3FF0,// Btr_rm64_r64
	0x1E80_3E7F,// Lfs_r16_m1616
	0x1E80_3E7F,// Lfs_r32_m1632
	0x0480_3E70,// Lfs_r64_m1664
	0x1E80_3E7F,// Lgs_r16_m1616
	0x1E80_3E7F,// Lgs_r32_m1632
	0x0480_3E70,// Lgs_r64_m1664
	0x1E00_3FFF,// Movzx_r16_rm8
	0x1E00_3FFF,// Movzx_r32_rm8
	0x1400_3FF0,// Movzx_r64_rm8
	0x1E00_3FFF,// Movzx_r16_rm16
	0x1E00_3FFF,// Movzx_r32_rm16
	0x1400_3FF0,// Movzx_r64_rm16
	0x0A00_3FFF,// Jmpe_disp16
	0x0A00_3FFF,// Jmpe_disp32
	0x1E00_3FFF,// Popcnt_r16_rm16
	0x1E00_3FFF,// Popcnt_r32_rm32
	0x1400_3FF0,// Popcnt_r64_rm64
	0x1E82_3FFF,// Ud1_r16_rm16
	0x1E82_3FFF,// Ud1_r32_rm32
	0x1482_3FF0,// Ud1_r64_rm64
	0x1E00_3FFF,// Bt_rm16_imm8
	0x1E00_3FFF,// Bt_rm32_imm8
	0x1400_3FF0,// Bt_rm64_imm8
	0x1E00_3FFF,// Bts_rm16_imm8
	0x1E00_3FFF,// Bts_rm32_imm8
	0x1400_3FF0,// Bts_rm64_imm8
	0x1E00_3FFF,// Btr_rm16_imm8
	0x1E00_3FFF,// Btr_rm32_imm8
	0x1400_3FF0,// Btr_rm64_imm8
	0x1E00_3FFF,// Btc_rm16_imm8
	0x1E00_3FFF,// Btc_rm32_imm8
	0x1400_3FF0,// Btc_rm64_imm8
	0x1E00_3FFF,// Btc_rm16_r16
	0x1E00_3FFF,// Btc_rm32_r32
	0x1400_3FF0,// Btc_rm64_r64
	0x1E00_3FFF,// Bsf_r16_rm16
	0x1E00_3FFF,// Bsf_r32_rm32
	0x1400_3FF0,// Bsf_r64_rm64
	0x1E00_3FFF,// Tzcnt_r16_rm16
	0x1E00_3FFF,// Tzcnt_r32_rm32
	0x1400_3FF0,// Tzcnt_r64_rm64
	0x1E00_3FFF,// Bsr_r16_rm16
	0x1E00_3FFF,// Bsr_r32_rm32
	0x1400_3FF0,// Bsr_r64_rm64
	0x1E00_3FFF,// Lzcnt_r16_rm16
	0x1E00_3FFF,// Lzcnt_r32_rm32
	0x1400_3FF0,// Lzcnt_r64_rm64
	0x1E00_3FFF,// Movsx_r16_rm8
	0x1E00_3FFF,// Movsx_r32_rm8
	0x1400_3FF0,// Movsx_r64_rm8
	0x1E00_3FFF,// Movsx_r16_rm16
	0x1E00_3FFF,// Movsx_r32_rm16
	0x1400_3FF0,// Movsx_r64_rm16
	0x1E00_3FFF,// Xadd_rm8_r8
	0x1E00_3FFF,// Xadd_rm16_r16
	0x1E00_3FFF,// Xadd_rm32_r32
	0x1400_3FF0,// Xadd_rm64_r64
	0x1E00_3FFF,// Cmpps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	0x1E00_3FFF,// Cmppd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	0x1E00_3FFF,// Cmpss_xmm_xmmm32_imm8
	0x1E00_3FFA,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	0x1E00_3FFA,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	0x1E00_3FFF,// Cmpsd_xmm_xmmm64_imm8
	0x1E00_3FFA,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	0x1E00_3FFA,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	0x1E80_3FFF,// Movnti_m32_r32
	0x1480_3FF0,// Movnti_m64_r64
	0x7E80_3FFF,// Pinsrw_mm_r32m16_imm8
	0x7480_3FF0,// Pinsrw_mm_r64m16_imm8
	0x7E00_3FFF,// Pinsrw_xmm_r32m16_imm8
	0x7400_3FF0,// Pinsrw_xmm_r64m16_imm8
	0x1E00_3FFA,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x1400_3FF0,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x1E00_3FFA,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	0x1400_3FF0,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	0x7E80_3FFF,// Pextrw_r32_mm_imm8
	0x7480_3FF0,// Pextrw_r64_mm_imm8
	0x7E00_3FFF,// Pextrw_r32_xmm_imm8
	0x7400_3FF0,// Pextrw_r64_xmm_imm8
	0x1E00_3FFA,// VEX_Vpextrw_r32_xmm_imm8
	0x1400_3FF0,// VEX_Vpextrw_r64_xmm_imm8
	0x1E00_3FFA,// EVEX_Vpextrw_r32_xmm_imm8
	0x1400_3FF0,// EVEX_Vpextrw_r64_xmm_imm8
	0x1E00_3FFF,// Shufps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFF,// Shufpd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFF,// Cmpxchg8b_m64
	0x1400_3FF0,// Cmpxchg16b_m128
	0x1E84_3E7B,// Xrstors_mem
	0x1484_3E70,// Xrstors64_mem
	0x1E80_3FFF,// Xsavec_mem
	0x1480_3FF0,// Xsavec64_mem
	0x1E84_3E7B,// Xsaves_mem
	0x1484_3E70,// Xsaves64_mem
	0x1E82_3C72,// Vmptrld_m64
	0x1E82_3C72,// Vmclear_m64
	0x1E82_3E72,// Vmxon_m64
	0x1E84_3FFF,// Rdrand_r16
	0x1E84_3FFF,// Rdrand_r32
	0x1484_3FF0,// Rdrand_r64
	0x1E82_3C72,// Vmptrst_m64
	0x1E84_3FFF,// Rdseed_r16
	0x1E84_3FFF,// Rdseed_r32
	0x1484_3FF0,// Rdseed_r64
	0x0A00_3FFF,// Rdpid_r32
	0x1400_3FF0,// Rdpid_r64
	0x1E00_3FFF,// Bswap_r16
	0x1E00_3FFF,// Bswap_r32
	0x1400_3FF0,// Bswap_r64
	0x1E00_3FFF,// Addsubpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	0x1E00_3FFF,// Addsubps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddsubps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaddsubps_ymm_ymm_ymmm256
	0x7E80_3FFF,// Psrlw_mm_mmm64
	0x1E00_3FFF,// Psrlw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlw_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Psrld_mm_mmm64
	0x1E00_3FFF,// Psrld_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrld_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrld_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Psrlq_mm_mmm64
	0x1E00_3FFF,// Psrlq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlq_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Paddq_mm_mmm64
	0x1E00_3FFF,// Paddq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Pmullw_mm_mmm64
	0x1E00_3FFF,// Pmullw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmullw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmullw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	0x9E00_3FFF,// Movq_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vmovq_xmmm64_xmm
	0x1E00_3FFA,// EVEX_Vmovq_xmmm64_xmm
	0x7E80_3FFF,// Movq2dq_xmm_mm
	0x7E80_3FFF,// Movdq2q_mm_xmm
	0x7E80_3FFF,// Pmovmskb_r32_mm
	0x7480_3FF0,// Pmovmskb_r64_mm
	0x7E00_3FFF,// Pmovmskb_r32_xmm
	0x7400_3FF0,// Pmovmskb_r64_xmm
	0x1E00_3FFA,// VEX_Vpmovmskb_r32_xmm
	0x1400_3FF0,// VEX_Vpmovmskb_r64_xmm
	0x1E00_3FFA,// VEX_Vpmovmskb_r32_ymm
	0x1400_3FF0,// VEX_Vpmovmskb_r64_ymm
	0x7E80_3FFF,// Psubusb_mm_mmm64
	0x1E00_3FFF,// Psubusb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubusb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubusb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Psubusw_mm_mmm64
	0x1E00_3FFF,// Psubusw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubusw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubusw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Pminub_mm_mmm64
	0x1E00_3FFF,// Pminub_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminub_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminub_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Pand_mm_mmm64
	0x1E00_3FFF,// Pand_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpand_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpand_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Paddusb_mm_mmm64
	0x1E00_3FFF,// Paddusb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddusb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddusb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Paddusw_mm_mmm64
	0x1E00_3FFF,// Paddusw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddusw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddusw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Pmaxub_mm_mmm64
	0x1E00_3FFF,// Pmaxub_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxub_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxub_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Pandn_mm_mmm64
	0x1E00_3FFF,// Pandn_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpandn_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpandn_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	0x1E80_3FFF,// Pavgb_mm_mmm64
	0x1E00_3FFF,// Pavgb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpavgb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpavgb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Psraw_mm_mmm64
	0x1E00_3FFF,// Psraw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsraw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsraw_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Psrad_mm_mmm64
	0x1E00_3FFF,// Psrad_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrad_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrad_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	0x1E80_3FFF,// Pavgw_mm_mmm64
	0x1E00_3FFF,// Pavgw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpavgw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpavgw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Pmulhuw_mm_mmm64
	0x1E00_3FFF,// Pmulhuw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Pmulhw_mm_mmm64
	0x1E00_3FFF,// Pmulhw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFF,// Cvttpd2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvttpd2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvttpd2dq_xmm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	0x1E00_3FFF,// Cvtdq2pd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtdq2pd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtdq2pd_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	0x1E00_3FFA,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	0x1E00_3FFA,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	0x1E00_3FFF,// Cvtpd2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtpd2dq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtpd2dq_xmm_ymmm256
	0x1E00_3FFA,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	0x7E80_3FFF,// Movntq_m64_mm
	0x1E80_3FFF,// Movntdq_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntdq_m128_xmm
	0x1E80_3FFA,// VEX_Vmovntdq_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntdq_m128_xmm
	0x1E80_3FFA,// EVEX_Vmovntdq_m256_ymm
	0x1E80_3FFA,// EVEX_Vmovntdq_m512_zmm
	0x7E80_3FFF,// Psubsb_mm_mmm64
	0x1E00_3FFF,// Psubsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubsb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubsb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Psubsw_mm_mmm64
	0x1E00_3FFF,// Psubsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Pminsw_mm_mmm64
	0x1E00_3FFF,// Pminsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Por_mm_mmm64
	0x1E00_3FFF,// Por_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpor_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpor_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Paddsb_mm_mmm64
	0x1E00_3FFF,// Paddsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddsb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddsb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Paddsw_mm_mmm64
	0x1E00_3FFF,// Paddsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Pmaxsw_mm_mmm64
	0x1E00_3FFF,// Pmaxsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Pxor_mm_mmm64
	0x1E00_3FFF,// Pxor_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpxor_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpxor_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Lddqu_xmm_m128
	0x1E00_3FFA,// VEX_Vlddqu_xmm_m128
	0x1E00_3FFA,// VEX_Vlddqu_ymm_m256
	0x7E80_3FFF,// Psllw_mm_mmm64
	0x1E00_3FFF,// Psllw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllw_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Pslld_mm_mmm64
	0x1E00_3FFF,// Pslld_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpslld_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpslld_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	0x7E80_3FFF,// Psllq_mm_mmm64
	0x1E00_3FFF,// Psllq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllq_ymm_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	0x1E80_3FFF,// Pmuludq_mm_mmm64
	0x1E00_3FFF,// Pmuludq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmuludq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmuludq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Pmaddwd_mm_mmm64
	0x1E00_3FFF,// Pmaddwd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Psadbw_mm_mmm64
	0x1E00_3FFF,// Psadbw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsadbw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsadbw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	0x1E80_3FFF,// Maskmovq_rDI_mm_mm
	0x1E80_3FFF,// Maskmovdqu_rDI_xmm_xmm
	0x1E80_3FFA,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	0x7E80_3FFF,// Psubb_mm_mmm64
	0x1E00_3FFF,// Psubb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Psubw_mm_mmm64
	0x1E00_3FFF,// Psubw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Psubd_mm_mmm64
	0x1E00_3FFF,// Psubd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	0x1E80_3FFF,// Psubq_mm_mmm64
	0x1E00_3FFF,// Psubq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsubq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	0x7E80_3FFF,// Paddb_mm_mmm64
	0x1E00_3FFF,// Paddb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Paddw_mm_mmm64
	0x1E00_3FFF,// Paddw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	0x7E80_3FFF,// Paddd_mm_mmm64
	0x1E00_3FFF,// Paddd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpaddd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	0x0682_3FFF,// Ud0_r16_rm16
	0x0682_3FFF,// Ud0_r32_rm32
	0x0482_3FF0,// Ud0_r64_rm64
	0x1E80_3FFF,// Pshufb_mm_mmm64
	0x1E00_3FFF,// Pshufb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpshufb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpshufb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Phaddw_mm_mmm64
	0x1E00_3FFF,// Phaddw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddw_ymm_ymm_ymmm256
	0x1E80_3FFF,// Phaddd_mm_mmm64
	0x1E00_3FFF,// Phaddd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddd_ymm_ymm_ymmm256
	0x1E80_3FFF,// Phaddsw_mm_mmm64
	0x1E00_3FFF,// Phaddsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphaddsw_ymm_ymm_ymmm256
	0x1E80_3FFF,// Pmaddubsw_mm_mmm64
	0x1E00_3FFF,// Pmaddubsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	0x1E80_3FFF,// Phsubw_mm_mmm64
	0x1E00_3FFF,// Phsubw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubw_ymm_ymm_ymmm256
	0x1E80_3FFF,// Phsubd_mm_mmm64
	0x1E00_3FFF,// Phsubd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubd_ymm_ymm_ymmm256
	0x1E80_3FFF,// Phsubsw_mm_mmm64
	0x1E00_3FFF,// Phsubsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphsubsw_ymm_ymm_ymmm256
	0x1E80_3FFF,// Psignb_mm_mmm64
	0x1E00_3FFF,// Psignb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignb_ymm_ymm_ymmm256
	0x1E80_3FFF,// Psignw_mm_mmm64
	0x1E00_3FFF,// Psignw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignw_ymm_ymm_ymmm256
	0x1E80_3FFF,// Psignd_mm_mmm64
	0x1E00_3FFF,// Psignd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsignd_ymm_ymm_ymmm256
	0x1E80_3FFF,// Pmulhrsw_mm_mmm64
	0x1E00_3FFF,// Pmulhrsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// VEX_Vpermilps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpermilps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// VEX_Vpermilpd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpermilpd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// VEX_Vtestps_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vtestps_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vtestpd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vtestpd_ymm_ymmm256
	0x1E00_3FFF,// Pblendvb_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	0x1E00_3FFA,// VEX_Vcvtph2ps_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vcvtph2ps_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	0x1E00_3FFA,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	0x1E00_3FFF,// Blendvps_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	0x1E00_3FFF,// Blendvpd_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	0x1E00_3FFA,// VEX_Vpermps_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Ptest_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vptest_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vptest_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vbroadcastss_xmm_m32
	0x1E00_3FFA,// VEX_Vbroadcastss_ymm_m32
	0x1E00_3FFA,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	0x1E00_3FFA,// VEX_Vbroadcastsd_ymm_m64
	0x1E00_3FFA,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	0x1E00_3FFA,// VEX_Vbroadcastf128_ymm_m128
	0x1E00_3FFA,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	0x1E00_3FFA,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	0x1E80_3FFF,// Pabsb_mm_mmm64
	0x1E00_3FFF,// Pabsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsb_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpabsb_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpabsb_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpabsb_zmm_k1z_zmmm512
	0x1E80_3FFF,// Pabsw_mm_mmm64
	0x1E00_3FFF,// Pabsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsw_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpabsw_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpabsw_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpabsw_zmm_k1z_zmmm512
	0x1E80_3FFF,// Pabsd_mm_mmm64
	0x1E00_3FFF,// Pabsd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpabsd_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	0x1E00_3FFF,// Pmovsxbw_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxbw_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxbw_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	0x1E00_3FFF,// Pmovsxbd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovsxbd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovsxbd_ymm_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	0x1E00_3FFF,// Pmovsxbq_xmm_xmmm16
	0x1E00_3FFA,// VEX_Vpmovsxbq_xmm_xmmm16
	0x1E00_3FFA,// VEX_Vpmovsxbq_ymm_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	0x1E00_3FFA,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	0x1E00_3FFF,// Pmovsxwd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxwd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxwd_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	0x1E00_3FFF,// Pmovsxwq_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovsxwq_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovsxwq_ymm_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	0x1E00_3FFF,// Pmovsxdq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxdq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovsxdq_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	0x3E00_3FFA,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	0x3E00_3FFA,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	0x3E00_3FFA,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	0x3E00_3FFA,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	0x3E00_3FFA,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	0x3E00_3FFA,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	0x3E00_3FFA,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	0x3E00_3FFA,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	0x3E00_3FFA,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	0x3E00_3FFA,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	0x3E00_3FFA,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	0x3E00_3FFA,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	0x3E00_3FFA,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	0x3E00_3FFA,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	0x3E00_3FFA,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	0x3E00_3FFA,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	0x3E00_3FFA,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	0x3E00_3FFA,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	0x3E00_3FFA,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	0x3E00_3FFA,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	0x3E00_3FFA,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	0x3E00_3FFA,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	0x3E00_3FFA,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	0x3E00_3FFA,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	0x1E00_3FFF,// Pmuldq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmuldq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmuldq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmovm2b_xmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2b_ymm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2b_zmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2w_xmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2w_ymm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2w_zmm_kr
	0x1E00_3FFF,// Pcmpeqq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmovb2m_kr_xmm
	0x1E00_3FFA,// EVEX_Vpmovb2m_kr_ymm
	0x1E00_3FFA,// EVEX_Vpmovb2m_kr_zmm
	0x1E00_3FFA,// EVEX_Vpmovw2m_kr_xmm
	0x1E00_3FFA,// EVEX_Vpmovw2m_kr_ymm
	0x1E00_3FFA,// EVEX_Vpmovw2m_kr_zmm
	0x1E80_3FFF,// Movntdqa_xmm_m128
	0x1E80_3FFA,// VEX_Vmovntdqa_xmm_m128
	0x1E80_3FFA,// VEX_Vmovntdqa_ymm_m256
	0x1E80_3FFA,// EVEX_Vmovntdqa_xmm_m128
	0x1E80_3FFA,// EVEX_Vmovntdqa_ymm_m256
	0x1E80_3FFA,// EVEX_Vmovntdqa_zmm_m512
	0x1E00_3FFA,// EVEX_Vpbroadcastmb2q_xmm_kr
	0x1E00_3FFA,// EVEX_Vpbroadcastmb2q_ymm_kr
	0x1E00_3FFA,// EVEX_Vpbroadcastmb2q_zmm_kr
	0x1E00_3FFF,// Packusdw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackusdw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpackusdw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// VEX_Vmaskmovps_xmm_xmm_m128
	0x1E00_3FFA,// VEX_Vmaskmovps_ymm_ymm_m256
	0x1E00_3FFA,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vmaskmovpd_xmm_xmm_m128
	0x1E00_3FFA,// VEX_Vmaskmovpd_ymm_ymm_m256
	0x1E00_3FFA,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vmaskmovps_m128_xmm_xmm
	0x1E00_3FFA,// VEX_Vmaskmovps_m256_ymm_ymm
	0x1E00_3FFA,// VEX_Vmaskmovpd_m128_xmm_xmm
	0x1E00_3FFA,// VEX_Vmaskmovpd_m256_ymm_ymm
	0x1E00_3FFF,// Pmovzxbw_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxbw_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxbw_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	0x1E00_3FFF,// Pmovzxbd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovzxbd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovzxbd_ymm_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	0x1E00_3FFF,// Pmovzxbq_xmm_xmmm16
	0x1E00_3FFA,// VEX_Vpmovzxbq_xmm_xmmm16
	0x1E00_3FFA,// VEX_Vpmovzxbq_ymm_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	0x1E00_3FFA,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	0x1E00_3FFF,// Pmovzxwd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxwd_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxwd_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	0x1E00_3FFF,// Pmovzxwq_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovzxwq_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpmovzxwq_ymm_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	0x1E00_3FFF,// Pmovzxdq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxdq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpmovzxdq_ymm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	0x1E00_3FFA,// VEX_Vpermd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Pcmpgtq_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	0x1E00_3FFF,// Pminsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpmovm2d_xmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2d_ymm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2d_zmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2q_xmm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2q_ymm_kr
	0x1E00_3FFA,// EVEX_Vpmovm2q_zmm_kr
	0x1E00_3FFF,// Pminsd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminsd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmovd2m_kr_xmm
	0x1E00_3FFA,// EVEX_Vpmovd2m_kr_ymm
	0x1E00_3FFA,// EVEX_Vpmovd2m_kr_zmm
	0x1E00_3FFA,// EVEX_Vpmovq2m_kr_xmm
	0x1E00_3FFA,// EVEX_Vpmovq2m_kr_ymm
	0x1E00_3FFA,// EVEX_Vpmovq2m_kr_zmm
	0x1E00_3FFF,// Pminuw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminuw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminuw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpbroadcastmw2d_xmm_kr
	0x1E00_3FFA,// EVEX_Vpbroadcastmw2d_ymm_kr
	0x1E00_3FFA,// EVEX_Vpbroadcastmw2d_zmm_kr
	0x1E00_3FFF,// Pminud_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpminud_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Pmaxsb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	0x1E00_3FFF,// Pmaxsd_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Pmaxuw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFF,// Pmaxud_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmaxud_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Pmulld_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulld_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmulld_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFF,// Phminposuw_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vphminposuw_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFA,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFA,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFA,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	0x1E00_3FFA,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	0x1E00_3FFA,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// VEX_Vpsravd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsravd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// VEX_Vpsllvd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllvd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpsllvq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpsllvq_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	0x1E00_3FFA,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	0x1E00_3FFA,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	0x1E00_3FFA,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	0x1E00_3FFA,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	0x1E00_3FFA,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	0x1E00_3FFA,// VEX_Vpbroadcastd_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vpbroadcastd_ymm_xmmm32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	0x1E00_3FFA,// VEX_Vpbroadcastq_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vpbroadcastq_ymm_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	0x1E00_3FFA,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	0x1E00_3FFA,// VEX_Vbroadcasti128_ymm_m128
	0x1E00_3FFA,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	0x1E00_3FFA,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	0x1E00_3FFA,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	0x1E00_3FFA,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// VEX_Vpbroadcastb_xmm_xmmm8
	0x1E00_3FFA,// VEX_Vpbroadcastb_ymm_xmmm8
	0x1E00_3FFA,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	0x1E00_3FFA,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	0x1E00_3FFA,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	0x1E00_3FFA,// VEX_Vpbroadcastw_xmm_xmmm16
	0x1E00_3FFA,// VEX_Vpbroadcastw_ymm_xmmm16
	0x1E00_3FFA,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	0x1E00_3FFA,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	0x1E00_3FFA,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	0x1E00_3FFA,// EVEX_Vpbroadcastb_xmm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastb_ymm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastb_zmm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastw_xmm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastw_ymm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastw_zmm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_xmm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_ymm_k1z_r32
	0x1E00_3FFA,// EVEX_Vpbroadcastd_zmm_k1z_r32
	0x1400_3FF0,// EVEX_Vpbroadcastq_xmm_k1z_r64
	0x1400_3FF0,// EVEX_Vpbroadcastq_ymm_k1z_r64
	0x1400_3FF0,// EVEX_Vpbroadcastq_zmm_k1z_r64
	0x1E00_3FFA,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	0x0A82_3C72,// Invept_r32_m128
	0x1482_3C70,// Invept_r64_m128
	0x0A82_3C72,// Invvpid_r32_m128
	0x1482_3C70,// Invvpid_r64_m128
	0x0AA4_3E7B,// Invpcid_r32_m128
	0x14A4_3E70,// Invpcid_r64_m128
	0x1E00_3FFA,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vexpandps_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vexpandps_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vexpandps_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	0x1E00_3FFA,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	0x1E00_3FFA,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	0x1E00_3FFA,// EVEX_Vcompressps_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vcompressps_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vcompressps_zmmm512_k1z_zmm
	0x1E00_3FFA,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	0x1E00_3FFA,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	0x1E00_3FFA,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	0x1E00_3FFA,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	0x1E00_3FFA,// VEX_Vpmaskmovd_xmm_xmm_m128
	0x1E00_3FFA,// VEX_Vpmaskmovd_ymm_ymm_m256
	0x1E00_3FFA,// VEX_Vpmaskmovq_xmm_xmm_m128
	0x1E00_3FFA,// VEX_Vpmaskmovq_ymm_ymm_m256
	0x1E00_3FFA,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	0x1E00_3FFA,// VEX_Vpmaskmovd_m128_xmm_xmm
	0x1E00_3FFA,// VEX_Vpmaskmovd_m256_ymm_ymm
	0x1E00_3FFA,// VEX_Vpmaskmovq_m128_xmm_xmm
	0x1E00_3FFA,// VEX_Vpmaskmovq_m256_ymm_ymm
	0x1E00_3FFA,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	0x1E00_3FFA,// VEX_Vpgatherdd_xmm_vm32x_xmm
	0x1E00_3FFA,// VEX_Vpgatherdd_ymm_vm32y_ymm
	0x1E00_3FFA,// VEX_Vpgatherdq_xmm_vm32x_xmm
	0x1E00_3FFA,// VEX_Vpgatherdq_ymm_vm32x_ymm
	0x1E00_3FFA,// EVEX_Vpgatherdd_xmm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vpgatherdd_ymm_k1_vm32y
	0x1E00_3FFA,// EVEX_Vpgatherdd_zmm_k1_vm32z
	0x1E00_3FFA,// EVEX_Vpgatherdq_xmm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vpgatherdq_ymm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vpgatherdq_zmm_k1_vm32y
	0x1E00_3FFA,// VEX_Vpgatherqd_xmm_vm64x_xmm
	0x1E00_3FFA,// VEX_Vpgatherqd_xmm_vm64y_xmm
	0x1E00_3FFA,// VEX_Vpgatherqq_xmm_vm64x_xmm
	0x1E00_3FFA,// VEX_Vpgatherqq_ymm_vm64y_ymm
	0x1E00_3FFA,// EVEX_Vpgatherqd_xmm_k1_vm64x
	0x1E00_3FFA,// EVEX_Vpgatherqd_xmm_k1_vm64y
	0x1E00_3FFA,// EVEX_Vpgatherqd_ymm_k1_vm64z
	0x1E00_3FFA,// EVEX_Vpgatherqq_xmm_k1_vm64x
	0x1E00_3FFA,// EVEX_Vpgatherqq_ymm_k1_vm64y
	0x1E00_3FFA,// EVEX_Vpgatherqq_zmm_k1_vm64z
	0x1E00_3FFA,// VEX_Vgatherdps_xmm_vm32x_xmm
	0x1E00_3FFA,// VEX_Vgatherdps_ymm_vm32y_ymm
	0x1E00_3FFA,// VEX_Vgatherdpd_xmm_vm32x_xmm
	0x1E00_3FFA,// VEX_Vgatherdpd_ymm_vm32x_ymm
	0x1E00_3FFA,// EVEX_Vgatherdps_xmm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vgatherdps_ymm_k1_vm32y
	0x1E00_3FFA,// EVEX_Vgatherdps_zmm_k1_vm32z
	0x1E00_3FFA,// EVEX_Vgatherdpd_xmm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vgatherdpd_ymm_k1_vm32x
	0x1E00_3FFA,// EVEX_Vgatherdpd_zmm_k1_vm32y
	0x1E00_3FFA,// VEX_Vgatherqps_xmm_vm64x_xmm
	0x1E00_3FFA,// VEX_Vgatherqps_xmm_vm64y_xmm
	0x1E00_3FFA,// VEX_Vgatherqpd_xmm_vm64x_xmm
	0x1E00_3FFA,// VEX_Vgatherqpd_ymm_vm64y_ymm
	0x1E00_3FFA,// EVEX_Vgatherqps_xmm_k1_vm64x
	0x1E00_3FFA,// EVEX_Vgatherqps_xmm_k1_vm64y
	0x1E00_3FFA,// EVEX_Vgatherqps_ymm_k1_vm64z
	0x1E00_3FFA,// EVEX_Vgatherqpd_xmm_k1_vm64x
	0x1E00_3FFA,// EVEX_Vgatherqpd_ymm_k1_vm64y
	0x1E00_3FFA,// EVEX_Vgatherqpd_zmm_k1_vm64z
	0x1E00_3FFA,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	0x1E00_3FFA,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	0x1E00_3FFA,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_Vpscatterdd_vm32x_k1_xmm
	0x1E00_3FFA,// EVEX_Vpscatterdd_vm32y_k1_ymm
	0x1E00_3FFA,// EVEX_Vpscatterdd_vm32z_k1_zmm
	0x1E00_3FFA,// EVEX_Vpscatterdq_vm32x_k1_xmm
	0x1E00_3FFA,// EVEX_Vpscatterdq_vm32x_k1_ymm
	0x1E00_3FFA,// EVEX_Vpscatterdq_vm32y_k1_zmm
	0x1E00_3FFA,// EVEX_Vpscatterqd_vm64x_k1_xmm
	0x1E00_3FFA,// EVEX_Vpscatterqd_vm64y_k1_xmm
	0x1E00_3FFA,// EVEX_Vpscatterqd_vm64z_k1_ymm
	0x1E00_3FFA,// EVEX_Vpscatterqq_vm64x_k1_xmm
	0x1E00_3FFA,// EVEX_Vpscatterqq_vm64y_k1_ymm
	0x1E00_3FFA,// EVEX_Vpscatterqq_vm64z_k1_zmm
	0x1E00_3FFA,// EVEX_Vscatterdps_vm32x_k1_xmm
	0x1E00_3FFA,// EVEX_Vscatterdps_vm32y_k1_ymm
	0x1E00_3FFA,// EVEX_Vscatterdps_vm32z_k1_zmm
	0x1E00_3FFA,// EVEX_Vscatterdpd_vm32x_k1_xmm
	0x1E00_3FFA,// EVEX_Vscatterdpd_vm32x_k1_ymm
	0x1E00_3FFA,// EVEX_Vscatterdpd_vm32y_k1_zmm
	0x1E00_3FFA,// EVEX_Vscatterqps_vm64x_k1_xmm
	0x1E00_3FFA,// EVEX_Vscatterqps_vm64y_k1_xmm
	0x1E00_3FFA,// EVEX_Vscatterqps_vm64z_k1_ymm
	0x1E00_3FFA,// EVEX_Vscatterqpd_vm64x_k1_xmm
	0x1E00_3FFA,// EVEX_Vscatterqpd_vm64y_k1_ymm
	0x1E00_3FFA,// EVEX_Vscatterqpd_vm64z_k1_zmm
	0x1E00_3FFA,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	0x1E00_3FFA,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	0x1E00_3FFA,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	0x1E00_3FFA,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	0x1E00_3FFA,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	0x1E00_3FFA,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	0x1E00_3FFA,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	0x1E00_3FFA,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	0x1E00_3FFA,// EVEX_Vgatherpf0dps_vm32z_k1
	0x1E00_3FFA,// EVEX_Vgatherpf0dpd_vm32y_k1
	0x1E00_3FFA,// EVEX_Vgatherpf1dps_vm32z_k1
	0x1E00_3FFA,// EVEX_Vgatherpf1dpd_vm32y_k1
	0x1E00_3FFA,// EVEX_Vscatterpf0dps_vm32z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf0dpd_vm32y_k1
	0x1E00_3FFA,// EVEX_Vscatterpf1dps_vm32z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf1dpd_vm32y_k1
	0x1E00_3FFA,// EVEX_Vgatherpf0qps_vm64z_k1
	0x1E00_3FFA,// EVEX_Vgatherpf0qpd_vm64z_k1
	0x1E00_3FFA,// EVEX_Vgatherpf1qps_vm64z_k1
	0x1E00_3FFA,// EVEX_Vgatherpf1qpd_vm64z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf0qps_vm64z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf0qpd_vm64z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf1qps_vm64z_k1
	0x1E00_3FFA,// EVEX_Vscatterpf1qpd_vm64z_k1
	0x1E00_3FFF,// Sha1nexte_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFA,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFF,// Sha1msg1_xmm_xmmm128
	0x1E00_3FFF,// Sha1msg2_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFA,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFF,// Sha256rnds2_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFA,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	0x1E00_3FFF,// Sha256msg1_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	0x1E00_3FFA,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	0x1E00_3FFF,// Sha256msg2_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	0x1E00_3FFA,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	0x1E00_3FFF,// Gf2p8mulb_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	0x1E00_3FFF,// Aesimc_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesimc_xmm_xmmm128
	0x1E00_3FFF,// Aesenc_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesenc_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesenc_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesenc_xmm_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vaesenc_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesenc_zmm_zmm_zmmm512
	0x1E00_3FFF,// Aesenclast_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesenclast_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesenclast_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	0x1E00_3FFF,// Aesdec_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesdec_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesdec_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesdec_xmm_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vaesdec_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesdec_zmm_zmm_zmmm512
	0x1E00_3FFF,// Aesdeclast_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	0x1E00_3FFA,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	0x1E00_3FFA,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	0x1E00_3FFF,// Movbe_r16_m16
	0x1E00_3FFF,// Movbe_r32_m32
	0x1400_3FF0,// Movbe_r64_m64
	0x1E00_3FFF,// Crc32_r32_rm8
	0x1400_3FF0,// Crc32_r64_rm8
	0x1E00_3FFF,// Movbe_m16_r16
	0x1E00_3FFF,// Movbe_m32_r32
	0x1400_3FF0,// Movbe_m64_r64
	0x1E00_3FFF,// Crc32_r32_rm16
	0x1E00_3FFF,// Crc32_r32_rm32
	0x1400_3FF0,// Crc32_r64_rm64
	0x1E00_3FFA,// VEX_Andn_r32_r32_rm32
	0x1400_3FF0,// VEX_Andn_r64_r64_rm64
	0x1E00_3FFA,// VEX_Blsr_r32_rm32
	0x1400_3FF0,// VEX_Blsr_r64_rm64
	0x1E00_3FFA,// VEX_Blsmsk_r32_rm32
	0x1400_3FF0,// VEX_Blsmsk_r64_rm64
	0x1E00_3FFA,// VEX_Blsi_r32_rm32
	0x1400_3FF0,// VEX_Blsi_r64_rm64
	0x1E00_3FFA,// VEX_Bzhi_r32_rm32_r32
	0x1400_3FF0,// VEX_Bzhi_r64_rm64_r64
	0x1E00_3E7A,// Wrussd_m32_r32
	0x1400_3E70,// Wrussq_m64_r64
	0x1E00_3FFA,// VEX_Pext_r32_r32_rm32
	0x1400_3FF0,// VEX_Pext_r64_r64_rm64
	0x1E00_3FFA,// VEX_Pdep_r32_r32_rm32
	0x1400_3FF0,// VEX_Pdep_r64_r64_rm64
	0x1E00_3FFA,// Wrssd_m32_r32
	0x1400_3FF0,// Wrssq_m64_r64
	0x1E00_3FFF,// Adcx_r32_rm32
	0x1400_3FF0,// Adcx_r64_rm64
	0x1E00_3FFF,// Adox_r32_rm32
	0x1400_3FF0,// Adox_r64_rm64
	0x1E00_3FFA,// VEX_Mulx_r32_r32_rm32
	0x1400_3FF0,// VEX_Mulx_r64_r64_rm64
	0x1E00_3FFA,// VEX_Bextr_r32_rm32_r32
	0x1400_3FF0,// VEX_Bextr_r64_rm64_r64
	0x1E00_3FFA,// VEX_Shlx_r32_rm32_r32
	0x1400_3FF0,// VEX_Shlx_r64_rm64_r64
	0x1E00_3FFA,// VEX_Sarx_r32_rm32_r32
	0x1400_3FF0,// VEX_Sarx_r64_rm64_r64
	0x1E00_3FFA,// VEX_Shrx_r32_rm32_r32
	0x1400_3FF0,// VEX_Shrx_r64_rm64_r64
	0x0A00_3FFF,// Movdir64b_r16_m512
	0x1E00_3FFF,// Movdir64b_r32_m512
	0x1400_3FF0,// Movdir64b_r64_m512
	0x0A84_3E7B,// Enqcmds_r16_m512
	0x1E84_3E7B,// Enqcmds_r32_m512
	0x1484_3E70,// Enqcmds_r64_m512
	0x0A84_3FFF,// Enqcmd_r16_m512
	0x1E84_3FFF,// Enqcmd_r32_m512
	0x1484_3FF0,// Enqcmd_r64_m512
	0x1E00_3FFF,// Movdiri_m32_r32
	0x1400_3FF0,// Movdiri_m64_r64
	0x1E00_3FFA,// VEX_Vpermq_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFA,// VEX_Vpermpd_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFA,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// VEX_Vpermilps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpermilps_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	0x1E00_3FFA,// VEX_Vpermilpd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpermilpd_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	0x1E00_3FFA,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	0x1E00_3FFF,// Roundps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vroundps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vroundps_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	0x1E00_3FFF,// Roundpd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vroundpd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vroundpd_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	0x1E00_3FFF,// Roundss_xmm_xmmm32_imm8
	0x1E00_3FFA,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	0x1E00_3FFA,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	0x1E00_3FFF,// Roundsd_xmm_xmmm64_imm8
	0x1E00_3FFA,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	0x1E00_3FFA,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x1E00_3FFF,// Blendps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	0x1E00_3FFF,// Blendpd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	0x1E00_3FFF,// Pblendw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	0x1E80_3FFF,// Palignr_mm_mmm64_imm8
	0x1E00_3FFF,// Palignr_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	0x5E00_3FFF,// Pextrb_r32m8_xmm_imm8
	0x5400_3FF0,// Pextrb_r64m8_xmm_imm8
	0x5E00_3FFA,// VEX_Vpextrb_r32m8_xmm_imm8
	0x5400_3FF0,// VEX_Vpextrb_r64m8_xmm_imm8
	0x5E00_3FFA,// EVEX_Vpextrb_r32m8_xmm_imm8
	0x5400_3FF0,// EVEX_Vpextrb_r64m8_xmm_imm8
	0x7E00_3FFF,// Pextrw_r32m16_xmm_imm8
	0x7400_3FF0,// Pextrw_r64m16_xmm_imm8
	0x5E00_3FFA,// VEX_Vpextrw_r32m16_xmm_imm8
	0x5400_3FF0,// VEX_Vpextrw_r64m16_xmm_imm8
	0x5E00_3FFA,// EVEX_Vpextrw_r32m16_xmm_imm8
	0x5400_3FF0,// EVEX_Vpextrw_r64m16_xmm_imm8
	0x5E00_3FFF,// Pextrd_rm32_xmm_imm8
	0x5400_3FF0,// Pextrq_rm64_xmm_imm8
	0x5E00_3FFA,// VEX_Vpextrd_rm32_xmm_imm8
	0x5400_3FF0,// VEX_Vpextrq_rm64_xmm_imm8
	0x5E00_3FFA,// EVEX_Vpextrd_rm32_xmm_imm8
	0x5400_3FF0,// EVEX_Vpextrq_rm64_xmm_imm8
	0x1E00_3FFF,// Extractps_rm32_xmm_imm8
	0x1400_3FF0,// Extractps_r64m32_xmm_imm8
	0x1E00_3FFA,// VEX_Vextractps_rm32_xmm_imm8
	0x1400_3FF0,// VEX_Vextractps_r64m32_xmm_imm8
	0x1E00_3FFA,// EVEX_Vextractps_rm32_xmm_imm8
	0x1400_3FF0,// EVEX_Vextractps_r64m32_xmm_imm8
	0x1E00_3FFA,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vextractf128_xmmm128_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	0x1E00_3FFA,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	0x1E00_3FFA,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	0x1E00_3FFA,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	0x1E00_3FFA,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	0x1E00_3FFA,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	0x1E00_3FFA,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	0x1E00_3FFF,// Pinsrb_xmm_r32m8_imm8
	0x1400_3FF0,// Pinsrb_xmm_r64m8_imm8
	0x1E00_3FFA,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x1400_3FF0,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x1E00_3FFA,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	0x1400_3FF0,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	0x1E00_3FFF,// Insertps_xmm_xmmm32_imm8
	0x1E00_3FFA,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x1E00_3FFA,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	0x1E00_3FFF,// Pinsrd_xmm_rm32_imm8
	0x1400_3FF0,// Pinsrq_xmm_rm64_imm8
	0x1E00_3FFA,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x1400_3FF0,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x1E00_3FFA,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	0x1400_3FF0,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	0x1E00_3FFA,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	0x1E00_3FFA,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	0x1E00_3FFA,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x1E00_3FFA,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x1E00_3FFA,// VEX_Kshiftrb_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftrw_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftrd_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftrq_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftlb_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftlw_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftld_kr_kr_imm8
	0x1E00_3FFA,// VEX_Kshiftlq_kr_kr_imm8
	0x1E00_3FFA,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vextracti128_xmmm128_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	0x1E00_3FFA,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	0x1E00_3FFA,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	0x1E00_3FFF,// Dpps_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	0x1E00_3FFF,// Dppd_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFF,// Mpsadbw_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFF,// Pclmulqdq_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	0x1E00_3FFA,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	0x1E00_3FFA,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	0x1E00_3FFA,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	0x1E00_3FFA,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	0x1E00_3FFA,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	0x1E00_3FFA,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	0x1E00_3FFA,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	0x1E00_3FFA,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	0x1E00_3FFA,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x1E00_3FFA,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x1E00_3FFA,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x1E00_3FFA,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x1E00_3FFA,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	0x1E00_3FFA,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	0x1E00_3FFA,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	0x1E00_3FFA,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x1E00_3FFA,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	0x1E00_3FFA,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	0x1E00_3FFA,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	0x1E00_3FFA,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	0x1E00_3FFA,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFF,// Pcmpestrm_xmm_xmmm128_imm8
	0x1400_3FF0,// Pcmpestrm64_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	0x1400_3FF0,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	0x1E00_3FFF,// Pcmpestri_xmm_xmmm128_imm8
	0x1400_3FF0,// Pcmpestri64_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	0x1400_3FF0,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	0x1E00_3FFF,// Pcmpistrm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	0x1E00_3FFF,// Pcmpistri_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	0x3E00_3FFA,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	0x3E00_3FFA,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	0x3E00_3FFA,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	0x3E00_3FFA,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	0x3E00_3FFA,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	0x3E00_3FFA,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	0x3E00_3FFA,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	0x3E00_3FFA,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	0x1E00_3FFA,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	0x1E00_3FFA,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	0x1E00_3FFA,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	0x1E00_3FFA,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	0x1E00_3FFA,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	0x1E00_3FFA,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	0x1E00_3FFA,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	0x1E00_3FFA,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	0x1E00_3FFA,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFA,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	0x1E00_3FFA,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	0x1E00_3FFA,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	0x1E00_3FFA,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	0x1E00_3FFA,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	0x1E00_3FFA,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	0x1E00_3FFF,// Sha1rnds4_xmm_xmmm128_imm8
	0x1E00_3FFF,// Gf2p8affineqb_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFF,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	0x1E00_3FFA,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	0x1E00_3FFF,// Aeskeygenassist_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	0x1E00_3FFA,// VEX_Rorx_r32_rm32_imm8
	0x1400_3FF0,// VEX_Rorx_r64_rm64_imm8
	0x1E00_3FFA,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	0x1E00_3FFA,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	0x1E00_3FFA,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vprotb_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vprotw_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vprotd_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vprotq_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	0x1E00_3FFA,// XOP_Blcfill_r32_rm32
	0x1400_3FF0,// XOP_Blcfill_r64_rm64
	0x1E00_3FFA,// XOP_Blsfill_r32_rm32
	0x1400_3FF0,// XOP_Blsfill_r64_rm64
	0x1E00_3FFA,// XOP_Blcs_r32_rm32
	0x1400_3FF0,// XOP_Blcs_r64_rm64
	0x1E00_3FFA,// XOP_Tzmsk_r32_rm32
	0x1400_3FF0,// XOP_Tzmsk_r64_rm64
	0x1E00_3FFA,// XOP_Blcic_r32_rm32
	0x1400_3FF0,// XOP_Blcic_r64_rm64
	0x1E00_3FFA,// XOP_Blsic_r32_rm32
	0x1400_3FF0,// XOP_Blsic_r64_rm64
	0x1E00_3FFA,// XOP_T1mskc_r32_rm32
	0x1400_3FF0,// XOP_T1mskc_r64_rm64
	0x1E00_3FFA,// XOP_Blcmsk_r32_rm32
	0x1400_3FF0,// XOP_Blcmsk_r64_rm64
	0x1E00_3FFA,// XOP_Blci_r32_rm32
	0x1400_3FF0,// XOP_Blci_r64_rm64
	0x1E00_3FFA,// XOP_Llwpcb_r32
	0x1400_3FF0,// XOP_Llwpcb_r64
	0x1E00_3FFA,// XOP_Slwpcb_r32
	0x1400_3FF0,// XOP_Slwpcb_r64
	0x1E00_3FFA,// XOP_Vfrczps_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vfrczps_ymm_ymmm256
	0x1E00_3FFA,// XOP_Vfrczpd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vfrczpd_ymm_ymmm256
	0x1E00_3FFA,// XOP_Vfrczss_xmm_xmmm32
	0x1E00_3FFA,// XOP_Vfrczsd_xmm_xmmm64
	0x1E00_3FFA,// XOP_Vprotb_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vprotb_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vprotw_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vprotw_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vprotd_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vprotd_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vprotq_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vprotq_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshlb_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshlb_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshlw_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshlw_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshld_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshld_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshlq_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshlq_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshab_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshab_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshaw_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshaw_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshad_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshad_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vpshaq_xmm_xmmm128_xmm
	0x1E00_3FFA,// XOP_Vpshaq_xmm_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddbw_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddbd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddbq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddwd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddwq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphadddq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddubw_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddubd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddubq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphadduwd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphadduwq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphaddudq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphsubbw_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphsubwd_xmm_xmmm128
	0x1E00_3FFA,// XOP_Vphsubdq_xmm_xmmm128
	0x1E00_3FFA,// XOP_Bextr_r32_rm32_imm32
	0x1400_3FF0,// XOP_Bextr_r64_rm64_imm32
	0x1E00_3FFA,// XOP_Lwpins_r32_rm32_imm32
	0x1400_3FF0,// XOP_Lwpins_r64_rm32_imm32
	0x1E00_3FFA,// XOP_Lwpval_r32_rm32_imm32
	0x1400_3FF0,// XOP_Lwpval_r64_rm32_imm32
	0x7E00_3FFF,// D3NOW_Pi2fw_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pi2fd_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pf2iw_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pf2id_mm_mmm64
	0x6A00_3FFF,// D3NOW_Pfrcpv_mm_mmm64
	0x6A00_3FFF,// D3NOW_Pfrsqrtv_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfnacc_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfpnacc_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfcmpge_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfmin_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfrcp_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfrsqrt_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfsub_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfadd_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfcmpgt_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfmax_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfrcpit1_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfrsqit1_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfsubr_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfacc_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfcmpeq_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfmul_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pfrcpit2_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pmulhrw_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pswapd_mm_mmm64
	0x7E00_3FFF,// D3NOW_Pavgusb_mm_mmm64
	0x1420_3E70,// Rmpadjust
	0x1400_3E70,// Rmpupdate
	0x1400_3E70,// Psmash
	0x0A20_3E7B,// Pvalidatew
	0x1E20_3E7B,// Pvalidated
	0x1420_3E70,// Pvalidateq
	0x1E00_3FFF,// Serialize
	0x1F00_3FFF,// Xsusldtrk
	0x1F00_3FFF,// Xresldtrk
	0x0A20_3E7A,// Invlpgbw
	0x1E20_3E7A,// Invlpgbd
	0x1420_3E70,// Invlpgbq
	0x1E20_3E7A,// Tlbsync
	0x1E00_3FFF,// Prefetchreserved3_m8
	0x1E00_3FFF,// Prefetchreserved4_m8
	0x1E00_3FFF,// Prefetchreserved5_m8
	0x1E00_3FFF,// Prefetchreserved6_m8
	0x1E00_3FFF,// Prefetchreserved7_m8
	0x1800_3FFF,// Ud0
	0x1E10_3FFF,// Vmgexit
	0x1482_3E70,// Getsecq
	0x1440_3FF0,// VEX_Ldtilecfg_m512
	0x1440_3FF0,// VEX_Tilerelease
	0x1440_3FF0,// VEX_Sttilecfg_m512
	0x1440_3FF0,// VEX_Tilezero_tmm
	0x1440_3FF0,// VEX_Tileloaddt1_tmm_sibmem
	0x1440_3FF0,// VEX_Tilestored_sibmem_tmm
	0x1440_3FF0,// VEX_Tileloadd_tmm_sibmem
	0x1440_3FF0,// VEX_Tdpbf16ps_tmm_tmm_tmm
	0x1440_3FF0,// VEX_Tdpbuud_tmm_tmm_tmm
	0x1440_3FF0,// VEX_Tdpbusd_tmm_tmm_tmm
	0x1440_3FF0,// VEX_Tdpbsud_tmm_tmm_tmm
	0x1440_3FF0,// VEX_Tdpbssd_tmm_tmm_tmm
	0x0A80_3FFF,// Fnstdw_AX
	0x0A80_3FFF,// Fnstsg_AX
	0x0A00_3E6B,// Rdshr_rm32
	0x0A00_3E6B,// Wrshr_rm32
	0x0A00_3E7B,// Smint
	0x0A00_3E7B,// Dmint
	0x0A00_3E7B,// Rdm
	0x0A00_3E7B,// Svdc_m80_Sreg
	0x0A00_3E7B,// Rsdc_Sreg_m80
	0x0A00_3E7B,// Svldt_m80
	0x0A00_3E7B,// Rsldt_m80
	0x0A00_3E7B,// Svts_m80
	0x0A00_3E7B,// Rsts_m80
	0x0A00_3E7B,// Smint_0F7E
	0x0A00_3E7B,// Bb0_reset
	0x0A00_3E7B,// Bb1_reset
	0x0A00_3E7B,// Cpu_write
	0x0A00_3E7B,// Cpu_read
	0x0A00_3FFF,// Altinst
	0x6A00_3FFF,// Paveb_mm_mmm64
	0x6A00_3FFF,// Paddsiw_mm_mmm64
	0x6A00_3FFF,// Pmagw_mm_mmm64
	0x6A00_3FFF,// Pdistib_mm_m64
	0x6A00_3FFF,// Psubsiw_mm_mmm64
	0x6A00_3FFF,// Pmvzb_mm_m64
	0x6A00_3FFF,// Pmulhrw_mm_mmm64
	0x6A00_3FFF,// Pmvnzb_mm_m64
	0x6A00_3FFF,// Pmvlzb_mm_m64
	0x6A00_3FFF,// Pmvgezb_mm_m64
	0x6A00_3FFF,// Pmulhriw_mm_mmm64
	0x6A00_3FFF,// Pmachriw_mm_m64
	0x0A00_3FFF,// Cyrix_D9D7
	0x0A00_3FFF,// Cyrix_D9E2
	0x0A00_3FFF,// Ftstp
	0x0A00_3FFF,// Cyrix_D9E7
	0x0A00_3FFF,// Frint2
	0x0A00_3FFF,// Frichop
	0x0A00_3FFF,// Cyrix_DED8
	0x0A00_3FFF,// Cyrix_DEDA
	0x0A00_3FFF,// Cyrix_DEDC
	0x0A00_3FFF,// Cyrix_DEDD
	0x0A00_3FFF,// Cyrix_DEDE
	0x0A00_3FFF,// Frinear
	0x1E82_387B,// Tdcall
	0x1480_2470,// Seamret
	0x1480_2470,// Seamops
	0x1482_1C50,// Seamcall
	0x1E80_3FFF,// Aesencwide128kl_m384
	0x1E80_3FFF,// Aesdecwide128kl_m384
	0x1E80_3FFF,// Aesencwide256kl_m512
	0x1E80_3FFF,// Aesdecwide256kl_m512
	0x1E84_3E7B,// Loadiwkey_xmm_xmm
	0x7E80_3FFF,// Aesenc128kl_xmm_m384
	0x7E80_3FFF,// Aesdec128kl_xmm_m384
	0x7E80_3FFF,// Aesenc256kl_xmm_m512
	0x7E80_3FFF,// Aesdec256kl_xmm_m512
	0x3E80_3FFF,// Encodekey128_r32_r32
	0x3E80_3FFF,// Encodekey256_r32_r32
	0x1E00_3FFA,// VEX_Vbroadcastss_xmm_xmm
	0x1E00_3FFA,// VEX_Vbroadcastss_ymm_xmm
	0x1E00_3FFA,// VEX_Vbroadcastsd_ymm_xmm
	0x1E10_3FFF,// Vmgexit_F2
	0x1440_3E70,// Uiret
	0x1400_3E70,// Testui
	0x1440_3E70,// Clui
	0x1440_3E70,// Stui
	0x1404_3E70,// Senduipi_r64
	0x1E40_3E7B,// Hreset_imm8
	0x1E00_3FFA,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	0x0A00_3FFF,// Ccs_hash_16
	0x1E00_3FFF,// Ccs_hash_32
	0x1400_3FF0,// Ccs_hash_64
	0x0A00_3FFF,// Ccs_encrypt_16
	0x1E00_3FFF,// Ccs_encrypt_32
	0x1400_3FF0,// Ccs_encrypt_64
	0x1400_3E70,// Lkgs_rm16
	0x1400_3E70,// Lkgs_r32m16
	0x1400_3E70,// Lkgs_r64m16
	0x1400_3E70,// Eretu
	0x1400_3E70,// Erets
	0x1E00_3FFA,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	0x1E00_3FFA,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	0x1E00_3FFA,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	0x1E00_3FFA,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	0x1E00_3FFA,// EVEX_Vcomish_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	0x1E00_3FFA,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	0x1E00_3FFA,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	0x1E00_3FFA,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	0x1E00_3FFA,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	0x1E00_3FFA,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	0x1E00_3FFA,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	0x1E00_3FFA,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	0x1E00_3FFA,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	0x1E00_3FFA,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	0x1E00_3FFA,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vcvtsh2si_r32_xmmm16_er
	0x1400_3FF0,// EVEX_Vcvtsh2si_r64_xmmm16_er
	0x1E00_3FFA,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	0x1400_3FF0,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	0x1E00_3FFA,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	0x1E00_3FFA,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	0x1E00_3FFA,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	0x1E00_3FFA,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	0x1E00_3FFA,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	0x1E00_3FFA,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	0x1E00_3FFA,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	0x1E00_3FFA,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	0x1E00_3FFA,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	0x1E00_3FFA,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	0x1E00_3FFA,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	0x1400_3FF0,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	0x1400_3FF0,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	0x1E00_3FFA,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	0x1E00_3FFA,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	0x1E00_3FFA,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	0x1E00_3FFA,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	0x1400_3FF0,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	0x1E00_3FFA,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	0x1E00_3FFA,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	0x1E00_3FFA,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	0x1E00_3FFA,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	0x1E00_3FFA,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	0x1E00_3FFA,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	0x1E00_3FFA,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	0x1E00_3FFA,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	0x1E00_3FFA,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	0x1E00_3FFA,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	0x1E00_3FFA,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	0x1E00_3FFA,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	0x1E00_3FFA,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x1E00_3FFA,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	0x1E00_3FFA,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	0x1E00_3FFA,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	0x1E00_3FFA,// EVEX_Vmovsh_xmm_k1z_m16
	0x1E00_3FFA,// EVEX_Vmovsh_m16_k1_xmm
	0x1E00_3FFA,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	0x1E00_3FFA,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	0x1E00_3FFA,// EVEX_Vmovw_xmm_r32m16
	0x1400_3FF0,// EVEX_Vmovw_xmm_r64m16
	0x1E00_3FFA,// EVEX_Vmovw_r32m16_xmm
	0x1400_3FF0,// EVEX_Vmovw_r64m16_xmm
	0x1E00_3FFA,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	0x1E00_3FFA,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	0x1E00_3FFA,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	0x1E00_3FFA,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	0x1E00_3FFA,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	0x1E00_3FFA,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x1E00_3FFA,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	0x1E00_3FFA,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	0x1E00_3FFA,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	0x1E00_3FFA,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	0x1E00_3FFA,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	0x1E00_3FFA,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	0x1E00_3FFA,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	0x1E00_3FFA,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	0x1E00_3FFA,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	0x1E00_3FFA,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	0x1E00_3FFA,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	0x1E00_3FFA,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	0x1E00_3FFA,// EVEX_Vucomish_xmm_xmmm16_sae
	0x1E00_3FFF,// Rdudbg
	0x1E00_3FFF,// Wrudbg
	0x1400_3FF0,// VEX_KNC_Jkzd_kr_rel8_64
	0x1400_3FF0,// VEX_KNC_Jknzd_kr_rel8_64
	0x1400_3FF0,// VEX_KNC_Vprefetchnta_m8
	0x1400_3FF0,// VEX_KNC_Vprefetch0_m8
	0x1400_3FF0,// VEX_KNC_Vprefetch1_m8
	0x1400_3FF0,// VEX_KNC_Vprefetch2_m8
	0x1400_3FF0,// VEX_KNC_Vprefetchenta_m8
	0x1400_3FF0,// VEX_KNC_Vprefetche0_m8
	0x1400_3FF0,// VEX_KNC_Vprefetche1_m8
	0x1400_3FF0,// VEX_KNC_Vprefetche2_m8
	0x1400_3FF0,// VEX_KNC_Kand_kr_kr
	0x1400_3FF0,// VEX_KNC_Kandn_kr_kr
	0x1400_3FF0,// VEX_KNC_Kandnr_kr_kr
	0x1400_3FF0,// VEX_KNC_Knot_kr_kr
	0x1400_3FF0,// VEX_KNC_Kor_kr_kr
	0x1400_3FF0,// VEX_KNC_Kxnor_kr_kr
	0x1400_3FF0,// VEX_KNC_Kxor_kr_kr
	0x1400_3FF0,// VEX_KNC_Kmerge2l1h_kr_kr
	0x1400_3FF0,// VEX_KNC_Kmerge2l1l_kr_kr
	0x1400_3FF0,// VEX_KNC_Jkzd_kr_rel32_64
	0x1400_3FF0,// VEX_KNC_Jknzd_kr_rel32_64
	0x1400_3FF0,// VEX_KNC_Kmov_kr_kr
	0x1400_3FF0,// VEX_KNC_Kmov_kr_r32
	0x1400_3FF0,// VEX_KNC_Kmov_r32_kr
	0x1400_3FF0,// VEX_KNC_Kconcath_r64_kr_kr
	0x1400_3FF0,// VEX_KNC_Kconcatl_r64_kr_kr
	0x1400_3FF0,// VEX_KNC_Kortest_kr_kr
	0x1400_3FF0,// VEX_KNC_Delay_r32
	0x1400_3FF0,// VEX_KNC_Delay_r64
	0x1400_3FF0,// VEX_KNC_Spflt_r32
	0x1400_3FF0,// VEX_KNC_Spflt_r64
	0x1400_3FF0,// VEX_KNC_Clevict1_m8
	0x1400_3FF0,// VEX_KNC_Clevict0_m8
	0x3400_3FF0,// VEX_KNC_Popcnt_r32_r32
	0x3400_3FF0,// VEX_KNC_Popcnt_r64_r64
	0x3400_3FF0,// VEX_KNC_Tzcnt_r32_r32
	0x3400_3FF0,// VEX_KNC_Tzcnt_r64_r64
	0x3400_3FF0,// VEX_KNC_Tzcnti_r32_r32
	0x3400_3FF0,// VEX_KNC_Tzcnti_r64_r64
	0x3400_3FF0,// VEX_KNC_Lzcnt_r32_r32
	0x3400_3FF0,// VEX_KNC_Lzcnt_r64_r64
	0x1400_3FF0,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	0x1400_3FF0,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	0x1400_3FF0,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	0x1400_3FF0,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	0x1400_3FF0,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	0x1400_3FF0,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	0x1400_3FF0,// VEX_KNC_Kextract_kr_r64_imm8
	0x1400_3FF0,// MVEX_Vprefetchnta_m
	0x1400_3FF0,// MVEX_Vprefetch0_m
	0x1400_3FF0,// MVEX_Vprefetch1_m
	0x1400_3FF0,// MVEX_Vprefetch2_m
	0x1400_3FF0,// MVEX_Vprefetchenta_m
	0x1400_3FF0,// MVEX_Vprefetche0_m
	0x1400_3FF0,// MVEX_Vprefetche1_m
	0x1400_3FF0,// MVEX_Vprefetche2_m
	0x1400_3FF0,// MVEX_Vmovaps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vmovapd_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vmovaps_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vmovapd_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vmovnrapd_m_k1_zmm
	0x1400_3FF0,// MVEX_Vmovnrngoapd_m_k1_zmm
	0x1400_3FF0,// MVEX_Vmovnraps_m_k1_zmm
	0x1400_3FF0,// MVEX_Vmovnrngoaps_m_k1_zmm
	0x1400_3FF0,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	0x3400_3FF0,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	0x3400_3FF0,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vmovdqa32_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vmovdqa64_mt_k1_zmm
	0x1400_3FF0,// MVEX_Clevict1_m
	0x1400_3FF0,// MVEX_Clevict0_m
	0x3400_3FF0,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	0x3400_3FF0,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vbroadcastss_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vbroadcastsd_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	0x3400_3FF0,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgetexpps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vgetexppd_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	0x1400_3FF0,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	0x1400_3FF0,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	0x1400_3FF0,// MVEX_Vpbroadcastd_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vpbroadcastq_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	0xD400_3FF0,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	0xD400_3FF0,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	0xD400_3FF0,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	0xD400_3FF0,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	0x1400_3FF0,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	0x1400_3FF0,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	0xD400_3FF0,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	0xD400_3FF0,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	0x3400_3FF0,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpgatherdd_zmm_k1_mvt
	0x1400_3FF0,// MVEX_Vpgatherdq_zmm_k1_mvt
	0x1400_3FF0,// MVEX_Vgatherdps_zmm_k1_mvt
	0x1400_3FF0,// MVEX_Vgatherdpd_zmm_k1_mvt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	0x1400_3FF0,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpscatterdd_mvt_k1_zmm
	0x1400_3FF0,// MVEX_Vpscatterdq_mvt_k1_zmm
	0x1400_3FF0,// MVEX_Vscatterdps_mvt_k1_zmm
	0x1400_3FF0,// MVEX_Vscatterdpd_mvt_k1_zmm
	0x1400_3FF0,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	0x1400_3FF0,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	0x1400_3FF0,// MVEX_Vgatherpf0hintdps_mvt_k1
	0x1400_3FF0,// MVEX_Vgatherpf0hintdpd_mvt_k1
	0x1400_3FF0,// MVEX_Vgatherpf0dps_mvt_k1
	0x1400_3FF0,// MVEX_Vgatherpf1dps_mvt_k1
	0x1400_3FF0,// MVEX_Vscatterpf0hintdps_mvt_k1
	0x1400_3FF0,// MVEX_Vscatterpf0hintdpd_mvt_k1
	0x1400_3FF0,// MVEX_Vscatterpf0dps_mvt_k1
	0x1400_3FF0,// MVEX_Vscatterpf1dps_mvt_k1
	0x1400_3FF0,// MVEX_Vexp223ps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vlog2ps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	0x1400_3FF0,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	0x1400_3FF0,// MVEX_Vloadunpackld_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vloadunpacklq_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vpackstoreld_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vpackstorelq_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vloadunpacklps_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vloadunpacklpd_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vpackstorelps_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vpackstorelpd_mt_k1_zmm
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	0x1400_3FF0,// MVEX_Vloadunpackhd_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vloadunpackhq_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vpackstorehd_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vpackstorehq_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vloadunpackhps_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vloadunpackhpd_zmm_k1_mt
	0x1400_3FF0,// MVEX_Vpackstorehps_mt_k1_zmm
	0x1400_3FF0,// MVEX_Vpackstorehpd_mt_k1_zmm
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	0x1400_3FF0,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	0x3400_3FF0,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	0x3400_3FF0,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	0x1400_3FF0,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	0x1400_3FF0,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	0x0A00_3FFF,// Via_undoc_F30FA6F0_16
	0x1E00_3FFF,// Via_undoc_F30FA6F0_32
	0x1400_3FF0,// Via_undoc_F30FA6F0_64
	0x0A00_3FFF,// Via_undoc_F30FA6F8_16
	0x1E00_3FFF,// Via_undoc_F30FA6F8_32
	0x1400_3FF0,// Via_undoc_F30FA6F8_64
	0x0A00_3FFF,// Xsha512_16
	0x1E00_3FFF,// Xsha512_32
	0x1400_3FF0,// Xsha512_64
	0x0A00_3FFF,// Xstore_alt_16
	0x1E00_3FFF,// Xstore_alt_32
	0x1400_3FF0,// Xstore_alt_64
	0x0A00_3FFF,// Xsha512_alt_16
	0x1E00_3FFF,// Xsha512_alt_32
	0x1400_3FF0,// Xsha512_alt_64
	0x1E00_3FFF,// Zero_bytes
	0x1EA4_3E7B,// Wrmsrns
	0x14A4_3E70,// Wrmsrlist
	0x14A4_3E70,// Rdmsrlist
	0x1420_3E70,// Rmpquery
	0x1E00_3FFF,// Prefetchit1_m8
	0x1E00_3FFF,// Prefetchit0_m8
	0x1E40_3FFF,// Aadd_m32_r32
	0x1440_3FF0,// Aadd_m64_r64
	0x1E40_3FFF,// Aand_m32_r32
	0x1440_3FF0,// Aand_m64_r64
	0x1E40_3FFF,// Axor_m32_r32
	0x1440_3FF0,// Axor_m64_r64
	0x1E40_3FFF,// Aor_m32_r32
	0x1440_3FF0,// Aor_m64_r64
	0x1E00_3FFA,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	0x1440_3FF0,// VEX_Tdpfp16ps_tmm_tmm_tmm
	0x1E00_3FFA,// VEX_Vcvtneps2bf16_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vcvtneps2bf16_xmm_ymmm256
	0x1E00_3FFA,// VEX_Vcvtneoph2ps_xmm_m128
	0x1E00_3FFA,// VEX_Vcvtneoph2ps_ymm_m256
	0x1E00_3FFA,// VEX_Vcvtneeph2ps_xmm_m128
	0x1E00_3FFA,// VEX_Vcvtneeph2ps_ymm_m256
	0x1E00_3FFA,// VEX_Vcvtneebf162ps_xmm_m128
	0x1E00_3FFA,// VEX_Vcvtneebf162ps_ymm_m256
	0x1E00_3FFA,// VEX_Vcvtneobf162ps_xmm_m128
	0x1E00_3FFA,// VEX_Vcvtneobf162ps_ymm_m256
	0x1E00_3FFA,// VEX_Vbcstnesh2ps_xmm_m16
	0x1E00_3FFA,// VEX_Vbcstnesh2ps_ymm_m16
	0x1E00_3FFA,// VEX_Vbcstnebf162ps_xmm_m16
	0x1E00_3FFA,// VEX_Vbcstnebf162ps_ymm_m16
	0x1E00_3FFA,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	0x3400_3FF0,// VEX_Cmpoxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpoxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnoxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnoxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpbxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpbxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnbxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnbxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpzxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpzxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnzxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnzxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpbexadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpbexadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnbexadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnbexadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpsxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpsxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnsxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnsxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmppxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmppxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnpxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnpxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmplxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmplxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnlxadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnlxadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmplexadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmplexadd_m64_r64_r64
	0x3400_3FF0,// VEX_Cmpnlexadd_m32_r32_r32
	0x3400_3FF0,// VEX_Cmpnlexadd_m64_r64_r64
	0x1440_3FF0,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	0x1440_3FF0,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	0x1400_3E70,// Pbndkb
	0x1E00_3FFA,// VEX_Vsha512rnds2_ymm_ymm_xmm
	0x1E00_3FFA,// VEX_Vsha512msg1_ymm_xmm
	0x1E00_3FFA,// VEX_Vsha512msg2_ymm_ymm
	0x1E00_3FFA,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsm4key4_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsm4key4_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	0x1E00_3FFA,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	0x1E00_3FFA,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];
