// SPDX-License-Identifier: MIT
// Copyright (C) 2018-present iced project and contributors

// ⚠️This file was generated by GENERATOR!🦹‍♂️

use crate::iced_constants::IcedConstants;

#[rustfmt::skip]
pub(super) static OP_COUNT: [u8; IcedConstants::CODE_ENUM_COUNT] = [
	0,// INVALID
	0,// DeclareByte
	0,// DeclareWord
	0,// DeclareDword
	0,// DeclareQword
	2,// Add_rm8_r8
	2,// Add_rm16_r16
	2,// Add_rm32_r32
	2,// Add_rm64_r64
	2,// Add_r8_rm8
	2,// Add_r16_rm16
	2,// Add_r32_rm32
	2,// Add_r64_rm64
	2,// Add_AL_imm8
	2,// Add_AX_imm16
	2,// Add_EAX_imm32
	2,// Add_RAX_imm32
	1,// Pushw_ES
	1,// Pushd_ES
	1,// Popw_ES
	1,// Popd_ES
	2,// Or_rm8_r8
	2,// Or_rm16_r16
	2,// Or_rm32_r32
	2,// Or_rm64_r64
	2,// Or_r8_rm8
	2,// Or_r16_rm16
	2,// Or_r32_rm32
	2,// Or_r64_rm64
	2,// Or_AL_imm8
	2,// Or_AX_imm16
	2,// Or_EAX_imm32
	2,// Or_RAX_imm32
	1,// Pushw_CS
	1,// Pushd_CS
	1,// Popw_CS
	2,// Adc_rm8_r8
	2,// Adc_rm16_r16
	2,// Adc_rm32_r32
	2,// Adc_rm64_r64
	2,// Adc_r8_rm8
	2,// Adc_r16_rm16
	2,// Adc_r32_rm32
	2,// Adc_r64_rm64
	2,// Adc_AL_imm8
	2,// Adc_AX_imm16
	2,// Adc_EAX_imm32
	2,// Adc_RAX_imm32
	1,// Pushw_SS
	1,// Pushd_SS
	1,// Popw_SS
	1,// Popd_SS
	2,// Sbb_rm8_r8
	2,// Sbb_rm16_r16
	2,// Sbb_rm32_r32
	2,// Sbb_rm64_r64
	2,// Sbb_r8_rm8
	2,// Sbb_r16_rm16
	2,// Sbb_r32_rm32
	2,// Sbb_r64_rm64
	2,// Sbb_AL_imm8
	2,// Sbb_AX_imm16
	2,// Sbb_EAX_imm32
	2,// Sbb_RAX_imm32
	1,// Pushw_DS
	1,// Pushd_DS
	1,// Popw_DS
	1,// Popd_DS
	2,// And_rm8_r8
	2,// And_rm16_r16
	2,// And_rm32_r32
	2,// And_rm64_r64
	2,// And_r8_rm8
	2,// And_r16_rm16
	2,// And_r32_rm32
	2,// And_r64_rm64
	2,// And_AL_imm8
	2,// And_AX_imm16
	2,// And_EAX_imm32
	2,// And_RAX_imm32
	0,// Daa
	2,// Sub_rm8_r8
	2,// Sub_rm16_r16
	2,// Sub_rm32_r32
	2,// Sub_rm64_r64
	2,// Sub_r8_rm8
	2,// Sub_r16_rm16
	2,// Sub_r32_rm32
	2,// Sub_r64_rm64
	2,// Sub_AL_imm8
	2,// Sub_AX_imm16
	2,// Sub_EAX_imm32
	2,// Sub_RAX_imm32
	0,// Das
	2,// Xor_rm8_r8
	2,// Xor_rm16_r16
	2,// Xor_rm32_r32
	2,// Xor_rm64_r64
	2,// Xor_r8_rm8
	2,// Xor_r16_rm16
	2,// Xor_r32_rm32
	2,// Xor_r64_rm64
	2,// Xor_AL_imm8
	2,// Xor_AX_imm16
	2,// Xor_EAX_imm32
	2,// Xor_RAX_imm32
	0,// Aaa
	2,// Cmp_rm8_r8
	2,// Cmp_rm16_r16
	2,// Cmp_rm32_r32
	2,// Cmp_rm64_r64
	2,// Cmp_r8_rm8
	2,// Cmp_r16_rm16
	2,// Cmp_r32_rm32
	2,// Cmp_r64_rm64
	2,// Cmp_AL_imm8
	2,// Cmp_AX_imm16
	2,// Cmp_EAX_imm32
	2,// Cmp_RAX_imm32
	0,// Aas
	1,// Inc_r16
	1,// Inc_r32
	1,// Dec_r16
	1,// Dec_r32
	1,// Push_r16
	1,// Push_r32
	1,// Push_r64
	1,// Pop_r16
	1,// Pop_r32
	1,// Pop_r64
	0,// Pushaw
	0,// Pushad
	0,// Popaw
	0,// Popad
	2,// Bound_r16_m1616
	2,// Bound_r32_m3232
	2,// Arpl_rm16_r16
	2,// Arpl_r32m16_r32
	2,// Movsxd_r16_rm16
	2,// Movsxd_r32_rm32
	2,// Movsxd_r64_rm32
	1,// Push_imm16
	1,// Pushd_imm32
	1,// Pushq_imm32
	3,// Imul_r16_rm16_imm16
	3,// Imul_r32_rm32_imm32
	3,// Imul_r64_rm64_imm32
	1,// Pushw_imm8
	1,// Pushd_imm8
	1,// Pushq_imm8
	3,// Imul_r16_rm16_imm8
	3,// Imul_r32_rm32_imm8
	3,// Imul_r64_rm64_imm8
	2,// Insb_m8_DX
	2,// Insw_m16_DX
	2,// Insd_m32_DX
	2,// Outsb_DX_m8
	2,// Outsw_DX_m16
	2,// Outsd_DX_m32
	1,// Jo_rel8_16
	1,// Jo_rel8_32
	1,// Jo_rel8_64
	1,// Jno_rel8_16
	1,// Jno_rel8_32
	1,// Jno_rel8_64
	1,// Jb_rel8_16
	1,// Jb_rel8_32
	1,// Jb_rel8_64
	1,// Jae_rel8_16
	1,// Jae_rel8_32
	1,// Jae_rel8_64
	1,// Je_rel8_16
	1,// Je_rel8_32
	1,// Je_rel8_64
	1,// Jne_rel8_16
	1,// Jne_rel8_32
	1,// Jne_rel8_64
	1,// Jbe_rel8_16
	1,// Jbe_rel8_32
	1,// Jbe_rel8_64
	1,// Ja_rel8_16
	1,// Ja_rel8_32
	1,// Ja_rel8_64
	1,// Js_rel8_16
	1,// Js_rel8_32
	1,// Js_rel8_64
	1,// Jns_rel8_16
	1,// Jns_rel8_32
	1,// Jns_rel8_64
	1,// Jp_rel8_16
	1,// Jp_rel8_32
	1,// Jp_rel8_64
	1,// Jnp_rel8_16
	1,// Jnp_rel8_32
	1,// Jnp_rel8_64
	1,// Jl_rel8_16
	1,// Jl_rel8_32
	1,// Jl_rel8_64
	1,// Jge_rel8_16
	1,// Jge_rel8_32
	1,// Jge_rel8_64
	1,// Jle_rel8_16
	1,// Jle_rel8_32
	1,// Jle_rel8_64
	1,// Jg_rel8_16
	1,// Jg_rel8_32
	1,// Jg_rel8_64
	2,// Add_rm8_imm8
	2,// Or_rm8_imm8
	2,// Adc_rm8_imm8
	2,// Sbb_rm8_imm8
	2,// And_rm8_imm8
	2,// Sub_rm8_imm8
	2,// Xor_rm8_imm8
	2,// Cmp_rm8_imm8
	2,// Add_rm16_imm16
	2,// Add_rm32_imm32
	2,// Add_rm64_imm32
	2,// Or_rm16_imm16
	2,// Or_rm32_imm32
	2,// Or_rm64_imm32
	2,// Adc_rm16_imm16
	2,// Adc_rm32_imm32
	2,// Adc_rm64_imm32
	2,// Sbb_rm16_imm16
	2,// Sbb_rm32_imm32
	2,// Sbb_rm64_imm32
	2,// And_rm16_imm16
	2,// And_rm32_imm32
	2,// And_rm64_imm32
	2,// Sub_rm16_imm16
	2,// Sub_rm32_imm32
	2,// Sub_rm64_imm32
	2,// Xor_rm16_imm16
	2,// Xor_rm32_imm32
	2,// Xor_rm64_imm32
	2,// Cmp_rm16_imm16
	2,// Cmp_rm32_imm32
	2,// Cmp_rm64_imm32
	2,// Add_rm8_imm8_82
	2,// Or_rm8_imm8_82
	2,// Adc_rm8_imm8_82
	2,// Sbb_rm8_imm8_82
	2,// And_rm8_imm8_82
	2,// Sub_rm8_imm8_82
	2,// Xor_rm8_imm8_82
	2,// Cmp_rm8_imm8_82
	2,// Add_rm16_imm8
	2,// Add_rm32_imm8
	2,// Add_rm64_imm8
	2,// Or_rm16_imm8
	2,// Or_rm32_imm8
	2,// Or_rm64_imm8
	2,// Adc_rm16_imm8
	2,// Adc_rm32_imm8
	2,// Adc_rm64_imm8
	2,// Sbb_rm16_imm8
	2,// Sbb_rm32_imm8
	2,// Sbb_rm64_imm8
	2,// And_rm16_imm8
	2,// And_rm32_imm8
	2,// And_rm64_imm8
	2,// Sub_rm16_imm8
	2,// Sub_rm32_imm8
	2,// Sub_rm64_imm8
	2,// Xor_rm16_imm8
	2,// Xor_rm32_imm8
	2,// Xor_rm64_imm8
	2,// Cmp_rm16_imm8
	2,// Cmp_rm32_imm8
	2,// Cmp_rm64_imm8
	2,// Test_rm8_r8
	2,// Test_rm16_r16
	2,// Test_rm32_r32
	2,// Test_rm64_r64
	2,// Xchg_rm8_r8
	2,// Xchg_rm16_r16
	2,// Xchg_rm32_r32
	2,// Xchg_rm64_r64
	2,// Mov_rm8_r8
	2,// Mov_rm16_r16
	2,// Mov_rm32_r32
	2,// Mov_rm64_r64
	2,// Mov_r8_rm8
	2,// Mov_r16_rm16
	2,// Mov_r32_rm32
	2,// Mov_r64_rm64
	2,// Mov_rm16_Sreg
	2,// Mov_r32m16_Sreg
	2,// Mov_r64m16_Sreg
	2,// Lea_r16_m
	2,// Lea_r32_m
	2,// Lea_r64_m
	2,// Mov_Sreg_rm16
	2,// Mov_Sreg_r32m16
	2,// Mov_Sreg_r64m16
	1,// Pop_rm16
	1,// Pop_rm32
	1,// Pop_rm64
	0,// Nopw
	0,// Nopd
	0,// Nopq
	2,// Xchg_r16_AX
	2,// Xchg_r32_EAX
	2,// Xchg_r64_RAX
	0,// Pause
	0,// Cbw
	0,// Cwde
	0,// Cdqe
	0,// Cwd
	0,// Cdq
	0,// Cqo
	1,// Call_ptr1616
	1,// Call_ptr1632
	0,// Wait
	0,// Pushfw
	0,// Pushfd
	0,// Pushfq
	0,// Popfw
	0,// Popfd
	0,// Popfq
	0,// Sahf
	0,// Lahf
	2,// Mov_AL_moffs8
	2,// Mov_AX_moffs16
	2,// Mov_EAX_moffs32
	2,// Mov_RAX_moffs64
	2,// Mov_moffs8_AL
	2,// Mov_moffs16_AX
	2,// Mov_moffs32_EAX
	2,// Mov_moffs64_RAX
	2,// Movsb_m8_m8
	2,// Movsw_m16_m16
	2,// Movsd_m32_m32
	2,// Movsq_m64_m64
	2,// Cmpsb_m8_m8
	2,// Cmpsw_m16_m16
	2,// Cmpsd_m32_m32
	2,// Cmpsq_m64_m64
	2,// Test_AL_imm8
	2,// Test_AX_imm16
	2,// Test_EAX_imm32
	2,// Test_RAX_imm32
	2,// Stosb_m8_AL
	2,// Stosw_m16_AX
	2,// Stosd_m32_EAX
	2,// Stosq_m64_RAX
	2,// Lodsb_AL_m8
	2,// Lodsw_AX_m16
	2,// Lodsd_EAX_m32
	2,// Lodsq_RAX_m64
	2,// Scasb_AL_m8
	2,// Scasw_AX_m16
	2,// Scasd_EAX_m32
	2,// Scasq_RAX_m64
	2,// Mov_r8_imm8
	2,// Mov_r16_imm16
	2,// Mov_r32_imm32
	2,// Mov_r64_imm64
	2,// Rol_rm8_imm8
	2,// Ror_rm8_imm8
	2,// Rcl_rm8_imm8
	2,// Rcr_rm8_imm8
	2,// Shl_rm8_imm8
	2,// Shr_rm8_imm8
	2,// Sal_rm8_imm8
	2,// Sar_rm8_imm8
	2,// Rol_rm16_imm8
	2,// Rol_rm32_imm8
	2,// Rol_rm64_imm8
	2,// Ror_rm16_imm8
	2,// Ror_rm32_imm8
	2,// Ror_rm64_imm8
	2,// Rcl_rm16_imm8
	2,// Rcl_rm32_imm8
	2,// Rcl_rm64_imm8
	2,// Rcr_rm16_imm8
	2,// Rcr_rm32_imm8
	2,// Rcr_rm64_imm8
	2,// Shl_rm16_imm8
	2,// Shl_rm32_imm8
	2,// Shl_rm64_imm8
	2,// Shr_rm16_imm8
	2,// Shr_rm32_imm8
	2,// Shr_rm64_imm8
	2,// Sal_rm16_imm8
	2,// Sal_rm32_imm8
	2,// Sal_rm64_imm8
	2,// Sar_rm16_imm8
	2,// Sar_rm32_imm8
	2,// Sar_rm64_imm8
	1,// Retnw_imm16
	1,// Retnd_imm16
	1,// Retnq_imm16
	0,// Retnw
	0,// Retnd
	0,// Retnq
	2,// Les_r16_m1616
	2,// Les_r32_m1632
	2,// Lds_r16_m1616
	2,// Lds_r32_m1632
	2,// Mov_rm8_imm8
	1,// Xabort_imm8
	2,// Mov_rm16_imm16
	2,// Mov_rm32_imm32
	2,// Mov_rm64_imm32
	1,// Xbegin_rel16
	1,// Xbegin_rel32
	2,// Enterw_imm16_imm8
	2,// Enterd_imm16_imm8
	2,// Enterq_imm16_imm8
	0,// Leavew
	0,// Leaved
	0,// Leaveq
	1,// Retfw_imm16
	1,// Retfd_imm16
	1,// Retfq_imm16
	0,// Retfw
	0,// Retfd
	0,// Retfq
	0,// Int3
	1,// Int_imm8
	0,// Into
	0,// Iretw
	0,// Iretd
	0,// Iretq
	2,// Rol_rm8_1
	2,// Ror_rm8_1
	2,// Rcl_rm8_1
	2,// Rcr_rm8_1
	2,// Shl_rm8_1
	2,// Shr_rm8_1
	2,// Sal_rm8_1
	2,// Sar_rm8_1
	2,// Rol_rm16_1
	2,// Rol_rm32_1
	2,// Rol_rm64_1
	2,// Ror_rm16_1
	2,// Ror_rm32_1
	2,// Ror_rm64_1
	2,// Rcl_rm16_1
	2,// Rcl_rm32_1
	2,// Rcl_rm64_1
	2,// Rcr_rm16_1
	2,// Rcr_rm32_1
	2,// Rcr_rm64_1
	2,// Shl_rm16_1
	2,// Shl_rm32_1
	2,// Shl_rm64_1
	2,// Shr_rm16_1
	2,// Shr_rm32_1
	2,// Shr_rm64_1
	2,// Sal_rm16_1
	2,// Sal_rm32_1
	2,// Sal_rm64_1
	2,// Sar_rm16_1
	2,// Sar_rm32_1
	2,// Sar_rm64_1
	2,// Rol_rm8_CL
	2,// Ror_rm8_CL
	2,// Rcl_rm8_CL
	2,// Rcr_rm8_CL
	2,// Shl_rm8_CL
	2,// Shr_rm8_CL
	2,// Sal_rm8_CL
	2,// Sar_rm8_CL
	2,// Rol_rm16_CL
	2,// Rol_rm32_CL
	2,// Rol_rm64_CL
	2,// Ror_rm16_CL
	2,// Ror_rm32_CL
	2,// Ror_rm64_CL
	2,// Rcl_rm16_CL
	2,// Rcl_rm32_CL
	2,// Rcl_rm64_CL
	2,// Rcr_rm16_CL
	2,// Rcr_rm32_CL
	2,// Rcr_rm64_CL
	2,// Shl_rm16_CL
	2,// Shl_rm32_CL
	2,// Shl_rm64_CL
	2,// Shr_rm16_CL
	2,// Shr_rm32_CL
	2,// Shr_rm64_CL
	2,// Sal_rm16_CL
	2,// Sal_rm32_CL
	2,// Sal_rm64_CL
	2,// Sar_rm16_CL
	2,// Sar_rm32_CL
	2,// Sar_rm64_CL
	1,// Aam_imm8
	1,// Aad_imm8
	0,// Salc
	1,// Xlat_m8
	1,// Fadd_m32fp
	1,// Fmul_m32fp
	1,// Fcom_m32fp
	1,// Fcomp_m32fp
	1,// Fsub_m32fp
	1,// Fsubr_m32fp
	1,// Fdiv_m32fp
	1,// Fdivr_m32fp
	2,// Fadd_st0_sti
	2,// Fmul_st0_sti
	2,// Fcom_st0_sti
	2,// Fcomp_st0_sti
	2,// Fsub_st0_sti
	2,// Fsubr_st0_sti
	2,// Fdiv_st0_sti
	2,// Fdivr_st0_sti
	1,// Fld_m32fp
	1,// Fst_m32fp
	1,// Fstp_m32fp
	1,// Fldenv_m14byte
	1,// Fldenv_m28byte
	1,// Fldcw_m2byte
	1,// Fnstenv_m14byte
	1,// Fstenv_m14byte
	1,// Fnstenv_m28byte
	1,// Fstenv_m28byte
	1,// Fnstcw_m2byte
	1,// Fstcw_m2byte
	1,// Fld_sti
	2,// Fxch_st0_sti
	0,// Fnop
	1,// Fstpnce_sti
	0,// Fchs
	0,// Fabs
	0,// Ftst
	0,// Fxam
	0,// Fld1
	0,// Fldl2t
	0,// Fldl2e
	0,// Fldpi
	0,// Fldlg2
	0,// Fldln2
	0,// Fldz
	0,// F2xm1
	0,// Fyl2x
	0,// Fptan
	0,// Fpatan
	0,// Fxtract
	0,// Fprem1
	0,// Fdecstp
	0,// Fincstp
	0,// Fprem
	0,// Fyl2xp1
	0,// Fsqrt
	0,// Fsincos
	0,// Frndint
	0,// Fscale
	0,// Fsin
	0,// Fcos
	1,// Fiadd_m32int
	1,// Fimul_m32int
	1,// Ficom_m32int
	1,// Ficomp_m32int
	1,// Fisub_m32int
	1,// Fisubr_m32int
	1,// Fidiv_m32int
	1,// Fidivr_m32int
	2,// Fcmovb_st0_sti
	2,// Fcmove_st0_sti
	2,// Fcmovbe_st0_sti
	2,// Fcmovu_st0_sti
	0,// Fucompp
	1,// Fild_m32int
	1,// Fisttp_m32int
	1,// Fist_m32int
	1,// Fistp_m32int
	1,// Fld_m80fp
	1,// Fstp_m80fp
	2,// Fcmovnb_st0_sti
	2,// Fcmovne_st0_sti
	2,// Fcmovnbe_st0_sti
	2,// Fcmovnu_st0_sti
	0,// Fneni
	0,// Feni
	0,// Fndisi
	0,// Fdisi
	0,// Fnclex
	0,// Fclex
	0,// Fninit
	0,// Finit
	0,// Fnsetpm
	0,// Fsetpm
	0,// Frstpm
	2,// Fucomi_st0_sti
	2,// Fcomi_st0_sti
	1,// Fadd_m64fp
	1,// Fmul_m64fp
	1,// Fcom_m64fp
	1,// Fcomp_m64fp
	1,// Fsub_m64fp
	1,// Fsubr_m64fp
	1,// Fdiv_m64fp
	1,// Fdivr_m64fp
	2,// Fadd_sti_st0
	2,// Fmul_sti_st0
	2,// Fcom_st0_sti_DCD0
	2,// Fcomp_st0_sti_DCD8
	2,// Fsubr_sti_st0
	2,// Fsub_sti_st0
	2,// Fdivr_sti_st0
	2,// Fdiv_sti_st0
	1,// Fld_m64fp
	1,// Fisttp_m64int
	1,// Fst_m64fp
	1,// Fstp_m64fp
	1,// Frstor_m94byte
	1,// Frstor_m108byte
	1,// Fnsave_m94byte
	1,// Fsave_m94byte
	1,// Fnsave_m108byte
	1,// Fsave_m108byte
	1,// Fnstsw_m2byte
	1,// Fstsw_m2byte
	1,// Ffree_sti
	2,// Fxch_st0_sti_DDC8
	1,// Fst_sti
	1,// Fstp_sti
	2,// Fucom_st0_sti
	2,// Fucomp_st0_sti
	1,// Fiadd_m16int
	1,// Fimul_m16int
	1,// Ficom_m16int
	1,// Ficomp_m16int
	1,// Fisub_m16int
	1,// Fisubr_m16int
	1,// Fidiv_m16int
	1,// Fidivr_m16int
	2,// Faddp_sti_st0
	2,// Fmulp_sti_st0
	2,// Fcomp_st0_sti_DED0
	0,// Fcompp
	2,// Fsubrp_sti_st0
	2,// Fsubp_sti_st0
	2,// Fdivrp_sti_st0
	2,// Fdivp_sti_st0
	1,// Fild_m16int
	1,// Fisttp_m16int
	1,// Fist_m16int
	1,// Fistp_m16int
	1,// Fbld_m80bcd
	1,// Fild_m64int
	1,// Fbstp_m80bcd
	1,// Fistp_m64int
	1,// Ffreep_sti
	2,// Fxch_st0_sti_DFC8
	1,// Fstp_sti_DFD0
	1,// Fstp_sti_DFD8
	1,// Fnstsw_AX
	1,// Fstsw_AX
	1,// Fstdw_AX
	1,// Fstsg_AX
	2,// Fucomip_st0_sti
	2,// Fcomip_st0_sti
	1,// Loopne_rel8_16_CX
	1,// Loopne_rel8_32_CX
	1,// Loopne_rel8_16_ECX
	1,// Loopne_rel8_32_ECX
	1,// Loopne_rel8_64_ECX
	1,// Loopne_rel8_16_RCX
	1,// Loopne_rel8_64_RCX
	1,// Loope_rel8_16_CX
	1,// Loope_rel8_32_CX
	1,// Loope_rel8_16_ECX
	1,// Loope_rel8_32_ECX
	1,// Loope_rel8_64_ECX
	1,// Loope_rel8_16_RCX
	1,// Loope_rel8_64_RCX
	1,// Loop_rel8_16_CX
	1,// Loop_rel8_32_CX
	1,// Loop_rel8_16_ECX
	1,// Loop_rel8_32_ECX
	1,// Loop_rel8_64_ECX
	1,// Loop_rel8_16_RCX
	1,// Loop_rel8_64_RCX
	1,// Jcxz_rel8_16
	1,// Jcxz_rel8_32
	1,// Jecxz_rel8_16
	1,// Jecxz_rel8_32
	1,// Jecxz_rel8_64
	1,// Jrcxz_rel8_16
	1,// Jrcxz_rel8_64
	2,// In_AL_imm8
	2,// In_AX_imm8
	2,// In_EAX_imm8
	2,// Out_imm8_AL
	2,// Out_imm8_AX
	2,// Out_imm8_EAX
	1,// Call_rel16
	1,// Call_rel32_32
	1,// Call_rel32_64
	1,// Jmp_rel16
	1,// Jmp_rel32_32
	1,// Jmp_rel32_64
	1,// Jmp_ptr1616
	1,// Jmp_ptr1632
	1,// Jmp_rel8_16
	1,// Jmp_rel8_32
	1,// Jmp_rel8_64
	2,// In_AL_DX
	2,// In_AX_DX
	2,// In_EAX_DX
	2,// Out_DX_AL
	2,// Out_DX_AX
	2,// Out_DX_EAX
	0,// Int1
	0,// Hlt
	0,// Cmc
	2,// Test_rm8_imm8
	2,// Test_rm8_imm8_F6r1
	1,// Not_rm8
	1,// Neg_rm8
	1,// Mul_rm8
	1,// Imul_rm8
	1,// Div_rm8
	1,// Idiv_rm8
	2,// Test_rm16_imm16
	2,// Test_rm32_imm32
	2,// Test_rm64_imm32
	2,// Test_rm16_imm16_F7r1
	2,// Test_rm32_imm32_F7r1
	2,// Test_rm64_imm32_F7r1
	1,// Not_rm16
	1,// Not_rm32
	1,// Not_rm64
	1,// Neg_rm16
	1,// Neg_rm32
	1,// Neg_rm64
	1,// Mul_rm16
	1,// Mul_rm32
	1,// Mul_rm64
	1,// Imul_rm16
	1,// Imul_rm32
	1,// Imul_rm64
	1,// Div_rm16
	1,// Div_rm32
	1,// Div_rm64
	1,// Idiv_rm16
	1,// Idiv_rm32
	1,// Idiv_rm64
	0,// Clc
	0,// Stc
	0,// Cli
	0,// Sti
	0,// Cld
	0,// Std
	1,// Inc_rm8
	1,// Dec_rm8
	1,// Inc_rm16
	1,// Inc_rm32
	1,// Inc_rm64
	1,// Dec_rm16
	1,// Dec_rm32
	1,// Dec_rm64
	1,// Call_rm16
	1,// Call_rm32
	1,// Call_rm64
	1,// Call_m1616
	1,// Call_m1632
	1,// Call_m1664
	1,// Jmp_rm16
	1,// Jmp_rm32
	1,// Jmp_rm64
	1,// Jmp_m1616
	1,// Jmp_m1632
	1,// Jmp_m1664
	1,// Push_rm16
	1,// Push_rm32
	1,// Push_rm64
	1,// Sldt_rm16
	1,// Sldt_r32m16
	1,// Sldt_r64m16
	1,// Str_rm16
	1,// Str_r32m16
	1,// Str_r64m16
	1,// Lldt_rm16
	1,// Lldt_r32m16
	1,// Lldt_r64m16
	1,// Ltr_rm16
	1,// Ltr_r32m16
	1,// Ltr_r64m16
	1,// Verr_rm16
	1,// Verr_r32m16
	1,// Verr_r64m16
	1,// Verw_rm16
	1,// Verw_r32m16
	1,// Verw_r64m16
	1,// Jmpe_rm16
	1,// Jmpe_rm32
	1,// Sgdt_m1632_16
	1,// Sgdt_m1632
	1,// Sgdt_m1664
	1,// Sidt_m1632_16
	1,// Sidt_m1632
	1,// Sidt_m1664
	1,// Lgdt_m1632_16
	1,// Lgdt_m1632
	1,// Lgdt_m1664
	1,// Lidt_m1632_16
	1,// Lidt_m1632
	1,// Lidt_m1664
	1,// Smsw_rm16
	1,// Smsw_r32m16
	1,// Smsw_r64m16
	1,// Rstorssp_m64
	1,// Lmsw_rm16
	1,// Lmsw_r32m16
	1,// Lmsw_r64m16
	1,// Invlpg_m
	0,// Enclv
	0,// Vmcall
	0,// Vmlaunch
	0,// Vmresume
	0,// Vmxoff
	0,// Pconfig
	0,// Monitorw
	0,// Monitord
	0,// Monitorq
	0,// Mwait
	0,// Clac
	0,// Stac
	0,// Encls
	0,// Xgetbv
	0,// Xsetbv
	0,// Vmfunc
	0,// Xend
	0,// Xtest
	0,// Enclu
	0,// Vmrunw
	0,// Vmrund
	0,// Vmrunq
	0,// Vmmcall
	0,// Vmloadw
	0,// Vmloadd
	0,// Vmloadq
	0,// Vmsavew
	0,// Vmsaved
	0,// Vmsaveq
	0,// Stgi
	0,// Clgi
	0,// Skinit
	0,// Invlpgaw
	0,// Invlpgad
	0,// Invlpgaq
	0,// Setssbsy
	0,// Saveprevssp
	0,// Rdpkru
	0,// Wrpkru
	0,// Swapgs
	0,// Rdtscp
	0,// Monitorxw
	0,// Monitorxd
	0,// Monitorxq
	0,// Mcommit
	0,// Mwaitx
	0,// Clzerow
	0,// Clzerod
	0,// Clzeroq
	0,// Rdpru
	2,// Lar_r16_rm16
	2,// Lar_r32_r32m16
	2,// Lar_r64_r64m16
	2,// Lsl_r16_rm16
	2,// Lsl_r32_r32m16
	2,// Lsl_r64_r64m16
	0,// Storeall
	0,// Loadall286
	0,// Syscall
	0,// Clts
	0,// Loadall386
	0,// Sysretd
	0,// Sysretq
	0,// Invd
	0,// Wbinvd
	0,// Wbnoinvd
	0,// Cl1invmb
	0,// Ud2
	2,// Reservednop_rm16_r16_0F0D
	2,// Reservednop_rm32_r32_0F0D
	2,// Reservednop_rm64_r64_0F0D
	1,// Prefetch_m8
	1,// Prefetchw_m8
	1,// Prefetchwt1_m8
	0,// Femms
	2,// Umov_rm8_r8
	2,// Umov_rm16_r16
	2,// Umov_rm32_r32
	2,// Umov_r8_rm8
	2,// Umov_r16_rm16
	2,// Umov_r32_rm32
	2,// Movups_xmm_xmmm128
	2,// VEX_Vmovups_xmm_xmmm128
	2,// VEX_Vmovups_ymm_ymmm256
	2,// EVEX_Vmovups_xmm_k1z_xmmm128
	2,// EVEX_Vmovups_ymm_k1z_ymmm256
	2,// EVEX_Vmovups_zmm_k1z_zmmm512
	2,// Movupd_xmm_xmmm128
	2,// VEX_Vmovupd_xmm_xmmm128
	2,// VEX_Vmovupd_ymm_ymmm256
	2,// EVEX_Vmovupd_xmm_k1z_xmmm128
	2,// EVEX_Vmovupd_ymm_k1z_ymmm256
	2,// EVEX_Vmovupd_zmm_k1z_zmmm512
	2,// Movss_xmm_xmmm32
	3,// VEX_Vmovss_xmm_xmm_xmm
	2,// VEX_Vmovss_xmm_m32
	3,// EVEX_Vmovss_xmm_k1z_xmm_xmm
	2,// EVEX_Vmovss_xmm_k1z_m32
	2,// Movsd_xmm_xmmm64
	3,// VEX_Vmovsd_xmm_xmm_xmm
	2,// VEX_Vmovsd_xmm_m64
	3,// EVEX_Vmovsd_xmm_k1z_xmm_xmm
	2,// EVEX_Vmovsd_xmm_k1z_m64
	2,// Movups_xmmm128_xmm
	2,// VEX_Vmovups_xmmm128_xmm
	2,// VEX_Vmovups_ymmm256_ymm
	2,// EVEX_Vmovups_xmmm128_k1z_xmm
	2,// EVEX_Vmovups_ymmm256_k1z_ymm
	2,// EVEX_Vmovups_zmmm512_k1z_zmm
	2,// Movupd_xmmm128_xmm
	2,// VEX_Vmovupd_xmmm128_xmm
	2,// VEX_Vmovupd_ymmm256_ymm
	2,// EVEX_Vmovupd_xmmm128_k1z_xmm
	2,// EVEX_Vmovupd_ymmm256_k1z_ymm
	2,// EVEX_Vmovupd_zmmm512_k1z_zmm
	2,// Movss_xmmm32_xmm
	3,// VEX_Vmovss_xmm_xmm_xmm_0F11
	2,// VEX_Vmovss_m32_xmm
	3,// EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11
	2,// EVEX_Vmovss_m32_k1_xmm
	2,// Movsd_xmmm64_xmm
	3,// VEX_Vmovsd_xmm_xmm_xmm_0F11
	2,// VEX_Vmovsd_m64_xmm
	3,// EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11
	2,// EVEX_Vmovsd_m64_k1_xmm
	2,// Movhlps_xmm_xmm
	2,// Movlps_xmm_m64
	3,// VEX_Vmovhlps_xmm_xmm_xmm
	3,// VEX_Vmovlps_xmm_xmm_m64
	3,// EVEX_Vmovhlps_xmm_xmm_xmm
	3,// EVEX_Vmovlps_xmm_xmm_m64
	2,// Movlpd_xmm_m64
	3,// VEX_Vmovlpd_xmm_xmm_m64
	3,// EVEX_Vmovlpd_xmm_xmm_m64
	2,// Movsldup_xmm_xmmm128
	2,// VEX_Vmovsldup_xmm_xmmm128
	2,// VEX_Vmovsldup_ymm_ymmm256
	2,// EVEX_Vmovsldup_xmm_k1z_xmmm128
	2,// EVEX_Vmovsldup_ymm_k1z_ymmm256
	2,// EVEX_Vmovsldup_zmm_k1z_zmmm512
	2,// Movddup_xmm_xmmm64
	2,// VEX_Vmovddup_xmm_xmmm64
	2,// VEX_Vmovddup_ymm_ymmm256
	2,// EVEX_Vmovddup_xmm_k1z_xmmm64
	2,// EVEX_Vmovddup_ymm_k1z_ymmm256
	2,// EVEX_Vmovddup_zmm_k1z_zmmm512
	2,// Movlps_m64_xmm
	2,// VEX_Vmovlps_m64_xmm
	2,// EVEX_Vmovlps_m64_xmm
	2,// Movlpd_m64_xmm
	2,// VEX_Vmovlpd_m64_xmm
	2,// EVEX_Vmovlpd_m64_xmm
	2,// Unpcklps_xmm_xmmm128
	3,// VEX_Vunpcklps_xmm_xmm_xmmm128
	3,// VEX_Vunpcklps_ymm_ymm_ymmm256
	3,// EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32
	2,// Unpcklpd_xmm_xmmm128
	3,// VEX_Vunpcklpd_xmm_xmm_xmmm128
	3,// VEX_Vunpcklpd_ymm_ymm_ymmm256
	3,// EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64
	2,// Unpckhps_xmm_xmmm128
	3,// VEX_Vunpckhps_xmm_xmm_xmmm128
	3,// VEX_Vunpckhps_ymm_ymm_ymmm256
	3,// EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32
	2,// Unpckhpd_xmm_xmmm128
	3,// VEX_Vunpckhpd_xmm_xmm_xmmm128
	3,// VEX_Vunpckhpd_ymm_ymm_ymmm256
	3,// EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64
	2,// Movlhps_xmm_xmm
	3,// VEX_Vmovlhps_xmm_xmm_xmm
	3,// EVEX_Vmovlhps_xmm_xmm_xmm
	2,// Movhps_xmm_m64
	3,// VEX_Vmovhps_xmm_xmm_m64
	3,// EVEX_Vmovhps_xmm_xmm_m64
	2,// Movhpd_xmm_m64
	3,// VEX_Vmovhpd_xmm_xmm_m64
	3,// EVEX_Vmovhpd_xmm_xmm_m64
	2,// Movshdup_xmm_xmmm128
	2,// VEX_Vmovshdup_xmm_xmmm128
	2,// VEX_Vmovshdup_ymm_ymmm256
	2,// EVEX_Vmovshdup_xmm_k1z_xmmm128
	2,// EVEX_Vmovshdup_ymm_k1z_ymmm256
	2,// EVEX_Vmovshdup_zmm_k1z_zmmm512
	2,// Movhps_m64_xmm
	2,// VEX_Vmovhps_m64_xmm
	2,// EVEX_Vmovhps_m64_xmm
	2,// Movhpd_m64_xmm
	2,// VEX_Vmovhpd_m64_xmm
	2,// EVEX_Vmovhpd_m64_xmm
	2,// Reservednop_rm16_r16_0F18
	2,// Reservednop_rm32_r32_0F18
	2,// Reservednop_rm64_r64_0F18
	2,// Reservednop_rm16_r16_0F19
	2,// Reservednop_rm32_r32_0F19
	2,// Reservednop_rm64_r64_0F19
	2,// Reservednop_rm16_r16_0F1A
	2,// Reservednop_rm32_r32_0F1A
	2,// Reservednop_rm64_r64_0F1A
	2,// Reservednop_rm16_r16_0F1B
	2,// Reservednop_rm32_r32_0F1B
	2,// Reservednop_rm64_r64_0F1B
	2,// Reservednop_rm16_r16_0F1C
	2,// Reservednop_rm32_r32_0F1C
	2,// Reservednop_rm64_r64_0F1C
	2,// Reservednop_rm16_r16_0F1D
	2,// Reservednop_rm32_r32_0F1D
	2,// Reservednop_rm64_r64_0F1D
	2,// Reservednop_rm16_r16_0F1E
	2,// Reservednop_rm32_r32_0F1E
	2,// Reservednop_rm64_r64_0F1E
	2,// Reservednop_rm16_r16_0F1F
	2,// Reservednop_rm32_r32_0F1F
	2,// Reservednop_rm64_r64_0F1F
	1,// Prefetchnta_m8
	1,// Prefetcht0_m8
	1,// Prefetcht1_m8
	1,// Prefetcht2_m8
	2,// Bndldx_bnd_mib
	2,// Bndmov_bnd_bndm64
	2,// Bndmov_bnd_bndm128
	2,// Bndcl_bnd_rm32
	2,// Bndcl_bnd_rm64
	2,// Bndcu_bnd_rm32
	2,// Bndcu_bnd_rm64
	2,// Bndstx_mib_bnd
	2,// Bndmov_bndm64_bnd
	2,// Bndmov_bndm128_bnd
	2,// Bndmk_bnd_m32
	2,// Bndmk_bnd_m64
	2,// Bndcn_bnd_rm32
	2,// Bndcn_bnd_rm64
	1,// Cldemote_m8
	1,// Rdsspd_r32
	1,// Rdsspq_r64
	0,// Endbr64
	0,// Endbr32
	1,// Nop_rm16
	1,// Nop_rm32
	1,// Nop_rm64
	2,// Mov_r32_cr
	2,// Mov_r64_cr
	2,// Mov_r32_dr
	2,// Mov_r64_dr
	2,// Mov_cr_r32
	2,// Mov_cr_r64
	2,// Mov_dr_r32
	2,// Mov_dr_r64
	2,// Mov_r32_tr
	2,// Mov_tr_r32
	2,// Movaps_xmm_xmmm128
	2,// VEX_Vmovaps_xmm_xmmm128
	2,// VEX_Vmovaps_ymm_ymmm256
	2,// EVEX_Vmovaps_xmm_k1z_xmmm128
	2,// EVEX_Vmovaps_ymm_k1z_ymmm256
	2,// EVEX_Vmovaps_zmm_k1z_zmmm512
	2,// Movapd_xmm_xmmm128
	2,// VEX_Vmovapd_xmm_xmmm128
	2,// VEX_Vmovapd_ymm_ymmm256
	2,// EVEX_Vmovapd_xmm_k1z_xmmm128
	2,// EVEX_Vmovapd_ymm_k1z_ymmm256
	2,// EVEX_Vmovapd_zmm_k1z_zmmm512
	2,// Movaps_xmmm128_xmm
	2,// VEX_Vmovaps_xmmm128_xmm
	2,// VEX_Vmovaps_ymmm256_ymm
	2,// EVEX_Vmovaps_xmmm128_k1z_xmm
	2,// EVEX_Vmovaps_ymmm256_k1z_ymm
	2,// EVEX_Vmovaps_zmmm512_k1z_zmm
	2,// Movapd_xmmm128_xmm
	2,// VEX_Vmovapd_xmmm128_xmm
	2,// VEX_Vmovapd_ymmm256_ymm
	2,// EVEX_Vmovapd_xmmm128_k1z_xmm
	2,// EVEX_Vmovapd_ymmm256_k1z_ymm
	2,// EVEX_Vmovapd_zmmm512_k1z_zmm
	2,// Cvtpi2ps_xmm_mmm64
	2,// Cvtpi2pd_xmm_mmm64
	2,// Cvtsi2ss_xmm_rm32
	2,// Cvtsi2ss_xmm_rm64
	3,// VEX_Vcvtsi2ss_xmm_xmm_rm32
	3,// VEX_Vcvtsi2ss_xmm_xmm_rm64
	3,// EVEX_Vcvtsi2ss_xmm_xmm_rm32_er
	3,// EVEX_Vcvtsi2ss_xmm_xmm_rm64_er
	2,// Cvtsi2sd_xmm_rm32
	2,// Cvtsi2sd_xmm_rm64
	3,// VEX_Vcvtsi2sd_xmm_xmm_rm32
	3,// VEX_Vcvtsi2sd_xmm_xmm_rm64
	3,// EVEX_Vcvtsi2sd_xmm_xmm_rm32_er
	3,// EVEX_Vcvtsi2sd_xmm_xmm_rm64_er
	2,// Movntps_m128_xmm
	2,// VEX_Vmovntps_m128_xmm
	2,// VEX_Vmovntps_m256_ymm
	2,// EVEX_Vmovntps_m128_xmm
	2,// EVEX_Vmovntps_m256_ymm
	2,// EVEX_Vmovntps_m512_zmm
	2,// Movntpd_m128_xmm
	2,// VEX_Vmovntpd_m128_xmm
	2,// VEX_Vmovntpd_m256_ymm
	2,// EVEX_Vmovntpd_m128_xmm
	2,// EVEX_Vmovntpd_m256_ymm
	2,// EVEX_Vmovntpd_m512_zmm
	2,// Movntss_m32_xmm
	2,// Movntsd_m64_xmm
	2,// Cvttps2pi_mm_xmmm64
	2,// Cvttpd2pi_mm_xmmm128
	2,// Cvttss2si_r32_xmmm32
	2,// Cvttss2si_r64_xmmm32
	2,// VEX_Vcvttss2si_r32_xmmm32
	2,// VEX_Vcvttss2si_r64_xmmm32
	2,// EVEX_Vcvttss2si_r32_xmmm32_sae
	2,// EVEX_Vcvttss2si_r64_xmmm32_sae
	2,// Cvttsd2si_r32_xmmm64
	2,// Cvttsd2si_r64_xmmm64
	2,// VEX_Vcvttsd2si_r32_xmmm64
	2,// VEX_Vcvttsd2si_r64_xmmm64
	2,// EVEX_Vcvttsd2si_r32_xmmm64_sae
	2,// EVEX_Vcvttsd2si_r64_xmmm64_sae
	2,// Cvtps2pi_mm_xmmm64
	2,// Cvtpd2pi_mm_xmmm128
	2,// Cvtss2si_r32_xmmm32
	2,// Cvtss2si_r64_xmmm32
	2,// VEX_Vcvtss2si_r32_xmmm32
	2,// VEX_Vcvtss2si_r64_xmmm32
	2,// EVEX_Vcvtss2si_r32_xmmm32_er
	2,// EVEX_Vcvtss2si_r64_xmmm32_er
	2,// Cvtsd2si_r32_xmmm64
	2,// Cvtsd2si_r64_xmmm64
	2,// VEX_Vcvtsd2si_r32_xmmm64
	2,// VEX_Vcvtsd2si_r64_xmmm64
	2,// EVEX_Vcvtsd2si_r32_xmmm64_er
	2,// EVEX_Vcvtsd2si_r64_xmmm64_er
	2,// Ucomiss_xmm_xmmm32
	2,// VEX_Vucomiss_xmm_xmmm32
	2,// EVEX_Vucomiss_xmm_xmmm32_sae
	2,// Ucomisd_xmm_xmmm64
	2,// VEX_Vucomisd_xmm_xmmm64
	2,// EVEX_Vucomisd_xmm_xmmm64_sae
	2,// Comiss_xmm_xmmm32
	2,// Comisd_xmm_xmmm64
	2,// VEX_Vcomiss_xmm_xmmm32
	2,// VEX_Vcomisd_xmm_xmmm64
	2,// EVEX_Vcomiss_xmm_xmmm32_sae
	2,// EVEX_Vcomisd_xmm_xmmm64_sae
	0,// Wrmsr
	0,// Rdtsc
	0,// Rdmsr
	0,// Rdpmc
	0,// Sysenter
	0,// Sysexitd
	0,// Sysexitq
	0,// Getsecd
	2,// Cmovo_r16_rm16
	2,// Cmovo_r32_rm32
	2,// Cmovo_r64_rm64
	2,// Cmovno_r16_rm16
	2,// Cmovno_r32_rm32
	2,// Cmovno_r64_rm64
	2,// Cmovb_r16_rm16
	2,// Cmovb_r32_rm32
	2,// Cmovb_r64_rm64
	2,// Cmovae_r16_rm16
	2,// Cmovae_r32_rm32
	2,// Cmovae_r64_rm64
	2,// Cmove_r16_rm16
	2,// Cmove_r32_rm32
	2,// Cmove_r64_rm64
	2,// Cmovne_r16_rm16
	2,// Cmovne_r32_rm32
	2,// Cmovne_r64_rm64
	2,// Cmovbe_r16_rm16
	2,// Cmovbe_r32_rm32
	2,// Cmovbe_r64_rm64
	2,// Cmova_r16_rm16
	2,// Cmova_r32_rm32
	2,// Cmova_r64_rm64
	2,// Cmovs_r16_rm16
	2,// Cmovs_r32_rm32
	2,// Cmovs_r64_rm64
	2,// Cmovns_r16_rm16
	2,// Cmovns_r32_rm32
	2,// Cmovns_r64_rm64
	2,// Cmovp_r16_rm16
	2,// Cmovp_r32_rm32
	2,// Cmovp_r64_rm64
	2,// Cmovnp_r16_rm16
	2,// Cmovnp_r32_rm32
	2,// Cmovnp_r64_rm64
	2,// Cmovl_r16_rm16
	2,// Cmovl_r32_rm32
	2,// Cmovl_r64_rm64
	2,// Cmovge_r16_rm16
	2,// Cmovge_r32_rm32
	2,// Cmovge_r64_rm64
	2,// Cmovle_r16_rm16
	2,// Cmovle_r32_rm32
	2,// Cmovle_r64_rm64
	2,// Cmovg_r16_rm16
	2,// Cmovg_r32_rm32
	2,// Cmovg_r64_rm64
	3,// VEX_Kandw_kr_kr_kr
	3,// VEX_Kandq_kr_kr_kr
	3,// VEX_Kandb_kr_kr_kr
	3,// VEX_Kandd_kr_kr_kr
	3,// VEX_Kandnw_kr_kr_kr
	3,// VEX_Kandnq_kr_kr_kr
	3,// VEX_Kandnb_kr_kr_kr
	3,// VEX_Kandnd_kr_kr_kr
	2,// VEX_Knotw_kr_kr
	2,// VEX_Knotq_kr_kr
	2,// VEX_Knotb_kr_kr
	2,// VEX_Knotd_kr_kr
	3,// VEX_Korw_kr_kr_kr
	3,// VEX_Korq_kr_kr_kr
	3,// VEX_Korb_kr_kr_kr
	3,// VEX_Kord_kr_kr_kr
	3,// VEX_Kxnorw_kr_kr_kr
	3,// VEX_Kxnorq_kr_kr_kr
	3,// VEX_Kxnorb_kr_kr_kr
	3,// VEX_Kxnord_kr_kr_kr
	3,// VEX_Kxorw_kr_kr_kr
	3,// VEX_Kxorq_kr_kr_kr
	3,// VEX_Kxorb_kr_kr_kr
	3,// VEX_Kxord_kr_kr_kr
	3,// VEX_Kaddw_kr_kr_kr
	3,// VEX_Kaddq_kr_kr_kr
	3,// VEX_Kaddb_kr_kr_kr
	3,// VEX_Kaddd_kr_kr_kr
	3,// VEX_Kunpckwd_kr_kr_kr
	3,// VEX_Kunpckdq_kr_kr_kr
	3,// VEX_Kunpckbw_kr_kr_kr
	2,// Movmskps_r32_xmm
	2,// Movmskps_r64_xmm
	2,// VEX_Vmovmskps_r32_xmm
	2,// VEX_Vmovmskps_r64_xmm
	2,// VEX_Vmovmskps_r32_ymm
	2,// VEX_Vmovmskps_r64_ymm
	2,// Movmskpd_r32_xmm
	2,// Movmskpd_r64_xmm
	2,// VEX_Vmovmskpd_r32_xmm
	2,// VEX_Vmovmskpd_r64_xmm
	2,// VEX_Vmovmskpd_r32_ymm
	2,// VEX_Vmovmskpd_r64_ymm
	2,// Sqrtps_xmm_xmmm128
	2,// VEX_Vsqrtps_xmm_xmmm128
	2,// VEX_Vsqrtps_ymm_ymmm256
	2,// EVEX_Vsqrtps_xmm_k1z_xmmm128b32
	2,// EVEX_Vsqrtps_ymm_k1z_ymmm256b32
	2,// EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er
	2,// Sqrtpd_xmm_xmmm128
	2,// VEX_Vsqrtpd_xmm_xmmm128
	2,// VEX_Vsqrtpd_ymm_ymmm256
	2,// EVEX_Vsqrtpd_xmm_k1z_xmmm128b64
	2,// EVEX_Vsqrtpd_ymm_k1z_ymmm256b64
	2,// EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er
	2,// Sqrtss_xmm_xmmm32
	3,// VEX_Vsqrtss_xmm_xmm_xmmm32
	3,// EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er
	2,// Sqrtsd_xmm_xmmm64
	3,// VEX_Vsqrtsd_xmm_xmm_xmmm64
	3,// EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er
	2,// Rsqrtps_xmm_xmmm128
	2,// VEX_Vrsqrtps_xmm_xmmm128
	2,// VEX_Vrsqrtps_ymm_ymmm256
	2,// Rsqrtss_xmm_xmmm32
	3,// VEX_Vrsqrtss_xmm_xmm_xmmm32
	2,// Rcpps_xmm_xmmm128
	2,// VEX_Vrcpps_xmm_xmmm128
	2,// VEX_Vrcpps_ymm_ymmm256
	2,// Rcpss_xmm_xmmm32
	3,// VEX_Vrcpss_xmm_xmm_xmmm32
	2,// Andps_xmm_xmmm128
	3,// VEX_Vandps_xmm_xmm_xmmm128
	3,// VEX_Vandps_ymm_ymm_ymmm256
	3,// EVEX_Vandps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vandps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vandps_zmm_k1z_zmm_zmmm512b32
	2,// Andpd_xmm_xmmm128
	3,// VEX_Vandpd_xmm_xmm_xmmm128
	3,// VEX_Vandpd_ymm_ymm_ymmm256
	3,// EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64
	2,// Andnps_xmm_xmmm128
	3,// VEX_Vandnps_xmm_xmm_xmmm128
	3,// VEX_Vandnps_ymm_ymm_ymmm256
	3,// EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32
	2,// Andnpd_xmm_xmmm128
	3,// VEX_Vandnpd_xmm_xmm_xmmm128
	3,// VEX_Vandnpd_ymm_ymm_ymmm256
	3,// EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64
	2,// Orps_xmm_xmmm128
	3,// VEX_Vorps_xmm_xmm_xmmm128
	3,// VEX_Vorps_ymm_ymm_ymmm256
	3,// EVEX_Vorps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vorps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vorps_zmm_k1z_zmm_zmmm512b32
	2,// Orpd_xmm_xmmm128
	3,// VEX_Vorpd_xmm_xmm_xmmm128
	3,// VEX_Vorpd_ymm_ymm_ymmm256
	3,// EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64
	2,// Xorps_xmm_xmmm128
	3,// VEX_Vxorps_xmm_xmm_xmmm128
	3,// VEX_Vxorps_ymm_ymm_ymmm256
	3,// EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32
	2,// Xorpd_xmm_xmmm128
	3,// VEX_Vxorpd_xmm_xmm_xmmm128
	3,// VEX_Vxorpd_ymm_ymm_ymmm256
	3,// EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64
	2,// Addps_xmm_xmmm128
	3,// VEX_Vaddps_xmm_xmm_xmmm128
	3,// VEX_Vaddps_ymm_ymm_ymmm256
	3,// EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er
	2,// Addpd_xmm_xmmm128
	3,// VEX_Vaddpd_xmm_xmm_xmmm128
	3,// VEX_Vaddpd_ymm_ymm_ymmm256
	3,// EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er
	2,// Addss_xmm_xmmm32
	3,// VEX_Vaddss_xmm_xmm_xmmm32
	3,// EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er
	2,// Addsd_xmm_xmmm64
	3,// VEX_Vaddsd_xmm_xmm_xmmm64
	3,// EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er
	2,// Mulps_xmm_xmmm128
	3,// VEX_Vmulps_xmm_xmm_xmmm128
	3,// VEX_Vmulps_ymm_ymm_ymmm256
	3,// EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er
	2,// Mulpd_xmm_xmmm128
	3,// VEX_Vmulpd_xmm_xmm_xmmm128
	3,// VEX_Vmulpd_ymm_ymm_ymmm256
	3,// EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er
	2,// Mulss_xmm_xmmm32
	3,// VEX_Vmulss_xmm_xmm_xmmm32
	3,// EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er
	2,// Mulsd_xmm_xmmm64
	3,// VEX_Vmulsd_xmm_xmm_xmmm64
	3,// EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er
	2,// Cvtps2pd_xmm_xmmm64
	2,// VEX_Vcvtps2pd_xmm_xmmm64
	2,// VEX_Vcvtps2pd_ymm_xmmm128
	2,// EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae
	2,// Cvtpd2ps_xmm_xmmm128
	2,// VEX_Vcvtpd2ps_xmm_xmmm128
	2,// VEX_Vcvtpd2ps_xmm_ymmm256
	2,// EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er
	2,// Cvtss2sd_xmm_xmmm32
	3,// VEX_Vcvtss2sd_xmm_xmm_xmmm32
	3,// EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae
	2,// Cvtsd2ss_xmm_xmmm64
	3,// VEX_Vcvtsd2ss_xmm_xmm_xmmm64
	3,// EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er
	2,// Cvtdq2ps_xmm_xmmm128
	2,// VEX_Vcvtdq2ps_xmm_xmmm128
	2,// VEX_Vcvtdq2ps_ymm_ymmm256
	2,// EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er
	2,// Cvtps2dq_xmm_xmmm128
	2,// VEX_Vcvtps2dq_xmm_xmmm128
	2,// VEX_Vcvtps2dq_ymm_ymmm256
	2,// EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er
	2,// Cvttps2dq_xmm_xmmm128
	2,// VEX_Vcvttps2dq_xmm_xmmm128
	2,// VEX_Vcvttps2dq_ymm_ymmm256
	2,// EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae
	2,// Subps_xmm_xmmm128
	3,// VEX_Vsubps_xmm_xmm_xmmm128
	3,// VEX_Vsubps_ymm_ymm_ymmm256
	3,// EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er
	2,// Subpd_xmm_xmmm128
	3,// VEX_Vsubpd_xmm_xmm_xmmm128
	3,// VEX_Vsubpd_ymm_ymm_ymmm256
	3,// EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er
	2,// Subss_xmm_xmmm32
	3,// VEX_Vsubss_xmm_xmm_xmmm32
	3,// EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er
	2,// Subsd_xmm_xmmm64
	3,// VEX_Vsubsd_xmm_xmm_xmmm64
	3,// EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er
	2,// Minps_xmm_xmmm128
	3,// VEX_Vminps_xmm_xmm_xmmm128
	3,// VEX_Vminps_ymm_ymm_ymmm256
	3,// EVEX_Vminps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vminps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae
	2,// Minpd_xmm_xmmm128
	3,// VEX_Vminpd_xmm_xmm_xmmm128
	3,// VEX_Vminpd_ymm_ymm_ymmm256
	3,// EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae
	2,// Minss_xmm_xmmm32
	3,// VEX_Vminss_xmm_xmm_xmmm32
	3,// EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae
	2,// Minsd_xmm_xmmm64
	3,// VEX_Vminsd_xmm_xmm_xmmm64
	3,// EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae
	2,// Divps_xmm_xmmm128
	3,// VEX_Vdivps_xmm_xmm_xmmm128
	3,// VEX_Vdivps_ymm_ymm_ymmm256
	3,// EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er
	2,// Divpd_xmm_xmmm128
	3,// VEX_Vdivpd_xmm_xmm_xmmm128
	3,// VEX_Vdivpd_ymm_ymm_ymmm256
	3,// EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er
	2,// Divss_xmm_xmmm32
	3,// VEX_Vdivss_xmm_xmm_xmmm32
	3,// EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er
	2,// Divsd_xmm_xmmm64
	3,// VEX_Vdivsd_xmm_xmm_xmmm64
	3,// EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er
	2,// Maxps_xmm_xmmm128
	3,// VEX_Vmaxps_xmm_xmm_xmmm128
	3,// VEX_Vmaxps_ymm_ymm_ymmm256
	3,// EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae
	2,// Maxpd_xmm_xmmm128
	3,// VEX_Vmaxpd_xmm_xmm_xmmm128
	3,// VEX_Vmaxpd_ymm_ymm_ymmm256
	3,// EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae
	2,// Maxss_xmm_xmmm32
	3,// VEX_Vmaxss_xmm_xmm_xmmm32
	3,// EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae
	2,// Maxsd_xmm_xmmm64
	3,// VEX_Vmaxsd_xmm_xmm_xmmm64
	3,// EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae
	2,// Punpcklbw_mm_mmm32
	2,// Punpcklbw_xmm_xmmm128
	3,// VEX_Vpunpcklbw_xmm_xmm_xmmm128
	3,// VEX_Vpunpcklbw_ymm_ymm_ymmm256
	3,// EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512
	2,// Punpcklwd_mm_mmm32
	2,// Punpcklwd_xmm_xmmm128
	3,// VEX_Vpunpcklwd_xmm_xmm_xmmm128
	3,// VEX_Vpunpcklwd_ymm_ymm_ymmm256
	3,// EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512
	2,// Punpckldq_mm_mmm32
	2,// Punpckldq_xmm_xmmm128
	3,// VEX_Vpunpckldq_xmm_xmm_xmmm128
	3,// VEX_Vpunpckldq_ymm_ymm_ymmm256
	3,// EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32
	2,// Packsswb_mm_mmm64
	2,// Packsswb_xmm_xmmm128
	3,// VEX_Vpacksswb_xmm_xmm_xmmm128
	3,// VEX_Vpacksswb_ymm_ymm_ymmm256
	3,// EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512
	2,// Pcmpgtb_mm_mmm64
	2,// Pcmpgtb_xmm_xmmm128
	3,// VEX_Vpcmpgtb_xmm_xmm_xmmm128
	3,// VEX_Vpcmpgtb_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128
	3,// EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256
	3,// EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512
	2,// Pcmpgtw_mm_mmm64
	2,// Pcmpgtw_xmm_xmmm128
	3,// VEX_Vpcmpgtw_xmm_xmm_xmmm128
	3,// VEX_Vpcmpgtw_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128
	3,// EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256
	3,// EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512
	2,// Pcmpgtd_mm_mmm64
	2,// Pcmpgtd_xmm_xmmm128
	3,// VEX_Vpcmpgtd_xmm_xmm_xmmm128
	3,// VEX_Vpcmpgtd_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32
	3,// EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32
	3,// EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32
	2,// Packuswb_mm_mmm64
	2,// Packuswb_xmm_xmmm128
	3,// VEX_Vpackuswb_xmm_xmm_xmmm128
	3,// VEX_Vpackuswb_ymm_ymm_ymmm256
	3,// EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512
	2,// Punpckhbw_mm_mmm64
	2,// Punpckhbw_xmm_xmmm128
	3,// VEX_Vpunpckhbw_xmm_xmm_xmmm128
	3,// VEX_Vpunpckhbw_ymm_ymm_ymmm256
	3,// EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512
	2,// Punpckhwd_mm_mmm64
	2,// Punpckhwd_xmm_xmmm128
	3,// VEX_Vpunpckhwd_xmm_xmm_xmmm128
	3,// VEX_Vpunpckhwd_ymm_ymm_ymmm256
	3,// EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512
	2,// Punpckhdq_mm_mmm64
	2,// Punpckhdq_xmm_xmmm128
	3,// VEX_Vpunpckhdq_xmm_xmm_xmmm128
	3,// VEX_Vpunpckhdq_ymm_ymm_ymmm256
	3,// EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32
	2,// Packssdw_mm_mmm64
	2,// Packssdw_xmm_xmmm128
	3,// VEX_Vpackssdw_xmm_xmm_xmmm128
	3,// VEX_Vpackssdw_ymm_ymm_ymmm256
	3,// EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32
	2,// Punpcklqdq_xmm_xmmm128
	3,// VEX_Vpunpcklqdq_xmm_xmm_xmmm128
	3,// VEX_Vpunpcklqdq_ymm_ymm_ymmm256
	3,// EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64
	2,// Punpckhqdq_xmm_xmmm128
	3,// VEX_Vpunpckhqdq_xmm_xmm_xmmm128
	3,// VEX_Vpunpckhqdq_ymm_ymm_ymmm256
	3,// EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64
	2,// Movd_mm_rm32
	2,// Movq_mm_rm64
	2,// Movd_xmm_rm32
	2,// Movq_xmm_rm64
	2,// VEX_Vmovd_xmm_rm32
	2,// VEX_Vmovq_xmm_rm64
	2,// EVEX_Vmovd_xmm_rm32
	2,// EVEX_Vmovq_xmm_rm64
	2,// Movq_mm_mmm64
	2,// Movdqa_xmm_xmmm128
	2,// VEX_Vmovdqa_xmm_xmmm128
	2,// VEX_Vmovdqa_ymm_ymmm256
	2,// EVEX_Vmovdqa32_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqa32_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqa32_zmm_k1z_zmmm512
	2,// EVEX_Vmovdqa64_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqa64_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqa64_zmm_k1z_zmmm512
	2,// Movdqu_xmm_xmmm128
	2,// VEX_Vmovdqu_xmm_xmmm128
	2,// VEX_Vmovdqu_ymm_ymmm256
	2,// EVEX_Vmovdqu32_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqu32_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqu32_zmm_k1z_zmmm512
	2,// EVEX_Vmovdqu64_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqu64_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqu64_zmm_k1z_zmmm512
	2,// EVEX_Vmovdqu8_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqu8_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqu8_zmm_k1z_zmmm512
	2,// EVEX_Vmovdqu16_xmm_k1z_xmmm128
	2,// EVEX_Vmovdqu16_ymm_k1z_ymmm256
	2,// EVEX_Vmovdqu16_zmm_k1z_zmmm512
	3,// Pshufw_mm_mmm64_imm8
	3,// Pshufd_xmm_xmmm128_imm8
	3,// VEX_Vpshufd_xmm_xmmm128_imm8
	3,// VEX_Vpshufd_ymm_ymmm256_imm8
	3,// EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8
	3,// Pshufhw_xmm_xmmm128_imm8
	3,// VEX_Vpshufhw_xmm_xmmm128_imm8
	3,// VEX_Vpshufhw_ymm_ymmm256_imm8
	3,// EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8
	3,// EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8
	3,// EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8
	3,// Pshuflw_xmm_xmmm128_imm8
	3,// VEX_Vpshuflw_xmm_xmmm128_imm8
	3,// VEX_Vpshuflw_ymm_ymmm256_imm8
	3,// EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8
	3,// EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8
	3,// EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8
	2,// Psrlw_mm_imm8
	2,// Psrlw_xmm_imm8
	3,// VEX_Vpsrlw_xmm_xmm_imm8
	3,// VEX_Vpsrlw_ymm_ymm_imm8
	3,// EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8
	3,// EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8
	3,// EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8
	2,// Psraw_mm_imm8
	2,// Psraw_xmm_imm8
	3,// VEX_Vpsraw_xmm_xmm_imm8
	3,// VEX_Vpsraw_ymm_ymm_imm8
	3,// EVEX_Vpsraw_xmm_k1z_xmmm128_imm8
	3,// EVEX_Vpsraw_ymm_k1z_ymmm256_imm8
	3,// EVEX_Vpsraw_zmm_k1z_zmmm512_imm8
	2,// Psllw_mm_imm8
	2,// Psllw_xmm_imm8
	3,// VEX_Vpsllw_xmm_xmm_imm8
	3,// VEX_Vpsllw_ymm_ymm_imm8
	3,// EVEX_Vpsllw_xmm_k1z_xmmm128_imm8
	3,// EVEX_Vpsllw_ymm_k1z_ymmm256_imm8
	3,// EVEX_Vpsllw_zmm_k1z_zmmm512_imm8
	3,// EVEX_Vprord_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vprord_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vprord_zmm_k1z_zmmm512b32_imm8
	3,// EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8
	3,// EVEX_Vprold_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vprold_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vprold_zmm_k1z_zmmm512b32_imm8
	3,// EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8
	2,// Psrld_mm_imm8
	2,// Psrld_xmm_imm8
	3,// VEX_Vpsrld_xmm_xmm_imm8
	3,// VEX_Vpsrld_ymm_ymm_imm8
	3,// EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8
	2,// Psrad_mm_imm8
	2,// Psrad_xmm_imm8
	3,// VEX_Vpsrad_xmm_xmm_imm8
	3,// VEX_Vpsrad_ymm_ymm_imm8
	3,// EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8
	3,// EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8
	2,// Pslld_mm_imm8
	2,// Pslld_xmm_imm8
	3,// VEX_Vpslld_xmm_xmm_imm8
	3,// VEX_Vpslld_ymm_ymm_imm8
	3,// EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8
	2,// Psrlq_mm_imm8
	2,// Psrlq_xmm_imm8
	3,// VEX_Vpsrlq_xmm_xmm_imm8
	3,// VEX_Vpsrlq_ymm_ymm_imm8
	3,// EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8
	2,// Psrldq_xmm_imm8
	3,// VEX_Vpsrldq_xmm_xmm_imm8
	3,// VEX_Vpsrldq_ymm_ymm_imm8
	3,// EVEX_Vpsrldq_xmm_xmmm128_imm8
	3,// EVEX_Vpsrldq_ymm_ymmm256_imm8
	3,// EVEX_Vpsrldq_zmm_zmmm512_imm8
	2,// Psllq_mm_imm8
	2,// Psllq_xmm_imm8
	3,// VEX_Vpsllq_xmm_xmm_imm8
	3,// VEX_Vpsllq_ymm_ymm_imm8
	3,// EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8
	2,// Pslldq_xmm_imm8
	3,// VEX_Vpslldq_xmm_xmm_imm8
	3,// VEX_Vpslldq_ymm_ymm_imm8
	3,// EVEX_Vpslldq_xmm_xmmm128_imm8
	3,// EVEX_Vpslldq_ymm_ymmm256_imm8
	3,// EVEX_Vpslldq_zmm_zmmm512_imm8
	2,// Pcmpeqb_mm_mmm64
	2,// Pcmpeqb_xmm_xmmm128
	3,// VEX_Vpcmpeqb_xmm_xmm_xmmm128
	3,// VEX_Vpcmpeqb_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128
	3,// EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256
	3,// EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512
	2,// Pcmpeqw_mm_mmm64
	2,// Pcmpeqw_xmm_xmmm128
	3,// VEX_Vpcmpeqw_xmm_xmm_xmmm128
	3,// VEX_Vpcmpeqw_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128
	3,// EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256
	3,// EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512
	2,// Pcmpeqd_mm_mmm64
	2,// Pcmpeqd_xmm_xmmm128
	3,// VEX_Vpcmpeqd_xmm_xmm_xmmm128
	3,// VEX_Vpcmpeqd_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32
	3,// EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32
	3,// EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32
	0,// Emms
	0,// VEX_Vzeroupper
	0,// VEX_Vzeroall
	2,// Vmread_rm32_r32
	2,// Vmread_rm64_r64
	2,// EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae
	2,// EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae
	3,// Extrq_xmm_imm8_imm8
	2,// EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae
	2,// EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae
	2,// EVEX_Vcvttss2usi_r32_xmmm32_sae
	2,// EVEX_Vcvttss2usi_r64_xmmm32_sae
	4,// Insertq_xmm_xmm_imm8_imm8
	2,// EVEX_Vcvttsd2usi_r32_xmmm64_sae
	2,// EVEX_Vcvttsd2usi_r64_xmmm64_sae
	2,// Vmwrite_r32_rm32
	2,// Vmwrite_r64_rm64
	2,// EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er
	2,// Extrq_xmm_xmm
	2,// EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er
	2,// EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er
	2,// EVEX_Vcvtss2usi_r32_xmmm32_er
	2,// EVEX_Vcvtss2usi_r64_xmmm32_er
	2,// Insertq_xmm_xmm
	2,// EVEX_Vcvtsd2usi_r32_xmmm64_er
	2,// EVEX_Vcvtsd2usi_r64_xmmm64_er
	2,// EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae
	2,// EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae
	2,// EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er
	2,// EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er
	2,// EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32
	2,// EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er
	2,// EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er
	2,// EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er
	3,// EVEX_Vcvtusi2ss_xmm_xmm_rm32_er
	3,// EVEX_Vcvtusi2ss_xmm_xmm_rm64_er
	3,// EVEX_Vcvtusi2sd_xmm_xmm_rm32_er
	3,// EVEX_Vcvtusi2sd_xmm_xmm_rm64_er
	2,// Haddpd_xmm_xmmm128
	3,// VEX_Vhaddpd_xmm_xmm_xmmm128
	3,// VEX_Vhaddpd_ymm_ymm_ymmm256
	2,// Haddps_xmm_xmmm128
	3,// VEX_Vhaddps_xmm_xmm_xmmm128
	3,// VEX_Vhaddps_ymm_ymm_ymmm256
	2,// Hsubpd_xmm_xmmm128
	3,// VEX_Vhsubpd_xmm_xmm_xmmm128
	3,// VEX_Vhsubpd_ymm_ymm_ymmm256
	2,// Hsubps_xmm_xmmm128
	3,// VEX_Vhsubps_xmm_xmm_xmmm128
	3,// VEX_Vhsubps_ymm_ymm_ymmm256
	2,// Movd_rm32_mm
	2,// Movq_rm64_mm
	2,// Movd_rm32_xmm
	2,// Movq_rm64_xmm
	2,// VEX_Vmovd_rm32_xmm
	2,// VEX_Vmovq_rm64_xmm
	2,// EVEX_Vmovd_rm32_xmm
	2,// EVEX_Vmovq_rm64_xmm
	2,// Movq_xmm_xmmm64
	2,// VEX_Vmovq_xmm_xmmm64
	2,// EVEX_Vmovq_xmm_xmmm64
	2,// Movq_mmm64_mm
	2,// Movdqa_xmmm128_xmm
	2,// VEX_Vmovdqa_xmmm128_xmm
	2,// VEX_Vmovdqa_ymmm256_ymm
	2,// EVEX_Vmovdqa32_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqa32_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqa32_zmmm512_k1z_zmm
	2,// EVEX_Vmovdqa64_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqa64_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqa64_zmmm512_k1z_zmm
	2,// Movdqu_xmmm128_xmm
	2,// VEX_Vmovdqu_xmmm128_xmm
	2,// VEX_Vmovdqu_ymmm256_ymm
	2,// EVEX_Vmovdqu32_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqu32_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqu32_zmmm512_k1z_zmm
	2,// EVEX_Vmovdqu64_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqu64_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqu64_zmmm512_k1z_zmm
	2,// EVEX_Vmovdqu8_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqu8_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqu8_zmmm512_k1z_zmm
	2,// EVEX_Vmovdqu16_xmmm128_k1z_xmm
	2,// EVEX_Vmovdqu16_ymmm256_k1z_ymm
	2,// EVEX_Vmovdqu16_zmmm512_k1z_zmm
	1,// Jo_rel16
	1,// Jo_rel32_32
	1,// Jo_rel32_64
	1,// Jno_rel16
	1,// Jno_rel32_32
	1,// Jno_rel32_64
	1,// Jb_rel16
	1,// Jb_rel32_32
	1,// Jb_rel32_64
	1,// Jae_rel16
	1,// Jae_rel32_32
	1,// Jae_rel32_64
	1,// Je_rel16
	1,// Je_rel32_32
	1,// Je_rel32_64
	1,// Jne_rel16
	1,// Jne_rel32_32
	1,// Jne_rel32_64
	1,// Jbe_rel16
	1,// Jbe_rel32_32
	1,// Jbe_rel32_64
	1,// Ja_rel16
	1,// Ja_rel32_32
	1,// Ja_rel32_64
	1,// Js_rel16
	1,// Js_rel32_32
	1,// Js_rel32_64
	1,// Jns_rel16
	1,// Jns_rel32_32
	1,// Jns_rel32_64
	1,// Jp_rel16
	1,// Jp_rel32_32
	1,// Jp_rel32_64
	1,// Jnp_rel16
	1,// Jnp_rel32_32
	1,// Jnp_rel32_64
	1,// Jl_rel16
	1,// Jl_rel32_32
	1,// Jl_rel32_64
	1,// Jge_rel16
	1,// Jge_rel32_32
	1,// Jge_rel32_64
	1,// Jle_rel16
	1,// Jle_rel32_32
	1,// Jle_rel32_64
	1,// Jg_rel16
	1,// Jg_rel32_32
	1,// Jg_rel32_64
	1,// Seto_rm8
	1,// Setno_rm8
	1,// Setb_rm8
	1,// Setae_rm8
	1,// Sete_rm8
	1,// Setne_rm8
	1,// Setbe_rm8
	1,// Seta_rm8
	1,// Sets_rm8
	1,// Setns_rm8
	1,// Setp_rm8
	1,// Setnp_rm8
	1,// Setl_rm8
	1,// Setge_rm8
	1,// Setle_rm8
	1,// Setg_rm8
	2,// VEX_Kmovw_kr_km16
	2,// VEX_Kmovq_kr_km64
	2,// VEX_Kmovb_kr_km8
	2,// VEX_Kmovd_kr_km32
	2,// VEX_Kmovw_m16_kr
	2,// VEX_Kmovq_m64_kr
	2,// VEX_Kmovb_m8_kr
	2,// VEX_Kmovd_m32_kr
	2,// VEX_Kmovw_kr_r32
	2,// VEX_Kmovb_kr_r32
	2,// VEX_Kmovd_kr_r32
	2,// VEX_Kmovq_kr_r64
	2,// VEX_Kmovw_r32_kr
	2,// VEX_Kmovb_r32_kr
	2,// VEX_Kmovd_r32_kr
	2,// VEX_Kmovq_r64_kr
	2,// VEX_Kortestw_kr_kr
	2,// VEX_Kortestq_kr_kr
	2,// VEX_Kortestb_kr_kr
	2,// VEX_Kortestd_kr_kr
	2,// VEX_Ktestw_kr_kr
	2,// VEX_Ktestq_kr_kr
	2,// VEX_Ktestb_kr_kr
	2,// VEX_Ktestd_kr_kr
	1,// Pushw_FS
	1,// Pushd_FS
	1,// Pushq_FS
	1,// Popw_FS
	1,// Popd_FS
	1,// Popq_FS
	0,// Cpuid
	2,// Bt_rm16_r16
	2,// Bt_rm32_r32
	2,// Bt_rm64_r64
	3,// Shld_rm16_r16_imm8
	3,// Shld_rm32_r32_imm8
	3,// Shld_rm64_r64_imm8
	3,// Shld_rm16_r16_CL
	3,// Shld_rm32_r32_CL
	3,// Shld_rm64_r64_CL
	0,// Montmul_16
	0,// Montmul_32
	0,// Montmul_64
	0,// Xsha1_16
	0,// Xsha1_32
	0,// Xsha1_64
	0,// Xsha256_16
	0,// Xsha256_32
	0,// Xsha256_64
	2,// Xbts_r16_rm16
	2,// Xbts_r32_rm32
	0,// Xstore_16
	0,// Xstore_32
	0,// Xstore_64
	0,// Xcryptecb_16
	0,// Xcryptecb_32
	0,// Xcryptecb_64
	0,// Xcryptcbc_16
	0,// Xcryptcbc_32
	0,// Xcryptcbc_64
	0,// Xcryptctr_16
	0,// Xcryptctr_32
	0,// Xcryptctr_64
	0,// Xcryptcfb_16
	0,// Xcryptcfb_32
	0,// Xcryptcfb_64
	0,// Xcryptofb_16
	0,// Xcryptofb_32
	0,// Xcryptofb_64
	2,// Ibts_rm16_r16
	2,// Ibts_rm32_r32
	2,// Cmpxchg486_rm8_r8
	2,// Cmpxchg486_rm16_r16
	2,// Cmpxchg486_rm32_r32
	1,// Pushw_GS
	1,// Pushd_GS
	1,// Pushq_GS
	1,// Popw_GS
	1,// Popd_GS
	1,// Popq_GS
	0,// Rsm
	2,// Bts_rm16_r16
	2,// Bts_rm32_r32
	2,// Bts_rm64_r64
	3,// Shrd_rm16_r16_imm8
	3,// Shrd_rm32_r32_imm8
	3,// Shrd_rm64_r64_imm8
	3,// Shrd_rm16_r16_CL
	3,// Shrd_rm32_r32_CL
	3,// Shrd_rm64_r64_CL
	1,// Fxsave_m512byte
	1,// Fxsave64_m512byte
	1,// Rdfsbase_r32
	1,// Rdfsbase_r64
	1,// Fxrstor_m512byte
	1,// Fxrstor64_m512byte
	1,// Rdgsbase_r32
	1,// Rdgsbase_r64
	1,// Ldmxcsr_m32
	1,// Wrfsbase_r32
	1,// Wrfsbase_r64
	1,// VEX_Vldmxcsr_m32
	1,// Stmxcsr_m32
	1,// Wrgsbase_r32
	1,// Wrgsbase_r64
	1,// VEX_Vstmxcsr_m32
	1,// Xsave_mem
	1,// Xsave64_mem
	1,// Ptwrite_rm32
	1,// Ptwrite_rm64
	1,// Xrstor_mem
	1,// Xrstor64_mem
	1,// Incsspd_r32
	1,// Incsspq_r64
	1,// Xsaveopt_mem
	1,// Xsaveopt64_mem
	1,// Clwb_m8
	1,// Tpause_r32
	1,// Tpause_r64
	1,// Clrssbsy_m64
	1,// Umonitor_r16
	1,// Umonitor_r32
	1,// Umonitor_r64
	1,// Umwait_r32
	1,// Umwait_r64
	1,// Clflush_m8
	1,// Clflushopt_m8
	0,// Lfence
	0,// Lfence_E9
	0,// Lfence_EA
	0,// Lfence_EB
	0,// Lfence_EC
	0,// Lfence_ED
	0,// Lfence_EE
	0,// Lfence_EF
	0,// Mfence
	0,// Mfence_F1
	0,// Mfence_F2
	0,// Mfence_F3
	0,// Mfence_F4
	0,// Mfence_F5
	0,// Mfence_F6
	0,// Mfence_F7
	0,// Sfence
	0,// Sfence_F9
	0,// Sfence_FA
	0,// Sfence_FB
	0,// Sfence_FC
	0,// Sfence_FD
	0,// Sfence_FE
	0,// Sfence_FF
	0,// Pcommit
	2,// Imul_r16_rm16
	2,// Imul_r32_rm32
	2,// Imul_r64_rm64
	2,// Cmpxchg_rm8_r8
	2,// Cmpxchg_rm16_r16
	2,// Cmpxchg_rm32_r32
	2,// Cmpxchg_rm64_r64
	2,// Lss_r16_m1616
	2,// Lss_r32_m1632
	2,// Lss_r64_m1664
	2,// Btr_rm16_r16
	2,// Btr_rm32_r32
	2,// Btr_rm64_r64
	2,// Lfs_r16_m1616
	2,// Lfs_r32_m1632
	2,// Lfs_r64_m1664
	2,// Lgs_r16_m1616
	2,// Lgs_r32_m1632
	2,// Lgs_r64_m1664
	2,// Movzx_r16_rm8
	2,// Movzx_r32_rm8
	2,// Movzx_r64_rm8
	2,// Movzx_r16_rm16
	2,// Movzx_r32_rm16
	2,// Movzx_r64_rm16
	1,// Jmpe_disp16
	1,// Jmpe_disp32
	2,// Popcnt_r16_rm16
	2,// Popcnt_r32_rm32
	2,// Popcnt_r64_rm64
	2,// Ud1_r16_rm16
	2,// Ud1_r32_rm32
	2,// Ud1_r64_rm64
	2,// Bt_rm16_imm8
	2,// Bt_rm32_imm8
	2,// Bt_rm64_imm8
	2,// Bts_rm16_imm8
	2,// Bts_rm32_imm8
	2,// Bts_rm64_imm8
	2,// Btr_rm16_imm8
	2,// Btr_rm32_imm8
	2,// Btr_rm64_imm8
	2,// Btc_rm16_imm8
	2,// Btc_rm32_imm8
	2,// Btc_rm64_imm8
	2,// Btc_rm16_r16
	2,// Btc_rm32_r32
	2,// Btc_rm64_r64
	2,// Bsf_r16_rm16
	2,// Bsf_r32_rm32
	2,// Bsf_r64_rm64
	2,// Tzcnt_r16_rm16
	2,// Tzcnt_r32_rm32
	2,// Tzcnt_r64_rm64
	2,// Bsr_r16_rm16
	2,// Bsr_r32_rm32
	2,// Bsr_r64_rm64
	2,// Lzcnt_r16_rm16
	2,// Lzcnt_r32_rm32
	2,// Lzcnt_r64_rm64
	2,// Movsx_r16_rm8
	2,// Movsx_r32_rm8
	2,// Movsx_r64_rm8
	2,// Movsx_r16_rm16
	2,// Movsx_r32_rm16
	2,// Movsx_r64_rm16
	2,// Xadd_rm8_r8
	2,// Xadd_rm16_r16
	2,// Xadd_rm32_r32
	2,// Xadd_rm64_r64
	3,// Cmpps_xmm_xmmm128_imm8
	4,// VEX_Vcmpps_xmm_xmm_xmmm128_imm8
	4,// VEX_Vcmpps_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8
	4,// EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8
	4,// EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae
	3,// Cmppd_xmm_xmmm128_imm8
	4,// VEX_Vcmppd_xmm_xmm_xmmm128_imm8
	4,// VEX_Vcmppd_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8
	4,// EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8
	4,// EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae
	3,// Cmpss_xmm_xmmm32_imm8
	4,// VEX_Vcmpss_xmm_xmm_xmmm32_imm8
	4,// EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae
	3,// Cmpsd_xmm_xmmm64_imm8
	4,// VEX_Vcmpsd_xmm_xmm_xmmm64_imm8
	4,// EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae
	2,// Movnti_m32_r32
	2,// Movnti_m64_r64
	3,// Pinsrw_mm_r32m16_imm8
	3,// Pinsrw_mm_r64m16_imm8
	3,// Pinsrw_xmm_r32m16_imm8
	3,// Pinsrw_xmm_r64m16_imm8
	4,// VEX_Vpinsrw_xmm_xmm_r32m16_imm8
	4,// VEX_Vpinsrw_xmm_xmm_r64m16_imm8
	4,// EVEX_Vpinsrw_xmm_xmm_r32m16_imm8
	4,// EVEX_Vpinsrw_xmm_xmm_r64m16_imm8
	3,// Pextrw_r32_mm_imm8
	3,// Pextrw_r64_mm_imm8
	3,// Pextrw_r32_xmm_imm8
	3,// Pextrw_r64_xmm_imm8
	3,// VEX_Vpextrw_r32_xmm_imm8
	3,// VEX_Vpextrw_r64_xmm_imm8
	3,// EVEX_Vpextrw_r32_xmm_imm8
	3,// EVEX_Vpextrw_r64_xmm_imm8
	3,// Shufps_xmm_xmmm128_imm8
	4,// VEX_Vshufps_xmm_xmm_xmmm128_imm8
	4,// VEX_Vshufps_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8
	3,// Shufpd_xmm_xmmm128_imm8
	4,// VEX_Vshufpd_xmm_xmm_xmmm128_imm8
	4,// VEX_Vshufpd_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8
	1,// Cmpxchg8b_m64
	1,// Cmpxchg16b_m128
	1,// Xrstors_mem
	1,// Xrstors64_mem
	1,// Xsavec_mem
	1,// Xsavec64_mem
	1,// Xsaves_mem
	1,// Xsaves64_mem
	1,// Vmptrld_m64
	1,// Vmclear_m64
	1,// Vmxon_m64
	1,// Rdrand_r16
	1,// Rdrand_r32
	1,// Rdrand_r64
	1,// Vmptrst_m64
	1,// Rdseed_r16
	1,// Rdseed_r32
	1,// Rdseed_r64
	1,// Rdpid_r32
	1,// Rdpid_r64
	1,// Bswap_r16
	1,// Bswap_r32
	1,// Bswap_r64
	2,// Addsubpd_xmm_xmmm128
	3,// VEX_Vaddsubpd_xmm_xmm_xmmm128
	3,// VEX_Vaddsubpd_ymm_ymm_ymmm256
	2,// Addsubps_xmm_xmmm128
	3,// VEX_Vaddsubps_xmm_xmm_xmmm128
	3,// VEX_Vaddsubps_ymm_ymm_ymmm256
	2,// Psrlw_mm_mmm64
	2,// Psrlw_xmm_xmmm128
	3,// VEX_Vpsrlw_xmm_xmm_xmmm128
	3,// VEX_Vpsrlw_ymm_ymm_xmmm128
	3,// EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128
	2,// Psrld_mm_mmm64
	2,// Psrld_xmm_xmmm128
	3,// VEX_Vpsrld_xmm_xmm_xmmm128
	3,// VEX_Vpsrld_ymm_ymm_xmmm128
	3,// EVEX_Vpsrld_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsrld_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsrld_zmm_k1z_zmm_xmmm128
	2,// Psrlq_mm_mmm64
	2,// Psrlq_xmm_xmmm128
	3,// VEX_Vpsrlq_xmm_xmm_xmmm128
	3,// VEX_Vpsrlq_ymm_ymm_xmmm128
	3,// EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128
	2,// Paddq_mm_mmm64
	2,// Paddq_xmm_xmmm128
	3,// VEX_Vpaddq_xmm_xmm_xmmm128
	3,// VEX_Vpaddq_ymm_ymm_ymmm256
	3,// EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64
	2,// Pmullw_mm_mmm64
	2,// Pmullw_xmm_xmmm128
	3,// VEX_Vpmullw_xmm_xmm_xmmm128
	3,// VEX_Vpmullw_ymm_ymm_ymmm256
	3,// EVEX_Vpmullw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmullw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmullw_zmm_k1z_zmm_zmmm512
	2,// Movq_xmmm64_xmm
	2,// VEX_Vmovq_xmmm64_xmm
	2,// EVEX_Vmovq_xmmm64_xmm
	2,// Movq2dq_xmm_mm
	2,// Movdq2q_mm_xmm
	2,// Pmovmskb_r32_mm
	2,// Pmovmskb_r64_mm
	2,// Pmovmskb_r32_xmm
	2,// Pmovmskb_r64_xmm
	2,// VEX_Vpmovmskb_r32_xmm
	2,// VEX_Vpmovmskb_r64_xmm
	2,// VEX_Vpmovmskb_r32_ymm
	2,// VEX_Vpmovmskb_r64_ymm
	2,// Psubusb_mm_mmm64
	2,// Psubusb_xmm_xmmm128
	3,// VEX_Vpsubusb_xmm_xmm_xmmm128
	3,// VEX_Vpsubusb_ymm_ymm_ymmm256
	3,// EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512
	2,// Psubusw_mm_mmm64
	2,// Psubusw_xmm_xmmm128
	3,// VEX_Vpsubusw_xmm_xmm_xmmm128
	3,// VEX_Vpsubusw_ymm_ymm_ymmm256
	3,// EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512
	2,// Pminub_mm_mmm64
	2,// Pminub_xmm_xmmm128
	3,// VEX_Vpminub_xmm_xmm_xmmm128
	3,// VEX_Vpminub_ymm_ymm_ymmm256
	3,// EVEX_Vpminub_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpminub_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpminub_zmm_k1z_zmm_zmmm512
	2,// Pand_mm_mmm64
	2,// Pand_xmm_xmmm128
	3,// VEX_Vpand_xmm_xmm_xmmm128
	3,// VEX_Vpand_ymm_ymm_ymmm256
	3,// EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64
	2,// Paddusb_mm_mmm64
	2,// Paddusb_xmm_xmmm128
	3,// VEX_Vpaddusb_xmm_xmm_xmmm128
	3,// VEX_Vpaddusb_ymm_ymm_ymmm256
	3,// EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512
	2,// Paddusw_mm_mmm64
	2,// Paddusw_xmm_xmmm128
	3,// VEX_Vpaddusw_xmm_xmm_xmmm128
	3,// VEX_Vpaddusw_ymm_ymm_ymmm256
	3,// EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512
	2,// Pmaxub_mm_mmm64
	2,// Pmaxub_xmm_xmmm128
	3,// VEX_Vpmaxub_xmm_xmm_xmmm128
	3,// VEX_Vpmaxub_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512
	2,// Pandn_mm_mmm64
	2,// Pandn_xmm_xmmm128
	3,// VEX_Vpandn_xmm_xmm_xmmm128
	3,// VEX_Vpandn_ymm_ymm_ymmm256
	3,// EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64
	2,// Pavgb_mm_mmm64
	2,// Pavgb_xmm_xmmm128
	3,// VEX_Vpavgb_xmm_xmm_xmmm128
	3,// VEX_Vpavgb_ymm_ymm_ymmm256
	3,// EVEX_Vpavgb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpavgb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpavgb_zmm_k1z_zmm_zmmm512
	2,// Psraw_mm_mmm64
	2,// Psraw_xmm_xmmm128
	3,// VEX_Vpsraw_xmm_xmm_xmmm128
	3,// VEX_Vpsraw_ymm_ymm_xmmm128
	3,// EVEX_Vpsraw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsraw_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsraw_zmm_k1z_zmm_xmmm128
	2,// Psrad_mm_mmm64
	2,// Psrad_xmm_xmmm128
	3,// VEX_Vpsrad_xmm_xmm_xmmm128
	3,// VEX_Vpsrad_ymm_ymm_xmmm128
	3,// EVEX_Vpsrad_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsrad_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsrad_zmm_k1z_zmm_xmmm128
	3,// EVEX_Vpsraq_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsraq_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsraq_zmm_k1z_zmm_xmmm128
	2,// Pavgw_mm_mmm64
	2,// Pavgw_xmm_xmmm128
	3,// VEX_Vpavgw_xmm_xmm_xmmm128
	3,// VEX_Vpavgw_ymm_ymm_ymmm256
	3,// EVEX_Vpavgw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpavgw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpavgw_zmm_k1z_zmm_zmmm512
	2,// Pmulhuw_mm_mmm64
	2,// Pmulhuw_xmm_xmmm128
	3,// VEX_Vpmulhuw_xmm_xmm_xmmm128
	3,// VEX_Vpmulhuw_ymm_ymm_ymmm256
	3,// EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512
	2,// Pmulhw_mm_mmm64
	2,// Pmulhw_xmm_xmmm128
	3,// VEX_Vpmulhw_xmm_xmm_xmmm128
	3,// VEX_Vpmulhw_ymm_ymm_ymmm256
	3,// EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512
	2,// Cvttpd2dq_xmm_xmmm128
	2,// VEX_Vcvttpd2dq_xmm_xmmm128
	2,// VEX_Vcvttpd2dq_xmm_ymmm256
	2,// EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae
	2,// Cvtdq2pd_xmm_xmmm64
	2,// VEX_Vcvtdq2pd_xmm_xmmm64
	2,// VEX_Vcvtdq2pd_ymm_xmmm128
	2,// EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32
	2,// EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32
	2,// EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er
	2,// EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64
	2,// EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er
	2,// Cvtpd2dq_xmm_xmmm128
	2,// VEX_Vcvtpd2dq_xmm_xmmm128
	2,// VEX_Vcvtpd2dq_xmm_ymmm256
	2,// EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er
	2,// Movntq_m64_mm
	2,// Movntdq_m128_xmm
	2,// VEX_Vmovntdq_m128_xmm
	2,// VEX_Vmovntdq_m256_ymm
	2,// EVEX_Vmovntdq_m128_xmm
	2,// EVEX_Vmovntdq_m256_ymm
	2,// EVEX_Vmovntdq_m512_zmm
	2,// Psubsb_mm_mmm64
	2,// Psubsb_xmm_xmmm128
	3,// VEX_Vpsubsb_xmm_xmm_xmmm128
	3,// VEX_Vpsubsb_ymm_ymm_ymmm256
	3,// EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512
	2,// Psubsw_mm_mmm64
	2,// Psubsw_xmm_xmmm128
	3,// VEX_Vpsubsw_xmm_xmm_xmmm128
	3,// VEX_Vpsubsw_ymm_ymm_ymmm256
	3,// EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512
	2,// Pminsw_mm_mmm64
	2,// Pminsw_xmm_xmmm128
	3,// VEX_Vpminsw_xmm_xmm_xmmm128
	3,// VEX_Vpminsw_ymm_ymm_ymmm256
	3,// EVEX_Vpminsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpminsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpminsw_zmm_k1z_zmm_zmmm512
	2,// Por_mm_mmm64
	2,// Por_xmm_xmmm128
	3,// VEX_Vpor_xmm_xmm_xmmm128
	3,// VEX_Vpor_ymm_ymm_ymmm256
	3,// EVEX_Vpord_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpord_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpord_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vporq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vporq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vporq_zmm_k1z_zmm_zmmm512b64
	2,// Paddsb_mm_mmm64
	2,// Paddsb_xmm_xmmm128
	3,// VEX_Vpaddsb_xmm_xmm_xmmm128
	3,// VEX_Vpaddsb_ymm_ymm_ymmm256
	3,// EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512
	2,// Paddsw_mm_mmm64
	2,// Paddsw_xmm_xmmm128
	3,// VEX_Vpaddsw_xmm_xmm_xmmm128
	3,// VEX_Vpaddsw_ymm_ymm_ymmm256
	3,// EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512
	2,// Pmaxsw_mm_mmm64
	2,// Pmaxsw_xmm_xmmm128
	3,// VEX_Vpmaxsw_xmm_xmm_xmmm128
	3,// VEX_Vpmaxsw_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512
	2,// Pxor_mm_mmm64
	2,// Pxor_xmm_xmmm128
	3,// VEX_Vpxor_xmm_xmm_xmmm128
	3,// VEX_Vpxor_ymm_ymm_ymmm256
	3,// EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64
	2,// Lddqu_xmm_m128
	2,// VEX_Vlddqu_xmm_m128
	2,// VEX_Vlddqu_ymm_m256
	2,// Psllw_mm_mmm64
	2,// Psllw_xmm_xmmm128
	3,// VEX_Vpsllw_xmm_xmm_xmmm128
	3,// VEX_Vpsllw_ymm_ymm_xmmm128
	3,// EVEX_Vpsllw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsllw_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsllw_zmm_k1z_zmm_xmmm128
	2,// Pslld_mm_mmm64
	2,// Pslld_xmm_xmmm128
	3,// VEX_Vpslld_xmm_xmm_xmmm128
	3,// VEX_Vpslld_ymm_ymm_xmmm128
	3,// EVEX_Vpslld_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpslld_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpslld_zmm_k1z_zmm_xmmm128
	2,// Psllq_mm_mmm64
	2,// Psllq_xmm_xmmm128
	3,// VEX_Vpsllq_xmm_xmm_xmmm128
	3,// VEX_Vpsllq_ymm_ymm_xmmm128
	3,// EVEX_Vpsllq_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsllq_ymm_k1z_ymm_xmmm128
	3,// EVEX_Vpsllq_zmm_k1z_zmm_xmmm128
	2,// Pmuludq_mm_mmm64
	2,// Pmuludq_xmm_xmmm128
	3,// VEX_Vpmuludq_xmm_xmm_xmmm128
	3,// VEX_Vpmuludq_ymm_ymm_ymmm256
	3,// EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64
	2,// Pmaddwd_mm_mmm64
	2,// Pmaddwd_xmm_xmmm128
	3,// VEX_Vpmaddwd_xmm_xmm_xmmm128
	3,// VEX_Vpmaddwd_ymm_ymm_ymmm256
	3,// EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512
	2,// Psadbw_mm_mmm64
	2,// Psadbw_xmm_xmmm128
	3,// VEX_Vpsadbw_xmm_xmm_xmmm128
	3,// VEX_Vpsadbw_ymm_ymm_ymmm256
	3,// EVEX_Vpsadbw_xmm_xmm_xmmm128
	3,// EVEX_Vpsadbw_ymm_ymm_ymmm256
	3,// EVEX_Vpsadbw_zmm_zmm_zmmm512
	3,// Maskmovq_rDI_mm_mm
	3,// Maskmovdqu_rDI_xmm_xmm
	3,// VEX_Vmaskmovdqu_rDI_xmm_xmm
	2,// Psubb_mm_mmm64
	2,// Psubb_xmm_xmmm128
	3,// VEX_Vpsubb_xmm_xmm_xmmm128
	3,// VEX_Vpsubb_ymm_ymm_ymmm256
	3,// EVEX_Vpsubb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubb_zmm_k1z_zmm_zmmm512
	2,// Psubw_mm_mmm64
	2,// Psubw_xmm_xmmm128
	3,// VEX_Vpsubw_xmm_xmm_xmmm128
	3,// VEX_Vpsubw_ymm_ymm_ymmm256
	3,// EVEX_Vpsubw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsubw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsubw_zmm_k1z_zmm_zmmm512
	2,// Psubd_mm_mmm64
	2,// Psubd_xmm_xmmm128
	3,// VEX_Vpsubd_xmm_xmm_xmmm128
	3,// VEX_Vpsubd_ymm_ymm_ymmm256
	3,// EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32
	2,// Psubq_mm_mmm64
	2,// Psubq_xmm_xmmm128
	3,// VEX_Vpsubq_xmm_xmm_xmmm128
	3,// VEX_Vpsubq_ymm_ymm_ymmm256
	3,// EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64
	2,// Paddb_mm_mmm64
	2,// Paddb_xmm_xmmm128
	3,// VEX_Vpaddb_xmm_xmm_xmmm128
	3,// VEX_Vpaddb_ymm_ymm_ymmm256
	3,// EVEX_Vpaddb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddb_zmm_k1z_zmm_zmmm512
	2,// Paddw_mm_mmm64
	2,// Paddw_xmm_xmmm128
	3,// VEX_Vpaddw_xmm_xmm_xmmm128
	3,// VEX_Vpaddw_ymm_ymm_ymmm256
	3,// EVEX_Vpaddw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpaddw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpaddw_zmm_k1z_zmm_zmmm512
	2,// Paddd_mm_mmm64
	2,// Paddd_xmm_xmmm128
	3,// VEX_Vpaddd_xmm_xmm_xmmm128
	3,// VEX_Vpaddd_ymm_ymm_ymmm256
	3,// EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32
	2,// Ud0_r16_rm16
	2,// Ud0_r32_rm32
	2,// Ud0_r64_rm64
	2,// Pshufb_mm_mmm64
	2,// Pshufb_xmm_xmmm128
	3,// VEX_Vpshufb_xmm_xmm_xmmm128
	3,// VEX_Vpshufb_ymm_ymm_ymmm256
	3,// EVEX_Vpshufb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpshufb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpshufb_zmm_k1z_zmm_zmmm512
	2,// Phaddw_mm_mmm64
	2,// Phaddw_xmm_xmmm128
	3,// VEX_Vphaddw_xmm_xmm_xmmm128
	3,// VEX_Vphaddw_ymm_ymm_ymmm256
	2,// Phaddd_mm_mmm64
	2,// Phaddd_xmm_xmmm128
	3,// VEX_Vphaddd_xmm_xmm_xmmm128
	3,// VEX_Vphaddd_ymm_ymm_ymmm256
	2,// Phaddsw_mm_mmm64
	2,// Phaddsw_xmm_xmmm128
	3,// VEX_Vphaddsw_xmm_xmm_xmmm128
	3,// VEX_Vphaddsw_ymm_ymm_ymmm256
	2,// Pmaddubsw_mm_mmm64
	2,// Pmaddubsw_xmm_xmmm128
	3,// VEX_Vpmaddubsw_xmm_xmm_xmmm128
	3,// VEX_Vpmaddubsw_ymm_ymm_ymmm256
	3,// EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512
	2,// Phsubw_mm_mmm64
	2,// Phsubw_xmm_xmmm128
	3,// VEX_Vphsubw_xmm_xmm_xmmm128
	3,// VEX_Vphsubw_ymm_ymm_ymmm256
	2,// Phsubd_mm_mmm64
	2,// Phsubd_xmm_xmmm128
	3,// VEX_Vphsubd_xmm_xmm_xmmm128
	3,// VEX_Vphsubd_ymm_ymm_ymmm256
	2,// Phsubsw_mm_mmm64
	2,// Phsubsw_xmm_xmmm128
	3,// VEX_Vphsubsw_xmm_xmm_xmmm128
	3,// VEX_Vphsubsw_ymm_ymm_ymmm256
	2,// Psignb_mm_mmm64
	2,// Psignb_xmm_xmmm128
	3,// VEX_Vpsignb_xmm_xmm_xmmm128
	3,// VEX_Vpsignb_ymm_ymm_ymmm256
	2,// Psignw_mm_mmm64
	2,// Psignw_xmm_xmmm128
	3,// VEX_Vpsignw_xmm_xmm_xmmm128
	3,// VEX_Vpsignw_ymm_ymm_ymmm256
	2,// Psignd_mm_mmm64
	2,// Psignd_xmm_xmmm128
	3,// VEX_Vpsignd_xmm_xmm_xmmm128
	3,// VEX_Vpsignd_ymm_ymm_ymmm256
	2,// Pmulhrsw_mm_mmm64
	2,// Pmulhrsw_xmm_xmmm128
	3,// VEX_Vpmulhrsw_xmm_xmm_xmmm128
	3,// VEX_Vpmulhrsw_ymm_ymm_ymmm256
	3,// EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512
	3,// VEX_Vpermilps_xmm_xmm_xmmm128
	3,// VEX_Vpermilps_ymm_ymm_ymmm256
	3,// EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32
	3,// VEX_Vpermilpd_xmm_xmm_xmmm128
	3,// VEX_Vpermilpd_ymm_ymm_ymmm256
	3,// EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64
	2,// VEX_Vtestps_xmm_xmmm128
	2,// VEX_Vtestps_ymm_ymmm256
	2,// VEX_Vtestpd_xmm_xmmm128
	2,// VEX_Vtestpd_ymm_ymmm256
	2,// Pblendvb_xmm_xmmm128
	3,// EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vpmovuswb_xmmm64_k1z_xmm
	2,// EVEX_Vpmovuswb_xmmm128_k1z_ymm
	2,// EVEX_Vpmovuswb_ymmm256_k1z_zmm
	3,// EVEX_Vpsravw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsravw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsravw_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vpmovusdb_xmmm32_k1z_xmm
	2,// EVEX_Vpmovusdb_xmmm64_k1z_ymm
	2,// EVEX_Vpmovusdb_xmmm128_k1z_zmm
	3,// EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vpmovusqb_xmmm16_k1z_xmm
	2,// EVEX_Vpmovusqb_xmmm32_k1z_ymm
	2,// EVEX_Vpmovusqb_xmmm64_k1z_zmm
	2,// VEX_Vcvtph2ps_xmm_xmmm64
	2,// VEX_Vcvtph2ps_ymm_xmmm128
	2,// EVEX_Vcvtph2ps_xmm_k1z_xmmm64
	2,// EVEX_Vcvtph2ps_ymm_k1z_xmmm128
	2,// EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae
	2,// EVEX_Vpmovusdw_xmmm64_k1z_xmm
	2,// EVEX_Vpmovusdw_xmmm128_k1z_ymm
	2,// EVEX_Vpmovusdw_ymmm256_k1z_zmm
	2,// Blendvps_xmm_xmmm128
	3,// EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vpmovusqw_xmmm32_k1z_xmm
	2,// EVEX_Vpmovusqw_xmmm64_k1z_ymm
	2,// EVEX_Vpmovusqw_xmmm128_k1z_zmm
	2,// Blendvpd_xmm_xmmm128
	3,// EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vpmovusqd_xmmm64_k1z_xmm
	2,// EVEX_Vpmovusqd_xmmm128_k1z_ymm
	2,// EVEX_Vpmovusqd_ymmm256_k1z_zmm
	3,// VEX_Vpermps_ymm_ymm_ymmm256
	3,// EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64
	2,// Ptest_xmm_xmmm128
	2,// VEX_Vptest_xmm_xmmm128
	2,// VEX_Vptest_ymm_ymmm256
	2,// VEX_Vbroadcastss_xmm_m32
	2,// VEX_Vbroadcastss_ymm_m32
	2,// EVEX_Vbroadcastss_xmm_k1z_xmmm32
	2,// EVEX_Vbroadcastss_ymm_k1z_xmmm32
	2,// EVEX_Vbroadcastss_zmm_k1z_xmmm32
	2,// VEX_Vbroadcastsd_ymm_m64
	2,// EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64
	2,// EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64
	2,// EVEX_Vbroadcastsd_ymm_k1z_xmmm64
	2,// EVEX_Vbroadcastsd_zmm_k1z_xmmm64
	2,// VEX_Vbroadcastf128_ymm_m128
	2,// EVEX_Vbroadcastf32x4_ymm_k1z_m128
	2,// EVEX_Vbroadcastf32x4_zmm_k1z_m128
	2,// EVEX_Vbroadcastf64x2_ymm_k1z_m128
	2,// EVEX_Vbroadcastf64x2_zmm_k1z_m128
	2,// EVEX_Vbroadcastf32x8_zmm_k1z_m256
	2,// EVEX_Vbroadcastf64x4_zmm_k1z_m256
	2,// Pabsb_mm_mmm64
	2,// Pabsb_xmm_xmmm128
	2,// VEX_Vpabsb_xmm_xmmm128
	2,// VEX_Vpabsb_ymm_ymmm256
	2,// EVEX_Vpabsb_xmm_k1z_xmmm128
	2,// EVEX_Vpabsb_ymm_k1z_ymmm256
	2,// EVEX_Vpabsb_zmm_k1z_zmmm512
	2,// Pabsw_mm_mmm64
	2,// Pabsw_xmm_xmmm128
	2,// VEX_Vpabsw_xmm_xmmm128
	2,// VEX_Vpabsw_ymm_ymmm256
	2,// EVEX_Vpabsw_xmm_k1z_xmmm128
	2,// EVEX_Vpabsw_ymm_k1z_ymmm256
	2,// EVEX_Vpabsw_zmm_k1z_zmmm512
	2,// Pabsd_mm_mmm64
	2,// Pabsd_xmm_xmmm128
	2,// VEX_Vpabsd_xmm_xmmm128
	2,// VEX_Vpabsd_ymm_ymmm256
	2,// EVEX_Vpabsd_xmm_k1z_xmmm128b32
	2,// EVEX_Vpabsd_ymm_k1z_ymmm256b32
	2,// EVEX_Vpabsd_zmm_k1z_zmmm512b32
	2,// EVEX_Vpabsq_xmm_k1z_xmmm128b64
	2,// EVEX_Vpabsq_ymm_k1z_ymmm256b64
	2,// EVEX_Vpabsq_zmm_k1z_zmmm512b64
	2,// Pmovsxbw_xmm_xmmm64
	2,// VEX_Vpmovsxbw_xmm_xmmm64
	2,// VEX_Vpmovsxbw_ymm_xmmm128
	2,// EVEX_Vpmovsxbw_xmm_k1z_xmmm64
	2,// EVEX_Vpmovsxbw_ymm_k1z_xmmm128
	2,// EVEX_Vpmovsxbw_zmm_k1z_ymmm256
	2,// EVEX_Vpmovswb_xmmm64_k1z_xmm
	2,// EVEX_Vpmovswb_xmmm128_k1z_ymm
	2,// EVEX_Vpmovswb_ymmm256_k1z_zmm
	2,// Pmovsxbd_xmm_xmmm32
	2,// VEX_Vpmovsxbd_xmm_xmmm32
	2,// VEX_Vpmovsxbd_ymm_xmmm64
	2,// EVEX_Vpmovsxbd_xmm_k1z_xmmm32
	2,// EVEX_Vpmovsxbd_ymm_k1z_xmmm64
	2,// EVEX_Vpmovsxbd_zmm_k1z_xmmm128
	2,// EVEX_Vpmovsdb_xmmm32_k1z_xmm
	2,// EVEX_Vpmovsdb_xmmm64_k1z_ymm
	2,// EVEX_Vpmovsdb_xmmm128_k1z_zmm
	2,// Pmovsxbq_xmm_xmmm16
	2,// VEX_Vpmovsxbq_xmm_xmmm16
	2,// VEX_Vpmovsxbq_ymm_xmmm32
	2,// EVEX_Vpmovsxbq_xmm_k1z_xmmm16
	2,// EVEX_Vpmovsxbq_ymm_k1z_xmmm32
	2,// EVEX_Vpmovsxbq_zmm_k1z_xmmm64
	2,// EVEX_Vpmovsqb_xmmm16_k1z_xmm
	2,// EVEX_Vpmovsqb_xmmm32_k1z_ymm
	2,// EVEX_Vpmovsqb_xmmm64_k1z_zmm
	2,// Pmovsxwd_xmm_xmmm64
	2,// VEX_Vpmovsxwd_xmm_xmmm64
	2,// VEX_Vpmovsxwd_ymm_xmmm128
	2,// EVEX_Vpmovsxwd_xmm_k1z_xmmm64
	2,// EVEX_Vpmovsxwd_ymm_k1z_xmmm128
	2,// EVEX_Vpmovsxwd_zmm_k1z_ymmm256
	2,// EVEX_Vpmovsdw_xmmm64_k1z_xmm
	2,// EVEX_Vpmovsdw_xmmm128_k1z_ymm
	2,// EVEX_Vpmovsdw_ymmm256_k1z_zmm
	2,// Pmovsxwq_xmm_xmmm32
	2,// VEX_Vpmovsxwq_xmm_xmmm32
	2,// VEX_Vpmovsxwq_ymm_xmmm64
	2,// EVEX_Vpmovsxwq_xmm_k1z_xmmm32
	2,// EVEX_Vpmovsxwq_ymm_k1z_xmmm64
	2,// EVEX_Vpmovsxwq_zmm_k1z_xmmm128
	2,// EVEX_Vpmovsqw_xmmm32_k1z_xmm
	2,// EVEX_Vpmovsqw_xmmm64_k1z_ymm
	2,// EVEX_Vpmovsqw_xmmm128_k1z_zmm
	2,// Pmovsxdq_xmm_xmmm64
	2,// VEX_Vpmovsxdq_xmm_xmmm64
	2,// VEX_Vpmovsxdq_ymm_xmmm128
	2,// EVEX_Vpmovsxdq_xmm_k1z_xmmm64
	2,// EVEX_Vpmovsxdq_ymm_k1z_xmmm128
	2,// EVEX_Vpmovsxdq_zmm_k1z_ymmm256
	2,// EVEX_Vpmovsqd_xmmm64_k1z_xmm
	2,// EVEX_Vpmovsqd_xmmm128_k1z_ymm
	2,// EVEX_Vpmovsqd_ymmm256_k1z_zmm
	3,// EVEX_Vptestmb_kr_k1_xmm_xmmm128
	3,// EVEX_Vptestmb_kr_k1_ymm_ymmm256
	3,// EVEX_Vptestmb_kr_k1_zmm_zmmm512
	3,// EVEX_Vptestmw_kr_k1_xmm_xmmm128
	3,// EVEX_Vptestmw_kr_k1_ymm_ymmm256
	3,// EVEX_Vptestmw_kr_k1_zmm_zmmm512
	3,// EVEX_Vptestnmb_kr_k1_xmm_xmmm128
	3,// EVEX_Vptestnmb_kr_k1_ymm_ymmm256
	3,// EVEX_Vptestnmb_kr_k1_zmm_zmmm512
	3,// EVEX_Vptestnmw_kr_k1_xmm_xmmm128
	3,// EVEX_Vptestnmw_kr_k1_ymm_ymmm256
	3,// EVEX_Vptestnmw_kr_k1_zmm_zmmm512
	3,// EVEX_Vptestmd_kr_k1_xmm_xmmm128b32
	3,// EVEX_Vptestmd_kr_k1_ymm_ymmm256b32
	3,// EVEX_Vptestmd_kr_k1_zmm_zmmm512b32
	3,// EVEX_Vptestmq_kr_k1_xmm_xmmm128b64
	3,// EVEX_Vptestmq_kr_k1_ymm_ymmm256b64
	3,// EVEX_Vptestmq_kr_k1_zmm_zmmm512b64
	3,// EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32
	3,// EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32
	3,// EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32
	3,// EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64
	3,// EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64
	3,// EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64
	2,// Pmuldq_xmm_xmmm128
	3,// VEX_Vpmuldq_xmm_xmm_xmmm128
	3,// VEX_Vpmuldq_ymm_ymm_ymmm256
	3,// EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vpmovm2b_xmm_kr
	2,// EVEX_Vpmovm2b_ymm_kr
	2,// EVEX_Vpmovm2b_zmm_kr
	2,// EVEX_Vpmovm2w_xmm_kr
	2,// EVEX_Vpmovm2w_ymm_kr
	2,// EVEX_Vpmovm2w_zmm_kr
	2,// Pcmpeqq_xmm_xmmm128
	3,// VEX_Vpcmpeqq_xmm_xmm_xmmm128
	3,// VEX_Vpcmpeqq_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64
	3,// EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64
	3,// EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64
	2,// EVEX_Vpmovb2m_kr_xmm
	2,// EVEX_Vpmovb2m_kr_ymm
	2,// EVEX_Vpmovb2m_kr_zmm
	2,// EVEX_Vpmovw2m_kr_xmm
	2,// EVEX_Vpmovw2m_kr_ymm
	2,// EVEX_Vpmovw2m_kr_zmm
	2,// Movntdqa_xmm_m128
	2,// VEX_Vmovntdqa_xmm_m128
	2,// VEX_Vmovntdqa_ymm_m256
	2,// EVEX_Vmovntdqa_xmm_m128
	2,// EVEX_Vmovntdqa_ymm_m256
	2,// EVEX_Vmovntdqa_zmm_m512
	2,// EVEX_Vpbroadcastmb2q_xmm_kr
	2,// EVEX_Vpbroadcastmb2q_ymm_kr
	2,// EVEX_Vpbroadcastmb2q_zmm_kr
	2,// Packusdw_xmm_xmmm128
	3,// VEX_Vpackusdw_xmm_xmm_xmmm128
	3,// VEX_Vpackusdw_ymm_ymm_ymmm256
	3,// EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32
	3,// VEX_Vmaskmovps_xmm_xmm_m128
	3,// VEX_Vmaskmovps_ymm_ymm_m256
	3,// EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vmaskmovpd_xmm_xmm_m128
	3,// VEX_Vmaskmovpd_ymm_ymm_m256
	3,// EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vmaskmovps_m128_xmm_xmm
	3,// VEX_Vmaskmovps_m256_ymm_ymm
	3,// VEX_Vmaskmovpd_m128_xmm_xmm
	3,// VEX_Vmaskmovpd_m256_ymm_ymm
	2,// Pmovzxbw_xmm_xmmm64
	2,// VEX_Vpmovzxbw_xmm_xmmm64
	2,// VEX_Vpmovzxbw_ymm_xmmm128
	2,// EVEX_Vpmovzxbw_xmm_k1z_xmmm64
	2,// EVEX_Vpmovzxbw_ymm_k1z_xmmm128
	2,// EVEX_Vpmovzxbw_zmm_k1z_ymmm256
	2,// EVEX_Vpmovwb_xmmm64_k1z_xmm
	2,// EVEX_Vpmovwb_xmmm128_k1z_ymm
	2,// EVEX_Vpmovwb_ymmm256_k1z_zmm
	2,// Pmovzxbd_xmm_xmmm32
	2,// VEX_Vpmovzxbd_xmm_xmmm32
	2,// VEX_Vpmovzxbd_ymm_xmmm64
	2,// EVEX_Vpmovzxbd_xmm_k1z_xmmm32
	2,// EVEX_Vpmovzxbd_ymm_k1z_xmmm64
	2,// EVEX_Vpmovzxbd_zmm_k1z_xmmm128
	2,// EVEX_Vpmovdb_xmmm32_k1z_xmm
	2,// EVEX_Vpmovdb_xmmm64_k1z_ymm
	2,// EVEX_Vpmovdb_xmmm128_k1z_zmm
	2,// Pmovzxbq_xmm_xmmm16
	2,// VEX_Vpmovzxbq_xmm_xmmm16
	2,// VEX_Vpmovzxbq_ymm_xmmm32
	2,// EVEX_Vpmovzxbq_xmm_k1z_xmmm16
	2,// EVEX_Vpmovzxbq_ymm_k1z_xmmm32
	2,// EVEX_Vpmovzxbq_zmm_k1z_xmmm64
	2,// EVEX_Vpmovqb_xmmm16_k1z_xmm
	2,// EVEX_Vpmovqb_xmmm32_k1z_ymm
	2,// EVEX_Vpmovqb_xmmm64_k1z_zmm
	2,// Pmovzxwd_xmm_xmmm64
	2,// VEX_Vpmovzxwd_xmm_xmmm64
	2,// VEX_Vpmovzxwd_ymm_xmmm128
	2,// EVEX_Vpmovzxwd_xmm_k1z_xmmm64
	2,// EVEX_Vpmovzxwd_ymm_k1z_xmmm128
	2,// EVEX_Vpmovzxwd_zmm_k1z_ymmm256
	2,// EVEX_Vpmovdw_xmmm64_k1z_xmm
	2,// EVEX_Vpmovdw_xmmm128_k1z_ymm
	2,// EVEX_Vpmovdw_ymmm256_k1z_zmm
	2,// Pmovzxwq_xmm_xmmm32
	2,// VEX_Vpmovzxwq_xmm_xmmm32
	2,// VEX_Vpmovzxwq_ymm_xmmm64
	2,// EVEX_Vpmovzxwq_xmm_k1z_xmmm32
	2,// EVEX_Vpmovzxwq_ymm_k1z_xmmm64
	2,// EVEX_Vpmovzxwq_zmm_k1z_xmmm128
	2,// EVEX_Vpmovqw_xmmm32_k1z_xmm
	2,// EVEX_Vpmovqw_xmmm64_k1z_ymm
	2,// EVEX_Vpmovqw_xmmm128_k1z_zmm
	2,// Pmovzxdq_xmm_xmmm64
	2,// VEX_Vpmovzxdq_xmm_xmmm64
	2,// VEX_Vpmovzxdq_ymm_xmmm128
	2,// EVEX_Vpmovzxdq_xmm_k1z_xmmm64
	2,// EVEX_Vpmovzxdq_ymm_k1z_xmmm128
	2,// EVEX_Vpmovzxdq_zmm_k1z_ymmm256
	2,// EVEX_Vpmovqd_xmmm64_k1z_xmm
	2,// EVEX_Vpmovqd_xmmm128_k1z_ymm
	2,// EVEX_Vpmovqd_ymmm256_k1z_zmm
	3,// VEX_Vpermd_ymm_ymm_ymmm256
	3,// EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64
	2,// Pcmpgtq_xmm_xmmm128
	3,// VEX_Vpcmpgtq_xmm_xmm_xmmm128
	3,// VEX_Vpcmpgtq_ymm_ymm_ymmm256
	3,// EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64
	3,// EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64
	3,// EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64
	2,// Pminsb_xmm_xmmm128
	3,// VEX_Vpminsb_xmm_xmm_xmmm128
	3,// VEX_Vpminsb_ymm_ymm_ymmm256
	3,// EVEX_Vpminsb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpminsb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpminsb_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vpmovm2d_xmm_kr
	2,// EVEX_Vpmovm2d_ymm_kr
	2,// EVEX_Vpmovm2d_zmm_kr
	2,// EVEX_Vpmovm2q_xmm_kr
	2,// EVEX_Vpmovm2q_ymm_kr
	2,// EVEX_Vpmovm2q_zmm_kr
	2,// Pminsd_xmm_xmmm128
	3,// VEX_Vpminsd_xmm_xmm_xmmm128
	3,// VEX_Vpminsd_ymm_ymm_ymmm256
	3,// EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vpmovd2m_kr_xmm
	2,// EVEX_Vpmovd2m_kr_ymm
	2,// EVEX_Vpmovd2m_kr_zmm
	2,// EVEX_Vpmovq2m_kr_xmm
	2,// EVEX_Vpmovq2m_kr_ymm
	2,// EVEX_Vpmovq2m_kr_zmm
	2,// Pminuw_xmm_xmmm128
	3,// VEX_Vpminuw_xmm_xmm_xmmm128
	3,// VEX_Vpminuw_ymm_ymm_ymmm256
	3,// EVEX_Vpminuw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpminuw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpminuw_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vpbroadcastmw2d_xmm_kr
	2,// EVEX_Vpbroadcastmw2d_ymm_kr
	2,// EVEX_Vpbroadcastmw2d_zmm_kr
	2,// Pminud_xmm_xmmm128
	3,// VEX_Vpminud_xmm_xmm_xmmm128
	3,// VEX_Vpminud_ymm_ymm_ymmm256
	3,// EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64
	2,// Pmaxsb_xmm_xmmm128
	3,// VEX_Vpmaxsb_xmm_xmm_xmmm128
	3,// VEX_Vpmaxsb_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512
	2,// Pmaxsd_xmm_xmmm128
	3,// VEX_Vpmaxsd_xmm_xmm_xmmm128
	3,// VEX_Vpmaxsd_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64
	2,// Pmaxuw_xmm_xmmm128
	3,// VEX_Vpmaxuw_xmm_xmm_xmmm128
	3,// VEX_Vpmaxuw_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512
	2,// Pmaxud_xmm_xmmm128
	3,// VEX_Vpmaxud_xmm_xmm_xmmm128
	3,// VEX_Vpmaxud_ymm_ymm_ymmm256
	3,// EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64
	2,// Pmulld_xmm_xmmm128
	3,// VEX_Vpmulld_xmm_xmm_xmmm128
	3,// VEX_Vpmulld_ymm_ymm_ymmm256
	3,// EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64
	2,// Phminposuw_xmm_xmmm128
	2,// VEX_Vphminposuw_xmm_xmmm128
	2,// EVEX_Vgetexpps_xmm_k1z_xmmm128b32
	2,// EVEX_Vgetexpps_ymm_k1z_ymmm256b32
	2,// EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae
	2,// EVEX_Vgetexppd_xmm_k1z_xmmm128b64
	2,// EVEX_Vgetexppd_ymm_k1z_ymmm256b64
	2,// EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae
	3,// EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae
	3,// EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae
	2,// EVEX_Vplzcntd_xmm_k1z_xmmm128b32
	2,// EVEX_Vplzcntd_ymm_k1z_ymmm256b32
	2,// EVEX_Vplzcntd_zmm_k1z_zmmm512b32
	2,// EVEX_Vplzcntq_xmm_k1z_xmmm128b64
	2,// EVEX_Vplzcntq_ymm_k1z_ymmm256b64
	2,// EVEX_Vplzcntq_zmm_k1z_zmmm512b64
	3,// VEX_Vpsrlvd_xmm_xmm_xmmm128
	3,// VEX_Vpsrlvd_ymm_ymm_ymmm256
	3,// VEX_Vpsrlvq_xmm_xmm_xmmm128
	3,// VEX_Vpsrlvq_ymm_ymm_ymmm256
	3,// EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64
	3,// VEX_Vpsravd_xmm_xmm_xmmm128
	3,// VEX_Vpsravd_ymm_ymm_ymmm256
	3,// EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64
	3,// VEX_Vpsllvd_xmm_xmm_xmmm128
	3,// VEX_Vpsllvd_ymm_ymm_ymmm256
	3,// VEX_Vpsllvq_xmm_xmm_xmmm128
	3,// VEX_Vpsllvq_ymm_ymm_ymmm256
	3,// EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vrcp14ps_xmm_k1z_xmmm128b32
	2,// EVEX_Vrcp14ps_ymm_k1z_ymmm256b32
	2,// EVEX_Vrcp14ps_zmm_k1z_zmmm512b32
	2,// EVEX_Vrcp14pd_xmm_k1z_xmmm128b64
	2,// EVEX_Vrcp14pd_ymm_k1z_ymmm256b64
	2,// EVEX_Vrcp14pd_zmm_k1z_zmmm512b64
	3,// EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32
	3,// EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64
	2,// EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32
	2,// EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32
	2,// EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32
	2,// EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64
	2,// EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64
	2,// EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64
	3,// EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32
	3,// EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64
	3,// EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128
	3,// EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128
	2,// EVEX_Vpopcntb_xmm_k1z_xmmm128
	2,// EVEX_Vpopcntb_ymm_k1z_ymmm256
	2,// EVEX_Vpopcntb_zmm_k1z_zmmm512
	2,// EVEX_Vpopcntw_xmm_k1z_xmmm128
	2,// EVEX_Vpopcntw_ymm_k1z_ymmm256
	2,// EVEX_Vpopcntw_zmm_k1z_zmmm512
	2,// EVEX_Vpopcntd_xmm_k1z_xmmm128b32
	2,// EVEX_Vpopcntd_ymm_k1z_ymmm256b32
	2,// EVEX_Vpopcntd_zmm_k1z_zmmm512b32
	2,// EVEX_Vpopcntq_xmm_k1z_xmmm128b64
	2,// EVEX_Vpopcntq_ymm_k1z_ymmm256b64
	2,// EVEX_Vpopcntq_zmm_k1z_zmmm512b64
	2,// VEX_Vpbroadcastd_xmm_xmmm32
	2,// VEX_Vpbroadcastd_ymm_xmmm32
	2,// EVEX_Vpbroadcastd_xmm_k1z_xmmm32
	2,// EVEX_Vpbroadcastd_ymm_k1z_xmmm32
	2,// EVEX_Vpbroadcastd_zmm_k1z_xmmm32
	2,// VEX_Vpbroadcastq_xmm_xmmm64
	2,// VEX_Vpbroadcastq_ymm_xmmm64
	2,// EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64
	2,// EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64
	2,// EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64
	2,// EVEX_Vpbroadcastq_xmm_k1z_xmmm64
	2,// EVEX_Vpbroadcastq_ymm_k1z_xmmm64
	2,// EVEX_Vpbroadcastq_zmm_k1z_xmmm64
	2,// VEX_Vbroadcasti128_ymm_m128
	2,// EVEX_Vbroadcasti32x4_ymm_k1z_m128
	2,// EVEX_Vbroadcasti32x4_zmm_k1z_m128
	2,// EVEX_Vbroadcasti64x2_ymm_k1z_m128
	2,// EVEX_Vbroadcasti64x2_zmm_k1z_m128
	2,// EVEX_Vbroadcasti32x8_zmm_k1z_m256
	2,// EVEX_Vbroadcasti64x4_zmm_k1z_m256
	2,// EVEX_Vpexpandb_xmm_k1z_xmmm128
	2,// EVEX_Vpexpandb_ymm_k1z_ymmm256
	2,// EVEX_Vpexpandb_zmm_k1z_zmmm512
	2,// EVEX_Vpexpandw_xmm_k1z_xmmm128
	2,// EVEX_Vpexpandw_ymm_k1z_ymmm256
	2,// EVEX_Vpexpandw_zmm_k1z_zmmm512
	2,// EVEX_Vpcompressb_xmmm128_k1z_xmm
	2,// EVEX_Vpcompressb_ymmm256_k1z_ymm
	2,// EVEX_Vpcompressb_zmmm512_k1z_zmm
	2,// EVEX_Vpcompressw_xmmm128_k1z_xmm
	2,// EVEX_Vpcompressw_ymmm256_k1z_ymm
	2,// EVEX_Vpcompressw_zmmm512_k1z_zmm
	3,// EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vp2intersectd_kp1_xmm_xmmm128b32
	3,// EVEX_Vp2intersectd_kp1_ymm_ymmm256b32
	3,// EVEX_Vp2intersectd_kp1_zmm_zmmm512b32
	3,// EVEX_Vp2intersectq_kp1_xmm_xmmm128b64
	3,// EVEX_Vp2intersectq_kp1_ymm_ymmm256b64
	3,// EVEX_Vp2intersectq_kp1_zmm_zmmm512b64
	3,// EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512
	2,// EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32
	2,// EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32
	3,// EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64
	2,// VEX_Vpbroadcastb_xmm_xmmm8
	2,// VEX_Vpbroadcastb_ymm_xmmm8
	2,// EVEX_Vpbroadcastb_xmm_k1z_xmmm8
	2,// EVEX_Vpbroadcastb_ymm_k1z_xmmm8
	2,// EVEX_Vpbroadcastb_zmm_k1z_xmmm8
	2,// VEX_Vpbroadcastw_xmm_xmmm16
	2,// VEX_Vpbroadcastw_ymm_xmmm16
	2,// EVEX_Vpbroadcastw_xmm_k1z_xmmm16
	2,// EVEX_Vpbroadcastw_ymm_k1z_xmmm16
	2,// EVEX_Vpbroadcastw_zmm_k1z_xmmm16
	2,// EVEX_Vpbroadcastb_xmm_k1z_r32
	2,// EVEX_Vpbroadcastb_ymm_k1z_r32
	2,// EVEX_Vpbroadcastb_zmm_k1z_r32
	2,// EVEX_Vpbroadcastw_xmm_k1z_r32
	2,// EVEX_Vpbroadcastw_ymm_k1z_r32
	2,// EVEX_Vpbroadcastw_zmm_k1z_r32
	2,// EVEX_Vpbroadcastd_xmm_k1z_r32
	2,// EVEX_Vpbroadcastd_ymm_k1z_r32
	2,// EVEX_Vpbroadcastd_zmm_k1z_r32
	2,// EVEX_Vpbroadcastq_xmm_k1z_r64
	2,// EVEX_Vpbroadcastq_ymm_k1z_r64
	2,// EVEX_Vpbroadcastq_zmm_k1z_r64
	3,// EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32
	3,// EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64
	2,// Invept_r32_m128
	2,// Invept_r64_m128
	2,// Invvpid_r32_m128
	2,// Invvpid_r64_m128
	2,// Invpcid_r32_m128
	2,// Invpcid_r64_m128
	3,// EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64
	2,// EVEX_Vexpandps_xmm_k1z_xmmm128
	2,// EVEX_Vexpandps_ymm_k1z_ymmm256
	2,// EVEX_Vexpandps_zmm_k1z_zmmm512
	2,// EVEX_Vexpandpd_xmm_k1z_xmmm128
	2,// EVEX_Vexpandpd_ymm_k1z_ymmm256
	2,// EVEX_Vexpandpd_zmm_k1z_zmmm512
	2,// EVEX_Vpexpandd_xmm_k1z_xmmm128
	2,// EVEX_Vpexpandd_ymm_k1z_ymmm256
	2,// EVEX_Vpexpandd_zmm_k1z_zmmm512
	2,// EVEX_Vpexpandq_xmm_k1z_xmmm128
	2,// EVEX_Vpexpandq_ymm_k1z_ymmm256
	2,// EVEX_Vpexpandq_zmm_k1z_zmmm512
	2,// EVEX_Vcompressps_xmmm128_k1z_xmm
	2,// EVEX_Vcompressps_ymmm256_k1z_ymm
	2,// EVEX_Vcompressps_zmmm512_k1z_zmm
	2,// EVEX_Vcompresspd_xmmm128_k1z_xmm
	2,// EVEX_Vcompresspd_ymmm256_k1z_ymm
	2,// EVEX_Vcompresspd_zmmm512_k1z_zmm
	2,// EVEX_Vpcompressd_xmmm128_k1z_xmm
	2,// EVEX_Vpcompressd_ymmm256_k1z_ymm
	2,// EVEX_Vpcompressd_zmmm512_k1z_zmm
	2,// EVEX_Vpcompressq_xmmm128_k1z_xmm
	2,// EVEX_Vpcompressq_ymmm256_k1z_ymm
	2,// EVEX_Vpcompressq_zmmm512_k1z_zmm
	3,// VEX_Vpmaskmovd_xmm_xmm_m128
	3,// VEX_Vpmaskmovd_ymm_ymm_m256
	3,// VEX_Vpmaskmovq_xmm_xmm_m128
	3,// VEX_Vpmaskmovq_ymm_ymm_m256
	3,// EVEX_Vpermb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermb_zmm_k1z_zmm_zmmm512
	3,// EVEX_Vpermw_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vpermw_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vpermw_zmm_k1z_zmm_zmmm512
	3,// VEX_Vpmaskmovd_m128_xmm_xmm
	3,// VEX_Vpmaskmovd_m256_ymm_ymm
	3,// VEX_Vpmaskmovq_m128_xmm_xmm
	3,// VEX_Vpmaskmovq_m256_ymm_ymm
	3,// EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128
	3,// EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256
	3,// EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512
	3,// VEX_Vpgatherdd_xmm_vm32x_xmm
	3,// VEX_Vpgatherdd_ymm_vm32y_ymm
	3,// VEX_Vpgatherdq_xmm_vm32x_xmm
	3,// VEX_Vpgatherdq_ymm_vm32x_ymm
	2,// EVEX_Vpgatherdd_xmm_k1_vm32x
	2,// EVEX_Vpgatherdd_ymm_k1_vm32y
	2,// EVEX_Vpgatherdd_zmm_k1_vm32z
	2,// EVEX_Vpgatherdq_xmm_k1_vm32x
	2,// EVEX_Vpgatherdq_ymm_k1_vm32x
	2,// EVEX_Vpgatherdq_zmm_k1_vm32y
	3,// VEX_Vpgatherqd_xmm_vm64x_xmm
	3,// VEX_Vpgatherqd_xmm_vm64y_xmm
	3,// VEX_Vpgatherqq_xmm_vm64x_xmm
	3,// VEX_Vpgatherqq_ymm_vm64y_ymm
	2,// EVEX_Vpgatherqd_xmm_k1_vm64x
	2,// EVEX_Vpgatherqd_xmm_k1_vm64y
	2,// EVEX_Vpgatherqd_ymm_k1_vm64z
	2,// EVEX_Vpgatherqq_xmm_k1_vm64x
	2,// EVEX_Vpgatherqq_ymm_k1_vm64y
	2,// EVEX_Vpgatherqq_zmm_k1_vm64z
	3,// VEX_Vgatherdps_xmm_vm32x_xmm
	3,// VEX_Vgatherdps_ymm_vm32y_ymm
	3,// VEX_Vgatherdpd_xmm_vm32x_xmm
	3,// VEX_Vgatherdpd_ymm_vm32x_ymm
	2,// EVEX_Vgatherdps_xmm_k1_vm32x
	2,// EVEX_Vgatherdps_ymm_k1_vm32y
	2,// EVEX_Vgatherdps_zmm_k1_vm32z
	2,// EVEX_Vgatherdpd_xmm_k1_vm32x
	2,// EVEX_Vgatherdpd_ymm_k1_vm32x
	2,// EVEX_Vgatherdpd_zmm_k1_vm32y
	3,// VEX_Vgatherqps_xmm_vm64x_xmm
	3,// VEX_Vgatherqps_xmm_vm64y_xmm
	3,// VEX_Vgatherqpd_xmm_vm64x_xmm
	3,// VEX_Vgatherqpd_ymm_vm64y_ymm
	2,// EVEX_Vgatherqps_xmm_k1_vm64x
	2,// EVEX_Vgatherqps_xmm_k1_vm64y
	2,// EVEX_Vgatherqps_ymm_k1_vm64z
	2,// EVEX_Vgatherqpd_xmm_k1_vm64x
	2,// EVEX_Vgatherqpd_ymm_k1_vm64y
	2,// EVEX_Vgatherqpd_zmm_k1_vm64z
	3,// VEX_Vfmaddsub132ps_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub132ps_ymm_ymm_ymmm256
	3,// VEX_Vfmaddsub132pd_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmsubadd132ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd132ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsubadd132pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd132ps_xmm_xmm_xmmm128
	3,// VEX_Vfmadd132ps_ymm_ymm_ymmm256
	3,// VEX_Vfmadd132pd_xmm_xmm_xmmm128
	3,// VEX_Vfmadd132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd132ss_xmm_xmm_xmmm32
	3,// VEX_Vfmadd132sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfmsub132ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsub132ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsub132pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsub132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// EVEX_V4fmaddps_zmm_k1z_zmmp3_m128
	3,// VEX_Vfmsub132ss_xmm_xmm_xmmm32
	3,// VEX_Vfmsub132sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er
	3,// EVEX_V4fmaddss_xmm_k1z_xmmp3_m128
	3,// VEX_Vfnmadd132ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd132ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmadd132pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmadd132ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmadd132sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfnmsub132ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub132ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmsub132pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub132pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmsub132ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmsub132sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er
	2,// EVEX_Vpscatterdd_vm32x_k1_xmm
	2,// EVEX_Vpscatterdd_vm32y_k1_ymm
	2,// EVEX_Vpscatterdd_vm32z_k1_zmm
	2,// EVEX_Vpscatterdq_vm32x_k1_xmm
	2,// EVEX_Vpscatterdq_vm32x_k1_ymm
	2,// EVEX_Vpscatterdq_vm32y_k1_zmm
	2,// EVEX_Vpscatterqd_vm64x_k1_xmm
	2,// EVEX_Vpscatterqd_vm64y_k1_xmm
	2,// EVEX_Vpscatterqd_vm64z_k1_ymm
	2,// EVEX_Vpscatterqq_vm64x_k1_xmm
	2,// EVEX_Vpscatterqq_vm64y_k1_ymm
	2,// EVEX_Vpscatterqq_vm64z_k1_zmm
	2,// EVEX_Vscatterdps_vm32x_k1_xmm
	2,// EVEX_Vscatterdps_vm32y_k1_ymm
	2,// EVEX_Vscatterdps_vm32z_k1_zmm
	2,// EVEX_Vscatterdpd_vm32x_k1_xmm
	2,// EVEX_Vscatterdpd_vm32x_k1_ymm
	2,// EVEX_Vscatterdpd_vm32y_k1_zmm
	2,// EVEX_Vscatterqps_vm64x_k1_xmm
	2,// EVEX_Vscatterqps_vm64y_k1_xmm
	2,// EVEX_Vscatterqps_vm64z_k1_ymm
	2,// EVEX_Vscatterqpd_vm64x_k1_xmm
	2,// EVEX_Vscatterqpd_vm64y_k1_ymm
	2,// EVEX_Vscatterqpd_vm64z_k1_zmm
	3,// VEX_Vfmaddsub213ps_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub213ps_ymm_ymm_ymmm256
	3,// VEX_Vfmaddsub213pd_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmsubadd213ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd213ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsubadd213pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd213ps_xmm_xmm_xmmm128
	3,// VEX_Vfmadd213ps_ymm_ymm_ymmm256
	3,// VEX_Vfmadd213pd_xmm_xmm_xmmm128
	3,// VEX_Vfmadd213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd213ss_xmm_xmm_xmmm32
	3,// VEX_Vfmadd213sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfmsub213ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsub213ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsub213pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsub213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128
	3,// VEX_Vfmsub213ss_xmm_xmm_xmmm32
	3,// VEX_Vfmsub213sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er
	3,// EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128
	3,// VEX_Vfnmadd213ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd213ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmadd213pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmadd213ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmadd213sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfnmsub213ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub213ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmsub213pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub213pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmsub213ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmsub213sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er
	3,// EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64
	3,// EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64
	3,// VEX_Vfmaddsub231ps_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub231ps_ymm_ymm_ymmm256
	3,// VEX_Vfmaddsub231pd_xmm_xmm_xmmm128
	3,// VEX_Vfmaddsub231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmsubadd231ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd231ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsubadd231pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsubadd231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd231ps_xmm_xmm_xmmm128
	3,// VEX_Vfmadd231ps_ymm_ymm_ymmm256
	3,// VEX_Vfmadd231pd_xmm_xmm_xmmm128
	3,// VEX_Vfmadd231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmadd231ss_xmm_xmm_xmmm32
	3,// VEX_Vfmadd231sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfmsub231ps_xmm_xmm_xmmm128
	3,// VEX_Vfmsub231ps_ymm_ymm_ymmm256
	3,// VEX_Vfmsub231pd_xmm_xmm_xmmm128
	3,// VEX_Vfmsub231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfmsub231ss_xmm_xmm_xmmm32
	3,// VEX_Vfmsub231sd_xmm_xmm_xmmm64
	3,// EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfnmadd231ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd231ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmadd231pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmadd231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmadd231ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmadd231sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er
	3,// VEX_Vfnmsub231ps_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub231ps_ymm_ymm_ymmm256
	3,// VEX_Vfnmsub231pd_xmm_xmm_xmmm128
	3,// VEX_Vfnmsub231pd_ymm_ymm_ymmm256
	3,// EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64
	3,// EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64
	3,// EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er
	3,// VEX_Vfnmsub231ss_xmm_xmm_xmmm32
	3,// VEX_Vfnmsub231sd_xmm_xmm_xmmm64
	3,// EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er
	2,// EVEX_Vpconflictd_xmm_k1z_xmmm128b32
	2,// EVEX_Vpconflictd_ymm_k1z_ymmm256b32
	2,// EVEX_Vpconflictd_zmm_k1z_zmmm512b32
	2,// EVEX_Vpconflictq_xmm_k1z_xmmm128b64
	2,// EVEX_Vpconflictq_ymm_k1z_ymmm256b64
	2,// EVEX_Vpconflictq_zmm_k1z_zmmm512b64
	1,// EVEX_Vgatherpf0dps_vm32z_k1
	1,// EVEX_Vgatherpf0dpd_vm32y_k1
	1,// EVEX_Vgatherpf1dps_vm32z_k1
	1,// EVEX_Vgatherpf1dpd_vm32y_k1
	1,// EVEX_Vscatterpf0dps_vm32z_k1
	1,// EVEX_Vscatterpf0dpd_vm32y_k1
	1,// EVEX_Vscatterpf1dps_vm32z_k1
	1,// EVEX_Vscatterpf1dpd_vm32y_k1
	1,// EVEX_Vgatherpf0qps_vm64z_k1
	1,// EVEX_Vgatherpf0qpd_vm64z_k1
	1,// EVEX_Vgatherpf1qps_vm64z_k1
	1,// EVEX_Vgatherpf1qpd_vm64z_k1
	1,// EVEX_Vscatterpf0qps_vm64z_k1
	1,// EVEX_Vscatterpf0qpd_vm64z_k1
	1,// EVEX_Vscatterpf1qps_vm64z_k1
	1,// EVEX_Vscatterpf1qpd_vm64z_k1
	2,// Sha1nexte_xmm_xmmm128
	2,// EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae
	2,// EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae
	2,// Sha1msg1_xmm_xmmm128
	2,// Sha1msg2_xmm_xmmm128
	2,// EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae
	2,// EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae
	2,// Sha256rnds2_xmm_xmmm128
	3,// EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae
	3,// EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae
	2,// Sha256msg1_xmm_xmmm128
	2,// EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae
	2,// EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae
	2,// Sha256msg2_xmm_xmmm128
	3,// EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae
	3,// EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae
	2,// Gf2p8mulb_xmm_xmmm128
	3,// VEX_Vgf2p8mulb_xmm_xmm_xmmm128
	3,// VEX_Vgf2p8mulb_ymm_ymm_ymmm256
	3,// EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128
	3,// EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256
	3,// EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512
	2,// Aesimc_xmm_xmmm128
	2,// VEX_Vaesimc_xmm_xmmm128
	2,// Aesenc_xmm_xmmm128
	3,// VEX_Vaesenc_xmm_xmm_xmmm128
	3,// VEX_Vaesenc_ymm_ymm_ymmm256
	3,// EVEX_Vaesenc_xmm_xmm_xmmm128
	3,// EVEX_Vaesenc_ymm_ymm_ymmm256
	3,// EVEX_Vaesenc_zmm_zmm_zmmm512
	2,// Aesenclast_xmm_xmmm128
	3,// VEX_Vaesenclast_xmm_xmm_xmmm128
	3,// VEX_Vaesenclast_ymm_ymm_ymmm256
	3,// EVEX_Vaesenclast_xmm_xmm_xmmm128
	3,// EVEX_Vaesenclast_ymm_ymm_ymmm256
	3,// EVEX_Vaesenclast_zmm_zmm_zmmm512
	2,// Aesdec_xmm_xmmm128
	3,// VEX_Vaesdec_xmm_xmm_xmmm128
	3,// VEX_Vaesdec_ymm_ymm_ymmm256
	3,// EVEX_Vaesdec_xmm_xmm_xmmm128
	3,// EVEX_Vaesdec_ymm_ymm_ymmm256
	3,// EVEX_Vaesdec_zmm_zmm_zmmm512
	2,// Aesdeclast_xmm_xmmm128
	3,// VEX_Vaesdeclast_xmm_xmm_xmmm128
	3,// VEX_Vaesdeclast_ymm_ymm_ymmm256
	3,// EVEX_Vaesdeclast_xmm_xmm_xmmm128
	3,// EVEX_Vaesdeclast_ymm_ymm_ymmm256
	3,// EVEX_Vaesdeclast_zmm_zmm_zmmm512
	2,// Movbe_r16_m16
	2,// Movbe_r32_m32
	2,// Movbe_r64_m64
	2,// Crc32_r32_rm8
	2,// Crc32_r64_rm8
	2,// Movbe_m16_r16
	2,// Movbe_m32_r32
	2,// Movbe_m64_r64
	2,// Crc32_r32_rm16
	2,// Crc32_r32_rm32
	2,// Crc32_r64_rm64
	3,// VEX_Andn_r32_r32_rm32
	3,// VEX_Andn_r64_r64_rm64
	2,// VEX_Blsr_r32_rm32
	2,// VEX_Blsr_r64_rm64
	2,// VEX_Blsmsk_r32_rm32
	2,// VEX_Blsmsk_r64_rm64
	2,// VEX_Blsi_r32_rm32
	2,// VEX_Blsi_r64_rm64
	3,// VEX_Bzhi_r32_rm32_r32
	3,// VEX_Bzhi_r64_rm64_r64
	2,// Wrussd_m32_r32
	2,// Wrussq_m64_r64
	3,// VEX_Pext_r32_r32_rm32
	3,// VEX_Pext_r64_r64_rm64
	3,// VEX_Pdep_r32_r32_rm32
	3,// VEX_Pdep_r64_r64_rm64
	2,// Wrssd_m32_r32
	2,// Wrssq_m64_r64
	2,// Adcx_r32_rm32
	2,// Adcx_r64_rm64
	2,// Adox_r32_rm32
	2,// Adox_r64_rm64
	3,// VEX_Mulx_r32_r32_rm32
	3,// VEX_Mulx_r64_r64_rm64
	3,// VEX_Bextr_r32_rm32_r32
	3,// VEX_Bextr_r64_rm64_r64
	3,// VEX_Shlx_r32_rm32_r32
	3,// VEX_Shlx_r64_rm64_r64
	3,// VEX_Sarx_r32_rm32_r32
	3,// VEX_Sarx_r64_rm64_r64
	3,// VEX_Shrx_r32_rm32_r32
	3,// VEX_Shrx_r64_rm64_r64
	2,// Movdir64b_r16_m512
	2,// Movdir64b_r32_m512
	2,// Movdir64b_r64_m512
	2,// Enqcmds_r16_m512
	2,// Enqcmds_r32_m512
	2,// Enqcmds_r64_m512
	2,// Enqcmd_r16_m512
	2,// Enqcmd_r32_m512
	2,// Enqcmd_r64_m512
	2,// Movdiri_m32_r32
	2,// Movdiri_m64_r64
	3,// VEX_Vpermq_ymm_ymmm256_imm8
	3,// EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8
	3,// VEX_Vpermpd_ymm_ymmm256_imm8
	3,// EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8
	4,// VEX_Vpblendd_xmm_xmm_xmmm128_imm8
	4,// VEX_Vpblendd_ymm_ymm_ymmm256_imm8
	4,// EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8
	3,// VEX_Vpermilps_xmm_xmmm128_imm8
	3,// VEX_Vpermilps_ymm_ymmm256_imm8
	3,// EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8
	3,// VEX_Vpermilpd_xmm_xmmm128_imm8
	3,// VEX_Vpermilpd_ymm_ymmm256_imm8
	3,// EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8
	4,// VEX_Vperm2f128_ymm_ymm_ymmm256_imm8
	3,// Roundps_xmm_xmmm128_imm8
	3,// VEX_Vroundps_xmm_xmmm128_imm8
	3,// VEX_Vroundps_ymm_ymmm256_imm8
	3,// EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae
	3,// Roundpd_xmm_xmmm128_imm8
	3,// VEX_Vroundpd_xmm_xmmm128_imm8
	3,// VEX_Vroundpd_ymm_ymmm256_imm8
	3,// EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae
	3,// Roundss_xmm_xmmm32_imm8
	4,// VEX_Vroundss_xmm_xmm_xmmm32_imm8
	4,// EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae
	3,// Roundsd_xmm_xmmm64_imm8
	4,// VEX_Vroundsd_xmm_xmm_xmmm64_imm8
	4,// EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae
	3,// Blendps_xmm_xmmm128_imm8
	4,// VEX_Vblendps_xmm_xmm_xmmm128_imm8
	4,// VEX_Vblendps_ymm_ymm_ymmm256_imm8
	3,// Blendpd_xmm_xmmm128_imm8
	4,// VEX_Vblendpd_xmm_xmm_xmmm128_imm8
	4,// VEX_Vblendpd_ymm_ymm_ymmm256_imm8
	3,// Pblendw_xmm_xmmm128_imm8
	4,// VEX_Vpblendw_xmm_xmm_xmmm128_imm8
	4,// VEX_Vpblendw_ymm_ymm_ymmm256_imm8
	3,// Palignr_mm_mmm64_imm8
	3,// Palignr_xmm_xmmm128_imm8
	4,// VEX_Vpalignr_xmm_xmm_xmmm128_imm8
	4,// VEX_Vpalignr_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8
	4,// EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8
	4,// EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8
	3,// Pextrb_r32m8_xmm_imm8
	3,// Pextrb_r64m8_xmm_imm8
	3,// VEX_Vpextrb_r32m8_xmm_imm8
	3,// VEX_Vpextrb_r64m8_xmm_imm8
	3,// EVEX_Vpextrb_r32m8_xmm_imm8
	3,// EVEX_Vpextrb_r64m8_xmm_imm8
	3,// Pextrw_r32m16_xmm_imm8
	3,// Pextrw_r64m16_xmm_imm8
	3,// VEX_Vpextrw_r32m16_xmm_imm8
	3,// VEX_Vpextrw_r64m16_xmm_imm8
	3,// EVEX_Vpextrw_r32m16_xmm_imm8
	3,// EVEX_Vpextrw_r64m16_xmm_imm8
	3,// Pextrd_rm32_xmm_imm8
	3,// Pextrq_rm64_xmm_imm8
	3,// VEX_Vpextrd_rm32_xmm_imm8
	3,// VEX_Vpextrq_rm64_xmm_imm8
	3,// EVEX_Vpextrd_rm32_xmm_imm8
	3,// EVEX_Vpextrq_rm64_xmm_imm8
	3,// Extractps_rm32_xmm_imm8
	3,// Extractps_r64m32_xmm_imm8
	3,// VEX_Vextractps_rm32_xmm_imm8
	3,// VEX_Vextractps_r64m32_xmm_imm8
	3,// EVEX_Vextractps_rm32_xmm_imm8
	3,// EVEX_Vextractps_r64m32_xmm_imm8
	4,// VEX_Vinsertf128_ymm_ymm_xmmm128_imm8
	4,// EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8
	4,// EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8
	4,// EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8
	4,// EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8
	3,// VEX_Vextractf128_xmmm128_ymm_imm8
	3,// EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8
	3,// EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8
	3,// EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8
	3,// EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8
	4,// EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8
	4,// EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8
	3,// EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8
	3,// EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8
	3,// VEX_Vcvtps2ph_xmmm64_xmm_imm8
	3,// VEX_Vcvtps2ph_xmmm128_ymm_imm8
	3,// EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8
	3,// EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8
	3,// EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae
	4,// EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8
	4,// EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8
	4,// EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8
	4,// EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8
	4,// EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8
	4,// EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8
	4,// EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8
	4,// EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8
	4,// EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8
	4,// EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8
	4,// EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8
	4,// EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8
	3,// Pinsrb_xmm_r32m8_imm8
	3,// Pinsrb_xmm_r64m8_imm8
	4,// VEX_Vpinsrb_xmm_xmm_r32m8_imm8
	4,// VEX_Vpinsrb_xmm_xmm_r64m8_imm8
	4,// EVEX_Vpinsrb_xmm_xmm_r32m8_imm8
	4,// EVEX_Vpinsrb_xmm_xmm_r64m8_imm8
	3,// Insertps_xmm_xmmm32_imm8
	4,// VEX_Vinsertps_xmm_xmm_xmmm32_imm8
	4,// EVEX_Vinsertps_xmm_xmm_xmmm32_imm8
	3,// Pinsrd_xmm_rm32_imm8
	3,// Pinsrq_xmm_rm64_imm8
	4,// VEX_Vpinsrd_xmm_xmm_rm32_imm8
	4,// VEX_Vpinsrq_xmm_xmm_rm64_imm8
	4,// EVEX_Vpinsrd_xmm_xmm_rm32_imm8
	4,// EVEX_Vpinsrq_xmm_xmm_rm64_imm8
	4,// EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8
	4,// EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8
	3,// EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae
	3,// EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae
	4,// EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae
	4,// EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae
	3,// VEX_Kshiftrb_kr_kr_imm8
	3,// VEX_Kshiftrw_kr_kr_imm8
	3,// VEX_Kshiftrd_kr_kr_imm8
	3,// VEX_Kshiftrq_kr_kr_imm8
	3,// VEX_Kshiftlb_kr_kr_imm8
	3,// VEX_Kshiftlw_kr_kr_imm8
	3,// VEX_Kshiftld_kr_kr_imm8
	3,// VEX_Kshiftlq_kr_kr_imm8
	4,// VEX_Vinserti128_ymm_ymm_xmmm128_imm8
	4,// EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8
	4,// EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8
	4,// EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8
	4,// EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8
	3,// VEX_Vextracti128_xmmm128_ymm_imm8
	3,// EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8
	3,// EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8
	3,// EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8
	3,// EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8
	4,// EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8
	4,// EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8
	3,// EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8
	3,// EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8
	4,// EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8
	4,// EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8
	4,// EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8
	4,// EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8
	4,// EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8
	4,// EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8
	4,// EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8
	4,// EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8
	4,// EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8
	4,// EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8
	4,// EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8
	4,// EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8
	3,// Dpps_xmm_xmmm128_imm8
	4,// VEX_Vdpps_xmm_xmm_xmmm128_imm8
	4,// VEX_Vdpps_ymm_ymm_ymmm256_imm8
	3,// Dppd_xmm_xmmm128_imm8
	4,// VEX_Vdppd_xmm_xmm_xmmm128_imm8
	3,// Mpsadbw_xmm_xmmm128_imm8
	4,// VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8
	4,// VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8
	4,// EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8
	4,// EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8
	4,// EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8
	3,// Pclmulqdq_xmm_xmmm128_imm8
	4,// VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	4,// VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8
	4,// EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8
	4,// VEX_Vperm2i128_ymm_ymm_ymmm256_imm8
	5,// VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm4
	5,// VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm4
	5,// VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm4
	5,// VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm4
	5,// VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm4
	5,// VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm4
	5,// VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm4
	5,// VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm4
	4,// VEX_Vblendvps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vblendvps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vblendvpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vblendvpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vpblendvb_xmm_xmm_xmmm128_xmm
	4,// VEX_Vpblendvb_ymm_ymm_ymmm256_ymm
	4,// EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	4,// EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	4,// EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae
	4,// EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae
	4,// EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae
	4,// EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae
	4,// EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae
	4,// EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae
	3,// EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8
	3,// EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8
	3,// EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae
	3,// EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8
	3,// EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8
	3,// EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae
	4,// EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae
	4,// EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae
	4,// VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256
	3,// Pcmpestrm_xmm_xmmm128_imm8
	3,// Pcmpestrm64_xmm_xmmm128_imm8
	3,// VEX_Vpcmpestrm_xmm_xmmm128_imm8
	3,// VEX_Vpcmpestrm64_xmm_xmmm128_imm8
	3,// Pcmpestri_xmm_xmmm128_imm8
	3,// Pcmpestri64_xmm_xmmm128_imm8
	3,// VEX_Vpcmpestri_xmm_xmmm128_imm8
	3,// VEX_Vpcmpestri64_xmm_xmmm128_imm8
	3,// Pcmpistrm_xmm_xmmm128_imm8
	3,// VEX_Vpcmpistrm_xmm_xmmm128_imm8
	3,// Pcmpistri_xmm_xmmm128_imm8
	3,// VEX_Vpcmpistri_xmm_xmmm128_imm8
	3,// EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8
	3,// EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8
	3,// EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8
	3,// EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8
	3,// EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8
	3,// EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8
	3,// EVEX_Vfpclassss_kr_k1_xmmm32_imm8
	3,// EVEX_Vfpclasssd_kr_k1_xmmm64_imm8
	4,// VEX_Vfmaddps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmaddps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmaddps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmaddps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmaddss_xmm_xmm_xmmm32_xmm
	4,// VEX_Vfmaddss_xmm_xmm_xmm_xmmm32
	4,// VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm
	4,// VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64
	4,// VEX_Vfmsubps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmsubps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmsubps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmsubps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfmsubss_xmm_xmm_xmmm32_xmm
	4,// VEX_Vfmsubss_xmm_xmm_xmm_xmmm32
	4,// VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm
	4,// VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64
	4,// EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8
	4,// EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8
	4,// EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8
	4,// EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8
	4,// EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8
	4,// EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8
	4,// EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8
	4,// EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8
	4,// EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8
	4,// EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8
	4,// EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8
	4,// VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm
	4,// VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32
	4,// VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm
	4,// VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64
	4,// VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm
	4,// VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm
	4,// VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128
	4,// VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256
	4,// VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm
	4,// VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32
	4,// VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm
	4,// VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64
	3,// Sha1rnds4_xmm_xmmm128_imm8
	3,// Gf2p8affineqb_xmm_xmmm128_imm8
	4,// VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8
	4,// VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8
	3,// Gf2p8affineinvqb_xmm_xmmm128_imm8
	4,// VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8
	4,// VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8
	4,// EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8
	4,// EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8
	4,// EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8
	3,// Aeskeygenassist_xmm_xmmm128_imm8
	3,// VEX_Vaeskeygenassist_xmm_xmmm128_imm8
	3,// VEX_Rorx_r32_rm32_imm8
	3,// VEX_Rorx_r64_rm64_imm8
	4,// XOP_Vpmacssww_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacsww_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacswd_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpcmov_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpcmov_ymm_ymm_ymmm256_ymm
	4,// XOP_Vpcmov_xmm_xmm_xmm_xmmm128
	4,// XOP_Vpcmov_ymm_ymm_ymm_ymmm256
	4,// XOP_Vpperm_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpperm_xmm_xmm_xmm_xmmm128
	4,// XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm
	4,// XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm
	3,// XOP_Vprotb_xmm_xmmm128_imm8
	3,// XOP_Vprotw_xmm_xmmm128_imm8
	3,// XOP_Vprotd_xmm_xmmm128_imm8
	3,// XOP_Vprotq_xmm_xmmm128_imm8
	4,// XOP_Vpcomb_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomw_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomd_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomq_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomub_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomuw_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomud_xmm_xmm_xmmm128_imm8
	4,// XOP_Vpcomuq_xmm_xmm_xmmm128_imm8
	2,// XOP_Blcfill_r32_rm32
	2,// XOP_Blcfill_r64_rm64
	2,// XOP_Blsfill_r32_rm32
	2,// XOP_Blsfill_r64_rm64
	2,// XOP_Blcs_r32_rm32
	2,// XOP_Blcs_r64_rm64
	2,// XOP_Tzmsk_r32_rm32
	2,// XOP_Tzmsk_r64_rm64
	2,// XOP_Blcic_r32_rm32
	2,// XOP_Blcic_r64_rm64
	2,// XOP_Blsic_r32_rm32
	2,// XOP_Blsic_r64_rm64
	2,// XOP_T1mskc_r32_rm32
	2,// XOP_T1mskc_r64_rm64
	2,// XOP_Blcmsk_r32_rm32
	2,// XOP_Blcmsk_r64_rm64
	2,// XOP_Blci_r32_rm32
	2,// XOP_Blci_r64_rm64
	1,// XOP_Llwpcb_r32
	1,// XOP_Llwpcb_r64
	1,// XOP_Slwpcb_r32
	1,// XOP_Slwpcb_r64
	2,// XOP_Vfrczps_xmm_xmmm128
	2,// XOP_Vfrczps_ymm_ymmm256
	2,// XOP_Vfrczpd_xmm_xmmm128
	2,// XOP_Vfrczpd_ymm_ymmm256
	2,// XOP_Vfrczss_xmm_xmmm32
	2,// XOP_Vfrczsd_xmm_xmmm64
	3,// XOP_Vprotb_xmm_xmmm128_xmm
	3,// XOP_Vprotb_xmm_xmm_xmmm128
	3,// XOP_Vprotw_xmm_xmmm128_xmm
	3,// XOP_Vprotw_xmm_xmm_xmmm128
	3,// XOP_Vprotd_xmm_xmmm128_xmm
	3,// XOP_Vprotd_xmm_xmm_xmmm128
	3,// XOP_Vprotq_xmm_xmmm128_xmm
	3,// XOP_Vprotq_xmm_xmm_xmmm128
	3,// XOP_Vpshlb_xmm_xmmm128_xmm
	3,// XOP_Vpshlb_xmm_xmm_xmmm128
	3,// XOP_Vpshlw_xmm_xmmm128_xmm
	3,// XOP_Vpshlw_xmm_xmm_xmmm128
	3,// XOP_Vpshld_xmm_xmmm128_xmm
	3,// XOP_Vpshld_xmm_xmm_xmmm128
	3,// XOP_Vpshlq_xmm_xmmm128_xmm
	3,// XOP_Vpshlq_xmm_xmm_xmmm128
	3,// XOP_Vpshab_xmm_xmmm128_xmm
	3,// XOP_Vpshab_xmm_xmm_xmmm128
	3,// XOP_Vpshaw_xmm_xmmm128_xmm
	3,// XOP_Vpshaw_xmm_xmm_xmmm128
	3,// XOP_Vpshad_xmm_xmmm128_xmm
	3,// XOP_Vpshad_xmm_xmm_xmmm128
	3,// XOP_Vpshaq_xmm_xmmm128_xmm
	3,// XOP_Vpshaq_xmm_xmm_xmmm128
	2,// XOP_Vphaddbw_xmm_xmmm128
	2,// XOP_Vphaddbd_xmm_xmmm128
	2,// XOP_Vphaddbq_xmm_xmmm128
	2,// XOP_Vphaddwd_xmm_xmmm128
	2,// XOP_Vphaddwq_xmm_xmmm128
	2,// XOP_Vphadddq_xmm_xmmm128
	2,// XOP_Vphaddubw_xmm_xmmm128
	2,// XOP_Vphaddubd_xmm_xmmm128
	2,// XOP_Vphaddubq_xmm_xmmm128
	2,// XOP_Vphadduwd_xmm_xmmm128
	2,// XOP_Vphadduwq_xmm_xmmm128
	2,// XOP_Vphaddudq_xmm_xmmm128
	2,// XOP_Vphsubbw_xmm_xmmm128
	2,// XOP_Vphsubwd_xmm_xmmm128
	2,// XOP_Vphsubdq_xmm_xmmm128
	3,// XOP_Bextr_r32_rm32_imm32
	3,// XOP_Bextr_r64_rm64_imm32
	3,// XOP_Lwpins_r32_rm32_imm32
	3,// XOP_Lwpins_r64_rm32_imm32
	3,// XOP_Lwpval_r32_rm32_imm32
	3,// XOP_Lwpval_r64_rm32_imm32
	2,// D3NOW_Pi2fw_mm_mmm64
	2,// D3NOW_Pi2fd_mm_mmm64
	2,// D3NOW_Pf2iw_mm_mmm64
	2,// D3NOW_Pf2id_mm_mmm64
	2,// D3NOW_Pfrcpv_mm_mmm64
	2,// D3NOW_Pfrsqrtv_mm_mmm64
	2,// D3NOW_Pfnacc_mm_mmm64
	2,// D3NOW_Pfpnacc_mm_mmm64
	2,// D3NOW_Pfcmpge_mm_mmm64
	2,// D3NOW_Pfmin_mm_mmm64
	2,// D3NOW_Pfrcp_mm_mmm64
	2,// D3NOW_Pfrsqrt_mm_mmm64
	2,// D3NOW_Pfsub_mm_mmm64
	2,// D3NOW_Pfadd_mm_mmm64
	2,// D3NOW_Pfcmpgt_mm_mmm64
	2,// D3NOW_Pfmax_mm_mmm64
	2,// D3NOW_Pfrcpit1_mm_mmm64
	2,// D3NOW_Pfrsqit1_mm_mmm64
	2,// D3NOW_Pfsubr_mm_mmm64
	2,// D3NOW_Pfacc_mm_mmm64
	2,// D3NOW_Pfcmpeq_mm_mmm64
	2,// D3NOW_Pfmul_mm_mmm64
	2,// D3NOW_Pfrcpit2_mm_mmm64
	2,// D3NOW_Pmulhrw_mm_mmm64
	2,// D3NOW_Pswapd_mm_mmm64
	2,// D3NOW_Pavgusb_mm_mmm64
	0,// Rmpadjust
	0,// Rmpupdate
	0,// Psmash
	0,// Pvalidatew
	0,// Pvalidated
	0,// Pvalidateq
	0,// Serialize
	0,// Xsusldtrk
	0,// Xresldtrk
	0,// Invlpgbw
	0,// Invlpgbd
	0,// Invlpgbq
	0,// Tlbsync
	1,// Prefetchreserved3_m8
	1,// Prefetchreserved4_m8
	1,// Prefetchreserved5_m8
	1,// Prefetchreserved6_m8
	1,// Prefetchreserved7_m8
	0,// Ud0
	0,// Vmgexit
	0,// Getsecq
	1,// VEX_Ldtilecfg_m512
	0,// VEX_Tilerelease
	1,// VEX_Sttilecfg_m512
	1,// VEX_Tilezero_tmm
	2,// VEX_Tileloaddt1_tmm_sibmem
	2,// VEX_Tilestored_sibmem_tmm
	2,// VEX_Tileloadd_tmm_sibmem
	3,// VEX_Tdpbf16ps_tmm_tmm_tmm
	3,// VEX_Tdpbuud_tmm_tmm_tmm
	3,// VEX_Tdpbusd_tmm_tmm_tmm
	3,// VEX_Tdpbsud_tmm_tmm_tmm
	3,// VEX_Tdpbssd_tmm_tmm_tmm
	1,// Fnstdw_AX
	1,// Fnstsg_AX
	1,// Rdshr_rm32
	1,// Wrshr_rm32
	0,// Smint
	0,// Dmint
	0,// Rdm
	2,// Svdc_m80_Sreg
	2,// Rsdc_Sreg_m80
	1,// Svldt_m80
	1,// Rsldt_m80
	1,// Svts_m80
	1,// Rsts_m80
	0,// Smint_0F7E
	0,// Bb0_reset
	0,// Bb1_reset
	0,// Cpu_write
	0,// Cpu_read
	0,// Altinst
	2,// Paveb_mm_mmm64
	2,// Paddsiw_mm_mmm64
	2,// Pmagw_mm_mmm64
	2,// Pdistib_mm_m64
	2,// Psubsiw_mm_mmm64
	2,// Pmvzb_mm_m64
	2,// Pmulhrw_mm_mmm64
	2,// Pmvnzb_mm_m64
	2,// Pmvlzb_mm_m64
	2,// Pmvgezb_mm_m64
	2,// Pmulhriw_mm_mmm64
	2,// Pmachriw_mm_m64
	0,// Cyrix_D9D7
	0,// Cyrix_D9E2
	0,// Ftstp
	0,// Cyrix_D9E7
	0,// Frint2
	0,// Frichop
	0,// Cyrix_DED8
	0,// Cyrix_DEDA
	0,// Cyrix_DEDC
	0,// Cyrix_DEDD
	0,// Cyrix_DEDE
	0,// Frinear
	0,// Tdcall
	0,// Seamret
	0,// Seamops
	0,// Seamcall
	1,// Aesencwide128kl_m384
	1,// Aesdecwide128kl_m384
	1,// Aesencwide256kl_m512
	1,// Aesdecwide256kl_m512
	2,// Loadiwkey_xmm_xmm
	2,// Aesenc128kl_xmm_m384
	2,// Aesdec128kl_xmm_m384
	2,// Aesenc256kl_xmm_m512
	2,// Aesdec256kl_xmm_m512
	2,// Encodekey128_r32_r32
	2,// Encodekey256_r32_r32
	2,// VEX_Vbroadcastss_xmm_xmm
	2,// VEX_Vbroadcastss_ymm_xmm
	2,// VEX_Vbroadcastsd_ymm_xmm
	0,// Vmgexit_F2
	0,// Uiret
	0,// Testui
	0,// Clui
	0,// Stui
	1,// Senduipi_r64
	1,// Hreset_imm8
	3,// VEX_Vpdpbusd_xmm_xmm_xmmm128
	3,// VEX_Vpdpbusd_ymm_ymm_ymmm256
	3,// VEX_Vpdpbusds_xmm_xmm_xmmm128
	3,// VEX_Vpdpbusds_ymm_ymm_ymmm256
	3,// VEX_Vpdpwssd_xmm_xmm_xmmm128
	3,// VEX_Vpdpwssd_ymm_ymm_ymmm256
	3,// VEX_Vpdpwssds_xmm_xmm_xmmm128
	3,// VEX_Vpdpwssds_ymm_ymm_ymmm256
	0,// Ccs_hash_16
	0,// Ccs_hash_32
	0,// Ccs_hash_64
	0,// Ccs_encrypt_16
	0,// Ccs_encrypt_32
	0,// Ccs_encrypt_64
	1,// Lkgs_rm16
	1,// Lkgs_r32m16
	1,// Lkgs_r64m16
	0,// Eretu
	0,// Erets
	3,// EVEX_Vaddph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vaddph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vaddph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vaddsh_xmm_k1z_xmm_xmmm16_er
	4,// EVEX_Vcmpph_kr_k1_xmm_xmmm128b16_imm8
	4,// EVEX_Vcmpph_kr_k1_ymm_ymmm256b16_imm8
	4,// EVEX_Vcmpph_kr_k1_zmm_zmmm512b16_imm8_sae
	4,// EVEX_Vcmpsh_kr_k1_xmm_xmmm16_imm8_sae
	2,// EVEX_Vcomish_xmm_xmmm16_sae
	2,// EVEX_Vcvtdq2ph_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtdq2ph_xmm_k1z_ymmm256b32
	2,// EVEX_Vcvtdq2ph_ymm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtpd2ph_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtpd2ph_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtpd2ph_xmm_k1z_zmmm512b64_er
	2,// EVEX_Vcvtph2dq_xmm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2dq_ymm_k1z_xmmm128b16
	2,// EVEX_Vcvtph2dq_zmm_k1z_ymmm256b16_er
	2,// EVEX_Vcvtph2pd_xmm_k1z_xmmm32b16
	2,// EVEX_Vcvtph2pd_ymm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2pd_zmm_k1z_xmmm128b16_sae
	2,// EVEX_Vcvtph2psx_xmm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2psx_ymm_k1z_xmmm128b16
	2,// EVEX_Vcvtph2psx_zmm_k1z_ymmm256b16_sae
	2,// EVEX_Vcvtph2qq_xmm_k1z_xmmm32b16
	2,// EVEX_Vcvtph2qq_ymm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2qq_zmm_k1z_xmmm128b16_er
	2,// EVEX_Vcvtph2udq_xmm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2udq_ymm_k1z_xmmm128b16
	2,// EVEX_Vcvtph2udq_zmm_k1z_ymmm256b16_er
	2,// EVEX_Vcvtph2uqq_xmm_k1z_xmmm32b16
	2,// EVEX_Vcvtph2uqq_ymm_k1z_xmmm64b16
	2,// EVEX_Vcvtph2uqq_zmm_k1z_xmmm128b16_er
	2,// EVEX_Vcvtph2uw_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvtph2uw_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvtph2uw_zmm_k1z_zmmm512b16_er
	2,// EVEX_Vcvtph2w_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvtph2w_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvtph2w_zmm_k1z_zmmm512b16_er
	2,// EVEX_Vcvtps2phx_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtps2phx_xmm_k1z_ymmm256b32
	2,// EVEX_Vcvtps2phx_ymm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtqq2ph_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtqq2ph_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtqq2ph_xmm_k1z_zmmm512b64_er
	3,// EVEX_Vcvtsd2sh_xmm_k1z_xmm_xmmm64_er
	3,// EVEX_Vcvtsh2sd_xmm_k1z_xmm_xmmm16_sae
	2,// EVEX_Vcvtsh2si_r32_xmmm16_er
	2,// EVEX_Vcvtsh2si_r64_xmmm16_er
	3,// EVEX_Vcvtsh2ss_xmm_k1z_xmm_xmmm16_sae
	2,// EVEX_Vcvtsh2usi_r32_xmmm16_er
	2,// EVEX_Vcvtsh2usi_r64_xmmm16_er
	3,// EVEX_Vcvtsi2sh_xmm_xmm_rm32_er
	3,// EVEX_Vcvtsi2sh_xmm_xmm_rm64_er
	3,// EVEX_Vcvtss2sh_xmm_k1z_xmm_xmmm32_er
	2,// EVEX_Vcvttph2dq_xmm_k1z_xmmm64b16
	2,// EVEX_Vcvttph2dq_ymm_k1z_xmmm128b16
	2,// EVEX_Vcvttph2dq_zmm_k1z_ymmm256b16_sae
	2,// EVEX_Vcvttph2qq_xmm_k1z_xmmm32b16
	2,// EVEX_Vcvttph2qq_ymm_k1z_xmmm64b16
	2,// EVEX_Vcvttph2qq_zmm_k1z_xmmm128b16_sae
	2,// EVEX_Vcvttph2udq_xmm_k1z_xmmm64b16
	2,// EVEX_Vcvttph2udq_ymm_k1z_xmmm128b16
	2,// EVEX_Vcvttph2udq_zmm_k1z_ymmm256b16_sae
	2,// EVEX_Vcvttph2uqq_xmm_k1z_xmmm32b16
	2,// EVEX_Vcvttph2uqq_ymm_k1z_xmmm64b16
	2,// EVEX_Vcvttph2uqq_zmm_k1z_xmmm128b16_sae
	2,// EVEX_Vcvttph2uw_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvttph2uw_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvttph2uw_zmm_k1z_zmmm512b16_sae
	2,// EVEX_Vcvttph2w_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvttph2w_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvttph2w_zmm_k1z_zmmm512b16_sae
	2,// EVEX_Vcvttsh2si_r32_xmmm16_sae
	2,// EVEX_Vcvttsh2si_r64_xmmm16_sae
	2,// EVEX_Vcvttsh2usi_r32_xmmm16_sae
	2,// EVEX_Vcvttsh2usi_r64_xmmm16_sae
	2,// EVEX_Vcvtudq2ph_xmm_k1z_xmmm128b32
	2,// EVEX_Vcvtudq2ph_xmm_k1z_ymmm256b32
	2,// EVEX_Vcvtudq2ph_ymm_k1z_zmmm512b32_er
	2,// EVEX_Vcvtuqq2ph_xmm_k1z_xmmm128b64
	2,// EVEX_Vcvtuqq2ph_xmm_k1z_ymmm256b64
	2,// EVEX_Vcvtuqq2ph_xmm_k1z_zmmm512b64_er
	3,// EVEX_Vcvtusi2sh_xmm_xmm_rm32_er
	3,// EVEX_Vcvtusi2sh_xmm_xmm_rm64_er
	2,// EVEX_Vcvtuw2ph_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvtuw2ph_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvtuw2ph_zmm_k1z_zmmm512b16_er
	2,// EVEX_Vcvtw2ph_xmm_k1z_xmmm128b16
	2,// EVEX_Vcvtw2ph_ymm_k1z_ymmm256b16
	2,// EVEX_Vcvtw2ph_zmm_k1z_zmmm512b16_er
	3,// EVEX_Vdivph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vdivph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vdivph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vdivsh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfcmaddcph_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfcmaddcph_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfcmaddcph_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmaddcph_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmaddcph_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmaddcph_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfcmaddcsh_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmaddcsh_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfcmulcph_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfcmulcph_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfcmulcph_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfmulcph_xmm_k1z_xmm_xmmm128b32
	3,// EVEX_Vfmulcph_ymm_k1z_ymm_ymmm256b32
	3,// EVEX_Vfmulcph_zmm_k1z_zmm_zmmm512b32_er
	3,// EVEX_Vfcmulcsh_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmulcsh_xmm_k1z_xmm_xmmm32_er
	3,// EVEX_Vfmaddsub132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmaddsub132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmaddsub132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmaddsub213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmaddsub213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmaddsub213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmaddsub231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmaddsub231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmaddsub231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsubadd132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsubadd132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsubadd132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsubadd213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsubadd213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsubadd213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsubadd231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsubadd231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsubadd231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmadd132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmadd132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmadd213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmadd213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmadd231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmadd231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmadd132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmadd132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmadd132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmadd213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmadd213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmadd213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmadd231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmadd231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmadd231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmadd132sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfmadd213sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfmadd231sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmadd132sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmadd213sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmadd231sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfmsub132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsub132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsub213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsub213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsub231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfmsub231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmsub132ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmsub132ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmsub132ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmsub213ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmsub213ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmsub213ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfnmsub231ph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vfnmsub231ph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vfnmsub231ph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vfmsub132sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfmsub213sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfmsub231sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmsub132sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmsub213sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfnmsub231sh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vfpclassph_kr_k1_xmmm128b16_imm8
	3,// EVEX_Vfpclassph_kr_k1_ymmm256b16_imm8
	3,// EVEX_Vfpclassph_kr_k1_zmmm512b16_imm8
	3,// EVEX_Vfpclasssh_kr_k1_xmmm16_imm8
	2,// EVEX_Vgetexpph_xmm_k1z_xmmm128b16
	2,// EVEX_Vgetexpph_ymm_k1z_ymmm256b16
	2,// EVEX_Vgetexpph_zmm_k1z_zmmm512b16_sae
	3,// EVEX_Vgetexpsh_xmm_k1z_xmm_xmmm16_sae
	3,// EVEX_Vgetmantph_xmm_k1z_xmmm128b16_imm8
	3,// EVEX_Vgetmantph_ymm_k1z_ymmm256b16_imm8
	3,// EVEX_Vgetmantph_zmm_k1z_zmmm512b16_imm8_sae
	4,// EVEX_Vgetmantsh_xmm_k1z_xmm_xmmm16_imm8_sae
	3,// EVEX_Vmaxph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vmaxph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vmaxph_zmm_k1z_zmm_zmmm512b16_sae
	3,// EVEX_Vmaxsh_xmm_k1z_xmm_xmmm16_sae
	3,// EVEX_Vminph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vminph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vminph_zmm_k1z_zmm_zmmm512b16_sae
	3,// EVEX_Vminsh_xmm_k1z_xmm_xmmm16_sae
	2,// EVEX_Vmovsh_xmm_k1z_m16
	2,// EVEX_Vmovsh_m16_k1_xmm
	3,// EVEX_Vmovsh_xmm_k1z_xmm_xmm
	3,// EVEX_Vmovsh_xmm_k1z_xmm_xmm_MAP5_11
	2,// EVEX_Vmovw_xmm_r32m16
	2,// EVEX_Vmovw_xmm_r64m16
	2,// EVEX_Vmovw_r32m16_xmm
	2,// EVEX_Vmovw_r64m16_xmm
	3,// EVEX_Vmulph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vmulph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vmulph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vmulsh_xmm_k1z_xmm_xmmm16_er
	2,// EVEX_Vrcpph_xmm_k1z_xmmm128b16
	2,// EVEX_Vrcpph_ymm_k1z_ymmm256b16
	2,// EVEX_Vrcpph_zmm_k1z_zmmm512b16
	3,// EVEX_Vrcpsh_xmm_k1z_xmm_xmmm16
	3,// EVEX_Vreduceph_xmm_k1z_xmmm128b16_imm8
	3,// EVEX_Vreduceph_ymm_k1z_ymmm256b16_imm8
	3,// EVEX_Vreduceph_zmm_k1z_zmmm512b16_imm8_sae
	4,// EVEX_Vreducesh_xmm_k1z_xmm_xmmm16_imm8_sae
	3,// EVEX_Vrndscaleph_xmm_k1z_xmmm128b16_imm8
	3,// EVEX_Vrndscaleph_ymm_k1z_ymmm256b16_imm8
	3,// EVEX_Vrndscaleph_zmm_k1z_zmmm512b16_imm8_sae
	4,// EVEX_Vrndscalesh_xmm_k1z_xmm_xmmm16_imm8_sae
	2,// EVEX_Vrsqrtph_xmm_k1z_xmmm128b16
	2,// EVEX_Vrsqrtph_ymm_k1z_ymmm256b16
	2,// EVEX_Vrsqrtph_zmm_k1z_zmmm512b16
	3,// EVEX_Vrsqrtsh_xmm_k1z_xmm_xmmm16
	3,// EVEX_Vscalefph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vscalefph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vscalefph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vscalefsh_xmm_k1z_xmm_xmmm16_er
	2,// EVEX_Vsqrtph_xmm_k1z_xmmm128b16
	2,// EVEX_Vsqrtph_ymm_k1z_ymmm256b16
	2,// EVEX_Vsqrtph_zmm_k1z_zmmm512b16_er
	3,// EVEX_Vsqrtsh_xmm_k1z_xmm_xmmm16_er
	3,// EVEX_Vsubph_xmm_k1z_xmm_xmmm128b16
	3,// EVEX_Vsubph_ymm_k1z_ymm_ymmm256b16
	3,// EVEX_Vsubph_zmm_k1z_zmm_zmmm512b16_er
	3,// EVEX_Vsubsh_xmm_k1z_xmm_xmmm16_er
	2,// EVEX_Vucomish_xmm_xmmm16_sae
	0,// Rdudbg
	0,// Wrudbg
	2,// VEX_KNC_Jkzd_kr_rel8_64
	2,// VEX_KNC_Jknzd_kr_rel8_64
	1,// VEX_KNC_Vprefetchnta_m8
	1,// VEX_KNC_Vprefetch0_m8
	1,// VEX_KNC_Vprefetch1_m8
	1,// VEX_KNC_Vprefetch2_m8
	1,// VEX_KNC_Vprefetchenta_m8
	1,// VEX_KNC_Vprefetche0_m8
	1,// VEX_KNC_Vprefetche1_m8
	1,// VEX_KNC_Vprefetche2_m8
	2,// VEX_KNC_Kand_kr_kr
	2,// VEX_KNC_Kandn_kr_kr
	2,// VEX_KNC_Kandnr_kr_kr
	2,// VEX_KNC_Knot_kr_kr
	2,// VEX_KNC_Kor_kr_kr
	2,// VEX_KNC_Kxnor_kr_kr
	2,// VEX_KNC_Kxor_kr_kr
	2,// VEX_KNC_Kmerge2l1h_kr_kr
	2,// VEX_KNC_Kmerge2l1l_kr_kr
	2,// VEX_KNC_Jkzd_kr_rel32_64
	2,// VEX_KNC_Jknzd_kr_rel32_64
	2,// VEX_KNC_Kmov_kr_kr
	2,// VEX_KNC_Kmov_kr_r32
	2,// VEX_KNC_Kmov_r32_kr
	3,// VEX_KNC_Kconcath_r64_kr_kr
	3,// VEX_KNC_Kconcatl_r64_kr_kr
	2,// VEX_KNC_Kortest_kr_kr
	1,// VEX_KNC_Delay_r32
	1,// VEX_KNC_Delay_r64
	1,// VEX_KNC_Spflt_r32
	1,// VEX_KNC_Spflt_r64
	1,// VEX_KNC_Clevict1_m8
	1,// VEX_KNC_Clevict0_m8
	2,// VEX_KNC_Popcnt_r32_r32
	2,// VEX_KNC_Popcnt_r64_r64
	2,// VEX_KNC_Tzcnt_r32_r32
	2,// VEX_KNC_Tzcnt_r64_r64
	2,// VEX_KNC_Tzcnti_r32_r32
	2,// VEX_KNC_Tzcnti_r64_r64
	2,// VEX_KNC_Lzcnt_r32_r32
	2,// VEX_KNC_Lzcnt_r64_r64
	2,// VEX_KNC_Undoc_r32_rm32_128_F3_0F38_W0_F0
	2,// VEX_KNC_Undoc_r64_rm64_128_F3_0F38_W1_F0
	2,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F0
	2,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F0
	2,// VEX_KNC_Undoc_r32_rm32_128_F2_0F38_W0_F1
	2,// VEX_KNC_Undoc_r64_rm64_128_F2_0F38_W1_F1
	3,// VEX_KNC_Kextract_kr_r64_imm8
	1,// MVEX_Vprefetchnta_m
	1,// MVEX_Vprefetch0_m
	1,// MVEX_Vprefetch1_m
	1,// MVEX_Vprefetch2_m
	1,// MVEX_Vprefetchenta_m
	1,// MVEX_Vprefetche0_m
	1,// MVEX_Vprefetche1_m
	1,// MVEX_Vprefetche2_m
	2,// MVEX_Vmovaps_zmm_k1_zmmmt
	2,// MVEX_Vmovapd_zmm_k1_zmmmt
	2,// MVEX_Vmovaps_mt_k1_zmm
	2,// MVEX_Vmovapd_mt_k1_zmm
	2,// MVEX_Vmovnrapd_m_k1_zmm
	2,// MVEX_Vmovnrngoapd_m_k1_zmm
	2,// MVEX_Vmovnraps_m_k1_zmm
	2,// MVEX_Vmovnrngoaps_m_k1_zmm
	3,// MVEX_Vaddps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vaddpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vmulps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vmulpd_zmm_k1_zmm_zmmmt
	2,// MVEX_Vcvtps2pd_zmm_k1_zmmmt
	2,// MVEX_Vcvtpd2ps_zmm_k1_zmmmt
	3,// MVEX_Vsubps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vsubpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpcmpgtd_kr_k1_zmm_zmmmt
	2,// MVEX_Vmovdqa32_zmm_k1_zmmmt
	2,// MVEX_Vmovdqa64_zmm_k1_zmmmt
	3,// MVEX_Vpshufd_zmm_k1_zmmmt_imm8
	3,// MVEX_Vpsrld_zmm_k1_zmmmt_imm8
	3,// MVEX_Vpsrad_zmm_k1_zmmmt_imm8
	3,// MVEX_Vpslld_zmm_k1_zmmmt_imm8
	3,// MVEX_Vpcmpeqd_kr_k1_zmm_zmmmt
	2,// MVEX_Vcvtudq2pd_zmm_k1_zmmmt
	2,// MVEX_Vmovdqa32_mt_k1_zmm
	2,// MVEX_Vmovdqa64_mt_k1_zmm
	1,// MVEX_Clevict1_m
	1,// MVEX_Clevict0_m
	4,// MVEX_Vcmpps_kr_k1_zmm_zmmmt_imm8
	4,// MVEX_Vcmppd_kr_k1_zmm_zmmmt_imm8
	3,// MVEX_Vpandd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpandq_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpandnd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpandnq_zmm_k1_zmm_zmmmt
	2,// MVEX_Vcvtdq2pd_zmm_k1_zmmmt
	3,// MVEX_Vpord_zmm_k1_zmm_zmmmt
	3,// MVEX_Vporq_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpxord_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpxorq_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpsubd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpaddd_zmm_k1_zmm_zmmmt
	2,// MVEX_Vbroadcastss_zmm_k1_mt
	2,// MVEX_Vbroadcastsd_zmm_k1_mt
	2,// MVEX_Vbroadcastf32x4_zmm_k1_mt
	2,// MVEX_Vbroadcastf64x4_zmm_k1_mt
	3,// MVEX_Vptestmd_kr_k1_zmm_zmmmt
	3,// MVEX_Vpermd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpminsd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpminud_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmaxsd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmaxud_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmulld_zmm_k1_zmm_zmmmt
	2,// MVEX_Vgetexpps_zmm_k1_zmmmt
	2,// MVEX_Vgetexppd_zmm_k1_zmmmt
	3,// MVEX_Vpsrlvd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpsravd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpsllvd_zmm_k1_zmm_zmmmt
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_48
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_49
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4A
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_4B
	3,// MVEX_Vaddnps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vaddnpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vgmaxabsps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vgminps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vgminpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vgmaxps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vgmaxpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_54
	3,// MVEX_Vfixupnanps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfixupnanpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_56
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_57
	2,// MVEX_Vpbroadcastd_zmm_k1_mt
	2,// MVEX_Vpbroadcastq_zmm_k1_mt
	2,// MVEX_Vbroadcasti32x4_zmm_k1_mt
	2,// MVEX_Vbroadcasti64x4_zmm_k1_mt
	3,// MVEX_Vpadcd_zmm_k1_kr_zmmmt
	3,// MVEX_Vpaddsetcd_zmm_k1_kr_zmmmt
	3,// MVEX_Vpsbbd_zmm_k1_kr_zmmmt
	3,// MVEX_Vpsubsetbd_zmm_k1_kr_zmmmt
	3,// MVEX_Vpblendmd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpblendmq_zmm_k1_zmm_zmmmt
	3,// MVEX_Vblendmps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vblendmpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_67
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_68
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_69
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6A
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_6B
	3,// MVEX_Vpsubrd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vsubrps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vsubrpd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpsbbrd_zmm_k1_kr_zmmmt
	3,// MVEX_Vpsubrsetbd_zmm_k1_kr_zmmmt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_70
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_71
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_72
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_73
	3,// MVEX_Vpcmpltd_kr_k1_zmm_zmmmt
	3,// MVEX_Vscaleps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmulhud_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmulhd_zmm_k1_zmm_zmmmt
	2,// MVEX_Vpgatherdd_zmm_k1_mvt
	2,// MVEX_Vpgatherdq_zmm_k1_mvt
	2,// MVEX_Vgatherdps_zmm_k1_mvt
	2,// MVEX_Vgatherdpd_zmm_k1_mvt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_94
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_94
	3,// MVEX_Vfmadd132ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmadd132pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub132ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub132pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd132ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd132pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub132ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub132pd_zmm_k1_zmm_zmmmt
	2,// MVEX_Vpscatterdd_mvt_k1_zmm
	2,// MVEX_Vpscatterdq_mvt_k1_zmm
	2,// MVEX_Vscatterdps_mvt_k1_zmm
	2,// MVEX_Vscatterdpd_mvt_k1_zmm
	3,// MVEX_Vfmadd233ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmadd213ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmadd213pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub213ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub213pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd213ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd213pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub213ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub213pd_zmm_k1_zmm_zmmmt
	2,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B0
	2,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_B2
	3,// MVEX_Vpmadd233d_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpmadd231d_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmadd231ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmadd231pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub231ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfmsub231pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd231ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmadd231pd_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub231ps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vfnmsub231pd_zmm_k1_zmm_zmmmt
	2,// MVEX_Undoc_zmm_k1_mvt_512_66_0F38_W0_C0
	1,// MVEX_Vgatherpf0hintdps_mvt_k1
	1,// MVEX_Vgatherpf0hintdpd_mvt_k1
	1,// MVEX_Vgatherpf0dps_mvt_k1
	1,// MVEX_Vgatherpf1dps_mvt_k1
	1,// MVEX_Vscatterpf0hintdps_mvt_k1
	1,// MVEX_Vscatterpf0hintdpd_mvt_k1
	1,// MVEX_Vscatterpf0dps_mvt_k1
	1,// MVEX_Vscatterpf1dps_mvt_k1
	2,// MVEX_Vexp223ps_zmm_k1_zmmmt
	2,// MVEX_Vlog2ps_zmm_k1_zmmmt
	2,// MVEX_Vrcp23ps_zmm_k1_zmmmt
	2,// MVEX_Vrsqrt23ps_zmm_k1_zmmmt
	3,// MVEX_Vaddsetsps_zmm_k1_zmm_zmmmt
	3,// MVEX_Vpaddsetsd_zmm_k1_zmm_zmmmt
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CE
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W1_CE
	3,// MVEX_Undoc_zmm_k1_zmm_zmmmt_512_66_0F38_W0_CF
	2,// MVEX_Vloadunpackld_zmm_k1_mt
	2,// MVEX_Vloadunpacklq_zmm_k1_mt
	2,// MVEX_Vpackstoreld_mt_k1_zmm
	2,// MVEX_Vpackstorelq_mt_k1_zmm
	2,// MVEX_Vloadunpacklps_zmm_k1_mt
	2,// MVEX_Vloadunpacklpd_zmm_k1_mt
	2,// MVEX_Vpackstorelps_mt_k1_zmm
	2,// MVEX_Vpackstorelpd_mt_k1_zmm
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D2
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D2
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D3
	2,// MVEX_Vloadunpackhd_zmm_k1_mt
	2,// MVEX_Vloadunpackhq_zmm_k1_mt
	2,// MVEX_Vpackstorehd_mt_k1_zmm
	2,// MVEX_Vpackstorehq_mt_k1_zmm
	2,// MVEX_Vloadunpackhps_zmm_k1_mt
	2,// MVEX_Vloadunpackhpd_zmm_k1_mt
	2,// MVEX_Vpackstorehps_mt_k1_zmm
	2,// MVEX_Vpackstorehpd_mt_k1_zmm
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D6
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_66_0F38_W0_D6
	2,// MVEX_Undoc_zmm_k1_zmmmt_512_0F38_W0_D7
	4,// MVEX_Valignd_zmm_k1_zmm_zmmmt_imm8
	3,// MVEX_Vpermf32x4_zmm_k1_zmmmt_imm8
	4,// MVEX_Vpcmpud_kr_k1_zmm_zmmmt_imm8
	4,// MVEX_Vpcmpd_kr_k1_zmm_zmmmt_imm8
	3,// MVEX_Vgetmantps_zmm_k1_zmmmt_imm8
	3,// MVEX_Vgetmantpd_zmm_k1_zmmmt_imm8
	3,// MVEX_Vrndfxpntps_zmm_k1_zmmmt_imm8
	3,// MVEX_Vrndfxpntpd_zmm_k1_zmmmt_imm8
	3,// MVEX_Vcvtfxpntudq2ps_zmm_k1_zmmmt_imm8
	3,// MVEX_Vcvtfxpntps2udq_zmm_k1_zmmmt_imm8
	3,// MVEX_Vcvtfxpntpd2udq_zmm_k1_zmmmt_imm8
	3,// MVEX_Vcvtfxpntdq2ps_zmm_k1_zmmmt_imm8
	3,// MVEX_Vcvtfxpntps2dq_zmm_k1_zmmmt_imm8
	3,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D0
	3,// MVEX_Undoc_zmm_k1_zmmmt_imm8_512_66_0F3A_W0_D1
	3,// MVEX_Vcvtfxpntpd2dq_zmm_k1_zmmmt_imm8
	0,// Via_undoc_F30FA6F0_16
	0,// Via_undoc_F30FA6F0_32
	0,// Via_undoc_F30FA6F0_64
	0,// Via_undoc_F30FA6F8_16
	0,// Via_undoc_F30FA6F8_32
	0,// Via_undoc_F30FA6F8_64
	0,// Xsha512_16
	0,// Xsha512_32
	0,// Xsha512_64
	0,// Xstore_alt_16
	0,// Xstore_alt_32
	0,// Xstore_alt_64
	0,// Xsha512_alt_16
	0,// Xsha512_alt_32
	0,// Xsha512_alt_64
	0,// Zero_bytes
	0,// Wrmsrns
	0,// Wrmsrlist
	0,// Rdmsrlist
	0,// Rmpquery
	1,// Prefetchit1_m8
	1,// Prefetchit0_m8
	2,// Aadd_m32_r32
	2,// Aadd_m64_r64
	2,// Aand_m32_r32
	2,// Aand_m64_r64
	2,// Axor_m32_r32
	2,// Axor_m64_r64
	2,// Aor_m32_r32
	2,// Aor_m64_r64
	3,// VEX_Vpdpbuud_xmm_xmm_xmmm128
	3,// VEX_Vpdpbuud_ymm_ymm_ymmm256
	3,// VEX_Vpdpbsud_xmm_xmm_xmmm128
	3,// VEX_Vpdpbsud_ymm_ymm_ymmm256
	3,// VEX_Vpdpbssd_xmm_xmm_xmmm128
	3,// VEX_Vpdpbssd_ymm_ymm_ymmm256
	3,// VEX_Vpdpbuuds_xmm_xmm_xmmm128
	3,// VEX_Vpdpbuuds_ymm_ymm_ymmm256
	3,// VEX_Vpdpbsuds_xmm_xmm_xmmm128
	3,// VEX_Vpdpbsuds_ymm_ymm_ymmm256
	3,// VEX_Vpdpbssds_xmm_xmm_xmmm128
	3,// VEX_Vpdpbssds_ymm_ymm_ymmm256
	3,// VEX_Tdpfp16ps_tmm_tmm_tmm
	2,// VEX_Vcvtneps2bf16_xmm_xmmm128
	2,// VEX_Vcvtneps2bf16_xmm_ymmm256
	2,// VEX_Vcvtneoph2ps_xmm_m128
	2,// VEX_Vcvtneoph2ps_ymm_m256
	2,// VEX_Vcvtneeph2ps_xmm_m128
	2,// VEX_Vcvtneeph2ps_ymm_m256
	2,// VEX_Vcvtneebf162ps_xmm_m128
	2,// VEX_Vcvtneebf162ps_ymm_m256
	2,// VEX_Vcvtneobf162ps_xmm_m128
	2,// VEX_Vcvtneobf162ps_ymm_m256
	2,// VEX_Vbcstnesh2ps_xmm_m16
	2,// VEX_Vbcstnesh2ps_ymm_m16
	2,// VEX_Vbcstnebf162ps_xmm_m16
	2,// VEX_Vbcstnebf162ps_ymm_m16
	3,// VEX_Vpmadd52luq_xmm_xmm_xmmm128
	3,// VEX_Vpmadd52luq_ymm_ymm_ymmm256
	3,// VEX_Vpmadd52huq_xmm_xmm_xmmm128
	3,// VEX_Vpmadd52huq_ymm_ymm_ymmm256
	3,// VEX_Cmpoxadd_m32_r32_r32
	3,// VEX_Cmpoxadd_m64_r64_r64
	3,// VEX_Cmpnoxadd_m32_r32_r32
	3,// VEX_Cmpnoxadd_m64_r64_r64
	3,// VEX_Cmpbxadd_m32_r32_r32
	3,// VEX_Cmpbxadd_m64_r64_r64
	3,// VEX_Cmpnbxadd_m32_r32_r32
	3,// VEX_Cmpnbxadd_m64_r64_r64
	3,// VEX_Cmpzxadd_m32_r32_r32
	3,// VEX_Cmpzxadd_m64_r64_r64
	3,// VEX_Cmpnzxadd_m32_r32_r32
	3,// VEX_Cmpnzxadd_m64_r64_r64
	3,// VEX_Cmpbexadd_m32_r32_r32
	3,// VEX_Cmpbexadd_m64_r64_r64
	3,// VEX_Cmpnbexadd_m32_r32_r32
	3,// VEX_Cmpnbexadd_m64_r64_r64
	3,// VEX_Cmpsxadd_m32_r32_r32
	3,// VEX_Cmpsxadd_m64_r64_r64
	3,// VEX_Cmpnsxadd_m32_r32_r32
	3,// VEX_Cmpnsxadd_m64_r64_r64
	3,// VEX_Cmppxadd_m32_r32_r32
	3,// VEX_Cmppxadd_m64_r64_r64
	3,// VEX_Cmpnpxadd_m32_r32_r32
	3,// VEX_Cmpnpxadd_m64_r64_r64
	3,// VEX_Cmplxadd_m32_r32_r32
	3,// VEX_Cmplxadd_m64_r64_r64
	3,// VEX_Cmpnlxadd_m32_r32_r32
	3,// VEX_Cmpnlxadd_m64_r64_r64
	3,// VEX_Cmplexadd_m32_r32_r32
	3,// VEX_Cmplexadd_m64_r64_r64
	3,// VEX_Cmpnlexadd_m32_r32_r32
	3,// VEX_Cmpnlexadd_m64_r64_r64
	3,// VEX_Tcmmrlfp16ps_tmm_tmm_tmm
	3,// VEX_Tcmmimfp16ps_tmm_tmm_tmm
	0,// Pbndkb
	3,// VEX_Vsha512rnds2_ymm_ymm_xmm
	2,// VEX_Vsha512msg1_ymm_xmm
	2,// VEX_Vsha512msg2_ymm_ymm
	3,// VEX_Vpdpwuud_xmm_xmm_xmmm128
	3,// VEX_Vpdpwuud_ymm_ymm_ymmm256
	3,// VEX_Vpdpwusd_xmm_xmm_xmmm128
	3,// VEX_Vpdpwusd_ymm_ymm_ymmm256
	3,// VEX_Vpdpwsud_xmm_xmm_xmmm128
	3,// VEX_Vpdpwsud_ymm_ymm_ymmm256
	3,// VEX_Vpdpwuuds_xmm_xmm_xmmm128
	3,// VEX_Vpdpwuuds_ymm_ymm_ymmm256
	3,// VEX_Vpdpwusds_xmm_xmm_xmmm128
	3,// VEX_Vpdpwusds_ymm_ymm_ymmm256
	3,// VEX_Vpdpwsuds_xmm_xmm_xmmm128
	3,// VEX_Vpdpwsuds_ymm_ymm_ymmm256
	3,// VEX_Vsm3msg1_xmm_xmm_xmmm128
	3,// VEX_Vsm3msg2_xmm_xmm_xmmm128
	3,// VEX_Vsm4key4_xmm_xmm_xmmm128
	3,// VEX_Vsm4key4_ymm_ymm_ymmm256
	3,// VEX_Vsm4rnds4_xmm_xmm_xmmm128
	3,// VEX_Vsm4rnds4_ymm_ymm_ymmm256
	4,// VEX_Vsm3rnds2_xmm_xmm_xmmm128_imm8
];
